@@ -2,21 +2,10 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
-#
-#  SLEPc is free software: you can redistribute it and/or modify it under  the
-#  terms of version 3 of the GNU Lesser General Public License as published by
-#  the Free Software Foundation.
-#
-#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-#  more details.
-#
-#  You  should have received a copy of the GNU Lesser General  Public  License
-#  along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+#  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 
@@ -85,21 +74,28 @@ def WriteModulesFile(modules,version,sdir):
   modules.write('set slepc_dir %s\n' % sdir)
   modules.write('setenv SLEPC_DIR $slepc_dir\n')
 
-def WritePkgconfigFile(pkgconfig,version,pversion,sdir,isinstall,prefixdir):
+def WritePkgconfigFile(pkgconfig,version,pversion,sdir,isinstall,prefixdir,slflag):
   ''' Write the contents of the pkg-config file '''
-  pkgconfig.write('Name: SLEPc, the Scalable Library for Eigenvalue Problem Computations\n')
-  pkgconfig.write('Description: A parallel library to compute eigenvalues and eigenvectors of large, sparse matrices with iterative methods. It is based on PETSc.\n')
+  pkgconfig.write('prefix=%s\n' % prefixdir)
+  pkgconfig.write('exec_prefix=${prefix}\n')
+  pkgconfig.write('includedir=${prefix}/include\n')
+  pkgconfig.write('libdir=${prefix}/lib\n\n')
+  pkgconfig.write('Name: SLEPc\n')
+  pkgconfig.write('Description: the Scalable Library for Eigenvalue Problem Computations\n')
   pkgconfig.write('Version: %s\n' % version)
-  pkgconfig.write('Requires: PETSc = %s\n' % pversion)
-  pkgconfig.write('Cflags: -I'+os.path.join(prefixdir,'include'))
+  pkgconfig.write('Requires: PETSc >= %s\n' % pversion)
+  pkgconfig.write('Cflags: -I${includedir}')
   if not isinstall:
     pkgconfig.write(' -I'+os.path.join(sdir,'include'))
-  pkgconfig.write('\nLibs: -L%s -lslepc\n' % os.path.join(prefixdir,'lib'))
+  pkgconfig.write('\nLibs:')
+  if slflag:
+    pkgconfig.write(' %s${libdir}' % slflag)
+  pkgconfig.write(' -L${libdir} -lslepc\n')
 
 def WriteCMakeConfigFile(cmakeconf):
   ''' Write the contents of the CMake configuration file '''
   cmakeconf.write('''
-set (SLEPC_PACKAGE_LIBS "${ARPACK_LIB}" "${BLZPACK_LIB}" "${TRLAN_LIB}" "${PRIMME_LIB}" "${FEAST_LIB}" "${BLOPEX_LIB}" )
+set (SLEPC_PACKAGE_LIBS "${ARPACK_LIB}" "${BLZPACK_LIB}" "${TRLAN_LIB}" "${PRIMME_LIB}" "${FEAST_LIB}" "${BLOPEX_LIB}" "${SLICOT_LIB}" )
 set (SLEPC_PACKAGE_INCLUDES "${PRIMME_INCLUDE}")
 find_library (PETSC_LIB petsc HINTS ${PETSc_BINARY_DIR}/lib )
 if (NOT PETSC_LIB) # Interpret missing libpetsc to mean that PETSc was built --with-single-library=0
@@ -112,6 +108,14 @@ if (NOT PETSC_LIB) # Interpret missing libpetsc to mean that PETSc was built --w
 endif ()
 ''')
 
+def WriteReconfigScript(reconfig,slepcdir,usedargs):
+  ''' Write the contents of the reconfigure script '''
+  reconfig.write('#!/usr/bin/env python\n\n')
+  reconfig.write('import os, sys\n')
+  if usedargs:
+    reconfig.write('sys.argv.extend(\''+usedargs+'\'.split())\n')
+  reconfig.write('execfile(os.path.join(\''+slepcdir+'\',\'config\',\'configure.py\'))\n')
+
 # Use en_US as language so that compiler messages are in English
 if 'LC_LOCAL' in os.environ and os.environ['LC_LOCAL'] != '' and os.environ['LC_LOCAL'] != 'en_US' and os.environ['LC_LOCAL']!= 'en_US.UTF-8': os.environ['LC_LOCAL'] = 'en_US.UTF-8'
 if 'LANG' in os.environ and os.environ['LANG'] != '' and os.environ['LANG'] != 'en_US' and os.environ['LANG'] != 'en_US.UTF-8': os.environ['LANG'] = 'en_US.UTF-8'
@@ -137,7 +141,7 @@ argdb = argdb.ArgDB(sys.argv)
 log   = log.Log()
 
 # Load classes for packages and process command-line options
-import slepc, petsc, arpack, blzpack, trlan, feast, primme, blopex, sowing, lapack
+import slepc, petsc, arpack, blzpack, trlan, feast, primme, blopex, sowing, lapack, slicot
 slepc   = slepc.SLEPc(argdb,log)
 petsc   = petsc.PETSc(argdb,log)
 arpack  = arpack.Arpack(argdb,log)
@@ -148,10 +152,11 @@ primme  = primme.Primme(argdb,log)
 trlan   = trlan.Trlan(argdb,log)
 sowing  = sowing.Sowing(argdb,log)
 lapack  = lapack.Lapack(argdb,log)
+slicot  = slicot.Slicot(argdb,log)
 
-externalpackages = [arpack, blopex, blzpack, feast, primme, trlan]
-optionspackages  = [slepc, arpack, blopex, blzpack, feast, primme, trlan, sowing]
-checkpackages    = [arpack, blopex, blzpack, feast, primme, trlan, lapack]
+externalpackages = [arpack, blopex, blzpack, feast, primme, slicot, trlan]
+optionspackages  = [slepc, arpack, blopex, blzpack, feast, primme, slicot, trlan, sowing]
+checkpackages    = [arpack, blopex, blzpack, feast, primme, slicot, trlan, lapack]
 
 # Print help if requested and check for wrong command-line options
 if argdb.PopHelp():
@@ -168,7 +173,7 @@ petsc.InitDir(slepc.prefixdir)
 slepc.InitDir()
 petsc.LoadVersion()
 slepc.LoadVersion()
-if petsc.version != slepc.version:
+if petsc.version < slepc.version:
   sys.exit('ERROR: This SLEPc version is not compatible with PETSc version '+petsc.version)
 
 # Check some information about PETSc configuration
@@ -248,22 +253,16 @@ if slepc.isinstall:
   modules  = CreateFile(modulesdir,slepc.lversion,log)
 else:
   modules  = CreateFile(modulesdir,slepc.lversion+'-'+archname,log)
+  reconfig = CreateFile(confdir,'reconfigure-'+archname+'.py',log)
+  reconfigpath = os.path.join(confdir,'reconfigure-'+archname+'.py')
 
 # Write initial part of file slepcvariables
+slepcvars.write('SLEPC_CONFIGURE_OPTIONS = '+argdb.UsedArgs()+'\n')
 slepcvars.write('SLEPC_DESTDIR = '+slepc.prefixdir+'\n')
 if emptyarch:
   slepcvars.write('INSTALLED_PETSC = 1\n')
-testruns = set(petsc.test_runs.split())
-testruns = testruns.intersection(set(['C','F90','Fortran','C_Complex','Fortran_Complex','C_NoComplex','Fortran_NoComplex']))
-if petsc.precision != '__float128':
-  testruns = testruns.union(set(['C_NoF128']))
 if slepc.datadir:
   slepcvars.write('DATAFILESPATH = '+slepc.datadir+'\n')
-  if petsc.scalar == 'complex':
-    testruns = testruns.union(set(['DATAFILESPATH_Complex']))
-  else:
-    testruns = testruns.union(set(['DATAFILESPATH']))
-slepcvars.write('TEST_RUNS = '+' '.join(testruns)+'\n')
 
 # Write initial part of file slepcconf.h
 slepcconf.write('#if !defined(__SLEPCCONF_H)\n')
@@ -301,14 +300,34 @@ if not petsc.havepackage:
 if petsc.singlelib:
   slepcvars.write('SHLIBS = libslepc\n')
   slepcvars.write('LIBNAME = '+os.path.join('${INSTALL_LIB_DIR}','libslepc.${AR_LIB_SUFFIX}')+'\n')
-  for module in ['SYS','MFN','EPS','SVD','PEP','NEP']:
-    slepcvars.write('SLEPC_'+module+'_LIB = ${CC_LINKER_SLFLAG}${SLEPC_LIB_DIR} -L${SLEPC_LIB_DIR} -lslepc ${SLEPC_EXTERNAL_LIB} ${PETSC_KSP_LIB}\n')
-  slepcvars.write('SLEPC_LIB = ${CC_LINKER_SLFLAG}${SLEPC_LIB_DIR} -L${SLEPC_LIB_DIR} -lslepc ${SLEPC_EXTERNAL_LIB} ${PETSC_KSP_LIB}\n')
+  for module in ['SYS','EPS','SVD','PEP','NEP','MFN','LME']:
+    slepcvars.write('SLEPC_'+module+'_LIB = ${CC_LINKER_SLFLAG}${SLEPC_LIB_DIR} -L${SLEPC_LIB_DIR} -lslepc ${SLEPC_EXTERNAL_LIB} ${PETSC_SNES_LIB}\n')
+  slepcvars.write('SLEPC_LIB = ${CC_LINKER_SLFLAG}${SLEPC_LIB_DIR} -L${SLEPC_LIB_DIR} -lslepc ${SLEPC_EXTERNAL_LIB} ${PETSC_SNES_LIB}\n')
 
 # Check for external packages and for missing LAPACK functions
 for pkg in checkpackages:
   pkg.Process(slepcconf,slepcvars,cmakeconf,petsc,archdir)
 
+# Determine which tests must be run
+testruns = set(petsc.test_runs.split())
+supported_tests = set(['C','C_Complex','C_NoComplex','C_NotSingle','C_NoComplex_NotSingle','VECCUDA','VECCUDA_Complex','VECCUDA_NoComplex'])
+if petsc.fortran:
+  supported_tests = supported_tests.union(set(['F90','Fortran','F90_Complex','Fortran_Complex','F90_NoComplex','Fortran_NoComplex','F90_NotSingle','Fortran_NotSingle','Fortran_NoComplex_NotSingle']))
+testruns = testruns.intersection(supported_tests)
+if not petsc.mpiuni:
+  if petsc.precision == 'double':
+    testruns = testruns.union(set(['C_Double']))
+  if petsc.precision == 'double' and petsc.scalar == 'real' and not petsc.ind64:
+    testruns = testruns.union(set(['C_DataFile']))
+  if slepc.datadir:
+    if petsc.scalar == 'complex':
+      testruns = testruns.union(set(['DATAFILESPATH_Complex']))
+    else:
+      testruns = testruns.union(set(['DATAFILESPATH']))
+  for pkg in externalpackages:
+    testruns = testruns.union(set(pkg.TestRuns(petsc)))
+slepcvars.write('TEST_RUNS = '+' '.join(testruns)+'\n')
+
 # Write Modules, pkg-config and CMake configuration files
 log.NewSection('Writing various configuration files...')
 log.write('Modules file in '+modulesdir)
@@ -317,24 +336,35 @@ if slepc.isinstall:
 else:
   WriteModulesFile(modules,slepc.lversion,slepc.dir)
 log.write('pkg-config file in '+pkgconfdir)
-WritePkgconfigFile(pkgconfig,slepc.lversion,petsc.lversion,slepc.dir,slepc.isinstall,slepc.prefixdir)
+slflag = ''
+if petsc.buildsharedlib: slflag = petsc.slflag
+WritePkgconfigFile(pkgconfig,slepc.lversion,petsc.version,slepc.dir,slepc.isinstall,slepc.prefixdir,slflag)
 log.write('CMake configure file in '+confdir)
 WriteCMakeConfigFile(cmakeconf)
 
+# Write reconfigure file
+if not slepc.isinstall:
+  WriteReconfigScript(reconfig,slepc.dir,argdb.UsedArgs())
+  try:
+    os.chmod(reconfigpath,0775)
+  except OSError, e:
+    log.Exit('ERROR: Unable to make reconfigure script executable:\n'+str(e))
+
 # Finish with configuration files (except slepcvars)
 slepcrules.close()
 slepcconf.write('#endif\n')
 slepcconf.close()
-modules.close()
-pkgconfig.close()
 cmakeconf.close()
+pkgconfig.close()
+modules.close()
+if not slepc.isinstall: reconfig.close()
 
 # Download sowing if requested and make Fortran stubs if necessary
 bfort = petsc.bfort
 if sowing.downloadpackage:
   bfort = sowing.Install(archdir,petsc.make)
 
-if slepc.isrepo and hasattr(petsc,'fc'):
+if slepc.isrepo and petsc.fortran:
   try:
     if not os.path.exists(bfort):
       bfort = os.path.join(archdir,'bin','bfort')
