@@ -1,21 +1,10 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
-#
-#  SLEPc is free software: you can redistribute it and/or modify it under  the
-#  terms of version 3 of the GNU Lesser General Public License as published by
-#  the Free Software Foundation.
-#
-#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-#  more details.
-#
-#  You  should have received a copy of the GNU Lesser General  Public  License
-#  along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+#  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 
@@ -34,6 +23,7 @@ class Arpack(package.Package):
     self.archive        = 'arpack-ng-'+self.version+'.tar.gz'
     self.dirname        = 'arpack-ng-'+self.version
     self.supportssingle = True
+    self.fortran        = True
     self.ProcessArgs(argdb)
 
   def Functions(self,petsc):
@@ -94,6 +84,8 @@ class Arpack(package.Package):
     confopt = '--prefix='+archdir+' F77="'+petsc.fc+'" FFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'"'
     if not petsc.mpiuni:
       confopt = confopt+' --enable-mpi'
+    if not petsc.buildsharedlib:
+      confopt = confopt+' --disable-shared'
     result,output = commands.getstatusoutput('cd '+builddir+'&& sh bootstrap && ./configure '+confopt+' && '+petsc.make+' && '+petsc.make+' install')
     self.log.write(output)
     if result:
