@@ -1,25 +1,14 @@
+!
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
-!
-!  SLEPc is free software: you can redistribute it and/or modify it under  the
-!  terms of version 3 of the GNU Lesser General Public License as published by
-!  the Free Software Foundation.
-!
-!  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-!  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-!  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-!  more details.
-!
-!  You  should have received a copy of the GNU Lesser General  Public  License
-!  along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+!  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !
 !  Single Fortran include file for all of SLEPc
 !
-
 #include "slepc/finclude/slepcsys.h"
 #include "slepc/finclude/slepcbv.h"
 #include "slepc/finclude/slepcfn.h"
@@ -31,3 +20,4 @@
 #include "slepc/finclude/slepcpep.h"
 #include "slepc/finclude/slepcnep.h"
 #include "slepc/finclude/slepcmfn.h"
+#include "slepc/finclude/slepclme.h"
