@@ -1,30 +1,26 @@
 /*
-   Necessary routines in BLAS and LAPACK not included in petscblaslapack.f
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Necessary routines in BLAS and LAPACK not included in petscblaslapack.h
+*/
 
 #if !defined(__SLEPCBLASLAPACK_H)
 #define __SLEPCBLASLAPACK_H
 #include <petscblaslapack.h>
 
+/* Macro to check nonzero info after LAPACK call */
+#define SlepcCheckLapackInfo(routine,info) \
+  do { \
+    if (info) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in LAPACK subroutine %s: info=%d",routine,(int)info); \
+  } while (0)
+
 /* Macros for building LAPACK names */
 #if defined(PETSC_BLASLAPACK_UNDERSCORE)
 #if defined(PETSC_USE_REAL_SINGLE)
