@@ -1,31 +1,23 @@
 /*
-   User interface for the SLEPC eigenproblem solvers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   User interface for the SLEPc linear eigenvalue solvers
+*/
 
 #if !defined(__SLEPCEPS_H)
 #define __SLEPCEPS_H
 #include <slepcst.h>
-#include <slepcip.h>
+#include <slepcbv.h>
 #include <slepcds.h>
+#include <slepcrg.h>
+#include <petscsnes.h>
 
 PETSC_EXTERN PetscErrorCode EPSInitializePackage(void);
 
@@ -55,6 +47,7 @@ typedef const char* EPSType;
 #define EPSGD          "gd"
 #define EPSJD          "jd"
 #define EPSRQCG        "rqcg"
+#define EPSLOBPCG      "lobpcg"
 #define EPSCISS        "ciss"
 #define EPSLAPACK      "lapack"
 #define EPSARPACK      "arpack"
@@ -85,11 +78,11 @@ typedef enum { EPS_HEP=1,
     EPSExtraction - Determines the type of extraction technique employed
     by the eigensolver
 
-    Level: beginner
+    Level: advanced
 
 .seealso: EPSSetExtraction(), EPSGetExtraction()
 E*/
-typedef enum { EPS_RITZ=1,
+typedef enum { EPS_RITZ,
                EPS_HARMONIC,
                EPS_HARMONIC_RELATIVE,
                EPS_HARMONIC_RIGHT,
@@ -123,10 +116,23 @@ typedef enum { EPS_LARGEST_MAGNITUDE=1,
 
 .seealso: EPSSetBalance()
 E*/
-typedef enum { EPS_BALANCE_NONE=1,
+typedef enum { EPS_BALANCE_NONE,
                EPS_BALANCE_ONESIDE,
                EPS_BALANCE_TWOSIDE,
                EPS_BALANCE_USER } EPSBalance;
+PETSC_EXTERN const char *EPSBalanceTypes[];
+
+/*E
+    EPSErrorType - The error type used to assess accuracy of computed solutions
+
+    Level: intermediate
+
+.seealso: EPSComputeError()
+E*/
+typedef enum { EPS_ERROR_ABSOLUTE,
+               EPS_ERROR_RELATIVE,
+               EPS_ERROR_BACKWARD } EPSErrorType;
+PETSC_EXTERN const char *EPSErrorTypes[];
 
 /*E
     EPSConv - Determines the convergence test
@@ -135,12 +141,40 @@ typedef enum { EPS_BALANCE_NONE=1,
 
 .seealso: EPSSetConvergenceTest(), EPSSetConvergenceTestFunction()
 E*/
-typedef enum { EPS_CONV_ABS=1,
-               EPS_CONV_EIG,
+typedef enum { EPS_CONV_ABS,
+               EPS_CONV_REL,
                EPS_CONV_NORM,
                EPS_CONV_USER } EPSConv;
 
-PETSC_EXTERN PetscErrorCode EPSCreate(MPI_Comm,EPS *);
+/*E
+    EPSStop - Determines the stopping test
+
+    Level: advanced
+
+.seealso: EPSSetStoppingTest(), EPSSetStoppingTestFunction()
+E*/
+typedef enum { EPS_STOP_BASIC,
+               EPS_STOP_USER } EPSStop;
+
+/*E
+    EPSConvergedReason - Reason an eigensolver was said to
+         have converged or diverged
+
+    Level: intermediate
+
+.seealso: EPSSolve(), EPSGetConvergedReason(), EPSSetTolerances()
+E*/
+typedef enum {/* converged */
+              EPS_CONVERGED_TOL                =  1,
+              EPS_CONVERGED_USER               =  2,
+              /* diverged */
+              EPS_DIVERGED_ITS                 = -1,
+              EPS_DIVERGED_BREAKDOWN           = -2,
+              EPS_DIVERGED_SYMMETRY_LOST       = -3,
+              EPS_CONVERGED_ITERATING          =  0} EPSConvergedReason;
+PETSC_EXTERN const char *const*EPSConvergedReasons;
+
+PETSC_EXTERN PetscErrorCode EPSCreate(MPI_Comm,EPS*);
 PETSC_EXTERN PetscErrorCode EPSDestroy(EPS*);
 PETSC_EXTERN PetscErrorCode EPSReset(EPS);
 PETSC_EXTERN PetscErrorCode EPSSetType(EPS,EPSType);
@@ -157,7 +191,16 @@ PETSC_EXTERN PetscErrorCode EPSSetFromOptions(EPS);
 PETSC_EXTERN PetscErrorCode EPSSetUp(EPS);
 PETSC_EXTERN PetscErrorCode EPSSolve(EPS);
 PETSC_EXTERN PetscErrorCode EPSView(EPS,PetscViewer);
-PETSC_EXTERN PetscErrorCode EPSPrintSolution(EPS,PetscViewer);
+PETSC_STATIC_INLINE PetscErrorCode EPSViewFromOptions(EPS eps,PetscObject obj,const char name[]) {return PetscObjectViewFromOptions((PetscObject)eps,obj,name);}
+PETSC_EXTERN PetscErrorCode EPSErrorView(EPS,EPSErrorType,PetscViewer);
+PETSC_DEPRECATED("Use EPSErrorView()") PETSC_STATIC_INLINE PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer v) {return EPSErrorView(eps,EPS_ERROR_RELATIVE,v);}
+PETSC_EXTERN PetscErrorCode EPSErrorViewFromOptions(EPS);
+PETSC_EXTERN PetscErrorCode EPSReasonView(EPS,PetscViewer);
+PETSC_EXTERN PetscErrorCode EPSReasonViewFromOptions(EPS);
+PETSC_EXTERN PetscErrorCode EPSValuesView(EPS,PetscViewer);
+PETSC_EXTERN PetscErrorCode EPSValuesViewFromOptions(EPS);
+PETSC_EXTERN PetscErrorCode EPSVectorsView(EPS,PetscViewer);
+PETSC_EXTERN PetscErrorCode EPSVectorsViewFromOptions(EPS);
 
 PETSC_EXTERN PetscErrorCode EPSSetTarget(EPS,PetscScalar);
 PETSC_EXTERN PetscErrorCode EPSGetTarget(EPS,PetscScalar*);
@@ -165,18 +208,26 @@ PETSC_EXTERN PetscErrorCode EPSSetInterval(EPS,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode EPSGetInterval(EPS,PetscReal*,PetscReal*);
 PETSC_EXTERN PetscErrorCode EPSSetST(EPS,ST);
 PETSC_EXTERN PetscErrorCode EPSGetST(EPS,ST*);
-PETSC_EXTERN PetscErrorCode EPSSetIP(EPS,IP);
-PETSC_EXTERN PetscErrorCode EPSGetIP(EPS,IP*);
+PETSC_EXTERN PetscErrorCode EPSSetBV(EPS,BV);
+PETSC_EXTERN PetscErrorCode EPSGetBV(EPS,BV*);
+PETSC_EXTERN PetscErrorCode EPSSetRG(EPS,RG);
+PETSC_EXTERN PetscErrorCode EPSGetRG(EPS,RG*);
 PETSC_EXTERN PetscErrorCode EPSSetDS(EPS,DS);
 PETSC_EXTERN PetscErrorCode EPSGetDS(EPS,DS*);
 PETSC_EXTERN PetscErrorCode EPSSetTolerances(EPS,PetscReal,PetscInt);
 PETSC_EXTERN PetscErrorCode EPSGetTolerances(EPS,PetscReal*,PetscInt*);
-PETSC_EXTERN PetscErrorCode EPSSetConvergenceTestFunction(EPS,PetscErrorCode (*)(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void*);
-PETSC_EXTERN PetscErrorCode EPSSetConvergenceTest(EPS eps,EPSConv conv);
-PETSC_EXTERN PetscErrorCode EPSGetConvergenceTest(EPS eps,EPSConv *conv);
-PETSC_EXTERN PetscErrorCode EPSConvergedEigRelative(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
+PETSC_EXTERN PetscErrorCode EPSSetConvergenceTestFunction(EPS,PetscErrorCode (*)(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void*,PetscErrorCode (*)(void*));
+PETSC_EXTERN PetscErrorCode EPSSetConvergenceTest(EPS,EPSConv);
+PETSC_EXTERN PetscErrorCode EPSGetConvergenceTest(EPS,EPSConv*);
 PETSC_EXTERN PetscErrorCode EPSConvergedAbsolute(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
-PETSC_EXTERN PetscErrorCode EPSConvergedNormRelative(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
+PETSC_EXTERN PetscErrorCode EPSConvergedRelative(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
+PETSC_EXTERN PetscErrorCode EPSConvergedNorm(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
+PETSC_EXTERN PetscErrorCode EPSSetStoppingTestFunction(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscInt,PetscInt,EPSConvergedReason*,void*),void*,PetscErrorCode (*)(void*));
+PETSC_EXTERN PetscErrorCode EPSSetStoppingTest(EPS,EPSStop);
+PETSC_EXTERN PetscErrorCode EPSGetStoppingTest(EPS,EPSStop*);
+PETSC_EXTERN PetscErrorCode EPSStoppingBasic(EPS,PetscInt,PetscInt,PetscInt,PetscInt,EPSConvergedReason*,void*);
+PETSC_EXTERN PetscErrorCode EPSGetConvergedReason(EPS,EPSConvergedReason*);
+
 PETSC_EXTERN PetscErrorCode EPSSetDimensions(EPS,PetscInt,PetscInt,PetscInt);
 PETSC_EXTERN PetscErrorCode EPSGetDimensions(EPS,PetscInt*,PetscInt*,PetscInt*);
 
@@ -184,40 +235,37 @@ PETSC_EXTERN PetscErrorCode EPSGetConverged(EPS,PetscInt*);
 PETSC_EXTERN PetscErrorCode EPSGetEigenpair(EPS,PetscInt,PetscScalar*,PetscScalar*,Vec,Vec);
 PETSC_EXTERN PetscErrorCode EPSGetEigenvalue(EPS,PetscInt,PetscScalar*,PetscScalar*);
 PETSC_EXTERN PetscErrorCode EPSGetEigenvector(EPS,PetscInt,Vec,Vec);
-PETSC_EXTERN PetscErrorCode EPSGetEigenvectorLeft(EPS,PetscInt,Vec,Vec);
-PETSC_EXTERN PetscErrorCode EPSComputeRelativeError(EPS,PetscInt,PetscReal*);
-PETSC_EXTERN PetscErrorCode EPSComputeRelativeErrorLeft(EPS,PetscInt,PetscReal*);
-PETSC_EXTERN PetscErrorCode EPSComputeResidualNorm(EPS,PetscInt,PetscReal*);
-PETSC_EXTERN PetscErrorCode EPSComputeResidualNormLeft(EPS,PetscInt,PetscReal*);
+
+PETSC_EXTERN PetscErrorCode EPSComputeError(EPS,PetscInt,EPSErrorType,PetscReal*);
+PETSC_DEPRECATED("Use EPSComputeError()") PETSC_STATIC_INLINE PetscErrorCode EPSComputeRelativeError(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_RELATIVE,r);}
+PETSC_DEPRECATED("Use EPSComputeError() with EPS_ERROR_ABSOLUTE") PETSC_STATIC_INLINE PetscErrorCode EPSComputeResidualNorm(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_ABSOLUTE,r);}
 PETSC_EXTERN PetscErrorCode EPSGetInvariantSubspace(EPS,Vec*);
-PETSC_EXTERN PetscErrorCode EPSGetInvariantSubspaceLeft(EPS,Vec*);
 PETSC_EXTERN PetscErrorCode EPSGetErrorEstimate(EPS,PetscInt,PetscReal*);
-PETSC_EXTERN PetscErrorCode EPSGetErrorEstimateLeft(EPS,PetscInt,PetscReal*);
 
 PETSC_EXTERN PetscErrorCode EPSMonitor(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt);
 PETSC_EXTERN PetscErrorCode EPSMonitorSet(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),void*,PetscErrorCode (*)(void**));
+PETSC_EXTERN PetscErrorCode EPSMonitorSetFromOptions(EPS,const char*,const char*,const char*,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscBool);
+PETSC_EXTERN PetscErrorCode EPSConvMonitorSetFromOptions(EPS,const char*,const char*,const char*,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,SlepcConvMonitor));
 PETSC_EXTERN PetscErrorCode EPSMonitorCancel(EPS);
-PETSC_EXTERN PetscErrorCode EPSGetMonitorContext(EPS,void **);
+PETSC_EXTERN PetscErrorCode EPSGetMonitorContext(EPS,void**);
 PETSC_EXTERN PetscErrorCode EPSGetIterationNumber(EPS,PetscInt*);
-PETSC_EXTERN PetscErrorCode EPSGetOperationCounters(EPS,PetscInt*,PetscInt*,PetscInt*);
 
 PETSC_EXTERN PetscErrorCode EPSSetWhichEigenpairs(EPS,EPSWhich);
 PETSC_EXTERN PetscErrorCode EPSGetWhichEigenpairs(EPS,EPSWhich*);
-PETSC_EXTERN PetscErrorCode EPSSetLeftVectorsWanted(EPS,PetscBool);
-PETSC_EXTERN PetscErrorCode EPSGetLeftVectorsWanted(EPS,PetscBool*);
-PETSC_EXTERN PetscErrorCode EPSSetMatrixNorms(EPS,PetscReal,PetscReal,PetscBool);
-PETSC_EXTERN PetscErrorCode EPSGetMatrixNorms(EPS,PetscReal*,PetscReal*,PetscBool*);
 PETSC_EXTERN PetscErrorCode EPSSetTrueResidual(EPS,PetscBool);
 PETSC_EXTERN PetscErrorCode EPSGetTrueResidual(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSSetPurify(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSGetPurify(EPS,PetscBool*);
 PETSC_EXTERN PetscErrorCode EPSSetEigenvalueComparison(EPS,PetscErrorCode (*func)(PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*),void*);
 PETSC_EXTERN PetscErrorCode EPSSetArbitrarySelection(EPS,PetscErrorCode (*func)(PetscScalar,PetscScalar,Vec,Vec,PetscScalar*,PetscScalar*,void*),void*);
 PETSC_EXTERN PetscErrorCode EPSIsGeneralized(EPS,PetscBool*);
 PETSC_EXTERN PetscErrorCode EPSIsHermitian(EPS,PetscBool*);
 PETSC_EXTERN PetscErrorCode EPSIsPositive(EPS,PetscBool*);
 
-PETSC_EXTERN PetscErrorCode EPSMonitorFirst(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
-PETSC_EXTERN PetscErrorCode EPSMonitorAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
-PETSC_EXTERN PetscErrorCode EPSMonitorConverged(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
+PETSC_EXTERN PetscErrorCode EPSMonitorFirst(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode EPSMonitorAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode EPSMonitorConverged(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,SlepcConvMonitor);
+PETSC_EXTERN PetscErrorCode EPSMonitorLGCreate(MPI_Comm,const char[],const char[],int,int,int,int,PetscDrawLG*);
 PETSC_EXTERN PetscErrorCode EPSMonitorLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
 PETSC_EXTERN PetscErrorCode EPSMonitorLGAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
 
@@ -225,43 +273,17 @@ PETSC_EXTERN PetscErrorCode EPSSetTrackAll(EPS,PetscBool);
 PETSC_EXTERN PetscErrorCode EPSGetTrackAll(EPS,PetscBool*);
 
 PETSC_EXTERN PetscErrorCode EPSSetDeflationSpace(EPS,PetscInt,Vec*);
-PETSC_EXTERN PetscErrorCode EPSRemoveDeflationSpace(EPS);
 PETSC_EXTERN PetscErrorCode EPSSetInitialSpace(EPS,PetscInt,Vec*);
-PETSC_EXTERN PetscErrorCode EPSSetInitialSpaceLeft(EPS,PetscInt,Vec*);
 
 PETSC_EXTERN PetscErrorCode EPSSetOptionsPrefix(EPS,const char*);
 PETSC_EXTERN PetscErrorCode EPSAppendOptionsPrefix(EPS,const char*);
 PETSC_EXTERN PetscErrorCode EPSGetOptionsPrefix(EPS,const char*[]);
 
-/*E
-    EPSConvergedReason - Reason an eigensolver was said to
-         have converged or diverged
-
-    Level: beginner
-
-.seealso: EPSSolve(), EPSGetConvergedReason(), EPSSetTolerances()
-E*/
-typedef enum {/* converged */
-              EPS_CONVERGED_TOL                =  2,
-              /* diverged */
-              EPS_DIVERGED_ITS                 = -3,
-              EPS_DIVERGED_BREAKDOWN           = -4,
-              EPS_CONVERGED_ITERATING          =  0} EPSConvergedReason;
-
-PETSC_EXTERN PetscErrorCode EPSGetConvergedReason(EPS,EPSConvergedReason *);
-
-PETSC_EXTERN PetscErrorCode EPSSortEigenvalues(EPS,PetscInt,PetscScalar*,PetscScalar*,PetscInt*);
-PETSC_EXTERN PetscErrorCode EPSCompareEigenvalues(EPS,PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*);
-
-PETSC_EXTERN PetscErrorCode EPSGetStartVector(EPS,PetscInt,Vec,PetscBool*);
-PETSC_EXTERN PetscErrorCode EPSGetStartVectorLeft(EPS,PetscInt,Vec,PetscBool*);
-
 PETSC_EXTERN PetscFunctionList EPSList;
-PETSC_EXTERN PetscBool         EPSRegisterAllCalled;
-PETSC_EXTERN PetscErrorCode EPSRegisterAll(void);
 PETSC_EXTERN PetscErrorCode EPSRegister(const char[],PetscErrorCode(*)(EPS));
 
 PETSC_EXTERN PetscErrorCode EPSSetWorkVecs(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSAllocateSolution(EPS,PetscInt);
 
 /* --------- options specific to particular eigensolvers -------- */
 
@@ -279,12 +301,33 @@ PETSC_EXTERN const char *EPSPowerShiftTypes[];
 
 PETSC_EXTERN PetscErrorCode EPSPowerSetShiftType(EPS,EPSPowerShiftType);
 PETSC_EXTERN PetscErrorCode EPSPowerGetShiftType(EPS,EPSPowerShiftType*);
+PETSC_EXTERN PetscErrorCode EPSPowerSetNonlinear(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSPowerGetNonlinear(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSPowerSetUpdate(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSPowerGetUpdate(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSPowerSetSNES(EPS,SNES);
+PETSC_EXTERN PetscErrorCode EPSPowerGetSNES(EPS,SNES*);
 
 PETSC_EXTERN PetscErrorCode EPSArnoldiSetDelayed(EPS,PetscBool);
 PETSC_EXTERN PetscErrorCode EPSArnoldiGetDelayed(EPS,PetscBool*);
 
 PETSC_EXTERN PetscErrorCode EPSKrylovSchurSetRestart(EPS,PetscReal);
 PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetRestart(EPS,PetscReal*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurSetLocking(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetLocking(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurSetPartitions(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetPartitions(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurSetDetectZeros(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetDetectZeros(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurSetDimensions(EPS,PetscInt,PetscInt,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetDimensions(EPS,PetscInt*,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurSetSubintervals(EPS,PetscReal*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetSubintervals(EPS,PetscReal**);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetInertias(EPS,PetscInt*,PetscReal**,PetscInt**);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommInfo(EPS,PetscInt*,PetscInt*,Vec*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommPairs(EPS,PetscInt,PetscScalar*,Vec);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommMats(EPS,Mat*,Mat*);
+PETSC_EXTERN PetscErrorCode EPSKrylovSchurUpdateSubcommMats(EPS,PetscScalar,PetscScalar,Mat,PetscScalar,PetscScalar, Mat,MatStructure,PetscBool);
 
 /*E
     EPSLanczosReorthogType - determines the type of reorthogonalization
@@ -315,7 +358,7 @@ PETSC_EXTERN PetscErrorCode EPSBlzpackSetNSteps(EPS,PetscInt);
 
 .seealso: EPSPRIMMESetMethod(), EPSPRIMMEGetMethod()
 E*/
-typedef enum { EPS_PRIMME_DYNAMIC,
+typedef enum { EPS_PRIMME_DYNAMIC=1,
                EPS_PRIMME_DEFAULT_MIN_TIME,
                EPS_PRIMME_DEFAULT_MIN_MATVECS,
                EPS_PRIMME_ARNOLDI,
@@ -332,65 +375,88 @@ typedef enum { EPS_PRIMME_DYNAMIC,
                EPS_PRIMME_LOBPCG_ORTHOBASISW } EPSPRIMMEMethod;
 PETSC_EXTERN const char *EPSPRIMMEMethods[];
 
-PETSC_EXTERN PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,PetscInt bs);
-PETSC_EXTERN PetscErrorCode EPSPRIMMESetMethod(EPS eps, EPSPRIMMEMethod method);
-PETSC_EXTERN PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs);
-PETSC_EXTERN PetscErrorCode EPSPRIMMEGetMethod(EPS eps, EPSPRIMMEMethod *method);
+PETSC_EXTERN PetscErrorCode EPSPRIMMESetBlockSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSPRIMMEGetBlockSize(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSPRIMMESetMethod(EPS,EPSPRIMMEMethod);
+PETSC_EXTERN PetscErrorCode EPSPRIMMEGetMethod(EPS,EPSPRIMMEMethod*);
 
-/*E
-    EPSOrthType - determines the orthogonalization used in the search subspace
+PETSC_EXTERN PetscErrorCode EPSGDSetKrylovStart(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSGDGetKrylovStart(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSGDSetBlockSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSGDGetBlockSize(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSGDSetRestart(EPS,PetscInt,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSGDGetRestart(EPS,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSGDSetInitialSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSGDGetInitialSize(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSGDSetBOrth(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSGDGetBOrth(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSGDSetDoubleExpansion(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSGDGetDoubleExpansion(EPS,PetscBool*);
 
-    Level: advanced
-
-.seealso: EPSGDSetBOrth(), EPSJDSetBOrth()
-E*/
-typedef enum { EPS_ORTH_I=1,
-               EPS_ORTH_B,
-               EPS_ORTH_BOPT } EPSOrthType;
-
-PETSC_EXTERN PetscErrorCode EPSGDSetKrylovStart(EPS eps,PetscBool krylovstart);
-PETSC_EXTERN PetscErrorCode EPSGDGetKrylovStart(EPS eps,PetscBool *krylovstart);
-PETSC_EXTERN PetscErrorCode EPSGDSetBlockSize(EPS eps,PetscInt blocksize);
-PETSC_EXTERN PetscErrorCode EPSGDGetBlockSize(EPS eps,PetscInt *blocksize);
-PETSC_EXTERN PetscErrorCode EPSGDSetRestart(EPS eps,PetscInt minv,PetscInt plusk);
-PETSC_EXTERN PetscErrorCode EPSGDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk);
-PETSC_EXTERN PetscErrorCode EPSGDSetInitialSize(EPS eps,PetscInt initialsize);
-PETSC_EXTERN PetscErrorCode EPSGDGetInitialSize(EPS eps,PetscInt *initialsize);
-PETSC_EXTERN PetscErrorCode EPSGDSetBOrth(EPS eps,EPSOrthType borth);
-PETSC_EXTERN PetscErrorCode EPSGDGetBOrth(EPS eps,EPSOrthType *borth);
-PETSC_EXTERN PetscErrorCode EPSGDGetWindowSizes(EPS eps,PetscInt *pwindow,PetscInt *qwindow);
-PETSC_EXTERN PetscErrorCode EPSGDSetWindowSizes(EPS eps,PetscInt pwindow,PetscInt qwindow);
-PETSC_EXTERN PetscErrorCode EPSGDSetDoubleExpansion(EPS eps,PetscBool use_gd2);
-PETSC_EXTERN PetscErrorCode EPSGDGetDoubleExpansion(EPS eps,PetscBool *flg);
-
-PETSC_EXTERN PetscErrorCode EPSJDSetKrylovStart(EPS eps,PetscBool krylovstart);
-PETSC_EXTERN PetscErrorCode EPSJDGetKrylovStart(EPS eps,PetscBool *krylovstart);
-PETSC_EXTERN PetscErrorCode EPSJDSetBlockSize(EPS eps,PetscInt blocksize);
-PETSC_EXTERN PetscErrorCode EPSJDGetBlockSize(EPS eps,PetscInt *blocksize);
-PETSC_EXTERN PetscErrorCode EPSJDSetRestart(EPS eps,PetscInt minv,PetscInt plusk);
-PETSC_EXTERN PetscErrorCode EPSJDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk);
-PETSC_EXTERN PetscErrorCode EPSJDSetInitialSize(EPS eps,PetscInt initialsize);
-PETSC_EXTERN PetscErrorCode EPSJDGetInitialSize(EPS eps,PetscInt *initialsize);
-PETSC_EXTERN PetscErrorCode EPSJDSetFix(EPS eps,PetscReal fix);
-PETSC_EXTERN PetscErrorCode EPSJDGetFix(EPS eps,PetscReal *fix);
-PETSC_EXTERN PetscErrorCode EPSJDSetConstCorrectionTol(EPS eps,PetscBool dynamic);
-PETSC_EXTERN PetscErrorCode EPSJDGetConstCorrectionTol(EPS eps,PetscBool *dynamic);
-PETSC_EXTERN PetscErrorCode EPSJDSetBOrth(EPS eps,EPSOrthType borth);
-PETSC_EXTERN PetscErrorCode EPSJDGetBOrth(EPS eps,EPSOrthType *borth);
-PETSC_EXTERN PetscErrorCode EPSJDGetWindowSizes(EPS eps,PetscInt *pwindow,PetscInt *qwindow);
-PETSC_EXTERN PetscErrorCode EPSJDSetWindowSizes(EPS eps,PetscInt pwindow,PetscInt qwindow);
+PETSC_EXTERN PetscErrorCode EPSJDSetKrylovStart(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSJDGetKrylovStart(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSJDSetBlockSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSJDGetBlockSize(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSJDSetRestart(EPS,PetscInt,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSJDGetRestart(EPS,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSJDSetInitialSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSJDGetInitialSize(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSJDSetFix(EPS,PetscReal);
+PETSC_EXTERN PetscErrorCode EPSJDGetFix(EPS,PetscReal*);
+PETSC_EXTERN PetscErrorCode EPSJDSetConstCorrectionTol(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSJDGetConstCorrectionTol(EPS,PetscBool*);
+PETSC_EXTERN PetscErrorCode EPSJDSetBOrth(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSJDGetBOrth(EPS,PetscBool*);
 
 PETSC_EXTERN PetscErrorCode EPSRQCGSetReset(EPS,PetscInt);
 PETSC_EXTERN PetscErrorCode EPSRQCGGetReset(EPS,PetscInt*);
 
+PETSC_EXTERN PetscErrorCode EPSLOBPCGSetBlockSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSLOBPCGGetBlockSize(EPS,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSLOBPCGSetRestart(EPS,PetscReal);
+PETSC_EXTERN PetscErrorCode EPSLOBPCGGetRestart(EPS,PetscReal*);
+PETSC_EXTERN PetscErrorCode EPSLOBPCGSetLocking(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSLOBPCGGetLocking(EPS,PetscBool*);
+
+/*E
+    EPSCISSQuadRule - determines the quadrature rule in the CISS solver
+
+    Level: advanced
+
+.seealso: EPSCISSSetQuadRule(), EPSCISSGetQuadRule()
+E*/
+typedef enum { EPS_CISS_QUADRULE_TRAPEZOIDAL=1,
+               EPS_CISS_QUADRULE_CHEBYSHEV } EPSCISSQuadRule;
+PETSC_EXTERN const char *EPSCISSQuadRules[];
+
+/*E
+    EPSCISSExtraction - determines the extraction technique in the CISS solver
+
+    Level: advanced
+
+.seealso: EPSCISSSetExtraction(), EPSCISSGetExtraction()
+E*/
+typedef enum { EPS_CISS_EXTRACTION_RITZ,
+               EPS_CISS_EXTRACTION_HANKEL } EPSCISSExtraction;
+PETSC_EXTERN const char *EPSCISSExtractions[];
+
+PETSC_EXTERN PetscErrorCode EPSCISSSetExtraction(EPS,EPSCISSExtraction);
+PETSC_EXTERN PetscErrorCode EPSCISSGetExtraction(EPS,EPSCISSExtraction*);
+PETSC_EXTERN PetscErrorCode EPSCISSSetQuadRule(EPS,EPSCISSQuadRule);
+PETSC_EXTERN PetscErrorCode EPSCISSGetQuadRule(EPS,EPSCISSQuadRule*);
 PETSC_EXTERN PetscErrorCode EPSCISSSetRegion(EPS,PetscScalar,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode EPSCISSGetRegion(EPS,PetscScalar*,PetscReal*,PetscReal*);
 PETSC_EXTERN PetscErrorCode EPSCISSSetSizes(EPS,PetscInt,PetscInt,PetscInt,PetscInt,PetscInt,PetscBool);
 PETSC_EXTERN PetscErrorCode EPSCISSGetSizes(EPS,PetscInt*,PetscInt*,PetscInt*,PetscInt*,PetscInt*,PetscBool*);
 PETSC_EXTERN PetscErrorCode EPSCISSSetThreshold(EPS,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode EPSCISSGetThreshold(EPS,PetscReal*,PetscReal*);
-PETSC_EXTERN PetscErrorCode EPSCISSSetRefinement(EPS,PetscInt,PetscInt,PetscInt);
-PETSC_EXTERN PetscErrorCode EPSCISSGetRefinement(EPS,PetscInt*,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSCISSSetRefinement(EPS,PetscInt,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSCISSGetRefinement(EPS,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode EPSCISSSetUseST(EPS,PetscBool);
+PETSC_EXTERN PetscErrorCode EPSCISSGetUseST(EPS,PetscBool*);
+
+PETSC_EXTERN PetscErrorCode EPSBLOPEXSetBlockSize(EPS,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSBLOPEXGetBlockSize(EPS,PetscInt*);
 
 PETSC_EXTERN PetscErrorCode EPSFEASTSetNumPoints(EPS,PetscInt);
 PETSC_EXTERN PetscErrorCode EPSFEASTGetNumPoints(EPS,PetscInt*);
