@@ -1,28 +1,22 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   User interface for the mathematical function object in SLEPc
+*/
 
 #if !defined(__SLEPCFN_H)
 #define __SLEPCFN_H
 #include <slepcsys.h>
 
+#define FN_MAX_SOLVE 6
+
 PETSC_EXTERN PetscErrorCode FNInitializePackage(void);
 /*S
    FN - Abstraction of a mathematical function.
@@ -64,6 +58,17 @@ typedef enum { FN_COMBINE_ADD,
                FN_COMBINE_DIVIDE,
                FN_COMBINE_COMPOSE } FNCombineType;
 
+/*E
+    FNParallelType - Indicates the parallel mode that will be used for matrix evaluation
+
+    Level: advanced
+
+.seealso: FNSetParallel()
+E*/
+typedef enum { FN_PARALLEL_REDUNDANT,
+               FN_PARALLEL_SYNCHRONIZED } FNParallelType;
+PETSC_EXTERN const char *FNParallelTypes[];
+
 PETSC_EXTERN PetscErrorCode FNCreate(MPI_Comm,FN*);
 PETSC_EXTERN PetscErrorCode FNSetType(FN,FNType);
 PETSC_EXTERN PetscErrorCode FNGetType(FN,FNType*);
@@ -77,6 +82,10 @@ PETSC_EXTERN PetscErrorCode FNDuplicate(FN,MPI_Comm,FN*);
 
 PETSC_EXTERN PetscErrorCode FNSetScale(FN,PetscScalar,PetscScalar);
 PETSC_EXTERN PetscErrorCode FNGetScale(FN,PetscScalar*,PetscScalar*);
+PETSC_EXTERN PetscErrorCode FNSetMethod(FN,PetscInt);
+PETSC_EXTERN PetscErrorCode FNGetMethod(FN,PetscInt*);
+PETSC_EXTERN PetscErrorCode FNSetParallel(FN,FNParallelType);
+PETSC_EXTERN PetscErrorCode FNGetParallel(FN,FNParallelType*);
 
 PETSC_EXTERN PetscErrorCode FNEvaluateFunction(FN,PetscScalar,PetscScalar*);
 PETSC_EXTERN PetscErrorCode FNEvaluateDerivative(FN,PetscScalar,PetscScalar*);
