@@ -1,25 +1,15 @@
+/*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
 /*
    SLEPc mathematics include file. Defines basic operations and functions.
    This file is included by slepcsys.h and should not be used directly.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #if !defined(__SLEPCMATH_H)
@@ -95,3 +85,35 @@ M*/
 
 #endif
 
+
+/*
+   SlepcSetFlushToZero - Set the FTZ flag in floating-point arithmetic.
+*/
+PETSC_STATIC_INLINE PetscErrorCode SlepcSetFlushToZero(unsigned int *state)
+{
+  PetscFunctionBegin;
+#if defined(PETSC_HAVE_XMMINTRIN_H)
+#if defined(_MM_FLUSH_ZERO_ON)
+  *state = _MM_GET_FLUSH_ZERO_MODE();
+  _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
+#else
+  *state = 0;
+#endif
+#endif
+  PetscFunctionReturn(0);
+}
+
+/*
+   SlepcResetFlushToZero - Reset the FTZ flag in floating-point arithmetic.
+*/
+PETSC_STATIC_INLINE PetscErrorCode SlepcResetFlushToZero(unsigned int *state)
+{
+  PetscFunctionBegin;
+#if defined(PETSC_HAVE_XMMINTRIN_H)
+#if defined(_MM_FLUSH_ZERO_MASK)
+  _MM_SET_FLUSH_ZERO_MODE(*state & _MM_FLUSH_ZERO_MASK);
+#endif
+#endif
+  PetscFunctionReturn(0);
+}
+
