@@ -1,25 +1,15 @@
 /*
-   User interface for SLEPc's nonlinear eigenvalue solvers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   User interface for SLEPc's nonlinear eigenvalue solvers
+*/
 
 #if !defined(__SLEPCNEP_H)
 #define __SLEPCNEP_H
@@ -57,6 +47,17 @@ typedef const char* NEPType;
 /* Logging support */
 PETSC_EXTERN PetscClassId NEP_CLASSID;
 
+/*E
+    NEPProblemType - Determines the type of the nonlinear eigenproblem
+
+    Level: intermediate
+
+.seealso: NEPSetProblemType(), NEPGetProblemType()
+E*/
+typedef enum { NEP_GENERAL=1,
+               NEP_RATIONAL     /* NEP defined in split form with all f_i rational */
+             } NEPProblemType;
+
 /*E
     NEPWhich - Determines which part of the spectrum is requested
 
@@ -158,6 +159,8 @@ PETSC_EXTERN PetscErrorCode NEPDestroy(NEP*);
 PETSC_EXTERN PetscErrorCode NEPReset(NEP);
 PETSC_EXTERN PetscErrorCode NEPSetType(NEP,NEPType);
 PETSC_EXTERN PetscErrorCode NEPGetType(NEP,NEPType*);
+PETSC_EXTERN PetscErrorCode NEPSetProblemType(NEP,NEPProblemType);
+PETSC_EXTERN PetscErrorCode NEPGetProblemType(NEP,NEPProblemType*);
 PETSC_EXTERN PetscErrorCode NEPSetTarget(NEP,PetscScalar);
 PETSC_EXTERN PetscErrorCode NEPGetTarget(NEP,PetscScalar*);
 PETSC_EXTERN PetscErrorCode NEPSetFromOptions(NEP);
@@ -283,8 +286,8 @@ PETSC_EXTERN PetscErrorCode NEPCISSGetRefinement(NEP,PetscInt*,PetscInt*);
 
 PETSC_EXTERN PetscErrorCode NEPInterpolSetPEP(NEP,PEP);
 PETSC_EXTERN PetscErrorCode NEPInterpolGetPEP(NEP,PEP*);
-PETSC_EXTERN PetscErrorCode NEPInterpolSetDegree(NEP,PetscInt);
-PETSC_EXTERN PetscErrorCode NEPInterpolGetDegree(NEP,PetscInt*);
+PETSC_EXTERN PetscErrorCode NEPInterpolSetInterpolation(NEP,PetscReal,PetscInt);
+PETSC_EXTERN PetscErrorCode NEPInterpolGetInterpolation(NEP,PetscReal*,PetscInt*);
 
 PETSC_EXTERN PetscErrorCode NEPNLEIGSSetSingularitiesFunction(NEP,PetscErrorCode (*)(NEP,PetscInt*,PetscScalar*,void*),void*);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSGetSingularitiesFunction(NEP,PetscErrorCode (**)(NEP,PetscInt*,PetscScalar*,void*),void **);
@@ -294,8 +297,6 @@ PETSC_EXTERN PetscErrorCode NEPNLEIGSSetLocking(NEP,PetscBool);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSGetLocking(NEP,PetscBool*);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSSetInterpolation(NEP,PetscReal,PetscInt);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSGetInterpolation(NEP,PetscReal*,PetscInt*);
-PETSC_EXTERN PetscErrorCode NEPNLEIGSSetTrueResidual(NEP,PetscBool);
-PETSC_EXTERN PetscErrorCode NEPNLEIGSGetTrueResidual(NEP,PetscBool*);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSSetRKShifts(NEP,PetscInt,PetscScalar*);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSGetRKShifts(NEP,PetscInt*,PetscScalar**);
 PETSC_EXTERN PetscErrorCode NEPNLEIGSGetKSPs(NEP,KSP**);
