@@ -1,25 +1,15 @@
 /*
-   User interface for SLEPc's singular value solvers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   User interface for SLEPc's singular value solvers
+*/
 
 #if !defined(__SLEPCSVD_H)
 #define __SLEPCSVD_H
@@ -52,6 +42,7 @@ typedef const char* SVDType;
 #define SVDLAPACK      "lapack"
 #define SVDLANCZOS     "lanczos"
 #define SVDTRLANCZOS   "trlanczos"
+#define SVDPRIMME      "primme"
 
 /* Logging support */
 PETSC_EXTERN PetscClassId SVD_CLASSID;
@@ -125,8 +116,9 @@ PETSC_EXTERN PetscErrorCode SVDSetType(SVD,SVDType);
 PETSC_EXTERN PetscErrorCode SVDGetType(SVD,SVDType*);
 PETSC_EXTERN PetscErrorCode SVDSetOperator(SVD,Mat);
 PETSC_EXTERN PetscErrorCode SVDGetOperator(SVD,Mat*);
-PETSC_EXTERN PetscErrorCode SVDSetInitialSpace(SVD,PetscInt,Vec*);
-PETSC_EXTERN PetscErrorCode SVDSetInitialSpaceLeft(SVD,PetscInt,Vec*);
+PETSC_EXTERN PetscErrorCode SVDSetInitialSpaces(SVD,PetscInt,Vec*,PetscInt,Vec*);
+PETSC_DEPRECATED("Use SVDSetInitialSpaces()") PETSC_STATIC_INLINE PetscErrorCode SVDSetInitialSpace(SVD svd,PetscInt nr,Vec *isr) {return SVDSetInitialSpaces(svd,nr,isr,0,NULL);}
+PETSC_DEPRECATED("Use SVDSetInitialSpaces()") PETSC_STATIC_INLINE PetscErrorCode SVDSetInitialSpaceLeft(SVD svd,PetscInt nl,Vec *isl) {return SVDSetInitialSpaces(svd,0,NULL,nl,isl);}
 PETSC_EXTERN PetscErrorCode SVDSetImplicitTranspose(SVD,PetscBool);
 PETSC_EXTERN PetscErrorCode SVDGetImplicitTranspose(SVD,PetscBool*);
 PETSC_EXTERN PetscErrorCode SVDSetDimensions(SVD,PetscInt,PetscInt,PetscInt);
@@ -187,6 +179,15 @@ PETSC_EXTERN PetscErrorCode SVDMonitorLGAll(SVD,PetscInt,PetscInt,PetscReal*,Pet
 PETSC_EXTERN PetscErrorCode SVDSetTrackAll(SVD,PetscBool);
 PETSC_EXTERN PetscErrorCode SVDGetTrackAll(SVD,PetscBool*);
 
+PETSC_EXTERN PetscFunctionList SVDList;
+PETSC_EXTERN PetscErrorCode SVDRegister(const char[],PetscErrorCode(*)(SVD));
+
+PETSC_EXTERN PetscErrorCode SVDAllocateSolution(SVD,PetscInt);
+
+/* --------- options specific to particular solvers -------- */
+
+PETSC_EXTERN PetscErrorCode SVDCrossSetExplicitMatrix(SVD,PetscBool);
+PETSC_EXTERN PetscErrorCode SVDCrossGetExplicitMatrix(SVD,PetscBool*);
 PETSC_EXTERN PetscErrorCode SVDCrossSetEPS(SVD,EPS);
 PETSC_EXTERN PetscErrorCode SVDCrossGetEPS(SVD,EPS*);
 
@@ -201,9 +202,21 @@ PETSC_EXTERN PetscErrorCode SVDLanczosGetOneSide(SVD,PetscBool*);
 PETSC_EXTERN PetscErrorCode SVDTRLanczosSetOneSide(SVD,PetscBool);
 PETSC_EXTERN PetscErrorCode SVDTRLanczosGetOneSide(SVD,PetscBool*);
 
-PETSC_EXTERN PetscFunctionList SVDList;
-PETSC_EXTERN PetscErrorCode SVDRegister(const char[],PetscErrorCode(*)(SVD));
+/*E
+    SVDPRIMMEMethod - determines the SVD method selected in the PRIMME library
 
-PETSC_EXTERN PetscErrorCode SVDAllocateSolution(SVD,PetscInt);
+    Level: advanced
+
+.seealso: SVDPRIMMESetMethod(), SVDPRIMMEGetMethod()
+E*/
+typedef enum { SVD_PRIMME_HYBRID=1,
+               SVD_PRIMME_NORMALEQUATIONS,
+               SVD_PRIMME_AUGMENTED } SVDPRIMMEMethod;
+PETSC_EXTERN const char *SVDPRIMMEMethods[];
+
+PETSC_EXTERN PetscErrorCode SVDPRIMMESetBlockSize(SVD,PetscInt);
+PETSC_EXTERN PetscErrorCode SVDPRIMMEGetBlockSize(SVD,PetscInt*);
+PETSC_EXTERN PetscErrorCode SVDPRIMMESetMethod(SVD,SVDPRIMMEMethod);
+PETSC_EXTERN PetscErrorCode SVDPRIMMEGetMethod(SVD,SVDPRIMMEMethod*);
 
 #endif
