@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "lobpcg"
 
    Method: Locally Optimal Block Preconditioned Conjugate Gradient
@@ -18,25 +26,6 @@
        [2] A. V. Knyazev et al., "Block Locally Optimal Preconditioned
            Eigenvalue Xolvers (BLOPEX) in Hypre and PETSc", SIAM J. Sci.
            Comput. 29(5):2224-2239, 2007.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
@@ -47,8 +36,6 @@ typedef struct {
   PetscReal restart;   /* restart parameter */
 } EPS_LOBPCG;
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetDimensions_LOBPCG"
 PetscErrorCode EPSSetDimensions_LOBPCG(EPS eps,PetscInt nev,PetscInt *ncv,PetscInt *mpd)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
@@ -64,13 +51,11 @@ PetscErrorCode EPSSetDimensions_LOBPCG(EPS eps,PetscInt nev,PetscInt *ncv,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_LOBPCG"
 PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
-  PetscBool      precond,istrivial;
+  PetscBool      istrivial;
 
   PetscFunctionBegin;
   if (!eps->ishermitian || (eps->isgeneralized && !eps->ispositive)) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"LOBPCG only works for Hermitian problems");
@@ -79,7 +64,7 @@ PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
   ierr = EPSSetDimensions_LOBPCG(eps,eps->nev,&eps->ncv,&eps->mpd);CHKERRQ(ierr);
   if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
   if (!eps->which) eps->which = EPS_SMALLEST_REAL;
-  if (eps->which!=EPS_SMALLEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
+  if (eps->which!=EPS_SMALLEST_REAL && eps->which!=EPS_LARGEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
   if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
   ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
@@ -87,10 +72,6 @@ PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
 
   if (!ctx->restart) ctx->restart = 0.9;
 
-  ierr = STSetUp(eps->st);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond);CHKERRQ(ierr);
-  if (!precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"LOBPCG only works with precond ST");
-
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
   ierr = EPS_SetInnerProduct(eps);CHKERRQ(ierr);
   ierr = DSSetType(eps->ds,DSGHEP);CHKERRQ(ierr);
@@ -99,29 +80,37 @@ PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_LOBPCG"
 PetscErrorCode EPSSolve_LOBPCG(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
-  PetscInt       i,j,k,ld,nv,ini,kini,nmat,nc,nconv,locked,guard,its;
+  PetscInt       i,j,k,ld,nv,ini,nmat,nc,nconv,locked,guard,its;
   PetscReal      norm;
-  PetscScalar    *eigr;
-  PetscBool      breakdown,countc;
+  PetscScalar    *eigr,dot;
+  PetscBool      breakdown,countc,flip=PETSC_FALSE,checkprecond=PETSC_FALSE;
   Mat            A,B,M;
-  Vec            v,w=eps->work[0];
+  Vec            v,z,w=eps->work[0];
   BV             X,Y,Z,R,P,AX,BX;
+  SlepcSC        sc;
 
   PetscFunctionBegin;
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
-  if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  if (nmat>1) { ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr); }
   else B = NULL;
 
   guard = (PetscInt)((1.0-ctx->restart)*ctx->bs);  /* number of guard vectors */
 
+  if (eps->which==EPS_LARGEST_REAL) {  /* flip spectrum */
+    flip = PETSC_TRUE;
+    ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
+    sc->comparison = SlepcCompareSmallestReal;
+  }
+
+  /* undocumented option to check for a positive-definite preconditioner (turn-off by default) */
+  ierr = PetscOptionsGetBool(NULL,NULL,"-eps_lobpcg_checkprecond",&checkprecond,NULL);CHKERRQ(ierr);
+
   /* 1. Allocate memory */
   ierr = PetscCalloc1(3*ctx->bs,&eigr);CHKERRQ(ierr);
   ierr = BVDuplicateResize(eps->V,3*ctx->bs,&Z);CHKERRQ(ierr);
@@ -146,14 +135,10 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
   }
 
   /* 2. Apply the constraints to the initial vectors */
-  kini = eps->nini;
-  while (kini<eps->ncv-ctx->bs) { /* Generate more initial vectors if necessary */
-    ierr = BVSetRandomColumn(eps->V,kini);CHKERRQ(ierr);
-    ierr = BVOrthogonalizeColumn(eps->V,kini,NULL,&norm,&breakdown);CHKERRQ(ierr);
-    if (norm>0.0 && !breakdown) {
-      ierr = BVScaleColumn(eps->V,kini,1.0/norm);CHKERRQ(ierr);
-      kini++;
-    }
+  /* 3. B-orthogonalize initial vectors */
+  for (k=eps->nini;k<eps->ncv-ctx->bs;k++) { /* Generate more initial vectors if necessary */
+    ierr = BVSetRandomColumn(eps->V,k);CHKERRQ(ierr);
+    ierr = BVOrthonormalizeColumn(eps->V,k,PETSC_TRUE,NULL,NULL);CHKERRQ(ierr);
   }
   nv = ctx->bs;
   ierr = BVSetActiveColumns(eps->V,0,nv);CHKERRQ(ierr);
@@ -161,23 +146,24 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
   ierr = BVCopy(eps->V,Z);CHKERRQ(ierr);
   ierr = BVCopy(Z,X);CHKERRQ(ierr);
 
-  /* 3. B-orthogonalize initial vectors */
-
   /* 4. Compute initial Ritz vectors */
   ierr = BVMatMult(X,A,AX);CHKERRQ(ierr);
   ierr = DSSetDimensions(eps->ds,nv,0,0,0);CHKERRQ(ierr);
   ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
   ierr = BVMatProject(AX,NULL,X,M);CHKERRQ(ierr);
+  if (flip) { ierr = MatScale(M,-1.0);CHKERRQ(ierr); }
   ierr = DSRestoreMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
   ierr = DSSetIdentity(eps->ds,DS_MAT_B);CHKERRQ(ierr);
   ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
-  ierr = DSSolve(eps->ds,eps->eigr,NULL);CHKERRQ(ierr);
-  ierr = DSSort(eps->ds,eps->eigr,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = DSSolve(eps->ds,eigr,NULL);CHKERRQ(ierr);
+  ierr = DSSort(eps->ds,eigr,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = DSSynchronize(eps->ds,eigr,NULL);CHKERRQ(ierr);
+  for (j=0;j<nv;j++) eps->eigr[j] = flip? -eigr[j]: eigr[j];
   ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
   ierr = DSGetMat(eps->ds,DS_MAT_X,&M);CHKERRQ(ierr);
   ierr = BVMultInPlace(X,M,0,nv);CHKERRQ(ierr);
   ierr = BVMultInPlace(AX,M,0,nv);CHKERRQ(ierr);
-  ierr = DSRestoreMat(eps->ds,DS_MAT_X,&M);CHKERRQ(ierr);
+  ierr = MatDestroy(&M);CHKERRQ(ierr);
 
   /* 5. Initialize range of active iterates */
   locked = 0;  /* hard-locked vectors, the leading locked columns of V are eigenvectors */
@@ -196,18 +182,18 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     }
 
     /* 7. Compute residuals */
-    ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
+    ini = (ctx->lock)? nconv: 0;
     ierr = BVCopy(AX,R);CHKERRQ(ierr);
-    if (B) {
-      ierr = BVMatMult(X,B,BX);CHKERRQ(ierr);
-      ierr = BVMult(R,-1.0,1.0,BX,M);CHKERRQ(ierr);
-    } else {
-      ierr = BVMult(R,-1.0,1.0,X,M);CHKERRQ(ierr);
+    if (B) { ierr = BVMatMult(X,B,BX);CHKERRQ(ierr); }
+    for (j=ini;j<ctx->bs;j++) {
+      ierr = BVGetColumn(R,j,&v);CHKERRQ(ierr);
+      ierr = BVGetColumn(B?BX:X,j,&z);CHKERRQ(ierr);
+      ierr = VecAXPY(v,-eps->eigr[locked+j],z);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(R,j,&v);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(B?BX:X,j,&z);CHKERRQ(ierr);
     }
-    ierr = DSRestoreMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
 
     /* 8. Compute residual norms and update index set of active iterates */
-    ini = (ctx->lock)? nconv: 0;
     k = ini;
     countc = PETSC_TRUE;
     for (j=ini;j<ctx->bs;j++) {
@@ -234,10 +220,9 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       ierr = BVCopy(X,eps->V);CHKERRQ(ierr);
     }
     if (eps->reason != EPS_CONVERGED_ITERATING) {
-      eps->its += its;
       break;
     } else if (nconv >= ctx->bs-guard) {
-      eps->its += its;
+      eps->its += its-1;
       its = 0;
     } else its++;
 
@@ -246,12 +231,6 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       /* extend constraints */
       ierr = BVSetActiveColumns(Y,nc+locked,nc+locked+nconv);CHKERRQ(ierr);
       ierr = BVCopy(X,Y);CHKERRQ(ierr);
-      for (j=0;j<nconv;j++) {
-        ierr = BVOrthogonalizeColumn(Y,nc+locked+j,NULL,&norm,&breakdown);CHKERRQ(ierr);
-        if (norm>0.0 && !breakdown) {
-          ierr = BVScaleColumn(Y,nc+locked+j,1.0/norm);CHKERRQ(ierr);
-        } else SETERRQ(PetscObjectComm((PetscObject)eps),1,"Orthogonalization of constraints failed");
-      }
       ierr = BVSetActiveColumns(Y,0,nc+locked+nconv);CHKERRQ(ierr);
 
       /* shift work BV's */
@@ -274,7 +253,11 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
         ierr = BVOrthogonalizeVec(Y,v,NULL,&norm,&breakdown);CHKERRQ(ierr);
         if (norm>0.0 && !breakdown) {
           ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
-        } else SETERRQ(PetscObjectComm((PetscObject)eps),1,"Orthogonalization of initial vector failed");
+        } else {
+          ierr = PetscInfo(eps,"Orthogonalization of initial vector failed\n");CHKERRQ(ierr);
+          eps->reason = EPS_DIVERGED_BREAKDOWN;
+          goto diverged;
+        }
         ierr = BVRestoreColumn(X,j,&v);CHKERRQ(ierr);
       }
       locked += nconv;
@@ -293,17 +276,19 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       ierr = DSSetDimensions(eps->ds,nv,0,0,0);CHKERRQ(ierr);
       ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
       ierr = BVMatProject(AX,NULL,X,M);CHKERRQ(ierr);
+      if (flip) { ierr = MatScale(M,-1.0);CHKERRQ(ierr); }
       ierr = DSRestoreMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
       ierr = DSSetIdentity(eps->ds,DS_MAT_B);CHKERRQ(ierr);
       ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
       ierr = DSSolve(eps->ds,eigr,NULL);CHKERRQ(ierr);
       ierr = DSSort(eps->ds,eigr,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
-      for (j=0;j<nv;j++) if (locked+j<eps->ncv) eps->eigr[locked+j] = eigr[j];
+      ierr = DSSynchronize(eps->ds,eigr,NULL);CHKERRQ(ierr);
+      for (j=0;j<nv;j++) if (locked+j<eps->ncv) eps->eigr[locked+j] = flip? -eigr[j]: eigr[j];
       ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
       ierr = DSGetMat(eps->ds,DS_MAT_X,&M);CHKERRQ(ierr);
       ierr = BVMultInPlace(X,M,0,nv);CHKERRQ(ierr);
       ierr = BVMultInPlace(AX,M,0,nv);CHKERRQ(ierr);
-      ierr = DSRestoreMat(eps->ds,DS_MAT_X,&M);CHKERRQ(ierr);
+      ierr = MatDestroy(&M);CHKERRQ(ierr);
 
       continue;   /* skip the rest of the iteration */
     }
@@ -322,11 +307,23 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     for (j=ini;j<ctx->bs;j++) {
       ierr = BVGetColumn(R,j,&v);CHKERRQ(ierr);
       ierr = STMatSolve(eps->st,v,w);CHKERRQ(ierr);
+      if (checkprecond) {
+        ierr = VecDot(v,w,&dot);CHKERRQ(ierr);
+        if (PetscRealPart(dot)<0.0) {
+          ierr = PetscInfo(eps,"The preconditioner is not positive-definite\n");CHKERRQ(ierr);
+          eps->reason = EPS_DIVERGED_BREAKDOWN;
+          goto diverged;
+        }
+      }
       if (nc+locked>0) {
         ierr = BVOrthogonalizeVec(Y,w,NULL,&norm,&breakdown);CHKERRQ(ierr);
         if (norm>0.0 && !breakdown) {
           ierr = VecScale(w,1.0/norm);CHKERRQ(ierr);
-        } else SETERRQ(PetscObjectComm((PetscObject)eps),1,"Orthogonalization of preconditioned residual failed");
+        } else {
+          ierr = PetscInfo(eps,"Orthogonalization of preconditioned residual failed\n");CHKERRQ(ierr);
+          eps->reason = EPS_DIVERGED_BREAKDOWN;
+          goto diverged;
+        }
       }
       ierr = VecCopy(w,v);CHKERRQ(ierr);
       ierr = BVRestoreColumn(R,j,&v);CHKERRQ(ierr);
@@ -358,22 +355,20 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     ierr = DSSetDimensions(eps->ds,nv,0,0,0);CHKERRQ(ierr);
     ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
     ierr = BVMatProject(Z,A,Z,M);CHKERRQ(ierr);
+    if (flip) { ierr = MatScale(M,-1.0);CHKERRQ(ierr); }
     ierr = DSRestoreMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
     ierr = DSGetMat(eps->ds,DS_MAT_B,&M);CHKERRQ(ierr);
-    if (B) {
-      ierr = BVMatProject(Z,B,Z,M);CHKERRQ(ierr);
-    } else {
-      ierr = BVDot(Z,Z,M);CHKERRQ(ierr);
-    }
+    ierr = BVMatProject(Z,B,Z,M);CHKERRQ(ierr); /* covers also the case B=NULL */
     ierr = DSRestoreMat(eps->ds,DS_MAT_B,&M);CHKERRQ(ierr);
-    
+
     /* 24. Solve the generalized eigenvalue problem */
     ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
     ierr = DSSolve(eps->ds,eigr,NULL);CHKERRQ(ierr);
     ierr = DSSort(eps->ds,eigr,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
-    for (j=0;j<nv;j++) if (locked+j<eps->ncv) eps->eigr[locked+j] = eigr[j];
+    ierr = DSSynchronize(eps->ds,eigr,NULL);CHKERRQ(ierr);
+    for (j=0;j<nv;j++) if (locked+j<eps->ncv) eps->eigr[locked+j] = flip? -eigr[j]: eigr[j];
     ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
-    
+
     /* 25-33. Compute Ritz vectors */
     ierr = DSGetMat(eps->ds,DS_MAT_X,&M);CHKERRQ(ierr);
     ierr = BVSetActiveColumns(Z,ctx->bs,nv);CHKERRQ(ierr);
@@ -391,9 +386,13 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       ierr = BVSetActiveColumns(X,nconv,ctx->bs);CHKERRQ(ierr);
     }
     ierr = BVMatMult(X,A,AX);CHKERRQ(ierr);
-    ierr = DSRestoreMat(eps->ds,DS_MAT_X,&M);CHKERRQ(ierr);
+    ierr = MatDestroy(&M);CHKERRQ(ierr);
   }
 
+diverged:
+  eps->its += its;
+
+  if (flip) sc->comparison = SlepcCompareLargestReal;
   ierr = PetscFree(eigr);CHKERRQ(ierr);
   ierr = BVDestroy(&Z);CHKERRQ(ierr);
   ierr = BVDestroy(&X);CHKERRQ(ierr);
@@ -409,19 +408,20 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGSetBlockSize_LOBPCG"
 static PetscErrorCode EPSLOBPCGSetBlockSize_LOBPCG(EPS eps,PetscInt bs)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
 
   PetscFunctionBegin;
-  ctx->bs = bs;
+  if (bs == PETSC_DEFAULT || bs == PETSC_DECIDE) bs = 1;
+  if (bs <= 0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid block size %D",bs);
+  if (ctx->bs != bs) {
+    ctx->bs = bs;
+    eps->state = EPS_STATE_INITIAL;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGSetBlockSize"
 /*@
    EPSLOBPCGSetBlockSize - Sets the block size of the LOBPCG method.
 
@@ -449,8 +449,6 @@ PetscErrorCode EPSLOBPCGSetBlockSize(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGGetBlockSize_LOBPCG"
 static PetscErrorCode EPSLOBPCGGetBlockSize_LOBPCG(EPS eps,PetscInt *bs)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
@@ -460,8 +458,6 @@ static PetscErrorCode EPSLOBPCGGetBlockSize_LOBPCG(EPS eps,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGGetBlockSize"
 /*@
    EPSLOBPCGGetBlockSize - Gets the block size used in the LOBPCG method.
 
@@ -488,8 +484,6 @@ PetscErrorCode EPSLOBPCGGetBlockSize(EPS eps,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGSetRestart_LOBPCG"
 static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
@@ -497,14 +491,12 @@ static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
   PetscFunctionBegin;
   if (restart==PETSC_DEFAULT) ctx->restart = 0.6;
   else {
-    if (restart<0.1 || restart>1.0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The restart argument must be in the range [0.1,1.0]");
+    if (restart<0.1 || restart>1.0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The restart argument %g must be in the range [0.1,1.0]",(double)restart);
     ctx->restart = restart;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGSetRestart"
 /*@
    EPSLOBPCGSetRestart - Sets the restart parameter for the LOBPCG method.
    The meaning of this parameter is the proportion of vectors within the
@@ -538,8 +530,6 @@ PetscErrorCode EPSLOBPCGSetRestart(EPS eps,PetscReal restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGGetRestart_LOBPCG"
 static PetscErrorCode EPSLOBPCGGetRestart_LOBPCG(EPS eps,PetscReal *restart)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
@@ -549,8 +539,6 @@ static PetscErrorCode EPSLOBPCGGetRestart_LOBPCG(EPS eps,PetscReal *restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGGetRestart"
 /*@
    EPSLOBPCGGetRestart - Gets the restart parameter used in the LOBPCG method.
 
@@ -577,8 +565,6 @@ PetscErrorCode EPSLOBPCGGetRestart(EPS eps,PetscReal *restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGSetLocking_LOBPCG"
 static PetscErrorCode EPSLOBPCGSetLocking_LOBPCG(EPS eps,PetscBool lock)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
@@ -588,8 +574,6 @@ static PetscErrorCode EPSLOBPCGSetLocking_LOBPCG(EPS eps,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGSetLocking"
 /*@
    EPSLOBPCGSetLocking - Choose between locking and non-locking variants of
    the LOBPCG method.
@@ -623,8 +607,6 @@ PetscErrorCode EPSLOBPCGSetLocking(EPS eps,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGGetLocking_LOBPCG"
 static PetscErrorCode EPSLOBPCGGetLocking_LOBPCG(EPS eps,PetscBool *lock)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
@@ -634,8 +616,6 @@ static PetscErrorCode EPSLOBPCGGetLocking_LOBPCG(EPS eps,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLOBPCGGetLocking"
 /*@
    EPSLOBPCGGetLocking - Gets the locking flag used in the LOBPCG method.
 
@@ -662,8 +642,6 @@ PetscErrorCode EPSLOBPCGGetLocking(EPS eps,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_LOBPCG"
 PetscErrorCode EPSView_LOBPCG(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -673,54 +651,36 @@ PetscErrorCode EPSView_LOBPCG(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  LOBPCG: block size %D\n",ctx->bs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  LOBPCG: restart parameter=%g (using %d guard vectors)\n",(double)ctx->restart,(int)((1.0-ctx->restart)*ctx->bs));CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  LOBPCG: soft locking %sactivated\n",ctx->lock?"":"de");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  block size %D\n",ctx->bs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  restart parameter=%g (using %d guard vectors)\n",(double)ctx->restart,(int)((1.0-ctx->restart)*ctx->bs));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  soft locking %sactivated\n",ctx->lock?"":"de");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_LOBPCG"
 PetscErrorCode EPSSetFromOptions_LOBPCG(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode ierr;
   PetscBool      lock,flg;
   PetscInt       bs;
   PetscReal      restart;
-  KSP            ksp;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS LOBPCG Options");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-eps_lobpcg_blocksize","LOBPCG block size","EPSLOBPCGSetBlockSize",20,&bs,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSLOBPCGSetBlockSize(eps,bs);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsReal("-eps_lobpcg_restart","Percentage of the block of vectors to force a restart","EPSLOBPCGSetRestart",0.5,&restart,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSLOBPCGSetRestart(eps,restart);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsBool("-eps_lobpcg_locking","Choose between locking and non-locking variants","EPSLOBPCGSetLocking",PETSC_TRUE,&lock,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSLOBPCGSetLocking(eps,lock);CHKERRQ(ierr);
-  }
 
-  /* Set STPrecond as the default ST */
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  }
+    ierr = PetscOptionsInt("-eps_lobpcg_blocksize","Block size","EPSLOBPCGSetBlockSize",20,&bs,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSLOBPCGSetBlockSize(eps,bs);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsReal("-eps_lobpcg_restart","Percentage of the block of vectors to force a restart","EPSLOBPCGSetRestart",0.5,&restart,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSLOBPCGSetRestart(eps,restart);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsBool("-eps_lobpcg_locking","Choose between locking and non-locking variants","EPSLOBPCGSetLocking",PETSC_TRUE,&lock,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSLOBPCGSetLocking(eps,lock);CHKERRQ(ierr); }
 
-  /* Set the default options of the KSP */
-  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
-  if (!((PetscObject)ksp)->type_name) {
-    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
-  }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_LOBPCG"
 PetscErrorCode EPSDestroy_LOBPCG(EPS eps)
 {
   PetscErrorCode ierr;
@@ -736,8 +696,6 @@ PetscErrorCode EPSDestroy_LOBPCG(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_LOBPCG"
 PETSC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
 {
   EPS_LOBPCG     *lobpcg;
@@ -748,14 +706,17 @@ PETSC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
   eps->data = (void*)lobpcg;
   lobpcg->lock = PETSC_TRUE;
 
-  eps->ops->setup          = EPSSetUp_LOBPCG;
+  eps->useds = PETSC_TRUE;
+  eps->categ = EPS_CATEGORY_PRECOND;
+
   eps->ops->solve          = EPSSolve_LOBPCG;
+  eps->ops->setup          = EPSSetUp_LOBPCG;
   eps->ops->setfromoptions = EPSSetFromOptions_LOBPCG;
   eps->ops->destroy        = EPSDestroy_LOBPCG;
   eps->ops->view           = EPSView_LOBPCG;
   eps->ops->backtransform  = EPSBackTransform_Default;
-  ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+  eps->ops->setdefaultst   = EPSSetDefaultST_GMRES;
+
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGSetBlockSize_C",EPSLOBPCGSetBlockSize_LOBPCG);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGGetBlockSize_C",EPSLOBPCGGetBlockSize_LOBPCG);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGSetRestart_C",EPSLOBPCGSetRestart_LOBPCG);CHKERRQ(ierr);
