@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "rqcg"
 
    Method: Rayleigh Quotient Conjugate Gradient
@@ -14,25 +22,6 @@
        [1] L. Bergamaschi et al., "Parallel preconditioned conjugate gradient
            optimization of the Rayleigh quotient for the solution of sparse
            eigenproblems", Appl. Math. Comput. 175(2):1694-1715, 2006.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
@@ -40,16 +29,14 @@
 PetscErrorCode EPSSolve_RQCG(EPS);
 
 typedef struct {
-  PetscInt nrest;
+  PetscInt nrest;         /* user-provided reset parameter */
+  PetscInt allocsize;     /* number of columns of work BV's allocated at setup */
   BV       AV,W,P,G;
 } EPS_RQCG;
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_RQCG"
 PetscErrorCode EPSSetUp_RQCG(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscBool      precond;
   PetscInt       nmat;
   EPS_RQCG       *ctx = (EPS_RQCG*)eps->data;
 
@@ -63,36 +50,40 @@ PetscErrorCode EPSSetUp_RQCG(EPS eps)
     ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr);
   } else if (eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Unsupported extraction type");
   if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
-  /* Set STPrecond as the default ST */
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  }
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond);CHKERRQ(ierr);
-  if (!precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"RQCG only works with precond ST");
 
   if (!ctx->nrest) ctx->nrest = 20;
 
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
   ierr = EPS_SetInnerProduct(eps);CHKERRQ(ierr);
-  ierr = BVDuplicateResize(eps->V,eps->mpd,&ctx->AV);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->AV);CHKERRQ(ierr);
+
   ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  if (nmat>1) {
-    ierr = BVDuplicate(ctx->AV,&ctx->W);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->W);CHKERRQ(ierr);
+  if (!ctx->allocsize) {
+    ctx->allocsize = eps->mpd;
+    ierr = BVDuplicateResize(eps->V,eps->mpd,&ctx->AV);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->AV);CHKERRQ(ierr);
+    if (nmat>1) {
+      ierr = BVDuplicate(ctx->AV,&ctx->W);CHKERRQ(ierr);
+      ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->W);CHKERRQ(ierr);
+    }
+    ierr = BVDuplicate(ctx->AV,&ctx->P);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->P);CHKERRQ(ierr);
+    ierr = BVDuplicate(ctx->AV,&ctx->G);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->G);CHKERRQ(ierr);
+  } else if (ctx->allocsize!=eps->mpd) {
+    ctx->allocsize = eps->mpd;
+    ierr = BVResize(ctx->AV,eps->mpd,PETSC_FALSE);CHKERRQ(ierr);
+    if (nmat>1) {
+      ierr = BVResize(ctx->W,eps->mpd,PETSC_FALSE);CHKERRQ(ierr);
+    }
+    ierr = BVResize(ctx->P,eps->mpd,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = BVResize(ctx->G,eps->mpd,PETSC_FALSE);CHKERRQ(ierr);
   }
-  ierr = BVDuplicate(ctx->AV,&ctx->P);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->P);CHKERRQ(ierr);
-  ierr = BVDuplicate(ctx->AV,&ctx->G);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->G);CHKERRQ(ierr);
   ierr = DSSetType(eps->ds,DSHEP);CHKERRQ(ierr);
   ierr = DSAllocate(eps->ds,eps->ncv);CHKERRQ(ierr);
   ierr = EPSSetWorkVecs(eps,1);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ExtractSubmatrix"
 /*
    ExtractSubmatrix - Returns B = A(k+1:end,k+1:end).
 */
@@ -115,24 +106,22 @@ static PetscErrorCode ExtractSubmatrix(Mat A,PetscInt k,Mat *B)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_RQCG"
 PetscErrorCode EPSSolve_RQCG(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_RQCG       *ctx = (EPS_RQCG*)eps->data;
   PetscInt       i,j,k,ld,nv,ncv = eps->ncv,kini,nmat;
   PetscScalar    *C,*gamma,g,pap,pbp,pbx,pax,nu,mu,alpha,beta;
-  PetscReal      resnorm,norm,a,b,c,disc,t;
-  PetscBool      reset,breakdown;
+  PetscReal      resnorm,a,b,c,disc,t;
+  PetscBool      reset;
   Mat            A,B,Q,Q1;
   Vec            v,av,bv,p,w=eps->work[0];
 
   PetscFunctionBegin;
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
-  if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  if (nmat>1) { ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr); }
   else B = NULL;
   ierr = PetscMalloc1(eps->mpd,&gamma);CHKERRQ(ierr);
 
@@ -141,19 +130,14 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
     eps->its++;
     nv = PetscMin(eps->nconv+eps->mpd,ncv);
     ierr = DSSetDimensions(eps->ds,nv,0,eps->nconv,0);CHKERRQ(ierr);
-    /* Generate more initial vectors if necessary */
-    while (kini<nv) {
+    for (;kini<nv;kini++) { /* Generate more initial vectors if necessary */
       ierr = BVSetRandomColumn(eps->V,kini);CHKERRQ(ierr);
-      ierr = BVOrthogonalizeColumn(eps->V,kini,NULL,&norm,&breakdown);CHKERRQ(ierr);
-      if (norm>0.0 && !breakdown) {
-        ierr = BVScaleColumn(eps->V,kini,1.0/norm);CHKERRQ(ierr);
-        kini++;
-      }
+      ierr = BVOrthonormalizeColumn(eps->V,kini,PETSC_TRUE,NULL,NULL);CHKERRQ(ierr);
     }
     reset = (eps->its>1 && (eps->its-1)%ctx->nrest==0)? PETSC_TRUE: PETSC_FALSE;
 
     if (reset) {
-      /* Prevent BVDotVec below to use B-product, restored a the end */
+      /* Prevent BVDotVec below to use B-product, restored at the end */
       ierr = BVSetMatrix(eps->V,NULL,PETSC_FALSE);CHKERRQ(ierr);
 
       /* Compute Rayleigh quotient */
@@ -166,7 +150,7 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
         ierr = BVGetColumn(ctx->AV,i-eps->nconv,&av);CHKERRQ(ierr);
         ierr = BVDotVec(eps->V,av,C+eps->nconv+i*ld);CHKERRQ(ierr);
         ierr = BVRestoreColumn(ctx->AV,i-eps->nconv,&av);CHKERRQ(ierr);
-        for (j=eps->nconv;j<i-1;j++) C[i+j*ld] = C[j+i*ld];
+        for (j=eps->nconv;j<i-1;j++) C[i+j*ld] = PetscConj(C[j+i*ld]);
       }
       ierr = DSRestoreArray(eps->ds,DS_MAT_A,&C);CHKERRQ(ierr);
       ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
@@ -174,6 +158,7 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
       /* Solve projected problem */
       ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
       ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+      ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
       /* Update vectors V(:,idx) = V * Y(:,idx) */
       ierr = DSGetMat(eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
@@ -230,7 +215,7 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
       for (i=0;i<nv-eps->nconv;i++) {
         ierr = BVGetColumn(ctx->G,i,&v);CHKERRQ(ierr);
         ierr = STMatSolve(eps->st,v,w);CHKERRQ(ierr);
-        ierr = VecDot(v,w,&g);CHKERRQ(ierr);
+        ierr = VecDot(w,v,&g);CHKERRQ(ierr);
         ierr = BVRestoreColumn(ctx->G,i,&v);CHKERRQ(ierr);
         beta = (!reset && eps->its>1)? g/gamma[i]: 0.0;
         gamma[i] = g;
@@ -248,20 +233,20 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
         ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
         ierr = BVGetColumn(ctx->AV,i-eps->nconv,&av);CHKERRQ(ierr);
         ierr = BVGetColumn(ctx->P,i-eps->nconv,&p);CHKERRQ(ierr);
-        ierr = VecDot(v,av,&nu);CHKERRQ(ierr);
-        ierr = VecDot(p,av,&pax);CHKERRQ(ierr);
+        ierr = VecDot(av,v,&nu);CHKERRQ(ierr);
+        ierr = VecDot(av,p,&pax);CHKERRQ(ierr);
         ierr = MatMult(A,p,w);CHKERRQ(ierr);
-        ierr = VecDot(p,w,&pap);CHKERRQ(ierr);
+        ierr = VecDot(w,p,&pap);CHKERRQ(ierr);
         if (B) {
           ierr = BVGetColumn(ctx->W,i-eps->nconv,&bv);CHKERRQ(ierr);
-          ierr = VecDot(v,bv,&mu);CHKERRQ(ierr);
-          ierr = VecDot(p,bv,&pbx);CHKERRQ(ierr);
+          ierr = VecDot(bv,v,&mu);CHKERRQ(ierr);
+          ierr = VecDot(bv,p,&pbx);CHKERRQ(ierr);
           ierr = BVRestoreColumn(ctx->W,i-eps->nconv,&bv);CHKERRQ(ierr);
           ierr = MatMult(B,p,w);CHKERRQ(ierr);
-          ierr = VecDot(p,w,&pbp);CHKERRQ(ierr);
+          ierr = VecDot(w,p,&pbp);CHKERRQ(ierr);
         } else {
           ierr = VecDot(v,v,&mu);CHKERRQ(ierr);
-          ierr = VecDot(p,v,&pbx);CHKERRQ(ierr);
+          ierr = VecDot(v,p,&pbx);CHKERRQ(ierr);
           ierr = VecDot(p,p,&pbp);CHKERRQ(ierr);
         }
         ierr = BVRestoreColumn(ctx->AV,i-eps->nconv,&av);CHKERRQ(ierr);
@@ -280,10 +265,7 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
         }
         ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
         ierr = BVRestoreColumn(ctx->P,i-eps->nconv,&p);CHKERRQ(ierr);
-        ierr = BVOrthogonalizeColumn(eps->V,i,NULL,&norm,&breakdown);CHKERRQ(ierr);
-        if (!breakdown && norm!=0.0) {
-          ierr = BVScaleColumn(eps->V,i,1.0/norm);CHKERRQ(ierr);
-        }
+        ierr = BVOrthonormalizeColumn(eps->V,i,PETSC_TRUE,NULL,NULL);CHKERRQ(ierr);
       }
     }
 
@@ -295,8 +277,6 @@ PetscErrorCode EPSSolve_RQCG(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSRQCGSetReset_RQCG"
 static PetscErrorCode EPSRQCGSetReset_RQCG(EPS eps,PetscInt nrest)
 {
   EPS_RQCG *ctx = (EPS_RQCG*)eps->data;
@@ -306,8 +286,6 @@ static PetscErrorCode EPSRQCGSetReset_RQCG(EPS eps,PetscInt nrest)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSRQCGSetReset"
 /*@
    EPSRQCGSetReset - Sets the reset parameter of the RQCG iteration. Every
    nrest iterations, the solver performs a Rayleigh-Ritz projection step.
@@ -336,8 +314,6 @@ PetscErrorCode EPSRQCGSetReset(EPS eps,PetscInt nrest)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSRQCGGetReset_RQCG"
 static PetscErrorCode EPSRQCGGetReset_RQCG(EPS eps,PetscInt *nrest)
 {
   EPS_RQCG *ctx = (EPS_RQCG*)eps->data;
@@ -347,8 +323,6 @@ static PetscErrorCode EPSRQCGGetReset_RQCG(EPS eps,PetscInt *nrest)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSRQCGGetReset"
 /*@
    EPSRQCGGetReset - Gets the reset parameter used in the RQCG method.
 
@@ -375,8 +349,6 @@ PetscErrorCode EPSRQCGGetReset(EPS eps,PetscInt *nrest)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_RQCG"
 PetscErrorCode EPSReset_RQCG(EPS eps)
 {
   PetscErrorCode ierr;
@@ -387,12 +359,10 @@ PetscErrorCode EPSReset_RQCG(EPS eps)
   ierr = BVDestroy(&ctx->W);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->P);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->G);CHKERRQ(ierr);
-  ctx->nrest = 0;
+  ctx->allocsize = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_RQCG"
 PetscErrorCode EPSSetFromOptions_RQCG(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode ierr;
@@ -401,16 +371,14 @@ PetscErrorCode EPSSetFromOptions_RQCG(PetscOptionItems *PetscOptionsObject,EPS e
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS RQCG Options");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-eps_rqcg_reset","RQCG reset parameter","EPSRQCGSetReset",20,&nrest,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSRQCGSetReset(eps,nrest);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsInt("-eps_rqcg_reset","Reset parameter","EPSRQCGSetReset",20,&nrest,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSRQCGSetReset(eps,nrest);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_RQCG"
 PetscErrorCode EPSDestroy_RQCG(EPS eps)
 {
   PetscErrorCode ierr;
@@ -422,8 +390,6 @@ PetscErrorCode EPSDestroy_RQCG(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_RQCG"
 PetscErrorCode EPSView_RQCG(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -433,13 +399,11 @@ PetscErrorCode EPSView_RQCG(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  RQCG: reset every %D iterations\n",ctx->nrest);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  reset every %D iterations\n",ctx->nrest);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_RQCG"
 PETSC_EXTERN PetscErrorCode EPSCreate_RQCG(EPS eps)
 {
   EPS_RQCG       *rqcg;
@@ -449,15 +413,18 @@ PETSC_EXTERN PetscErrorCode EPSCreate_RQCG(EPS eps)
   ierr = PetscNewLog(eps,&rqcg);CHKERRQ(ierr);
   eps->data = (void*)rqcg;
 
-  eps->ops->setup          = EPSSetUp_RQCG;
+  eps->useds = PETSC_TRUE;
+  eps->categ = EPS_CATEGORY_PRECOND;
+
   eps->ops->solve          = EPSSolve_RQCG;
+  eps->ops->setup          = EPSSetUp_RQCG;
   eps->ops->setfromoptions = EPSSetFromOptions_RQCG;
   eps->ops->destroy        = EPSDestroy_RQCG;
   eps->ops->reset          = EPSReset_RQCG;
   eps->ops->view           = EPSView_RQCG;
   eps->ops->backtransform  = EPSBackTransform_Default;
-  ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+  eps->ops->setdefaultst   = EPSSetDefaultST_GMRES;
+
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSRQCGSetReset_C",EPSRQCGSetReset_RQCG);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSRQCGGetReset_C",EPSRQCGGetReset_RQCG);CHKERRQ(ierr);
   PetscFunctionReturn(0);
