@@ -1,33 +1,19 @@
 /*
-   This file implements a wrapper to the ARPACK package
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   This file implements a wrapper to the ARPACK package
+*/
 
 #include <slepc/private/epsimpl.h>
 #include <../src/eps/impls/external/arpack/arpackp.h>
 
-PetscErrorCode EPSSolve_ARPACK(EPS);
-
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_ARPACK"
 PetscErrorCode EPSSetUp_ARPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -86,14 +72,9 @@ PetscErrorCode EPSSetUp_ARPACK(EPS eps)
   if (flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires a BV with contiguous storage");
   ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
   if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
-
-  /* dispatch solve method */
-  eps->ops->solve = EPSSolve_ARPACK;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_ARPACK"
 PetscErrorCode EPSSolve_ARPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -119,8 +100,6 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   ierr = PetscBLASIntCast(MPI_Comm_c2f(PetscObjectComm((PetscObject)eps)),&fcomm);CHKERRQ(ierr);
 #endif
   ierr = PetscBLASIntCast(eps->nloc,&n);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&x);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&y);CHKERRQ(ierr);
   ierr = EPSGetStartVector(eps,0,NULL);CHKERRQ(ierr);
   ierr = BVSetActiveColumns(eps->V,0,0);CHKERRQ(ierr);  /* just for deflation space */
   ierr = BVGetColumn(eps->V,0,&v0);CHKERRQ(ierr);
@@ -129,7 +108,7 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   ierr = VecGetArray(eps->work[1],&resid);CHKERRQ(ierr);
 
   ido  = 0;            /* first call to reverse communication interface */
-  info = 1;            /* indicates a initial vector is provided */
+  info = 1;            /* indicates an initial vector is provided */
   iparam[0] = 1;       /* use exact shifts */
   ierr = PetscBLASIntCast(eps->max_it,&iparam[2]);CHKERRQ(ierr);  /* max Arnoldi iterations */
   iparam[3] = 1;       /* blocksize */
@@ -148,7 +127,8 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&isSinv);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&isShift);CHKERRQ(ierr);
   ierr = STGetShift(eps->st,&sigmar);CHKERRQ(ierr);
-  ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(A,&x,&y);CHKERRQ(ierr);
 
   if (isSinv) {
     /* shift-and-invert mode */
@@ -161,8 +141,7 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
     bmat[0] = 'G';
   } else {
     /* regular mode */
-    if (eps->ishermitian && eps->isgeneralized)
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Spectral transformation not supported by ARPACK hermitian solver");
+    if (eps->ishermitian && eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Spectral transformation not supported by ARPACK hermitian solver");
     iparam[6] = 1;
     bmat[0] = 'I';
   }
@@ -254,7 +233,7 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   eps->its = iparam[2];
 
   if (info==3) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"No shift could be applied in xxAUPD.\nTry increasing the size of NCV relative to NEV");
-  else if (info!=0 && info!=1) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error reported by ARPACK subroutine xxAUPD (%d)",info);
+  else if (info!=0 && info!=1) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error reported by ARPACK subroutine xxAUPD (%d)",(int)info);
 
   rvec = PETSC_TRUE;
 
@@ -271,7 +250,7 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
     ierr = EPSMonitor(eps,eps->its,iparam[4],&ar->workl[ipntr[5]-1],eps->eigi,(PetscReal*)&ar->workl[ipntr[7]-1],eps->ncv);CHKERRQ(ierr);
     PetscStackCall("ARPACKneupd",ARPACKneupd_(&fcomm,&rvec,howmny,ar->select,eps->eigr,pV,&n,&sigmar,ar->workev,bmat,&n,which,&nev,&eps->tol,resid,&ncv,pV,&n,iparam,ipntr,ar->workd,ar->workl,&ar->lworkl,ar->rwork,&info));
 #endif
-    if (info!=0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error reported by ARPACK subroutine xxEUPD (%d)",info);
+    if (info!=0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error reported by ARPACK subroutine xxEUPD (%d)",(int)info);
   }
 
   ierr = VecRestoreArray(v0,&pV);CHKERRQ(ierr);
@@ -291,8 +270,6 @@ PetscErrorCode EPSSolve_ARPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBackTransform_ARPACK"
 PetscErrorCode EPSBackTransform_ARPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -306,8 +283,6 @@ PetscErrorCode EPSBackTransform_ARPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_ARPACK"
 PetscErrorCode EPSReset_ARPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -324,8 +299,6 @@ PetscErrorCode EPSReset_ARPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_ARPACK"
 PetscErrorCode EPSDestroy_ARPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -335,8 +308,6 @@ PetscErrorCode EPSDestroy_ARPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_ARPACK"
 PETSC_EXTERN PetscErrorCode EPSCreate_ARPACK(EPS eps)
 {
   EPS_ARPACK     *ctx;
@@ -346,10 +317,11 @@ PETSC_EXTERN PetscErrorCode EPSCreate_ARPACK(EPS eps)
   ierr = PetscNewLog(eps,&ctx);CHKERRQ(ierr);
   eps->data = (void*)ctx;
 
-  eps->ops->setup                = EPSSetUp_ARPACK;
-  eps->ops->destroy              = EPSDestroy_ARPACK;
-  eps->ops->reset                = EPSReset_ARPACK;
-  eps->ops->backtransform        = EPSBackTransform_ARPACK;
+  eps->ops->solve          = EPSSolve_ARPACK;
+  eps->ops->setup          = EPSSetUp_ARPACK;
+  eps->ops->destroy        = EPSDestroy_ARPACK;
+  eps->ops->reset          = EPSReset_ARPACK;
+  eps->ops->backtransform  = EPSBackTransform_ARPACK;
   PetscFunctionReturn(0);
 }
 
