@@ -1,25 +1,15 @@
 /*
-   This file implements a wrapper to the BLOPEX package
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   This file implements a wrapper to the BLOPEX package
+*/
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
 #include "slepc-interface.h"
@@ -30,8 +20,6 @@
 
 PetscInt slepc_blopex_useconstr = -1;
 
-PetscErrorCode EPSSolve_BLOPEX(EPS);
-
 typedef struct {
   lobpcg_Tolerance           tol;
   lobpcg_BLASLAPACKFunctions blap_fn;
@@ -41,8 +29,6 @@ typedef struct {
   PetscInt                   bs;     /* block size */
 } EPS_BLOPEX;
 
-#undef __FUNCT__
-#define __FUNCT__ "Precond_FnSingleVector"
 static void Precond_FnSingleVector(void *data,void *x,void *y)
 {
   PetscErrorCode ierr;
@@ -56,8 +42,6 @@ static void Precond_FnSingleVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Precond_FnMultiVector"
 static void Precond_FnMultiVector(void *data,void *x,void *y)
 {
   EPS_BLOPEX *blopex = (EPS_BLOPEX*)data;
@@ -67,8 +51,6 @@ static void Precond_FnMultiVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OperatorASingleVector"
 static void OperatorASingleVector(void *data,void *x,void *y)
 {
   PetscErrorCode ierr;
@@ -80,8 +62,8 @@ static void OperatorASingleVector(void *data,void *x,void *y)
 
   PetscFunctionBegin;
   ierr = STGetNumMatrices(blopex->st,&nmat);CHKERRABORT(comm,ierr);
-  ierr = STGetOperators(blopex->st,0,&A);CHKERRABORT(comm,ierr);
-  if (nmat>1) { ierr = STGetOperators(blopex->st,1,&B);CHKERRABORT(comm,ierr); }
+  ierr = STGetMatrix(blopex->st,0,&A);CHKERRABORT(comm,ierr);
+  if (nmat>1) { ierr = STGetMatrix(blopex->st,1,&B);CHKERRABORT(comm,ierr); }
   ierr = MatMult(A,(Vec)x,(Vec)y);CHKERRABORT(comm,ierr);
   ierr = STGetShift(blopex->st,&sigma);CHKERRABORT(comm,ierr);
   if (sigma != 0.0) {
@@ -95,8 +77,6 @@ static void OperatorASingleVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OperatorAMultiVector"
 static void OperatorAMultiVector(void *data,void *x,void *y)
 {
   EPS_BLOPEX *blopex = (EPS_BLOPEX*)data;
@@ -106,8 +86,6 @@ static void OperatorAMultiVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OperatorBSingleVector"
 static void OperatorBSingleVector(void *data,void *x,void *y)
 {
   PetscErrorCode ierr;
@@ -116,13 +94,11 @@ static void OperatorBSingleVector(void *data,void *x,void *y)
   Mat            B;
 
   PetscFunctionBegin;
-  ierr = STGetOperators(blopex->st,1,&B);CHKERRABORT(comm,ierr);
+  ierr = STGetMatrix(blopex->st,1,&B);CHKERRABORT(comm,ierr);
   ierr = MatMult(B,(Vec)x,(Vec)y);CHKERRABORT(comm,ierr);
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OperatorBMultiVector"
 static void OperatorBMultiVector(void *data,void *x,void *y)
 {
   EPS_BLOPEX *blopex = (EPS_BLOPEX*)data;
@@ -132,8 +108,6 @@ static void OperatorBMultiVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetDimensions_BLOPEX"
 PetscErrorCode EPSSetDimensions_BLOPEX(EPS eps,PetscInt nev,PetscInt *ncv,PetscInt *mpd)
 {
   PetscErrorCode ierr;
@@ -150,17 +124,15 @@ PetscErrorCode EPSSetDimensions_BLOPEX(EPS eps,PetscInt nev,PetscInt *ncv,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_BLOPEX"
 PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
 {
 #if defined(PETSC_MISSING_LAPACK_POTRF) || defined(PETSC_MISSING_LAPACK_SYGV)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"POTRF/SYGV - Lapack routine is unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"POTRF/SYGV - Lapack routines are unavailable");
 #else
   PetscErrorCode ierr;
   EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
-  PetscBool      isPrecond,istrivial,flg;
+  PetscBool      istrivial,flg;
 
   PetscFunctionBegin;
   if (!eps->ishermitian || (eps->isgeneralized && !eps->ispositive)) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"blopex only works for Hermitian problems");
@@ -175,9 +147,6 @@ PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
   ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
   if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
 
-  ierr = STSetUp(eps->st);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&isPrecond);CHKERRQ(ierr);
-  if (!isPrecond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"blopex only works with STPRECOND");
   blopex->st = eps->st;
 
   if (eps->converged == EPSConvergedRelative) {
@@ -207,15 +176,10 @@ PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
   blopex->blap_fn.dpotrf = PETSC_dpotrf_interface;
   blopex->blap_fn.dsygv = PETSC_dsygv_interface;
 #endif
-
-  /* dispatch solve method */
-  eps->ops->solve = EPSSolve_BLOPEX;
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_BLOPEX"
 PetscErrorCode EPSSolve_BLOPEX(EPS eps)
 {
   EPS_BLOPEX        *blopex = (EPS_BLOPEX*)eps->data;
@@ -316,8 +280,6 @@ PetscErrorCode EPSSolve_BLOPEX(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBLOPEXSetBlockSize_BLOPEX"
 static PetscErrorCode EPSBLOPEXSetBlockSize_BLOPEX(EPS eps,PetscInt bs)
 {
   EPS_BLOPEX *ctx = (EPS_BLOPEX*)eps->data;
@@ -327,8 +289,6 @@ static PetscErrorCode EPSBLOPEXSetBlockSize_BLOPEX(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBLOPEXSetBlockSize"
 /*@
    EPSBLOPEXSetBlockSize - Sets the block size of the BLOPEX solver.
 
@@ -356,8 +316,6 @@ PetscErrorCode EPSBLOPEXSetBlockSize(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBLOPEXGetBlockSize_BLOPEX"
 static PetscErrorCode EPSBLOPEXGetBlockSize_BLOPEX(EPS eps,PetscInt *bs)
 {
   EPS_BLOPEX *ctx = (EPS_BLOPEX*)eps->data;
@@ -367,8 +325,6 @@ static PetscErrorCode EPSBLOPEXGetBlockSize_BLOPEX(EPS eps,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBLOPEXGetBlockSize"
 /*@
    EPSBLOPEXGetBlockSize - Gets the block size used in the BLOPEX solver.
 
@@ -395,8 +351,6 @@ PetscErrorCode EPSBLOPEXGetBlockSize(EPS eps,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_BLOPEX"
 PetscErrorCode EPSReset_BLOPEX(EPS eps)
 {
   PetscErrorCode ierr;
@@ -407,8 +361,6 @@ PetscErrorCode EPSReset_BLOPEX(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_BLOPEX"
 PetscErrorCode EPSDestroy_BLOPEX(EPS eps)
 {
   PetscErrorCode ierr;
@@ -421,8 +373,6 @@ PetscErrorCode EPSDestroy_BLOPEX(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_BLOPEX"
 PetscErrorCode EPSView_BLOPEX(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -432,45 +382,29 @@ PetscErrorCode EPSView_BLOPEX(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  BLOPEX: block size %D\n",ctx->bs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  block size %D\n",ctx->bs);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_BLOPEX"
 PetscErrorCode EPSSetFromOptions_BLOPEX(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode ierr;
-  KSP            ksp;
   PetscBool      flg;
   PetscInt       bs;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS BLOPEX Options");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-eps_blopex_blocksize","BLOPEX block size","EPSBLOPEXSetBlockSize",20,&bs,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSBLOPEXSetBlockSize(eps,bs);CHKERRQ(ierr);
-  }
-  LOBPCG_SetFromOptionsRandomContext();
 
-  /* Set STPrecond as the default ST */
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  }
-  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-eps_blopex_blocksize","Block size","EPSBLOPEXSetBlockSize",20,&bs,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSBLOPEXSetBlockSize(eps,bs);CHKERRQ(ierr); }
 
-  /* Set the default options of the KSP */
-  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
-  if (!((PetscObject)ksp)->type_name) {
-    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
-  }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  LOBPCG_SetFromOptionsRandomContext();
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_BLOPEX"
 PETSC_EXTERN PetscErrorCode EPSCreate_BLOPEX(EPS eps)
 {
   EPS_BLOPEX     *ctx;
@@ -480,12 +414,17 @@ PETSC_EXTERN PetscErrorCode EPSCreate_BLOPEX(EPS eps)
   ierr = PetscNewLog(eps,&ctx);CHKERRQ(ierr);
   eps->data = (void*)ctx;
 
+  eps->categ = EPS_CATEGORY_PRECOND;
+
+  eps->ops->solve          = EPSSolve_BLOPEX;
   eps->ops->setup          = EPSSetUp_BLOPEX;
   eps->ops->setfromoptions = EPSSetFromOptions_BLOPEX;
   eps->ops->destroy        = EPSDestroy_BLOPEX;
   eps->ops->reset          = EPSReset_BLOPEX;
   eps->ops->view           = EPSView_BLOPEX;
   eps->ops->backtransform  = EPSBackTransform_Default;
+  eps->ops->setdefaultst   = EPSSetDefaultST_Precond;
+
   LOBPCG_InitRandomContext(PetscObjectComm((PetscObject)eps),NULL);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBLOPEXSetBlockSize_C",EPSBLOPEXSetBlockSize_BLOPEX);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBLOPEXGetBlockSize_C",EPSBLOPEXGetBlockSize_BLOPEX);CHKERRQ(ierr);
