@@ -1,32 +1,19 @@
 /*
-   This file implements a wrapper to the BLZPACK package
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   This file implements a wrapper to the BLZPACK package
+*/
 
-#include <slepc-private/epsimpl.h>    /*I "slepceps.h" I*/
-#include <slepc-private/stimpl.h>     /*I "slepcst.h" I*/
+#include <slepc/private/epsimpl.h>    /*I "slepceps.h" I*/
 #include <../src/eps/impls/external/blzpack/blzpackp.h>
 
-PetscErrorCode EPSSolve_BLZPACK(EPS);
-
 const char* blzpack_error[33] = {
   "",
   "illegal data, LFLAG ",
@@ -63,14 +50,12 @@ const char* blzpack_error[33] = {
   "parallel interface error "
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_BLZPACK"
 PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       listor,lrstor,ncuv,k1,k2,k3,k4;
   EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
-  PetscBool      issinv;
+  PetscBool      issinv,istrivial,flg;
 
   PetscFunctionBegin;
   if (eps->ncv) {
@@ -111,58 +96,61 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 
   listor = 123+k1*12;
   ierr = PetscFree(blz->istor);CHKERRQ(ierr);
-  ierr = PetscMalloc((17+listor)*sizeof(PetscBLASInt),&blz->istor);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,(17+listor)*sizeof(PetscBLASInt));CHKERRQ(ierr);
+  ierr = PetscMalloc1((17+listor),&blz->istor);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,(17+listor)*sizeof(PetscBLASInt));CHKERRQ(ierr);
   ierr = PetscBLASIntCast(listor,&blz->istor[14]);CHKERRQ(ierr);
 
   if (blz->slice) lrstor = eps->nloc*(k2*4+k1*2+k4)+k3;
   else lrstor = eps->nloc*(k2*4+k1)+k3;
 lrstor*=10;
   ierr = PetscFree(blz->rstor);CHKERRQ(ierr);
-  ierr = PetscMalloc((4+lrstor)*sizeof(PetscReal),&blz->rstor);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,(4+lrstor)*sizeof(PetscReal));CHKERRQ(ierr);
+  ierr = PetscMalloc1((4+lrstor),&blz->rstor);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,(4+lrstor)*sizeof(PetscReal));CHKERRQ(ierr);
   blz->rstor[3] = lrstor;
 
   ncuv = PetscMax(3,blz->block_size);
   ierr = PetscFree(blz->u);CHKERRQ(ierr);
-  ierr = PetscMalloc(ncuv*eps->nloc*sizeof(PetscScalar),&blz->u);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,ncuv*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscMalloc1(ncuv*eps->nloc,&blz->u);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,ncuv*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
   ierr = PetscFree(blz->v);CHKERRQ(ierr);
-  ierr = PetscMalloc(ncuv*eps->nloc*sizeof(PetscScalar),&blz->v);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,ncuv*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscMalloc1(ncuv*eps->nloc,&blz->v);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,ncuv*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
 
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
-  ierr = PetscMalloc(2*eps->ncv*sizeof(PetscReal),&blz->eig);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(eps,2*eps->ncv*sizeof(PetscReal));CHKERRQ(ierr);
+  ierr = PetscMalloc1(2*eps->ncv,&blz->eig);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)eps,2*eps->ncv*sizeof(PetscReal));CHKERRQ(ierr);
 
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
-  ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
-
-  /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Left vectors not supported in this solver");
-  eps->ops->solve = EPSSolve_BLZPACK;
+  ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
+  ierr = EPS_SetInnerProduct(eps);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->V,BVVECS,&flg);CHKERRQ(ierr);
+  if (flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires a BV with contiguous storage");
+  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
+  if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
+  if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"External packages do not support user-defined stopping test");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_BLZPACK"
 PetscErrorCode EPSSolve_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
   PetscInt       nn;
   PetscBLASInt   i,nneig,lflag,nvopu;
-  Vec            x,y;
+  Vec            x,y,v0;
   PetscScalar    sigma,*pV;
-  Mat            A;
+  Mat            A,M;
   KSP            ksp;
   PC             pc;
 
   PetscFunctionBegin;
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&x);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&y);CHKERRQ(ierr);
-  ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(A,&x,&y);CHKERRQ(ierr);
+  ierr = EPSGetStartVector(eps,0,NULL);CHKERRQ(ierr);
+  ierr = BVSetActiveColumns(eps->V,0,0);CHKERRQ(ierr);  /* just for deflation space */
+  ierr = BVGetColumn(eps->V,0,&v0);CHKERRQ(ierr);
+  ierr = VecGetArray(v0,&pV);CHKERRQ(ierr);
 
   if (eps->isgeneralized && !blz->slice) {
     ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr); /* shift of origin */
@@ -204,11 +192,11 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
         ierr = VecPlaceArray(x,blz->u+i*eps->nloc);CHKERRQ(ierr);
         ierr = VecPlaceArray(y,blz->v+i*eps->nloc);CHKERRQ(ierr);
         if (blz->slice || eps->isgeneralized) {
-          ierr = STMatSolve(eps->st,1,x,y);CHKERRQ(ierr);
+          ierr = STMatSolve(eps->st,x,y);CHKERRQ(ierr);
         } else {
           ierr = STApply(eps->st,x,y);CHKERRQ(ierr);
         }
-        ierr = IPOrthogonalize(eps->ip,0,NULL,eps->nds,NULL,eps->defl,y,NULL,NULL,NULL);CHKERRQ(ierr);
+        ierr = BVOrthogonalizeVec(eps->V,y,NULL,NULL,NULL);CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
         ierr = VecResetArray(y);CHKERRQ(ierr);
       }
@@ -227,32 +215,33 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
       for (i=0;i<nvopu;i++) {
         ierr = VecPlaceArray(x,blz->u+i*eps->nloc);CHKERRQ(ierr);
         ierr = VecPlaceArray(y,blz->v+i*eps->nloc);CHKERRQ(ierr);
-        ierr = IPApplyMatrix(eps->ip,x,y);CHKERRQ(ierr);
+        ierr = BVApplyMatrix(eps->V,x,y);CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
         ierr = VecResetArray(y);CHKERRQ(ierr);
       }
       break;
     case 3:
       /* update shift */
-      ierr = PetscInfo1(eps,"Factorization update (sigma=%g)\n",sigma);CHKERRQ(ierr);
+      ierr = PetscInfo1(eps,"Factorization update (sigma=%g)\n",(double)sigma);CHKERRQ(ierr);
       ierr = STSetShift(eps->st,sigma);CHKERRQ(ierr);
       ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
       ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-      ierr = PCFactorGetMatrix(pc,&A);CHKERRQ(ierr);
-      ierr = MatGetInertia(A,&nn,NULL,NULL);CHKERRQ(ierr);
+      ierr = PCFactorGetMatrix(pc,&M);CHKERRQ(ierr);
+      ierr = MatGetInertia(M,&nn,NULL,NULL);CHKERRQ(ierr);
       ierr = PetscBLASIntCast(nn,&nneig);CHKERRQ(ierr);
       break;
     case 4:
       /* copy the initial vector */
       ierr = VecPlaceArray(x,blz->v);CHKERRQ(ierr);
-      ierr = EPSGetStartVector(eps,0,x,NULL);CHKERRQ(ierr);
+      ierr = VecCopy(v0,x);CHKERRQ(ierr);
       ierr = VecResetArray(x);CHKERRQ(ierr);
       break;
     }
 
   } while (lflag > 0);
 
-  ierr = VecRestoreArray(eps->V[0],&pV);CHKERRQ(ierr);
+  ierr = VecRestoreArray(v0,&pV);CHKERRQ(ierr);
+  ierr = BVRestoreColumn(eps->V,0,&v0);CHKERRQ(ierr);
 
   eps->nconv  = BLZistorr_(blz->istor,"NTEIG",5);
   eps->reason = EPS_CONVERGED_TOL;
@@ -273,8 +262,6 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBackTransform_BLZPACK"
 PetscErrorCode EPSBackTransform_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -287,8 +274,6 @@ PetscErrorCode EPSBackTransform_BLZPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_BLZPACK"
 PetscErrorCode EPSReset_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -300,12 +285,9 @@ PetscErrorCode EPSReset_BLZPACK(EPS eps)
   ierr = PetscFree(blz->u);CHKERRQ(ierr);
   ierr = PetscFree(blz->v);CHKERRQ(ierr);
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
-  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_BLZPACK"
 PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -317,8 +299,6 @@ PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_BLZPACK"
 PetscErrorCode EPSView_BLZPACK(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -328,18 +308,16 @@ PetscErrorCode EPSView_BLZPACK(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: block size=%d\n",blz->block_size);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: maximum number of steps per run=%d\n",blz->nsteps);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  block size=%d\n",blz->block_size);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of steps per run=%d\n",blz->nsteps);CHKERRQ(ierr);
     if (blz->slice) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: computational interval [%f,%f]\n",eps->inta,eps->intb);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  computational interval [%f,%f]\n",eps->inta,eps->intb);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_BLZPACK"
-PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
+PetscErrorCode EPSSetFromOptions_BLZPACK(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode ierr;
   EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
@@ -347,29 +325,24 @@ PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
   PetscBool      flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("EPS BLZPACK Options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"EPS BLZPACK Options");CHKERRQ(ierr);
 
-  bs = blz->block_size;
-  ierr = PetscOptionsInt("-eps_blzpack_block_size","Block size","EPSBlzpackSetBlockSize",bs,&bs,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSBlzpackSetBlockSize(eps,bs);CHKERRQ(ierr);
-  }
+    bs = blz->block_size;
+    ierr = PetscOptionsInt("-eps_blzpack_blocksize","Block size","EPSBlzpackSetBlockSize",bs,&bs,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSBlzpackSetBlockSize(eps,bs);CHKERRQ(ierr); }
 
-  n = blz->nsteps;
-  ierr = PetscOptionsInt("-eps_blzpack_nsteps","Number of steps","EPSBlzpackSetNSteps",n,&n,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSBlzpackSetNSteps(eps,n);CHKERRQ(ierr);
-  }
+    n = blz->nsteps;
+    ierr = PetscOptionsInt("-eps_blzpack_nsteps","Number of steps","EPSBlzpackSetNSteps",n,&n,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSBlzpackSetNSteps(eps,n);CHKERRQ(ierr); }
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBlzpackSetBlockSize_BLZPACK"
 static PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
 {
-  EPS_BLZPACK *blz = (EPS_BLZPACK*)eps->data;;
+  PetscErrorCode ierr;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
 
   PetscFunctionBegin;
   if (bs == PETSC_DEFAULT) blz->block_size = 3;
@@ -380,8 +353,6 @@ static PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBlzpackSetBlockSize"
 /*@
    EPSBlzpackSetBlockSize - Sets the block size for the BLZPACK package.
 
@@ -392,7 +363,7 @@ static PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
 -  bs - block size
 
    Options Database Key:
-.  -eps_blzpack_block_size - Sets the value of the block size
+.  -eps_blzpack_blocksize - Sets the value of the block size
 
    Level: advanced
 @*/
@@ -407,11 +378,10 @@ PetscErrorCode EPSBlzpackSetBlockSize(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBlzpackSetNSteps_BLZPACK"
 static PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,PetscInt nsteps)
 {
-  EPS_BLZPACK *blz = (EPS_BLZPACK*)eps->data;
+  PetscErrorCode ierr;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
 
   PetscFunctionBegin;
   if (nsteps == PETSC_DEFAULT) blz->nsteps = 0;
@@ -421,8 +391,6 @@ static PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,PetscInt nsteps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBlzpackSetNSteps"
 /*@
    EPSBlzpackSetNSteps - Sets the maximum number of steps per run for the BLZPACK
    package.
@@ -450,23 +418,23 @@ PetscErrorCode EPSBlzpackSetNSteps(EPS eps,PetscInt nsteps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_BLZPACK"
 PETSC_EXTERN PetscErrorCode EPSCreate_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_BLZPACK    *blzpack;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(eps,EPS_BLZPACK,&blzpack);CHKERRQ(ierr);
-  eps->data                      = (void*)blzpack;
-  eps->ops->setup                = EPSSetUp_BLZPACK;
-  eps->ops->setfromoptions       = EPSSetFromOptions_BLZPACK;
-  eps->ops->destroy              = EPSDestroy_BLZPACK;
-  eps->ops->reset                = EPSReset_BLZPACK;
-  eps->ops->view                 = EPSView_BLZPACK;
-  eps->ops->backtransform        = EPSBackTransform_BLZPACK;
-  eps->ops->computevectors       = EPSComputeVectors_Default;
+  ierr = PetscNewLog(eps,&blzpack);CHKERRQ(ierr);
+  eps->data = (void*)blzpack;
+
+  eps->ops->solve          = EPSSolve_BLZPACK;
+  eps->ops->setup          = EPSSetUp_BLZPACK;
+  eps->ops->setfromoptions = EPSSetFromOptions_BLZPACK;
+  eps->ops->destroy        = EPSDestroy_BLZPACK;
+  eps->ops->reset          = EPSReset_BLZPACK;
+  eps->ops->view           = EPSView_BLZPACK;
+  eps->ops->backtransform  = EPSBackTransform_BLZPACK;
+
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBlzpackSetBlockSize_C",EPSBlzpackSetBlockSize_BLZPACK);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBlzpackSetNSteps_C",EPSBlzpackSetNSteps_BLZPACK);CHKERRQ(ierr);
   PetscFunctionReturn(0);
