@@ -1,55 +1,51 @@
 /*
-   This file implements a wrapper to the PRIMME package
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   This file implements a wrapper to the PRIMME package
+*/
 
 #include <slepc/private/epsimpl.h>    /*I "slepceps.h" I*/
 
-PetscErrorCode EPSSolve_PRIMME(EPS);
-
-EXTERN_C_BEGIN
 #include <primme.h>
-EXTERN_C_END
+
+#if defined(PETSC_USE_COMPLEX)
+#if defined(PETSC_USE_REAL_SINGLE)
+#define PRIMME_DRIVER cprimme
+#else
+#define PRIMME_DRIVER zprimme
+#endif
+#else
+#if defined(PETSC_USE_REAL_SINGLE)
+#define PRIMME_DRIVER sprimme
+#else
+#define PRIMME_DRIVER dprimme
+#endif
+#endif
 
 typedef struct {
   primme_params primme;           /* param struc */
   primme_preset_method method;    /* primme method */
   Mat       A;                    /* problem matrix */
-  EPS       eps;                  /* EPS current context */
   KSP       ksp;                  /* linear solver and preconditioner */
   Vec       x,y;                  /* auxiliary vectors */
-  PetscReal target;               /* a copy of eps's target */
+  double    target;               /* a copy of eps's target */
 } EPS_PRIMME;
 
-static void multMatvec_PRIMME(void *in,void *out,int *blockSize,primme_params *primme);
-static void applyPreconditioner_PRIMME(void *in,void *out,int *blockSize,struct primme_params *primme);
+static void multMatvec_PRIMME(void*,PRIMME_INT*,void*,PRIMME_INT*,int*,struct primme_params*,int*);
+static void applyPreconditioner_PRIMME(void*,PRIMME_INT*,void*,PRIMME_INT*,int*,struct primme_params*,int*);
 
-static void par_GlobalSumDouble(void *sendBuf,void *recvBuf,int *count,primme_params *primme)
+static void par_GlobalSumReal(void *sendBuf,void *recvBuf,int *count,primme_params *primme,int *ierr)
 {
-  PetscErrorCode ierr;
-  ierr = MPI_Allreduce((double*)sendBuf,(double*)recvBuf,*count,MPI_DOUBLE,MPI_SUM,PetscObjectComm((PetscObject)primme->commInfo));CHKERRABORT(PetscObjectComm((PetscObject)primme->commInfo),ierr);
+  *ierr = MPI_Allreduce(sendBuf,recvBuf,*count,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)primme->commInfo));CHKERRABORT(PetscObjectComm((PetscObject)primme->commInfo),*ierr);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_PRIMME"
 PetscErrorCode EPSSetUp_PRIMME(EPS eps)
 {
   PetscErrorCode ierr;
@@ -64,20 +60,16 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
 
   /* Check some constraints and set some default values */
   if (!eps->max_it) eps->max_it = PetscMax(1000,eps->n);
-  ierr = STGetOperators(eps->st,0,&ops->A);CHKERRQ(ierr);
+  ierr = STGetMatrix(eps->st,0,&ops->A);CHKERRQ(ierr);
   if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME is only available for Hermitian problems");
   if (eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME is not available for generalized problems");
   if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
   if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"External packages do not support user-defined stopping test");
   if (!eps->which) eps->which = EPS_LARGEST_REAL;
-  if (eps->converged != EPSConvergedAbsolute) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only supports absolute convergence test");
+  if (eps->converged != EPSConvergedAbsolute) { ierr = PetscInfo(eps,"Warning: using absolute convergence test\n");CHKERRQ(ierr); }
   ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
   if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
 
-  ierr = STSetUp(eps->st);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&flg);CHKERRQ(ierr);
-  if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME only works with STPRECOND");
-
   /* Transfer SLEPc options to PRIMME options */
   primme->n          = eps->n;
   primme->nLocal     = eps->nloc;
@@ -115,7 +107,7 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   /* If user sets ncv, maxBasisSize is modified. If not, ncv is set as maxBasisSize */
   if (eps->ncv) primme->maxBasisSize = eps->ncv;
   else eps->ncv = primme->maxBasisSize;
-  if (eps->ncv < eps->nev+primme->maxBlockSize)  SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME needs ncv >= nev+maxBlockSize");
+  if (eps->ncv < eps->nev+primme->maxBlockSize) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME needs ncv >= nev+maxBlockSize");
   if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
 
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
@@ -126,7 +118,6 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   if (flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires a BV with contiguous storage");
 
   /* Setup the preconditioner */
-  ops->eps = eps;
   if (primme->correctionParams.precondition) {
     ierr = STGetKSP(eps->st,&ops->ksp);CHKERRQ(ierr);
     ierr = PetscObjectTypeCompare((PetscObject)ops->ksp,KSPPREONLY,&flg);CHKERRQ(ierr);
@@ -136,18 +127,14 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   }
 
   /* Prepare auxiliary vectors */
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,eps->n,NULL,&ops->x);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,eps->n,NULL,&ops->y);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->x);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->y);CHKERRQ(ierr);
-
-  /* dispatch solve method */
-  eps->ops->solve = EPSSolve_PRIMME;
+  if (!ops->x) {
+    ierr = MatCreateVecsEmpty(ops->A,&ops->x,&ops->y);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->x);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->y);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_PRIMME"
 PetscErrorCode EPSSolve_PRIMME(EPS eps)
 {
   PetscErrorCode ierr;
@@ -169,12 +156,12 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   ierr = BVGetColumn(eps->V,0,&v0);CHKERRQ(ierr);
   ierr = VecGetArray(v0,&a);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = dprimme(eps->eigr,a,eps->errest,&ops->primme);
+  ierr = PRIMME_DRIVER(eps->eigr,a,eps->errest,&ops->primme);
   if (ierr) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"PRIMME library failed with error code=%d",ierr);
 #else
   /* PRIMME returns real eigenvalues, but SLEPc works with complex ones */
   ierr = PetscMalloc1(eps->ncv,&evals);CHKERRQ(ierr);
-  ierr = zprimme(evals,(Complex_Z*)a,eps->errest,&ops->primme);
+  ierr = PRIMME_DRIVER(evals,a,eps->errest,&ops->primme);
   if (ierr) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"PRIMME library failed with error code=%d",ierr);
   for (i=0;i<eps->ncv;i++) eps->eigr[i] = evals[i];
   ierr = PetscFree(evals);CHKERRQ(ierr);
@@ -188,67 +175,53 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "multMatvec_PRIMME"
-static void multMatvec_PRIMME(void *in,void *out,int *blockSize,primme_params *primme)
+static void multMatvec_PRIMME(void *xa,PRIMME_INT *ldx,void *ya,PRIMME_INT *ldy,int *blockSize,struct primme_params *primme,int *ierr)
 {
-  PetscErrorCode ierr;
-  PetscInt       i,N = primme->n;
-  EPS_PRIMME     *ops = (EPS_PRIMME*)primme->matrix;
-  Vec            x = ops->x,y = ops->y;
-  Mat            A = ops->A;
+  PetscInt   i;
+  EPS_PRIMME *ops = (EPS_PRIMME*)primme->matrix;
+  Vec        x = ops->x,y = ops->y;
+  Mat        A = ops->A;
 
   PetscFunctionBegin;
   for (i=0;i<*blockSize;i++) {
-    /* build vectors using 'in' an 'out' workspace */
-    ierr = VecPlaceArray(x,(PetscScalar*)in+N*i);CHKERRABORT(PetscObjectComm((PetscObject)A),ierr);
-    ierr = VecPlaceArray(y,(PetscScalar*)out+N*i);CHKERRABORT(PetscObjectComm((PetscObject)A),ierr);
-
-    ierr = MatMult(A,x,y);CHKERRABORT(PetscObjectComm((PetscObject)A),ierr);
-
-    ierr = VecResetArray(x);CHKERRABORT(PetscObjectComm((PetscObject)A),ierr);
-    ierr = VecResetArray(y);CHKERRABORT(PetscObjectComm((PetscObject)A),ierr);
+    *ierr = VecPlaceArray(x,(PetscScalar*)xa+(*ldx)*i);CHKERRABORT(PetscObjectComm((PetscObject)A),*ierr);
+    *ierr = VecPlaceArray(y,(PetscScalar*)ya+(*ldy)*i);CHKERRABORT(PetscObjectComm((PetscObject)A),*ierr);
+    *ierr = MatMult(A,x,y);CHKERRABORT(PetscObjectComm((PetscObject)A),*ierr);
+    *ierr = VecResetArray(x);CHKERRABORT(PetscObjectComm((PetscObject)A),*ierr);
+    *ierr = VecResetArray(y);CHKERRABORT(PetscObjectComm((PetscObject)A),*ierr);
   }
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "applyPreconditioner_PRIMME"
-static void applyPreconditioner_PRIMME(void *in,void *out,int *blockSize,struct primme_params *primme)
+static void applyPreconditioner_PRIMME(void *xa,PRIMME_INT *ldx,void *ya,PRIMME_INT *ldy,int *blockSize,struct primme_params *primme,int *ierr)
 {
-  PetscErrorCode ierr;
-  PetscInt       i,N = primme->n;
-  EPS_PRIMME     *ops = (EPS_PRIMME*)primme->matrix;
-  Vec            x = ops->x,y = ops->y;
+  PetscInt   i;
+  EPS_PRIMME *ops = (EPS_PRIMME*)primme->matrix;
+  Vec        x = ops->x,y = ops->y;
 
   PetscFunctionBegin;
   for (i=0;i<*blockSize;i++) {
-    /* build vectors using 'in' an 'out' workspace */
-    ierr = VecPlaceArray(x,(PetscScalar*)in+N*i);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-    ierr = VecPlaceArray(y,(PetscScalar*)out+N*i);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-    ierr = KSPSolve(ops->ksp,x,y);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-    ierr = VecResetArray(x);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
-    ierr = VecResetArray(y);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),ierr);
+    *ierr = VecPlaceArray(x,(PetscScalar*)xa+(*ldx)*i);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),*ierr);
+    *ierr = VecPlaceArray(y,(PetscScalar*)ya+(*ldy)*i);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),*ierr);
+    *ierr = KSPSolve(ops->ksp,x,y);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),*ierr);
+    *ierr = VecResetArray(x);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),*ierr);
+    *ierr = VecResetArray(y);CHKERRABORT(PetscObjectComm((PetscObject)ops->ksp),*ierr);
   }
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_PRIMME"
 PetscErrorCode EPSReset_PRIMME(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_PRIMME     *ops = (EPS_PRIMME*)eps->data;
 
   PetscFunctionBegin;
-  primme_Free(&ops->primme);
+  primme_free(&ops->primme);
   ierr = VecDestroy(&ops->x);CHKERRQ(ierr);
   ierr = VecDestroy(&ops->y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_PRIMME"
 PetscErrorCode EPSDestroy_PRIMME(EPS eps)
 {
   PetscErrorCode ierr;
@@ -262,8 +235,6 @@ PetscErrorCode EPSDestroy_PRIMME(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_PRIMME"
 PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode  ierr;
@@ -275,9 +246,9 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  PRIMME: block size=%D\n",primme->maxBlockSize);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  block size=%D\n",primme->maxBlockSize);CHKERRQ(ierr);
     ierr = EPSPRIMMEGetMethod(eps,&methodn);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  PRIMME: solver method: %s\n",EPSPRIMMEMethods[methodn]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  solver method: %s\n",EPSPRIMMEMethods[methodn]);CHKERRQ(ierr);
 
     /* Display PRIMME params */
     ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)eps),&rank);CHKERRQ(ierr);
@@ -286,8 +257,6 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_PRIMME"
 PetscErrorCode EPSSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode  ierr;
@@ -295,36 +264,20 @@ PetscErrorCode EPSSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,EPS
   PetscInt        bs;
   EPSPRIMMEMethod meth;
   PetscBool       flg;
-  KSP             ksp;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS PRIMME Options");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-eps_primme_block_size","Maximum block size","EPSPRIMMESetBlockSize",ctx->primme.maxBlockSize,&bs,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSPRIMMESetBlockSize(eps,bs);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsEnum("-eps_primme_method","Method for solving the eigenproblem","EPSPRIMMESetMethod",EPSPRIMMEMethods,(PetscEnum)ctx->method,(PetscEnum*)&meth,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSPRIMMESetMethod(eps,meth);CHKERRQ(ierr);
-  }
 
-  /* Set STPrecond as the default ST */
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  }
-  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-eps_primme_blocksize","Maximum block size","EPSPRIMMESetBlockSize",ctx->primme.maxBlockSize,&bs,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSPRIMMESetBlockSize(eps,bs);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsEnum("-eps_primme_method","Method for solving the eigenproblem","EPSPRIMMESetMethod",EPSPRIMMEMethods,(PetscEnum)ctx->method,(PetscEnum*)&meth,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSPRIMMESetMethod(eps,meth);CHKERRQ(ierr); }
 
-  /* Set the default options of the KSP */
-  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
-  if (!((PetscObject)ksp)->type_name) {
-    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
-  }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMESetBlockSize_PRIMME"
 static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
 {
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
@@ -336,8 +289,6 @@ static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMESetBlockSize"
 /*@
    EPSPRIMMESetBlockSize - The maximum block size that PRIMME will try to use.
 
@@ -348,7 +299,7 @@ static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
 -  bs - block size
 
    Options Database Key:
-.  -eps_primme_block_size - Sets the max allowed block size value
+.  -eps_primme_blocksize - Sets the max allowed block size value
 
    Notes:
    If the block size is not set, the value established by primme_initialize
@@ -374,8 +325,6 @@ PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMEGetBlockSize_PRIMME"
 static PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
 {
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
@@ -385,8 +334,6 @@ static PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMEGetBlockSize"
 /*@
    EPSPRIMMEGetBlockSize - Get the maximum block size the code will try to use.
 
@@ -413,8 +360,6 @@ PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMESetMethod_PRIMME"
 static PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps,EPSPRIMMEMethod method)
 {
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
@@ -424,8 +369,6 @@ static PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps,EPSPRIMMEMethod method)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMESetMethod"
 /*@
    EPSPRIMMESetMethod - Sets the method for the PRIMME library.
 
@@ -456,8 +399,6 @@ PetscErrorCode EPSPRIMMESetMethod(EPS eps,EPSPRIMMEMethod method)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMEGetMethod_PRIMME"
 static PetscErrorCode EPSPRIMMEGetMethod_PRIMME(EPS eps,EPSPRIMMEMethod *method)
 {
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
@@ -467,8 +408,6 @@ static PetscErrorCode EPSPRIMMEGetMethod_PRIMME(EPS eps,EPSPRIMMEMethod *method)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPRIMMEGetMethod"
 /*@
    EPSPRIMMEGetMethod - Gets the method for the PRIMME library.
 
@@ -495,8 +434,6 @@ PetscErrorCode EPSPRIMMEGetMethod(EPS eps,EPSPRIMMEMethod *method)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_PRIMME"
 PETSC_EXTERN PetscErrorCode EPSCreate_PRIMME(EPS eps)
 {
   PetscErrorCode ierr;
@@ -506,17 +443,21 @@ PETSC_EXTERN PetscErrorCode EPSCreate_PRIMME(EPS eps)
   ierr = PetscNewLog(eps,&primme);CHKERRQ(ierr);
   eps->data = (void*)primme;
 
+  primme_initialize(&primme->primme);
+  primme->primme.matrixMatvec = multMatvec_PRIMME;
+  primme->primme.globalSumReal = par_GlobalSumReal;
+  primme->method = (primme_preset_method)EPS_PRIMME_DEFAULT_MIN_TIME;
+
+  eps->categ = EPS_CATEGORY_PRECOND;
+
+  eps->ops->solve          = EPSSolve_PRIMME;
   eps->ops->setup          = EPSSetUp_PRIMME;
   eps->ops->setfromoptions = EPSSetFromOptions_PRIMME;
   eps->ops->destroy        = EPSDestroy_PRIMME;
   eps->ops->reset          = EPSReset_PRIMME;
   eps->ops->view           = EPSView_PRIMME;
   eps->ops->backtransform  = EPSBackTransform_Default;
-
-  primme_initialize(&primme->primme);
-  primme->primme.matrixMatvec = multMatvec_PRIMME;
-  primme->primme.globalSumDouble = par_GlobalSumDouble;
-  primme->method = (primme_preset_method)EPS_PRIMME_DEFAULT_MIN_TIME;
+  eps->ops->setdefaultst   = EPSSetDefaultST_Precond;
 
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMESetBlockSize_C",EPSPRIMMESetBlockSize_PRIMME);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMESetMethod_C",EPSPRIMMESetMethod_PRIMME);CHKERRQ(ierr);
