@@ -1,36 +1,25 @@
 /*
-   This file implements a wrapper to the TRLAN package
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   This file implements a wrapper to the TRLAN package
+*/
 
 #include <slepc/private/epsimpl.h>
 #include <../src/eps/impls/external/trlan/trlanp.h>
 
-PetscErrorCode EPSSolve_TRLAN(EPS);
-
 /* Nasty global variable to access EPS data from TRLan_ */
-static EPS globaleps;
+static struct {
+  EPS eps;
+  Vec x,y;
+} globaldata;
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_TRLAN"
 PetscErrorCode EPSSetUp_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
@@ -60,6 +49,7 @@ PetscErrorCode EPSSetUp_TRLAN(EPS eps)
   } else {
     ierr = PetscBLASIntCast(eps->nloc*(tr->maxlan+1-eps->ncv) + tr->maxlan*(tr->maxlan+10),&tr->lwork);CHKERRQ(ierr);
   }
+  if (tr->work) { ierr = PetscFree(tr->work);CHKERRQ(ierr); }
   ierr = PetscMalloc1(tr->lwork,&tr->work);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)eps,tr->lwork*sizeof(PetscReal));CHKERRQ(ierr);
 
@@ -68,38 +58,28 @@ PetscErrorCode EPSSetUp_TRLAN(EPS eps)
   if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
 
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
-
-  /* dispatch solve method */
-  eps->ops->solve = EPSSolve_TRLAN;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_TRLAN"
 static PetscBLASInt MatMult_TRLAN(PetscBLASInt *n,PetscBLASInt *m,PetscReal *xin,PetscBLASInt *ldx,PetscReal *yout,PetscBLASInt *ldy)
 {
   PetscErrorCode ierr;
-  Vec            x,y;
+  Vec            x=globaldata.x,y=globaldata.y;
+  EPS            eps=globaldata.eps;
   PetscBLASInt   i;
 
   PetscFunctionBegin;
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)globaleps),1,*n,PETSC_DECIDE,NULL,&x);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)globaleps),1,*n,PETSC_DECIDE,NULL,&y);CHKERRQ(ierr);
   for (i=0;i<*m;i++) {
     ierr = VecPlaceArray(x,(PetscScalar*)xin+i*(*ldx));CHKERRQ(ierr);
     ierr = VecPlaceArray(y,(PetscScalar*)yout+i*(*ldy));CHKERRQ(ierr);
-    ierr = STApply(globaleps->st,x,y);CHKERRQ(ierr);
-    ierr = BVOrthogonalizeVec(globaleps->V,y,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = STApply(eps->st,x,y);CHKERRQ(ierr);
+    ierr = BVOrthogonalizeVec(eps->V,y,NULL,NULL,NULL);CHKERRQ(ierr);
     ierr = VecResetArray(x);CHKERRQ(ierr);
     ierr = VecResetArray(y);CHKERRQ(ierr);
   }
-  ierr = VecDestroy(&x);CHKERRQ(ierr);
-  ierr = VecDestroy(&y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_TRLAN"
 PetscErrorCode EPSSolve_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
@@ -108,6 +88,7 @@ PetscErrorCode EPSSolve_TRLAN(EPS eps)
   EPS_TRLAN      *tr = (EPS_TRLAN*)eps->data;
   PetscScalar    *pV;
   Vec            v0;
+  Mat            A;
 
   PetscFunctionBegin;
   ierr = PetscBLASIntCast(eps->ncv,&ncv);CHKERRQ(ierr);
@@ -117,7 +98,9 @@ PetscErrorCode EPSSolve_TRLAN(EPS eps)
   else if (eps->which==EPS_SMALLEST_REAL) lohi = -1;
   else SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
 
-  globaleps = eps;
+  globaldata.eps = eps;
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(A,&globaldata.x,&globaldata.y);CHKERRQ(ierr);
 
   ipar[0]  = 0;            /* stat: error flag */
   ipar[1]  = lohi;         /* smallest (lohi<0) or largest eigenvalues (lohi>0) */
@@ -153,12 +136,12 @@ PetscErrorCode EPSSolve_TRLAN(EPS eps)
   eps->its    = ipar[25];
   eps->reason = EPS_CONVERGED_TOL;
 
+  ierr = VecDestroy(&globaldata.x);CHKERRQ(ierr);
+  ierr = VecDestroy(&globaldata.y);CHKERRQ(ierr);
   if (stat!=0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error in TRLAN (code=%d)",stat);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_TRLAN"
 PetscErrorCode EPSReset_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
@@ -169,8 +152,6 @@ PetscErrorCode EPSReset_TRLAN(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_TRLAN"
 PetscErrorCode EPSDestroy_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
@@ -180,8 +161,6 @@ PetscErrorCode EPSDestroy_TRLAN(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_TRLAN"
 PETSC_EXTERN PetscErrorCode EPSCreate_TRLAN(EPS eps)
 {
   EPS_TRLAN      *ctx;
@@ -191,10 +170,11 @@ PETSC_EXTERN PetscErrorCode EPSCreate_TRLAN(EPS eps)
   ierr = PetscNewLog(eps,&ctx);CHKERRQ(ierr);
   eps->data = (void*)ctx;
 
-  eps->ops->setup                = EPSSetUp_TRLAN;
-  eps->ops->destroy              = EPSDestroy_TRLAN;
-  eps->ops->reset                = EPSReset_TRLAN;
-  eps->ops->backtransform        = EPSBackTransform_Default;
+  eps->ops->solve          = EPSSolve_TRLAN;
+  eps->ops->setup          = EPSSetUp_TRLAN;
+  eps->ops->destroy        = EPSDestroy_TRLAN;
+  eps->ops->reset          = EPSReset_TRLAN;
+  eps->ops->backtransform  = EPSBackTransform_Default;
   PetscFunctionReturn(0);
 }
 
