@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "arnoldi"
 
    Method: Explicitly Restarted Arnoldi
@@ -12,37 +20,14 @@
 
        [1] "Arnoldi Methods in SLEPc", SLEPc Technical Report STR-4,
            available at http://slepc.upv.es.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
 
-PetscErrorCode EPSSolve_Arnoldi(EPS);
-
 typedef struct {
   PetscBool delayed;
 } EPS_ARNOLDI;
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_Arnoldi"
 PetscErrorCode EPSSetUp_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
@@ -68,20 +53,16 @@ PetscErrorCode EPSSetUp_Arnoldi(EPS eps)
   ierr = DSSetExtraRow(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
   ierr = DSAllocate(eps->ds,eps->ncv+1);CHKERRQ(ierr);
 
-  /* dispatch solve method */
   if (eps->isgeneralized && eps->ishermitian && !eps->ispositive) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Requested method does not work for indefinite problems");
-  eps->ops->solve = EPSSolve_Arnoldi;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_Arnoldi"
 PetscErrorCode EPSSolve_Arnoldi(EPS eps)
 {
   PetscErrorCode     ierr;
   PetscInt           k,nv,ld;
   Mat                U;
-  PetscScalar        *H,*X;
+  PetscScalar        *H;
   PetscReal          beta,gamma=1.0;
   PetscBool          breakdown,harmonic,refined;
   BVOrthogRefineType orthog_ref;
@@ -124,20 +105,18 @@ PetscErrorCode EPSSolve_Arnoldi(EPS eps)
     ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
     ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
     ierr = DSUpdateExtraRow(eps->ds);CHKERRQ(ierr);
+    ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
     /* Check convergence */
     ierr = EPSKrylovConvergence(eps,PETSC_FALSE,eps->nconv,nv-eps->nconv,beta,gamma,&k);CHKERRQ(ierr);
     if (refined) {
-      ierr = DSGetArray(eps->ds,DS_MAT_X,&X);CHKERRQ(ierr);
-      ierr = BVMultColumn(eps->V,1.0,0.0,k,X+k*ld);CHKERRQ(ierr);
-      ierr = DSRestoreArray(eps->ds,DS_MAT_X,&X);CHKERRQ(ierr);
-      ierr = DSGetMat(eps->ds,DS_MAT_Q,&U);CHKERRQ(ierr);
-      ierr = BVMultInPlace(eps->V,U,eps->nconv,nv);CHKERRQ(ierr);
+      ierr = DSGetMat(eps->ds,DS_MAT_X,&U);CHKERRQ(ierr);
+      ierr = BVMultInPlace(eps->V,U,eps->nconv,k+1);CHKERRQ(ierr);
       ierr = MatDestroy(&U);CHKERRQ(ierr);
-      ierr = BVOrthogonalizeColumn(eps->V,k,NULL,NULL,NULL);CHKERRQ(ierr);
+      ierr = BVOrthonormalizeColumn(eps->V,k,PETSC_FALSE,NULL,NULL);CHKERRQ(ierr);
     } else {
       ierr = DSGetMat(eps->ds,DS_MAT_Q,&U);CHKERRQ(ierr);
-      ierr = BVMultInPlace(eps->V,U,eps->nconv,nv);CHKERRQ(ierr);
+      ierr = BVMultInPlace(eps->V,U,eps->nconv,PetscMin(k+1,nv));CHKERRQ(ierr);
       ierr = MatDestroy(&U);CHKERRQ(ierr);
     }
     ierr = (*eps->stopping)(eps,eps->its,eps->max_it,k,eps->nev,&eps->reason,eps->stoppingctx);CHKERRQ(ierr);
@@ -160,8 +139,6 @@ PetscErrorCode EPSSolve_Arnoldi(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_Arnoldi"
 PetscErrorCode EPSSetFromOptions_Arnoldi(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode ierr;
@@ -170,16 +147,14 @@ PetscErrorCode EPSSetFromOptions_Arnoldi(PetscOptionItems *PetscOptionsObject,EP
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS Arnoldi Options");CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-eps_arnoldi_delayed","Arnoldi with delayed reorthogonalization","EPSArnoldiSetDelayed",arnoldi->delayed,&val,&set);CHKERRQ(ierr);
-  if (set) {
-    ierr = EPSArnoldiSetDelayed(eps,val);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsBool("-eps_arnoldi_delayed","Use delayed reorthogonalization","EPSArnoldiSetDelayed",arnoldi->delayed,&val,&set);CHKERRQ(ierr);
+    if (set) { ierr = EPSArnoldiSetDelayed(eps,val);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSArnoldiSetDelayed_Arnoldi"
 static PetscErrorCode EPSArnoldiSetDelayed_Arnoldi(EPS eps,PetscBool delayed)
 {
   EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI*)eps->data;
@@ -189,8 +164,6 @@ static PetscErrorCode EPSArnoldiSetDelayed_Arnoldi(EPS eps,PetscBool delayed)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSArnoldiSetDelayed"
 /*@
    EPSArnoldiSetDelayed - Activates or deactivates delayed reorthogonalization
    in the Arnoldi iteration.
@@ -224,8 +197,6 @@ PetscErrorCode EPSArnoldiSetDelayed(EPS eps,PetscBool delayed)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSArnoldiGetDelayed_Arnoldi"
 static PetscErrorCode EPSArnoldiGetDelayed_Arnoldi(EPS eps,PetscBool *delayed)
 {
   EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI*)eps->data;
@@ -235,8 +206,6 @@ static PetscErrorCode EPSArnoldiGetDelayed_Arnoldi(EPS eps,PetscBool *delayed)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSArnoldiGetDelayed"
 /*@
    EPSArnoldiGetDelayed - Gets the type of reorthogonalization used during the Arnoldi
    iteration.
@@ -264,8 +233,6 @@ PetscErrorCode EPSArnoldiGetDelayed(EPS eps,PetscBool *delayed)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_Arnoldi"
 PetscErrorCode EPSDestroy_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
@@ -277,8 +244,6 @@ PetscErrorCode EPSDestroy_Arnoldi(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_Arnoldi"
 PetscErrorCode EPSView_Arnoldi(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -287,16 +252,12 @@ PetscErrorCode EPSView_Arnoldi(EPS eps,PetscViewer viewer)
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
-  if (isascii) {
-    if (arnoldi->delayed) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Arnoldi: using delayed reorthogonalization\n");CHKERRQ(ierr);
-    }
+  if (isascii && arnoldi->delayed) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  using delayed reorthogonalization\n");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_Arnoldi"
 PETSC_EXTERN PetscErrorCode EPSCreate_Arnoldi(EPS eps)
 {
   EPS_ARNOLDI    *ctx;
@@ -306,12 +267,16 @@ PETSC_EXTERN PetscErrorCode EPSCreate_Arnoldi(EPS eps)
   ierr = PetscNewLog(eps,&ctx);CHKERRQ(ierr);
   eps->data = (void*)ctx;
 
-  eps->ops->setup                = EPSSetUp_Arnoldi;
-  eps->ops->setfromoptions       = EPSSetFromOptions_Arnoldi;
-  eps->ops->destroy              = EPSDestroy_Arnoldi;
-  eps->ops->view                 = EPSView_Arnoldi;
-  eps->ops->backtransform        = EPSBackTransform_Default;
-  eps->ops->computevectors       = EPSComputeVectors_Schur;
+  eps->useds = PETSC_TRUE;
+
+  eps->ops->solve          = EPSSolve_Arnoldi;
+  eps->ops->setup          = EPSSetUp_Arnoldi;
+  eps->ops->setfromoptions = EPSSetFromOptions_Arnoldi;
+  eps->ops->destroy        = EPSDestroy_Arnoldi;
+  eps->ops->view           = EPSView_Arnoldi;
+  eps->ops->backtransform  = EPSBackTransform_Default;
+  eps->ops->computevectors = EPSComputeVectors_Schur;
+
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSArnoldiSetDelayed_C",EPSArnoldiSetDelayed_Arnoldi);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSArnoldiGetDelayed_C",EPSArnoldiGetDelayed_Arnoldi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
