@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "krylovschur"
 
    Method: Krylov-Schur
@@ -19,32 +27,11 @@
 
        [3] "Practical Implementation of Harmonic Krylov-Schur", SLEPc Technical
             Report STR-9, available at http://slepc.upv.es.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
 #include "krylovschur.h"
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetArbitraryValues"
 PetscErrorCode EPSGetArbitraryValues(EPS eps,PetscScalar *rr,PetscScalar *ri)
 {
   PetscErrorCode ierr;
@@ -72,21 +59,92 @@ PetscErrorCode EPSGetArbitraryValues(EPS eps,PetscScalar *rr,PetscScalar *ri)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_KrylovSchur"
+static PetscErrorCode EstimateRange(Mat A,PetscReal *left,PetscReal *right)
+{
+  PetscErrorCode ierr;
+  PetscInt       nconv;
+  PetscScalar    eig0;
+  EPS            eps;
+
+  PetscFunctionBegin;
+  *left = 0.0; *right = 0.0;
+  ierr = EPSCreate(PetscObjectComm((PetscObject)A),&eps);CHKERRQ(ierr);
+  ierr = EPSSetOptionsPrefix(eps,"eps_filter_");CHKERRQ(ierr);
+  ierr = EPSSetOperators(eps,A,NULL);CHKERRQ(ierr);
+  ierr = EPSSetProblemType(eps,EPS_HEP);CHKERRQ(ierr);
+  ierr = EPSSetTolerances(eps,1e-3,50);CHKERRQ(ierr);
+  ierr = EPSSetWhichEigenpairs(eps,EPS_SMALLEST_REAL);CHKERRQ(ierr);
+  ierr = EPSSolve(eps);CHKERRQ(ierr);
+  ierr = EPSGetConverged(eps,&nconv);CHKERRQ(ierr);
+  if (nconv>0) {
+    ierr = EPSGetEigenvalue(eps,0,&eig0,NULL);CHKERRQ(ierr);
+  } else eig0 = eps->eigr[0];
+  *left = PetscRealPart(eig0);
+  ierr = EPSSetWhichEigenpairs(eps,EPS_LARGEST_REAL);CHKERRQ(ierr);
+  ierr = EPSSolve(eps);CHKERRQ(ierr);
+  ierr = EPSGetConverged(eps,&nconv);CHKERRQ(ierr);
+  if (nconv>0) {
+    ierr = EPSGetEigenvalue(eps,0,&eig0,NULL);CHKERRQ(ierr);
+  } else eig0 = eps->eigr[0];
+  *right = PetscRealPart(eig0);
+  ierr = EPSDestroy(&eps);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode EPSSetUp_KrylovSchur_Filter(EPS eps)
+{
+  PetscErrorCode ierr;
+  SlepcSC        sc;
+  PetscReal      rleft,rright;
+  Mat            A;
+
+  PetscFunctionBegin;
+  if (eps->intb >= PETSC_MAX_REAL && eps->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The defined computational interval should have at least one of their sides bounded");
+  if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Polynomial filter only available for symmetric/Hermitian eigenproblems");
+  if (eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Polynomial filters not available for generalized eigenproblems");
+  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs cannot be used with polynomial filters");
+  if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL*1e-2;  /* use tighter tolerance */
+  ierr = STFilterSetInterval(eps->st,eps->inta,eps->intb);CHKERRQ(ierr);
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  ierr = STFilterGetRange(eps->st,&rleft,&rright);CHKERRQ(ierr);
+  if (!rleft && !rright) {
+    ierr = EstimateRange(A,&rleft,&rright);CHKERRQ(ierr);
+    ierr = PetscInfo2(eps,"Setting eigenvalue range to [%g,%g]\n",(double)rleft,(double)rright);CHKERRQ(ierr);
+    ierr = STFilterSetRange(eps->st,rleft,rright);CHKERRQ(ierr);
+  }
+  if (!eps->ncv && eps->nev==1) eps->nev = 40;  /* user did not provide nev estimation */
+  ierr = EPSSetDimensions_Default(eps,eps->nev,&eps->ncv,&eps->mpd);CHKERRQ(ierr);
+  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must not be larger than nev+mpd");
+  if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
+
+  ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
+  sc->rg            = NULL;
+  sc->comparison    = SlepcCompareLargestReal;
+  sc->comparisonctx = NULL;
+  sc->map           = NULL;
+  sc->mapobj        = NULL;
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
 {
   PetscErrorCode    ierr;
   PetscReal         eta;
+  PetscBool         isfilt;
   BVOrthogType      otype;
   BVOrthogBlockType obtype;
   EPS_KRYLOVSCHUR   *ctx = (EPS_KRYLOVSCHUR*)eps->data;
-  enum { EPS_KS_DEFAULT,EPS_KS_SYMM,EPS_KS_SLICE,EPS_KS_INDEF } variant;
+  enum { EPS_KS_DEFAULT,EPS_KS_SYMM,EPS_KS_SLICE,EPS_KS_FILTER,EPS_KS_INDEF } variant;
 
   PetscFunctionBegin;
   /* spectrum slicing requires special treatment of default values */
   if (eps->which==EPS_ALL) {
-    ierr = EPSSetUp_KrylovSchur_Slice(eps);CHKERRQ(ierr);
+    ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilt);CHKERRQ(ierr);
+    if (isfilt) {
+      ierr = EPSSetUp_KrylovSchur_Filter(eps);CHKERRQ(ierr);
+    } else {
+      ierr = EPSSetUp_KrylovSchur_Slice(eps);CHKERRQ(ierr);
+    }
   } else {
     ierr = EPSSetDimensions_Default(eps,eps->nev,&eps->ncv,&eps->mpd);CHKERRQ(ierr);
     if (eps->ncv>eps->nev+eps->mpd) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must not be larger than nev+mpd");
@@ -100,8 +158,7 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
 
   if (!eps->extraction) {
     ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr);
-  } else if (eps->extraction!=EPS_RITZ && eps->extraction!=EPS_HARMONIC)
-    SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Unsupported extraction type");
+  } else if (eps->extraction!=EPS_RITZ && eps->extraction!=EPS_HARMONIC) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Unsupported extraction type");
   if (eps->extraction==EPS_HARMONIC && ctx->lock) { ierr = PetscInfo(eps,"Locking was requested but will be deactivated since is not supported with harmonic extraction\n");CHKERRQ(ierr); }
 
   if (!ctx->keep) ctx->keep = 0.5;
@@ -118,7 +175,7 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
   if (eps->ishermitian) {
     if (eps->which==EPS_ALL) {
       if (eps->isgeneralized && !eps->ispositive) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Spectrum slicing not implemented for indefinite problems");
-      else variant = EPS_KS_SLICE;
+      else variant = isfilt? EPS_KS_FILTER: EPS_KS_SLICE;
     } else if (eps->isgeneralized && !eps->ispositive) {
       variant = EPS_KS_INDEF;
     } else {
@@ -143,6 +200,7 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
       ierr = DSAllocate(eps->ds,eps->ncv+1);CHKERRQ(ierr);
       break;
     case EPS_KS_SYMM:
+    case EPS_KS_FILTER:
       eps->ops->solve = EPSSolve_KrylovSchur_Symm;
       eps->ops->computevectors = EPSComputeVectors_Hermitian;
       ierr = DSSetType(eps->ds,DSHEP);CHKERRQ(ierr);
@@ -170,8 +228,6 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_KrylovSchur_Default"
 PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -222,6 +278,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
       j=1;
     }
     ierr = DSSort(eps->ds,eps->eigr,eps->eigi,eps->rr,eps->ri,pj);CHKERRQ(ierr);
+    ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
     /* Check convergence */
     ierr = EPSKrylovConvergence(eps,PETSC_FALSE,eps->nconv,nv-eps->nconv,beta,gamma,&k);CHKERRQ(ierr);
@@ -229,7 +286,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
     nconv = k;
 
     /* Update l */
-    if (eps->reason != EPS_CONVERGED_ITERATING || breakdown) l = 0;
+    if (eps->reason != EPS_CONVERGED_ITERATING || breakdown || k==nv) l = 0;
     else {
       l = PetscMax(1,(PetscInt)((nv-k)*ctx->keep));
 #if !defined(PETSC_USE_COMPLEX)
@@ -244,7 +301,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
     if ((!ctx->lock || harmonic) && l>0) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
 
     if (eps->reason == EPS_CONVERGED_ITERATING) {
-      if (breakdown) {
+      if (breakdown || k==nv) {
         /* Start a new Arnoldi factorization */
         ierr = PetscInfo2(eps,"Breakdown in Krylov-Schur method (it=%D norm=%g)\n",eps->its,(double)beta);CHKERRQ(ierr);
         if (k<eps->nev) {
@@ -293,8 +350,6 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetRestart_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurSetRestart_KrylovSchur(EPS eps,PetscReal keep)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -302,14 +357,12 @@ static PetscErrorCode EPSKrylovSchurSetRestart_KrylovSchur(EPS eps,PetscReal kee
   PetscFunctionBegin;
   if (keep==PETSC_DEFAULT) ctx->keep = 0.5;
   else {
-    if (keep<0.1 || keep>0.9) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The keep argument must be in the range [0.1,0.9]");
+    if (keep<0.1 || keep>0.9) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The keep argument %g must be in the range [0.1,0.9]",keep);
     ctx->keep = keep;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetRestart"
 /*@
    EPSKrylovSchurSetRestart - Sets the restart parameter for the Krylov-Schur
    method, in particular the proportion of basis vectors that must be kept
@@ -342,8 +395,6 @@ PetscErrorCode EPSKrylovSchurSetRestart(EPS eps,PetscReal keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetRestart_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetRestart_KrylovSchur(EPS eps,PetscReal *keep)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -353,8 +404,6 @@ static PetscErrorCode EPSKrylovSchurGetRestart_KrylovSchur(EPS eps,PetscReal *ke
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetRestart"
 /*@
    EPSKrylovSchurGetRestart - Gets the restart parameter used in the
    Krylov-Schur method.
@@ -382,8 +431,6 @@ PetscErrorCode EPSKrylovSchurGetRestart(EPS eps,PetscReal *keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetLocking_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurSetLocking_KrylovSchur(EPS eps,PetscBool lock)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -393,8 +440,6 @@ static PetscErrorCode EPSKrylovSchurSetLocking_KrylovSchur(EPS eps,PetscBool loc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetLocking"
 /*@
    EPSKrylovSchurSetLocking - Choose between locking and non-locking variants of
    the Krylov-Schur method.
@@ -429,8 +474,6 @@ PetscErrorCode EPSKrylovSchurSetLocking(EPS eps,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetLocking_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetLocking_KrylovSchur(EPS eps,PetscBool *lock)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -440,8 +483,6 @@ static PetscErrorCode EPSKrylovSchurGetLocking_KrylovSchur(EPS eps,PetscBool *lo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetLocking"
 /*@
    EPSKrylovSchurGetLocking - Gets the locking flag used in the Krylov-Schur
    method.
@@ -469,8 +510,6 @@ PetscErrorCode EPSKrylovSchurGetLocking(EPS eps,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetPartitions_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurSetPartitions_KrylovSchur(EPS eps,PetscInt npart)
 {
   PetscErrorCode  ierr;
@@ -480,8 +519,8 @@ static PetscErrorCode EPSKrylovSchurSetPartitions_KrylovSchur(EPS eps,PetscInt n
   PetscFunctionBegin;
   if (ctx->npart!=npart) {
     if (ctx->commset) { ierr = PetscSubcommDestroy(&ctx->subc);CHKERRQ(ierr); }
-    if (ctx->eps) { ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr); }
-  } 
+    ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr);
+  }
   if (npart == PETSC_DEFAULT || npart == PETSC_DECIDE) {
     ctx->npart = 1;
   } else {
@@ -493,8 +532,6 @@ static PetscErrorCode EPSKrylovSchurSetPartitions_KrylovSchur(EPS eps,PetscInt n
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetPartitions"
 /*@
    EPSKrylovSchurSetPartitions - Sets the number of partitions for the
    case of doing spectrum slicing for a computational interval with the
@@ -533,8 +570,6 @@ PetscErrorCode EPSKrylovSchurSetPartitions(EPS eps,PetscInt npart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetPartitions_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetPartitions_KrylovSchur(EPS eps,PetscInt *npart)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -544,8 +579,6 @@ static PetscErrorCode EPSKrylovSchurGetPartitions_KrylovSchur(EPS eps,PetscInt *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetPartitions"
 /*@
    EPSKrylovSchurGetPartitions - Gets the number of partitions of the
    communicator in case of spectrum slicing.
@@ -573,8 +606,6 @@ PetscErrorCode EPSKrylovSchurGetPartitions(EPS eps,PetscInt *npart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetDetectZeros_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurSetDetectZeros_KrylovSchur(EPS eps,PetscBool detect)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -585,8 +616,6 @@ static PetscErrorCode EPSKrylovSchurSetDetectZeros_KrylovSchur(EPS eps,PetscBool
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetDetectZeros"
 /*@
    EPSKrylovSchurSetDetectZeros - Sets a flag to enforce detection of
    zeros during the factorizations throughout the spectrum slicing computation.
@@ -624,8 +653,6 @@ PetscErrorCode EPSKrylovSchurSetDetectZeros(EPS eps,PetscBool detect)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetDetectZeros_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetDetectZeros_KrylovSchur(EPS eps,PetscBool *detect)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -635,8 +662,6 @@ static PetscErrorCode EPSKrylovSchurGetDetectZeros_KrylovSchur(EPS eps,PetscBool
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetDetectZeros"
 /*@
    EPSKrylovSchurGetDetectZeros - Gets the flag that enforces zero detection
    in spectrum slicing.
@@ -664,8 +689,6 @@ PetscErrorCode EPSKrylovSchurGetDetectZeros(EPS eps,PetscBool *detect)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetDimensions_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurSetDimensions_KrylovSchur(EPS eps,PetscInt nev,PetscInt ncv,PetscInt mpd)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -689,8 +712,6 @@ static PetscErrorCode EPSKrylovSchurSetDimensions_KrylovSchur(EPS eps,PetscInt n
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetDimensions"
 /*@
    EPSKrylovSchurSetDimensions - Sets the dimensions used for each subsolve
    step in case of doing spectrum slicing for a computational interval.
@@ -726,8 +747,6 @@ PetscErrorCode EPSKrylovSchurSetDimensions(EPS eps,PetscInt nev,PetscInt ncv,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetDimensions_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetDimensions_KrylovSchur(EPS eps,PetscInt *nev,PetscInt *ncv,PetscInt *mpd)
 {
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
@@ -739,8 +758,6 @@ static PetscErrorCode EPSKrylovSchurGetDimensions_KrylovSchur(EPS eps,PetscInt *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetDimensions"
 /*@
    EPSKrylovSchurGetDimensions - Gets the dimensions used for each subsolve
    step in case of doing spectrum slicing for a computational interval.
@@ -769,8 +786,6 @@ PetscErrorCode EPSKrylovSchurGetDimensions(EPS eps,PetscInt *nev,PetscInt *ncv,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetSubintervals_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurSetSubintervals_KrylovSchur(EPS eps,PetscReal* subint)
 {
   PetscErrorCode  ierr;
@@ -779,7 +794,7 @@ static PetscErrorCode EPSKrylovSchurSetSubintervals_KrylovSchur(EPS eps,PetscRea
 
   PetscFunctionBegin;
   if (subint[0]!=eps->inta || subint[ctx->npart]!=eps->intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"First and last values must match the endpoints of EPSSetInterval()");
-  for (i=0;i<ctx->npart;i++) if (subint[i]>=subint[i+1]) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Array must contain values in ascending order");
+  for (i=0;i<ctx->npart;i++) if (subint[i]>subint[i+1]) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Array must contain values in ascending order");
   if (ctx->subintervals) { ierr = PetscFree(ctx->subintervals);CHKERRQ(ierr); }
   ierr = PetscMalloc1(ctx->npart+1,&ctx->subintervals);CHKERRQ(ierr);
   for (i=0;i<ctx->npart+1;i++) ctx->subintervals[i] = subint[i];
@@ -788,8 +803,6 @@ static PetscErrorCode EPSKrylovSchurSetSubintervals_KrylovSchur(EPS eps,PetscRea
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurSetSubintervals"
 /*@C
    EPSKrylovSchurSetSubintervals - Sets the points that delimit the
    subintervals to be used in spectrum slicing with several partitions.
@@ -824,8 +837,6 @@ PetscErrorCode EPSKrylovSchurSetSubintervals(EPS eps,PetscReal *subint)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubintervals_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetSubintervals_KrylovSchur(EPS eps,PetscReal **subint)
 {
   PetscErrorCode  ierr;
@@ -842,8 +853,6 @@ static PetscErrorCode EPSKrylovSchurGetSubintervals_KrylovSchur(EPS eps,PetscRea
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubintervals"
 /*@C
    EPSKrylovSchurGetSubintervals - Returns the points that delimit the
    subintervals used in spectrum slicing with several partitions.
@@ -877,7 +886,7 @@ static PetscErrorCode EPSKrylovSchurGetSubintervals_KrylovSchur(EPS eps,PetscRea
 
 .seealso: EPSKrylovSchurSetSubintervals(), EPSKrylovSchurGetPartitions(), EPSSetInterval()
 @*/
-PetscErrorCode EPSKrylovSchurGetSubintervals(EPS eps,PetscReal** subint)
+PetscErrorCode EPSKrylovSchurGetSubintervals(EPS eps,PetscReal **subint)
 {
   PetscErrorCode ierr;
 
@@ -888,13 +897,11 @@ PetscErrorCode EPSKrylovSchurGetSubintervals(EPS eps,PetscReal** subint)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetInertias_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetInertias_KrylovSchur(EPS eps,PetscInt *n,PetscReal **shifts,PetscInt **inertias)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
-  PetscInt        i;
+  PetscInt        i,numsh;
   EPS_SR          sr = ctx->sr;
 
   PetscFunctionBegin;
@@ -904,37 +911,38 @@ static PetscErrorCode EPSKrylovSchurGetInertias_KrylovSchur(EPS eps,PetscInt *n,
   case EPS_STATE_INITIAL:
     break;
   case EPS_STATE_SETUP:
-    *n = ctx->npart+1;
-    ierr = PetscMalloc1(*n,shifts);CHKERRQ(ierr);
-    ierr = PetscMalloc1(*n,inertias);CHKERRQ(ierr);
-    (*shifts)[0] = eps->inta;
-    (*inertias)[0] = (sr->dir==1)?sr->inertia0:sr->inertia1;
-    if (ctx->npart==1) {
-      (*shifts)[1] = eps->intb;
-      (*inertias)[1] = (sr->dir==1)?sr->inertia1:sr->inertia0;
-    } else {
-      for (i=1;i<*n;i++) {
-        (*shifts)[i] = ctx->subintervals[i];
-        (*inertias)[i] = (*inertias)[i-1]+ctx->nconv_loc[i-1];
-      }
+    numsh = ctx->npart+1;
+    if (n) *n = numsh;
+    if (shifts) {
+      ierr = PetscMalloc1(numsh,shifts);CHKERRQ(ierr);
+      (*shifts)[0] = eps->inta;
+      if (ctx->npart==1) (*shifts)[1] = eps->intb;
+      else for (i=1;i<numsh;i++) (*shifts)[i] = ctx->subintervals[i];
+    }
+    if (inertias) {
+      ierr = PetscMalloc1(numsh,inertias);CHKERRQ(ierr);
+      (*inertias)[0] = (sr->dir==1)?sr->inertia0:sr->inertia1;
+      if (ctx->npart==1) (*inertias)[1] = (sr->dir==1)?sr->inertia1:sr->inertia0;
+      else for (i=1;i<numsh;i++) (*inertias)[i] = (*inertias)[i-1]+ctx->nconv_loc[i-1];
     }
     break;
   case EPS_STATE_SOLVED:
   case EPS_STATE_EIGENVECTORS:
-    *n = ctx->nshifts;
-    ierr = PetscMalloc1(*n,shifts);CHKERRQ(ierr);
-    ierr = PetscMalloc1(*n,inertias);CHKERRQ(ierr);
-    for (i=0;i<*n;i++) {
-      (*shifts)[i] = ctx->shifts[i];
-      (*inertias)[i] = ctx->inertias[i];
+    numsh = ctx->nshifts;
+    if (n) *n = numsh;
+    if (shifts) {
+      ierr = PetscMalloc1(numsh,shifts);CHKERRQ(ierr);
+      for (i=0;i<numsh;i++) (*shifts)[i] = ctx->shifts[i];
+    }
+    if (inertias) {
+      ierr = PetscMalloc1(numsh,inertias);CHKERRQ(ierr);
+      for (i=0;i<numsh;i++) (*inertias)[i] = ctx->inertias[i];
     }
     break;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetInertias"
 /*@C
    EPSKrylovSchurGetInertias - Gets the values of the shifts and their
    corresponding inertias in case of doing spectrum slicing for a
@@ -958,7 +966,22 @@ static PetscErrorCode EPSKrylovSchurGetInertias_KrylovSchur(EPS eps,PetscInt *n,
 
    This function is only available for spectrum slicing runs.
 
-   The returned arrays should be freed by the user.
+   The returned arrays should be freed by the user. Can pass NULL in any of
+   the two arrays if not required.
+
+   Fortran Notes:
+   The calling sequence from Fortran is
+.vb
+   EPSKrylovSchurGetInertias(eps,n,shifts,inertias,ierr)
+   integer n
+   double precision shifts(*)
+   integer inertias(*)
+.ve
+   The arrays should be at least of length n. The value of n can be determined
+   by an initial call
+.vb
+   EPSKrylovSchurGetInertias(eps,n,PETSC_NULL_REAL,PETSC_NULL_INTEGER,ierr)
+.ve
 
    Level: advanced
 
@@ -975,8 +998,6 @@ PetscErrorCode EPSKrylovSchurGetInertias(EPS eps,PetscInt *n,PetscReal **shifts,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubcommInfo_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetSubcommInfo_KrylovSchur(EPS eps,PetscInt *k,PetscInt *n,Vec *v)
 {
   PetscErrorCode  ierr;
@@ -994,8 +1015,6 @@ static PetscErrorCode EPSKrylovSchurGetSubcommInfo_KrylovSchur(EPS eps,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubcommInfo"
 /*@C
    EPSKrylovSchurGetSubcommInfo - Gets information related to the case of
    doing spectrum slicing for a computational interval with multiple
@@ -1031,8 +1050,6 @@ PetscErrorCode EPSKrylovSchurGetSubcommInfo(EPS eps,PetscInt *k,PetscInt *n,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubcommPairs_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetSubcommPairs_KrylovSchur(EPS eps,PetscInt i,PetscScalar *eig,Vec v)
 {
   PetscErrorCode  ierr;
@@ -1048,8 +1065,6 @@ static PetscErrorCode EPSKrylovSchurGetSubcommPairs_KrylovSchur(EPS eps,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubcommPairs"
 /*@C
    EPSKrylovSchurGetSubcommPairs - Gets the i-th eigenpair stored
    internally in the subcommunicator to which the calling process belongs.
@@ -1087,8 +1102,6 @@ PetscErrorCode EPSKrylovSchurGetSubcommPairs(EPS eps,PetscInt i,PetscScalar *eig
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubcommMats_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurGetSubcommMats_KrylovSchur(EPS eps,Mat *A,Mat *B)
 {
   PetscErrorCode  ierr;
@@ -1101,8 +1114,6 @@ static PetscErrorCode EPSKrylovSchurGetSubcommMats_KrylovSchur(EPS eps,Mat *A,Ma
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurGetSubcommMats"
 /*@C
    EPSKrylovSchurGetSubcommMats - Gets the eigenproblem matrices stored
    internally in the subcommunicator to which the calling process belongs.
@@ -1136,21 +1147,19 @@ PetscErrorCode EPSKrylovSchurGetSubcommMats(EPS eps,Mat *A,Mat *B)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurUpdateSubcommMats_KrylovSchur"
 static PetscErrorCode EPSKrylovSchurUpdateSubcommMats_KrylovSchur(EPS eps,PetscScalar a,PetscScalar ap,Mat Au,PetscScalar b,PetscScalar bp, Mat Bu,MatStructure str,PetscBool globalup)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data,*subctx;
   Mat             A,B=NULL,Ag,Bg=NULL;
   PetscBool       reuse=PETSC_TRUE;
-   
+
   PetscFunctionBegin;
   if (!ctx->sr) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"Only available in interval computations, see EPSSetInterval()");
   if (!eps->state) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"Must call EPSSetUp() first");
   ierr = EPSGetOperators(eps,&Ag,&Bg);CHKERRQ(ierr);
   ierr = EPSGetOperators(ctx->eps,&A,&B);CHKERRQ(ierr);
-  
+
   ierr = MatScale(A,a);CHKERRQ(ierr);
   if (Au) {
     ierr = MatAXPY(A,ap,Au,str);CHKERRQ(ierr);
@@ -1177,13 +1186,13 @@ static PetscErrorCode EPSKrylovSchurUpdateSubcommMats_KrylovSchur(EPS eps,PetscS
       if (ctx->submata && !reuse) {
         ierr = MatDestroyMatrices(1,&ctx->submata);CHKERRQ(ierr);
       }
-      ierr = MatGetSubMatrices(A,1,&ctx->isrow,&ctx->iscol,(reuse)?MAT_REUSE_MATRIX:MAT_INITIAL_MATRIX,&ctx->submata);CHKERRQ(ierr);
+      ierr = MatCreateSubMatrices(A,1,&ctx->isrow,&ctx->iscol,(reuse)?MAT_REUSE_MATRIX:MAT_INITIAL_MATRIX,&ctx->submata);CHKERRQ(ierr);
       ierr = MatCreateMPIMatConcatenateSeqMat(((PetscObject)Ag)->comm,ctx->submata[0],PETSC_DECIDE,MAT_REUSE_MATRIX,&Ag);CHKERRQ(ierr);
       if (B) {
         if (ctx->submatb && !reuse) {
           ierr = MatDestroyMatrices(1,&ctx->submatb);CHKERRQ(ierr);
         }
-        ierr = MatGetSubMatrices(B,1,&ctx->isrow,&ctx->iscol,(reuse)?MAT_REUSE_MATRIX:MAT_INITIAL_MATRIX,&ctx->submatb);CHKERRQ(ierr);
+        ierr = MatCreateSubMatrices(B,1,&ctx->isrow,&ctx->iscol,(reuse)?MAT_REUSE_MATRIX:MAT_INITIAL_MATRIX,&ctx->submatb);CHKERRQ(ierr);
         ierr = MatCreateMPIMatConcatenateSeqMat(((PetscObject)Bg)->comm,ctx->submatb[0],PETSC_DECIDE,MAT_REUSE_MATRIX,&Bg);CHKERRQ(ierr);
       }
     }
@@ -1194,8 +1203,6 @@ static PetscErrorCode EPSKrylovSchurUpdateSubcommMats_KrylovSchur(EPS eps,PetscS
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchurUpdateSubcommMats"
 /*@C
    EPSKrylovSchurUpdateSubcommMats - Update the eigenproblem matrices stored
    internally in the subcommunicator to which the calling process belongs.
@@ -1234,7 +1241,7 @@ static PetscErrorCode EPSKrylovSchurUpdateSubcommMats_KrylovSchur(EPS eps,PetscS
 
 .seealso: EPSSetInterval(), EPSKrylovSchurSetPartitions(), EPSKrylovSchurGetSubcommMats()
 @*/
-PetscErrorCode EPSKrylovSchurUpdateSubcommMats(EPS eps,PetscScalar s,PetscScalar a,Mat Au,PetscScalar t,PetscScalar b, Mat Bu,MatStructure str,PetscBool globalup)
+PetscErrorCode EPSKrylovSchurUpdateSubcommMats(EPS eps,PetscScalar s,PetscScalar a,Mat Au,PetscScalar t,PetscScalar b,Mat Bu,MatStructure str,PetscBool globalup)
 {
   PetscErrorCode ierr;
 
@@ -1252,8 +1259,6 @@ PetscErrorCode EPSKrylovSchurUpdateSubcommMats(EPS eps,PetscScalar s,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_KrylovSchur"
 PetscErrorCode EPSSetFromOptions_KrylovSchur(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode  ierr;
@@ -1264,62 +1269,59 @@ PetscErrorCode EPSSetFromOptions_KrylovSchur(PetscOptionItems *PetscOptionsObjec
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS Krylov-Schur Options");CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-eps_krylovschur_restart","Proportion of vectors kept after restart","EPSKrylovSchurSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSKrylovSchurSetRestart(eps,keep);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsBool("-eps_krylovschur_locking","Choose between locking and non-locking variants","EPSKrylovSchurSetLocking",PETSC_TRUE,&lock,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSKrylovSchurSetLocking(eps,lock);CHKERRQ(ierr);
-  }
-  i = ctx->npart;
-  ierr = PetscOptionsInt("-eps_krylovschur_partitions","Number of partitions of the communicator for spectrum slicing","EPSKrylovSchurSetPartitions",ctx->npart,&i,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSKrylovSchurSetPartitions(eps,i);CHKERRQ(ierr);
-  }
-  b = ctx->detect;
-  ierr = PetscOptionsBool("-eps_krylovschur_detect_zeros","Check zeros during factorizations at subinterval boundaries","EPSKrylovSchurSetDetectZeros",ctx->detect,&b,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSKrylovSchurSetDetectZeros(eps,b);CHKERRQ(ierr);
-  }
-  i = 1;
-  j = k = PETSC_DECIDE;
-  ierr = PetscOptionsInt("-eps_krylovschur_nev","Number of eigenvalues to compute in each subsolve (only for spectrum slicing)","EPSKrylovSchurSetDimensions",40,&i,&f1);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-eps_krylovschur_ncv","Number of basis vectors in each subsolve (only for spectrum slicing)","EPSKrylovSchurSetDimensions",80,&j,&f2);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-eps_krylovschur_mpd","Maximum dimension of projected problem in each subsolve (only for spectrum slicing)","EPSKrylovSchurSetDimensions",80,&k,&f3);CHKERRQ(ierr);
-  if (f1 || f2 || f3) {
-    ierr = EPSKrylovSchurSetDimensions(eps,i,j,k);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsReal("-eps_krylovschur_restart","Proportion of vectors kept after restart","EPSKrylovSchurSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSKrylovSchurSetRestart(eps,keep);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsBool("-eps_krylovschur_locking","Choose between locking and non-locking variants","EPSKrylovSchurSetLocking",PETSC_TRUE,&lock,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSKrylovSchurSetLocking(eps,lock);CHKERRQ(ierr); }
+
+    i = ctx->npart;
+    ierr = PetscOptionsInt("-eps_krylovschur_partitions","Number of partitions of the communicator for spectrum slicing","EPSKrylovSchurSetPartitions",ctx->npart,&i,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSKrylovSchurSetPartitions(eps,i);CHKERRQ(ierr); }
+
+    b = ctx->detect;
+    ierr = PetscOptionsBool("-eps_krylovschur_detect_zeros","Check zeros during factorizations at subinterval boundaries","EPSKrylovSchurSetDetectZeros",ctx->detect,&b,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSKrylovSchurSetDetectZeros(eps,b);CHKERRQ(ierr); }
+
+    i = 1;
+    j = k = PETSC_DECIDE;
+    ierr = PetscOptionsInt("-eps_krylovschur_nev","Number of eigenvalues to compute in each subsolve (only for spectrum slicing)","EPSKrylovSchurSetDimensions",40,&i,&f1);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-eps_krylovschur_ncv","Number of basis vectors in each subsolve (only for spectrum slicing)","EPSKrylovSchurSetDimensions",80,&j,&f2);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-eps_krylovschur_mpd","Maximum dimension of projected problem in each subsolve (only for spectrum slicing)","EPSKrylovSchurSetDimensions",80,&k,&f3);CHKERRQ(ierr);
+    if (f1 || f2 || f3) { ierr = EPSKrylovSchurSetDimensions(eps,i,j,k);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_KrylovSchur"
 PetscErrorCode EPSView_KrylovSchur(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
-  PetscBool       isascii;
+  PetscBool       isascii,isfilt;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Krylov-Schur: %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  Krylov-Schur: using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
     if (eps->which==EPS_ALL) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Krylov-Schur: doing spectrum slicing with nev=%D, ncv=%D, mpd=%D\n",ctx->nev,ctx->ncv,ctx->mpd);CHKERRQ(ierr);
-      if (ctx->npart>1) {
-        ierr = PetscViewerASCIIPrintf(viewer,"  Krylov-Schur: multi-communicator spectrum slicing with %D partitions\n",ctx->npart);CHKERRQ(ierr);
-        if (ctx->detect) { ierr = PetscViewerASCIIPrintf(viewer,"  Krylov-Schur: detecting zeros when factorizing at subinterval boundaries\n");CHKERRQ(ierr); }
+      ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilt);CHKERRQ(ierr);
+      if (isfilt) {
+        ierr = PetscViewerASCIIPrintf(viewer,"  using filtering to extract all eigenvalues in an interval\n");CHKERRQ(ierr);
+      } else {
+        ierr = PetscViewerASCIIPrintf(viewer,"  doing spectrum slicing with nev=%D, ncv=%D, mpd=%D\n",ctx->nev,ctx->ncv,ctx->mpd);CHKERRQ(ierr);
+        if (ctx->npart>1) {
+          ierr = PetscViewerASCIIPrintf(viewer,"  multi-communicator spectrum slicing with %D partitions\n",ctx->npart);CHKERRQ(ierr);
+          if (ctx->detect) { ierr = PetscViewerASCIIPrintf(viewer,"  detecting zeros when factorizing at subinterval boundaries\n");CHKERRQ(ierr); }
+        }
       }
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_KrylovSchur"
 PetscErrorCode EPSDestroy_KrylovSchur(EPS eps)
 {
   PetscErrorCode ierr;
@@ -1346,20 +1348,32 @@ PetscErrorCode EPSDestroy_KrylovSchur(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_KrylovSchur"
 PetscErrorCode EPSReset_KrylovSchur(EPS eps)
 {
   PetscErrorCode ierr;
+  PetscBool      isfilt;
+
   PetscFunctionBegin;
-  if (eps->which==EPS_ALL) {
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilt);CHKERRQ(ierr);
+  if (eps->which==EPS_ALL && !isfilt) {
     ierr = EPSReset_KrylovSchur_Slice(eps);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_KrylovSchur"
+PetscErrorCode EPSSetDefaultST_KrylovSchur(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (eps->which==EPS_ALL) {
+    if (!((PetscObject)eps->st)->type_name) {
+      ierr = STSetType(eps->st,STSINVERT);CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 PETSC_EXTERN PetscErrorCode EPSCreate_KrylovSchur(EPS eps)
 {
   EPS_KRYLOVSCHUR *ctx;
@@ -1374,12 +1388,17 @@ PETSC_EXTERN PetscErrorCode EPSCreate_KrylovSchur(EPS eps)
   ctx->detect = PETSC_FALSE;
   ctx->global = PETSC_TRUE;
 
+  eps->useds = PETSC_TRUE;
+
+  /* solve and computevectors determined at setup */
   eps->ops->setup          = EPSSetUp_KrylovSchur;
   eps->ops->setfromoptions = EPSSetFromOptions_KrylovSchur;
   eps->ops->destroy        = EPSDestroy_KrylovSchur;
   eps->ops->reset          = EPSReset_KrylovSchur;
   eps->ops->view           = EPSView_KrylovSchur;
   eps->ops->backtransform  = EPSBackTransform_Default;
+  eps->ops->setdefaultst   = EPSSetDefaultST_KrylovSchur;
+
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurSetRestart_C",EPSKrylovSchurSetRestart_KrylovSchur);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetRestart_C",EPSKrylovSchurGetRestart_KrylovSchur);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurSetLocking_C",EPSKrylovSchurSetLocking_KrylovSchur);CHKERRQ(ierr);
