@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "krylovschur"
 
    Method: Krylov-Schur with spectrum slicing for symmetric eigenproblems
@@ -13,25 +21,6 @@
        [2] C. Campos and J.E. Roman, "Spectrum slicing strategies based
            on restarted Lanczos methods", Numer. Algor. 60(2):279-295,
            2012.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>
@@ -47,13 +36,11 @@ static const char citation[] =
   "   number = \"2\",\n"
   "   pages = \"279--295\",\n"
   "   year = \"2012,\"\n"
-  "   doi = \"http://dx.doi.org/10.1007/s11075-012-9564-z\"\n"
+  "   doi = \"https://doi.org/10.1007/s11075-012-9564-z\"\n"
   "}\n";
 
 #define SLICE_PTOL PETSC_SQRT_MACHINE_EPSILON
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceResetSR"
 static PetscErrorCode EPSSliceResetSR(EPS eps) {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
@@ -81,8 +68,6 @@ static PetscErrorCode EPSSliceResetSR(EPS eps) {
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_KrylovSchur_Slice"
 PetscErrorCode EPSReset_KrylovSchur_Slice(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -114,8 +99,6 @@ PetscErrorCode EPSReset_KrylovSchur_Slice(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceAllocateSolution"
 /*
   EPSSliceAllocateSolution - Allocate memory storage for common variables such
   as eigenvalues and eigenvectors. The argument extra is used for methods
@@ -155,7 +138,7 @@ static PetscErrorCode EPSSliceAllocateSolution(EPS eps,PetscInt extra)
     ierr = BVGetType(eps->V,&type);CHKERRQ(ierr);
     ierr = BVSetType(sr->V,type);CHKERRQ(ierr);
   }
-  ierr = STMatCreateVecs(eps->st,&t,NULL);CHKERRQ(ierr);
+  ierr = STMatCreateVecsEmpty(eps->st,&t,NULL);CHKERRQ(ierr);
   ierr = BVSetSizesFromVec(sr->V,t,k);CHKERRQ(ierr);
   ierr = VecDestroy(&t);CHKERRQ(ierr);
   ierr = EPS_SetInnerProduct(eps);CHKERRQ(ierr);
@@ -166,8 +149,6 @@ static PetscErrorCode EPSSliceAllocateSolution(EPS eps,PetscInt extra)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceGetEPS"
 static PetscErrorCode EPSSliceGetEPS(EPS eps)
 {
   PetscErrorCode     ierr;
@@ -325,8 +306,6 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceGetInertia"
 static PetscErrorCode EPSSliceGetInertia(EPS eps,PetscReal shift,PetscInt *inertia,PetscInt *zeros)
 {
   PetscErrorCode ierr;
@@ -356,8 +335,15 @@ static PetscErrorCode EPSSliceGetInertia(EPS eps,PetscReal shift,PetscInt *inert
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_KrylovSchur_Slice"
+/*
+   Dummy backtransform operation
+ */
+static PetscErrorCode EPSBackTransform_Skip(EPS eps)
+{
+  PetscFunctionBegin;
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -365,20 +351,19 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data,*ctx_glob;
   EPS_SR          sr,sr_loc,sr_glob;
   PetscInt        nEigs,dssz=1,i,zeros=0,off=0;
-  PetscMPIInt     nproc,rank,aux;
+  PetscMPIInt     nproc,rank=0,aux;
   PetscReal       r;
   MPI_Request     req;
   Mat             A,B=NULL;
+  SlepcSC         sc;
+  PetscInt        flg=0;
+  DSParallelType  ptype;
 
   PetscFunctionBegin;
   if (ctx->global) {
-    if (eps->inta==0.0 && eps->intb==0.0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Must define a computational interval when using EPS_ALL");
     if (eps->intb >= PETSC_MAX_REAL && eps->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The defined computational interval should have at least one of their sides bounded");
     if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Spectrum slicing only available for symmetric/Hermitian eigenproblems");
     if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs cannot be used with spectrum slicing");
-    if (!((PetscObject)(eps->st))->type_name) { /* default to shift-and-invert */
-      ierr = STSetType(eps->st,STSINVERT);CHKERRQ(ierr);
-    }
     ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&issinv,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
     if (!issinv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Shift-and-invert or Cayley ST is needed for spectrum slicing");
     if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL*1e-2;  /* use tighter tolerance */
@@ -386,7 +371,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     if (ctx->nev==1) ctx->nev = PetscMin(40,eps->n);  /* nev not set, use default value */
     if (eps->n>10 && ctx->nev<10) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"nev cannot be less than 10 in spectrum slicing runs");
   }
-  eps->ops->backtransform = NULL;
+  eps->ops->backtransform = EPSBackTransform_Skip;
 
   /* create spectrum slicing context and initialize it */
   ierr = EPSSliceResetSR(eps);CHKERRQ(ierr);
@@ -401,7 +386,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
 
   if (ctx->npart==1 || ctx->global) {
     /* check presence of ends and finding direction */
-    if ((eps->inta > PETSC_MIN_REAL && eps->inta != 0.0) || eps->intb >= PETSC_MAX_REAL) {
+    if ((eps->inta > PETSC_MIN_REAL && !(ctx->subintervals && ctx->subintervals[0]==ctx->subintervals[1])) || eps->intb >= PETSC_MAX_REAL) {
       sr->int0 = eps->inta;
       sr->int1 = eps->intb;
       sr->dir = 1;
@@ -425,7 +410,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     if (ctx->npart>1) {
       if ((sr->dir>0&&ctx->subc->color==0)||(sr->dir<0&&ctx->subc->color==ctx->npart-1)) sr->inertia0 = sr_loc->inertia0;
       ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
-      if (rank==0) {
+      if (!rank) {
         ierr = MPI_Bcast(&sr->inertia0,1,MPIU_INT,(sr->dir>0)?0:ctx->npart-1,ctx->commrank);CHKERRQ(ierr);
       }
       ierr = MPI_Bcast(&sr->inertia0,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
@@ -472,19 +457,25 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
 
     /* compute inertia0 */
     ierr = EPSSliceGetInertia(eps,sr->int0,&sr->inertia0,ctx->detect?&zeros:NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsGetInt(NULL,NULL,"-eps_krylovschur_hiteigenvalue",&flg,NULL);CHKERRQ(ierr);
     if (zeros) { /* error in factorization */
-      if (ctx->npart==1 || ctx_glob->subintset || ((sr->dir>0 && ctx->subc->color==0) || (sr->dir<0 && ctx->subc->color==ctx->npart-1))) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in an interval endpoint defined by user");
-      else { /* perturb shift */
-        sr->int0 *= (1.0+SLICE_PTOL);
-        ierr = EPSSliceGetInertia(eps,sr->int0,&sr->inertia0,&zeros);CHKERRQ(ierr);
-        if (zeros) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_CONV_FAILED,"Inertia computation fails in %g",sr->int1);
+      if (sr->int0==ctx->eps->inta || sr->int0==ctx->eps->intb) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in the interval endpoint");
+      else if(ctx_glob->subintset && !flg) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in an interval endpoint defined by user");
+      else {
+        if (flg==1) { /* idle subgroup */
+          sr->inertia0 = -1;
+        } else { /* perturb shift */
+          sr->int0 *= (1.0+SLICE_PTOL);
+          ierr = EPSSliceGetInertia(eps,sr->int0,&sr->inertia0,&zeros);CHKERRQ(ierr);
+          if (zeros) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_CONV_FAILED,"Inertia computation fails in %g",sr->int1);
+        }
       }
     }
     if (ctx->npart>1) {
       /* inertia1 is received from neighbour */
       ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
       if (!rank) {
-        if ((sr->dir>0 && ctx->subc->color>0) || (sr->dir<0 && ctx->subc->color<ctx->npart-1)) { /* send inertia0 to neighbour0 */
+        if ( sr->inertia0!=-1 && ((sr->dir>0 && ctx->subc->color>0) || (sr->dir<0 && ctx->subc->color<ctx->npart-1)) ) { /* send inertia0 to neighbour0 */
           ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
           ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
         }
@@ -492,6 +483,11 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
           ierr = MPI_Recv(&(sr->inertia1),1,MPIU_INT,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRQ(ierr);
           ierr = MPI_Recv(&(sr->int1),1,MPIU_REAL,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRQ(ierr);
         }
+        if ( sr->inertia0==-1 && !(sr->dir>0 && ctx->subc->color==ctx->npart-1) && !(sr->dir<0 && ctx->subc->color==0)) {
+          sr->inertia0 = sr->inertia1; sr->int0 = sr->int1;
+          ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+          ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+        }
       }
       if ((sr->dir>0 && ctx->subc->color<ctx->npart-1)||(sr->dir<0 && ctx->subc->color>0)) {
         ierr = MPI_Bcast(&sr->inertia1,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
@@ -503,6 +499,11 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     if (ctx->npart==1 || ((sr->dir>0 && ctx->subc->color==ctx->npart-1) || (sr->dir<0 && ctx->subc->color==0))) {
       ierr = EPSSliceGetInertia(eps,sr->int1,&sr->inertia1,ctx->detect?&zeros:NULL);CHKERRQ(ierr);
       if (zeros) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in an interval endpoint defined by user");
+      if (!rank && sr->inertia0==-1) {
+        sr->inertia0 = sr->inertia1; sr->int0 = sr->int1;
+        ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+        ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+      }
       if (sr->hasEnd) {
         sr->dir = -sr->dir; r = sr->int0; sr->int0 = sr->int1; sr->int1 = r;
         i = sr->inertia0; sr->inertia0 = sr->inertia1; sr->inertia1 = i;
@@ -516,6 +517,16 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
       ierr = EPSSliceAllocateSolution(eps,1);CHKERRQ(ierr);
     }
     dssz = eps->ncv+1;
+    if (sr->numEigs>0) {
+      ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
+      sc->rg            = NULL;
+      sc->comparison    = SlepcCompareLargestMagnitude;
+      sc->comparisonctx = NULL;
+      sc->map           = NULL;
+      sc->mapobj        = NULL;
+    }
+    ierr = DSGetParallel(ctx->eps->ds,&ptype);CHKERRQ(ierr);
+    ierr = DSSetParallel(eps->ds,ptype);CHKERRQ(ierr);
   }
   ierr = DSSetType(eps->ds,DSHEP);CHKERRQ(ierr);
   ierr = DSSetCompact(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
@@ -524,7 +535,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
   ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
   ierr = PetscObjectStateGet((PetscObject)A,&ctx->Astate);CHKERRQ(ierr);
   ierr = PetscObjectGetId((PetscObject)A,&ctx->Aid);CHKERRQ(ierr);
-  if (B) { 
+  if (B) {
     ierr = PetscObjectStateGet((PetscObject)B,&ctx->Bstate);CHKERRQ(ierr);
     ierr = PetscObjectGetId((PetscObject)B,&ctx->Bid);CHKERRQ(ierr);
   } else {
@@ -534,8 +545,6 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceGatherEigenVectors"
 static PetscErrorCode EPSSliceGatherEigenVectors(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -598,8 +607,6 @@ static PetscErrorCode EPSSliceGatherEigenVectors(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSComputeVectors_Slice"
 /*
   EPSComputeVectors_Slice - Recover Eigenvectors from subcomunicators
  */
@@ -618,8 +625,6 @@ PetscErrorCode EPSComputeVectors_Slice(EPS eps)
 
 #define SWAP(a,b,t) {t=a;a=b;b=t;}
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceGetInertias"
 static PetscErrorCode EPSSliceGetInertias(EPS eps,PetscInt *n,PetscReal **shifts,PetscInt **inertias)
 {
   PetscErrorCode  ierr;
@@ -674,8 +679,6 @@ static PetscErrorCode EPSSliceGetInertias(EPS eps,PetscInt *n,PetscReal **shifts
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSliceGatherSolution"
 static PetscErrorCode EPSSliceGatherSolution(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -706,7 +709,7 @@ static PetscErrorCode EPSSliceGatherSolution(EPS eps)
   ierr = PetscMalloc1(ctx->npart,&ns_loc);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(ns,&aux);CHKERRQ(ierr);
-  if (rank==0) { ierr = MPI_Allgather(&aux,1,MPI_INT,ns_loc,1,MPI_INT,ctx->commrank);CHKERRQ(ierr); }
+  if (!rank) { ierr = MPI_Allgather(&aux,1,MPI_INT,ns_loc,1,MPI_INT,ctx->commrank);CHKERRQ(ierr); }
   ierr = PetscMPIIntCast(ctx->npart,&aux);CHKERRQ(ierr);
   ierr = MPI_Bcast(ns_loc,aux,MPI_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
   ctx->nshifts = 0;
@@ -734,7 +737,7 @@ static PetscErrorCode EPSSliceGatherSolution(EPS eps)
     ierr = MPI_Allreduce(&sr_loc->itsKs,&eps->its,1,MPIU_INT,MPI_SUM,ctx->commrank);CHKERRQ(ierr);
   } else { /* subcommunicators with different size */
     ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
-    if (rank==0) {
+    if (!rank) {
       ierr = PetscMPIIntCast(sr_loc->numEigs,&aux);CHKERRQ(ierr);
       ierr = MPI_Allgatherv(eigr_loc,aux,MPIU_SCALAR,eps->eigr,ctx->nconv_loc,disp,MPIU_SCALAR,ctx->commrank);CHKERRQ(ierr); /* eigenvalues */
       ierr = MPI_Allgatherv(perm_loc,aux,MPIU_INT,eps->perm,ctx->nconv_loc,disp,MPIU_INT,ctx->commrank);CHKERRQ(ierr); /* perm */
@@ -770,8 +773,6 @@ static PetscErrorCode EPSSliceGatherSolution(EPS eps)
 /*
    Fills the fields of a shift structure
 */
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreateShift"
 static PetscErrorCode EPSCreateShift(EPS eps,PetscReal val,EPS_shift neighb0,EPS_shift neighb1)
 {
   PetscErrorCode  ierr;
@@ -809,8 +810,6 @@ static PetscErrorCode EPSCreateShift(EPS eps,PetscReal val,EPS_shift neighb0,EPS
 }
 
 /* Prepare for Rational Krylov update */
-#undef __FUNCT__
-#define __FUNCT__ "EPSPrepareRational"
 static PetscErrorCode EPSPrepareRational(EPS eps)
 {
   EPS_KRYLOVSCHUR  *ctx=(EPS_KRYLOVSCHUR*)eps->data;
@@ -855,8 +854,6 @@ static PetscErrorCode EPSPrepareRational(EPS eps)
 }
 
 /* Provides next shift to be computed */
-#undef __FUNCT__
-#define __FUNCT__ "EPSExtractShift"
 static PetscErrorCode EPSExtractShift(EPS eps)
 {
   PetscErrorCode   ierr;
@@ -894,15 +891,13 @@ static PetscErrorCode EPSExtractShift(EPS eps)
    Allows searching an specific amount of eigenvalues in the subintervals left and right.
    Returns whether the search has succeeded
 */
-#undef __FUNCT__
-#define __FUNCT__ "EPSKrylovSchur_Slice"
 static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
-  PetscInt        i,conv,k,l,ld,nv,*iwork,j,p;
+  PetscInt        i,k,l,ld,nv,*iwork,j;
   Mat             U;
-  PetscScalar     *Q,*A,rtmp;
+  PetscScalar     *Q,*A;
   PetscReal       *a,*b,beta;
   PetscBool       breakdown;
   PetscInt        count0,count1;
@@ -976,50 +971,10 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
       ierr = DSSolve(eps->ds,eps->eigr,NULL);CHKERRQ(ierr);
       ierr = DSSort(eps->ds,eps->eigr,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
     }
+    ierr = DSSynchronize(eps->ds,eps->eigr,NULL);CHKERRQ(ierr);
+
     /* Residual */
     ierr = EPSKrylovConvergence(eps,PETSC_TRUE,eps->nconv,nv-eps->nconv,beta,1.0,&k);CHKERRQ(ierr);
-
-    if (ctx->lock) {
-      /* Check convergence */
-      ierr = DSGetArrayReal(eps->ds,DS_MAT_T,&a);CHKERRQ(ierr);
-      b = a + ld;
-      conv = 0;
-      j = k = eps->nconv;
-      for (i=eps->nconv;i<nv;i++) if (eps->errest[i] < eps->tol) conv++;
-      for (i=eps->nconv;i<nv;i++) {
-        if (eps->errest[i] < eps->tol) {
-          iwork[j++]=i;
-        } else iwork[conv+k++]=i;
-      }
-      for (i=eps->nconv;i<nv;i++) {
-        a[i]=PetscRealPart(eps->eigr[i]);
-        b[i]=eps->errest[i];
-      }
-      for (i=eps->nconv;i<nv;i++) {
-        eps->eigr[i] = a[iwork[i]];
-        eps->errest[i] = b[iwork[i]];
-      }
-      for (i=eps->nconv;i<nv;i++) {
-        a[i]=PetscRealPart(eps->eigr[i]);
-        b[i]=eps->errest[i];
-      }
-      ierr = DSRestoreArrayReal(eps->ds,DS_MAT_T,&a);CHKERRQ(ierr);
-      ierr = DSGetArray(eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-      for (i=eps->nconv;i<nv;i++) {
-        p=iwork[i];
-        if (p!=i) {
-          j=i+1;
-          while (iwork[j]!=i) j++;
-          iwork[j]=p;iwork[i]=i;
-          for (k=0;k<nv;k++) {
-            rtmp=Q[k+p*ld];Q[k+p*ld]=Q[k+i*ld];Q[k+i*ld]=rtmp;
-          }
-        }
-      }
-      ierr = DSRestoreArray(eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-      k=eps->nconv+conv;
-    }
-
     /* Checking values obtained for completing */
     for (i=0;i<k;i++) {
       sr->back[i]=eps->eigr[i];
@@ -1055,9 +1010,9 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
     }
     /* Update l */
     if (eps->reason == EPS_CONVERGED_ITERATING) l = PetscMax(1,(PetscInt)((nv-k)*ctx->keep));
-    else l = 0;
-    if (!ctx->lock && l>0) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
+    else l = nv-k;
     if (breakdown) l=0;
+    if (!ctx->lock && l>0 && eps->reason == EPS_CONVERGED_ITERATING) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
 
     if (eps->reason == EPS_CONVERGED_ITERATING) {
       if (breakdown) {
@@ -1109,7 +1064,7 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
   }
   sPres->comp[0] = PetscNot(count0 < sPres->nsch[0]);
   sPres->comp[1] = PetscNot(count1 < sPres->nsch[1]);
-  if (count0 > sPres->nsch[0] || count1 > sPres->nsch[1])SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
+  if (count0 > sPres->nsch[0] || count1 > sPres->nsch[1]) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
   ierr = PetscFree(iwork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -1117,8 +1072,6 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
 /*
   Obtains value of subsequent shift
 */
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetNewShiftValue"
 static PetscErrorCode EPSGetNewShiftValue(EPS eps,PetscInt side,PetscReal *newS)
 {
   PetscReal       lambda,d_prev;
@@ -1197,8 +1150,6 @@ static PetscErrorCode EPSGetNewShiftValue(EPS eps,PetscInt side,PetscReal *newS)
 /*
   Function for sorting an array of real values
 */
-#undef __FUNCT__
-#define __FUNCT__ "sortRealEigenvalues"
 static PetscErrorCode sortRealEigenvalues(PetscScalar *r,PetscInt *perm,PetscInt nr,PetscBool prev,PetscInt dir)
 {
   PetscReal      re;
@@ -1218,8 +1169,6 @@ static PetscErrorCode sortRealEigenvalues(PetscScalar *r,PetscInt *perm,PetscInt
 }
 
 /* Stores the pairs obtained since the last shift in the global arrays */
-#undef __FUNCT__
-#define __FUNCT__ "EPSStoreEigenpairs"
 static PetscErrorCode EPSStoreEigenpairs(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -1250,21 +1199,17 @@ static PetscErrorCode EPSStoreEigenpairs(EPS eps)
       sr->eigr[count] = lambda;
       sr->errest[count] = err;
       /* Explicit purification */
+      ierr = BVGetColumn(eps->V,eps->perm[i],&w);CHKERRQ(ierr);
       if (eps->purify) {
         ierr = BVGetColumn(sr->V,count,&v);CHKERRQ(ierr);
-        ierr = BVGetColumn(eps->V,eps->perm[i],&w);CHKERRQ(ierr);
         ierr = STApply(eps->st,w,v);CHKERRQ(ierr);
         ierr = BVRestoreColumn(sr->V,count,&v);CHKERRQ(ierr);
-        ierr = BVRestoreColumn(eps->V,eps->perm[i],&w);CHKERRQ(ierr);
-        ierr = BVNormColumn(sr->V,count,NORM_2,&norm);CHKERRQ(ierr);
-        ierr = BVScaleColumn(sr->V,count,1.0/norm);CHKERRQ(ierr);
       } else {
-        ierr = BVGetColumn(eps->V,eps->perm[i],&w);CHKERRQ(ierr);
         ierr = BVInsertVec(sr->V,count,w);CHKERRQ(ierr);
-        ierr = BVRestoreColumn(eps->V,eps->perm[i],&w);CHKERRQ(ierr);
-        ierr = BVNormColumn(sr->V,count,NORM_2,&norm);CHKERRQ(ierr);
-        ierr = BVScaleColumn(sr->V,count,1.0/norm);CHKERRQ(ierr);
       }
+      ierr = BVRestoreColumn(eps->V,eps->perm[i],&w);CHKERRQ(ierr);
+      ierr = BVNormColumn(sr->V,count,NORM_2,&norm);CHKERRQ(ierr);
+      ierr = BVScaleColumn(sr->V,count,1.0/norm);CHKERRQ(ierr);
       count++;
     }
   }
@@ -1275,8 +1220,6 @@ static PetscErrorCode EPSStoreEigenpairs(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLookForDeflation"
 static PetscErrorCode EPSLookForDeflation(EPS eps)
 {
   PetscErrorCode  ierr;
@@ -1314,12 +1257,12 @@ static PetscErrorCode EPSLookForDeflation(EPS eps)
   /* The number of values on each side are found */
   if (sPres->neighb[0]) {
     sPres->nsch[0] = (sr->dir)*(sPres->inertia - sPres->neighb[0]->inertia)-count0;
-    if (sPres->nsch[0]<0)SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
+    if (sPres->nsch[0]<0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
   } else sPres->nsch[0] = 0;
 
   if (sPres->neighb[1]) {
     sPres->nsch[1] = (sr->dir)*(sPres->neighb[1]->inertia - sPres->inertia) - count1;
-    if (sPres->nsch[1]<0)SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
+    if (sPres->nsch[1]<0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
   } else sPres->nsch[1] = (sr->dir)*(sr->inertia1 - sPres->inertia);
 
   /* Completing vector of indexes for deflation */
@@ -1347,12 +1290,10 @@ static PetscErrorCode EPSLookForDeflation(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_KrylovSchur_Slice"
 PetscErrorCode EPSSolve_KrylovSchur_Slice(EPS eps)
 {
   PetscErrorCode   ierr;
-  PetscInt         i,lds;
+  PetscInt         i,lds,ti;
   PetscReal        newS;
   EPS_KRYLOVSCHUR  *ctx=(EPS_KRYLOVSCHUR*)eps->data;
   EPS_SR           sr=ctx->sr;
@@ -1388,7 +1329,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Slice(EPS eps)
     ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
     ierr = PetscObjectStateGet((PetscObject)A,&Astate);CHKERRQ(ierr);
     ierr = PetscObjectGetId((PetscObject)A,&Aid);CHKERRQ(ierr);
-    if (B) { 
+    if (B) {
       ierr = PetscObjectStateGet((PetscObject)B,&Bstate);CHKERRQ(ierr);
       ierr = PetscObjectGetId((PetscObject)B,&Bid);CHKERRQ(ierr);
     }
@@ -1461,6 +1402,11 @@ PetscErrorCode EPSSolve_KrylovSchur_Slice(EPS eps)
     eps->reason = EPS_CONVERGED_TOL;
     eps->its    = sr->itsKs;
     eps->nds    = 0;
+    if (sr->dir<0) {
+      for (i=0;i<eps->nconv/2;i++) {
+        ti = sr->perm[i]; sr->perm[i] = sr->perm[eps->nconv-1-i]; sr->perm[eps->nconv-1-i] = ti;
+      }
+    }
   }
   PetscFunctionReturn(0);
 }
