@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "lanczos"
 
    Method: Explicitly Restarted Symmetric/Hermitian Lanczos
@@ -14,39 +22,17 @@
 
        [1] "Lanczos Methods in SLEPc", SLEPc Technical Report STR-5,
            available at http://slepc.upv.es.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
 #include <slepcblaslapack.h>
 
-PetscErrorCode EPSSolve_Lanczos(EPS);
-
 typedef struct {
-  EPSLanczosReorthogType reorthog;
-  BV                     AV;
+  EPSLanczosReorthogType reorthog;      /* user-provided reorthogonalization parameter */
+  PetscInt               allocsize;     /* number of columns of work BV's allocated at setup */
+  BV                     AV;            /* work BV used in selective reorthogonalization */
 } EPS_LANCZOS;
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_Lanczos"
 PetscErrorCode EPSSetUp_Lanczos(EPS eps)
 {
   EPS_LANCZOS        *lanczos = (EPS_LANCZOS*)eps->data;
@@ -80,7 +66,13 @@ PetscErrorCode EPSSetUp_Lanczos(EPS eps)
     ierr = PetscInfo(eps,"Switching to MGS orthogonalization\n");CHKERRQ(ierr);
   }
   if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_SELECTIVE) {
-    ierr = BVDuplicate(eps->V,&lanczos->AV);CHKERRQ(ierr);
+    if (!lanczos->allocsize) {
+      ierr = BVDuplicate(eps->V,&lanczos->AV);CHKERRQ(ierr);
+      ierr = BVGetSizes(lanczos->AV,NULL,NULL,&lanczos->allocsize);CHKERRQ(ierr);
+    } else { /* make sure V and AV have the same size */
+      ierr = BVGetSizes(eps->V,NULL,NULL,&lanczos->allocsize);CHKERRQ(ierr);
+      ierr = BVResize(lanczos->AV,lanczos->allocsize,PETSC_FALSE);CHKERRQ(ierr);
+    }
   }
 
   ierr = DSSetType(eps->ds,DSHEP);CHKERRQ(ierr);
@@ -90,15 +82,11 @@ PetscErrorCode EPSSetUp_Lanczos(EPS eps)
     ierr = EPSSetWorkVecs(eps,1);CHKERRQ(ierr);
   }
 
-  /* dispatch solve method */
   if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
   if (eps->isgeneralized && eps->ishermitian && !eps->ispositive) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Requested method does not work for indefinite problems");
-  eps->ops->solve = EPSSolve_Lanczos;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLocalLanczos"
 /*
    EPSLocalLanczos - Local reorthogonalization.
 
@@ -149,8 +137,6 @@ static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DenseTridiagonal"
 /*
    DenseTridiagonal - Solves a real tridiagonal Hermitian Eigenvalue Problem.
 
@@ -201,7 +187,7 @@ static PetscErrorCode DenseTridiagonal(PetscInt n_,PetscReal *D,PetscReal *E,Pet
   PetscStackCallBLAS("LAPACKstevr",LAPACKstevr_(jobz,"A",&n,D,E,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,iwork,&liwork,&info));
 #endif
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack DSTEVR %d",info);
+  SlepcCheckLapackInfo("stevr",info);
 #if defined(PETSC_USE_COMPLEX)
   if (V) {
     for (i=0;i<n;i++)
@@ -215,8 +201,6 @@ static PetscErrorCode DenseTridiagonal(PetscInt n_,PetscReal *D,PetscReal *E,Pet
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSelectiveLanczos"
 /*
    EPSSelectiveLanczos - Selective reorthogonalization.
 */
@@ -299,8 +283,6 @@ static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *be
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "update_omega"
 static void update_omega(PetscReal *omega,PetscReal *omega_old,PetscInt j,PetscReal *alpha,PetscReal *beta,PetscReal eps1,PetscReal anorm)
 {
   PetscInt  k;
@@ -336,8 +318,6 @@ static void update_omega(PetscReal *omega,PetscReal *omega_old,PetscInt j,PetscR
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "compute_int"
 static void compute_int(PetscBool *which,PetscReal *mu,PetscInt j,PetscReal delta,PetscReal eta)
 {
   PetscInt  i,k,maxpos;
@@ -378,8 +358,6 @@ static void compute_int(PetscBool *which,PetscReal *mu,PetscInt j,PetscReal delt
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSPartialLanczos"
 /*
    EPSPartialLanczos - Partial reorthogonalization.
 */
@@ -490,8 +468,6 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSBasicLanczos"
 /*
    EPSBasicLanczos - Computes an m-step Lanczos factorization. The first k
    columns are assumed to be locked and therefore they are not modified. On
@@ -523,14 +499,14 @@ static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
     case EPS_LANCZOS_REORTHOG_LOCAL:
       ierr = EPSLocalLanczos(eps,alpha,beta,k,m,breakdown);CHKERRQ(ierr);
       break;
-    case EPS_LANCZOS_REORTHOG_SELECTIVE:
-      ierr = EPSSelectiveLanczos(eps,alpha,beta,k,m,breakdown,anorm);CHKERRQ(ierr);
-      break;
     case EPS_LANCZOS_REORTHOG_FULL:
       ierr = EPSFullLanczos(eps,alpha,beta,k,m,breakdown);CHKERRQ(ierr);
       break;
-    case EPS_LANCZOS_REORTHOG_PARTIAL:
+    case EPS_LANCZOS_REORTHOG_SELECTIVE:
+      ierr = EPSSelectiveLanczos(eps,alpha,beta,k,m,breakdown,anorm);CHKERRQ(ierr);
+      break;
     case EPS_LANCZOS_REORTHOG_PERIODIC:
+    case EPS_LANCZOS_REORTHOG_PARTIAL:
       ierr = EPSPartialLanczos(eps,alpha,beta,k,m,breakdown,anorm);CHKERRQ(ierr);
       break;
     case EPS_LANCZOS_REORTHOG_DELAYED:
@@ -549,14 +525,10 @@ static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
       beta[n-1] = betam;
       ierr = PetscFree(T);CHKERRQ(ierr);
       break;
-    default:
-      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid reorthogonalization type");
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_Lanczos"
 PetscErrorCode EPSSolve_Lanczos(EPS eps)
 {
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS*)eps->data;
@@ -597,6 +569,7 @@ PetscErrorCode EPSSolve_Lanczos(EPS eps)
     /* Solve projected problem */
     ierr = DSSolve(eps->ds,ritz,NULL);CHKERRQ(ierr);
     ierr = DSSort(eps->ds,ritz,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = DSSynchronize(eps->ds,ritz,NULL);CHKERRQ(ierr);
 
     /* Estimate ||A|| */
     for (i=nconv;i<n;i++)
@@ -723,8 +696,7 @@ PetscErrorCode EPSSolve_Lanczos(EPS eps)
       ierr = BVCopyColumn(eps->V,n,nconv);CHKERRQ(ierr);
       if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_LOCAL && !breakdown) {
         /* Reorthonormalize restart vector */
-        ierr = BVOrthogonalizeColumn(eps->V,nconv,NULL,&norm,&breakdown);CHKERRQ(ierr);
-        ierr = BVScaleColumn(eps->V,nconv,1.0/norm);CHKERRQ(ierr);
+        ierr = BVOrthonormalizeColumn(eps->V,nconv,PETSC_FALSE,NULL,&breakdown);CHKERRQ(ierr);
       }
       if (breakdown) {
         /* Use random vector for restarting */
@@ -743,8 +715,6 @@ PetscErrorCode EPSSolve_Lanczos(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetFromOptions_Lanczos"
 PetscErrorCode EPSSetFromOptions_Lanczos(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode         ierr;
@@ -754,16 +724,14 @@ PetscErrorCode EPSSetFromOptions_Lanczos(PetscOptionItems *PetscOptionsObject,EP
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS Lanczos Options");CHKERRQ(ierr);
-  ierr = PetscOptionsEnum("-eps_lanczos_reorthog","Lanczos reorthogonalization","EPSLanczosSetReorthog",EPSLanczosReorthogTypes,(PetscEnum)lanczos->reorthog,(PetscEnum*)&reorthog,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = EPSLanczosSetReorthog(eps,reorthog);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsEnum("-eps_lanczos_reorthog","Lanczos reorthogonalization","EPSLanczosSetReorthog",EPSLanczosReorthogTypes,(PetscEnum)lanczos->reorthog,(PetscEnum*)&reorthog,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSLanczosSetReorthog(eps,reorthog);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLanczosSetReorthog_Lanczos"
 static PetscErrorCode EPSLanczosSetReorthog_Lanczos(EPS eps,EPSLanczosReorthogType reorthog)
 {
   EPS_LANCZOS *lanczos = (EPS_LANCZOS*)eps->data;
@@ -776,7 +744,10 @@ static PetscErrorCode EPSLanczosSetReorthog_Lanczos(EPS eps,EPSLanczosReorthogTy
     case EPS_LANCZOS_REORTHOG_SELECTIVE:
     case EPS_LANCZOS_REORTHOG_PERIODIC:
     case EPS_LANCZOS_REORTHOG_PARTIAL:
-      lanczos->reorthog = reorthog;
+      if (lanczos->reorthog != reorthog) {
+        lanczos->reorthog = reorthog;
+        eps->state = EPS_STATE_INITIAL;
+      }
       break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid reorthogonalization type");
@@ -784,8 +755,6 @@ static PetscErrorCode EPSLanczosSetReorthog_Lanczos(EPS eps,EPSLanczosReorthogTy
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLanczosSetReorthog"
 /*@
    EPSLanczosSetReorthog - Sets the type of reorthogonalization used during the Lanczos
    iteration.
@@ -815,8 +784,6 @@ PetscErrorCode EPSLanczosSetReorthog(EPS eps,EPSLanczosReorthogType reorthog)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLanczosGetReorthog_Lanczos"
 static PetscErrorCode EPSLanczosGetReorthog_Lanczos(EPS eps,EPSLanczosReorthogType *reorthog)
 {
   EPS_LANCZOS *lanczos = (EPS_LANCZOS*)eps->data;
@@ -826,8 +793,6 @@ static PetscErrorCode EPSLanczosGetReorthog_Lanczos(EPS eps,EPSLanczosReorthogTy
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSLanczosGetReorthog"
 /*@
    EPSLanczosGetReorthog - Gets the type of reorthogonalization used during
    the Lanczos iteration.
@@ -855,8 +820,6 @@ PetscErrorCode EPSLanczosGetReorthog(EPS eps,EPSLanczosReorthogType *reorthog)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset_Lanczos"
 PetscErrorCode EPSReset_Lanczos(EPS eps)
 {
   PetscErrorCode ierr;
@@ -864,11 +827,10 @@ PetscErrorCode EPSReset_Lanczos(EPS eps)
 
   PetscFunctionBegin;
   ierr = BVDestroy(&lanczos->AV);CHKERRQ(ierr);
+  lanczos->allocsize = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_Lanczos"
 PetscErrorCode EPSDestroy_Lanczos(EPS eps)
 {
   PetscErrorCode ierr;
@@ -880,8 +842,6 @@ PetscErrorCode EPSDestroy_Lanczos(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView_Lanczos"
 PetscErrorCode EPSView_Lanczos(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -891,13 +851,11 @@ PetscErrorCode EPSView_Lanczos(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Lanczos: %s reorthogonalization\n",EPSLanczosReorthogTypes[lanczos->reorthog]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %s reorthogonalization\n",EPSLanczosReorthogTypes[lanczos->reorthog]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_Lanczos"
 PETSC_EXTERN PetscErrorCode EPSCreate_Lanczos(EPS eps)
 {
   EPS_LANCZOS    *ctx;
@@ -907,13 +865,17 @@ PETSC_EXTERN PetscErrorCode EPSCreate_Lanczos(EPS eps)
   ierr = PetscNewLog(eps,&ctx);CHKERRQ(ierr);
   eps->data = (void*)ctx;
 
-  eps->ops->setup                = EPSSetUp_Lanczos;
-  eps->ops->setfromoptions       = EPSSetFromOptions_Lanczos;
-  eps->ops->destroy              = EPSDestroy_Lanczos;
-  eps->ops->reset                = EPSReset_Lanczos;
-  eps->ops->view                 = EPSView_Lanczos;
-  eps->ops->backtransform        = EPSBackTransform_Default;
-  eps->ops->computevectors       = EPSComputeVectors_Hermitian;
+  eps->useds = PETSC_TRUE;
+
+  eps->ops->solve          = EPSSolve_Lanczos;
+  eps->ops->setup          = EPSSetUp_Lanczos;
+  eps->ops->setfromoptions = EPSSetFromOptions_Lanczos;
+  eps->ops->destroy        = EPSDestroy_Lanczos;
+  eps->ops->reset          = EPSReset_Lanczos;
+  eps->ops->view           = EPSView_Lanczos;
+  eps->ops->backtransform  = EPSBackTransform_Default;
+  eps->ops->computevectors = EPSComputeVectors_Hermitian;
+
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSLanczosSetReorthog_C",EPSLanczosSetReorthog_Lanczos);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSLanczosGetReorthog_C",EPSLanczosGetReorthog_Lanczos);CHKERRQ(ierr);
   PetscFunctionReturn(0);
