@@ -1,36 +1,24 @@
+/*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
 /*
    This file implements a wrapper to the LAPACK eigenvalue subroutines.
    Generalized problems are transformed to standard ones only if necessary.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_LAPACK"
 PetscErrorCode EPSSetUp_LAPACK(EPS eps)
 {
   PetscErrorCode ierr,ierra,ierrb;
-  PetscBool      isshift,denseok=PETSC_FALSE;
-  Mat            A,B,OP,Adense=NULL,Bdense=NULL;
+  PetscBool      isshift,flg,denseok=PETSC_FALSE;
+  Mat            A,B,OP,shell,Ar,Br,Adense=NULL,Bdense=NULL;
   PetscScalar    shift,*Ap,*Bp;
   PetscInt       i,ld,nmat;
   KSP            ksp;
@@ -50,16 +38,26 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&isshift);CHKERRQ(ierr);
   if (isshift) {
     ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-    ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
-    if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
-    PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
-    ierra = SlepcMatConvertSeqDense(A,&Adense);CHKERRQ(ierr);
+    ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+    ierr = MatHasOperation(A,MATOP_CREATE_SUBMATRICES,&flg);CHKERRQ(ierr);
+    if (flg) {
+      PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
+      ierra  = MatCreateRedundantMatrix(A,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Ar);
+      if (!ierra) { ierra |= MatConvert(Ar,MATSEQDENSE,MAT_INITIAL_MATRIX,&Adense); }
+      ierra |= MatDestroy(&Ar);
+      PetscPopErrorHandler();
+    } else ierra = 1;
     if (nmat>1) {
-      ierrb = SlepcMatConvertSeqDense(B,&Bdense);CHKERRQ(ierr);
-    } else {
-      ierrb = 0;
-    }
-    PetscPopErrorHandler();
+      ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
+      ierr = MatHasOperation(B,MATOP_CREATE_SUBMATRICES,&flg);CHKERRQ(ierr);
+      if (flg) {
+        PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
+        ierrb  = MatCreateRedundantMatrix(B,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Br);
+        if (!ierrb) { ierrb |= MatConvert(Br,MATSEQDENSE,MAT_INITIAL_MATRIX,&Bdense); }
+        ierrb |= MatDestroy(&Br);
+        PetscPopErrorHandler();
+      } else ierrb = 1;
+    } else ierrb = 0;
     denseok = PetscNot(ierra || ierrb);
   }
 
@@ -101,9 +99,14 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
     ierr = PCSetType(pc,PCNONE);CHKERRQ(ierr);
   } else {
     ierr = PetscInfo(eps,"Using slow explicit operator\n");CHKERRQ(ierr);
-    ierr = STComputeExplicitOperator(eps->st,&OP);CHKERRQ(ierr);
+    ierr = STGetOperator(eps->st,&shell);CHKERRQ(ierr);
+    ierr = MatComputeExplicitOperator(shell,&OP);CHKERRQ(ierr);
+    ierr = MatDestroy(&shell);CHKERRQ(ierr);
     ierr = MatDestroy(&Adense);CHKERRQ(ierr);
-    ierr = SlepcMatConvertSeqDense(OP,&Adense);CHKERRQ(ierr);
+    ierr = MatCreateRedundantMatrix(OP,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
+    ierr = MatDestroy(&OP);CHKERRQ(ierr);
+    ierr = MatConvert(Ar,MATSEQDENSE,MAT_INITIAL_MATRIX,&Adense);CHKERRQ(ierr);
+    ierr = MatDestroy(&Ar);CHKERRQ(ierr);
   }
 
   /* fill DS matrices */
@@ -125,14 +128,12 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
     ierr = DSRestoreArray(eps->ds,DS_MAT_B,&Bp);CHKERRQ(ierr);
   }
   ierr = VecDestroy(&v);CHKERRQ(ierr);
+  ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
   ierr = MatDestroy(&Adense);CHKERRQ(ierr);
-  if (!denseok) { ierr = MatDestroy(&OP);CHKERRQ(ierr); }
-  if (denseok && eps->isgeneralized) { ierr = MatDestroy(&Bdense);CHKERRQ(ierr); }
+  ierr = MatDestroy(&Bdense);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_LAPACK"
 PetscErrorCode EPSSolve_LAPACK(EPS eps)
 {
   PetscErrorCode ierr;
@@ -143,6 +144,7 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
   PetscFunctionBegin;
   ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
   ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
   /* right eigenvectors */
   ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
@@ -163,14 +165,15 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_LAPACK"
 PETSC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
 {
   PetscFunctionBegin;
-  eps->ops->solve                = EPSSolve_LAPACK;
-  eps->ops->setup                = EPSSetUp_LAPACK;
-  eps->ops->backtransform        = EPSBackTransform_Default;
+  eps->useds = PETSC_TRUE;
+  eps->categ = EPS_CATEGORY_OTHER;
+
+  eps->ops->solve          = EPSSolve_LAPACK;
+  eps->ops->setup          = EPSSetUp_LAPACK;
+  eps->ops->backtransform  = EPSBackTransform_Default;
   PetscFunctionReturn(0);
 }
 
