@@ -1,25 +1,15 @@
 /*
-   The basic EPS routines, Create, Destroy, etc. are here.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic EPS routines
+*/
 
 #include <slepc/private/epsimpl.h>      /*I "slepceps.h" I*/
 
@@ -28,8 +18,6 @@ PetscBool         EPSRegisterAllCalled = PETSC_FALSE;
 PetscClassId      EPS_CLASSID = 0;
 PetscLogEvent     EPS_SetUp = 0,EPS_Solve = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate"
 /*@
    EPSCreate - Creates the default EPS context.
 
@@ -82,8 +70,10 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->purify          = PETSC_TRUE;
 
   eps->converged       = EPSConvergedRelative;
+  eps->convergeduser   = NULL;
   eps->convergeddestroy= NULL;
   eps->stopping        = EPSStoppingBasic;
+  eps->stoppinguser    = NULL;
   eps->stoppingdestroy = NULL;
   eps->arbitrary       = NULL;
   eps->convergedctx    = NULL;
@@ -109,11 +99,13 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->data            = NULL;
 
   eps->state           = EPS_STATE_INITIAL;
+  eps->categ           = EPS_CATEGORY_KRYLOV;
   eps->nconv           = 0;
   eps->its             = 0;
   eps->nloc            = 0;
   eps->nrma            = 0.0;
   eps->nrmb            = 0.0;
+  eps->useds           = PETSC_FALSE;
   eps->isgeneralized   = PETSC_FALSE;
   eps->ispositive      = PETSC_FALSE;
   eps->ishermitian     = PETSC_FALSE;
@@ -124,8 +116,6 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetType"
 /*@C
    EPSSetType - Selects the particular solver to be used in the EPS object.
 
@@ -179,8 +169,6 @@ PetscErrorCode EPSSetType(EPS eps,EPSType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetType"
 /*@C
    EPSGetType - Gets the EPS type as a string from the EPS object.
 
@@ -205,8 +193,6 @@ PetscErrorCode EPSGetType(EPS eps,EPSType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSRegister"
 /*@C
    EPSRegister - Adds a method to the eigenproblem solver package.
 
@@ -242,17 +228,21 @@ PetscErrorCode EPSRegister(const char *name,PetscErrorCode (*function)(EPS))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReset"
 /*@
-   EPSReset - Resets the EPS context to the initial state and removes any
-   allocated objects.
+   EPSReset - Resets the EPS context to the initial state (prior to setup)
+   and destroys any allocated Vecs and Mats.
 
    Collective on EPS
 
    Input Parameter:
 .  eps - eigensolver context obtained from EPSCreate()
 
+   Note:
+   This can be used when a problem of different matrix size wants to be solved.
+   All options that have previously been set are preserved, so in a next use
+   the solver configuration is the same, but new sizes for matrices and vectors
+   are allowed.
+
    Level: advanced
 
 .seealso: EPSDestroy()
@@ -260,19 +250,13 @@ PetscErrorCode EPSRegister(const char *name,PetscErrorCode (*function)(EPS))
 PetscErrorCode EPSReset(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       ncols;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  if (eps) PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  if (!eps) PetscFunctionReturn(0);
   if (eps->ops->reset) { ierr = (eps->ops->reset)(eps);CHKERRQ(ierr); }
   if (eps->st) { ierr = STReset(eps->st);CHKERRQ(ierr); }
-  if (eps->ds) { ierr = DSReset(eps->ds);CHKERRQ(ierr); }
   ierr = VecDestroy(&eps->D);CHKERRQ(ierr);
-  ierr = BVGetSizes(eps->V,NULL,NULL,&ncols);CHKERRQ(ierr);
-  if (ncols) {
-    ierr = PetscFree4(eps->eigr,eps->eigi,eps->errest,eps->perm);CHKERRQ(ierr);
-    ierr = PetscFree2(eps->rr,eps->ri);CHKERRQ(ierr);
-  }
   ierr = BVDestroy(&eps->V);CHKERRQ(ierr);
   ierr = VecDestroyVecs(eps->nwork,&eps->work);CHKERRQ(ierr);
   eps->nwork = 0;
@@ -280,8 +264,6 @@ PetscErrorCode EPSReset(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy"
 /*@
    EPSDestroy - Destroys the EPS context.
 
@@ -304,6 +286,12 @@ PetscErrorCode EPSDestroy(EPS *eps)
   if (--((PetscObject)(*eps))->refct > 0) { *eps = 0; PetscFunctionReturn(0); }
   ierr = EPSReset(*eps);CHKERRQ(ierr);
   if ((*eps)->ops->destroy) { ierr = (*(*eps)->ops->destroy)(*eps);CHKERRQ(ierr); }
+  if ((*eps)->eigr) {
+    ierr = PetscFree4((*eps)->eigr,(*eps)->eigi,(*eps)->errest,(*eps)->perm);CHKERRQ(ierr);
+  }
+  if ((*eps)->rr) {
+    ierr = PetscFree2((*eps)->rr,(*eps)->ri);CHKERRQ(ierr);
+  }
   ierr = STDestroy(&(*eps)->st);CHKERRQ(ierr);
   ierr = RGDestroy(&(*eps)->rg);CHKERRQ(ierr);
   ierr = DSDestroy(&(*eps)->ds);CHKERRQ(ierr);
@@ -319,8 +307,6 @@ PetscErrorCode EPSDestroy(EPS *eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetTarget"
 /*@
    EPSSetTarget - Sets the value of the target.
 
@@ -358,8 +344,6 @@ PetscErrorCode EPSSetTarget(EPS eps,PetscScalar target)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetTarget"
 /*@
    EPSGetTarget - Gets the value of the target.
 
@@ -387,8 +371,6 @@ PetscErrorCode EPSGetTarget(EPS eps,PetscScalar* target)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetInterval"
 /*@
    EPSSetInterval - Defines the computational interval for spectrum slicing.
 
@@ -423,15 +405,15 @@ PetscErrorCode EPSSetInterval(EPS eps,PetscReal inta,PetscReal intb)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveReal(eps,inta,2);
   PetscValidLogicalCollectiveReal(eps,intb,3);
-  if (inta>=intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
-  eps->inta = inta;
-  eps->intb = intb;
-  eps->state = EPS_STATE_INITIAL;
+  if (inta>intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
+  if (eps->inta != inta || eps->intb != intb) {
+    eps->inta = inta;
+    eps->intb = intb;
+    eps->state = EPS_STATE_INITIAL;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetInterval"
 /*@
    EPSGetInterval - Gets the computational interval for spectrum slicing.
 
@@ -455,15 +437,11 @@ PetscErrorCode EPSGetInterval(EPS eps,PetscReal* inta,PetscReal* intb)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(inta,2);
-  PetscValidPointer(intb,3);
   if (inta) *inta = eps->inta;
   if (intb) *intb = eps->intb;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetST"
 /*@
    EPSSetST - Associates a spectral transformation object to the eigensolver.
 
@@ -496,8 +474,6 @@ PetscErrorCode EPSSetST(EPS eps,ST st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetST"
 /*@
    EPSGetST - Obtain the spectral transformation (ST) object associated
    to the eigensolver object.
@@ -529,8 +505,6 @@ PetscErrorCode EPSGetST(EPS eps,ST *st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetBV"
 /*@
    EPSSetBV - Associates a basis vectors object to the eigensolver.
 
@@ -563,8 +537,6 @@ PetscErrorCode EPSSetBV(EPS eps,BV V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetBV"
 /*@
    EPSGetBV - Obtain the basis vectors object associated to the eigensolver object.
 
@@ -595,8 +567,6 @@ PetscErrorCode EPSGetBV(EPS eps,BV *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetRG"
 /*@
    EPSSetRG - Associates a region object to the eigensolver.
 
@@ -629,8 +599,6 @@ PetscErrorCode EPSSetRG(EPS eps,RG rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetRG"
 /*@
    EPSGetRG - Obtain the region object associated to the eigensolver.
 
@@ -661,8 +629,6 @@ PetscErrorCode EPSGetRG(EPS eps,RG *rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetDS"
 /*@
    EPSSetDS - Associates a direct solver object to the eigensolver.
 
@@ -695,8 +661,6 @@ PetscErrorCode EPSSetDS(EPS eps,DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetDS"
 /*@
    EPSGetDS - Obtain the direct solver object associated to the eigensolver object.
 
@@ -727,8 +691,6 @@ PetscErrorCode EPSGetDS(EPS eps,DS *ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSIsGeneralized"
 /*@
    EPSIsGeneralized - Ask if the EPS object corresponds to a generalized
    eigenvalue problem.
@@ -754,8 +716,6 @@ PetscErrorCode EPSIsGeneralized(EPS eps,PetscBool* is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSIsHermitian"
 /*@
    EPSIsHermitian - Ask if the EPS object corresponds to a Hermitian
    eigenvalue problem.
@@ -781,8 +741,6 @@ PetscErrorCode EPSIsHermitian(EPS eps,PetscBool* is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSIsPositive"
 /*@
    EPSIsPositive - Ask if the EPS object corresponds to an eigenvalue
    problem type that requires a positive (semi-) definite matrix B.
