@@ -1,30 +1,76 @@
 /*
-      EPS routines related to problem setup.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   EPS routines related to problem setup
+*/
 
 #include <slepc/private/epsimpl.h>       /*I "slepceps.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp"
+/*
+   Let the solver choose the ST type that should be used by default,
+   otherwise set it to SHIFT.
+   This is called at EPSSetFromOptions (before STSetFromOptions)
+   and also at EPSSetUp (in case EPSSetFromOptions was not called).
+*/
+PetscErrorCode EPSSetDefaultST(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (eps->ops->setdefaultst) { ierr = (*eps->ops->setdefaultst)(eps);CHKERRQ(ierr); }
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = STSetType(eps->st,STSHIFT);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/*
+   This is done by preconditioned eigensolvers that use the PC only.
+   It sets STPRECOND with KSPPREONLY.
+*/
+PetscErrorCode EPSSetDefaultST_Precond(EPS eps)
+{
+  PetscErrorCode ierr;
+  KSP            ksp;
+
+  PetscFunctionBegin;
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
+    ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
+  if (!((PetscObject)ksp)->type_name) {
+    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/*
+   This is done by preconditioned eigensolvers that can also use the KSP.
+   It sets STPRECOND with the default KSP (GMRES).
+*/
+PetscErrorCode EPSSetDefaultST_GMRES(EPS eps)
+{
+  PetscErrorCode ierr;
+  KSP            ksp;
+
+  PetscFunctionBegin;
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
+    ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
+  ierr = KSPSetTolerances(ksp,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT,5);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    EPSSetUp - Sets up all the internal data structures necessary for the
    execution of the eigensolver. Then calls STSetUp() for any set-up
@@ -50,7 +96,7 @@ PetscErrorCode EPSSetUp(EPS eps)
   Mat            A,B;
   SlepcSC        sc;
   PetscInt       k,nmat;
-  PetscBool      flg,istrivial;
+  PetscBool      flg,istrivial,precond;
 #if defined(PETSC_USE_COMPLEX)
   PetscScalar    sigma;
 #endif
@@ -68,13 +114,14 @@ PetscErrorCode EPSSetUp(EPS eps)
     ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
   }
   if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = PetscObjectTypeCompareAny((PetscObject)eps,&flg,EPSGD,EPSJD,EPSRQCG,EPSBLOPEX,EPSPRIMME,"");CHKERRQ(ierr);
-    ierr = STSetType(eps->st,flg?STPRECOND:STSHIFT);CHKERRQ(ierr);
-  }
+  ierr = EPSSetDefaultST(eps);CHKERRQ(ierr);
+
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond);CHKERRQ(ierr);
+  if (eps->categ==EPS_CATEGORY_PRECOND && !precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires ST=PRECOND");
+  if (eps->categ!=EPS_CATEGORY_PRECOND && precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"STPRECOND is intended for preconditioned eigensolvers only");
+
   ierr = STSetTransform(eps->st,PETSC_TRUE);CHKERRQ(ierr);
-  if (!eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
-  ierr = DSReset(eps->ds);CHKERRQ(ierr);
+  if (eps->useds && !eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
   if (!eps->rg) { ierr = EPSGetRG(eps,&eps->rg);CHKERRQ(ierr); }
   if (!((PetscObject)eps->rg)->type_name) {
     ierr = RGSetType(eps->rg,RGINTERVAL);CHKERRQ(ierr);
@@ -105,11 +152,11 @@ PetscErrorCode EPSSetUp(EPS eps)
   /* initialization of matrix norms */
   if (eps->conv==EPS_CONV_NORM) {
     if (!eps->nrma) {
-      ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
+      ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
       ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
     }
     if (nmat>1 && !eps->nrmb) {
-      ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr);
+      ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
       ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
     }
   }
@@ -117,6 +164,19 @@ PetscErrorCode EPSSetUp(EPS eps)
   /* call specific solver setup */
   ierr = (*eps->ops->setup)(eps);CHKERRQ(ierr);
 
+  /* if purification is set, check that it really makes sense */
+  if (eps->purify) {
+    if (eps->categ==EPS_CATEGORY_PRECOND || eps->categ==EPS_CATEGORY_CONTOUR) eps->purify = PETSC_FALSE;
+    else {
+      if (!eps->isgeneralized) eps->purify = PETSC_FALSE;
+      else if (!eps->ishermitian && !eps->ispositive) eps->purify = PETSC_FALSE;
+      else {
+        ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
+        if (flg) eps->purify = PETSC_FALSE;
+      }
+    }
+  }
+
   /* check extraction */
   ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&flg,STPRECOND,STSHIFT,"");CHKERRQ(ierr);
   if (!flg && eps->extraction && eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cannot use a spectral transformation combined with harmonic extraction");
@@ -159,8 +219,10 @@ PetscErrorCode EPSSetUp(EPS eps)
       eps->sc->comparisonctx = &eps->target;
       break;
     case EPS_TARGET_IMAGINARY:
+#if defined(PETSC_USE_COMPLEX)
       eps->sc->comparison    = SlepcCompareTargetImaginary;
       eps->sc->comparisonctx = &eps->target;
+#endif
       break;
     case EPS_ALL:
       eps->sc->comparison    = SlepcCompareSmallestReal;
@@ -172,21 +234,17 @@ PetscErrorCode EPSSetUp(EPS eps)
   eps->sc->map    = NULL;
   eps->sc->mapobj = NULL;
 
-  /* fill sorting criterion for DS */
-  ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
-  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
-  if (eps->which==EPS_ALL) {
-    sc->rg            = NULL;
-    sc->comparison    = SlepcCompareLargestMagnitude;
-    sc->comparisonctx = NULL;
-    sc->map           = NULL;
-    sc->mapobj        = NULL;
-  } else {
-    sc->rg            = istrivial? NULL: eps->rg;
-    sc->comparison    = eps->sc->comparison;
-    sc->comparisonctx = eps->sc->comparisonctx;
-    sc->map           = SlepcMap_ST;
-    sc->mapobj        = (PetscObject)eps->st;
+  if (eps->useds) {
+    /* fill sorting criterion for DS */
+    ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
+    ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
+    if (eps->which!=EPS_ALL) {
+      sc->rg            = istrivial? NULL: eps->rg;
+      sc->comparison    = eps->sc->comparison;
+      sc->comparisonctx = eps->sc->comparisonctx;
+      sc->map           = SlepcMap_ST;
+      sc->mapobj        = (PetscObject)eps->st;
+    }
   }
 
   /* Build balancing matrix if required */
@@ -210,6 +268,8 @@ PetscErrorCode EPSSetUp(EPS eps)
 #endif
   ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
   if (flg && eps->problem_type == EPS_PGNHEP) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+  ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&flg,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
+  if (flg && (eps->categ==EPS_CATEGORY_KRYLOV || eps->categ==EPS_CATEGORY_OTHER) && (eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_IMAGINARY && eps->which!=EPS_ALL && eps->which!=EPS_WHICH_USER)) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Shift-and-invert requires a target 'which' (see EPSSetWhichEigenpairs), for instance -st_type sinvert -eps_target 0 -eps_target_magnitude");
 
   /* process deflation and initial vectors */
   if (eps->nds<0) {
@@ -232,9 +292,7 @@ PetscErrorCode EPSSetUp(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetOperators"
-/*@
+/*@C
    EPSSetOperators - Sets the matrices associated with the eigenvalue problem.
 
    Collective on EPS and Mat
@@ -252,7 +310,7 @@ PetscErrorCode EPSSetUp(EPS eps)
 
    Level: beginner
 
-.seealso: EPSSolve(), EPSSetUp(), EPSReset(), EPSGetST(), STGetOperators()
+.seealso: EPSSolve(), EPSSetUp(), EPSReset(), EPSGetST(), STGetMatrix()
 @*/
 PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 {
@@ -284,13 +342,11 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
     mat[1] = B;
     nmat = 2;
   } else nmat = 1;
-  ierr = STSetOperators(eps->st,nmat,mat);CHKERRQ(ierr);
+  ierr = STSetMatrices(eps->st,nmat,mat);CHKERRQ(ierr);
   eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetOperators"
 /*@
    EPSGetOperators - Gets the matrices associated with the eigensystem.
 
@@ -305,7 +361,7 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 
    Level: intermediate
 
-.seealso: EPSSolve(), EPSGetST(), STGetOperators(), STSetOperators()
+.seealso: EPSSolve(), EPSGetST(), STGetMatrix(), STSetMatrices()
 @*/
 PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 {
@@ -316,19 +372,17 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   ierr = EPSGetST(eps,&st);CHKERRQ(ierr);
-  if (A) { ierr = STGetOperators(st,0,A);CHKERRQ(ierr); }
+  if (A) { ierr = STGetMatrix(st,0,A);CHKERRQ(ierr); }
   if (B) {
     ierr = STGetNumMatrices(st,&k);CHKERRQ(ierr);
     if (k==1) B = NULL;
     else {
-      ierr = STGetOperators(st,1,B);CHKERRQ(ierr);
+      ierr = STGetMatrix(st,1,B);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetDeflationSpace"
 /*@
    EPSSetDeflationSpace - Specify a basis of vectors that constitute the deflation
    space.
@@ -369,9 +423,7 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetInitialSpace"
-/*@
+/*@C
    EPSSetInitialSpace - Specify a basis of vectors that constitute the initial
    space, that is, the subspace from which the solver starts to iterate.
 
@@ -412,8 +464,6 @@ PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec *is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetDimensions_Default"
 /*
   EPSSetDimensions_Default - Set reasonable values for ncv, mpd if not set
   by the user. This is called at setup.
@@ -444,8 +494,6 @@ PetscErrorCode EPSSetDimensions_Default(EPS eps,PetscInt nev,PetscInt *ncv,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSAllocateSolution"
 /*@
    EPSAllocateSolution - Allocate memory storage for common variables such
    as eigenvalues and eigenvectors.
@@ -479,9 +527,7 @@ PetscErrorCode EPSAllocateSolution(EPS eps,PetscInt extra)
 
   /* allocate space for eigenvalues and friends */
   if (requested != oldsize || !eps->eigr) {
-    if (oldsize) {
-      ierr = PetscFree4(eps->eigr,eps->eigi,eps->errest,eps->perm);CHKERRQ(ierr);
-    }
+    ierr = PetscFree4(eps->eigr,eps->eigi,eps->errest,eps->perm);CHKERRQ(ierr);
     ierr = PetscMalloc4(requested,&eps->eigr,requested,&eps->eigi,requested,&eps->errest,requested,&eps->perm);CHKERRQ(ierr);
     cnt = 2*newc*sizeof(PetscScalar) + 2*newc*sizeof(PetscReal) + newc*sizeof(PetscInt);
     ierr = PetscLogObjectMemory((PetscObject)eps,cnt);CHKERRQ(ierr);
@@ -502,7 +548,7 @@ PetscErrorCode EPSAllocateSolution(EPS eps,PetscInt extra)
     if (!((PetscObject)(eps->V))->type_name) {
       ierr = BVSetType(eps->V,BVSVEC);CHKERRQ(ierr);
     }
-    ierr = STMatCreateVecs(eps->st,&t,NULL);CHKERRQ(ierr);
+    ierr = STMatCreateVecsEmpty(eps->st,&t,NULL);CHKERRQ(ierr);
     ierr = BVSetSizesFromVec(eps->V,t,requested);CHKERRQ(ierr);
     ierr = VecDestroy(&t);CHKERRQ(ierr);
   } else {
