@@ -1,52 +1,101 @@
 /*
-      EPS routines related to the solution process.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   EPS routines related to the solution process
+*/
 
 #include <slepc/private/epsimpl.h>   /*I "slepceps.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSComputeVectors"
 PetscErrorCode EPSComputeVectors(EPS eps)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   EPSCheckSolved(eps,1);
-  switch (eps->state) {
-  case EPS_STATE_SOLVED:
-    if (eps->ops->computevectors) {
-      ierr = (*eps->ops->computevectors)(eps);CHKERRQ(ierr);
-    }
-    break;
-  default:
-    break;
+  if (eps->state==EPS_STATE_SOLVED && eps->ops->computevectors) {
+    ierr = (*eps->ops->computevectors)(eps);CHKERRQ(ierr);
   }
   eps->state = EPS_STATE_EIGENVECTORS;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve"
+#define SWAP(a,b,t) {t=a;a=b;b=t;}
+
+static PetscErrorCode EPSComputeValues(EPS eps)
+{
+  PetscErrorCode ierr;
+  PetscBool      injective,iscomp,isfilter;
+  PetscInt       i,n,aux,nconv0;
+  Mat            A,B=NULL,G,Z;
+
+  PetscFunctionBegin;
+  switch (eps->categ) {
+    case EPS_CATEGORY_KRYLOV:
+    case EPS_CATEGORY_OTHER:
+      ierr = STIsInjective(eps->st,&injective);CHKERRQ(ierr);
+      if (injective) {
+        /* one-to-one mapping: backtransform eigenvalues */
+        if (eps->ops->backtransform) {
+          ierr = (*eps->ops->backtransform)(eps);CHKERRQ(ierr);
+        } else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, spectral transform should have a backtransform operation");
+      } else {
+        /* compute eigenvalues from Rayleigh quotient */
+        ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+        if (!n) break;
+        ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
+        ierr = BVSetActiveColumns(eps->V,0,n);CHKERRQ(ierr);
+        ierr = DSGetCompact(eps->ds,&iscomp);CHKERRQ(ierr);
+        ierr = DSSetCompact(eps->ds,PETSC_FALSE);CHKERRQ(ierr);
+        ierr = DSGetMat(eps->ds,DS_MAT_A,&G);CHKERRQ(ierr);
+        ierr = BVMatProject(eps->V,A,eps->V,G);CHKERRQ(ierr);
+        ierr = DSRestoreMat(eps->ds,DS_MAT_A,&G);CHKERRQ(ierr);
+        if (B) {
+          ierr = DSGetMat(eps->ds,DS_MAT_B,&G);CHKERRQ(ierr);
+          ierr = BVMatProject(eps->V,B,eps->V,G);CHKERRQ(ierr);
+          ierr = DSRestoreMat(eps->ds,DS_MAT_A,&G);CHKERRQ(ierr);
+        }
+        ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
+        ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+        ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
+        ierr = DSSetCompact(eps->ds,iscomp);CHKERRQ(ierr);
+        if (eps->ishermitian && (!eps->isgeneralized || eps->ispositive)) { /* V = V * Z */
+          ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
+          ierr = DSGetMat(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
+          ierr = BVMultInPlace(eps->V,Z,0,n);CHKERRQ(ierr);
+          ierr = MatDestroy(&Z);CHKERRQ(ierr);
+        }
+        /* in case of STFILTER discard computed eigenvalues that lie outside the wanted interval */
+        ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilter);CHKERRQ(ierr);
+        if (isfilter) {
+          nconv0 = eps->nconv;
+          for (i=0;i<eps->nconv;i++) {
+            if (PetscRealPart(eps->eigr[eps->perm[i]])<eps->inta || PetscRealPart(eps->eigr[eps->perm[i]])>eps->intb) {
+              eps->nconv--;
+              if (i<eps->nconv) { SWAP(eps->perm[i],eps->perm[eps->nconv],aux); i--; }
+            }
+          }
+          if (nconv0>eps->nconv) {
+            ierr = PetscInfo1(eps,"Discarded %D computed eigenvalues lying outside the interval\n",nconv0-eps->nconv);CHKERRQ(ierr);
+          }
+        }
+      }
+      break;
+    case EPS_CATEGORY_PRECOND:
+    case EPS_CATEGORY_CONTOUR:
+      /* eigenvalues already available as an output of the solver */
+      break;
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
    EPSSolve - Solves the eigensystem.
 
@@ -73,12 +122,9 @@ PetscErrorCode EPSComputeVectors(EPS eps)
 PetscErrorCode EPSSolve(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       i,nmat;
-  PetscScalar    dot;
-  PetscBool      iscayley;
+  PetscInt       i;
   STMatMode      matmode;
   Mat            A,B;
-  Vec            w,x;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -110,15 +156,12 @@ PetscErrorCode EPSSolve(EPS eps)
 
   if (!eps->reason) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
 
-  /* Map eigenvalues back to the original problem, necessary in some
-  * spectral transformations */
-  if (eps->ops->backtransform) {
-    ierr = (*eps->ops->backtransform)(eps);CHKERRQ(ierr);
-  }
+  /* Map eigenvalues back to the original problem if appropriate */
+  ierr = EPSComputeValues(eps);CHKERRQ(ierr);
 
 #if !defined(PETSC_USE_COMPLEX)
   /* reorder conjugate eigenvalues (positive imaginary first) */
-  for (i=0; i<eps->nconv-1; i++) {
+  for (i=0;i<eps->nconv-1;i++) {
     if (eps->eigi[i] != 0) {
       if (eps->eigi[i] < 0) {
         eps->eigi[i] = -eps->eigi[i];
@@ -132,25 +175,6 @@ PetscErrorCode EPSSolve(EPS eps)
   }
 #endif
 
-  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
-  if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
-
-  /* In the case of Cayley transform, eigenvectors need to be B-normalized */
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&iscayley);CHKERRQ(ierr);
-  if (iscayley && nmat>1 && eps->ishermitian) {
-    ierr = MatCreateVecs(B,NULL,&w);CHKERRQ(ierr);
-    ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
-    for (i=0;i<eps->nconv;i++) {
-      ierr = BVGetColumn(eps->V,i,&x);CHKERRQ(ierr);
-      ierr = MatMult(B,x,w);CHKERRQ(ierr);
-      ierr = VecDot(w,x,&dot);CHKERRQ(ierr);
-      ierr = VecScale(x,1.0/PetscSqrtScalar(dot));CHKERRQ(ierr);
-      ierr = BVRestoreColumn(eps->V,i,&x);CHKERRQ(ierr);
-    }
-    ierr = VecDestroy(&w);CHKERRQ(ierr);
-  }
-
   /* sort eigenvalues according to eps->which parameter */
   ierr = SlepcSortEigenvalues(eps->sc,eps->nconv,eps->eigr,eps->eigi,eps->perm);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(EPS_Solve,eps,0,0,0);CHKERRQ(ierr);
@@ -161,8 +185,11 @@ PetscErrorCode EPSSolve(EPS eps)
   ierr = EPSErrorViewFromOptions(eps);CHKERRQ(ierr);
   ierr = EPSValuesViewFromOptions(eps);CHKERRQ(ierr);
   ierr = EPSVectorsViewFromOptions(eps);CHKERRQ(ierr);
+  ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
   ierr = MatViewFromOptions(A,(PetscObject)eps,"-eps_view_mat0");CHKERRQ(ierr);
-  if (nmat>1) { ierr = MatViewFromOptions(B,(PetscObject)eps,"-eps_view_mat1");CHKERRQ(ierr); }
+  if (eps->isgeneralized) {
+    ierr = MatViewFromOptions(B,(PetscObject)eps,"-eps_view_mat1");CHKERRQ(ierr);
+  }
 
   /* Remove deflation and initial subspaces */
   if (eps->nds) {
@@ -173,8 +200,6 @@ PetscErrorCode EPSSolve(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetIterationNumber"
 /*@
    EPSGetIterationNumber - Gets the current iteration number. If the
    call to EPSSolve() is complete, then it returns the number of iterations
@@ -188,8 +213,6 @@ PetscErrorCode EPSSolve(EPS eps)
    Output Parameter:
 .  its - number of iterations
 
-   Level: intermediate
-
    Note:
    During the i-th iteration this call returns i-1. If EPSSolve() is
    complete, then parameter "its" contains either the iteration number at
@@ -197,6 +220,8 @@ PetscErrorCode EPSSolve(EPS eps)
    Call EPSGetConvergedReason() to determine if the solver converged or
    failed and why.
 
+   Level: intermediate
+
 .seealso: EPSGetConvergedReason(), EPSSetTolerances()
 @*/
 PetscErrorCode EPSGetIterationNumber(EPS eps,PetscInt *its)
@@ -208,8 +233,6 @@ PetscErrorCode EPSGetIterationNumber(EPS eps,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetConverged"
 /*@
    EPSGetConverged - Gets the number of converged eigenpairs.
 
@@ -238,8 +261,6 @@ PetscErrorCode EPSGetConverged(EPS eps,PetscInt *nconv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetConvergedReason"
 /*@
    EPSGetConvergedReason - Gets the reason why the EPSSolve() iteration was
    stopped.
@@ -253,7 +274,6 @@ PetscErrorCode EPSGetConverged(EPS eps,PetscInt *nconv)
 .  reason - negative value indicates diverged, positive value converged
 
    Notes:
-
    Possible values for reason are
 +  EPS_CONVERGED_TOL - converged up to tolerance
 .  EPS_CONVERGED_USER - converged due to a user-defined condition
@@ -277,8 +297,6 @@ PetscErrorCode EPSGetConvergedReason(EPS eps,EPSConvergedReason *reason)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetInvariantSubspace"
 /*@
    EPSGetInvariantSubspace - Gets an orthonormal basis of the computed invariant
    subspace.
@@ -329,9 +347,7 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetEigenpair"
-/*@
+/*@C
    EPSGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    EPSSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
@@ -384,9 +400,7 @@ PetscErrorCode EPSGetEigenpair(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetEigenvalue"
-/*@
+/*@C
    EPSGetEigenvalue - Gets the i-th eigenvalue as computed by EPSSolve().
 
    Not Collective
@@ -419,7 +433,7 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   EPSCheckSolved(eps,1);
-  if (i<0 || i>=eps->nconv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
   k = eps->perm[i];
 #if defined(PETSC_USE_COMPLEX)
   if (eigr) *eigr = eps->eigr[k];
@@ -431,9 +445,7 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetEigenvector"
-/*@
+/*@C
    EPSGetEigenvector - Gets the i-th right eigenvector as computed by EPSSolve().
 
    Logically Collective on EPS
@@ -504,8 +516,6 @@ PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetErrorEstimate"
 /*@
    EPSGetErrorEstimate - Returns the error estimate associated to the i-th
    computed eigenpair.
@@ -534,13 +544,11 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps,PetscInt i,PetscReal *errest)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(errest,3);
   EPSCheckSolved(eps,1);
-  if (i<0 || i>=eps->nconv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
-  if (errest) *errest = eps->errest[eps->perm[i]];
+  if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  *errest = eps->errest[eps->perm[i]];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSComputeResidualNorm_Private"
 /*
    EPSComputeResidualNorm_Private - Computes the norm of the residual vector
    associated with an eigenpair.
@@ -564,11 +572,11 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscScalar kr,PetscScalar
   PetscFunctionBegin;
   u = z[0]; w = z[2];
   ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
-  if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
+  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
+  if (nmat>1) { ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr); }
 
 #if !defined(PETSC_USE_COMPLEX)
-  v = z[1]; 
+  v = z[1];
   if (ki == 0 || PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
     ierr = MatMult(A,xr,u);CHKERRQ(ierr);                             /* u=A*x */
@@ -602,8 +610,6 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscScalar kr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSComputeError"
 /*@
    EPSComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
@@ -631,7 +637,7 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   PetscErrorCode ierr;
   Mat            A,B;
   Vec            xr,xi,w[3];
-  PetscReal      t;
+  PetscReal      t,vecnorm=1.0;
   PetscScalar    kr,ki;
   PetscBool      flg;
 
@@ -660,31 +666,36 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   ierr = EPSGetEigenpair(eps,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
   ierr = EPSComputeResidualNorm_Private(eps,kr,ki,xr,xi,w,error);CHKERRQ(ierr);
 
+  /* compute 2-norm of eigenvector */
+  if (eps->problem_type==EPS_GHEP) {
+    ierr = VecNorm(xr,NORM_2,&vecnorm);CHKERRQ(ierr);
+  }
+
   /* compute error */
   switch (type) {
     case EPS_ERROR_ABSOLUTE:
       break;
     case EPS_ERROR_RELATIVE:
-      *error /= SlepcAbsEigenvalue(kr,ki);
+      *error /= SlepcAbsEigenvalue(kr,ki)*vecnorm;
       break;
     case EPS_ERROR_BACKWARD:
       /* initialization of matrix norms */
       if (!eps->nrma) {
-        ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
+        ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
         ierr = MatHasOperation(A,MATOP_NORM,&flg);CHKERRQ(ierr);
         if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
         ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
       }
       if (eps->isgeneralized) {
         if (!eps->nrmb) {
-          ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr);
+          ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
           ierr = MatHasOperation(B,MATOP_NORM,&flg);CHKERRQ(ierr);
           if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
           ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
         }
       } else eps->nrmb = 1.0;
       t = SlepcAbsEigenvalue(kr,ki);
-      *error /= eps->nrma+t*eps->nrmb;
+      *error /= (eps->nrma+t*eps->nrmb)*vecnorm;
       break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid error type");
@@ -692,8 +703,6 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSGetStartVector"
 /*
    EPSGetStartVector - Generate a suitable vector to be used as the starting vector
    for the recurrence that builds the right subspace.
