@@ -1,31 +1,19 @@
 /*
-   The EPS routines related to various viewers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   EPS routines related to various viewers
+*/
 
 #include <slepc/private/epsimpl.h>      /*I "slepceps.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSView"
 /*@C
    EPSView - Prints the EPS data structure.
 
@@ -56,9 +44,9 @@
 PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  const char     *type,*extr,*bal;
+  const char     *type=NULL,*extr=NULL,*bal=NULL;
   char           str[50];
-  PetscBool      isascii,ispower,isexternal,istrivial;
+  PetscBool      isascii,isexternal,istrivial;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -81,35 +69,33 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
     }
     if (eps->problem_type) {
       switch (eps->problem_type) {
-        case EPS_HEP:   type = HERM " eigenvalue problem"; break;
-        case EPS_GHEP:  type = "generalized " HERM " eigenvalue problem"; break;
-        case EPS_NHEP:  type = "non-" HERM " eigenvalue problem"; break;
-        case EPS_GNHEP: type = "generalized non-" HERM " eigenvalue problem"; break;
+        case EPS_HEP:    type = HERM " eigenvalue problem"; break;
+        case EPS_GHEP:   type = "generalized " HERM " eigenvalue problem"; break;
+        case EPS_NHEP:   type = "non-" HERM " eigenvalue problem"; break;
+        case EPS_GNHEP:  type = "generalized non-" HERM " eigenvalue problem"; break;
         case EPS_PGNHEP: type = "generalized non-" HERM " eigenvalue problem with " HERM " positive definite B"; break;
-        case EPS_GHIEP: type = "generalized " HERM "-indefinite eigenvalue problem"; break;
-        default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->problem_type");
+        case EPS_GHIEP:  type = "generalized " HERM "-indefinite eigenvalue problem"; break;
       }
     } else type = "not yet set";
     ierr = PetscViewerASCIIPrintf(viewer,"  problem type: %s\n",type);CHKERRQ(ierr);
     if (eps->extraction) {
       switch (eps->extraction) {
-        case EPS_RITZ:             extr = "Rayleigh-Ritz"; break;
-        case EPS_HARMONIC:         extr = "harmonic Ritz"; break;
-        case EPS_HARMONIC_RELATIVE:extr = "relative harmonic Ritz"; break;
-        case EPS_HARMONIC_RIGHT:   extr = "right harmonic Ritz"; break;
-        case EPS_HARMONIC_LARGEST: extr = "largest harmonic Ritz"; break;
-        case EPS_REFINED:          extr = "refined Ritz"; break;
-        case EPS_REFINED_HARMONIC: extr = "refined harmonic Ritz"; break;
-        default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->extraction");
+        case EPS_RITZ:              extr = "Rayleigh-Ritz"; break;
+        case EPS_HARMONIC:          extr = "harmonic Ritz"; break;
+        case EPS_HARMONIC_RELATIVE: extr = "relative harmonic Ritz"; break;
+        case EPS_HARMONIC_RIGHT:    extr = "right harmonic Ritz"; break;
+        case EPS_HARMONIC_LARGEST:  extr = "largest harmonic Ritz"; break;
+        case EPS_REFINED:           extr = "refined Ritz"; break;
+        case EPS_REFINED_HARMONIC:  extr = "refined harmonic Ritz"; break;
       }
       ierr = PetscViewerASCIIPrintf(viewer,"  extraction type: %s\n",extr);CHKERRQ(ierr);
     }
     if (!eps->ishermitian && eps->balance!=EPS_BALANCE_NONE) {
       switch (eps->balance) {
-        case EPS_BALANCE_ONESIDE:   bal = "one-sided Krylov"; break;
-        case EPS_BALANCE_TWOSIDE:   bal = "two-sided Krylov"; break;
-        case EPS_BALANCE_USER:      bal = "user-defined matrix"; break;
-        default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->balance");
+        case EPS_BALANCE_NONE:    break;
+        case EPS_BALANCE_ONESIDE: bal = "one-sided Krylov"; break;
+        case EPS_BALANCE_TWOSIDE: bal = "two-sided Krylov"; break;
+        case EPS_BALANCE_USER:    bal = "user-defined matrix"; break;
       }
       ierr = PetscViewerASCIIPrintf(viewer,"  balancing enabled: %s",bal);CHKERRQ(ierr);
       ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
@@ -165,10 +151,9 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
           ierr = PetscViewerASCIIPrintf(viewer,"all eigenvalues in the region\n");CHKERRQ(ierr);
         }
         break;
-      default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
     }
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
-    if (eps->isgeneralized && eps->ishermitian && eps->purify) {
+    if (eps->purify) {
       ierr = PetscViewerASCIIPrintf(viewer,"  postprocessing eigenvectors with purification\n");CHKERRQ(ierr);
     }
     if (eps->trueres) {
@@ -212,7 +197,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       ierr = (*eps->ops->view)(eps,viewer);CHKERRQ(ierr);
     }
   }
-  ierr = PetscObjectTypeCompareAny((PetscObject)eps,&isexternal,EPSARPACK,EPSBLZPACK,EPSTRLAN,EPSBLOPEX,EPSPRIMME,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)eps,&isexternal,EPSARPACK,EPSBLOPEX,EPSBLZPACK,EPSFEAST,EPSPRIMME,EPSTRLAN,"");CHKERRQ(ierr);
   if (!isexternal) {
     ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
     if (!eps->V) { ierr = EPSGetBV(eps,&eps->V);CHKERRQ(ierr); }
@@ -221,8 +206,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
       if (!istrivial) { ierr = RGView(eps->rg,viewer);CHKERRQ(ierr); }
     }
-    ierr = PetscObjectTypeCompare((PetscObject)eps,EPSPOWER,&ispower);CHKERRQ(ierr);
-    if (!ispower) {
+    if (eps->useds) {
       if (!eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
       ierr = DSView(eps->ds,viewer);CHKERRQ(ierr);
     }
@@ -233,8 +217,6 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReasonView"
 /*@C
    EPSReasonView - Displays the reason an EPS solve converged or diverged.
 
@@ -270,11 +252,9 @@ PetscErrorCode EPSReasonView(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSReasonViewFromOptions"
 /*@
    EPSReasonViewFromOptions - Processes command line options to determine if/how
-   the EPS converged reason is to be viewed. 
+   the EPS converged reason is to be viewed.
 
    Collective on EPS
 
@@ -305,30 +285,28 @@ PetscErrorCode EPSReasonViewFromOptions(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSErrorView_ASCII"
 static PetscErrorCode EPSErrorView_ASCII(EPS eps,EPSErrorType etype,PetscViewer viewer)
 {
-  PetscBool      errok;
-  PetscReal      error,re,im;
-  PetscScalar    kr,ki;
-  PetscInt       i,j,nvals;
+  PetscReal      error;
+  PetscInt       i,j,k,nvals;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (eps->which!=EPS_ALL && eps->nconv<eps->nev) {
+  nvals = (eps->which==EPS_ALL)? eps->nconv: eps->nev;
+  if (eps->which!=EPS_ALL && eps->nconv<nvals) {
     ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D eigenvalues converged\n\n",eps->nev);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-  errok = PETSC_TRUE;
-  nvals = (eps->which==EPS_ALL)? eps->nconv: eps->nev;
+  if (eps->which==EPS_ALL && !nvals) {
+    ierr = PetscViewerASCIIPrintf(viewer," No eigenvalues have been found\n\n");CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
   for (i=0;i<nvals;i++) {
     ierr = EPSComputeError(eps,i,etype,&error);CHKERRQ(ierr);
-    errok = (errok && error<5.0*eps->tol)? PETSC_TRUE: PETSC_FALSE;
-  }
-  if (!errok) {
-    ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",nvals);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
+    if (error>=5.0*eps->tol) {
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",nvals);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
   }
   if (eps->which==EPS_ALL) {
     ierr = PetscViewerASCIIPrintf(viewer," Found %D eigenvalues, all of them computed up to the required tolerance:",nvals);CHKERRQ(ierr);
@@ -338,21 +316,8 @@ static PetscErrorCode EPSErrorView_ASCII(EPS eps,EPSErrorType etype,PetscViewer
   for (i=0;i<=(nvals-1)/8;i++) {
     ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
     for (j=0;j<PetscMin(8,nvals-8*i);j++) {
-      ierr = EPSGetEigenpair(eps,8*i+j,&kr,&ki,NULL,NULL);CHKERRQ(ierr);
-#if defined(PETSC_USE_COMPLEX)
-      re = PetscRealPart(kr);
-      im = PetscImaginaryPart(kr);
-#else
-      re = kr;
-      im = ki;
-#endif
-      if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
-      if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
-      if (im!=0.0) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%.5f%+.5fi",(double)re,(double)im);CHKERRQ(ierr);
-      } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"%.5f",(double)re);CHKERRQ(ierr);
-      }
+      k = eps->perm[8*i+j];
+      ierr = SlepcPrintEigenvalueASCII(eps->eigr[k],eps->eigi[k]);CHKERRQ(ierr);
       if (8*i+j+1<nvals) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
     }
   }
@@ -360,8 +325,6 @@ static PetscErrorCode EPSErrorView_ASCII(EPS eps,EPSErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSErrorView_DETAIL"
 static PetscErrorCode EPSErrorView_DETAIL(EPS eps,EPSErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -405,8 +368,6 @@ static PetscErrorCode EPSErrorView_DETAIL(EPS eps,EPSErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSErrorView_MATLAB"
 static PetscErrorCode EPSErrorView_MATLAB(EPS eps,EPSErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -419,14 +380,12 @@ static PetscErrorCode EPSErrorView_MATLAB(EPS eps,EPSErrorType etype,PetscViewer
   ierr = PetscViewerASCIIPrintf(viewer,"Error_%s = [\n",name);CHKERRQ(ierr);
   for (i=0;i<eps->nconv;i++) {
     ierr = EPSComputeError(eps,i,etype,&error);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",error);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",(double)error);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"];\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSErrorView"
 /*@C
    EPSErrorView - Displays the errors associated with the computed solution
    (as well as the eigenvalues).
@@ -486,11 +445,9 @@ PetscErrorCode EPSErrorView(EPS eps,EPSErrorType etype,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSErrorViewFromOptions"
 /*@
    EPSErrorViewFromOptions - Processes command line options to determine if/how
-   the errors of the computed solution are to be viewed. 
+   the errors of the computed solution are to be viewed.
 
    Collective on EPS
 
@@ -535,8 +492,6 @@ PetscErrorCode EPSErrorViewFromOptions(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSValuesView_DRAW"
 static PetscErrorCode EPSValuesView_DRAW(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -547,8 +502,8 @@ static PetscErrorCode EPSValuesView_DRAW(EPS eps,PetscViewer viewer)
 
   PetscFunctionBegin;
   if (!eps->nconv) PetscFunctionReturn(0);
-  ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+  ierr = PetscDrawSetTitle(draw,"Computed Eigenvalues");CHKERRQ(ierr);
   ierr = PetscDrawSPCreate(draw,1,&drawsp);CHKERRQ(ierr);
   for (i=0;i<eps->nconv;i++) {
     k = eps->perm[i];
@@ -564,15 +519,11 @@ static PetscErrorCode EPSValuesView_DRAW(EPS eps,PetscViewer viewer)
   ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscDrawSPSave(drawsp);CHKERRQ(ierr);
   ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSValuesView_ASCII"
 static PetscErrorCode EPSValuesView_ASCII(EPS eps,PetscViewer viewer)
 {
-  PetscReal      re,im;
   PetscInt       i,k;
   PetscErrorCode ierr;
 
@@ -580,27 +531,14 @@ static PetscErrorCode EPSValuesView_ASCII(EPS eps,PetscViewer viewer)
   ierr = PetscViewerASCIIPrintf(viewer,"Eigenvalues = \n");CHKERRQ(ierr);
   for (i=0;i<eps->nconv;i++) {
     k = eps->perm[i];
-#if defined(PETSC_USE_COMPLEX)
-    re = PetscRealPart(eps->eigr[k]);
-    im = PetscImaginaryPart(eps->eigr[k]);
-#else
-    re = eps->eigr[k];
-    im = eps->eigi[k];
-#endif
-    if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
-    if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
-    if (im!=0.0) {
-      ierr = PetscViewerASCIIPrintf(viewer,"   %.5f%+.5fi\n",(double)re,(double)im);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"   %.5f\n",(double)re);CHKERRQ(ierr);
-    }
+    ierr = PetscViewerASCIIPrintf(viewer,"   ");CHKERRQ(ierr);
+    ierr = SlepcPrintEigenvalueASCII(eps->eigr[k],eps->eigi[k]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSValuesView_MATLAB"
 static PetscErrorCode EPSValuesView_MATLAB(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -630,8 +568,6 @@ static PetscErrorCode EPSValuesView_MATLAB(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSValuesView"
 /*@C
    EPSValuesView - Displays the computed eigenvalues in a viewer.
 
@@ -682,11 +618,9 @@ PetscErrorCode EPSValuesView(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSValuesViewFromOptions"
 /*@
    EPSValuesViewFromOptions - Processes command line options to determine if/how
-   the computed eigenvalues are to be viewed. 
+   the computed eigenvalues are to be viewed.
 
    Collective on EPS
 
@@ -717,8 +651,6 @@ PetscErrorCode EPSValuesViewFromOptions(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSVectorsView"
 /*@C
    EPSVectorsView - Outputs computed eigenvectors to a viewer.
 
@@ -770,11 +702,9 @@ PetscErrorCode EPSVectorsView(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSVectorsViewFromOptions"
 /*@
    EPSVectorsViewFromOptions - Processes command line options to determine if/how
-   the computed eigenvectors are to be viewed. 
+   the computed eigenvectors are to be viewed.
 
    Collective on EPS
 
