@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc matrix function solver: "krylov"
 
    Method: Arnoldi with Eiermann-Ernst restart
@@ -14,32 +22,11 @@
        [1] M. Eiermann and O. Ernst, "A restarted Krylov subspace method
            for the evaluation of matrix functions", SIAM J. Numer. Anal.
            44(6):2481-2504, 2006.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/mfnimpl.h>
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetUp_Krylov"
 PetscErrorCode MFNSetUp_Krylov(MFN mfn)
 {
   PetscErrorCode ierr;
@@ -53,13 +40,12 @@ PetscErrorCode MFNSetUp_Krylov(MFN mfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNBasicArnoldi"
 PetscErrorCode MFNBasicArnoldi(MFN mfn,PetscScalar *H,PetscInt ldh,PetscInt k,PetscInt *M,PetscReal *beta,PetscBool *breakdown)
 {
   PetscErrorCode ierr;
-  PetscInt       j,m = *M;
-  Vec            vj,vj1;
+  PetscScalar    *a;
+  PetscInt       j,nc,n,m = *M;
+  Vec            vj,vj1,buf;
 
   PetscFunctionBegin;
   ierr = BVSetActiveColumns(mfn->V,0,m);CHKERRQ(ierr);
@@ -69,20 +55,24 @@ PetscErrorCode MFNBasicArnoldi(MFN mfn,PetscScalar *H,PetscInt ldh,PetscInt k,Pe
     ierr = MatMult(mfn->A,vj,vj1);CHKERRQ(ierr);
     ierr = BVRestoreColumn(mfn->V,j,&vj);CHKERRQ(ierr);
     ierr = BVRestoreColumn(mfn->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = BVOrthogonalizeColumn(mfn->V,j+1,H+ldh*j,beta,breakdown);CHKERRQ(ierr);
-    H[j+1+ldh*j] = *beta;
+    ierr = BVOrthonormalizeColumn(mfn->V,j+1,PETSC_FALSE,beta,breakdown);CHKERRQ(ierr);
     if (*breakdown) {
       *M = j+1;
       break;
-    } else {
-      ierr = BVScaleColumn(mfn->V,j+1,1.0/(*beta));CHKERRQ(ierr);
     }
   }
+  /* extract Hessenberg matrix from the BV object */
+  ierr = BVGetNumConstraints(mfn->V,&nc);CHKERRQ(ierr);
+  ierr = BVGetSizes(mfn->V,NULL,NULL,&n);CHKERRQ(ierr);
+  ierr = BVGetBufferVec(mfn->V,&buf);CHKERRQ(ierr);
+  ierr = VecGetArray(buf,&a);CHKERRQ(ierr);
+  for (j=k;j<*M;j++) {
+    ierr = PetscMemcpy(H+j*ldh,a+nc+(j+1)*(nc+n),(j+2)*sizeof(PetscScalar));CHKERRQ(ierr);
+  }
+  ierr = VecRestoreArray(buf,&a);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSolve_Krylov"
 PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
 {
   PetscErrorCode ierr;
@@ -156,8 +146,8 @@ PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
     ierr = VecRestoreArray(F,&farray);CHKERRQ(ierr);
 
     /* check convergence */
+    if (mfn->its >= mfn->max_it) mfn->reason = MFN_DIVERGED_ITS;
     if (mfn->its>1) {
-      if (mfn->its >= mfn->max_it) mfn->reason = MFN_DIVERGED_ITS;
       if (m<mfn->ncv || breakdown || beta==0.0 || nrm<mfn->tol) mfn->reason = MFN_CONVERGED_TOL;
     }
 
@@ -176,8 +166,6 @@ PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNCreate_Krylov"
 PETSC_EXTERN PetscErrorCode MFNCreate_Krylov(MFN mfn)
 {
   PetscFunctionBegin;
