@@ -1,32 +1,20 @@
 /*
-      MFN routines related to options that can be set via the command-line
-      or procedurally.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   MFN routines related to options that can be set via the command-line
+   or procedurally
+*/
 
 #include <slepc/private/mfnimpl.h>   /*I "slepcmfn.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNMonitorSetFromOptions"
 /*@C
    MFNMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
@@ -38,12 +26,11 @@
 .  name     - the monitor option name
 .  help     - message indicating what monitoring is done
 .  manual   - manual page for the monitor
-.  monitor  - the monitor function, whose context is a PetscViewerAndFormat
--  trackall - whether this monitor tracks all eigenvalues or not
+-  monitor  - the monitor function, whose context is a PetscViewerAndFormat
 
    Level: developer
 
-.seealso: MFNMonitorSet(), MFNSetTrackAll(), MFNConvMonitorSetFromOptions()
+.seealso: MFNMonitorSet()
 @*/
 PetscErrorCode MFNMonitorSetFromOptions(MFN mfn,const char name[],const char help[],const char manual[],PetscErrorCode (*monitor)(MFN,PetscInt,PetscReal,PetscViewerAndFormat*))
 {
@@ -63,8 +50,6 @@ PetscErrorCode MFNMonitorSetFromOptions(MFN mfn,const char name[],const char hel
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetFromOptions"
 /*@
    MFNSetFromOptions - Sets MFN options from the options database.
    This routine must be called before MFNSetUp() if the user is to be
@@ -96,11 +81,7 @@ PetscErrorCode MFNSetFromOptions(MFN mfn)
     ierr = PetscOptionsFList("-mfn_type","Matrix Function method","MFNSetType",MFNList,(char*)(((PetscObject)mfn)->type_name?((PetscObject)mfn)->type_name:MFNKRYLOV),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = MFNSetType(mfn,type);CHKERRQ(ierr);
-    }
-    /*
-      Set the type if it was never set.
-    */
-    if (!((PetscObject)mfn)->type_name) {
+    } else if (!((PetscObject)mfn)->type_name) {
       ierr = MFNSetType(mfn,MFNKRYLOV);CHKERRQ(ierr);
     }
 
@@ -109,14 +90,10 @@ PetscErrorCode MFNSetFromOptions(MFN mfn)
     if (!flg1) i = PETSC_DEFAULT;
     r = mfn->tol;
     ierr = PetscOptionsReal("-mfn_tol","Tolerance","MFNSetTolerances",mfn->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:mfn->tol,&r,&flg2);CHKERRQ(ierr);
-    if (flg1 || flg2) {
-      ierr = MFNSetTolerances(mfn,r,i);CHKERRQ(ierr);
-    }
+    if (flg1 || flg2) { ierr = MFNSetTolerances(mfn,r,i);CHKERRQ(ierr); }
 
     ierr = PetscOptionsInt("-mfn_ncv","Number of basis vectors","MFNSetDimensions",mfn->ncv,&i,&flg);CHKERRQ(ierr);
-    if (flg) {
-      ierr = MFNSetDimensions(mfn,i);CHKERRQ(ierr);
-    }
+    if (flg) { ierr = MFNSetDimensions(mfn,i);CHKERRQ(ierr); }
 
     ierr = PetscOptionsBool("-mfn_error_if_not_converged","Generate error if solver does not converge","MFNSetErrorIfNotConverged",mfn->errorifnotconverged,&mfn->errorifnotconverged,NULL);CHKERRQ(ierr);
 
@@ -140,8 +117,8 @@ PetscErrorCode MFNSetFromOptions(MFN mfn)
       ierr = MFNMonitorLGCreate(PetscObjectComm((PetscObject)mfn),NULL,"Error estimate",PETSC_DECIDE,PETSC_DECIDE,300,300,&lg);CHKERRQ(ierr);
       ierr = MFNMonitorSet(mfn,MFNMonitorLG,lg,(PetscErrorCode (*)(void**))PetscDrawLGDestroy);CHKERRQ(ierr);
     }
-  /* -----------------------------------------------------------------------*/
 
+    /* -----------------------------------------------------------------------*/
     ierr = PetscOptionsName("-mfn_view","Print detailed information on solver used","MFNView",NULL);CHKERRQ(ierr);
 
     if (mfn->ops->setfromoptions) {
@@ -157,9 +134,7 @@ PetscErrorCode MFNSetFromOptions(MFN mfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetTolerances"
-/*@
+/*@C
    MFNGetTolerances - Gets the tolerance and maximum iteration count used
    by the MFN convergence tests.
 
@@ -188,8 +163,6 @@ PetscErrorCode MFNGetTolerances(MFN mfn,PetscReal *tol,PetscInt *maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetTolerances"
 /*@
    MFNSetTolerances - Sets the tolerance and maximum iteration count used
    by the MFN convergence tests.
@@ -235,8 +208,6 @@ PetscErrorCode MFNSetTolerances(MFN mfn,PetscReal tol,PetscInt maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetDimensions"
 /*@
    MFNGetDimensions - Gets the dimension of the subspace used by the solver.
 
@@ -261,8 +232,6 @@ PetscErrorCode MFNGetDimensions(MFN mfn,PetscInt *ncv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetDimensions"
 /*@
    MFNSetDimensions - Sets the dimension of the subspace to be used by the solver.
 
@@ -298,8 +267,6 @@ PetscErrorCode MFNSetDimensions(MFN mfn,PetscInt ncv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetErrorIfNotConverged"
 /*@
    MFNSetErrorIfNotConverged - Causes MFNSolve() to generate an error if the
    solver has not converged.
@@ -330,8 +297,6 @@ PetscErrorCode MFNSetErrorIfNotConverged(MFN mfn,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetErrorIfNotConverged"
 /*@
    MFNGetErrorIfNotConverged - Return a flag indicating whether MFNSolve() will
    generate an error if the solver does not converge.
@@ -357,8 +322,6 @@ PetscErrorCode MFNGetErrorIfNotConverged(MFN mfn,PetscBool *flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetOptionsPrefix"
 /*@C
    MFNSetOptionsPrefix - Sets the prefix used for searching for all
    MFN options in the database.
@@ -399,8 +362,6 @@ PetscErrorCode MFNSetOptionsPrefix(MFN mfn,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNAppendOptionsPrefix"
 /*@C
    MFNAppendOptionsPrefix - Appends to the prefix used for searching for all
    MFN options in the database.
@@ -426,15 +387,13 @@ PetscErrorCode MFNAppendOptionsPrefix(MFN mfn,const char *prefix)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
   if (!mfn->V) { ierr = MFNGetBV(mfn,&mfn->V);CHKERRQ(ierr); }
-  ierr = BVSetOptionsPrefix(mfn->V,prefix);CHKERRQ(ierr);
+  ierr = BVAppendOptionsPrefix(mfn->V,prefix);CHKERRQ(ierr);
   if (!mfn->fn) { ierr = MFNGetFN(mfn,&mfn->fn);CHKERRQ(ierr); }
-  ierr = FNSetOptionsPrefix(mfn->fn,prefix);CHKERRQ(ierr);
+  ierr = FNAppendOptionsPrefix(mfn->fn,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)mfn,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetOptionsPrefix"
 /*@C
    MFNGetOptionsPrefix - Gets the prefix used for searching for all
    MFN options in the database.
