@@ -1,30 +1,18 @@
 /*
-      MFN routines related to problem setup.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   MFN routines related to problem setup
+*/
 
 #include <slepc/private/mfnimpl.h>       /*I "slepcmfn.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetUp"
 /*@
    MFNSetUp - Sets up all the internal data structures necessary for the
    execution of the matrix function solver.
@@ -82,8 +70,6 @@ PetscErrorCode MFNSetUp(MFN mfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetOperator"
 /*@
    MFNSetOperator - Sets the matrix for which the matrix function is to be computed.
 
@@ -114,14 +100,13 @@ PetscErrorCode MFNSetOperator(MFN mfn,Mat A)
   ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
   if (m!=n) SETERRQ(PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_WRONG,"A is a non-square matrix");
   if (mfn->setupcalled) { ierr = MFNReset(mfn);CHKERRQ(ierr); }
+  else { ierr = MatDestroy(&mfn->A);CHKERRQ(ierr); }
   ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-  ierr = MatDestroy(&mfn->A);CHKERRQ(ierr);
   mfn->A = A;
+  mfn->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetOperator"
 /*@
    MFNGetOperator - Gets the matrix associated with the MFN object.
 
@@ -146,8 +131,6 @@ PetscErrorCode MFNGetOperator(MFN mfn,Mat *A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNAllocateSolution"
 /*@
    MFNAllocateSolution - Allocate memory storage for common variables such
    as the basis vectors.
@@ -155,7 +138,7 @@ PetscErrorCode MFNGetOperator(MFN mfn,Mat *A)
    Collective on MFN
 
    Input Parameters:
-+  mfn   - eigensolver context
++  mfn   - matrix function context
 -  extra - number of additional positions, used for methods that require a
            working basis slightly larger than ncv
 
@@ -183,7 +166,7 @@ PetscErrorCode MFNAllocateSolution(MFN mfn,PetscInt extra)
     if (!((PetscObject)(mfn->V))->type_name) {
       ierr = BVSetType(mfn->V,BVSVEC);CHKERRQ(ierr);
     }
-    ierr = MatCreateVecs(mfn->A,&t,NULL);CHKERRQ(ierr);
+    ierr = MatCreateVecsEmpty(mfn->A,&t,NULL);CHKERRQ(ierr);
     ierr = BVSetSizesFromVec(mfn->V,t,requested);CHKERRQ(ierr);
     ierr = VecDestroy(&t);CHKERRQ(ierr);
   } else {
