@@ -1,21 +1,10 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
@@ -47,6 +36,7 @@ PetscErrorCode MatGetDiagonal_Fun(Mat,Vec);
 PetscErrorCode MatDestroy_Fun(Mat);
 PetscErrorCode MatDuplicate_Fun(Mat,MatDuplicateOption,Mat*);
 PetscErrorCode MatMult_Jac(Mat,Vec,Vec);
+PetscErrorCode MatGetDiagonal_Jac(Mat,Vec);
 PetscErrorCode MatDestroy_Jac(Mat);
 
 typedef struct {
@@ -62,15 +52,12 @@ typedef struct {
   PetscReal   h;       /* mesh spacing */
 } ApplicationCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   NEP            nep;             /* nonlinear eigensolver context */
   Mat            F,J;             /* Function and Jacobian matrices */
   ApplicationCtx ctx;             /* user-defined context */
   MatCtx         *ctxF,*ctxJ;     /* contexts for shell matrices */
-  NEPType        type;
   PetscInt       n=128,nev;
   KSP            ksp;
   PC             pc;
@@ -78,9 +65,9 @@ int main(int argc,char **argv)
   PetscBool      terse;
   PetscErrorCode ierr;
 
-  SlepcInitialize(&argc,&argv,(char*)0,help);
+  ierr = SlepcInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
-  if (size != 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This is a uniprocessor example only!");
+  if (size != 1) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"This is a uniprocessor example only!");
   ierr = PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"\n1-D Nonlinear Eigenproblem, n=%D\n\n",n);CHKERRQ(ierr);
   ctx.h = 1.0/(PetscReal)n;
@@ -122,6 +109,7 @@ int main(int argc,char **argv)
 
   ierr = MatCreateShell(PETSC_COMM_WORLD,n,n,n,n,(void*)ctxJ,&J);CHKERRQ(ierr);
   ierr = MatShellSetOperation(J,MATOP_MULT,(void(*)())MatMult_Jac);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(J,MATOP_GET_DIAGONAL,(void(*)())MatGetDiagonal_Jac);CHKERRQ(ierr);
   ierr = MatShellSetOperation(J,MATOP_DESTROY,(void(*)())MatDestroy_Jac);CHKERRQ(ierr);
 
   /*
@@ -151,8 +139,6 @@ int main(int argc,char **argv)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
   ierr = NEPSolve(nep);CHKERRQ(ierr);
-  ierr = NEPGetType(nep,&type);CHKERRQ(ierr);
-  ierr = PetscPrintf(PETSC_COMM_WORLD," Solution method: %s\n",type);CHKERRQ(ierr);
   ierr = NEPGetDimensions(nep,&nev,NULL,NULL);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD," Number of requested eigenvalues: %D\n",nev);CHKERRQ(ierr);
 
@@ -178,8 +164,6 @@ int main(int argc,char **argv)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialGuess"
 /*
    FormInitialGuess - Computes initial guess.
 
@@ -196,8 +180,6 @@ PetscErrorCode FormInitialGuess(Vec x)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormFunction"
 /*
    FormFunction - Computes Function matrix  T(lambda)
 
@@ -222,8 +204,6 @@ PetscErrorCode FormFunction(NEP nep,PetscScalar lambda,Mat fun,Mat B,void *ctx)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobian"
 /*
    FormJacobian - Computes Jacobian matrix  T'(lambda)
 
@@ -248,8 +228,6 @@ PetscErrorCode FormJacobian(NEP nep,PetscScalar lambda,Mat jac,void *ctx)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Fun"
 PetscErrorCode MatMult_Fun(Mat A,Vec x,Vec y)
 {
   PetscErrorCode    ierr;
@@ -281,15 +259,13 @@ PetscErrorCode MatMult_Fun(Mat A,Vec x,Vec y)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "MatGetDiagonal_Fun"
 PetscErrorCode MatGetDiagonal_Fun(Mat A,Vec diag)
 {
-  PetscErrorCode    ierr;
-  MatCtx            *ctx;
-  PetscInt          n;
-  PetscScalar       *pd,c,d;
-  PetscReal         h;
+  PetscErrorCode ierr;
+  MatCtx         *ctx;
+  PetscInt       n;
+  PetscScalar    *pd,c,d;
+  PetscReal      h;
 
   PetscFunctionBeginUser;
   ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
@@ -305,8 +281,6 @@ PetscErrorCode MatGetDiagonal_Fun(Mat A,Vec diag)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_Fun"
 PetscErrorCode MatDestroy_Fun(Mat A)
 {
   MatCtx         *ctx;
@@ -319,8 +293,6 @@ PetscErrorCode MatDestroy_Fun(Mat A)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "MatDuplicate_Fun"
 PetscErrorCode MatDuplicate_Fun(Mat A,MatDuplicateOption op,Mat *B)
 {
   MatCtx         *actx,*bctx;
@@ -347,8 +319,6 @@ PetscErrorCode MatDuplicate_Fun(Mat A,MatDuplicateOption op,Mat *B)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Jac"
 PetscErrorCode MatMult_Jac(Mat A,Vec x,Vec y)
 {
   PetscErrorCode    ierr;
@@ -379,8 +349,27 @@ PetscErrorCode MatMult_Jac(Mat A,Vec x,Vec y)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_Jac"
+PetscErrorCode MatGetDiagonal_Jac(Mat A,Vec diag)
+{
+  PetscErrorCode ierr;
+  MatCtx         *ctx;
+  PetscInt       n;
+  PetscScalar    *pd,c;
+  PetscReal      h;
+
+  PetscFunctionBeginUser;
+  ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
+  ierr = VecGetSize(diag,&n);CHKERRQ(ierr);
+  h = ctx->h;
+  c = ctx->kappa/(ctx->lambda-ctx->kappa);
+  ierr = VecSet(diag,-2.0*h/3.0);CHKERRQ(ierr);
+  ierr = VecGetArray(diag,&pd);CHKERRQ(ierr);
+  pd[n-1] = -h/3.0-c*c;
+  ierr = VecRestoreArray(diag,&pd);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/* ------------------------------------------------------------------- */
 PetscErrorCode MatDestroy_Jac(Mat A)
 {
   MatCtx         *ctx;
