@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "ciss"
 
    Method: Contour Integral Spectral Slicing
@@ -20,25 +28,6 @@
        [2] T. Sakurai and H. Tadano, "CIRR: a Rayleigh-Ritz type method with
            contour integral for generalized eigenvalue problems", Hokkaido
            Math. J. 36:745-757, 2007.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/nepimpl.h>         /*I "slepcnep.h" I*/
@@ -67,15 +56,12 @@ typedef struct {
   BV           S;
   BV           Y;
   KSP          *ksp;
-  Mat          *kspMat;
   PetscBool    useconj;
   PetscReal    est_eig;
   PetscSubcomm subcomm;
   PetscBool    usest;
 } NEP_CISS;
 
-#undef __FUNCT__
-#define __FUNCT__ "SetSolverComm"
 static PetscErrorCode SetSolverComm(NEP nep)
 {
   PetscErrorCode ierr;
@@ -84,21 +70,18 @@ static PetscErrorCode SetSolverComm(NEP nep)
 
   PetscFunctionBegin;
   if (ctx->useconj) N = N/2;
-  if (!ctx->subcomm) {
-    ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&ctx->subcomm);CHKERRQ(ierr);
-    ierr = PetscSubcommSetNumber(ctx->subcomm,ctx->num_subcomm);CHKERRQ(ierr);CHKERRQ(ierr);
-    ierr = PetscSubcommSetType(ctx->subcomm,PETSC_SUBCOMM_INTERLACED);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm));CHKERRQ(ierr);
-    ierr = PetscSubcommSetFromOptions(ctx->subcomm);CHKERRQ(ierr);
-  }
+  ierr = PetscSubcommDestroy(&ctx->subcomm);CHKERRQ(ierr);
+  ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&ctx->subcomm);CHKERRQ(ierr);
+  ierr = PetscSubcommSetNumber(ctx->subcomm,ctx->num_subcomm);CHKERRQ(ierr);CHKERRQ(ierr);
+  ierr = PetscSubcommSetType(ctx->subcomm,PETSC_SUBCOMM_INTERLACED);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm));CHKERRQ(ierr);
+  ierr = PetscSubcommSetFromOptions(ctx->subcomm);CHKERRQ(ierr);
   ctx->subcomm_id = ctx->subcomm->color;
   ctx->num_solve_point = N / ctx->num_subcomm;
   if ((N%ctx->num_subcomm) > ctx->subcomm_id) ctx->num_solve_point+=1;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SetPathParameter"
 static PetscErrorCode SetPathParameter(NEP nep)
 {
   PetscErrorCode ierr;
@@ -123,8 +106,6 @@ static PetscErrorCode SetPathParameter(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CISSVecSetRandom"
 static PetscErrorCode CISSVecSetRandom(BV V,PetscInt i0,PetscInt i1)
 {
   PetscErrorCode ierr;
@@ -149,14 +130,12 @@ static PetscErrorCode CISSVecSetRandom(BV V,PetscInt i0,PetscInt i1)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SolveLinearSystem"
 static PetscErrorCode SolveLinearSystem(NEP nep,Mat T,Mat dT,BV V,PetscInt L_start,PetscInt L_end,PetscBool initksp)
 {
   PetscErrorCode ierr;
   NEP_CISS       *ctx = (NEP_CISS*)nep->data;
   PetscInt       i,j,p_id;
-  Mat            Fz;
+  Mat            Fz,kspMat;
   PC             pc;
   Vec            Bvj,vj,yj;
   KSP            ksp;
@@ -173,8 +152,9 @@ static PetscErrorCode SolveLinearSystem(NEP nep,Mat T,Mat dT,BV V,PetscInt L_sta
     ierr = NEPComputeFunction(nep,ctx->omega[p_id],T,T);CHKERRQ(ierr);
     ierr = NEPComputeJacobian(nep,ctx->omega[p_id],dT);CHKERRQ(ierr);
     if (!ctx->usest && initksp == PETSC_TRUE) {
-      ierr = MatDuplicate(T,MAT_COPY_VALUES,&ctx->kspMat[i]);CHKERRQ(ierr);
-      ierr = KSPSetOperators(ctx->ksp[i],ctx->kspMat[i],ctx->kspMat[i]);CHKERRQ(ierr);
+      ierr = MatDuplicate(T,MAT_COPY_VALUES,&kspMat);CHKERRQ(ierr);
+      ierr = KSPSetOperators(ctx->ksp[i],kspMat,kspMat);CHKERRQ(ierr);
+      ierr = MatDestroy(&kspMat);CHKERRQ(ierr);
       ierr = KSPSetType(ctx->ksp[i],KSPPREONLY);CHKERRQ(ierr);
       ierr = KSPGetPC(ctx->ksp[i],&pc);CHKERRQ(ierr);
       ierr = PCSetType(pc,PCLU);CHKERRQ(ierr);
@@ -185,6 +165,7 @@ static PetscErrorCode SolveLinearSystem(NEP nep,Mat T,Mat dT,BV V,PetscInt L_sta
       ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
       ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
       ierr = PCSetType(pc,PCLU);CHKERRQ(ierr);
+      ierr = KSPSetTolerances(ksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
       ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
     }
     for (j=L_start;j<L_end;j++) {
@@ -199,7 +180,7 @@ static PetscErrorCode SolveLinearSystem(NEP nep,Mat T,Mat dT,BV V,PetscInt L_sta
       ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
       ierr = BVRestoreColumn(ctx->Y,i*ctx->L_max+j,&yj);CHKERRQ(ierr);
     }
-    if (ctx->usest && i<ctx->num_solve_point-1) { ierr =  KSPReset(ksp);CHKERRQ(ierr); }
+    if (ctx->usest && i<ctx->num_solve_point-1) { ierr = KSPReset(ksp);CHKERRQ(ierr); }
   }
   if (ctx->usest) {
     ierr = MatDestroy(&Fz);CHKERRQ(ierr);
@@ -209,8 +190,6 @@ static PetscErrorCode SolveLinearSystem(NEP nep,Mat T,Mat dT,BV V,PetscInt L_sta
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EstimateNumberEigs"
 static PetscErrorCode EstimateNumberEigs(NEP nep,PetscInt *L_add)
 {
   PetscErrorCode ierr;
@@ -252,8 +231,6 @@ static PetscErrorCode EstimateNumberEigs(NEP nep,PetscInt *L_add)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CalcMu"
 static PetscErrorCode CalcMu(NEP nep, PetscScalar *Mu)
 {
   PetscErrorCode ierr;
@@ -291,7 +268,7 @@ static PetscErrorCode CalcMu(NEP nep, PetscScalar *Mu)
         }
       }
     }
-    for (i=0;i<ctx->num_solve_point;i++) 
+    for (i=0;i<ctx->num_solve_point;i++)
       ppk[i] *= ctx->pp[i*ctx->subcomm->n + ctx->subcomm_id];
   }
   for (i=0;i<2*ctx->M*ctx->L*ctx->L;i++) temp2[i] /= sub_size;
@@ -302,8 +279,6 @@ static PetscErrorCode CalcMu(NEP nep, PetscScalar *Mu)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BlockHankel"
 static PetscErrorCode BlockHankel(NEP nep,PetscScalar *Mu,PetscInt s,PetscScalar *H)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -311,14 +286,12 @@ static PetscErrorCode BlockHankel(NEP nep,PetscScalar *Mu,PetscInt s,PetscScalar
 
   PetscFunctionBegin;
   for (k=0;k<L*M;k++)
-    for (j=0;j<M;j++) 
-      for (i=0;i<L;i++) 
+    for (j=0;j<M;j++)
+      for (i=0;i<L;i++)
         H[j*L+i+k*L*M] = Mu[i+k*L+(j+s)*L*L];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVD_H0"
 static PetscErrorCode SVD_H0(NEP nep,PetscScalar *S,PetscInt *K)
 {
 #if defined(PETSC_MISSING_LAPACK_GESVD)
@@ -347,7 +320,7 @@ static PetscErrorCode SVD_H0(NEP nep,PetscScalar *S,PetscInt *K)
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("N","N",&m,&n,S,&lda,ctx->sigma,NULL,&ldu,NULL,&ldvt,work,&lwork,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
+  SlepcCheckLapackInfo("gesvd",info);
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
   (*K) = 0;
   for (i=0;i<ml;i++) {
@@ -361,8 +334,6 @@ static PetscErrorCode SVD_H0(NEP nep,PetscScalar *S,PetscInt *K)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ConstructS"
 static PetscErrorCode ConstructS(NEP nep)
 {
   PetscErrorCode ierr;
@@ -405,8 +376,6 @@ static PetscErrorCode ConstructS(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "isGhost"
 static PetscErrorCode isGhost(NEP nep,PetscInt ld,PetscInt nv,PetscBool *fl)
 {
   PetscErrorCode ierr;
@@ -442,8 +411,6 @@ static PetscErrorCode isGhost(NEP nep,PetscInt ld,PetscInt nv,PetscBool *fl)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetUp_CISS"
 PetscErrorCode NEPSetUp_CISS(NEP nep)
 {
   PetscErrorCode ierr;
@@ -483,18 +450,27 @@ PetscErrorCode NEPSetUp_CISS(NEP nep)
   ierr = SetSolverComm(nep);CHKERRQ(ierr);
 
   ierr = NEPAllocateSolution(nep,0);CHKERRQ(ierr);
+  if (ctx->weight) { ierr = PetscFree4(ctx->weight,ctx->omega,ctx->pp,ctx->sigma);CHKERRQ(ierr); }
   ierr = PetscMalloc4(ctx->N,&ctx->weight,ctx->N,&ctx->omega,ctx->N,&ctx->pp,ctx->L_max*ctx->M,&ctx->sigma);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,3*ctx->N*sizeof(PetscScalar)+ctx->L_max*ctx->N*sizeof(PetscReal));CHKERRQ(ierr);
 
   /* allocate basis vectors */
+  ierr = BVDestroy(&ctx->S);CHKERRQ(ierr);
   ierr = BVDuplicateResize(nep->V,ctx->L_max*ctx->M,&ctx->S);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->S);CHKERRQ(ierr);
+  ierr = BVDestroy(&ctx->V);CHKERRQ(ierr);
   ierr = BVDuplicateResize(nep->V,ctx->L_max,&ctx->V);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->V);CHKERRQ(ierr);
 
   if (!ctx->usest) {
-    ierr = PetscMalloc2(ctx->num_solve_point,&ctx->ksp,ctx->num_solve_point,&ctx->kspMat);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)nep,ctx->num_solve_point*sizeof(KSP)+ctx->num_solve_point*sizeof(Mat));CHKERRQ(ierr);
+    if (ctx->ksp) {
+      for (i=0;i<ctx->num_solve_point;i++) {
+        ierr = KSPDestroy(&ctx->ksp[i]);CHKERRQ(ierr);
+      }
+      ierr = PetscFree(ctx->ksp);CHKERRQ(ierr);
+    }
+    ierr = PetscMalloc1(ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)nep,ctx->num_solve_point*sizeof(KSP));CHKERRQ(ierr);
     for (i=0;i<ctx->num_solve_point;i++) {
       ierr = KSPCreate(PetscSubcommChild(ctx->subcomm),&ctx->ksp[i]);CHKERRQ(ierr);
       ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->ksp[i],(PetscObject)nep,1);CHKERRQ(ierr);
@@ -502,9 +478,11 @@ PetscErrorCode NEPSetUp_CISS(NEP nep)
       ierr = KSPSetOptionsPrefix(ctx->ksp[i],((PetscObject)nep)->prefix);CHKERRQ(ierr);
       ierr = KSPAppendOptionsPrefix(ctx->ksp[i],"nep_ciss_");CHKERRQ(ierr);
       ierr = KSPSetErrorIfNotConverged(ctx->ksp[i],PETSC_TRUE);CHKERRQ(ierr);
+      ierr = KSPSetTolerances(ctx->ksp[i],SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
     }
   }
 
+  ierr = BVDestroy(&ctx->Y);CHKERRQ(ierr);
   ierr = BVDuplicateResize(nep->V,ctx->num_solve_point*ctx->L_max,&ctx->Y);CHKERRQ(ierr);
 
   ierr = DSSetType(nep->ds,DSGNHEP);CHKERRQ(ierr);
@@ -514,8 +492,6 @@ PetscErrorCode NEPSetUp_CISS(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSolve_CISS"
 PetscErrorCode NEPSolve_CISS(NEP nep)
 {
   PetscErrorCode ierr;
@@ -593,11 +569,12 @@ PetscErrorCode NEPSolve_CISS(NEP nep)
         temp[i+j*ld] = H1[i+j*ctx->L*ctx->M];
     ierr = DSRestoreArray(nep->ds,DS_MAT_A,&temp);CHKERRQ(ierr);
     ierr = DSGetArray(nep->ds,DS_MAT_B,&temp);CHKERRQ(ierr);
-    for (j=0;j<nv;j++) 
+    for (j=0;j<nv;j++)
       for (i=0;i<nv;i++)
         temp[i+j*ld] = H0[i+j*ctx->L*ctx->M];
     ierr = DSRestoreArray(nep->ds,DS_MAT_B,&temp);CHKERRQ(ierr);
     ierr = DSSolve(nep->ds,nep->eigr,nep->eigi);CHKERRQ(ierr);
+    ierr = DSSynchronize(nep->ds,nep->eigr,nep->eigi);CHKERRQ(ierr);
     ierr = DSVectors(nep->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
     for (i=0;i<nv;i++){
       nep->eigr[i] = (nep->eigr[i]*radius+center)*rgscale;
@@ -615,6 +592,7 @@ PetscErrorCode NEPSolve_CISS(NEP nep)
       } else rr[i] = 0.0;
     }
     ierr = DSSort(nep->ds,nep->eigr,nep->eigi,rr,NULL,&nep->nconv);CHKERRQ(ierr);
+    ierr = DSSynchronize(nep->ds,nep->eigr,nep->eigi);CHKERRQ(ierr);
     for (i=0;i<nv;i++){
       nep->eigr[i] = (nep->eigr[i]*radius+center)*rgscale;
 #if !defined(PETSC_USE_COMPLEX)
@@ -661,12 +639,10 @@ PetscErrorCode NEPSolve_CISS(NEP nep)
       ierr = SolveLinearSystem(nep,nep->function,nep->jacobian,ctx->V,0,ctx->L,PETSC_FALSE);CHKERRQ(ierr);
     }
   }
-  ierr = PetscFree3(Mu,H0,H1);CHKERRQ(ierr);  
+  ierr = PetscFree3(Mu,H0,H1);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSSetSizes_CISS"
 static PetscErrorCode NEPCISSSetSizes_CISS(NEP nep,PetscInt ip,PetscInt bs,PetscInt ms,PetscInt npart,PetscInt bsmax,PetscBool realmats)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -707,11 +683,10 @@ static PetscErrorCode NEPCISSSetSizes_CISS(NEP nep,PetscInt ip,PetscInt bs,Petsc
     else ctx->L_max = bsmax;
   }
   ctx->isreal = realmats;
+  nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSSetSizes"
 /*@
    NEPCISSSetSizes - Sets the values of various size parameters in the CISS solver.
 
@@ -763,8 +738,6 @@ PetscErrorCode NEPCISSSetSizes(NEP nep,PetscInt ip,PetscInt bs,PetscInt ms,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSGetSizes_CISS"
 static PetscErrorCode NEPCISSGetSizes_CISS(NEP nep,PetscInt *ip,PetscInt *bs,PetscInt *ms,PetscInt *npart,PetscInt *bsmax,PetscBool *realmats)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -779,8 +752,6 @@ static PetscErrorCode NEPCISSGetSizes_CISS(NEP nep,PetscInt *ip,PetscInt *bs,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSGetSizes"
 /*@
    NEPCISSGetSizes - Gets the values of various size parameters in the CISS solver.
 
@@ -811,8 +782,6 @@ PetscErrorCode NEPCISSGetSizes(NEP nep,PetscInt *ip,PetscInt *bs,PetscInt *ms,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSSetThreshold_CISS"
 static PetscErrorCode NEPCISSSetThreshold_CISS(NEP nep,PetscReal delta,PetscReal spur)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -833,8 +802,6 @@ static PetscErrorCode NEPCISSSetThreshold_CISS(NEP nep,PetscReal delta,PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSSetThreshold"
 /*@
    NEPCISSSetThreshold - Sets the values of various threshold parameters in
    the CISS solver.
@@ -866,8 +833,6 @@ PetscErrorCode NEPCISSSetThreshold(NEP nep,PetscReal delta,PetscReal spur)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSGetThreshold_CISS"
 static PetscErrorCode NEPCISSGetThreshold_CISS(NEP nep,PetscReal *delta,PetscReal *spur)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -878,8 +843,6 @@ static PetscErrorCode NEPCISSGetThreshold_CISS(NEP nep,PetscReal *delta,PetscRea
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSGetThreshold"
 /*@
    NEPCISSGetThreshold - Gets the values of various threshold parameters in
    the CISS solver.
@@ -907,8 +870,6 @@ PetscErrorCode NEPCISSGetThreshold(NEP nep,PetscReal *delta,PetscReal *spur)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSSetRefinement_CISS"
 static PetscErrorCode NEPCISSSetRefinement_CISS(NEP nep,PetscInt inner,PetscInt blsize)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -929,8 +890,6 @@ static PetscErrorCode NEPCISSSetRefinement_CISS(NEP nep,PetscInt inner,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSSetRefinement"
 /*@
    NEPCISSSetRefinement - Sets the values of various refinement parameters
    in the CISS solver.
@@ -962,8 +921,6 @@ PetscErrorCode NEPCISSSetRefinement(NEP nep,PetscInt inner,PetscInt blsize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSGetRefinement_CISS"
 static PetscErrorCode NEPCISSGetRefinement_CISS(NEP nep,PetscInt *inner,PetscInt *blsize)
 {
   NEP_CISS *ctx = (NEP_CISS*)nep->data;
@@ -974,8 +931,6 @@ static PetscErrorCode NEPCISSGetRefinement_CISS(NEP nep,PetscInt *inner,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCISSGetRefinement"
 /*@
    NEPCISSGetRefinement - Gets the values of various refinement parameters
    in the CISS solver.
@@ -1003,8 +958,6 @@ PetscErrorCode NEPCISSGetRefinement(NEP nep, PetscInt *inner, PetscInt *blsize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPReset_CISS"
 PetscErrorCode NEPReset_CISS(NEP nep)
 {
   PetscErrorCode ierr;
@@ -1012,8 +965,6 @@ PetscErrorCode NEPReset_CISS(NEP nep)
   NEP_CISS       *ctx = (NEP_CISS*)nep->data;
 
   PetscFunctionBegin;
-  ierr = PetscSubcommDestroy(&ctx->subcomm);CHKERRQ(ierr);
-  ierr = PetscFree4(ctx->weight,ctx->omega,ctx->pp,ctx->sigma);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->S);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->V);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->Y);CHKERRQ(ierr);
@@ -1021,16 +972,11 @@ PetscErrorCode NEPReset_CISS(NEP nep)
     for (i=0;i<ctx->num_solve_point;i++) {
       ierr = KSPDestroy(&ctx->ksp[i]);CHKERRQ(ierr);
     }
-    for (i=0;i<ctx->num_solve_point;i++) {
-      ierr = MatDestroy(&ctx->kspMat[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree2(ctx->ksp,ctx->kspMat);CHKERRQ(ierr);
+    ierr = PetscFree(ctx->ksp);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetFromOptions_CISS"
 PetscErrorCode NEPSetFromOptions_CISS(PetscOptionItems *PetscOptionsObject,NEP nep)
 {
   PetscErrorCode ierr;
@@ -1041,36 +987,37 @@ PetscErrorCode NEPSetFromOptions_CISS(PetscOptionItems *PetscOptionsObject,NEP n
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"NEP CISS Options");CHKERRQ(ierr);
 
-  ierr = NEPCISSGetSizes(nep,&i1,&i2,&i3,&i4,&i5,&b1);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_integration_points","CISS number of integration points","NEPCISSSetSizes",i1,&i1,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_blocksize","CISS block size","NEPCISSSetSizes",i2,&i2,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_moments","CISS moment size","NEPCISSSetSizes",i3,&i3,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_partitions","CISS number of partitions","NEPCISSSetSizes",i4,&i4,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_maxblocksize","CISS maximum block size","NEPCISSSetSizes",i5,&i5,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-nep_ciss_realmats","CISS flag indicating that T(z) is real for real z","NEPCISSSetSizes",b1,&b1,NULL);CHKERRQ(ierr);
-  ierr = NEPCISSSetSizes(nep,i1,i2,i3,i4,i5,b1);CHKERRQ(ierr);
+    ierr = NEPCISSGetSizes(nep,&i1,&i2,&i3,&i4,&i5,&b1);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_integration_points","Number of integration points","NEPCISSSetSizes",i1,&i1,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_blocksize","Block size","NEPCISSSetSizes",i2,&i2,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_moments","Moment size","NEPCISSSetSizes",i3,&i3,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_partitions","Number of partitions","NEPCISSSetSizes",i4,&i4,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_maxblocksize","Maximum block size","NEPCISSSetSizes",i5,&i5,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-nep_ciss_realmats","True if T(z) is real for real z","NEPCISSSetSizes",b1,&b1,NULL);CHKERRQ(ierr);
+    ierr = NEPCISSSetSizes(nep,i1,i2,i3,i4,i5,b1);CHKERRQ(ierr);
 
-  ierr = NEPCISSGetThreshold(nep,&r1,&r2);CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-nep_ciss_delta","CISS threshold for numerical rank","NEPCISSSetThreshold",r1,&r1,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-nep_ciss_spurious_threshold","CISS threshold for the spurious eigenpairs","NEPCISSSetThreshold",r2,&r2,NULL);CHKERRQ(ierr);
-  ierr = NEPCISSSetThreshold(nep,r1,r2);CHKERRQ(ierr);
+    ierr = NEPCISSGetThreshold(nep,&r1,&r2);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-nep_ciss_delta","Threshold for numerical rank","NEPCISSSetThreshold",r1,&r1,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-nep_ciss_spurious_threshold","Threshold for the spurious eigenpairs","NEPCISSSetThreshold",r2,&r2,NULL);CHKERRQ(ierr);
+    ierr = NEPCISSSetThreshold(nep,r1,r2);CHKERRQ(ierr);
 
-  ierr = NEPCISSGetRefinement(nep,&i6,&i7);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_refine_inner","CISS number of inner iterative refinement iterations","NEPCISSSetRefinement",i6,&i6,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_ciss_refine_blocksize","CISS number of blocksize iterative refinement iterations","NEPCISSSetRefinement",i7,&i7,NULL);CHKERRQ(ierr);
-  ierr = NEPCISSSetRefinement(nep,i6,i7);CHKERRQ(ierr);
+    ierr = NEPCISSGetRefinement(nep,&i6,&i7);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_refine_inner","Number of inner iterative refinement iterations","NEPCISSSetRefinement",i6,&i6,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_ciss_refine_blocksize","Number of blocksize iterative refinement iterations","NEPCISSSetRefinement",i7,&i7,NULL);CHKERRQ(ierr);
+    ierr = NEPCISSSetRefinement(nep,i6,i7);CHKERRQ(ierr);
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPDestroy_CISS"
 PetscErrorCode NEPDestroy_CISS(NEP nep)
 {
   PetscErrorCode ierr;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
 
   PetscFunctionBegin;
+  ierr = PetscSubcommDestroy(&ctx->subcomm);CHKERRQ(ierr);
+  ierr = PetscFree4(ctx->weight,ctx->omega,ctx->pp,ctx->sigma);CHKERRQ(ierr);
   ierr = PetscFree(nep->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetSizes_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetSizes_C",NULL);CHKERRQ(ierr);
@@ -1081,8 +1028,6 @@ PetscErrorCode NEPDestroy_CISS(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPView_CISS"
 PetscErrorCode NEPView_CISS(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -1092,12 +1037,12 @@ PetscErrorCode NEPView_CISS(NEP nep,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  CISS: sizes { integration points: %D, block size: %D, moment size: %D, partitions: %D, maximum block size: %D }\n",ctx->N,ctx->L,ctx->M,ctx->num_subcomm,ctx->L_max);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  sizes { integration points: %D, block size: %D, moment size: %D, partitions: %D, maximum block size: %D }\n",ctx->N,ctx->L,ctx->M,ctx->num_subcomm,ctx->L_max);CHKERRQ(ierr);
     if (ctx->isreal) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  CISS: exploiting symmetry of integration points\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  exploiting symmetry of integration points\n");CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer,"  CISS: threshold { delta: %g, spurious threshold: %g }\n",(double)ctx->delta,(double)ctx->spurious_threshold);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  CISS: iterative refinement  { inner: %D, blocksize: %D }\n",ctx->refine_inner, ctx->refine_blocksize);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  threshold { delta: %g, spurious threshold: %g }\n",(double)ctx->delta,(double)ctx->spurious_threshold);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  iterative refinement  { inner: %D, blocksize: %D }\n",ctx->refine_inner, ctx->refine_blocksize);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     if (!ctx->usest && ctx->ksp[0]) { ierr = KSPView(ctx->ksp[0],viewer);CHKERRQ(ierr); }
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
@@ -1105,8 +1050,6 @@ PetscErrorCode NEPView_CISS(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCreate_CISS"
 PETSC_EXTERN PetscErrorCode NEPCreate_CISS(NEP nep)
 {
   PetscErrorCode ierr;
@@ -1115,30 +1058,30 @@ PETSC_EXTERN PetscErrorCode NEPCreate_CISS(NEP nep)
   PetscFunctionBegin;
   ierr = PetscNewLog(nep,&ctx);CHKERRQ(ierr);
   nep->data = ctx;
+  /* set default values of parameters */
+  ctx->N                  = 32;
+  ctx->L                  = 16;
+  ctx->M                  = ctx->N/4;
+  ctx->delta              = 1e-12;
+  ctx->L_max              = 64;
+  ctx->spurious_threshold = 1e-4;
+  ctx->usest              = PETSC_FALSE;
+  ctx->isreal             = PETSC_FALSE;
+  ctx->num_subcomm        = 1;
+
   nep->ops->solve          = NEPSolve_CISS;
   nep->ops->setup          = NEPSetUp_CISS;
   nep->ops->setfromoptions = NEPSetFromOptions_CISS;
   nep->ops->reset          = NEPReset_CISS;
   nep->ops->destroy        = NEPDestroy_CISS;
   nep->ops->view           = NEPView_CISS;
+
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetSizes_C",NEPCISSSetSizes_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetSizes_C",NEPCISSGetSizes_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetThreshold_C",NEPCISSSetThreshold_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetThreshold_C",NEPCISSGetThreshold_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetRefinement_C",NEPCISSSetRefinement_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetRefinement_C",NEPCISSGetRefinement_CISS);CHKERRQ(ierr);
-  /* set default values of parameters */
-  ctx->N       = 32;
-  ctx->L       = 16;
-  ctx->M       = ctx->N/4;
-  ctx->delta   = 1e-12;
-  ctx->L_max   = 64;
-  ctx->spurious_threshold = 1e-4;
-  ctx->usest   = PETSC_FALSE;
-  ctx->isreal  = PETSC_FALSE;
-  ctx->refine_inner = 0;
-  ctx->refine_blocksize = 0;
-  ctx->num_subcomm = 1;
   PetscFunctionReturn(0);
 }
 
