@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc nonlinear eigensolver: "interpol"
 
    Method: Polynomial interpolation
@@ -13,25 +21,6 @@
 
        [1] C. Effenberger and D. Kresser, "Chebyshev interpolation for
            nonlinear eigenvalue problems", BIT 52:933-951, 2012.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/nepimpl.h>         /*I "slepcnep.h" I*/
@@ -39,11 +28,11 @@
 
 typedef struct {
   PEP       pep;
-  PetscInt  deg;
+  PetscReal tol;       /* tolerance for norm of polynomial coefficients */
+  PetscInt  maxdeg;    /* maximum degree of interpolation polynomial */
+  PetscInt  deg;       /* actual degree of interpolation polynomial */
 } NEP_INTERPOL;
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetUp_Interpol"
 PetscErrorCode NEPSetUp_Interpol(NEP nep)
 {
   PetscErrorCode ierr;
@@ -64,15 +53,14 @@ PetscErrorCode NEPSetUp_Interpol(NEP nep)
 
   /* transfer PEP options */
   if (!ctx->pep) { ierr = NEPInterpolGetPEP(nep,&ctx->pep);CHKERRQ(ierr); }
-  ierr = PEPSetBV(ctx->pep,nep->V);CHKERRQ(ierr);
   ierr = PEPSetBasis(ctx->pep,PEP_BASIS_CHEBYSHEV1);CHKERRQ(ierr);
   ierr = PEPSetWhichEigenpairs(ctx->pep,PEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
   ierr = PEPGetST(ctx->pep,&st);CHKERRQ(ierr);
   ierr = STSetType(st,STSINVERT);CHKERRQ(ierr);
   ierr = PEPSetDimensions(ctx->pep,nep->nev,nep->ncv?nep->ncv:PETSC_DEFAULT,nep->mpd?nep->mpd:PETSC_DEFAULT);CHKERRQ(ierr);
-  tol=ctx->pep->tol;
-  if (tol==PETSC_DEFAULT) tol = (nep->tol==PETSC_DEFAULT)?SLEPC_DEFAULT_TOL/10.0:nep->tol/10.0;
-  its=ctx->pep->max_it;
+  ierr = PEPGetTolerances(ctx->pep,&tol,&its);CHKERRQ(ierr);
+  if (tol==PETSC_DEFAULT) tol = (nep->tol==PETSC_DEFAULT)?SLEPC_DEFAULT_TOL:nep->tol;
+  if (ctx->tol==PETSC_DEFAULT) ctx->tol = tol;
   if (!its) its = nep->max_it?nep->max_it:PETSC_DEFAULT;
   ierr = PEPSetTolerances(ctx->pep,tol,its);CHKERRQ(ierr);
   ierr = NEPGetTrackAll(nep,&trackall);CHKERRQ(ierr);
@@ -100,10 +88,8 @@ PetscErrorCode NEPSetUp_Interpol(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ChebyshevNodes"
 /*
-  Input: 
+  Input:
     d, number of nodes to compute
     a,b, interval extrems
   Output:
@@ -124,19 +110,25 @@ static PetscErrorCode ChebyshevNodes(PetscInt d,PetscReal a,PetscReal b,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSolve_Interpol"
 PetscErrorCode NEPSolve_Interpol(NEP nep)
 {
   PetscErrorCode ierr;
   NEP_INTERPOL   *ctx = (NEP_INTERPOL*)nep->data;
-  Mat            *A;   /*T=nep->function,Tp=nep->jacobian;*/
+  Mat            *A;
   PetscScalar    *x,*fx,t;
-  PetscReal      *cs,a,b,s;
-  PetscInt       i,j,k,deg=ctx->deg;
+  PetscReal      *cs,a,b,s,aprox,aprox0=1.0,*matnorm;
+  PetscInt       i,j,k,deg=ctx->maxdeg;
+  PetscBool      hasmnorm;
+  Vec            vr,vi=NULL;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc4(deg+1,&A,(deg+1)*(deg+1),&cs,deg+1,&x,(deg+1)*nep->nt,&fx);CHKERRQ(ierr);
+  ierr = PetscMalloc5(deg+1,&A,(deg+1)*(deg+1),&cs,deg+1,&x,(deg+1)*nep->nt,&fx,nep->nt,&matnorm);CHKERRQ(ierr);
+  for  (j=0;j<nep->nt;j++) {
+    ierr = MatHasOperation(nep->A[j],MATOP_NORM,&hasmnorm);CHKERRQ(ierr);
+    if (!hasmnorm) break;
+    ierr = MatNorm(nep->A[j],NORM_INFINITY,matnorm+j);CHKERRQ(ierr);
+  }
+  if (!hasmnorm) for (j=0;j<nep->nt;j++) matnorm[j] = 1.0;
   ierr = RGIntervalGetEndpoints(nep->rg,&a,&b,NULL,NULL);CHKERRQ(ierr);
   ierr = ChebyshevNodes(deg,a,b,x,cs);CHKERRQ(ierr);
   for (j=0;j<nep->nt;j++) {
@@ -144,53 +136,70 @@ PetscErrorCode NEPSolve_Interpol(NEP nep)
       ierr = FNEvaluateFunction(nep->f[j],x[i],&fx[i+j*(deg+1)]);CHKERRQ(ierr);
     }
   }
-
   /* Polynomial coefficients */
+  ctx->deg = deg;
   for (k=0;k<=deg;k++) {
     ierr = MatDuplicate(nep->A[0],MAT_COPY_VALUES,&A[k]);CHKERRQ(ierr);
     t = 0.0;
     for (i=0;i<deg+1;i++) t += fx[i]*cs[i*(deg+1)+k];
-    t *= 2.0/(deg+1); 
+    t *= 2.0/(deg+1);
     if (k==0) t /= 2.0;
+    aprox = matnorm[0]*PetscAbsScalar(t);
     ierr = MatScale(A[k],t);CHKERRQ(ierr);
     for (j=1;j<nep->nt;j++) {
       t = 0.0;
       for (i=0;i<deg+1;i++) t += fx[i+j*(deg+1)]*cs[i*(deg+1)+k];
-      t *= 2.0/(deg+1); 
+      t *= 2.0/(deg+1);
       if (k==0) t /= 2.0;
+      aprox += matnorm[j]*PetscAbsScalar(t);
       ierr = MatAXPY(A[k],t,nep->A[j],nep->mstr);CHKERRQ(ierr);
     }
+    if (k==0) aprox0 = aprox;
+    if (aprox/aprox0<ctx->tol) { ctx->deg = k; deg = k; break; }
   }
 
   ierr = PEPSetOperators(ctx->pep,deg+1,A);CHKERRQ(ierr);
   for (k=0;k<=deg;k++) {
     ierr = MatDestroy(&A[k]);CHKERRQ(ierr);
   }
-  ierr = PetscFree4(A,cs,x,fx);CHKERRQ(ierr);
+  ierr = PetscFree5(A,cs,x,fx,matnorm);CHKERRQ(ierr);
 
   /* Solve polynomial eigenproblem */
   ierr = PEPSolve(ctx->pep);CHKERRQ(ierr);
   ierr = PEPGetConverged(ctx->pep,&nep->nconv);CHKERRQ(ierr);
   ierr = PEPGetIterationNumber(ctx->pep,&nep->its);CHKERRQ(ierr);
   ierr = PEPGetConvergedReason(ctx->pep,(PEPConvergedReason*)&nep->reason);CHKERRQ(ierr);
+  ierr = BVSetActiveColumns(nep->V,0,nep->nconv);CHKERRQ(ierr);
+  ierr = BVCreateVec(nep->V,&vr);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)   
+  ierr = VecDuplicate(vr,&vi);CHKERRQ(ierr);
+#endif
   s = 2.0/(b-a);
   for (i=0;i<nep->nconv;i++) {
-    ierr = PEPGetEigenpair(ctx->pep,i,&nep->eigr[i],&nep->eigi[i],NULL,NULL);CHKERRQ(ierr);
+    ierr = PEPGetEigenpair(ctx->pep,i,&nep->eigr[i],&nep->eigi[i],vr,vi);CHKERRQ(ierr);
     nep->eigr[i] /= s;
     nep->eigr[i] += (a+b)/2.0;
     nep->eigi[i] /= s;
+    ierr = BVInsertVec(nep->V,i,vr);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)   
+    if (nep->eigi[i]!=0.0) {
+      ierr = BVInsertVec(nep->V,++i,vi);CHKERRQ(ierr);
+    }
+#endif
   }
+  ierr = VecDestroy(&vr);CHKERRQ(ierr);
+  ierr = VecDestroy(&vi);CHKERRQ(ierr);
+
   nep->state = NEP_STATE_EIGENVECTORS;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPMonitor_Interpol"
 static PetscErrorCode PEPMonitor_Interpol(PEP pep,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
   PetscInt       i,n;
   NEP            nep = (NEP)ctx;
   PetscReal      a,b,s;
+  ST             st;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -200,86 +209,118 @@ static PetscErrorCode PEPMonitor_Interpol(PEP pep,PetscInt its,PetscInt nconv,Pe
     nep->eigi[i]   = eigi[i];
     nep->errest[i] = errest[i];
   }
-  ierr = STBackTransform(pep->st,n,nep->eigr,nep->eigi);CHKERRQ(ierr);
+  ierr = PEPGetST(pep,&st);CHKERRQ(ierr);
+  ierr = STBackTransform(st,n,nep->eigr,nep->eigi);CHKERRQ(ierr);
   ierr = RGIntervalGetEndpoints(nep->rg,&a,&b,NULL,NULL);CHKERRQ(ierr);
   s = 2.0/(b-a);
   for (i=0;i<n;i++) {
     nep->eigr[i] /= s;
     nep->eigr[i] += (a+b)/2.0;
     nep->eigi[i] /= s;
-  }  
+  }
   ierr = NEPMonitor(nep,its,nconv,nep->eigr,nep->eigi,nep->errest,nest);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetFromOptions_Interpol"
 PetscErrorCode NEPSetFromOptions_Interpol(PetscOptionItems *PetscOptionsObject,NEP nep)
+{
+  PetscErrorCode ierr;
+  NEP_INTERPOL   *ctx = (NEP_INTERPOL*)nep->data;
+  PetscInt       i;
+  PetscBool      flg1,flg2;
+  PetscReal      r;
+
+  PetscFunctionBegin;
+  ierr = PetscOptionsHead(PetscOptionsObject,"NEP Interpol Options");CHKERRQ(ierr);
+
+    ierr = NEPInterpolGetInterpolation(nep,&r,&i);CHKERRQ(ierr);
+    if (!i) i = PETSC_DEFAULT;
+    ierr = PetscOptionsInt("-nep_interpol_interpolation_degree","Maximum degree of polynomial interpolation","NEPInterpolSetInterpolation",i,&i,&flg1);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-nep_interpol_interpolation_tol","Tolerance for interpolation coefficients","NEPInterpolSetInterpolation",r,&r,&flg2);CHKERRQ(ierr);
+    if (flg1 || flg2) { ierr = NEPInterpolSetInterpolation(nep,r,i);CHKERRQ(ierr); }
+
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  if (!ctx->pep) { ierr = NEPInterpolGetPEP(nep,&ctx->pep);CHKERRQ(ierr); }
+  ierr = PEPSetFromOptions(ctx->pep);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode NEPInterpolSetInterpolation_Interpol(NEP nep,PetscReal tol,PetscInt degree)
 {
   PetscErrorCode ierr;
   NEP_INTERPOL   *ctx = (NEP_INTERPOL*)nep->data;
 
   PetscFunctionBegin;
-  if (!ctx->pep) { ierr = NEPInterpolGetPEP(nep,&ctx->pep);CHKERRQ(ierr); }
-  ierr = PEPSetFromOptions(ctx->pep);CHKERRQ(ierr);
-  ierr = PetscOptionsHead(PetscOptionsObject,"NEP Interpol Options");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-nep_interpol_degree","Degree of interpolation polynomial","NEPInterpolSetDegree",ctx->deg,&ctx->deg,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  if (tol == PETSC_DEFAULT) {
+    ctx->tol   = PETSC_DEFAULT;
+    nep->state = NEP_STATE_INITIAL;
+  } else {
+    if (tol <= 0.0) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+    ctx->tol = tol;
+  }
+  if (degree == PETSC_DEFAULT || degree == PETSC_DECIDE) {
+    ctx->maxdeg = 0;
+    if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
+    nep->state = NEP_STATE_INITIAL;
+  } else {
+    if (degree <= 0) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of degree. Must be > 0");
+    if (ctx->maxdeg != degree) {
+      ctx->maxdeg = degree;
+      if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
+      nep->state = NEP_STATE_INITIAL;
+    }
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolSetDegree_Interpol"
-static PetscErrorCode NEPInterpolSetDegree_Interpol(NEP nep,PetscInt deg)
-{
-  NEP_INTERPOL *ctx = (NEP_INTERPOL*)nep->data;
-
-  PetscFunctionBegin;
-  ctx->deg = deg;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolSetDegree"
 /*@
-   NEPInterpolSetDegree - Sets the degree of the interpolation polynomial.
+   NEPInterpolSetInterpolation - Sets the tolerance and maximum degree when building
+   the interpolation polynomial.
 
    Collective on NEP
 
    Input Parameters:
 +  nep - nonlinear eigenvalue solver
--  deg - polynomial degree
+.  tol - tolerance to stop computing polynomial coefficients
+-  deg - maximum degree of interpolation
+
+   Options Database Key:
++  -nep_interpol_interpolation_tol <tol> - Sets the tolerance to stop computing polynomial coefficients
+-  -nep_interpol_interpolation_degree <degree> - Sets the maximum degree of interpolation
+
+   Notes:
+   Use PETSC_DEFAULT for either argument to assign a reasonably good value.
 
    Level: advanced
 
-.seealso: NEPInterpolGetDegree()
+.seealso: NEPInterpolGetInterpolation()
 @*/
-PetscErrorCode NEPInterpolSetDegree(NEP nep,PetscInt deg)
+PetscErrorCode NEPInterpolSetInterpolation(NEP nep,PetscReal tol,PetscInt deg)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidLogicalCollectiveInt(nep,deg,2);
-  ierr = PetscTryMethod(nep,"NEPInterpolSetDegree_C",(NEP,PetscInt),(nep,deg));CHKERRQ(ierr);
+  PetscValidLogicalCollectiveReal(nep,tol,2);
+  PetscValidLogicalCollectiveInt(nep,deg,3);
+  ierr = PetscTryMethod(nep,"NEPInterpolSetInterpolation_C",(NEP,PetscReal,PetscInt),(nep,tol,deg));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolGetDegree_Interpol"
-static PetscErrorCode NEPInterpolGetDegree_Interpol(NEP nep,PetscInt *deg)
+static PetscErrorCode NEPInterpolGetInterpolation_Interpol(NEP nep,PetscReal *tol,PetscInt *deg)
 {
   NEP_INTERPOL *ctx = (NEP_INTERPOL*)nep->data;
 
   PetscFunctionBegin;
-  *deg = ctx->deg;
+  if (tol) *tol = ctx->tol;
+  if (deg) *deg = ctx->maxdeg;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolGetDegree"
 /*@
-   NEPInterpolGetDegree - Gets the degree of the interpolation polynomial.
+   NEPInterpolGetInterpolation - Gets the tolerance and maximum degree when building
+   the interpolation polynomial.
 
    Not Collective
 
@@ -287,25 +328,23 @@ static PetscErrorCode NEPInterpolGetDegree_Interpol(NEP nep,PetscInt *deg)
 .  nep - nonlinear eigenvalue solver
 
    Output Parameter:
-.  deg - the polynomial degree
++  tol - tolerance to stop computing polynomial coefficients
+-  deg - maximum degree of interpolation
 
    Level: advanced
 
-.seealso: NEPInterpolSetDegree()
+.seealso: NEPInterpolSetInterpolation()
 @*/
-PetscErrorCode NEPInterpolGetDegree(NEP nep,PetscInt *deg)
+PetscErrorCode NEPInterpolGetInterpolation(NEP nep,PetscReal *tol,PetscInt *deg)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(deg,2);
-  ierr = PetscUseMethod(nep,"NEPInterpolGetDegree_C",(NEP,PetscInt*),(nep,deg));CHKERRQ(ierr);
+  ierr = PetscUseMethod(nep,"NEPInterpolGetInterpolation_C",(NEP,PetscReal*,PetscInt*),(nep,tol,deg));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolSetPEP_Interpol"
 static PetscErrorCode NEPInterpolSetPEP_Interpol(NEP nep,PEP pep)
 {
   PetscErrorCode ierr;
@@ -320,8 +359,6 @@ static PetscErrorCode NEPInterpolSetPEP_Interpol(NEP nep,PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolSetPEP"
 /*@
    NEPInterpolSetPEP - Associate a polynomial eigensolver object (PEP) to the
    nonlinear eigenvalue solver.
@@ -348,21 +385,16 @@ PetscErrorCode NEPInterpolSetPEP(NEP nep,PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolGetPEP_Interpol"
 static PetscErrorCode NEPInterpolGetPEP_Interpol(NEP nep,PEP *pep)
 {
   PetscErrorCode ierr;
   NEP_INTERPOL   *ctx = (NEP_INTERPOL*)nep->data;
-  ST             st;
 
   PetscFunctionBegin;
   if (!ctx->pep) {
     ierr = PEPCreate(PetscObjectComm((PetscObject)nep),&ctx->pep);CHKERRQ(ierr);
     ierr = PEPSetOptionsPrefix(ctx->pep,((PetscObject)nep)->prefix);CHKERRQ(ierr);
     ierr = PEPAppendOptionsPrefix(ctx->pep,"nep_interpol_");CHKERRQ(ierr);
-    ierr = PEPGetST(ctx->pep,&st);CHKERRQ(ierr);
-    ierr = STSetOptionsPrefix(st,((PetscObject)ctx->pep)->prefix);CHKERRQ(ierr);
     ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->pep,(PetscObject)nep,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->pep);CHKERRQ(ierr);
     ierr = PEPMonitorSet(ctx->pep,PEPMonitor_Interpol,nep,NULL);CHKERRQ(ierr);
@@ -371,8 +403,6 @@ static PetscErrorCode NEPInterpolGetPEP_Interpol(NEP nep,PEP *pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPInterpolGetPEP"
 /*@
    NEPInterpolGetPEP - Retrieve the polynomial eigensolver object (PEP)
    associated with the nonlinear eigenvalue solver.
@@ -400,8 +430,6 @@ PetscErrorCode NEPInterpolGetPEP(NEP nep,PEP *pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPView_Interpol"
 PetscErrorCode NEPView_Interpol(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -412,7 +440,8 @@ PetscErrorCode NEPView_Interpol(NEP nep,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     if (!ctx->pep) { ierr = NEPInterpolGetPEP(nep,&ctx->pep);CHKERRQ(ierr); }
-    ierr = PetscViewerASCIIPrintf(viewer,"  Interpol: polynomial degree %D\n",ctx->deg);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  polynomial degree %D, max=%D\n",ctx->deg,ctx->maxdeg);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  tolerance for norm of polynomial coefficients %g\n",ctx->tol);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = PEPView(ctx->pep,viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
@@ -420,20 +449,16 @@ PetscErrorCode NEPView_Interpol(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPReset_Interpol"
 PetscErrorCode NEPReset_Interpol(NEP nep)
 {
   PetscErrorCode ierr;
   NEP_INTERPOL   *ctx = (NEP_INTERPOL*)nep->data;
 
   PetscFunctionBegin;
-  if (!ctx->pep) { ierr = PEPReset(ctx->pep);CHKERRQ(ierr); }
+  ierr = PEPReset(ctx->pep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPDestroy_Interpol"
 PetscErrorCode NEPDestroy_Interpol(NEP nep)
 {
   PetscErrorCode ierr;
@@ -442,15 +467,13 @@ PetscErrorCode NEPDestroy_Interpol(NEP nep)
   PetscFunctionBegin;
   ierr = PEPDestroy(&ctx->pep);CHKERRQ(ierr);
   ierr = PetscFree(nep->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolSetDegree_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolGetDegree_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolSetInterpolation_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolGetInterpolation_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolSetPEP_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolGetPEP_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCreate_Interpol"
 PETSC_EXTERN PetscErrorCode NEPCreate_Interpol(NEP nep)
 {
   PetscErrorCode ierr;
@@ -458,8 +481,9 @@ PETSC_EXTERN PetscErrorCode NEPCreate_Interpol(NEP nep)
 
   PetscFunctionBegin;
   ierr = PetscNewLog(nep,&ctx);CHKERRQ(ierr);
-  ctx->deg  = 5;
-  nep->data = (void*)ctx;
+  nep->data   = (void*)ctx;
+  ctx->maxdeg = 5;
+  ctx->tol    = PETSC_DEFAULT;
 
   nep->ops->solve          = NEPSolve_Interpol;
   nep->ops->setup          = NEPSetUp_Interpol;
@@ -467,8 +491,9 @@ PETSC_EXTERN PetscErrorCode NEPCreate_Interpol(NEP nep)
   nep->ops->reset          = NEPReset_Interpol;
   nep->ops->destroy        = NEPDestroy_Interpol;
   nep->ops->view           = NEPView_Interpol;
-  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolSetDegree_C",NEPInterpolSetDegree_Interpol);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolGetDegree_C",NEPInterpolGetDegree_Interpol);CHKERRQ(ierr);
+
+  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolSetInterpolation_C",NEPInterpolSetInterpolation_Interpol);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolGetInterpolation_C",NEPInterpolGetInterpolation_Interpol);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolSetPEP_C",NEPInterpolSetPEP_Interpol);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPInterpolGetPEP_C",NEPInterpolGetPEP_Interpol);CHKERRQ(ierr);
   PetscFunctionReturn(0);
