@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc nonlinear eigensolver: "narnoldi"
 
    Method: Nonlinear Arnoldi
@@ -12,25 +20,6 @@
 
        [1] H. Voss, "An Arnoldi method for nonlinear eigenvalue problems",
            BIT 44:387-401, 2004.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/nepimpl.h>         /*I "slepcnep.h" I*/
@@ -39,8 +28,6 @@ typedef struct {
   KSP      ksp;              /* linear solver object */
 } NEP_NARNOLDI;
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPNArnoldi_KSPSolve"
 PETSC_STATIC_INLINE PetscErrorCode NEPNArnoldi_KSPSolve(NEP nep,Vec b,Vec x)
 {
   PetscErrorCode ierr;
@@ -54,8 +41,6 @@ PETSC_STATIC_INLINE PetscErrorCode NEPNArnoldi_KSPSolve(NEP nep,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetUp_NArnoldi"
 PetscErrorCode NEPSetUp_NArnoldi(NEP nep)
 {
   PetscErrorCode ierr;
@@ -83,8 +68,6 @@ PetscErrorCode NEPSetUp_NArnoldi(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSolve_NArnoldi"
 PetscErrorCode NEPSolve_NArnoldi(NEP nep)
 {
   PetscErrorCode     ierr;
@@ -125,6 +108,7 @@ PetscErrorCode NEPSolve_NArnoldi(NEP nep)
     ierr = DSSetDimensions(nep->ds,n,0,0,0);CHKERRQ(ierr);
     ierr = DSSetState(nep->ds,DS_STATE_RAW);CHKERRQ(ierr);
     ierr = DSSolve(nep->ds,nep->eigr,NULL);CHKERRQ(ierr);
+    ierr = DSSynchronize(nep->ds,nep->eigr,NULL);CHKERRQ(ierr);
     lambda = nep->eigr[0];
 
     /* compute Ritz vector, x = V*s */
@@ -160,13 +144,12 @@ PetscErrorCode NEPSolve_NArnoldi(NEP nep)
       }
 
       /* orthonormalize */
-      ierr = BVOrthogonalizeColumn(nep->V,n,NULL,&beta,&breakdown);CHKERRQ(ierr);
+      ierr = BVOrthonormalizeColumn(nep->V,n,PETSC_FALSE,&beta,&breakdown);CHKERRQ(ierr);
       if (breakdown || beta==0.0) {
         ierr = PetscInfo1(nep,"iter=%D, orthogonalization failed, stopping solve\n",nep->its);CHKERRQ(ierr);
         nep->reason = NEP_DIVERGED_BREAKDOWN;
         break;
       }
-      ierr = BVScaleColumn(nep->V,n,1.0/beta);CHKERRQ(ierr);
 
       /* update projected matrices */
       ierr = DSSetDimensions(nep->ds,n+1,0,0,0);CHKERRQ(ierr);
@@ -178,22 +161,20 @@ PetscErrorCode NEPSolve_NArnoldi(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetFromOptions_NArnoldi"
 PetscErrorCode NEPSetFromOptions_NArnoldi(PetscOptionItems *PetscOptionsObject,NEP nep)
 {
   PetscErrorCode ierr;
   NEP_NARNOLDI   *ctx = (NEP_NARNOLDI*)nep->data;
 
   PetscFunctionBegin;
+  ierr = PetscOptionsHead(PetscOptionsObject,"NEP N-Arnoldi Options");CHKERRQ(ierr);
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
+
   if (!ctx->ksp) { ierr = NEPNArnoldiGetKSP(nep,&ctx->ksp);CHKERRQ(ierr); }
-  ierr = KSPSetOperators(ctx->ksp,nep->function,nep->function_pre);CHKERRQ(ierr);
   ierr = KSPSetFromOptions(ctx->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPNArnoldiSetKSP_NArnoldi"
 static PetscErrorCode NEPNArnoldiSetKSP_NArnoldi(NEP nep,KSP ksp)
 {
   PetscErrorCode ierr;
@@ -208,8 +189,6 @@ static PetscErrorCode NEPNArnoldiSetKSP_NArnoldi(NEP nep,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPNArnoldiSetKSP"
 /*@
    NEPNArnoldiSetKSP - Associate a linear solver object (KSP) to the nonlinear
    eigenvalue solver.
@@ -236,8 +215,6 @@ PetscErrorCode NEPNArnoldiSetKSP(NEP nep,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPNArnoldiGetKSP_NArnoldi"
 static PetscErrorCode NEPNArnoldiGetKSP_NArnoldi(NEP nep,KSP *ksp)
 {
   PetscErrorCode ierr;
@@ -251,13 +228,12 @@ static PetscErrorCode NEPNArnoldiGetKSP_NArnoldi(NEP nep,KSP *ksp)
     ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->ksp,(PetscObject)nep,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->ksp);CHKERRQ(ierr);
     ierr = KSPSetErrorIfNotConverged(ctx->ksp,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(ctx->ksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
   }
   *ksp = ctx->ksp;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPNArnoldiGetKSP"
 /*@
    NEPNArnoldiGetKSP - Retrieve the linear solver object (KSP) associated with
    the nonlinear eigenvalue solver.
@@ -285,8 +261,6 @@ PetscErrorCode NEPNArnoldiGetKSP(NEP nep,KSP *ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPView_NArnoldi"
 PetscErrorCode NEPView_NArnoldi(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -304,8 +278,16 @@ PetscErrorCode NEPView_NArnoldi(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPDestroy_NArnoldi"
+PetscErrorCode NEPReset_NArnoldi(NEP nep)
+{
+  PetscErrorCode ierr;
+  NEP_NARNOLDI   *ctx = (NEP_NARNOLDI*)nep->data;
+
+  PetscFunctionBegin;
+  ierr = KSPReset(ctx->ksp);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode NEPDestroy_NArnoldi(NEP nep)
 {
   PetscErrorCode ierr;
@@ -319,8 +301,6 @@ PetscErrorCode NEPDestroy_NArnoldi(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCreate_NArnoldi"
 PETSC_EXTERN PetscErrorCode NEPCreate_NArnoldi(NEP nep)
 {
   PetscErrorCode ierr;
@@ -333,8 +313,10 @@ PETSC_EXTERN PetscErrorCode NEPCreate_NArnoldi(NEP nep)
   nep->ops->solve          = NEPSolve_NArnoldi;
   nep->ops->setup          = NEPSetUp_NArnoldi;
   nep->ops->setfromoptions = NEPSetFromOptions_NArnoldi;
+  nep->ops->reset          = NEPReset_NArnoldi;
   nep->ops->destroy        = NEPDestroy_NArnoldi;
   nep->ops->view           = NEPView_NArnoldi;
+
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPNArnoldiSetKSP_C",NEPNArnoldiSetKSP_NArnoldi);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPNArnoldiGetKSP_C",NEPNArnoldiGetKSP_NArnoldi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
