@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc nonlinear eigensolver: "rii"
 
    Method: Residual inverse iteration
@@ -12,25 +20,6 @@
 
        [1] A. Neumaier, "Residual inverse iteration for the nonlinear
            eigenvalue problem", SIAM J. Numer. Anal. 22(5):914-923, 1985.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/nepimpl.h>         /*I "slepcnep.h" I*/
@@ -42,8 +31,6 @@ typedef struct {
   KSP       ksp;              /* linear solver object */
 } NEP_RII;
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRII_KSPSolve"
 PETSC_STATIC_INLINE PetscErrorCode NEPRII_KSPSolve(NEP nep,Vec b,Vec x)
 {
   PetscErrorCode ierr;
@@ -57,8 +44,6 @@ PETSC_STATIC_INLINE PetscErrorCode NEPRII_KSPSolve(NEP nep,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetUp_RII"
 PetscErrorCode NEPSetUp_RII(NEP nep)
 {
   PetscErrorCode ierr;
@@ -81,8 +66,6 @@ PetscErrorCode NEPSetUp_RII(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSolve_RII"
 PetscErrorCode NEPSolve_RII(NEP nep)
 {
   PetscErrorCode     ierr;
@@ -113,7 +96,7 @@ PetscErrorCode NEPSolve_RII(NEP nep)
   while (nep->reason == NEP_CONVERGED_ITERATING) {
     nep->its++;
 
-    /* Use Newton's method to compute nonlinear Rayleigh functional. Current eigenvalue 
+    /* Use Newton's method to compute nonlinear Rayleigh functional. Current eigenvalue
        estimate as starting value. */
     inner_its=0;
     do {
@@ -177,8 +160,6 @@ PetscErrorCode NEPSolve_RII(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetFromOptions_RII"
 PetscErrorCode NEPSetFromOptions_RII(PetscOptionItems *PetscOptionsObject,NEP nep)
 {
   PetscErrorCode ierr;
@@ -187,21 +168,23 @@ PetscErrorCode NEPSetFromOptions_RII(PetscOptionItems *PetscOptionsObject,NEP ne
   PetscInt       i;
 
   PetscFunctionBegin;
-  if (!ctx->ksp) { ierr = NEPRIIGetKSP(nep,&ctx->ksp);CHKERRQ(ierr); }
-  ierr = KSPSetOperators(ctx->ksp,nep->function,nep->function_pre);CHKERRQ(ierr);
-  ierr = KSPSetFromOptions(ctx->ksp);CHKERRQ(ierr);
   ierr = PetscOptionsHead(PetscOptionsObject,"NEP RII Options");CHKERRQ(ierr);
+
     ierr = PetscOptionsInt("-nep_rii_max_it","Maximum number of Newton iterations for updating Rayleigh functional","NEPRIISetMaximumIterations",ctx->max_inner_it,&ctx->max_inner_it,NULL);CHKERRQ(ierr);
+
     ierr = PetscOptionsBool("-nep_rii_const_correction_tol","Constant correction tolerance for the linear solver","NEPRIISetConstCorrectionTol",ctx->cctol,&ctx->cctol,NULL);CHKERRQ(ierr);
+
     i = 0;
     ierr = PetscOptionsInt("-nep_rii_lag_preconditioner","Interval to rebuild preconditioner","NEPRIISetLagPreconditioner",ctx->lag,&i,&flg);CHKERRQ(ierr);
     if (flg) { ierr = NEPRIISetLagPreconditioner(nep,i);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  if (!ctx->ksp) { ierr = NEPRIIGetKSP(nep,&ctx->ksp);CHKERRQ(ierr); }
+  ierr = KSPSetFromOptions(ctx->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetMaximumIterations_RII"
 static PetscErrorCode NEPRIISetMaximumIterations_RII(NEP nep,PetscInt its)
 {
   NEP_RII *ctx = (NEP_RII*)nep->data;
@@ -211,8 +194,6 @@ static PetscErrorCode NEPRIISetMaximumIterations_RII(NEP nep,PetscInt its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetMaximumIterations"
 /*@
    NEPRIISetMaximumIterations - Sets the maximum number of inner iterations to be
    used in the RII solver. These are the Newton iterations related to the computation
@@ -239,8 +220,6 @@ PetscErrorCode NEPRIISetMaximumIterations(NEP nep,PetscInt its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetMaximumIterations_RII"
 static PetscErrorCode NEPRIIGetMaximumIterations_RII(NEP nep,PetscInt *its)
 {
   NEP_RII *ctx = (NEP_RII*)nep->data;
@@ -250,8 +229,6 @@ static PetscErrorCode NEPRIIGetMaximumIterations_RII(NEP nep,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetMaximumIterations"
 /*@
    NEPRIIGetMaximumIterations - Gets the maximum number of inner iterations of RII.
 
@@ -278,8 +255,6 @@ PetscErrorCode NEPRIIGetMaximumIterations(NEP nep,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetLagPreconditioner_RII"
 static PetscErrorCode NEPRIISetLagPreconditioner_RII(NEP nep,PetscInt lag)
 {
   NEP_RII *ctx = (NEP_RII*)nep->data;
@@ -290,8 +265,6 @@ static PetscErrorCode NEPRIISetLagPreconditioner_RII(NEP nep,PetscInt lag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetLagPreconditioner"
 /*@
    NEPRIISetLagPreconditioner - Determines when the preconditioner is rebuilt in the
    nonlinear solve.
@@ -326,8 +299,6 @@ PetscErrorCode NEPRIISetLagPreconditioner(NEP nep,PetscInt lag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetLagPreconditioner_RII"
 static PetscErrorCode NEPRIIGetLagPreconditioner_RII(NEP nep,PetscInt *lag)
 {
   NEP_RII *ctx = (NEP_RII*)nep->data;
@@ -337,8 +308,6 @@ static PetscErrorCode NEPRIIGetLagPreconditioner_RII(NEP nep,PetscInt *lag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetLagPreconditioner"
 /*@
    NEPRIIGetLagPreconditioner - Indicates how often the preconditioner is rebuilt.
 
@@ -365,8 +334,6 @@ PetscErrorCode NEPRIIGetLagPreconditioner(NEP nep,PetscInt *lag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetConstCorrectionTol_RII"
 static PetscErrorCode NEPRIISetConstCorrectionTol_RII(NEP nep,PetscBool cct)
 {
   NEP_RII *ctx = (NEP_RII*)nep->data;
@@ -376,8 +343,6 @@ static PetscErrorCode NEPRIISetConstCorrectionTol_RII(NEP nep,PetscBool cct)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetConstCorrectionTol"
 /*@
    NEPRIISetConstCorrectionTol - Sets a flag to keep the tolerance used
    in the linear solver constant.
@@ -412,8 +377,6 @@ PetscErrorCode NEPRIISetConstCorrectionTol(NEP nep,PetscBool cct)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetConstCorrectionTol_RII"
 static PetscErrorCode NEPRIIGetConstCorrectionTol_RII(NEP nep,PetscBool *cct)
 {
   NEP_RII *ctx = (NEP_RII*)nep->data;
@@ -423,8 +386,6 @@ static PetscErrorCode NEPRIIGetConstCorrectionTol_RII(NEP nep,PetscBool *cct)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetConstCorrectionTol"
 /*@
    NEPRIIGetConstCorrectionTol - Returns the constant tolerance flag.
 
@@ -451,8 +412,6 @@ PetscErrorCode NEPRIIGetConstCorrectionTol(NEP nep,PetscBool *cct)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetKSP_RII"
 static PetscErrorCode NEPRIISetKSP_RII(NEP nep,KSP ksp)
 {
   PetscErrorCode ierr;
@@ -467,8 +426,6 @@ static PetscErrorCode NEPRIISetKSP_RII(NEP nep,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIISetKSP"
 /*@
    NEPRIISetKSP - Associate a linear solver object (KSP) to the nonlinear
    eigenvalue solver.
@@ -495,8 +452,6 @@ PetscErrorCode NEPRIISetKSP(NEP nep,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetKSP_RII"
 static PetscErrorCode NEPRIIGetKSP_RII(NEP nep,KSP *ksp)
 {
   PetscErrorCode ierr;
@@ -510,13 +465,12 @@ static PetscErrorCode NEPRIIGetKSP_RII(NEP nep,KSP *ksp)
     ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->ksp,(PetscObject)nep,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->ksp);CHKERRQ(ierr);
     ierr = KSPSetErrorIfNotConverged(ctx->ksp,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(ctx->ksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
   }
   *ksp = ctx->ksp;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRIIGetKSP"
 /*@
    NEPRIIGetKSP - Retrieve the linear solver object (KSP) associated with
    the nonlinear eigenvalue solver.
@@ -544,8 +498,6 @@ PetscErrorCode NEPRIIGetKSP(NEP nep,KSP *ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPView_RII"
 PetscErrorCode NEPView_RII(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -556,12 +508,12 @@ PetscErrorCode NEPView_RII(NEP nep,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     if (!ctx->ksp) { ierr = NEPRIIGetKSP(nep,&ctx->ksp);CHKERRQ(ierr); }
-    ierr = PetscViewerASCIIPrintf(viewer,"  RII: maximum number of inner iterations: %D\n",ctx->max_inner_it);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of inner iterations: %D\n",ctx->max_inner_it);CHKERRQ(ierr);
     if (ctx->cctol) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  RII: using a constant tolerance for the linear solver\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  using a constant tolerance for the linear solver\n");CHKERRQ(ierr);
     }
     if (ctx->lag) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  RII: updating the preconditioner every %D iterations\n",ctx->lag);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  updating the preconditioner every %D iterations\n",ctx->lag);CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = KSPView(ctx->ksp,viewer);CHKERRQ(ierr);
@@ -570,8 +522,16 @@ PetscErrorCode NEPView_RII(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPDestroy_RII"
+PetscErrorCode NEPReset_RII(NEP nep)
+{
+  PetscErrorCode ierr;
+  NEP_RII        *ctx = (NEP_RII*)nep->data;
+
+  PetscFunctionBegin;
+  ierr = KSPReset(ctx->ksp);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode NEPDestroy_RII(NEP nep)
 {
   PetscErrorCode ierr;
@@ -591,8 +551,6 @@ PetscErrorCode NEPDestroy_RII(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCreate_RII"
 PETSC_EXTERN PetscErrorCode NEPCreate_RII(NEP nep)
 {
   PetscErrorCode ierr;
@@ -600,16 +558,18 @@ PETSC_EXTERN PetscErrorCode NEPCreate_RII(NEP nep)
 
   PetscFunctionBegin;
   ierr = PetscNewLog(nep,&ctx);CHKERRQ(ierr);
+  nep->data = (void*)ctx;
   ctx->max_inner_it = 10;
   ctx->lag          = 1;
   ctx->cctol        = PETSC_FALSE;
-  nep->data = (void*)ctx;
 
   nep->ops->solve          = NEPSolve_RII;
   nep->ops->setup          = NEPSetUp_RII;
   nep->ops->setfromoptions = NEPSetFromOptions_RII;
+  nep->ops->reset          = NEPReset_RII;
   nep->ops->destroy        = NEPDestroy_RII;
   nep->ops->view           = NEPView_RII;
+
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPRIISetMaximumIterations_C",NEPRIISetMaximumIterations_RII);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPRIIGetMaximumIterations_C",NEPRIIGetMaximumIterations_RII);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPRIISetLagPreconditioner_C",NEPRIISetLagPreconditioner_RII);CHKERRQ(ierr);
