@@ -1,25 +1,15 @@
 /*
-   Basic NEP routines, Create, View, etc.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic NEP routines
+*/
 
 #include <slepc/private/nepimpl.h>      /*I "slepcnep.h" I*/
 
@@ -28,8 +18,6 @@ PetscBool         NEPRegisterAllCalled = PETSC_FALSE;
 PetscClassId      NEP_CLASSID = 0;
 PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_DerivativesEval = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPCreate"
 /*@
    NEPCreate - Creates the default NEP context.
 
@@ -66,6 +54,7 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->conv            = NEP_CONV_REL;
   nep->stop            = NEP_STOP_BASIC;
   nep->which           = (NEPWhich)0;
+  nep->problem_type    = (NEPProblemType)0;
   nep->refine          = NEP_REFINE_NONE;
   nep->npart           = 1;
   nep->rtol            = PETSC_DEFAULT;
@@ -80,8 +69,10 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->computederivatives = NULL;
   nep->derivativesctx  = NULL;
   nep->converged       = NEPConvergedRelative;
+  nep->convergeduser   = NULL;
   nep->convergeddestroy= NULL;
   nep->stopping        = NEPStoppingBasic;
+  nep->stoppinguser    = NULL;
   nep->stoppingdestroy = NULL;
   nep->convergedctx    = NULL;
   nep->stoppingctx     = NULL;
@@ -121,8 +112,6 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetType"
 /*@C
    NEPSetType - Selects the particular solver to be used in the NEP object.
 
@@ -175,8 +164,6 @@ PetscErrorCode NEPSetType(NEP nep,NEPType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetType"
 /*@C
    NEPGetType - Gets the NEP type as a string from the NEP object.
 
@@ -201,8 +188,6 @@ PetscErrorCode NEPGetType(NEP nep,NEPType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRegister"
 /*@C
    NEPRegister - Adds a method to the nonlinear eigenproblem solver package.
 
@@ -238,8 +223,6 @@ PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPReset_Problem"
 /*
    NEPReset_Problem - Destroys the problem matrices.
 @*/
@@ -261,14 +244,13 @@ PetscErrorCode NEPReset_Problem(NEP nep)
     }
     ierr = PetscFree(nep->f);CHKERRQ(ierr);
     ierr = PetscFree(nep->nrma);CHKERRQ(ierr);
+    nep->nt = 0;
   }
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "NEPReset"
 /*@
-   NEPReset - Resets the NEP context to the initial state and removes any
-   allocated objects.
+   NEPReset - Resets the NEP context to the initial state (prior to setup)
+   and destroys any allocated Vecs and Mats.
 
    Collective on NEP
 
@@ -282,28 +264,20 @@ PetscErrorCode NEPReset_Problem(NEP nep)
 PetscErrorCode NEPReset(NEP nep)
 {
   PetscErrorCode ierr;
-  PetscInt       ncols;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  if (nep) PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  if (!nep) PetscFunctionReturn(0);
   if (nep->ops->reset) { ierr = (nep->ops->reset)(nep);CHKERRQ(ierr); }
-  if (nep->ds) { ierr = DSReset(nep->ds);CHKERRQ(ierr); }
+  if (nep->refineksp) { ierr = KSPReset(nep->refineksp);CHKERRQ(ierr); }
   ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
-  ierr = BVGetSizes(nep->V,NULL,NULL,&ncols);CHKERRQ(ierr);
-  if (ncols) {
-    ierr = PetscFree4(nep->eigr,nep->eigi,nep->errest,nep->perm);CHKERRQ(ierr);
-  }
   ierr = BVDestroy(&nep->V);CHKERRQ(ierr);
   ierr = VecDestroyVecs(nep->nwork,&nep->work);CHKERRQ(ierr);
-  ierr = KSPDestroy(&nep->refineksp);CHKERRQ(ierr);
-  ierr = PetscSubcommDestroy(&nep->refinesubc);CHKERRQ(ierr);
   nep->nwork = 0;
   nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPDestroy"
 /*@
    NEPDestroy - Destroys the NEP context.
 
@@ -326,8 +300,13 @@ PetscErrorCode NEPDestroy(NEP *nep)
   if (--((PetscObject)(*nep))->refct > 0) { *nep = 0; PetscFunctionReturn(0); }
   ierr = NEPReset(*nep);CHKERRQ(ierr);
   if ((*nep)->ops->destroy) { ierr = (*(*nep)->ops->destroy)(*nep);CHKERRQ(ierr); }
+  if ((*nep)->eigr) {
+    ierr = PetscFree4((*nep)->eigr,(*nep)->eigi,(*nep)->errest,(*nep)->perm);CHKERRQ(ierr);
+  }
   ierr = RGDestroy(&(*nep)->rg);CHKERRQ(ierr);
   ierr = DSDestroy(&(*nep)->ds);CHKERRQ(ierr);
+  ierr = KSPDestroy(&(*nep)->refineksp);CHKERRQ(ierr);
+  ierr = PetscSubcommDestroy(&(*nep)->refinesubc);CHKERRQ(ierr);
   ierr = PetscFree((*nep)->sc);CHKERRQ(ierr);
   /* just in case the initial vectors have not been used */
   ierr = SlepcBasisDestroy_Private(&(*nep)->nini,&(*nep)->IS);CHKERRQ(ierr);
@@ -339,8 +318,6 @@ PetscErrorCode NEPDestroy(NEP *nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetBV"
 /*@
    NEPSetBV - Associates a basis vectors object to the nonlinear eigensolver.
 
@@ -373,8 +350,6 @@ PetscErrorCode NEPSetBV(NEP nep,BV bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetBV"
 /*@
    NEPGetBV - Obtain the basis vectors object associated to the nonlinear
    eigensolver object.
@@ -406,8 +381,6 @@ PetscErrorCode NEPGetBV(NEP nep,BV *bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetRG"
 /*@
    NEPSetRG - Associates a region object to the nonlinear eigensolver.
 
@@ -440,8 +413,6 @@ PetscErrorCode NEPSetRG(NEP nep,RG rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetRG"
 /*@
    NEPGetRG - Obtain the region object associated to the
    nonlinear eigensolver object.
@@ -473,8 +444,6 @@ PetscErrorCode NEPGetRG(NEP nep,RG *rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetDS"
 /*@
    NEPSetDS - Associates a direct solver object to the nonlinear eigensolver.
 
@@ -507,8 +476,6 @@ PetscErrorCode NEPSetDS(NEP nep,DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetDS"
 /*@
    NEPGetDS - Obtain the direct solver object associated to the
    nonlinear eigensolver object.
@@ -540,8 +507,6 @@ PetscErrorCode NEPGetDS(NEP nep,DS *ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPRefineGetKSP"
 /*@
    NEPRefineGetKSP - Obtain the ksp object used by the eigensolver
    object in the refinement phase.
@@ -577,14 +542,12 @@ PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->refineksp);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)nep)->prefix);CHKERRQ(ierr);
     ierr = KSPAppendOptionsPrefix(*ksp,"nep_refine_");CHKERRQ(ierr);
-    ierr = KSPSetErrorIfNotConverged(*ksp,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(nep->refineksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
   }
   *ksp = nep->refineksp;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetTarget"
 /*@
    NEPSetTarget - Sets the value of the target.
 
@@ -618,8 +581,6 @@ PetscErrorCode NEPSetTarget(NEP nep,PetscScalar target)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetTarget"
 /*@
    NEPGetTarget - Gets the value of the target.
 
@@ -647,8 +608,6 @@ PetscErrorCode NEPGetTarget(NEP nep,PetscScalar* target)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetFunction"
 /*@C
    NEPSetFunction - Sets the function to compute the nonlinear Function T(lambda)
    as well as the location to store the matrix.
@@ -689,9 +648,8 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
   if (A) PetscCheckSameComm(nep,1,A,2);
   if (B) PetscCheckSameComm(nep,1,B,3);
 
-  if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) {  /* clean previous user info */
-    ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
-  }
+  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
+  else if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
 
   if (fun) nep->computefunction = fun;
   if (ctx) nep->functionctx     = ctx;
@@ -705,12 +663,11 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
     ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
     nep->function_pre = B;
   }
-  nep->fui = NEP_USER_INTERFACE_CALLBACK;
+  nep->fui   = NEP_USER_INTERFACE_CALLBACK;
+  nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetFunction"
 /*@C
    NEPGetFunction - Returns the Function matrix and optionally the user
    provided context for evaluating the Function.
@@ -742,8 +699,6 @@ PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetJacobian"
 /*@C
    NEPSetJacobian - Sets the function to compute Jacobian T'(lambda) as well
    as the location to store the matrix.
@@ -780,9 +735,8 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
   if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
   if (A) PetscCheckSameComm(nep,1,A,2);
 
-  if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) {  /* clean previous user info */
-    ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
-  }
+  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
+  else if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
 
   if (jac) nep->computejacobian = jac;
   if (ctx) nep->jacobianctx     = ctx;
@@ -791,12 +745,11 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
     ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
     nep->jacobian = A;
   }
-  nep->fui = NEP_USER_INTERFACE_CALLBACK;
+  nep->fui   = NEP_USER_INTERFACE_CALLBACK;
+  nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetJacobian"
 /*@C
    NEPGetJacobian - Returns the Jacobian matrix and optionally the user
    provided routine and context for evaluating the Jacobian.
@@ -826,8 +779,6 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetSplitOperator"
 /*@
    NEPSetSplitOperator - Sets the operator of the nonlinear eigenvalue problem
    in split form.
@@ -867,14 +818,14 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt n,Mat A[],FN f[],MatStructur
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,n,2);
-  if (n <= 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more terms, you have %D",n);
+  if (n <= 0) SETERRQ1(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more terms, you have %D",n);
   PetscValidPointer(A,3);
   PetscCheckSameComm(nep,1,*A,3);
   PetscValidPointer(f,4);
   PetscCheckSameComm(nep,1,*f,4);
   if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
-  /* clean previously stored information */
-  ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
+  else { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
+
   /* allocate space and copy matrices and functions */
   ierr = PetscMalloc1(n,&nep->A);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(Mat));CHKERRQ(ierr);
@@ -892,14 +843,13 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt n,Mat A[],FN f[],MatStructur
   }
   ierr = PetscCalloc1(n,&nep->nrma);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(PetscReal));CHKERRQ(ierr);
-  nep->nt   = n;
-  nep->mstr = str;
-  nep->fui  = NEP_USER_INTERFACE_SPLIT;
+  nep->nt    = n;
+  nep->mstr  = str;
+  nep->fui   = NEP_USER_INTERFACE_SPLIT;
+  nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetSplitOperatorTerm"
 /*@
    NEPGetSplitOperatorTerm - Gets the matrices and functions associated with
    the nonlinear operator in split form.
@@ -923,14 +873,12 @@ PetscErrorCode NEPGetSplitOperatorTerm(NEP nep,PetscInt k,Mat *A,FN *f)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   NEPCheckSplit(nep,1);
-  if (k<0 || k>=nep->nt) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",nep->nt-1);
+  if (k<0 || k>=nep->nt) SETERRQ1(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",nep->nt-1);
   if (A) *A = nep->A[k];
   if (f) *f = nep->f[k];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetSplitOperatorInfo"
 /*@
    NEPGetSplitOperatorInfo - Returns the number of terms of the split form of
    the nonlinear operator, as well as the structure flag for matrices.
@@ -958,8 +906,6 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetDerivatives"
 /*@C
    NEPSetDerivatives - Sets the function to compute the k-th derivative T^(k)(lambda)
    for any value of k (including 0), as well as the location to store the matrix.
@@ -988,9 +934,8 @@ PetscErrorCode NEPSetDerivatives(NEP nep,Mat A,PetscErrorCode (*der)(NEP,PetscSc
   if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
   if (A) PetscCheckSameComm(nep,1,A,2);
 
-  if (nep->fui && nep->fui!=NEP_USER_INTERFACE_DERIVATIVES) {  /* clean previous user info */
-    ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
-  }
+  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
+  else { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
 
   if (der) nep->computederivatives = der;
   if (ctx) nep->derivativesctx     = ctx;
@@ -999,12 +944,11 @@ PetscErrorCode NEPSetDerivatives(NEP nep,Mat A,PetscErrorCode (*der)(NEP,PetscSc
     ierr = MatDestroy(&nep->derivatives);CHKERRQ(ierr);
     nep->derivatives = A;
   }
-  nep->fui = NEP_USER_INTERFACE_DERIVATIVES;
+  nep->fui   = NEP_USER_INTERFACE_DERIVATIVES;
+  nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetDerivatives"
 /*@C
    NEPGetDerivatives - Returns the derivatives matrix and optionally the user
    provided routine and context for evaluating the derivatives.
