@@ -1,31 +1,19 @@
 /*
-      NEP routines related to monitors.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   NEP routines related to monitors
+*/
 
 #include <slepc/private/nepimpl.h>      /*I "slepcnep.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitor"
 /*
    Runs the user provided monitor routines, if any.
 */
@@ -41,8 +29,6 @@ PetscErrorCode NEPMonitor(NEP nep,PetscInt it,PetscInt nconv,PetscScalar *eigr,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorSet"
 /*@C
    NEPMonitorSet - Sets an ADDITIONAL function to be called at every
    iteration to monitor the error estimates for each requested eigenpair.
@@ -102,8 +88,6 @@ PetscErrorCode NEPMonitorSet(NEP nep,PetscErrorCode (*monitor)(NEP,PetscInt,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorCancel"
 /*@
    NEPMonitorCancel - Clears all monitors for a NEP object.
 
@@ -137,8 +121,6 @@ PetscErrorCode NEPMonitorCancel(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetMonitorContext"
 /*@C
    NEPGetMonitorContext - Gets the monitor context, as set by
    NEPMonitorSet() for the FIRST monitor only.
@@ -163,8 +145,6 @@ PetscErrorCode NEPGetMonitorContext(NEP nep,void **ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorAll"
 /*@C
    NEPMonitorAll - Print the current approximate values and
    error estimates at each iteration of the nonlinear eigensolver.
@@ -219,8 +199,6 @@ PetscErrorCode NEPMonitorAll(NEP nep,PetscInt its,PetscInt nconv,PetscScalar *ei
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorFirst"
 /*@C
    NEPMonitorFirst - Print the first unconverged approximate value and
    error estimate at each iteration of the nonlinear eigensolver.
@@ -273,8 +251,6 @@ PetscErrorCode NEPMonitorFirst(NEP nep,PetscInt its,PetscInt nconv,PetscScalar *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorConverged"
 /*@C
    NEPMonitorConverged - Print the approximate values and
    error estimates as they converge.
@@ -332,8 +308,6 @@ PetscErrorCode NEPMonitorConverged(NEP nep,PetscInt its,PetscInt nconv,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorLGCreate"
 /*@C
    NEPMonitorLGCreate - Creates a line graph context for use with
    NEP to monitor convergence.
@@ -378,8 +352,6 @@ PetscErrorCode NEPMonitorLGCreate(MPI_Comm comm,const char host[],const char lab
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorLG"
 PetscErrorCode NEPMonitorLG(NEP nep,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
   PetscDrawLG    lg = (PetscDrawLG)ctx;
@@ -404,8 +376,6 @@ PetscErrorCode NEPMonitorLG(NEP nep,PetscInt its,PetscInt nconv,PetscScalar *eig
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorLGAll"
 PetscErrorCode NEPMonitorLGAll(NEP nep,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
   PetscDrawLG    lg = (PetscDrawLG)ctx;
