@@ -1,32 +1,20 @@
 /*
-      NEP routines related to options that can be set via the command-line
-      or procedurally.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   NEP routines related to options that can be set via the command-line
+   or procedurally
+*/
 
 #include <slepc/private/nepimpl.h>       /*I "slepcnep.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPMonitorSetFromOptions"
 /*@C
    NEPMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
@@ -66,8 +54,6 @@ PetscErrorCode NEPMonitorSetFromOptions(NEP nep,const char name[],const char hel
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPConvMonitorSetFromOptions"
 /*@C
    NEPConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
@@ -103,8 +89,6 @@ PetscErrorCode NEPConvMonitorSetFromOptions(NEP nep,const char name[],const char
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetFromOptions"
 /*@
    NEPSetFromOptions - Sets NEP options from the options database.
    This routine must be called before NEPSetUp() if the user is to be
@@ -122,46 +106,49 @@ PetscErrorCode NEPConvMonitorSetFromOptions(NEP nep,const char name[],const char
 @*/
 PetscErrorCode NEPSetFromOptions(NEP nep)
 {
-  PetscErrorCode ierr;
-  char           type[256];
-  PetscBool      set,flg,flg1,flg2,flg3;
-  PetscReal      r;
-  PetscScalar    s;
-  PetscInt       i,j,k;
-  PetscDrawLG    lg;
+  PetscErrorCode  ierr;
+  char            type[256];
+  PetscBool       set,flg,flg1,flg2,flg3,flg4,flg5;
+  PetscReal       r;
+  PetscScalar     s;
+  PetscInt        i,j,k;
+  PetscDrawLG     lg;
+  NEPRefine       refine;
+  NEPRefineScheme scheme;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   ierr = NEPRegisterAll();CHKERRQ(ierr);
   ierr = PetscObjectOptionsBegin((PetscObject)nep);CHKERRQ(ierr);
-    ierr = PetscOptionsFList("-nep_type","Nonlinear Eigenvalue Problem method","NEPSetType",NEPList,(char*)(((PetscObject)nep)->type_name?((PetscObject)nep)->type_name:NEPRII),type,256,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsFList("-nep_type","Nonlinear eigensolver method","NEPSetType",NEPList,(char*)(((PetscObject)nep)->type_name?((PetscObject)nep)->type_name:NEPRII),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = NEPSetType(nep,type);CHKERRQ(ierr);
     } else if (!((PetscObject)nep)->type_name) {
       ierr = NEPSetType(nep,NEPRII);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsEnum("-nep_refine","Iterative refinement method","NEPSetRefine",NEPRefineTypes,(PetscEnum)nep->refine,(PetscEnum*)&nep->refine,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupBegin("-nep_general","General nonlinear eigenvalue problem","NEPSetProblemType",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetProblemType(nep,NEP_GENERAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroupEnd("-nep_rational","Rational eigenvalue problem","NEPSetProblemType",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetProblemType(nep,NEP_RATIONAL);CHKERRQ(ierr); }
 
+    refine = nep->refine;
+    ierr = PetscOptionsEnum("-nep_refine","Iterative refinement method","NEPSetRefine",NEPRefineTypes,(PetscEnum)refine,(PetscEnum*)&refine,&flg1);CHKERRQ(ierr);
     i = nep->npart;
-    ierr = PetscOptionsInt("-nep_refine_partitions","Number of partitions of the communicator for iterative refinement","NEPSetRefine",nep->npart,&i,&flg1);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_refine_partitions","Number of partitions of the communicator for iterative refinement","NEPSetRefine",nep->npart,&i,&flg2);CHKERRQ(ierr);
     r = nep->rtol;
-    ierr = PetscOptionsReal("-nep_refine_tol","Tolerance for iterative refinement","NEPSetRefine",nep->rtol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/1000:nep->rtol,&r,&flg2);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-nep_refine_tol","Tolerance for iterative refinement","NEPSetRefine",nep->rtol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/1000:nep->rtol,&r,&flg3);CHKERRQ(ierr);
     j = nep->rits;
-    ierr = PetscOptionsInt("-nep_refine_its","Maximum number of iterations for iterative refinement","NEPSetRefine",nep->rits,&j,&flg3);CHKERRQ(ierr);
-    if (flg1 || flg2 || flg3) {
-      ierr = NEPSetRefine(nep,nep->refine,i,r,j,nep->scheme);CHKERRQ(ierr);
-    }
-
-    ierr = PetscOptionsEnum("-nep_refine_scheme","Scheme used for linear systems within iterative refinement","NEPSetRefine",NEPRefineSchemes,(PetscEnum)nep->scheme,(PetscEnum*)&nep->scheme,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-nep_refine_its","Maximum number of iterations for iterative refinement","NEPSetRefine",nep->rits,&j,&flg4);CHKERRQ(ierr);
+    scheme = nep->scheme;
+    ierr = PetscOptionsEnum("-nep_refine_scheme","Scheme used for linear systems within iterative refinement","NEPSetRefine",NEPRefineSchemes,(PetscEnum)scheme,(PetscEnum*)&scheme,&flg5);CHKERRQ(ierr);
+    if (flg1 || flg2 || flg3 || flg4 || flg5) { ierr = NEPSetRefine(nep,refine,i,r,j,scheme);CHKERRQ(ierr); }
 
     i = nep->max_it? nep->max_it: PETSC_DEFAULT;
     ierr = PetscOptionsInt("-nep_max_it","Maximum number of iterations","NEPSetTolerances",nep->max_it,&i,&flg1);CHKERRQ(ierr);
     r = nep->tol;
     ierr = PetscOptionsReal("-nep_tol","Tolerance","NEPSetTolerances",nep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:nep->tol,&r,&flg2);CHKERRQ(ierr);
-    if (flg1 || flg2) {
-      ierr = NEPSetTolerances(nep,r,i);CHKERRQ(ierr);
-    }
+    if (flg1 || flg2) { ierr = NEPSetTolerances(nep,r,i);CHKERRQ(ierr); }
 
     ierr = PetscOptionsBoolGroupBegin("-nep_conv_rel","Relative error convergence test","NEPSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) { ierr = NEPSetConvergenceTest(nep,NEP_CONV_REL);CHKERRQ(ierr); }
@@ -187,9 +174,32 @@ PetscErrorCode NEPSetFromOptions(NEP nep)
       ierr = NEPSetDimensions(nep,i,j,k);CHKERRQ(ierr);
     }
 
+    ierr = PetscOptionsBoolGroupBegin("-nep_largest_magnitude","Compute largest eigenvalues in magnitude","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_LARGEST_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_smallest_magnitude","Compute smallest eigenvalues in magnitude","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_largest_real","Compute eigenvalues with largest real parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_LARGEST_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_smallest_real","Compute eigenvalues with smallest real parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_SMALLEST_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_largest_imaginary","Compute eigenvalues with largest imaginary parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_LARGEST_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_smallest_imaginary","Compute eigenvalues with smallest imaginary parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_SMALLEST_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_target_magnitude","Compute eigenvalues closest to target","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_target_real","Compute eigenvalues with real parts closest to target","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-nep_target_imaginary","Compute eigenvalues with imaginary parts closest to target","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroupEnd("-nep_all","Compute all eigenvalues in a region","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_ALL);CHKERRQ(ierr); }
+
     ierr = PetscOptionsScalar("-nep_target","Value of the target","NEPSetTarget",nep->target,&s,&flg);CHKERRQ(ierr);
     if (flg) {
-      ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
+      if (nep->which!=NEP_TARGET_REAL && nep->which!=NEP_TARGET_IMAGINARY) {
+        ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
+      }
       ierr = NEPSetTarget(nep,s);CHKERRQ(ierr);
     }
 
@@ -221,29 +231,8 @@ PetscErrorCode NEPSetFromOptions(NEP nep)
       ierr = NEPMonitorSet(nep,NEPMonitorLGAll,lg,(PetscErrorCode (*)(void**))PetscDrawLGDestroy);CHKERRQ(ierr);
       ierr = NEPSetTrackAll(nep,PETSC_TRUE);CHKERRQ(ierr);
     }
-  /* -----------------------------------------------------------------------*/
-
-    ierr = PetscOptionsBoolGroupBegin("-nep_largest_magnitude","compute largest eigenvalues in magnitude","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_LARGEST_MAGNITUDE);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_smallest_magnitude","compute smallest eigenvalues in magnitude","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_largest_real","compute largest real parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_LARGEST_REAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_smallest_real","compute smallest real parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_SMALLEST_REAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_largest_imaginary","compute largest imaginary parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_LARGEST_IMAGINARY);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_smallest_imaginary","compute smallest imaginary parts","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_SMALLEST_IMAGINARY);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_target_magnitude","compute nearest eigenvalues to target","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_MAGNITUDE);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_target_real","compute eigenvalues with real parts close to target","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_REAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-nep_target_imaginary","compute eigenvalues with imaginary parts close to target","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_TARGET_IMAGINARY);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroupEnd("-nep_all","compute all eigenvalues in a region","NEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = NEPSetWhichEigenpairs(nep,NEP_ALL);CHKERRQ(ierr); }
 
+    /* -----------------------------------------------------------------------*/
     ierr = PetscOptionsName("-nep_view","Print detailed information on solver used","NEPView",NULL);CHKERRQ(ierr);
     ierr = PetscOptionsName("-nep_view_vectors","View computed eigenvectors","NEPVectorsView",NULL);CHKERRQ(ierr);
     ierr = PetscOptionsName("-nep_view_values","View computed eigenvalues","NEPValuesView",NULL);CHKERRQ(ierr);
@@ -265,12 +254,11 @@ PetscErrorCode NEPSetFromOptions(NEP nep)
   ierr = DSSetFromOptions(nep->ds);CHKERRQ(ierr);
   if (!nep->refineksp) { ierr = NEPRefineGetKSP(nep,&nep->refineksp);CHKERRQ(ierr); }
   ierr = KSPSetFromOptions(nep->refineksp);CHKERRQ(ierr);
+  if (nep->fui==NEP_USER_INTERFACE_SPLIT) for (i=0;i<nep->nt;i++) {ierr = FNSetFromOptions(nep->f[i]);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetTolerances"
-/*@
+/*@C
    NEPGetTolerances - Gets the tolerance and maximum iteration count used
    by the NEP convergence tests.
 
@@ -299,8 +287,6 @@ PetscErrorCode NEPGetTolerances(NEP nep,PetscReal *tol,PetscInt *maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetTolerances"
 /*@
    NEPSetTolerances - Sets the tolerance and maximum iteration count used
    by the NEP convergence tests.
@@ -346,9 +332,7 @@ PetscErrorCode NEPSetTolerances(NEP nep,PetscReal tol,PetscInt maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetDimensions"
-/*@
+/*@C
    NEPGetDimensions - Gets the number of eigenvalues to compute
    and the dimension of the subspace.
 
@@ -379,8 +363,6 @@ PetscErrorCode NEPGetDimensions(NEP nep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetDimensions"
 /*@
    NEPSetDimensions - Sets the number of eigenvalues to compute
    and the dimension of the subspace.
@@ -440,8 +422,6 @@ PetscErrorCode NEPSetDimensions(NEP nep,PetscInt nev,PetscInt ncv,PetscInt mpd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetWhichEigenpairs"
 /*@
     NEPSetWhichEigenpairs - Specifies which portion of the spectrum is
     to be sought.
@@ -524,8 +504,6 @@ PetscErrorCode NEPSetWhichEigenpairs(NEP nep,NEPWhich which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetWhichEigenpairs"
 /*@
     NEPGetWhichEigenpairs - Returns which portion of the spectrum is to be
     sought.
@@ -554,8 +532,6 @@ PetscErrorCode NEPGetWhichEigenpairs(NEP nep,NEPWhich *which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetEigenvalueComparison"
 /*@C
    NEPSetEigenvalueComparison - Specifies the eigenvalue comparison function
    when NEPSetWhichEigenpairs() is set to NEP_WHICH_USER.
@@ -563,7 +539,7 @@ PetscErrorCode NEPGetWhichEigenpairs(NEP nep,NEPWhich *which)
    Logically Collective on NEP
 
    Input Parameters:
-+  pep  - eigensolver context obtained from NEPCreate()
++  nep  - eigensolver context obtained from NEPCreate()
 .  func - a pointer to the comparison function
 -  ctx  - a context pointer (the last parameter to the comparison function)
 
@@ -587,18 +563,78 @@ $   func(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *r
 
 .seealso: NEPSetWhichEigenpairs(), NEPWhich
 @*/
-PetscErrorCode NEPSetEigenvalueComparison(NEP pep,PetscErrorCode (*func)(PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*),void* ctx)
+PetscErrorCode NEPSetEigenvalueComparison(NEP nep,PetscErrorCode (*func)(PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*),void* ctx)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pep,NEP_CLASSID,1);
-  pep->sc->comparison    = func;
-  pep->sc->comparisonctx = ctx;
-  pep->which             = NEP_WHICH_USER;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  nep->sc->comparison    = func;
+  nep->sc->comparisonctx = ctx;
+  nep->which             = NEP_WHICH_USER;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   NEPSetProblemType - Specifies the type of the nonlinear eigenvalue problem.
+
+   Logically Collective on NEP
+
+   Input Parameters:
++  nep  - the nonlinear eigensolver context
+-  type - a known type of nonlinear eigenvalue problem
+
+   Options Database Keys:
++  -nep_general - general problem with no particular structure
+-  -nep_rational - a rational eigenvalue problem defined in split form with all f_i rational
+
+   Notes:
+   Allowed values for the problem type are: general (NEP_GENERAL), and rational
+   (NEP_RATIONAL).
+
+   This function is used to provide a hint to the NEP solver to exploit certain
+   properties of the nonlinear eigenproblem. This hint may be used or not,
+   depending on the solver. By default, no particular structure is assumed.
+
+   Level: intermediate
+
+.seealso: NEPSetType(), NEPGetProblemType(), NEPProblemType
+@*/
+PetscErrorCode NEPSetProblemType(NEP nep,NEPProblemType type)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(nep,type,2);
+  if (type!=NEP_GENERAL && type!=NEP_RATIONAL) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
+  if (type != nep->problem_type) {
+    nep->problem_type = type;
+    nep->state = NEP_STATE_INITIAL;
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   NEPGetProblemType - Gets the problem type from the NEP object.
+
+   Not Collective
+
+   Input Parameter:
+.  nep - the nonlinear eigensolver context
+
+   Output Parameter:
+.  type - the problem type
+
+   Level: intermediate
+
+.seealso: NEPSetProblemType(), NEPProblemType
+@*/
+PetscErrorCode NEPGetProblemType(NEP nep,NEPProblemType *type)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidPointer(type,2);
+  *type = nep->problem_type;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetConvergenceTestFunction"
 /*@C
    NEPSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
@@ -638,18 +674,19 @@ PetscErrorCode NEPSetConvergenceTestFunction(NEP nep,PetscErrorCode (*func)(NEP,
   if (nep->convergeddestroy) {
     ierr = (*nep->convergeddestroy)(nep->convergedctx);CHKERRQ(ierr);
   }
-  nep->converged        = func;
+  nep->convergeduser    = func;
   nep->convergeddestroy = destroy;
   nep->convergedctx     = ctx;
   if (func == NEPConvergedRelative) nep->conv = NEP_CONV_REL;
   else if (func == NEPConvergedNorm) nep->conv = NEP_CONV_NORM;
   else if (func == NEPConvergedAbsolute) nep->conv = NEP_CONV_ABS;
-  else nep->conv = NEP_CONV_USER;
+  else {
+    nep->conv      = NEP_CONV_USER;
+    nep->converged = nep->convergeduser;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetConvergenceTest"
 /*@
    NEPSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
@@ -685,7 +722,10 @@ PetscErrorCode NEPSetConvergenceTest(NEP nep,NEPConv conv)
     case NEP_CONV_ABS:  nep->converged = NEPConvergedAbsolute; break;
     case NEP_CONV_REL:  nep->converged = NEPConvergedRelative; break;
     case NEP_CONV_NORM: nep->converged = NEPConvergedNorm; break;
-    case NEP_CONV_USER: break;
+    case NEP_CONV_USER:
+      if (!nep->convergeduser) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"Must call NEPSetConvergenceTestFunction() first");
+      nep->converged = nep->convergeduser;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'conv' value");
   }
@@ -693,8 +733,6 @@ PetscErrorCode NEPSetConvergenceTest(NEP nep,NEPConv conv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetConvergenceTest"
 /*@
    NEPGetConvergenceTest - Gets the method used to compute the error estimate
    used in the convergence test.
@@ -720,8 +758,6 @@ PetscErrorCode NEPGetConvergenceTest(NEP nep,NEPConv *conv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetStoppingTestFunction"
 /*@C
    NEPSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the eigensolver.
@@ -764,16 +800,17 @@ PetscErrorCode NEPSetStoppingTestFunction(NEP nep,PetscErrorCode (*func)(NEP,Pet
   if (nep->stoppingdestroy) {
     ierr = (*nep->stoppingdestroy)(nep->stoppingctx);CHKERRQ(ierr);
   }
-  nep->stopping        = func;
+  nep->stoppinguser    = func;
   nep->stoppingdestroy = destroy;
   nep->stoppingctx     = ctx;
   if (func == NEPStoppingBasic) nep->stop = NEP_STOP_BASIC;
-  else nep->stop = NEP_STOP_USER;
+  else {
+    nep->stop     = NEP_STOP_USER;
+    nep->stopping = nep->stoppinguser;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetStoppingTest"
 /*@
    NEPSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the eigensolver.
@@ -804,7 +841,10 @@ PetscErrorCode NEPSetStoppingTest(NEP nep,NEPStop stop)
   PetscValidLogicalCollectiveEnum(nep,stop,2);
   switch (stop) {
     case NEP_STOP_BASIC: nep->stopping = NEPStoppingBasic; break;
-    case NEP_STOP_USER:  break;
+    case NEP_STOP_USER:
+      if (!nep->stoppinguser) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"Must call NEPSetStoppingTestFunction() first");
+      nep->stopping = nep->stoppinguser;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'stop' value");
   }
@@ -812,8 +852,6 @@ PetscErrorCode NEPSetStoppingTest(NEP nep,NEPStop stop)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetStoppingTest"
 /*@
    NEPGetStoppingTest - Gets the method used to decide the termination of the outer
    loop of the eigensolver.
@@ -839,8 +877,6 @@ PetscErrorCode NEPGetStoppingTest(NEP nep,NEPStop *stop)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetTrackAll"
 /*@
    NEPSetTrackAll - Specifies if the solver must compute the residual of all
    approximate eigenpairs or not.
@@ -872,8 +908,6 @@ PetscErrorCode NEPSetTrackAll(NEP nep,PetscBool trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetTrackAll"
 /*@
    NEPGetTrackAll - Returns the flag indicating whether all residual norms must
    be computed or not.
@@ -899,8 +933,6 @@ PetscErrorCode NEPGetTrackAll(NEP nep,PetscBool *trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetRefine"
 /*@
    NEPSetRefine - Specifies the refinement type (and options) to be used
    after the solve.
@@ -942,7 +974,7 @@ PetscErrorCode NEPGetTrackAll(NEP nep,PetscBool *trackall)
    refinement iterations (just one by default).
 
    The scheme argument is used to change the way in which linear systems are
-   solved. Possible choices are: explicit, mixed block elimination (MBE), 
+   solved. Possible choices are: explicit, mixed block elimination (MBE),
    and Schur complement.
 
    Level: intermediate
@@ -963,6 +995,10 @@ PetscErrorCode NEPSetRefine(NEP nep,NEPRefine refine,PetscInt npart,PetscReal to
   PetscValidLogicalCollectiveEnum(nep,scheme,6);
   nep->refine = refine;
   if (refine) {  /* process parameters only if not REFINE_NONE */
+    if (npart!=nep->npart) {
+      ierr = PetscSubcommDestroy(&nep->refinesubc);CHKERRQ(ierr);
+      ierr = KSPDestroy(&nep->refineksp);CHKERRQ(ierr);
+    }
     if (npart == PETSC_DEFAULT || npart == PETSC_DECIDE) {
       nep->npart = 1;
     } else {
@@ -971,7 +1007,7 @@ PetscErrorCode NEPSetRefine(NEP nep,NEPRefine refine,PetscInt npart,PetscReal to
       nep->npart = npart;
     }
     if (tol == PETSC_DEFAULT || tol == PETSC_DECIDE) {
-      nep->rtol = PetscMax(nep->tol/1000,PETSC_MACHINE_EPSILON);
+      nep->rtol = PETSC_DEFAULT;
     } else {
       if (tol<=0.0) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
       nep->rtol = tol;
@@ -988,9 +1024,7 @@ PetscErrorCode NEPSetRefine(NEP nep,NEPRefine refine,PetscInt npart,PetscReal to
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetRefine"
-/*@
+/*@C
    NEPGetRefine - Gets the refinement strategy used by the NEP object, and the
    associated parameters.
 
@@ -1025,8 +1059,6 @@ PetscErrorCode NEPGetRefine(NEP nep,NEPRefine *refine,PetscInt *npart,PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetOptionsPrefix"
 /*@C
    NEPSetOptionsPrefix - Sets the prefix used for searching for all
    NEP options in the database.
@@ -1069,8 +1101,6 @@ PetscErrorCode NEPSetOptionsPrefix(NEP nep,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPAppendOptionsPrefix"
 /*@C
    NEPAppendOptionsPrefix - Appends to the prefix used for searching for all
    NEP options in the database.
@@ -1096,17 +1126,15 @@ PetscErrorCode NEPAppendOptionsPrefix(NEP nep,const char *prefix)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (!nep->V) { ierr = NEPGetBV(nep,&nep->V);CHKERRQ(ierr); }
-  ierr = BVSetOptionsPrefix(nep->V,prefix);CHKERRQ(ierr);
+  ierr = BVAppendOptionsPrefix(nep->V,prefix);CHKERRQ(ierr);
   if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
-  ierr = DSSetOptionsPrefix(nep->ds,prefix);CHKERRQ(ierr);
+  ierr = DSAppendOptionsPrefix(nep->ds,prefix);CHKERRQ(ierr);
   if (!nep->rg) { ierr = NEPGetRG(nep,&nep->rg);CHKERRQ(ierr); }
-  ierr = RGSetOptionsPrefix(nep->rg,prefix);CHKERRQ(ierr);
+  ierr = RGAppendOptionsPrefix(nep->rg,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)nep,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetOptionsPrefix"
 /*@C
    NEPGetOptionsPrefix - Gets the prefix used for searching for all
    NEP options in the database.
