@@ -1,25 +1,15 @@
 /*
-   Newton refinement for NEP, simple version.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Newton refinement for NEP, simple version
+*/
 
 #include <slepc/private/nepimpl.h>
 #include <slepcblaslapack.h>
@@ -27,7 +17,6 @@
 #define NREF_MAXIT 10
 
 typedef struct {
-  PetscSubcomm  subc;
   VecScatter    *scatter_id,nst;
   Mat           *A;
   Vec           nv,vg,v,w;
@@ -40,14 +29,12 @@ typedef struct {
   PetscScalar  M4,m3;
 } FSubctx;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFSMult"
 static PetscErrorCode MatFSMult(Mat M ,Vec x,Vec y)
 {
   PetscErrorCode ierr;
   FSubctx        *ctx;
   PetscScalar    t;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(M,&ctx);CHKERRQ(ierr);
   ierr = VecDot(x,ctx->M3,&t);CHKERRQ(ierr);
@@ -57,8 +44,6 @@ static PetscErrorCode MatFSMult(Mat M ,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSimpleNRefSetUp"
 static PetscErrorCode NEPSimpleNRefSetUp(NEP nep,NEPSimpNRefctx **ctx_)
 {
   PetscErrorCode ierr;
@@ -74,19 +59,21 @@ static PetscErrorCode NEPSimpleNRefSetUp(NEP nep,NEPSimpNRefctx **ctx_)
   if (nep->npart==1) {
     ctx->A  = nep->A;
     ctx->fn = nep->f;
+    nep->refinesubc = NULL;
+    ctx->scatter_id = NULL;
   } else {
     ierr = PetscMalloc2(nep->nt,&ctx->A,nep->npart,&ctx->scatter_id);CHKERRQ(ierr);
 
     /* Duplicate matrices */
     for (i=0;i<nep->nt;i++) {
-      ierr = MatCreateRedundantMatrix(nep->A[i],0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&ctx->A[i]);CHKERRQ(ierr);
+      ierr = MatCreateRedundantMatrix(nep->A[i],0,PetscSubcommChild(nep->refinesubc),MAT_INITIAL_MATRIX,&ctx->A[i]);CHKERRQ(ierr);
     }
     ierr = MatCreateVecs(ctx->A[0],&ctx->v,NULL);CHKERRQ(ierr);
 
     /* Duplicate FNs */
     ierr = PetscMalloc1(nep->nt,&ctx->fn);CHKERRQ(ierr);
     for (i=0;i<nep->nt;i++) {
-      ierr = FNDuplicate(nep->f[i],PetscSubcommChild(ctx->subc),&ctx->fn[i]);CHKERRQ(ierr);
+      ierr = FNDuplicate(nep->f[i],PetscSubcommChild(nep->refinesubc),&ctx->fn[i]);CHKERRQ(ierr);
     }
 
     /* Create scatters for sending vectors to each subcommucator */
@@ -126,7 +113,7 @@ static PetscErrorCode NEPSimpleNRefSetUp(NEP nep,NEPSimpNRefctx **ctx_)
       for (i=n0;i<m0;i++) {
         idx1[i-n0] = i;
       }
-      ierr = ISCreateGeneral(PetscObjectComm((PetscObject)nep),(m0-n0),idx1,PETSC_COPY_VALUES,&is1);CHKERRQ(ierr);
+      ierr = ISCreateGeneral(PetscObjectComm((PetscObject)ctx->A[0]),(m0-n0),idx1,PETSC_COPY_VALUES,&is1);CHKERRQ(ierr);
       ierr = VecScatterCreate(v,is1,ctx->nv,is1,&ctx->nst);CHKERRQ(ierr);
       if (nep->npart==1) {
         ierr = BVRestoreColumn(nep->V,0,&v);CHKERRQ(ierr);
@@ -134,14 +121,12 @@ static PetscErrorCode NEPSimpleNRefSetUp(NEP nep,NEPSimpNRefctx **ctx_)
       ierr = PetscFree(idx1);CHKERRQ(ierr);
       ierr = ISDestroy(&is1);CHKERRQ(ierr);
     }
-  }  PetscFunctionReturn(0);  
+  }  PetscFunctionReturn(0);
 }
 
 /*
   Gather Eigenpair idx from subcommunicator with color sc
 */
-#undef __FUNCT__
-#define __FUNCT__ "NEPSimpleNRefGatherEigenpair"
 static PetscErrorCode NEPSimpleNRefGatherEigenpair(NEP nep,NEPSimpNRefctx *ctx,PetscInt sc,PetscInt idx,PetscInt *fail)
 {
   PetscErrorCode ierr;
@@ -151,23 +136,23 @@ static PetscErrorCode NEPSimpleNRefGatherEigenpair(NEP nep,NEPSimpNRefctx *ctx,P
   PetscScalar    *array;
 
   PetscFunctionBegin;
+  ierr = MPI_Comm_size(comm,&nproc);CHKERRQ(ierr);
+  p = (nproc/nep->npart)*(sc+1)+PetscMin(sc+1,nproc%nep->npart)-1;
   if (nep->npart>1) {
-    ierr = MPI_Comm_size(comm,&nproc);CHKERRQ(ierr);
-    p = (nproc/nep->npart)*sc+PetscMin(sc,nproc%nep->npart);
     /* Communicate convergence successful */
     ierr = MPI_Bcast(fail,1,MPIU_INT,p,comm);CHKERRQ(ierr);
     if (!(*fail)) {
-      /* Process 0 of subcommunicator sc broadcasts the eigenvalue */ 
+      /* Process 0 of subcommunicator sc broadcasts the eigenvalue */
       ierr = MPI_Bcast(&nep->eigr[idx],1,MPIU_SCALAR,p,comm);CHKERRQ(ierr);
       /* Gather nep->V[idx] from the subcommuniator sc */
       ierr = BVGetColumn(nep->V,idx,&v);CHKERRQ(ierr);
-      if (ctx->subc->color==sc) {
+      if (nep->refinesubc->color==sc) {
         ierr = VecGetArray(ctx->v,&array);CHKERRQ(ierr);
         ierr = VecPlaceArray(ctx->vg,array);CHKERRQ(ierr);
       }
       ierr = VecScatterBegin(ctx->scatter_id[sc],ctx->vg,v,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
       ierr = VecScatterEnd(ctx->scatter_id[sc],ctx->vg,v,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-      if (ctx->subc->color==sc) {
+      if (nep->refinesubc->color==sc) {
         ierr = VecResetArray(ctx->vg);CHKERRQ(ierr);
         ierr = VecRestoreArray(ctx->v,&array);CHKERRQ(ierr);
       }
@@ -175,32 +160,28 @@ static PetscErrorCode NEPSimpleNRefGatherEigenpair(NEP nep,NEPSimpNRefctx *ctx,P
     }
   } else {
     if (nep->scheme==NEP_REFINE_SCHEME_EXPLICIT && !(*fail)) {
-      ierr = MPI_Comm_size(comm,&nproc);CHKERRQ(ierr);
-      p = (nproc/nep->npart)*sc+PetscMin(sc,nproc%nep->npart);
       ierr = MPI_Bcast(&nep->eigr[idx],1,MPIU_SCALAR,p,comm);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSimpleNRefScatterEigenvector"
 static PetscErrorCode NEPSimpleNRefScatterEigenvector(NEP nep,NEPSimpNRefctx *ctx,PetscInt sc,PetscInt idx)
 {
   PetscErrorCode ierr;
   Vec            v;
   PetscScalar    *array;
-  
+
   PetscFunctionBegin;
   if (nep->npart>1) {
     ierr = BVGetColumn(nep->V,idx,&v);CHKERRQ(ierr);
-    if (ctx->subc->color==sc) {
+    if (nep->refinesubc->color==sc) {
       ierr = VecGetArray(ctx->v,&array);CHKERRQ(ierr);
       ierr = VecPlaceArray(ctx->vg,array);CHKERRQ(ierr);
     }
     ierr = VecScatterBegin(ctx->scatter_id[sc],v,ctx->vg,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     ierr = VecScatterEnd(ctx->scatter_id[sc],v,ctx->vg,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-    if (ctx->subc->color==sc) {
+    if (nep->refinesubc->color==sc) {
       ierr = VecResetArray(ctx->vg);CHKERRQ(ierr);
       ierr = VecRestoreArray(ctx->v,&array);CHKERRQ(ierr);
     }
@@ -209,13 +190,11 @@ static PetscErrorCode NEPSimpleNRefScatterEigenvector(NEP nep,NEPSimpNRefctx *ct
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSimpleNRefSetUpSystem"
 static PetscErrorCode NEPSimpleNRefSetUpSystem(NEP nep,NEPSimpNRefctx *ctx,Mat *A,PetscInt idx,Mat *Mt,Mat *T,Mat *P,PetscBool ini,Vec t,Vec v)
 {
   PetscErrorCode    ierr;
   PetscInt          i,st,ml,m0,n0,m1,mg;
-  PetscInt          *dnz,*onz,ncols,*cols2,*nnz,nt=nep->nt;
+  PetscInt          *dnz,*onz,ncols,*cols2=NULL,*nnz,nt=nep->nt;
   PetscScalar       zero=0.0,*coeffs,*coeffs2;
   PetscMPIInt       rank,size;
   MPI_Comm          comm;
@@ -236,7 +215,7 @@ static PetscErrorCode NEPSimpleNRefSetUpSystem(NEP nep,NEPSimpNRefctx *ctx,Mat *
       ierr = MatShellSetOperation(*T,MATOP_MULT,(void(*)(void))MatFSMult);CHKERRQ(ierr);
     } else {
       ierr = MatShellGetContext(*T,&fctx);CHKERRQ(ierr);
-    }    
+    }
     M=fctx->M1;
     break;
   case NEP_REFINE_SCHEME_MBE:
@@ -319,7 +298,7 @@ static PetscErrorCode NEPSimpleNRefSetUpSystem(NEP nep,NEPSimpNRefctx *ctx,Mat *
       *Mt = M;
       *P  = *T;
     }
-  
+
     /* Set values */
     ierr = VecGetArrayRead(w,&array);CHKERRQ(ierr);
     for (i=m0;i<m1;i++) {
@@ -345,7 +324,7 @@ static PetscErrorCode NEPSimpleNRefSetUpSystem(NEP nep,NEPSimpNRefctx *ctx,Mat *
         ierr = MatSetValues(*T,1,&mg,1,&mg,&zero,INSERT_VALUES);CHKERRQ(ierr);
       }
       ierr = VecRestoreArrayRead(ctx->nv,&array);CHKERRQ(ierr);
-    } else {  
+    } else {
       ierr = PetscMalloc1(m1-m0,&cols2);CHKERRQ(ierr);
       for (i=0;i<m1-m0;i++) cols2[i]=m0+i;
       ierr = VecGetArrayRead(v,&array);CHKERRQ(ierr);
@@ -355,27 +334,27 @@ static PetscErrorCode NEPSimpleNRefSetUpSystem(NEP nep,NEPSimpNRefctx *ctx,Mat *
     }
     ierr = VecConjugate(v);CHKERRQ(ierr);
     ierr = MatAssemblyBegin(*T,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = MatAssemblyEnd(*T,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);  
+    ierr = MatAssemblyEnd(*T,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
     ierr = PetscFree(cols2);CHKERRQ(ierr);
-    break; 
+    break;
   case NEP_REFINE_SCHEME_SCHUR:
     fctx->M2 = ctx->w;
     fctx->M3 = v;
-    fctx->m3 = 0.0;
-    for (i=1;i<nt-1;i++) fctx->m3 += PetscConj(coeffs[i])*coeffs[i];
-    fctx->M4 = 0.0;
-    for (i=1;i<nt-1;i++) fctx->M4 += PetscConj(coeffs[i])*coeffs2[i];
+    fctx->m3 = 1.0+PetscConj(nep->eigr[idx])*nep->eigr[idx];
+    fctx->M4 = PetscConj(nep->eigr[idx]);
     fctx->M1 = M;
     if (ini) {
       ierr = MatDuplicate(M,MAT_COPY_VALUES,P);CHKERRQ(ierr);
     } else {
       ierr = MatCopy(M,*P,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
     }
-    ierr = VecConjugate(v);CHKERRQ(ierr);
-    ierr = VecPointwiseMult(t,v,w);CHKERRQ(ierr);
-    ierr = VecConjugate(v);CHKERRQ(ierr);
-    ierr = VecScale(t,-fctx->m3/fctx->M4);CHKERRQ(ierr);
-    ierr = MatDiagonalSet(*P,t,ADD_VALUES);CHKERRQ(ierr);
+    if (fctx->M4!=0.0) {
+      ierr = VecConjugate(v);CHKERRQ(ierr);
+      ierr = VecPointwiseMult(t,v,w);CHKERRQ(ierr);
+      ierr = VecConjugate(v);CHKERRQ(ierr);
+      ierr = VecScale(t,-fctx->m3/fctx->M4);CHKERRQ(ierr);
+      ierr = MatDiagonalSet(*P,t,ADD_VALUES);CHKERRQ(ierr);
+    }
     break;
   case NEP_REFINE_SCHEME_MBE:
     *T = M;
@@ -386,8 +365,6 @@ static PetscErrorCode NEPSimpleNRefSetUpSystem(NEP nep,NEPSimpNRefctx *ctx,Mat *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPNewtonRefinementSimple"
 PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,PetscInt k)
 {
   PetscErrorCode    ierr;
@@ -408,7 +385,7 @@ PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,
   ierr = PetscLogEventBegin(NEP_Refine,nep,0,0,0);CHKERRQ(ierr);
   ierr = NEPSimpleNRefSetUp(nep,&ctx);CHKERRQ(ierr);
   its = (maxits)?*maxits:NREF_MAXIT;
-  comm = (nep->npart==1)?PetscObjectComm((PetscObject)nep):PetscSubcommChild(ctx->subc);
+  comm = (nep->npart==1)?PetscObjectComm((PetscObject)nep):PetscSubcommChild(nep->refinesubc);
   if (!nep->refineksp) { ierr = NEPRefineGetKSP(nep,&nep->refineksp);CHKERRQ(ierr); }
   if (nep->npart==1) {
     ierr = BVGetColumn(nep->V,0,&v);CHKERRQ(ierr);
@@ -425,8 +402,8 @@ PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,
   ierr = PetscMalloc3(nep->npart,&idx_sc,nep->npart,&its_sc,nep->npart,&fail_sc);CHKERRQ(ierr);
   for (i=0;i<nep->npart;i++) fail_sc[i] = 0;
   for (i=0;i<nep->npart;i++) its_sc[i] = 0;
-  color = (nep->npart==1)?0:ctx->subc->color;
-   
+  color = (nep->npart==1)?0:nep->refinesubc->color;
+
   /* Loop performing iterative refinements */
   while (!solved) {
     for (i=0;i<nep->npart;i++) {
@@ -448,6 +425,7 @@ PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,
         if (error<=tol || its_sc[i]>=its || fail_sc[i]) {
           idx_sc[i] = idx++;
           its_sc[i] = 0;
+          fail_sc[i] = 0;
           if (idx_sc[i]<k) { ierr = NEPSimpleNRefScatterEigenvector(nep,ctx,i,idx_sc[i]);CHKERRQ(ierr); }
         } else {
           sc_pend = PETSC_FALSE;
@@ -464,7 +442,7 @@ PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,
 #endif
       if (nep->npart==1) {
         ierr = BVGetColumn(nep->V,idx_sc[color],&v);CHKERRQ(ierr);
-      } else v = ctx->v; 
+      } else v = ctx->v;
       ierr = NEPSimpleNRefSetUpSystem(nep,ctx,ctx->A,idx_sc[color],&Mt,&T,&P,ini,t[0],v);CHKERRQ(ierr);
       ierr = KSPSetOperators(nep->refineksp,T,P);CHKERRQ(ierr);
       if (ini) {
@@ -541,19 +519,22 @@ PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,
         }
         break;
       case NEP_REFINE_SCHEME_SCHUR:
+        fail_sc[color] = 1;
         ierr = MatShellGetContext(T,&fctx);CHKERRQ(ierr);
-        ierr = MatMult(fctx->M1,v,r);CHKERRQ(ierr);
-        ierr = KSPSolve(nep->refineksp,r,dv);CHKERRQ(ierr);
-        ierr = KSPGetConvergedReason(nep->refineksp,&reason);CHKERRQ(ierr);
-        if (reason>0) {
-          ierr = VecDot(dv,v,&deig);CHKERRQ(ierr);
-          deig *= -fctx->m3/fctx->M4;
-          ierr = VecAXPY(v,-1.0,dv);CHKERRQ(ierr);
-          ierr = VecNorm(v,NORM_2,&norm);CHKERRQ(ierr);
-          ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
-          nep->eigr[idx_sc[color]] -= deig;
-          fail_sc[color] = 0;
-        } else fail_sc[color] = 1;
+        if (fctx->M4!=0.0) {
+          ierr = MatMult(fctx->M1,v,r);CHKERRQ(ierr);
+          ierr = KSPSolve(nep->refineksp,r,dv);CHKERRQ(ierr);
+          ierr = KSPGetConvergedReason(nep->refineksp,&reason);CHKERRQ(ierr);
+          if (reason>0) {
+            ierr = VecDot(dv,v,&deig);CHKERRQ(ierr);
+            deig *= -fctx->m3/fctx->M4;
+            ierr = VecAXPY(v,-1.0,dv);CHKERRQ(ierr);
+            ierr = VecNorm(v,NORM_2,&norm);CHKERRQ(ierr);
+            ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
+            nep->eigr[idx_sc[color]] -= deig;
+            fail_sc[color] = 0;
+          }
+        }
         break;
       }
       if (nep->npart==1) { ierr = BVRestoreColumn(nep->V,idx_sc[color],&v);CHKERRQ(ierr); }
@@ -592,6 +573,12 @@ PetscErrorCode NEPNewtonRefinementSimple(NEP nep,PetscInt *maxits,PetscReal tol,
       ierr = PetscFree(ctx->fn);CHKERRQ(ierr);
     }
   }
+  if (nep->scheme==NEP_REFINE_SCHEME_MBE) {
+    if (nep->npart>1) {
+      for (i=0;i<nep->nt;i++) { ierr = FNDestroy(&ctx->fn[i]);CHKERRQ(ierr); }
+      ierr = PetscFree(ctx->fn);CHKERRQ(ierr);
+    }
+  }
   ierr = MatDestroy(&T);CHKERRQ(ierr);
   ierr = PetscFree(ctx);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(NEP_Refine,nep,0,0,0);CHKERRQ(ierr);
