@@ -1,30 +1,18 @@
 /*
-      NEP routines related to problem setup.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   NEP routines related to problem setup
+*/
 
 #include <slepc/private/nepimpl.h>       /*I "slepcnep.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetUp"
 /*@
    NEPSetUp - Sets up all the internal data structures necessary for the
    execution of the NEP solver.
@@ -69,7 +57,6 @@ PetscErrorCode NEPSetUp(NEP nep)
     ierr = NEPSetType(nep,NEPRII);CHKERRQ(ierr);
   }
   if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
-  ierr = DSReset(nep->ds);CHKERRQ(ierr);
   if (!nep->rg) { ierr = NEPGetRG(nep,&nep->rg);CHKERRQ(ierr); }
   if (!((PetscObject)nep->rg)->type_name) {
     ierr = RGSetType(nep->rg,RGINTERVAL);CHKERRQ(ierr);
@@ -97,6 +84,11 @@ PetscErrorCode NEPSetUp(NEP nep)
     break;
   }
 
+  /* set default problem type */
+  if (!nep->problem_type) {
+    ierr = NEPSetProblemType(nep,NEP_GENERAL);CHKERRQ(ierr);
+  }
+
   /* check consistency of refinement options */
   if (nep->refine) {
     if (nep->fui!=NEP_USER_INTERFACE_SPLIT) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Iterative refinement only implemented in split form");
@@ -125,8 +117,7 @@ PetscErrorCode NEPSetUp(NEP nep)
       }
     }
     if (nep->scheme==NEP_REFINE_SCHEME_SCHUR) {
-      if (nep->nt<=2) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"The Schur scheme for refinement is only available for problems defined with more than 2 matrices");
-      if (nep->npart>1) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"The Schur scheme for refinement does not support subcommunicators"); 
+      if (nep->npart>1) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"The Schur scheme for refinement does not support subcommunicators");
     }
   }
   /* call specific solver setup */
@@ -178,8 +169,10 @@ PetscErrorCode NEPSetUp(NEP nep)
       nep->sc->comparisonctx = &nep->target;
       break;
     case NEP_TARGET_IMAGINARY:
+#if defined(PETSC_USE_COMPLEX)
       nep->sc->comparison    = SlepcCompareTargetImaginary;
       nep->sc->comparisonctx = &nep->target;
+#endif
       break;
     case NEP_ALL:
       nep->sc->comparison    = SlepcCompareSmallestReal;
@@ -217,9 +210,7 @@ PetscErrorCode NEPSetUp(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetInitialSpace"
-/*@
+/*@C
    NEPSetInitialSpace - Specify a basis of vectors that constitute the initial
    space, that is, the subspace from which the solver starts to iterate.
 
@@ -258,8 +249,6 @@ PetscErrorCode NEPSetInitialSpace(NEP nep,PetscInt n,Vec *is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetDimensions_Default"
 /*
   NEPSetDimensions_Default - Set reasonable values for ncv, mpd if not set
   by the user. This is called at setup.
@@ -282,8 +271,6 @@ PetscErrorCode NEPSetDimensions_Default(NEP nep,PetscInt nev,PetscInt *ncv,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPAllocateSolution"
 /*@
    NEPAllocateSolution - Allocate memory storage for common variables such
    as eigenvalues and eigenvectors.
@@ -318,9 +305,7 @@ PetscErrorCode NEPAllocateSolution(NEP nep,PetscInt extra)
 
   /* allocate space for eigenvalues and friends */
   if (requested != oldsize || !nep->eigr) {
-    if (oldsize) {
-      ierr = PetscFree4(nep->eigr,nep->eigi,nep->errest,nep->perm);CHKERRQ(ierr);
-    }
+    ierr = PetscFree4(nep->eigr,nep->eigi,nep->errest,nep->perm);CHKERRQ(ierr);
     ierr = PetscMalloc4(requested,&nep->eigr,requested,&nep->eigi,requested,&nep->errest,requested,&nep->perm);CHKERRQ(ierr);
     cnt = newc*sizeof(PetscScalar) + newc*sizeof(PetscReal) + newc*sizeof(PetscInt);
     ierr = PetscLogObjectMemory((PetscObject)nep,cnt);CHKERRQ(ierr);
@@ -336,7 +321,7 @@ PetscErrorCode NEPAllocateSolution(NEP nep,PetscInt extra)
     else {
       ierr = NEPGetFunction(nep,&T,NULL,NULL,NULL);CHKERRQ(ierr);
     }
-    ierr = MatCreateVecs(T,&t,NULL);CHKERRQ(ierr);
+    ierr = MatCreateVecsEmpty(T,&t,NULL);CHKERRQ(ierr);
     ierr = BVSetSizesFromVec(nep->V,t,requested);CHKERRQ(ierr);
     ierr = VecDestroy(&t);CHKERRQ(ierr);
   } else {
