@@ -1,31 +1,19 @@
 /*
-   The NEP routines related to various viewers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   NEP routines related to various viewers
+*/
 
 #include <slepc/private/nepimpl.h>      /*I "slepcnep.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPView"
 /*@C
    NEPView - Prints the NEP data structure.
 
@@ -56,9 +44,11 @@
 PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
+  const char     *type=NULL;
   char           str[50];
   PetscInt       i;
   PetscBool      isascii,istrivial,nods;
+  PetscViewer    sviewer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -74,13 +64,20 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
       ierr = (*nep->ops->view)(nep,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
+    if (nep->problem_type) {
+      switch (nep->problem_type) {
+        case NEP_GENERAL:  type = "general nonlinear eigenvalue problem"; break;
+        case NEP_RATIONAL: type = "rational eigenvalue problem"; break;
+      }
+    } else type = "not yet set";
+    ierr = PetscViewerASCIIPrintf(viewer,"  problem type: %s\n",type);CHKERRQ(ierr);
     if (nep->fui) {
       switch (nep->fui) {
       case NEP_USER_INTERFACE_CALLBACK:
         ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator from user callbacks\n");CHKERRQ(ierr);
         break;
       case NEP_USER_INTERFACE_SPLIT:
-        ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator in split form\n");CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator in split form, with %D terms\n",nep->nt);CHKERRQ(ierr);
         break;
       case NEP_USER_INTERFACE_DERIVATIVES:
         ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator from user callbacks for the successive derivatives\n");CHKERRQ(ierr);
@@ -128,7 +125,6 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
       case NEP_ALL:
         ierr = PetscViewerASCIIPrintf(viewer,"all eigenvalues in the region\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(PetscObjectComm((PetscObject)nep),1,"Wrong value of nep->which");
     }
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",nep->nev);CHKERRQ(ierr);
@@ -184,11 +180,19 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
     ierr = DSView(nep->ds,viewer);CHKERRQ(ierr);
   }
   ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
+  if (nep->refine!=NEP_REFINE_NONE) {
+    if (nep->npart>1) {
+      if (nep->refinesubc->color==0) {
+        ierr = PetscViewerASCIIGetStdout(PetscSubcommChild(nep->refinesubc),&sviewer);CHKERRQ(ierr);
+        ierr = KSPView(nep->refineksp,sviewer);CHKERRQ(ierr);
+      }
+    } else {
+      ierr = KSPView(nep->refineksp,viewer);CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPReasonView"
 /*@C
    NEPReasonView - Displays the reason a NEP solve converged or diverged.
 
@@ -224,11 +228,9 @@ PetscErrorCode NEPReasonView(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPReasonViewFromOptions"
 /*@
    NEPReasonViewFromOptions - Processes command line options to determine if/how
-   the NEP converged reason is to be viewed. 
+   the NEP converged reason is to be viewed.
 
    Collective on NEP
 
@@ -259,30 +261,28 @@ PetscErrorCode NEPReasonViewFromOptions(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPErrorView_ASCII"
 static PetscErrorCode NEPErrorView_ASCII(NEP nep,NEPErrorType etype,PetscViewer viewer)
 {
-  PetscBool      errok;
-  PetscReal      error,re,im;
-  PetscScalar    kr,ki;
-  PetscInt       i,j,nvals;
+  PetscReal      error;
+  PetscInt       i,j,k,nvals;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (nep->which!=NEP_ALL && nep->nconv<nep->nev) {
+  nvals = (nep->which==NEP_ALL)? nep->nconv: nep->nev;
+  if (nep->which!=NEP_ALL && nep->nconv<nvals) {
     ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D eigenvalues converged\n\n",nep->nev);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-  errok = PETSC_TRUE;
-  nvals = (nep->which==NEP_ALL)? nep->nconv: nep->nev;
+  if (nep->which==NEP_ALL && !nvals) {
+    ierr = PetscViewerASCIIPrintf(viewer," No eigenvalues have been found\n\n");CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
   for (i=0;i<nvals;i++) {
     ierr = NEPComputeError(nep,i,etype,&error);CHKERRQ(ierr);
-    errok = (errok && error<5.0*nep->tol)? PETSC_TRUE: PETSC_FALSE;
-  }
-  if (!errok) {
-    ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",nvals);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
+    if (error>=5.0*nep->tol) {
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",nvals);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
   }
   if (nep->which==NEP_ALL) {
     ierr = PetscViewerASCIIPrintf(viewer," Found %D eigenvalues, all of them computed up to the required tolerance:",nvals);CHKERRQ(ierr);
@@ -292,21 +292,8 @@ static PetscErrorCode NEPErrorView_ASCII(NEP nep,NEPErrorType etype,PetscViewer
   for (i=0;i<=(nvals-1)/8;i++) {
     ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
     for (j=0;j<PetscMin(8,nvals-8*i);j++) {
-      ierr = NEPGetEigenpair(nep,8*i+j,&kr,&ki,NULL,NULL);CHKERRQ(ierr);
-#if defined(PETSC_USE_COMPLEX)
-      re = PetscRealPart(kr);
-      im = PetscImaginaryPart(kr);
-#else
-      re = kr;
-      im = ki;
-#endif
-      if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
-      if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
-      if (im!=0.0) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%.5f%+.5fi",(double)re,(double)im);CHKERRQ(ierr);
-      } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"%.5f",(double)re);CHKERRQ(ierr);
-      }
+      k = nep->perm[8*i+j];
+      ierr = SlepcPrintEigenvalueASCII(nep->eigr[k],nep->eigi[k]);CHKERRQ(ierr);
       if (8*i+j+1<nvals) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
     }
   }
@@ -314,8 +301,6 @@ static PetscErrorCode NEPErrorView_ASCII(NEP nep,NEPErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPErrorView_DETAIL"
 static PetscErrorCode NEPErrorView_DETAIL(NEP nep,NEPErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -359,8 +344,6 @@ static PetscErrorCode NEPErrorView_DETAIL(NEP nep,NEPErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPErrorView_MATLAB"
 static PetscErrorCode NEPErrorView_MATLAB(NEP nep,NEPErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -373,14 +356,12 @@ static PetscErrorCode NEPErrorView_MATLAB(NEP nep,NEPErrorType etype,PetscViewer
   ierr = PetscViewerASCIIPrintf(viewer,"Error_%s = [\n",name);CHKERRQ(ierr);
   for (i=0;i<nep->nconv;i++) {
     ierr = NEPComputeError(nep,i,etype,&error);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",error);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",(double)error);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"];\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPErrorView"
 /*@C
    NEPErrorView - Displays the errors associated with the computed solution
    (as well as the eigenvalues).
@@ -440,11 +421,9 @@ PetscErrorCode NEPErrorView(NEP nep,NEPErrorType etype,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPErrorViewFromOptions"
 /*@
    NEPErrorViewFromOptions - Processes command line options to determine if/how
-   the errors of the computed solution are to be viewed. 
+   the errors of the computed solution are to be viewed.
 
    Collective on NEP
 
@@ -489,8 +468,6 @@ PetscErrorCode NEPErrorViewFromOptions(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPValuesView_DRAW"
 static PetscErrorCode NEPValuesView_DRAW(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -501,8 +478,8 @@ static PetscErrorCode NEPValuesView_DRAW(NEP nep,PetscViewer viewer)
 
   PetscFunctionBegin;
   if (!nep->nconv) PetscFunctionReturn(0);
-  ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+  ierr = PetscDrawSetTitle(draw,"Computed Eigenvalues");CHKERRQ(ierr);
   ierr = PetscDrawSPCreate(draw,1,&drawsp);CHKERRQ(ierr);
   for (i=0;i<nep->nconv;i++) {
     k = nep->perm[i];
@@ -518,15 +495,11 @@ static PetscErrorCode NEPValuesView_DRAW(NEP nep,PetscViewer viewer)
   ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscDrawSPSave(drawsp);CHKERRQ(ierr);
   ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPValuesView_ASCII"
 static PetscErrorCode NEPValuesView_ASCII(NEP nep,PetscViewer viewer)
 {
-  PetscReal      re,im;
   PetscInt       i,k;
   PetscErrorCode ierr;
 
@@ -534,27 +507,14 @@ static PetscErrorCode NEPValuesView_ASCII(NEP nep,PetscViewer viewer)
   ierr = PetscViewerASCIIPrintf(viewer,"Eigenvalues = \n");CHKERRQ(ierr);
   for (i=0;i<nep->nconv;i++) {
     k = nep->perm[i];
-#if defined(PETSC_USE_COMPLEX)
-    re = PetscRealPart(nep->eigr[k]);
-    im = PetscImaginaryPart(nep->eigr[k]);
-#else
-    re = nep->eigr[k];
-    im = nep->eigi[k];
-#endif
-    if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
-    if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
-    if (im!=0.0) {
-      ierr = PetscViewerASCIIPrintf(viewer,"   %.5f%+.5fi\n",(double)re,(double)im);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"   %.5f\n",(double)re);CHKERRQ(ierr);
-    }
+    ierr = PetscViewerASCIIPrintf(viewer,"   ");CHKERRQ(ierr);
+    ierr = SlepcPrintEigenvalueASCII(nep->eigr[k],nep->eigi[k]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPValuesView_MATLAB"
 static PetscErrorCode NEPValuesView_MATLAB(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -584,8 +544,6 @@ static PetscErrorCode NEPValuesView_MATLAB(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPValuesView"
 /*@C
    NEPValuesView - Displays the computed eigenvalues in a viewer.
 
@@ -636,11 +594,9 @@ PetscErrorCode NEPValuesView(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPValuesViewFromOptions"
 /*@
    NEPValuesViewFromOptions - Processes command line options to determine if/how
-   the computed eigenvalues are to be viewed. 
+   the computed eigenvalues are to be viewed.
 
    Collective on NEP
 
@@ -671,8 +627,6 @@ PetscErrorCode NEPValuesViewFromOptions(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPVectorsView"
 /*@C
    NEPVectorsView - Outputs computed eigenvectors to a viewer.
 
@@ -724,11 +678,9 @@ PetscErrorCode NEPVectorsView(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPVectorsViewFromOptions"
 /*@
    NEPVectorsViewFromOptions - Processes command line options to determine if/how
-   the computed eigenvectors are to be viewed. 
+   the computed eigenvectors are to be viewed.
 
    Collective on NEP
 
