@@ -1,32 +1,20 @@
 /*
-   Common subroutines for all Krylov-type PEP solvers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Common subroutines for all Krylov-type PEP solvers
+*/
 
 #include <slepc/private/pepimpl.h>    /*I "slepcpep.h" I*/
 #include <slepcblaslapack.h>
 #include "pepkrylov.h"
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPExtractVectors_TOAR"
 PetscErrorCode PEPExtractVectors_TOAR(PEP pep)
 {
   PetscErrorCode ierr;
@@ -162,12 +150,10 @@ PetscErrorCode PEPExtractVectors_TOAR(PEP pep)
       }
       ierr = PetscFree2(tr,ti);CHKERRQ(ierr);
       break;
-    default:
-        SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Extraction not implemented in this solver");
     }
   }
 
-  /* update vectors V = V*S */ 
+  /* update vectors V = V*S */
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&S0);CHKERRQ(ierr);
   ierr = MatDenseGetArray(S0,&pS0);CHKERRQ(ierr);
   for (i=0;i<k;i++) {
@@ -180,15 +166,3 @@ PetscErrorCode PEPExtractVectors_TOAR(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPReset_TOAR"
-PetscErrorCode PEPReset_TOAR(PEP pep)
-{
-  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscFree(ctx->S);CHKERRQ(ierr);
-  ierr = PetscFree(ctx->qB);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
