@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc quadratic eigensolver: "qarnoldi"
 
    Method: Q-Arnoldi
@@ -13,25 +21,6 @@
        [1] K. Meerbergen, "The Quadratic Arnoldi method for the solution
            of the quadratic eigenvalue problem", SIAM J. Matrix Anal.
            Appl. 30(4):1462-1482, 2008.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/pepimpl.h>    /*I "slepcpep.h" I*/
@@ -42,8 +31,6 @@ typedef struct {
   PetscBool lock;         /* locking/non-locking variant */
 } PEP_QARNOLDI;
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetUp_QArnoldi"
 PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
 {
   PetscErrorCode ierr;
@@ -55,10 +42,7 @@ PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
   ierr = PEPSetDimensions_Default(pep,pep->nev,&pep->ncv,&pep->mpd);CHKERRQ(ierr);
   if (!ctx->lock && pep->mpd<pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Should not use mpd parameter in non-locking variant");
   if (!pep->max_it) pep->max_it = PetscMax(100,4*pep->n/pep->ncv);
-  /* Set STSHIFT as the default ST */
-  if (!((PetscObject)pep->st)->type_name) {
-    ierr = STSetType(pep->st,STSHIFT);CHKERRQ(ierr);
-  }
+
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSHIFT,&shift);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
   if (!shift && !sinv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Only STSHIFT and STSINVERT spectral transformations can be used");
@@ -77,7 +61,7 @@ PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
     pep->extract = PEP_EXTRACT_NONE;
   }
   if (pep->extract!=PEP_EXTRACT_NONE) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver does not support requested extraction");
- 
+
   if (!ctx->keep) ctx->keep = 0.5;
 
   ierr = PEPAllocateSolution(pep,0);CHKERRQ(ierr);
@@ -87,22 +71,9 @@ PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
   ierr = DSSetExtraRow(pep->ds,PETSC_TRUE);CHKERRQ(ierr);
   ierr = DSAllocate(pep->ds,pep->ncv+1);CHKERRQ(ierr);
 
-  /* process starting vector */
-  if (pep->nini>-2) {
-    ierr = BVSetRandomColumn(pep->V,0);CHKERRQ(ierr);
-    ierr = BVSetRandomColumn(pep->V,1);CHKERRQ(ierr);
-  } else {
-    ierr = BVInsertVec(pep->V,0,pep->IS[0]);CHKERRQ(ierr);
-    ierr = BVInsertVec(pep->V,1,pep->IS[1]);CHKERRQ(ierr);
-  }
-  if (pep->nini<0) {
-    ierr = SlepcBasisDestroy_Private(&pep->nini,&pep->IS);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPExtractVectors_QArnoldi"
 PetscErrorCode PEPExtractVectors_QArnoldi(PEP pep)
 {
   PetscErrorCode ierr;
@@ -116,23 +87,19 @@ PetscErrorCode PEPExtractVectors_QArnoldi(PEP pep)
   ierr = DSVectors(pep->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
   ierr = DSGetArray(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
 
-  /* update vectors V = V*X */ 
+  /* update vectors V = V*X */
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&X0);CHKERRQ(ierr);
   ierr = MatDenseGetArray(X0,&pX0);CHKERRQ(ierr);
   for (i=0;i<k;i++) {
     ierr = PetscMemcpy(pX0+i*k,X+i*ldds,k*sizeof(PetscScalar));CHKERRQ(ierr);
   }
   ierr = MatDenseRestoreArray(X0,&pX0);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(pep->V,0,k);CHKERRQ(ierr);
   ierr = BVMultInPlace(pep->V,X0,0,k);CHKERRQ(ierr);
   ierr = MatDestroy(&X0);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(pep->V,0,k);CHKERRQ(ierr);
   ierr = DSRestoreArray(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiCGS"
 /*
   Compute a step of Classical Gram-Schmidt orthogonalization
 */
@@ -176,8 +143,6 @@ static PetscErrorCode PEPQArnoldiCGS(PEP pep,PetscScalar *H,PetscBLASInt ldh,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldi"
 /*
   Compute a run of Q-Arnoldi iterations
 */
@@ -240,7 +205,6 @@ static PetscErrorCode PEPQArnoldi(PEP pep,PetscScalar *H,PetscInt ldh,PetscInt k
         if (norm < eta * onorm) *breakdown = PETSC_TRUE;
         else *breakdown = PETSC_FALSE;
         break;
-      default: SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of ip->orth_ref");
     }
     ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
     ierr = VecScale(w,1.0/norm);CHKERRQ(ierr);
@@ -254,8 +218,6 @@ static PetscErrorCode PEPQArnoldi(PEP pep,PetscScalar *H,PetscInt ldh,PetscInt k
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSolve_QArnoldi"
 PetscErrorCode PEPSolve_QArnoldi(PEP pep)
 {
   PetscErrorCode ierr;
@@ -276,6 +238,11 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
   ierr = STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
 
   /* Get the starting Arnoldi vector */
+  for (j=0;j<2;j++) {
+    if (j>=pep->nini) {
+      ierr = BVSetRandomColumn(pep->V,j);CHKERRQ(ierr);
+    }
+  }
   ierr = BVCopyVec(pep->V,0,v);CHKERRQ(ierr);
   ierr = BVCopyVec(pep->V,1,w);CHKERRQ(ierr);
   ierr = VecNorm(v,NORM_2,&x);CHKERRQ(ierr);
@@ -284,6 +251,11 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
   ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
   ierr = VecScale(w,1.0/norm);CHKERRQ(ierr);
 
+  /* clean projected matrix (including the extra-arrow) */
+  ierr = DSGetArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
+  ierr = PetscMemzero(S,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = DSRestoreArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
+  
    /* Restart loop */
   l = 0;
   while (pep->reason == PEP_CONVERGED_ITERATING) {
@@ -306,6 +278,7 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
     ierr = DSSolve(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
     ierr = DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
     ierr = DSUpdateExtraRow(pep->ds);CHKERRQ(ierr);
+    ierr = DSSynchronize(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
 
     /* Check convergence */
     ierr = PEPKrylovConvergence(pep,PETSC_FALSE,pep->nconv,nv-pep->nconv,beta,&k);CHKERRQ(ierr);
@@ -354,8 +327,6 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiSetRestart_QArnoldi"
 static PetscErrorCode PEPQArnoldiSetRestart_QArnoldi(PEP pep,PetscReal keep)
 {
   PEP_QARNOLDI *ctx = (PEP_QARNOLDI*)pep->data;
@@ -369,8 +340,6 @@ static PetscErrorCode PEPQArnoldiSetRestart_QArnoldi(PEP pep,PetscReal keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiSetRestart"
 /*@
    PEPQArnoldiSetRestart - Sets the restart parameter for the Q-Arnoldi
    method, in particular the proportion of basis vectors that must be kept
@@ -403,8 +372,6 @@ PetscErrorCode PEPQArnoldiSetRestart(PEP pep,PetscReal keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiGetRestart_QArnoldi"
 static PetscErrorCode PEPQArnoldiGetRestart_QArnoldi(PEP pep,PetscReal *keep)
 {
   PEP_QARNOLDI *ctx = (PEP_QARNOLDI*)pep->data;
@@ -414,8 +381,6 @@ static PetscErrorCode PEPQArnoldiGetRestart_QArnoldi(PEP pep,PetscReal *keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiGetRestart"
 /*@
    PEPQArnoldiGetRestart - Gets the restart parameter used in the Q-Arnoldi method.
 
@@ -442,8 +407,6 @@ PetscErrorCode PEPQArnoldiGetRestart(PEP pep,PetscReal *keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiSetLocking_QArnoldi"
 static PetscErrorCode PEPQArnoldiSetLocking_QArnoldi(PEP pep,PetscBool lock)
 {
   PEP_QARNOLDI *ctx = (PEP_QARNOLDI*)pep->data;
@@ -453,8 +416,6 @@ static PetscErrorCode PEPQArnoldiSetLocking_QArnoldi(PEP pep,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiSetLocking"
 /*@
    PEPQArnoldiSetLocking - Choose between locking and non-locking variants of
    the Q-Arnoldi method.
@@ -491,8 +452,6 @@ PetscErrorCode PEPQArnoldiSetLocking(PEP pep,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiGetLocking_QArnoldi"
 static PetscErrorCode PEPQArnoldiGetLocking_QArnoldi(PEP pep,PetscBool *lock)
 {
   PEP_QARNOLDI *ctx = (PEP_QARNOLDI*)pep->data;
@@ -502,8 +461,6 @@ static PetscErrorCode PEPQArnoldiGetLocking_QArnoldi(PEP pep,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPQArnoldiGetLocking"
 /*@
    PEPQArnoldiGetLocking - Gets the locking flag used in the Q-Arnoldi method.
 
@@ -530,8 +487,6 @@ PetscErrorCode PEPQArnoldiGetLocking(PEP pep,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetFromOptions_QArnoldi"
 PetscErrorCode PEPSetFromOptions_QArnoldi(PetscOptionItems *PetscOptionsObject,PEP pep)
 {
   PetscErrorCode ierr;
@@ -540,20 +495,17 @@ PetscErrorCode PEPSetFromOptions_QArnoldi(PetscOptionItems *PetscOptionsObject,P
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"PEP Q-Arnoldi Options");CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-pep_qarnoldi_restart","Proportion of vectors kept after restart","PEPQArnoldiSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PEPQArnoldiSetRestart(pep,keep);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsBool("-pep_qarnoldi_locking","Choose between locking and non-locking variants","PEPQArnoldiSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PEPQArnoldiSetLocking(pep,lock);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsReal("-pep_qarnoldi_restart","Proportion of vectors kept after restart","PEPQArnoldiSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPQArnoldiSetRestart(pep,keep);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsBool("-pep_qarnoldi_locking","Choose between locking and non-locking variants","PEPQArnoldiSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPQArnoldiSetLocking(pep,lock);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPView_QArnoldi"
 PetscErrorCode PEPView_QArnoldi(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -563,14 +515,12 @@ PetscErrorCode PEPView_QArnoldi(PEP pep,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Q-Arnoldi: %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  Q-Arnoldi: using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPDestroy_QArnoldi"
 PetscErrorCode PEPDestroy_QArnoldi(PEP pep)
 {
   PetscErrorCode ierr;
@@ -584,8 +534,6 @@ PetscErrorCode PEPDestroy_QArnoldi(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPCreate_QArnoldi"
 PETSC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
 {
   PEP_QARNOLDI   *ctx;
@@ -604,6 +552,8 @@ PETSC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
   pep->ops->backtransform  = PEPBackTransform_Default;
   pep->ops->computevectors = PEPComputeVectors_Default;
   pep->ops->extractvectors = PEPExtractVectors_QArnoldi;
+  pep->ops->setdefaultst   = PEPSetDefaultST_Transform;
+
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetRestart_C",PEPQArnoldiSetRestart_QArnoldi);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetRestart_C",PEPQArnoldiGetRestart_QArnoldi);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetLocking_C",PEPQArnoldiSetLocking_QArnoldi);CHKERRQ(ierr);
