@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc polynomial eigensolver: "stoar"
 
    Method: S-TOAR
@@ -13,25 +21,6 @@
        [1] C. Campos and J.E. Roman, "Restarted Q-Arnoldi-type methods
            exploiting symmetry in quadratic eigenvalue problems", BIT
            Numer. Math. (in press), 2016.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/pepimpl.h>         /*I "slepcpep.h" I*/
@@ -44,15 +33,13 @@ static const char citation[] =
   "   author = \"C. Campos and J. E. Roman\",\n"
   "   title = \"Restarted {Q-Arnoldi-type} methods exploiting symmetry in quadratic eigenvalue problems\",\n"
   "   journal = \"{BIT} Numer. Math.\",\n"
-  "   volume = \"to appear\",\n"
-  "   number = \"\",\n"
-  "   pages = \"\",\n"
+  "   volume = \"56\",\n"
+  "   number = \"4\",\n"
+  "   pages = \"1213--1236\",\n"
   "   year = \"2016,\"\n"
-  "   doi = \"http://dx.doi.org/10.1007/s10543-016-0601-5\"\n"
+  "   doi = \"https://doi.org/10.1007/s10543-016-0601-5\"\n"
   "}\n";
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARNorm"
 /*
   Compute B-norm of v=[v1;v2] whith  B=diag(-pep->T[0],pep->T[2])
 */
@@ -82,8 +69,6 @@ static PetscErrorCode PEPSTOARNorm(PEP pep,PetscInt j,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARqKqMupdates"
 static PetscErrorCode PEPSTOARqKqMupdates(PEP pep,PetscInt j,Vec *wv)
 {
   PetscErrorCode ierr;
@@ -118,25 +103,19 @@ static PetscErrorCode PEPSTOARqKqMupdates(PEP pep,PetscInt j,Vec *wv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetUp_STOAR"
 PetscErrorCode PEPSetUp_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
-  PetscBool      shift,sinv,flg,lindep;
+  PetscBool      shift,sinv,flg;
   PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
-  PetscInt       ld,i;
-  PetscReal      norm,*omega;
+  PetscInt       ld;
 
   PetscFunctionBegin;
   pep->lineariz = PETSC_TRUE;
   ierr = PEPSetDimensions_Default(pep,pep->nev,&pep->ncv,&pep->mpd);CHKERRQ(ierr);
   if (!ctx->lock && pep->mpd<pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Should not use mpd parameter in non-locking variant");
-  if (!pep->max_it) pep->max_it = PetscMax(100,2*pep->n/pep->ncv);
-  /* Set STSHIFT as the default ST */
-  if (!((PetscObject)pep->st)->type_name) {
-    ierr = STSetType(pep->st,STSHIFT);CHKERRQ(ierr);
-  }
+  if (!pep->max_it) pep->max_it = PetscMax(100,2*(pep->nmat-1)*pep->n/pep->ncv);
+
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSHIFT,&shift);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
   if (!shift && !sinv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Only STSHIFT and STSINVERT spectral transformations can be used");
@@ -160,43 +139,11 @@ PetscErrorCode PEPSetUp_STOAR(PEP pep)
   ierr = STGetNumMatrices(pep->st,&ctx->d);CHKERRQ(ierr);
   ctx->d--;
   ctx->ld = ld;
-  ierr = PetscCalloc1(ctx->d*ld*ld,&ctx->S);CHKERRQ(ierr);
-  ierr = PetscCalloc1(2*ld*ld,&ctx->qB);CHKERRQ(ierr);
-
-  /* process starting vector */
-  if (pep->nini>-2) {
-    ierr = BVSetRandomColumn(pep->V,0);CHKERRQ(ierr);
-    ierr = BVSetRandomColumn(pep->V,1);CHKERRQ(ierr);
-  } else {
-    ierr = BVInsertVec(pep->V,0,pep->IS[0]);CHKERRQ(ierr);
-    ierr = BVInsertVec(pep->V,1,pep->IS[1]);CHKERRQ(ierr);
-  }
-  ierr = BVOrthogonalizeColumn(pep->V,0,NULL,&norm,&lindep);CHKERRQ(ierr);
-  if (!lindep) {
-    ierr = BVScaleColumn(pep->V,0,1.0/norm);CHKERRQ(ierr);
-    ctx->S[0] = norm;
-    ierr = PEPSTOARqKqMupdates(pep,0,pep->work);CHKERRQ(ierr);
-  } else SETERRQ(PetscObjectComm((PetscObject)pep),1,"Problem with initial vector");
-  ierr = BVOrthogonalizeColumn(pep->V,1,ctx->S+ld,&norm,&lindep);CHKERRQ(ierr);
-  if (!lindep) {
-    ierr = BVScaleColumn(pep->V,1,1.0/norm);CHKERRQ(ierr);
-    ctx->S[1] = norm;
-    ierr = PEPSTOARqKqMupdates(pep,1,pep->work);CHKERRQ(ierr);
-  } else SETERRQ(PetscObjectComm((PetscObject)pep),1,"Problem with initial vector");
-
-  ierr = PEPSTOARNorm(pep,0,&norm);CHKERRQ(ierr);
-  for (i=0;i<2;i++) { ctx->S[i+ld] /= norm; ctx->S[i] /= norm; }
-  ierr = DSGetArrayReal(pep->ds,DS_MAT_D,&omega);CHKERRQ(ierr);
-  omega[0] = (norm>0)?1.0:-1.0;
-  ierr = DSRestoreArrayReal(pep->ds,DS_MAT_D,&omega);CHKERRQ(ierr);
-  if (pep->nini<0) {
-    ierr = SlepcBasisDestroy_Private(&pep->nini,&pep->IS);CHKERRQ(ierr);
-  }
+  if (ctx->S) { ierr = PetscFree2(ctx->S,ctx->qB);CHKERRQ(ierr); }
+  ierr = PetscCalloc2(ctx->d*ld*ld,&ctx->S,2*ld*ld,&ctx->qB);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOAROrth2"
 /*
   Computes GS orthogonalization  x = [z;x] - [Sp;Sq]*y,
   where y = Omega\([Sp;Sq]'*[qK zeros(size(qK,1)) ;zeros(size(qK,1)) qM]*[z;x]).
@@ -243,8 +190,6 @@ static PetscErrorCode PEPSTOAROrth2(PEP pep,PetscInt k,PetscReal *Omega,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARrun"
 /*
   Compute a run of Lanczos iterations. dim(work)=(ctx->ld)*4
 */
@@ -318,8 +263,6 @@ static PetscErrorCode PEPSTOARrun(PEP pep,PetscReal *a,PetscReal *b,PetscReal *o
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARTrunc"
 static PetscErrorCode PEPSTOARTrunc(PEP pep,PetscInt rs1,PetscInt cs1,PetscScalar *work,PetscReal *rwork)
 {
 #if defined(PETSC_MISSING_LAPACK_GESVD)
@@ -365,7 +308,7 @@ static PetscErrorCode PEPSTOARTrunc(PEP pep,PetscInt rs1,PetscInt cs1,PetscScala
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","S",&rs1_,&cs1t2,M,&rs1_,sg,U,&rs1_,V,&n_,work+nwu,&lw_,rwork+nrwu,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
+  SlepcCheckLapackInfo("gesvd",info);
 
   /* Update the corresponding vectors V(:,idx) = V*Q(:,idx) */
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,rs1,2*cs1,U,&G);CHKERRQ(ierr);
@@ -394,8 +337,6 @@ static PetscErrorCode PEPSTOARTrunc(PEP pep,PetscInt rs1,PetscInt cs1,PetscScala
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARSupdate"
 /*
   S <- S*Q
   columns s-s+ncu of S
@@ -428,8 +369,6 @@ static PetscErrorCode PEPSTOARSupdate(PetscScalar *S,PetscInt ld,PetscInt sr,Pet
 }
 
 #if 0
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARpreKConvergence"
 static PetscErrorCode PEPSTOARpreKConvergence(PEP pep,PetscInt nv,PetscReal *norm,Vec *w)
 {
   PetscErrorCode ierr;
@@ -457,13 +396,11 @@ static PetscErrorCode PEPSTOARpreKConvergence(PEP pep,PetscInt nv,PetscReal *nor
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSolve_STOAR"
 PetscErrorCode PEPSolve_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
-  PetscInt       j,k,l,nv=0,ld=ctx->ld,lds=ctx->d*ctx->ld,off,ldds,t;
+  PetscInt       j,k,l,nv=0,ld=ctx->ld,lds=ctx->d*ctx->ld,off,ldds,t,nq=0;
   PetscInt       lwa,lrwa,nwu=0,nrwu=0,nconv=0;
   PetscScalar    *S=ctx->S,*Q,*work;
   PetscReal      beta,norm=1.0,*omega,*a,*b,*r,*rwork;
@@ -479,6 +416,28 @@ PetscErrorCode PEPSolve_STOAR(PEP pep)
   ierr = RGPushScale(pep->rg,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
   ierr = STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
 
+  /* Get the starting Arnoldi vector */
+  for (j=0;j<ctx->d;j++) {
+    if (j>=pep->nini) {
+      ierr = BVSetRandomColumn(pep->V,nq);CHKERRQ(ierr);
+    }else {
+      ierr = BVCopyColumn(pep->V,j,nq);CHKERRQ(ierr);
+    }
+    ierr = BVOrthogonalizeColumn(pep->V,nq,ctx->S+j*ctx->ld,&norm,&breakdown);CHKERRQ(ierr);
+    if (!breakdown) {
+      ierr = BVScaleColumn(pep->V,nq,1.0/norm);CHKERRQ(ierr);
+      ctx->S[nq+j*ctx->ld] = norm;
+      ierr = PEPSTOARqKqMupdates(pep,nq,pep->work);CHKERRQ(ierr);
+      nq++;
+    }
+  }
+  if (nq<2) SETERRQ(PetscObjectComm((PetscObject)pep),1,"PEP: Problem with initial vector");
+  ierr = PEPSTOARNorm(pep,0,&norm);CHKERRQ(ierr);
+  for (j=0;j<2;j++) { ctx->S[j+ld] /= norm; ctx->S[j] /= norm; }
+  ierr = DSGetArrayReal(pep->ds,DS_MAT_D,&omega);CHKERRQ(ierr);
+  omega[0] = (norm>0)?1.0:-1.0;
+  ierr = DSRestoreArrayReal(pep->ds,DS_MAT_D,&omega);CHKERRQ(ierr);
+
   /* Restart loop */
   l = 0;
   ierr = DSGetLeadingDimension(pep->ds,&ldds);CHKERRQ(ierr);
@@ -508,6 +467,7 @@ PetscErrorCode PEPSolve_STOAR(PEP pep)
     /* Solve projected problem */
     ierr = DSSolve(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
     ierr = DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = DSSynchronize(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
 
     /* Check convergence */
     /* ierr = PEPSTOARpreKConvergence(pep,nv,&norm,pep->work);CHKERRQ(ierr);*/
@@ -598,8 +558,6 @@ PetscErrorCode PEPSolve_STOAR(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetFromOptions_STOAR"
 PetscErrorCode PEPSetFromOptions_STOAR(PetscOptionItems *PetscOptionsObject,PEP pep)
 {
   PetscErrorCode ierr;
@@ -607,16 +565,14 @@ PetscErrorCode PEPSetFromOptions_STOAR(PetscOptionItems *PetscOptionsObject,PEP
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"PEP STOAR Options");CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-pep_stoar_locking","Choose between locking and non-locking variants","PEPSTOARSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PEPSTOARSetLocking(pep,lock);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsBool("-pep_stoar_locking","Choose between locking and non-locking variants","PEPSTOARSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSTOARSetLocking(pep,lock);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARSetLocking_STOAR"
 static PetscErrorCode PEPSTOARSetLocking_STOAR(PEP pep,PetscBool lock)
 {
   PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
@@ -626,8 +582,6 @@ static PetscErrorCode PEPSTOARSetLocking_STOAR(PEP pep,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARSetLocking"
 /*@
    PEPSTOARSetLocking - Choose between locking and non-locking variants of
    the STOAR method.
@@ -662,8 +616,6 @@ PetscErrorCode PEPSTOARSetLocking(PEP pep,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARGetLocking_STOAR"
 static PetscErrorCode PEPSTOARGetLocking_STOAR(PEP pep,PetscBool *lock)
 {
   PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
@@ -673,8 +625,6 @@ static PetscErrorCode PEPSTOARGetLocking_STOAR(PEP pep,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSTOARGetLocking"
 /*@
    PEPSTOARGetLocking - Gets the locking flag used in the STOAR method.
 
@@ -701,8 +651,6 @@ PetscErrorCode PEPSTOARGetLocking(PEP pep,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPView_STOAR"
 PetscErrorCode PEPView_STOAR(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -712,26 +660,24 @@ PetscErrorCode PEPView_STOAR(PEP pep,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  STOAR: using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPDestroy_STOAR"
 PetscErrorCode PEPDestroy_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
+  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
 
   PetscFunctionBegin;
+  ierr = PetscFree2(ctx->S,ctx->qB);CHKERRQ(ierr);
   ierr = PetscFree(pep->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetLocking_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetLocking_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPCreate_STOAR"
 PETSC_EXTERN PetscErrorCode PEPCreate_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
@@ -745,12 +691,13 @@ PETSC_EXTERN PetscErrorCode PEPCreate_STOAR(PEP pep)
   pep->ops->solve          = PEPSolve_STOAR;
   pep->ops->setup          = PEPSetUp_STOAR;
   pep->ops->setfromoptions = PEPSetFromOptions_STOAR;
-  pep->ops->view           = PEPView_STOAR;
   pep->ops->destroy        = PEPDestroy_STOAR;
+  pep->ops->view           = PEPView_STOAR;
   pep->ops->backtransform  = PEPBackTransform_Default;
   pep->ops->computevectors = PEPComputeVectors_Default;
   pep->ops->extractvectors = PEPExtractVectors_TOAR;
-  pep->ops->reset          = PEPReset_TOAR;
+  pep->ops->setdefaultst   = PEPSetDefaultST_Transform;
+
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetLocking_C",PEPSTOARSetLocking_STOAR);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetLocking_C",PEPSTOARGetLocking_STOAR);CHKERRQ(ierr);
   PetscFunctionReturn(0);
