@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc polynomial eigensolver: "toar"
 
    Method: TOAR
@@ -16,25 +24,6 @@
        [2] C. Campos and J.E. Roman, "Parallel Krylov solvers for the
            polynomial eigenvalue problem in SLEPc", SIAM J. Sci. Comput.
            to appear, 2016.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/pepimpl.h>    /*I "slepcpep.h" I*/
@@ -47,15 +36,13 @@ static const char citation[] =
   "   author = \"C. Campos and J. E. Roman\",\n"
   "   title = \"Parallel {Krylov} solvers for the polynomial eigenvalue problem in {SLEPc}\",\n"
   "   journal = \"{SIAM} J. Sci. Comput.\",\n"
-  "   volume = \"to appear\",\n"
-  "   number = \"\",\n"
-  "   pages = \"\",\n"
+  "   volume = \"38\",\n"
+  "   number = \"5\",\n"
+  "   pages = \"S385--S411\",\n"
   "   year = \"2016,\"\n"
-  "   doi = \"http://dx.doi.org/10.xxxx/yyyy\"\n"
+  "   doi = \"https://doi.org/10.1137/15M1022458\"\n"
   "}\n";
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARSNorm2"
 /*
   Norm of [sp;sq]
 */
@@ -70,25 +57,19 @@ static PetscErrorCode PEPTOARSNorm2(PetscInt n,PetscScalar *S,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetUp_TOAR"
 PetscErrorCode PEPSetUp_TOAR(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
-  PetscBool      shift,sinv,flg,lindep;
-  PetscInt       i,lds,deg=pep->nmat-1,j;
-  PetscReal      norm;
+  PetscBool      shift,sinv,flg;
+  PetscInt       i,lds;
 
   PetscFunctionBegin;
   pep->lineariz = PETSC_TRUE;
   ierr = PEPSetDimensions_Default(pep,pep->nev,&pep->ncv,&pep->mpd);CHKERRQ(ierr);
   if (!ctx->lock && pep->mpd<pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Should not use mpd parameter in non-locking variant");
   if (!pep->max_it) pep->max_it = PetscMax(100,2*(pep->nmat-1)*pep->n/pep->ncv);
-  /* Set STSHIFT as the default ST */
-  if (!((PetscObject)pep->st)->type_name) {
-    ierr = STSetType(pep->st,STSHIFT);CHKERRQ(ierr);
-  }
+
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSHIFT,&shift);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
   if (!shift && !sinv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Only STSHIFT and STSINVERT spectral transformations can be used");
@@ -112,6 +93,7 @@ PetscErrorCode PEPSetUp_TOAR(PEP pep)
   ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
   if (!flg) {
     ierr = PetscMalloc1(pep->nmat,&pep->solvematcoeffs);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)pep,pep->nmat*sizeof(PetscScalar));CHKERRQ(ierr);
     if (sinv) {
       ierr = PEPEvaluateBasis(pep,pep->target,0,pep->solvematcoeffs,NULL);CHKERRQ(ierr);
     } else {
@@ -121,36 +103,11 @@ PetscErrorCode PEPSetUp_TOAR(PEP pep)
   }
   ctx->ld = pep->ncv+(pep->nmat-1);   /* number of rows of each fragment of S */
   lds = (pep->nmat-1)*ctx->ld;
+  if (ctx->S) { ierr = PetscFree(ctx->S);CHKERRQ(ierr); }
   ierr = PetscCalloc1(lds*ctx->ld,&ctx->S);CHKERRQ(ierr);
-
-  /* process starting vector */
-  ctx->nq = 0;
-  for (i=0;i<deg;i++) {
-    if (pep->nini>-deg) {
-      ierr = BVSetRandomColumn(pep->V,ctx->nq);CHKERRQ(ierr);
-    } else {
-      ierr = BVInsertVec(pep->V,ctx->nq,pep->IS[i]);CHKERRQ(ierr);
-    }
-    ierr = BVOrthogonalizeColumn(pep->V,ctx->nq,ctx->S+i*ctx->ld,&norm,&lindep);CHKERRQ(ierr);
-    if (!lindep) {
-      ierr = BVScaleColumn(pep->V,ctx->nq,1.0/norm);CHKERRQ(ierr);
-      ctx->S[ctx->nq+i*ctx->ld] = norm;
-      ctx->nq++;
-    }
-  }
-  if (ctx->nq==0) SETERRQ(PetscObjectComm((PetscObject)pep),1,"PEP: Problem with initial vector");
-  ierr = PEPTOARSNorm2(lds,ctx->S,&norm);CHKERRQ(ierr);
-  for (j=0;j<deg;j++) {
-    for (i=0;i<=j;i++) ctx->S[i+j*ctx->ld] /= norm;
-  }
-  if (pep->nini<0) {
-    ierr = SlepcBasisDestroy_Private(&pep->nini,&pep->IS);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOAROrth2"
 /*
  Computes GS orthogonalization   [z;x] - [Sp;Sq]*y,
  where y = ([Sp;Sq]'*[z;x]).
@@ -202,8 +159,6 @@ static PetscErrorCode PEPTOAROrth2(PEP pep,PetscScalar *S,PetscInt ld,PetscInt d
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARExtendBasis"
 /*
   Extend the TOAR basis by applying the the matrix operator
   over a vector which is decomposed in the TOAR way
@@ -288,8 +243,6 @@ static PetscErrorCode PEPTOARExtendBasis(PEP pep,PetscBool sinvert,PetscScalar s
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARCoefficients"
 /*
   Compute TOAR coefficients of the blocks of the new Arnoldi vector computed
 */
@@ -317,8 +270,6 @@ static PetscErrorCode PEPTOARCoefficients(PEP pep,PetscBool sinvert,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARrun"
 /*
   Compute a run of Arnoldi iterations dim(work)=ld
 */
@@ -337,7 +288,7 @@ static PetscErrorCode PEPTOARrun(PEP pep,PetscScalar sigma,PetscInt *nq,PetscSca
   if (!flg) {
     /* spectral transformation handled by the solver */
     ierr = PetscObjectTypeCompareAny((PetscObject)pep->st,&flg,STSINVERT,STSHIFT,"");CHKERRQ(ierr);
-    if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"STtype not supported fr TOAR without transforming matrices");
+    if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"ST type not supported for TOAR without transforming matrices");
     ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinvert);CHKERRQ(ierr);
   }
   for (j=k;j<m;j++) {
@@ -374,8 +325,6 @@ static PetscErrorCode PEPTOARrun(PEP pep,PetscScalar sigma,PetscInt *nq,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARTrunc"
 /*
   dim(rwork)=6*n; dim(work)=6*ld*lds+2*cs1
 */
@@ -431,7 +380,7 @@ static PetscErrorCode PEPTOARTrunc(PEP pep,PetscScalar *S,PetscInt ld,PetscInt d
 #else
     PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","S",&nrow_,&newctdeg,M,&nrow_,sg,pU+offu,&rs1_,V,&n_,work+nwu,&lw_,rwork+nrwu,&info));
 #endif
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
+    SlepcCheckLapackInfo("gesvd",info);
     /* SVD has rank min(newc,nrow) */
     rk = PetscMin(newc,nrow);
     for (i=0;i<rk;i++) {
@@ -468,7 +417,7 @@ static PetscErrorCode PEPTOARTrunc(PEP pep,PetscScalar *S,PetscInt ld,PetscInt d
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","S",&nrow_,&nnctdeg,M,&nrow_,sg,pU+offu+newc*rs1,&rs1_,V,&n_,work+nwu,&lw_,rwork+nrwu,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
+  SlepcCheckLapackInfo("gesvd",info);
   tol = PetscMax(rs1,deg*cs1)*PETSC_MACHINE_EPSILON*sg[0];
   for (i=0;i<PetscMin(n_,nnctdeg);i++) if (sg[i]>tol) rk++;
   rk = PetscMin(nnc+deg-1,rk);
@@ -500,10 +449,12 @@ static PetscErrorCode PEPTOARTrunc(PEP pep,PetscScalar *S,PetscInt ld,PetscInt d
   ierr = PetscBLASIntCast(rk,&rk_);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(cs1-lock,&nnc_);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&nrow_,&rk_,pU+offu,&rs1_,tau,work+nwu,&lw_,&info));
+  SlepcCheckLapackInfo("geqrf",info);
   for (i=0;i<deg;i++) {
     PetscStackCallBLAS("BLAStrmm",BLAStrmm_("L","U","N","N",&rk_,&nnc_,&sone,pU+offu,&rs1_,S+lock*lds+lock+i*ld,&lds_));
   }
   PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&nrow_,&rk_,&rk_,pU+offu,&rs1_,tau,work+nwu,&lw_,&info));
+  SlepcCheckLapackInfo("ungqr",info);
 
   /* update vectors V(:,idx) = V*Q(:,idx) */
   rk = rk+lock;
@@ -521,8 +472,6 @@ static PetscErrorCode PEPTOARTrunc(PEP pep,PetscScalar *S,PetscInt ld,PetscInt d
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARSupdate"
 /*
   S <- S*Q
   columns s-s+ncu of S
@@ -552,8 +501,6 @@ static PetscErrorCode PEPTOARSupdate(PetscScalar *S,PetscInt ld,PetscInt deg,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPEvaluateBasisM"
 /*
   Computes T_j = phi_idx(T). In T_j and T_p are phi_{idx-1}(T)
    and phi_{idx-2}(T) respectively or null if idx=0,1.
@@ -586,8 +533,6 @@ static PetscErrorCode PEPEvaluateBasisM(PEP pep,PetscInt k,PetscScalar *T,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPExtractInvariantPair"
 /* dim(work)=6*sr*k;*/
 static PetscErrorCode PEPExtractInvariantPair(PEP pep,PetscScalar sigma,PetscInt sr,PetscInt k,PetscScalar *S,PetscInt ld,PetscInt deg,PetscScalar *H,PetscInt ldh,PetscScalar *work)
 {
@@ -638,8 +583,10 @@ static PetscErrorCode PEPExtractInvariantPair(PEP pep,PetscScalar sigma,PetscInt
     }
     if (flg) {
       PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&k_,&k_,T,&k_,p,&info));
+      SlepcCheckLapackInfo("getrf",info);
       ierr = PetscBLASIntCast(nw-nwu,&lwork);CHKERRQ(ierr);
       PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&k_,T,&k_,p,work+nwu,&lwork,&info));
+      SlepcCheckLapackInfo("getri",info);
     }
     if (sigma!=0.0) for (i=0;i<k;i++) T[i+k*i] += sigma;
   } else {
@@ -682,7 +629,7 @@ static PetscErrorCode PEPExtractInvariantPair(PEP pep,PetscScalar sigma,PetscInt
     if (flg) {
       ierr = PetscMalloc1(pep->nmat,&A);CHKERRQ(ierr);
       for (i=0;i<pep->nmat;i++) {
-        ierr = STGetTOperators(pep->st,i,A+i);CHKERRQ(ierr);
+        ierr = STGetMatrixTransformed(pep->st,i,A+i);CHKERRQ(ierr);
       }
     } else A = pep->A;
     ierr = PetscMalloc1(pep->nmat-1,&R);CHKERRQ(ierr);
@@ -749,29 +696,25 @@ static PetscErrorCode PEPExtractInvariantPair(PEP pep,PetscScalar sigma,PetscInt
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","C",&k_,&k_,&k_,&sone,Hj,&k_,Hj,&k_,&sone,Bt,&k_));
     }
     PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&k_,&sr_,Bt,&k_,p,At,&k_,&info));
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESV %d",info);
+    SlepcCheckLapackInfo("gesv",info);
     for (j=0;j<sr;j++) {
       for (i=0;i<k;i++) S[i*lds+j] = PetscConj(At[j*k+i]);
     }
     break;
-  default:
-    SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Extraction not implemented in this solver");
   }
   ierr = PetscFree(p);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSolve_TOAR"
 PetscErrorCode PEPSolve_TOAR(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
-  PetscInt       i,j,k,l,nv=0,ld,lds,off,ldds,newn,nq=ctx->nq,nconv=0,locked=0,newc;
+  PetscInt       i,j,k,l,nv=0,ld,lds,off,ldds,newn,nq=0,nconv=0,locked=0,newc;
   PetscInt       lwa,lrwa,nwu=0,nrwu=0,nmat=pep->nmat,deg=nmat-1;
   PetscScalar    *S,*Q,*work,*H,sigma;
-  PetscReal      beta,*rwork;
+  PetscReal      beta,*rwork,norm;
   PetscBool      breakdown=PETSC_FALSE,flg,falselock=PETSC_FALSE,sinv=PETSC_FALSE;
 
   PetscFunctionBegin;
@@ -802,6 +745,7 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
       sigma /= pep->sfactor;
     } else {
       ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
+      pep->target = sinv?pep->target*pep->sfactor:pep->target/pep->sfactor;
       ierr = RGPushScale(pep->rg,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
       ierr = STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
     }
@@ -809,6 +753,31 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
 
   if (flg) sigma = 0.0;
 
+  /* clean projected matrix (including the extra-arrow) */
+  ierr = DSGetArray(pep->ds,DS_MAT_A,&H);CHKERRQ(ierr);
+  ierr = PetscMemzero(H,ldds*ldds*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = DSRestoreArray(pep->ds,DS_MAT_A,&H);CHKERRQ(ierr);
+  
+  /* Get the starting Arnoldi vector */
+  for (i=0;i<deg;i++) {
+    if (i>=pep->nini) {
+      ierr = BVSetRandomColumn(pep->V,nq);CHKERRQ(ierr);
+    } else {
+      ierr = BVCopyColumn(pep->V,i,nq);CHKERRQ(ierr);
+    }
+    ierr = BVOrthogonalizeColumn(pep->V,nq,S+i*ctx->ld,&norm,&breakdown);CHKERRQ(ierr);
+    if (!breakdown) {
+      ierr = BVScaleColumn(pep->V,nq,1.0/norm);CHKERRQ(ierr);
+      S[nq+i*ctx->ld] = norm;
+      nq++;
+    }
+  }
+  if (nq==0) SETERRQ(PetscObjectComm((PetscObject)pep),1,"PEP: Problem with initial vector");
+  ierr = PEPTOARSNorm2(lds,S,&norm);CHKERRQ(ierr);
+  for (j=0;j<deg;j++) {
+    for (i=0;i<=j;i++) S[i+j*ctx->ld] /= norm;
+  }
+
   /* restart loop */
   l = 0;
   while (pep->reason == PEP_CONVERGED_ITERATING) {
@@ -831,6 +800,7 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
     ierr = DSSolve(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
     ierr = DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
     ierr = DSUpdateExtraRow(pep->ds);CHKERRQ(ierr);
+    ierr = DSSynchronize(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
 
     /* check convergence */
     ierr = PEPKrylovConvergence(pep,PETSC_FALSE,pep->nconv,nv-pep->nconv,beta,&k);CHKERRQ(ierr);
@@ -859,7 +829,7 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
     /* copy last column of S */
     ierr = PetscMemcpy(S+lds*(k+l),S+lds*nv,lds*sizeof(PetscScalar));CHKERRQ(ierr);
 
-    if (breakdown) {
+    if (breakdown && pep->reason == PEP_CONVERGED_ITERATING) {
       /* stop if breakdown */
       ierr = PetscInfo2(pep,"Breakdown TOAR method (it=%D norm=%g)\n",pep->its,(double)beta);CHKERRQ(ierr);
       pep->reason = PEP_DIVERGED_BREAKDOWN;
@@ -867,7 +837,7 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
     if (pep->reason != PEP_CONVERGED_ITERATING) {l--; flg = PETSC_TRUE;}
     else flg = PETSC_FALSE;
     /* truncate S */
-    if (k+l+deg<nq) {
+    if (k+l+deg<=nq) {
       if (!falselock && ctx->lock) {
         newc = k-pep->nconv;
         ierr = PEPTOARTrunc(pep,S,ld,deg,&nq,k+l+1,locked,newc,flg,work+nwu,rwork+nrwu);CHKERRQ(ierr);
@@ -891,9 +861,10 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
       ierr = DSRestoreArray(pep->ds,DS_MAT_A,&H);CHKERRQ(ierr);
       ierr = DSSetDimensions(pep->ds,pep->nconv,0,0,0);CHKERRQ(ierr);
       ierr = DSSetState(pep->ds,DS_STATE_RAW);CHKERRQ(ierr);
-      ierr = PEPNewtonRefinement_TOAR(pep,sigma,&pep->rits,NULL,pep->nconv,S,lds,&nq);CHKERRQ(ierr);
+      ierr = PEPNewtonRefinement_TOAR(pep,sigma,&pep->rits,NULL,pep->nconv,S,lds);CHKERRQ(ierr);
       ierr = DSSolve(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
       ierr = DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+      ierr = DSSynchronize(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
       ierr = DSGetArray(pep->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
       ierr = PEPTOARSupdate(S,ld,deg,nq,0,pep->nconv,pep->nconv,Q,ldds,work+nwu);CHKERRQ(ierr);
       ierr = DSRestoreArray(pep->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
@@ -902,17 +873,10 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
       ierr = DSSetState(pep->ds,DS_STATE_RAW);CHKERRQ(ierr);
     }
   }
+  ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
   if (pep->refine!=PEP_REFINE_MULTIPLE || pep->rits==0) {
-    ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
-    if (!flg) {
-      if (pep->ops->backtransform) {
+    if (!flg && pep->ops->backtransform) {
         ierr = (*pep->ops->backtransform)(pep);CHKERRQ(ierr);
-      }
-      /* restore original values */
-      pep->target *= pep->sfactor;
-      ierr = STScaleShift(pep->st,pep->sfactor);CHKERRQ(ierr);
-    } else {
-      ierr = STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor);CHKERRQ(ierr);
     }
     if (pep->sfactor!=1.0) {
       for (j=0;j<pep->nconv;j++) {
@@ -926,6 +890,14 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
       }
     }
   }
+  /* restore original values */
+  if (!flg) {
+    pep->target *= pep->sfactor;
+    ierr = STScaleShift(pep->st,pep->sfactor);CHKERRQ(ierr);
+  } else {
+    ierr = STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor);CHKERRQ(ierr);
+    pep->target = (sinv)?pep->target/pep->sfactor:pep->target*pep->sfactor;
+  }
   if (pep->sfactor!=1.0) { ierr = RGPopScale(pep->rg);CHKERRQ(ierr); }
 
   /* change the state to raw so that DSVectors() computes eigenvectors from scratch */
@@ -936,8 +908,6 @@ PetscErrorCode PEPSolve_TOAR(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARSetRestart_TOAR"
 static PetscErrorCode PEPTOARSetRestart_TOAR(PEP pep,PetscReal keep)
 {
   PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
@@ -951,8 +921,6 @@ static PetscErrorCode PEPTOARSetRestart_TOAR(PEP pep,PetscReal keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARSetRestart"
 /*@
    PEPTOARSetRestart - Sets the restart parameter for the TOAR
    method, in particular the proportion of basis vectors that must be kept
@@ -985,8 +953,6 @@ PetscErrorCode PEPTOARSetRestart(PEP pep,PetscReal keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARGetRestart_TOAR"
 static PetscErrorCode PEPTOARGetRestart_TOAR(PEP pep,PetscReal *keep)
 {
   PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
@@ -996,8 +962,6 @@ static PetscErrorCode PEPTOARGetRestart_TOAR(PEP pep,PetscReal *keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARGetRestart"
 /*@
    PEPTOARGetRestart - Gets the restart parameter used in the TOAR method.
 
@@ -1024,8 +988,6 @@ PetscErrorCode PEPTOARGetRestart(PEP pep,PetscReal *keep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARSetLocking_TOAR"
 static PetscErrorCode PEPTOARSetLocking_TOAR(PEP pep,PetscBool lock)
 {
   PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
@@ -1035,8 +997,6 @@ static PetscErrorCode PEPTOARSetLocking_TOAR(PEP pep,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARSetLocking"
 /*@
    PEPTOARSetLocking - Choose between locking and non-locking variants of
    the TOAR method.
@@ -1071,8 +1031,6 @@ PetscErrorCode PEPTOARSetLocking(PEP pep,PetscBool lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARGetLocking_TOAR"
 static PetscErrorCode PEPTOARGetLocking_TOAR(PEP pep,PetscBool *lock)
 {
   PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
@@ -1082,8 +1040,6 @@ static PetscErrorCode PEPTOARGetLocking_TOAR(PEP pep,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPTOARGetLocking"
 /*@
    PEPTOARGetLocking - Gets the locking flag used in the TOAR method.
 
@@ -1110,8 +1066,6 @@ PetscErrorCode PEPTOARGetLocking(PEP pep,PetscBool *lock)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetFromOptions_TOAR"
 PetscErrorCode PEPSetFromOptions_TOAR(PetscOptionItems *PetscOptionsObject,PEP pep)
 {
   PetscErrorCode ierr;
@@ -1120,20 +1074,17 @@ PetscErrorCode PEPSetFromOptions_TOAR(PetscOptionItems *PetscOptionsObject,PEP p
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"PEP TOAR Options");CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-pep_toar_restart","Proportion of vectors kept after restart","PEPTOARSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PEPTOARSetRestart(pep,keep);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsBool("-pep_toar_locking","Choose between locking and non-locking variants","PEPTOARSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PEPTOARSetLocking(pep,lock);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsReal("-pep_toar_restart","Proportion of vectors kept after restart","PEPTOARSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPTOARSetRestart(pep,keep);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsBool("-pep_toar_locking","Choose between locking and non-locking variants","PEPTOARSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPTOARSetLocking(pep,lock);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPView_TOAR"
 PetscErrorCode PEPView_TOAR(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -1143,19 +1094,19 @@ PetscErrorCode PEPView_TOAR(PEP pep,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  TOAR: %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  TOAR: using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPDestroy_TOAR"
 PetscErrorCode PEPDestroy_TOAR(PEP pep)
 {
   PetscErrorCode ierr;
+  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
 
   PetscFunctionBegin;
+  ierr = PetscFree(ctx->S);CHKERRQ(ierr);
   ierr = PetscFree(pep->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPTOARSetRestart_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPTOARGetRestart_C",NULL);CHKERRQ(ierr);
@@ -1164,8 +1115,6 @@ PetscErrorCode PEPDestroy_TOAR(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPCreate_TOAR"
 PETSC_EXTERN PetscErrorCode PEPCreate_TOAR(PEP pep)
 {
   PEP_TOAR       *ctx;
@@ -1184,7 +1133,7 @@ PETSC_EXTERN PetscErrorCode PEPCreate_TOAR(PEP pep)
   pep->ops->backtransform  = PEPBackTransform_Default;
   pep->ops->computevectors = PEPComputeVectors_Default;
   pep->ops->extractvectors = PEPExtractVectors_TOAR;
-  pep->ops->reset          = PEPReset_TOAR;
+
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPTOARSetRestart_C",PEPTOARSetRestart_TOAR);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPTOARGetRestart_C",PEPTOARGetRestart_TOAR);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPTOARSetLocking_C",PEPTOARSetLocking_TOAR);CHKERRQ(ierr);
