@@ -1,31 +1,19 @@
 /*
-   Explicit linearization for polynomial eigenproblems.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Explicit linearization for polynomial eigenproblems
+*/
 
 #include <slepc/private/pepimpl.h>         /*I "slepcpep.h" I*/
 #include "linearp.h"
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Linear_Shift"
 static PetscErrorCode MatMult_Linear_Shift(Mat M,Vec x,Vec y)
 {
   PetscErrorCode    ierr;
@@ -52,7 +40,7 @@ static PetscErrorCode MatMult_Linear_Shift(Mat M,Vec x,Vec y)
   cb = pep->pbc+nmat;
   cg = pep->pbc+2*nmat;
   x1=ctx->w[0];x2=ctx->w[1];x3=ctx->w[2];y1=ctx->w[3];aux=ctx->w[4];
-  
+
   ierr = VecSet(y,0.0);CHKERRQ(ierr);
   ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
@@ -108,8 +96,6 @@ static PetscErrorCode MatMult_Linear_Shift(Mat M,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Linear_Sinvert"
 static PetscErrorCode MatMult_Linear_Sinvert(Mat M,Vec x,Vec y)
 {
   PetscErrorCode    ierr;
@@ -224,8 +210,6 @@ static PetscErrorCode MatMult_Linear_Sinvert(Mat M,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BackTransform_Linear"
 static PetscErrorCode BackTransform_Linear(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
   PetscErrorCode ierr;
@@ -239,8 +223,15 @@ static PetscErrorCode BackTransform_Linear(ST st,PetscInt n,PetscScalar *eigr,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Apply_Linear"
+/*
+   Dummy backtransform operation
+ */
+static PetscErrorCode BackTransform_Skip(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
+{
+  PetscFunctionBegin;
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode Apply_Linear(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -252,19 +243,17 @@ static PetscErrorCode Apply_Linear(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetUp_Linear"
 PetscErrorCode PEPSetUp_Linear(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_LINEAR     *ctx = (PEP_LINEAR*)pep->data;
   ST             st;
   PetscInt       i=0,deg=pep->nmat-1;
-  EPSWhich       which;
+  EPSWhich       which = EPS_LARGEST_MAGNITUDE;
   EPSProblemType ptype;
   PetscBool      trackall,istrivial,transf,shift,sinv,ks;
   PetscScalar    sigma,*epsarray,*peparray;
-  Vec            veps;
+  Vec            veps,w=NULL;
   /* function tables */
   PetscErrorCode (*fcreate[][2])(MPI_Comm,PEP_LINEAR*,Mat*) = {
     { MatCreateExplicit_Linear_N1A, MatCreateExplicit_Linear_N1B },   /* N1 */
@@ -280,10 +269,6 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
   pep->lineariz = PETSC_TRUE;
   if (!ctx->cform) ctx->cform = 1;
   ierr = STGetTransform(pep->st,&transf);CHKERRQ(ierr);
-  /* Set STSHIFT as the default ST */
-  if (!((PetscObject)pep->st)->type_name) {
-    ierr = STSetType(pep->st,STSHIFT);CHKERRQ(ierr);
-  }
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSHIFT,&shift);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
   if (!shift && !sinv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Only STSHIFT and STSINVERT spectral transformations can be used");
@@ -294,8 +279,8 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
   ierr = STSetUp(pep->st);CHKERRQ(ierr);
   if (!ctx->eps) { ierr = PEPLinearGetEPS(pep,&ctx->eps);CHKERRQ(ierr); }
   ierr = EPSGetST(ctx->eps,&st);CHKERRQ(ierr);
-  if (!transf) { ierr = EPSSetTarget(ctx->eps,pep->target);CHKERRQ(ierr); }
-  if (sinv && !transf) { ierr = STSetDefaultShift(st,pep->target);CHKERRQ(ierr); }
+  if (!transf && !ctx->usereps) { ierr = EPSSetTarget(ctx->eps,pep->target);CHKERRQ(ierr); }
+  if (sinv && !transf && !ctx->usereps) { ierr = STSetDefaultShift(st,pep->target);CHKERRQ(ierr); }
   /* compute scale factor if not set by user */
   ierr = PEPComputeScaleFactor(pep);CHKERRQ(ierr);
 
@@ -306,24 +291,23 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
     if (pep->scale==PEP_SCALE_DIAGONAL || pep->scale==PEP_SCALE_BOTH) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Diagonal scaling not allowed in PEPLINEAR with explicit matrices");
     if (sinv && !transf) { ierr = STSetType(st,STSINVERT);CHKERRQ(ierr); }
     ierr = RGPushScale(pep->rg,1.0/pep->sfactor);CHKERRQ(ierr);
-    ierr = STGetTOperators(pep->st,0,&ctx->K);CHKERRQ(ierr);
-    ierr = STGetTOperators(pep->st,1,&ctx->C);CHKERRQ(ierr);
-    ierr = STGetTOperators(pep->st,2,&ctx->M);CHKERRQ(ierr);
+    ierr = STGetMatrixTransformed(pep->st,0,&ctx->K);CHKERRQ(ierr);
+    ierr = STGetMatrixTransformed(pep->st,1,&ctx->C);CHKERRQ(ierr);
+    ierr = STGetMatrixTransformed(pep->st,2,&ctx->M);CHKERRQ(ierr);
     ctx->sfactor = pep->sfactor;
     ctx->dsfactor = pep->dsfactor;
-  
+
     ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
     ierr = MatDestroy(&ctx->B);CHKERRQ(ierr);
     ierr = VecDestroy(&ctx->w[0]);CHKERRQ(ierr);
     ierr = VecDestroy(&ctx->w[1]);CHKERRQ(ierr);
     ierr = VecDestroy(&ctx->w[2]);CHKERRQ(ierr);
     ierr = VecDestroy(&ctx->w[3]);CHKERRQ(ierr);
-  
+
     switch (pep->problem_type) {
       case PEP_GENERAL:    i = 0; break;
       case PEP_HERMITIAN:  i = 2; break;
       case PEP_GYROSCOPIC: i = 4; break;
-      default: SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of pep->problem_type");
     }
     i += ctx->cform-1;
 
@@ -344,12 +328,10 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
     ierr = STSetType(st,STSHELL);CHKERRQ(ierr);
     ierr = STShellSetContext(st,(PetscObject)ctx);CHKERRQ(ierr);
     if (!transf) { ierr = STShellSetBackTransform(st,BackTransform_Linear);CHKERRQ(ierr); }
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&ctx->w[0]);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&ctx->w[1]);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&ctx->w[2]);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&ctx->w[3]);CHKERRQ(ierr);
-    ierr = MatCreateVecs(pep->A[0],&ctx->w[4],NULL);CHKERRQ(ierr);
-    ierr = MatCreateVecs(pep->A[0],&ctx->w[5],NULL);CHKERRQ(ierr);
+    else { ierr = STShellSetBackTransform(st,BackTransform_Skip);CHKERRQ(ierr); }
+    ierr = MatCreateVecsEmpty(pep->A[0],&ctx->w[0],&ctx->w[1]);CHKERRQ(ierr);
+    ierr = MatCreateVecsEmpty(pep->A[0],&ctx->w[2],&ctx->w[3]);CHKERRQ(ierr);
+    ierr = MatCreateVecs(pep->A[0],&ctx->w[4],&ctx->w[5]);CHKERRQ(ierr);
     ierr = PetscLogObjectParents(pep,6,ctx->w);CHKERRQ(ierr);
     ierr = MatCreateShell(PetscObjectComm((PetscObject)pep),deg*pep->nloc,deg*pep->nloc,deg*pep->n,deg*pep->n,ctx,&ctx->A);CHKERRQ(ierr);
     if (sinv && !transf) {
@@ -364,6 +346,7 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
     ierr = PEPBasisCoefficients(pep,pep->pbc);CHKERRQ(ierr);
     if (!transf) {
       ierr = PetscMalloc1(pep->nmat,&pep->solvematcoeffs);CHKERRQ(ierr);
+      ierr = PetscLogObjectMemory((PetscObject)pep,pep->nmat*sizeof(PetscScalar));CHKERRQ(ierr);
       if (sinv) {
         ierr = PEPEvaluateBasis(pep,pep->target,0,pep->solvematcoeffs,NULL);CHKERRQ(ierr);
       } else {
@@ -376,7 +359,7 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
     if (pep->sfactor!=1.0) {
       for (i=0;i<pep->nmat;i++) {
         pep->pbc[pep->nmat+i] /= pep->sfactor;
-        pep->pbc[2*pep->nmat+i] /= pep->sfactor*pep->sfactor; 
+        pep->pbc[2*pep->nmat+i] /= pep->sfactor*pep->sfactor;
       }
     }
   }
@@ -390,28 +373,29 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
       ierr = EPSSetProblemType(ctx->eps,EPS_NHEP);CHKERRQ(ierr);
     }
   }
-  if (transf) which = EPS_LARGEST_MAGNITUDE;
-  else {
-    switch (pep->which) {
-        case PEP_LARGEST_MAGNITUDE:  which = EPS_LARGEST_MAGNITUDE; break;
-        case PEP_SMALLEST_MAGNITUDE: which = EPS_SMALLEST_MAGNITUDE; break;
-        case PEP_LARGEST_REAL:       which = EPS_LARGEST_REAL; break;
-        case PEP_SMALLEST_REAL:      which = EPS_SMALLEST_REAL; break;
-        case PEP_LARGEST_IMAGINARY:  which = EPS_LARGEST_IMAGINARY; break;
-        case PEP_SMALLEST_IMAGINARY: which = EPS_SMALLEST_IMAGINARY; break;
-        case PEP_TARGET_MAGNITUDE:   which = EPS_TARGET_MAGNITUDE; break;
-        case PEP_TARGET_REAL:        which = EPS_TARGET_REAL; break;
-        case PEP_TARGET_IMAGINARY:   which = EPS_TARGET_IMAGINARY; break;
-        case PEP_WHICH_USER:         which = EPS_WHICH_USER;
-          ierr = EPSSetEigenvalueComparison(ctx->eps,pep->sc->comparison,pep->sc->comparisonctx);CHKERRQ(ierr);
-          break;
-        default: SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of which");
+  if (!ctx->usereps) {
+    if (transf) which = EPS_LARGEST_MAGNITUDE;
+    else {
+      switch (pep->which) {
+          case PEP_LARGEST_MAGNITUDE:  which = EPS_LARGEST_MAGNITUDE; break;
+          case PEP_SMALLEST_MAGNITUDE: which = EPS_SMALLEST_MAGNITUDE; break;
+          case PEP_LARGEST_REAL:       which = EPS_LARGEST_REAL; break;
+          case PEP_SMALLEST_REAL:      which = EPS_SMALLEST_REAL; break;
+          case PEP_LARGEST_IMAGINARY:  which = EPS_LARGEST_IMAGINARY; break;
+          case PEP_SMALLEST_IMAGINARY: which = EPS_SMALLEST_IMAGINARY; break;
+          case PEP_TARGET_MAGNITUDE:   which = EPS_TARGET_MAGNITUDE; break;
+          case PEP_TARGET_REAL:        which = EPS_TARGET_REAL; break;
+          case PEP_TARGET_IMAGINARY:   which = EPS_TARGET_IMAGINARY; break;
+          case PEP_WHICH_USER:         which = EPS_WHICH_USER;
+            ierr = EPSSetEigenvalueComparison(ctx->eps,pep->sc->comparison,pep->sc->comparisonctx);CHKERRQ(ierr);
+            break;
+      }
     }
-  }
-  ierr = EPSSetWhichEigenpairs(ctx->eps,which);CHKERRQ(ierr);
+    ierr = EPSSetWhichEigenpairs(ctx->eps,which);CHKERRQ(ierr);
 
-  ierr = EPSSetDimensions(ctx->eps,pep->nev,pep->ncv?pep->ncv:PETSC_DEFAULT,pep->mpd?pep->mpd:PETSC_DEFAULT);CHKERRQ(ierr);
-  ierr = EPSSetTolerances(ctx->eps,pep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:pep->tol,pep->max_it?pep->max_it:PETSC_DEFAULT);CHKERRQ(ierr);
+    ierr = EPSSetDimensions(ctx->eps,pep->nev,pep->ncv?pep->ncv:PETSC_DEFAULT,pep->mpd?pep->mpd:PETSC_DEFAULT);CHKERRQ(ierr);
+    ierr = EPSSetTolerances(ctx->eps,pep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:pep->tol,pep->max_it?pep->max_it:PETSC_DEFAULT);CHKERRQ(ierr);
+  }
   ierr = RGIsTrivial(pep->rg,&istrivial);CHKERRQ(ierr);
   if (!istrivial) {
     if (transf) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"PEPLINEAR does not support a nontrivial region with st-transform");
@@ -428,32 +412,36 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
   }
 
   /* process initial vector */
-  if (pep->nini<=-deg) {
+  if (pep->nini<0) {
     ierr = VecCreateMPI(PetscObjectComm((PetscObject)ctx->eps),deg*pep->nloc,deg*pep->n,&veps);CHKERRQ(ierr);
     ierr = VecGetArray(veps,&epsarray);CHKERRQ(ierr);
     for (i=0;i<deg;i++) {
-      ierr = VecGetArray(pep->IS[i],&peparray);CHKERRQ(ierr);
-      ierr = PetscMemcpy(epsarray+i*pep->nloc,peparray,pep->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = VecRestoreArray(pep->IS[i],&peparray);CHKERRQ(ierr);
+      if (i<-pep->nini) {
+        ierr = VecGetArray(pep->IS[i],&peparray);CHKERRQ(ierr);
+        ierr = PetscMemcpy(epsarray+i*pep->nloc,peparray,pep->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = VecRestoreArray(pep->IS[i],&peparray);CHKERRQ(ierr);
+      } else {
+        if (!w) { ierr = VecDuplicate(pep->IS[0],&w);CHKERRQ(ierr); }
+        ierr = VecSetRandom(w,NULL);CHKERRQ(ierr);
+        ierr = VecGetArray(w,&peparray);CHKERRQ(ierr);
+        ierr = PetscMemcpy(epsarray+i*pep->nloc,peparray,pep->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = VecRestoreArray(w,&peparray);CHKERRQ(ierr);
+      }
     }
     ierr = VecRestoreArray(veps,&epsarray);CHKERRQ(ierr);
     ierr = EPSSetInitialSpace(ctx->eps,1,&veps);CHKERRQ(ierr);
     ierr = VecDestroy(&veps);CHKERRQ(ierr);
-  }
-  if (pep->nini<0) {
+    ierr = VecDestroy(&w);CHKERRQ(ierr);
     ierr = SlepcBasisDestroy_Private(&pep->nini,&pep->IS);CHKERRQ(ierr);
   }
 
   ierr = EPSSetUp(ctx->eps);CHKERRQ(ierr);
   ierr = EPSGetDimensions(ctx->eps,NULL,&pep->ncv,&pep->mpd);CHKERRQ(ierr);
   ierr = EPSGetTolerances(ctx->eps,NULL,&pep->max_it);CHKERRQ(ierr);
-  if (pep->nini>0) { ierr = PetscInfo(pep,"Ignoring initial vectors\n");CHKERRQ(ierr); }
   ierr = PEPAllocateSolution(pep,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearExtract_Residual"
 /*
    PEPLinearExtract_Residual - Auxiliary routine that copies the solution of the
    linear eigenproblem to the PEP object. The eigenvector of the generalized
@@ -486,10 +474,10 @@ static PetscErrorCode PEPLinearExtract_Residual(PEP pep,EPS eps)
 #endif
   ierr = EPSGetOperators(eps,&A,NULL);CHKERRQ(ierr);
   ierr = MatCreateVecs(A,&xr,NULL);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&wr);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(pep->A[0],&wr,NULL);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
   ierr = VecDuplicate(xr,&xi);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&wi);CHKERRQ(ierr);
+  ierr = VecDuplicateEmpty(wr,&wi);CHKERRQ(ierr);
 #endif
   for (i=0;i<pep->nconv;i++) {
     ierr = EPSGetEigenpair(eps,i,NULL,NULL,xr,xi);CHKERRQ(ierr);
@@ -499,7 +487,7 @@ static PetscErrorCode PEPLinearExtract_Residual(PEP pep,EPS eps)
       ierr = VecGetArrayRead(xi,&py);CHKERRQ(ierr);
       ierr = VecPlaceArray(wr,px);CHKERRQ(ierr);
       ierr = VecPlaceArray(wi,py);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(wr,wi,PETSC_TRUE,NULL);CHKERRQ(ierr);
+      ierr = VecNormalizeComplex(wr,wi,PETSC_TRUE,NULL);CHKERRQ(ierr);
       ierr = PEPComputeResidualNorm_Private(pep,er[i],ei[i],wr,wi,pep->work,&rn1);CHKERRQ(ierr);
       ierr = BVInsertVec(pep->V,i,wr);CHKERRQ(ierr);
       ierr = BVInsertVec(pep->V,i+1,wi);CHKERRQ(ierr);
@@ -508,7 +496,7 @@ static PetscErrorCode PEPLinearExtract_Residual(PEP pep,EPS eps)
         ierr = VecResetArray(wi);CHKERRQ(ierr);
         ierr = VecPlaceArray(wr,px+k*pep->nloc);CHKERRQ(ierr);
         ierr = VecPlaceArray(wi,py+k*pep->nloc);CHKERRQ(ierr);
-        ierr = SlepcVecNormalize(wr,wi,PETSC_TRUE,NULL);CHKERRQ(ierr);
+        ierr = VecNormalizeComplex(wr,wi,PETSC_TRUE,NULL);CHKERRQ(ierr);
         ierr = PEPComputeResidualNorm_Private(pep,er[i],ei[i],wr,wi,pep->work,&rn2);CHKERRQ(ierr);
         if (rn1>rn2) {
           ierr = BVInsertVec(pep->V,i,wr);CHKERRQ(ierr);
@@ -526,13 +514,13 @@ static PetscErrorCode PEPLinearExtract_Residual(PEP pep,EPS eps)
     {
       ierr = VecGetArrayRead(xr,&px);CHKERRQ(ierr);
       ierr = VecPlaceArray(wr,px);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(wr,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
+      ierr = VecNormalizeComplex(wr,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
       ierr = PEPComputeResidualNorm_Private(pep,er[i],ei[i],wr,NULL,pep->work,&rn1);CHKERRQ(ierr);
       ierr = BVInsertVec(pep->V,i,wr);CHKERRQ(ierr);
       for (k=1;k<pep->nmat-1;k++) {
         ierr = VecResetArray(wr);CHKERRQ(ierr);
         ierr = VecPlaceArray(wr,px+k*pep->nloc);CHKERRQ(ierr);
-        ierr = SlepcVecNormalize(wr,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
+        ierr = VecNormalizeComplex(wr,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
         ierr = PEPComputeResidualNorm_Private(pep,er[i],ei[i],wr,NULL,pep->work,&rn2);CHKERRQ(ierr);
         if (rn1>rn2) {
           ierr = BVInsertVec(pep->V,i,wr);CHKERRQ(ierr);
@@ -552,8 +540,6 @@ static PetscErrorCode PEPLinearExtract_Residual(PEP pep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearExtract_None"
 /*
    PEPLinearExtract_None - Same as PEPLinearExtract_Norm but always takes
    the first block.
@@ -573,7 +559,7 @@ static PetscErrorCode PEPLinearExtract_None(PEP pep,EPS eps)
   ierr = EPSGetOperators(eps,&A,NULL);CHKERRQ(ierr);
   ierr = MatCreateVecs(A,&xr,NULL);CHKERRQ(ierr);
   ierr = VecDuplicate(xr,&xi);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&w);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(pep->A[0],&w,NULL);CHKERRQ(ierr);
   for (i=0;i<pep->nconv;i++) {
     ierr = EPSGetEigenpair(eps,i,NULL,NULL,xr,xi);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
@@ -605,8 +591,6 @@ static PetscErrorCode PEPLinearExtract_None(PEP pep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearExtract_Norm"
 /*
    PEPLinearExtract_Norm - Auxiliary routine that copies the solution of the
    linear eigenproblem to the PEP object. The eigenvector of the generalized
@@ -634,7 +618,7 @@ static PetscErrorCode PEPLinearExtract_Norm(PEP pep,EPS eps)
 #if !defined(PETSC_USE_COMPLEX)
   ierr = VecDuplicate(xr,&xi);CHKERRQ(ierr);
 #endif
-  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)pep),1,pep->nloc,pep->n,NULL,&w);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(pep->A[0],&w,NULL);CHKERRQ(ierr);
   for (i=0;i<pep->nconv;i++) {
     ierr = EPSGetEigenpair(eps,i,NULL,NULL,xr,xi);CHKERRQ(ierr);
     if (SlepcAbsEigenvalue(er[i],ei[i])>1.0) offset = (pep->nmat-2)*pep->nloc;
@@ -670,13 +654,11 @@ static PetscErrorCode PEPLinearExtract_Norm(PEP pep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPExtractVectors_Linear"
 PetscErrorCode PEPExtractVectors_Linear(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_LINEAR     *ctx = (PEP_LINEAR*)pep->data;
-  
+
   PetscFunctionBegin;
   switch (pep->extract) {
   case PEP_EXTRACT_NONE:
@@ -688,14 +670,12 @@ PetscErrorCode PEPExtractVectors_Linear(PEP pep)
   case PEP_EXTRACT_RESIDUAL:
     ierr = PEPLinearExtract_Residual(pep,ctx->eps);CHKERRQ(ierr);
     break;
-  default:
+  case PEP_EXTRACT_STRUCTURED:
     SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Extraction not implemented in this solver");
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSolve_Linear"
 PetscErrorCode PEPSolve_Linear(PEP pep)
 {
   PetscErrorCode ierr;
@@ -733,16 +713,14 @@ PetscErrorCode PEPSolve_Linear(PEP pep)
     }
     if (!flg && !ctx->explicitmatrix) {
       ierr = STScaleShift(pep->st,pep->sfactor);CHKERRQ(ierr);
-    } 
+    }
   }
-  if (ctx->explicitmatrix) {
+  if (ctx->explicitmatrix || !flg) {
     ierr = RGPopScale(pep->rg);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSMonitor_Linear"
 static PetscErrorCode EPSMonitor_Linear(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
   PEP            pep = (PEP)ctx;
@@ -753,8 +731,6 @@ static PetscErrorCode EPSMonitor_Linear(EPS eps,PetscInt its,PetscInt nconv,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetFromOptions_Linear"
 PetscErrorCode PEPSetFromOptions_Linear(PetscOptionItems *PetscOptionsObject,PEP pep)
 {
   PetscErrorCode ierr;
@@ -764,22 +740,20 @@ PetscErrorCode PEPSetFromOptions_Linear(PetscOptionItems *PetscOptionsObject,PEP
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"PEP Linear Options");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-pep_linear_cform","Number of the companion form","PEPLinearSetCompanionForm",ctx->cform,&i,&set);CHKERRQ(ierr);
-  if (set) {
-    ierr = PEPLinearSetCompanionForm(pep,i);CHKERRQ(ierr);
-  }
-  ierr = PetscOptionsBool("-pep_linear_explicitmatrix","Use explicit matrix in linearization","PEPLinearSetExplicitMatrix",ctx->explicitmatrix,&val,&set);CHKERRQ(ierr);
-  if (set) {
-    ierr = PEPLinearSetExplicitMatrix(pep,val);CHKERRQ(ierr);
-  }
+
+    ierr = PetscOptionsInt("-pep_linear_cform","Number of the companion form (1 or 2)","PEPLinearSetCompanionForm",ctx->cform,&i,&set);CHKERRQ(ierr);
+    if (set) { ierr = PEPLinearSetCompanionForm(pep,i);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsBool("-pep_linear_explicitmatrix","Use explicit matrix in linearization","PEPLinearSetExplicitMatrix",ctx->explicitmatrix,&val,&set);CHKERRQ(ierr);
+    if (set) { ierr = PEPLinearSetExplicitMatrix(pep,val);CHKERRQ(ierr); }
+
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
+
   if (!ctx->eps) { ierr = PEPLinearGetEPS(pep,&ctx->eps);CHKERRQ(ierr); }
   ierr = EPSSetFromOptions(ctx->eps);CHKERRQ(ierr);
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearSetCompanionForm_Linear"
 static PetscErrorCode PEPLinearSetCompanionForm_Linear(PEP pep,PetscInt cform)
 {
   PEP_LINEAR *ctx = (PEP_LINEAR*)pep->data;
@@ -794,8 +768,6 @@ static PetscErrorCode PEPLinearSetCompanionForm_Linear(PEP pep,PetscInt cform)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearSetCompanionForm"
 /*@
    PEPLinearSetCompanionForm - Choose between the two companion forms available
    for the linearization of a quadratic eigenproblem.
@@ -824,8 +796,6 @@ PetscErrorCode PEPLinearSetCompanionForm(PEP pep,PetscInt cform)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearGetCompanionForm_Linear"
 static PetscErrorCode PEPLinearGetCompanionForm_Linear(PEP pep,PetscInt *cform)
 {
   PEP_LINEAR *ctx = (PEP_LINEAR*)pep->data;
@@ -835,8 +805,6 @@ static PetscErrorCode PEPLinearGetCompanionForm_Linear(PEP pep,PetscInt *cform)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearGetCompanionForm"
 /*@
    PEPLinearGetCompanionForm - Returns the number of the companion form that
    will be used for the linearization of a quadratic eigenproblem.
@@ -864,19 +832,18 @@ PetscErrorCode PEPLinearGetCompanionForm(PEP pep,PetscInt *cform)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearSetExplicitMatrix_Linear"
 static PetscErrorCode PEPLinearSetExplicitMatrix_Linear(PEP pep,PetscBool explicitmatrix)
 {
   PEP_LINEAR *ctx = (PEP_LINEAR*)pep->data;
 
   PetscFunctionBegin;
-  ctx->explicitmatrix = explicitmatrix;
+  if (ctx->explicitmatrix != explicitmatrix) {
+    ctx->explicitmatrix = explicitmatrix;
+    pep->state = PEP_STATE_INITIAL;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearSetExplicitMatrix"
 /*@
    PEPLinearSetExplicitMatrix - Indicate if the matrices A and B for the
    linearization of the problem must be built explicitly.
@@ -905,8 +872,6 @@ PetscErrorCode PEPLinearSetExplicitMatrix(PEP pep,PetscBool explicitmatrix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearGetExplicitMatrix_Linear"
 static PetscErrorCode PEPLinearGetExplicitMatrix_Linear(PEP pep,PetscBool *explicitmatrix)
 {
   PEP_LINEAR *ctx = (PEP_LINEAR*)pep->data;
@@ -916,8 +881,6 @@ static PetscErrorCode PEPLinearGetExplicitMatrix_Linear(PEP pep,PetscBool *expli
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearGetExplicitMatrix"
 /*@
    PEPLinearGetExplicitMatrix - Returns the flag indicating if the matrices
    A and B for the linearization are built explicitly.
@@ -945,8 +908,6 @@ PetscErrorCode PEPLinearGetExplicitMatrix(PEP pep,PetscBool *explicitmatrix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearSetEPS_Linear"
 static PetscErrorCode PEPLinearSetEPS_Linear(PEP pep,EPS eps)
 {
   PetscErrorCode ierr;
@@ -956,13 +917,12 @@ static PetscErrorCode PEPLinearSetEPS_Linear(PEP pep,EPS eps)
   ierr = PetscObjectReference((PetscObject)eps);CHKERRQ(ierr);
   ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr);
   ctx->eps = eps;
+  ctx->usereps = PETSC_TRUE;
   ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)ctx->eps);CHKERRQ(ierr);
   pep->state = PEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearSetEPS"
 /*@
    PEPLinearSetEPS - Associate an eigensolver object (EPS) to the
    polynomial eigenvalue solver.
@@ -989,21 +949,16 @@ PetscErrorCode PEPLinearSetEPS(PEP pep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearGetEPS_Linear"
 static PetscErrorCode PEPLinearGetEPS_Linear(PEP pep,EPS *eps)
 {
   PetscErrorCode ierr;
   PEP_LINEAR     *ctx = (PEP_LINEAR*)pep->data;
-  ST             st;
 
   PetscFunctionBegin;
   if (!ctx->eps) {
     ierr = EPSCreate(PetscObjectComm((PetscObject)pep),&ctx->eps);CHKERRQ(ierr);
     ierr = EPSSetOptionsPrefix(ctx->eps,((PetscObject)pep)->prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(ctx->eps,"pep_linear_");CHKERRQ(ierr);
-    ierr = EPSGetST(ctx->eps,&st);CHKERRQ(ierr);
-    ierr = STSetOptionsPrefix(st,((PetscObject)ctx->eps)->prefix);CHKERRQ(ierr);
     ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->eps,(PetscObject)pep,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)ctx->eps);CHKERRQ(ierr);
     ierr = EPSMonitorSet(ctx->eps,EPSMonitor_Linear,pep,NULL);CHKERRQ(ierr);
@@ -1012,8 +967,6 @@ static PetscErrorCode PEPLinearGetEPS_Linear(PEP pep,EPS *eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPLinearGetEPS"
 /*@
    PEPLinearGetEPS - Retrieve the eigensolver object (EPS) associated
    to the polynomial eigenvalue solver.
@@ -1041,8 +994,6 @@ PetscErrorCode PEPLinearGetEPS(PEP pep,EPS *eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPView_Linear"
 PetscErrorCode PEPView_Linear(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -1053,8 +1004,8 @@ PetscErrorCode PEPView_Linear(PEP pep,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     if (!ctx->eps) { ierr = PEPLinearGetEPS(pep,&ctx->eps);CHKERRQ(ierr); }
-    ierr = PetscViewerASCIIPrintf(viewer,"  Linear: %s matrices\n",ctx->explicitmatrix? "explicit": "implicit");CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  Linear: %s companion form\n",ctx->cform==1? "1st": "2nd");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %s matrices\n",ctx->explicitmatrix? "explicit": "implicit");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %s companion form\n",ctx->cform==1? "1st": "2nd");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = EPSView(ctx->eps,viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
@@ -1062,8 +1013,6 @@ PetscErrorCode PEPView_Linear(PEP pep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPReset_Linear"
 PetscErrorCode PEPReset_Linear(PEP pep)
 {
   PetscErrorCode ierr;
@@ -1082,8 +1031,6 @@ PetscErrorCode PEPReset_Linear(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPDestroy_Linear"
 PetscErrorCode PEPDestroy_Linear(PEP pep)
 {
   PetscErrorCode ierr;
@@ -1101,8 +1048,6 @@ PetscErrorCode PEPDestroy_Linear(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPCreate_Linear"
 PETSC_EXTERN PetscErrorCode PEPCreate_Linear(PEP pep)
 {
   PetscErrorCode ierr;
@@ -1122,6 +1067,7 @@ PETSC_EXTERN PetscErrorCode PEPCreate_Linear(PEP pep)
   pep->ops->backtransform  = PEPBackTransform_Default;
   pep->ops->computevectors = PEPComputeVectors_Default;
   pep->ops->extractvectors = PEPExtractVectors_Linear;
+
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPLinearSetCompanionForm_C",PEPLinearSetCompanionForm_Linear);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPLinearGetCompanionForm_C",PEPLinearGetCompanionForm_Linear);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPLinearSetEPS_C",PEPLinearSetEPS_Linear);CHKERRQ(ierr);
