@@ -1,25 +1,15 @@
 /*
-   The basic PEP routines, Create, Destroy, etc. are here.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic PEP routines
+*/
 
 #include <slepc/private/pepimpl.h>      /*I "slepcpep.h" I*/
 
@@ -28,8 +18,6 @@ PetscBool         PEPRegisterAllCalled = PETSC_FALSE;
 PetscClassId      PEP_CLASSID = 0;
 PetscLogEvent     PEP_SetUp = 0,PEP_Solve = 0,PEP_Refine = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPCreate"
 /*@
    PEPCreate - Creates the default PEP context.
 
@@ -85,8 +73,10 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
   pep->trackall        = PETSC_FALSE;
 
   pep->converged       = PEPConvergedRelative;
+  pep->convergeduser   = NULL;
   pep->convergeddestroy= NULL;
   pep->stopping        = PEPStoppingBasic;
+  pep->stoppinguser    = NULL;
   pep->stoppingdestroy = NULL;
   pep->convergedctx    = NULL;
   pep->stoppingctx     = NULL;
@@ -128,8 +118,6 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetType"
 /*@C
    PEPSetType - Selects the particular solver to be used in the PEP object.
 
@@ -183,8 +171,6 @@ PetscErrorCode PEPSetType(PEP pep,PEPType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetType"
 /*@C
    PEPGetType - Gets the PEP type as a string from the PEP object.
 
@@ -209,8 +195,6 @@ PetscErrorCode PEPGetType(PEP pep,PEPType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPRegister"
 /*@C
    PEPRegister - Adds a method to the polynomial eigenproblem solver package.
 
@@ -246,11 +230,9 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPReset"
 /*@
-   PEPReset - Resets the PEP context to the initial state and removes any
-   allocated objects.
+   PEPReset - Resets the PEP context to the initial state (prior to setup)
+   and destroys any allocated Vecs and Mats.
 
    Collective on PEP
 
@@ -264,13 +246,13 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
 PetscErrorCode PEPReset(PEP pep)
 {
   PetscErrorCode ierr;
-  PetscInt       ncols;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  if (pep) PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  if (!pep) PetscFunctionReturn(0);
   if (pep->ops->reset) { ierr = (pep->ops->reset)(pep);CHKERRQ(ierr); }
   if (pep->st) { ierr = STReset(pep->st);CHKERRQ(ierr); }
-  if (pep->ds) { ierr = DSReset(pep->ds);CHKERRQ(ierr); }
+  if (pep->refineksp) { ierr = KSPReset(pep->refineksp);CHKERRQ(ierr); }
   if (pep->nmat) {
     ierr = MatDestroyMatrices(pep->nmat,&pep->A);CHKERRQ(ierr);
     ierr = PetscFree2(pep->pbc,pep->nrma);CHKERRQ(ierr);
@@ -279,21 +261,13 @@ PetscErrorCode PEPReset(PEP pep)
   }
   ierr = VecDestroy(&pep->Dl);CHKERRQ(ierr);
   ierr = VecDestroy(&pep->Dr);CHKERRQ(ierr);
-  ierr = BVGetSizes(pep->V,NULL,NULL,&ncols);CHKERRQ(ierr);
-  if (ncols) {
-    ierr = PetscFree4(pep->eigr,pep->eigi,pep->errest,pep->perm);CHKERRQ(ierr);
-  }
   ierr = BVDestroy(&pep->V);CHKERRQ(ierr);
   ierr = VecDestroyVecs(pep->nwork,&pep->work);CHKERRQ(ierr);
-  ierr = KSPDestroy(&pep->refineksp);CHKERRQ(ierr);
-  ierr = PetscSubcommDestroy(&pep->refinesubc);CHKERRQ(ierr);
   pep->nwork = 0;
   pep->state = PEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPDestroy"
 /*@
    PEPDestroy - Destroys the PEP context.
 
@@ -316,9 +290,14 @@ PetscErrorCode PEPDestroy(PEP *pep)
   if (--((PetscObject)(*pep))->refct > 0) { *pep = 0; PetscFunctionReturn(0); }
   ierr = PEPReset(*pep);CHKERRQ(ierr);
   if ((*pep)->ops->destroy) { ierr = (*(*pep)->ops->destroy)(*pep);CHKERRQ(ierr); }
+  if ((*pep)->eigr) {
+    ierr = PetscFree4((*pep)->eigr,(*pep)->eigi,(*pep)->errest,(*pep)->perm);CHKERRQ(ierr);
+  }
   ierr = STDestroy(&(*pep)->st);CHKERRQ(ierr);
   ierr = RGDestroy(&(*pep)->rg);CHKERRQ(ierr);
   ierr = DSDestroy(&(*pep)->ds);CHKERRQ(ierr);
+  ierr = KSPDestroy(&(*pep)->refineksp);CHKERRQ(ierr);
+  ierr = PetscSubcommDestroy(&(*pep)->refinesubc);CHKERRQ(ierr);
   ierr = PetscFree((*pep)->sc);CHKERRQ(ierr);
   /* just in case the initial vectors have not been used */
   ierr = SlepcBasisDestroy_Private(&(*pep)->nini,&(*pep)->IS);CHKERRQ(ierr);
@@ -330,8 +309,6 @@ PetscErrorCode PEPDestroy(PEP *pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetBV"
 /*@
    PEPSetBV - Associates a basis vectors object to the polynomial eigensolver.
 
@@ -364,8 +341,6 @@ PetscErrorCode PEPSetBV(PEP pep,BV bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetBV"
 /*@
    PEPGetBV - Obtain the basis vectors object associated to the polynomial
    eigensolver object.
@@ -397,8 +372,6 @@ PetscErrorCode PEPGetBV(PEP pep,BV *bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetRG"
 /*@
    PEPSetRG - Associates a region object to the polynomial eigensolver.
 
@@ -431,8 +404,6 @@ PetscErrorCode PEPSetRG(PEP pep,RG rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetRG"
 /*@
    PEPGetRG - Obtain the region object associated to the
    polynomial eigensolver object.
@@ -464,8 +435,6 @@ PetscErrorCode PEPGetRG(PEP pep,RG *rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetDS"
 /*@
    PEPSetDS - Associates a direct solver object to the polynomial eigensolver.
 
@@ -498,8 +467,6 @@ PetscErrorCode PEPSetDS(PEP pep,DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetDS"
 /*@
    PEPGetDS - Obtain the direct solver object associated to the
    polynomial eigensolver object.
@@ -531,8 +498,6 @@ PetscErrorCode PEPGetDS(PEP pep,DS *ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetST"
 /*@
    PEPSetST - Associates a spectral transformation object to the eigensolver.
 
@@ -565,8 +530,6 @@ PetscErrorCode PEPSetST(PEP pep,ST st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetST"
 /*@
    PEPGetST - Obtain the spectral transformation (ST) object associated
    to the eigensolver object.
@@ -598,8 +561,6 @@ PetscErrorCode PEPGetST(PEP pep,ST *st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPRefineGetKSP"
 /*@
    PEPRefineGetKSP - Obtain the ksp object used by the eigensolver
    object in the refinement phase.
@@ -635,13 +596,12 @@ PetscErrorCode PEPRefineGetKSP(PEP pep,KSP *ksp)
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->refineksp);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)pep)->prefix);CHKERRQ(ierr);
     ierr = KSPAppendOptionsPrefix(*ksp,"pep_refine_");CHKERRQ(ierr);
+    ierr = KSPSetTolerances(pep->refineksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
   }
   *ksp = pep->refineksp;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetTarget"
 /*@
    PEPSetTarget - Sets the value of the target.
 
@@ -679,8 +639,6 @@ PetscErrorCode PEPSetTarget(PEP pep,PetscScalar target)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetTarget"
 /*@
    PEPGetTarget - Gets the value of the target.
 
