@@ -1,30 +1,18 @@
 /*
-     This file contains some simple default routines for common PEP operations.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Simple default routines for common PEP operations
+*/
 
 #include <slepc/private/pepimpl.h>     /*I "slepcpep.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetWorkVecs"
 /*@
    PEPSetWorkVecs - Sets a number of work vectors into a PEP object.
 
@@ -57,8 +45,6 @@ PetscErrorCode PEPSetWorkVecs(PEP pep,PetscInt nw)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPConvergedRelative"
 /*
   PEPConvergedRelative - Checks convergence relative to the eigenvalue.
 */
@@ -72,8 +58,6 @@ PetscErrorCode PEPConvergedRelative(PEP pep,PetscScalar eigr,PetscScalar eigi,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPConvergedNorm"
 /*
   PEPConvergedNorm - Checks convergence relative to the matrix norms.
 */
@@ -101,8 +85,6 @@ PetscErrorCode PEPConvergedNorm(PEP pep,PetscScalar eigr,PetscScalar eigi,PetscR
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPConvergedAbsolute"
 /*
   PEPConvergedAbsolute - Checks convergence absolutely.
 */
@@ -113,8 +95,6 @@ PetscErrorCode PEPConvergedAbsolute(PEP pep,PetscScalar eigr,PetscScalar eigi,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPStoppingBasic"
 /*@C
    PEPStoppingBasic - Default routine to determine whether the outer eigensolver
    iteration must be stopped.
@@ -163,8 +143,6 @@ PetscErrorCode PEPStoppingBasic(PEP pep,PetscInt its,PetscInt max_it,PetscInt nc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPBackTransform_Default"
 PetscErrorCode PEPBackTransform_Default(PEP pep)
 {
   PetscErrorCode ierr;
@@ -174,8 +152,6 @@ PetscErrorCode PEPBackTransform_Default(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPComputeVectors_Default"
 PetscErrorCode PEPComputeVectors_Default(PEP pep)
 {
   PetscErrorCode ierr;
@@ -203,7 +179,7 @@ PetscErrorCode PEPComputeVectors_Default(PEP pep)
     if (pep->eigi[i]!=0.0) {   /* first eigenvalue of a complex conjugate pair */
       ierr = BVGetColumn(pep->V,i,&v);CHKERRQ(ierr);
       ierr = BVGetColumn(pep->V,i+1,&v1);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(v,v1,PETSC_TRUE,NULL);CHKERRQ(ierr);
+      ierr = VecNormalizeComplex(v,v1,PETSC_TRUE,NULL);CHKERRQ(ierr);
       ierr = BVRestoreColumn(pep->V,i,&v);CHKERRQ(ierr);
       ierr = BVRestoreColumn(pep->V,i+1,&v1);CHKERRQ(ierr);
       i++;
@@ -211,15 +187,13 @@ PetscErrorCode PEPComputeVectors_Default(PEP pep)
 #endif
     {
       ierr = BVGetColumn(pep->V,i,&v);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(v,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
+      ierr = VecNormalizeComplex(v,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
       ierr = BVRestoreColumn(pep->V,i,&v);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPKrylovConvergence"
 /*
    PEPKrylovConvergence - This is the analogue to EPSKrylovConvergence, but
    for polynomial Krylov methods.
@@ -270,10 +244,8 @@ PetscErrorCode PEPKrylovConvergence(PEP pep,PetscBool getall,PetscInt kini,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPBuildDiagonalScaling"
 /*
-  PEPBuildDiagonalScaling - compute two diagonal matrices to be applied for balancing 
+  PEPBuildDiagonalScaling - compute two diagonal matrices to be applied for balancing
   in polynomial eigenproblems.
 */
 PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
@@ -296,7 +268,7 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
   ierr = STGetMatStructure(pep->st,&str);CHKERRQ(ierr);
   ierr = PetscMalloc1(nmat,&T);CHKERRQ(ierr);
   for (k=0;k<nmat;k++) {
-    ierr = STGetTOperators(pep->st,k,&T[k]);CHKERRQ(ierr);
+    ierr = STGetMatrixTransformed(pep->st,k,&T[k]);CHKERRQ(ierr);
   }
   /* Form local auxiliar matrix M */
   ierr = PetscObjectTypeCompareAny((PetscObject)T[0],&cont,MATMPIAIJ,MATSEQAIJ,"");CHKERRQ(ierr);
@@ -304,7 +276,7 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
   ierr = PetscObjectTypeCompare((PetscObject)T[0],MATMPIAIJ,&cont);CHKERRQ(ierr);
   if (cont) {
     ierr = MatMPIAIJGetLocalMat(T[0],MAT_INITIAL_MATRIX,&M);CHKERRQ(ierr);
-    flg = PETSC_TRUE; 
+    flg = PETSC_TRUE;
   } else {
     ierr = MatDuplicate(T[0],MAT_COPY_VALUES,&M);CHKERRQ(ierr);
   }
@@ -338,10 +310,10 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
     ierr = MatAXPY(M,w,A,str);CHKERRQ(ierr);
     if (flg || str!=SAME_NONZERO_PATTERN || k==nmat-2) {
       ierr = MatDestroy(&A);CHKERRQ(ierr);
-    } 
+    }
   }
   ierr = MatGetRowIJ(M,0,PETSC_FALSE,PETSC_FALSE,&nr,&ridx,&cidx,&cont);CHKERRQ(ierr);
-  if (!cont) SETERRQ(PetscObjectComm((PetscObject)T[0]), PETSC_ERR_SUP,"It is not possible to compute scaling diagonals for these PEP matrices");
+  if (!cont) SETERRQ(PetscObjectComm((PetscObject)T[0]),PETSC_ERR_SUP,"It is not possible to compute scaling diagonals for these PEP matrices");
   ierr = MatGetInfo(M,MAT_LOCAL,&info);CHKERRQ(ierr);
   nz = (PetscInt)info.nz_used;
   ierr = VecGetOwnershipRange(pep->Dl,&lst,&lend);CHKERRQ(ierr);
@@ -360,12 +332,12 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
   }
   for (it=0;it<pep->sits && cont;it++) {
     emaxl = 0; eminl = 0;
-    /* Column sum  */    
+    /* Column sum  */
     if (it>0) { /* it=0 has been already done*/
       ierr = MatSeqAIJGetArray(M,&array);CHKERRQ(ierr);
       ierr = PetscMemzero(aux,pep->n*sizeof(PetscReal));CHKERRQ(ierr);
       for (j=0;j<nz;j++) aux[cidx[j]] += PetscAbsScalar(array[j]);
-      ierr = MatSeqAIJRestoreArray(M,&array);CHKERRQ(ierr); 
+      ierr = MatSeqAIJRestoreArray(M,&array);CHKERRQ(ierr);
     }
     ierr = MPI_Allreduce(aux,csum,n,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)pep->Dr));CHKERRQ(ierr);
     /* Update Dr */
@@ -392,7 +364,7 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
       array[j] *= aux[cidx[j]];
     }
     ierr = MatSeqAIJRestoreArray(M,&array);CHKERRQ(ierr);
-    /* Row sum */    
+    /* Row sum */
     ierr = PetscMemzero(rsum,nr*sizeof(PetscReal));CHKERRQ(ierr);
     ierr = MatSeqAIJGetArray(M,&array);CHKERRQ(ierr);
     for (i=0;i<nr;i++) {
@@ -405,9 +377,9 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
       /* Scale M */
       for (j=ridx[i];j<ridx[i+1];j++) array[j] *= d*d;
       emaxl = PetscMax(emaxl,e);
-      eminl = PetscMin(eminl,e);      
+      eminl = PetscMin(eminl,e);
     }
-    ierr = MatSeqAIJRestoreArray(M,&array);CHKERRQ(ierr);  
+    ierr = MatSeqAIJRestoreArray(M,&array);CHKERRQ(ierr);
     /* Compute global max and min */
     ierr = MPI_Allreduce(&emaxl,&emax,1,MPIU_INT,MPI_MAX,PetscObjectComm((PetscObject)pep->Dl));CHKERRQ(ierr);
     ierr = MPI_Allreduce(&eminl,&emin,1,MPIU_INT,MPI_MIN,PetscObjectComm((PetscObject)pep->Dl));CHKERRQ(ierr);
@@ -422,8 +394,6 @@ PetscErrorCode PEPBuildDiagonalScaling(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPComputeScaleFactor"
 /*
    PEPComputeScaleFactor - compute sfactor as described in [Betcke 2008].
 */
@@ -449,8 +419,8 @@ PetscErrorCode PEPComputeScaleFactor(PEP pep)
   if (basis==PEP_BASIS_MONOMIAL) {
     ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
     if (flg) {
-      ierr = STGetTOperators(pep->st,0,&T[0]);CHKERRQ(ierr);
-      ierr = STGetTOperators(pep->st,pep->nmat-1,&T[1]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,0,&T[0]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,pep->nmat-1,&T[1]);CHKERRQ(ierr);
     } else {
       T[0] = pep->A[0];
       T[1] = pep->A[pep->nmat-1];
@@ -464,7 +434,7 @@ PetscErrorCode PEPComputeScaleFactor(PEP pep)
         pep->sfactor = PetscPowReal(norm0/norm1,1.0/(pep->nmat-1));
         pep->dsfactor = norm1;
         for (i=pep->nmat-2;i>0;i--) {
-          ierr = STGetTOperators(pep->st,i,&T[1]);CHKERRQ(ierr);
+          ierr = STGetMatrixTransformed(pep->st,i,&T[1]);CHKERRQ(ierr);
           ierr = MatHasOperation(T[1],MATOP_NORM,&has1);CHKERRQ(ierr);
           if (has1) {
             ierr = MatNorm(T[1],NORM_INFINITY,&norm1);CHKERRQ(ierr);
@@ -475,14 +445,12 @@ PetscErrorCode PEPComputeScaleFactor(PEP pep)
           pep->dsfactor = pep->dsfactor*pep->sfactor+norm0;
           pep->dsfactor = pep->nmat/pep->dsfactor;
         } else pep->dsfactor = 1.0;
-      } 
+      }
     }
-  } 
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPBasisCoefficients"
 /*
    PEPBasisCoefficients - compute polynomial basis coefficients
 */
@@ -490,7 +458,7 @@ PetscErrorCode PEPBasisCoefficients(PEP pep,PetscReal *pbc)
 {
   PetscReal *ca,*cb,*cg;
   PetscInt  k,nmat=pep->nmat;
-  
+
   PetscFunctionBegin;
   ca = pbc;
   cb = pbc+nmat;
@@ -511,7 +479,7 @@ PetscErrorCode PEPBasisCoefficients(PEP pep,PetscReal *pbc)
     ca[0] = .5; cb[0] = 0.0; cg[0] = 0.0;
     for (k=1;k<nmat;k++) {
       ca[k] = .5; cb[k] = 0.0; cg[k] = .5;
-    }    
+    }
     break;
   case PEP_BASIS_LEGENDRE:
     ca[0] = 1.0; cb[0] = 0.0; cg[0] = 0.0;
@@ -531,14 +499,10 @@ PetscErrorCode PEPBasisCoefficients(PEP pep,PetscReal *pbc)
       ca[k] = .5; cb[k] = 0.0; cg[k] = -k;
     }
     break;
-  default:
-    SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'basis' value");
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPEvaluateBasis"
 /*
    PEPEvaluateBasis - evaluate the polynomial basis on a given parameter sigma
 */
@@ -546,7 +510,7 @@ PetscErrorCode PEPEvaluateBasis(PEP pep,PetscScalar sigma,PetscScalar isigma,Pet
 {
   PetscInt   nmat=pep->nmat,k;
   PetscReal  *a=pep->pbc,*b=pep->pbc+nmat,*g=pep->pbc+2*nmat;
-  
+
   PetscFunctionBegin;
   if (ivals) for (k=0;k<nmat;k++) ivals[k] = 0.0;
   vals[0] = 1.0;
