@@ -1,32 +1,20 @@
 /*
-      PEP routines related to options that can be set via the command-line
-      or procedurally.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   PEP routines related to options that can be set via the command-line
+   or procedurally
+*/
 
 #include <slepc/private/pepimpl.h>       /*I "slepcpep.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPMonitorSetFromOptions"
 /*@C
    PEPMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
@@ -66,8 +54,6 @@ PetscErrorCode PEPMonitorSetFromOptions(PEP pep,const char name[],const char hel
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPConvMonitorSetFromOptions"
 /*@C
    PEPConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
@@ -103,8 +89,6 @@ PetscErrorCode PEPConvMonitorSetFromOptions(PEP pep,const char name[],const char
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetFromOptions"
 /*@
    PEPSetFromOptions - Sets PEP options from the options database.
    This routine must be called before PEPSetUp() if the user is to be
@@ -122,71 +106,69 @@ PetscErrorCode PEPConvMonitorSetFromOptions(PEP pep,const char name[],const char
 @*/
 PetscErrorCode PEPSetFromOptions(PEP pep)
 {
-  PetscErrorCode ierr;
-  char           type[256];
-  PetscBool      set,flg,flg1,flg2,flg3;
-  PetscReal      r,t;
-  PetscScalar    s;
-  PetscInt       i,j,k;
-  PetscDrawLG    lg;
+  PetscErrorCode  ierr;
+  char            type[256];
+  PetscBool       set,flg,flg1,flg2,flg3,flg4,flg5;
+  PetscReal       r,t;
+  PetscScalar     s;
+  PetscInt        i,j,k;
+  PetscDrawLG     lg;
+  PEPScale        scale;
+  PEPRefine       refine;
+  PEPRefineScheme scheme;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   ierr = PEPRegisterAll();CHKERRQ(ierr);
   ierr = PetscObjectOptionsBegin((PetscObject)pep);CHKERRQ(ierr);
-    ierr = PetscOptionsFList("-pep_type","Polynomial Eigenvalue Problem method","PEPSetType",PEPList,(char*)(((PetscObject)pep)->type_name?((PetscObject)pep)->type_name:PEPTOAR),type,256,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsFList("-pep_type","Polynomial eigensolver method","PEPSetType",PEPList,(char*)(((PetscObject)pep)->type_name?((PetscObject)pep)->type_name:PEPTOAR),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = PEPSetType(pep,type);CHKERRQ(ierr);
     } else if (!((PetscObject)pep)->type_name) {
       ierr = PEPSetType(pep,PEPTOAR);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsBoolGroupBegin("-pep_general","general polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupBegin("-pep_general","General polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetProblemType(pep,PEP_GENERAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_hermitian","hermitian polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-pep_hermitian","Hermitian polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetProblemType(pep,PEP_HERMITIAN);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroupEnd("-pep_gyroscopic","gyroscopic polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupEnd("-pep_gyroscopic","Gyroscopic polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetProblemType(pep,PEP_GYROSCOPIC);CHKERRQ(ierr); }
 
-    ierr = PetscOptionsEnum("-pep_scale","Scaling strategy","PEPSetScale",PEPScaleTypes,(PetscEnum)pep->scale,(PetscEnum*)&pep->scale,NULL);CHKERRQ(ierr);
-
+    scale = pep->scale;
+    ierr = PetscOptionsEnum("-pep_scale","Scaling strategy","PEPSetScale",PEPScaleTypes,(PetscEnum)scale,(PetscEnum*)&scale,&flg1);CHKERRQ(ierr);
     r = pep->sfactor;
-    ierr = PetscOptionsReal("-pep_scale_factor","Scale factor","PEPSetScale",pep->sfactor,&r,&flg1);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-pep_scale_factor","Scale factor","PEPSetScale",pep->sfactor,&r,&flg2);CHKERRQ(ierr);
+    if (!flg2 && r==1.0) r = PETSC_DEFAULT;
     j = pep->sits;
-    ierr = PetscOptionsInt("-pep_scale_its","Number of iterations in diagonal scaling","PEPSetScale",pep->sits,&j,&flg2);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-pep_scale_its","Number of iterations in diagonal scaling","PEPSetScale",pep->sits,&j,&flg3);CHKERRQ(ierr);
     t = pep->slambda;
-    ierr = PetscOptionsReal("-pep_scale_lambda","Estimate of eigenvalue (modulus) for diagonal scaling","PEPSetScale",pep->slambda,&t,&flg3);CHKERRQ(ierr);
-    if (flg1 || flg2 || flg3) {
-      ierr = PEPSetScale(pep,pep->scale,r,NULL,NULL,j,t);CHKERRQ(ierr);
-    }
+    ierr = PetscOptionsReal("-pep_scale_lambda","Estimate of eigenvalue (modulus) for diagonal scaling","PEPSetScale",pep->slambda,&t,&flg4);CHKERRQ(ierr);
+    if (flg1 || flg2 || flg3 || flg4) { ierr = PEPSetScale(pep,scale,r,NULL,NULL,j,t);CHKERRQ(ierr); }
 
     ierr = PetscOptionsEnum("-pep_extract","Extraction method","PEPSetExtract",PEPExtractTypes,(PetscEnum)pep->extract,(PetscEnum*)&pep->extract,NULL);CHKERRQ(ierr);
 
-    ierr = PetscOptionsEnum("-pep_refine","Iterative refinement method","PEPSetRefine",PEPRefineTypes,(PetscEnum)pep->refine,(PetscEnum*)&pep->refine,NULL);CHKERRQ(ierr);
-
+    refine = pep->refine;
+    ierr = PetscOptionsEnum("-pep_refine","Iterative refinement method","PEPSetRefine",PEPRefineTypes,(PetscEnum)refine,(PetscEnum*)&refine,&flg1);CHKERRQ(ierr);
     i = pep->npart;
-    ierr = PetscOptionsInt("-pep_refine_partitions","Number of partitions of the communicator for iterative refinement","PEPSetRefine",pep->npart,&i,&flg1);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-pep_refine_partitions","Number of partitions of the communicator for iterative refinement","PEPSetRefine",pep->npart,&i,&flg2);CHKERRQ(ierr);
     r = pep->rtol;
-    ierr = PetscOptionsReal("-pep_refine_tol","Tolerance for iterative refinement","PEPSetRefine",pep->rtol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/1000:pep->rtol,&r,&flg2);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-pep_refine_tol","Tolerance for iterative refinement","PEPSetRefine",pep->rtol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/1000:pep->rtol,&r,&flg3);CHKERRQ(ierr);
     j = pep->rits;
-    ierr = PetscOptionsInt("-pep_refine_its","Maximum number of iterations for iterative refinement","PEPSetRefine",pep->rits,&j,&flg3);CHKERRQ(ierr);
-    if (flg1 || flg2 || flg3) {
-      ierr = PEPSetRefine(pep,pep->refine,i,r,j,pep->scheme);CHKERRQ(ierr);
-    }
-
-    ierr = PetscOptionsEnum("-pep_refine_scheme","Scheme used for linear systems within iterative refinement","PEPSetRefine",PEPRefineSchemes,(PetscEnum)pep->scheme,(PetscEnum*)&pep->scheme,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-pep_refine_its","Maximum number of iterations for iterative refinement","PEPSetRefine",pep->rits,&j,&flg4);CHKERRQ(ierr);
+    scheme = pep->scheme;
+    ierr = PetscOptionsEnum("-pep_refine_scheme","Scheme used for linear systems within iterative refinement","PEPSetRefine",PEPRefineSchemes,(PetscEnum)scheme,(PetscEnum*)&scheme,&flg5);CHKERRQ(ierr);
+    if (flg1 || flg2 || flg3 || flg4 || flg5) { ierr = PEPSetRefine(pep,refine,i,r,j,scheme);CHKERRQ(ierr); }
 
     i = pep->max_it? pep->max_it: PETSC_DEFAULT;
     ierr = PetscOptionsInt("-pep_max_it","Maximum number of iterations","PEPSetTolerances",pep->max_it,&i,&flg1);CHKERRQ(ierr);
     r = pep->tol;
     ierr = PetscOptionsReal("-pep_tol","Tolerance","PEPSetTolerances",pep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:pep->tol,&r,&flg2);CHKERRQ(ierr);
-    if (flg1 || flg2) {
-      ierr = PEPSetTolerances(pep,r,i);CHKERRQ(ierr);
-    }
+    if (flg1 || flg2) { ierr = PEPSetTolerances(pep,r,i);CHKERRQ(ierr); }
 
     ierr = PetscOptionsBoolGroupBegin("-pep_conv_rel","Relative error convergence test","PEPSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetConvergenceTest(pep,PEP_CONV_REL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroupBegin("-pep_conv_norm","Convergence test relative to the matrix norms","PEPSetConvergenceTest",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-pep_conv_norm","Convergence test relative to the matrix norms","PEPSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetConvergenceTest(pep,PEP_CONV_NORM);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-pep_conv_abs","Absolute error convergence test","PEPSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetConvergenceTest(pep,PEP_CONV_ABS);CHKERRQ(ierr); }
@@ -204,18 +186,37 @@ PetscErrorCode PEPSetFromOptions(PEP pep)
     ierr = PetscOptionsInt("-pep_ncv","Number of basis vectors","PEPSetDimensions",pep->ncv,&j,&flg2);CHKERRQ(ierr);
     k = pep->mpd? pep->mpd: PETSC_DEFAULT;
     ierr = PetscOptionsInt("-pep_mpd","Maximum dimension of projected problem","PEPSetDimensions",pep->mpd,&k,&flg3);CHKERRQ(ierr);
-    if (flg1 || flg2 || flg3) {
-      ierr = PEPSetDimensions(pep,i,j,k);CHKERRQ(ierr);
-    }
+    if (flg1 || flg2 || flg3) { ierr = PEPSetDimensions(pep,i,j,k);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsEnum("-pep_basis","Polynomial basis","PEPSetBasis",PEPBasisTypes,(PetscEnum)pep->basis,(PetscEnum*)&pep->basis,NULL);CHKERRQ(ierr);
+
+    ierr = PetscOptionsBoolGroupBegin("-pep_largest_magnitude","Compute largest eigenvalues in magnitude","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_LARGEST_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_smallest_magnitude","Compute smallest eigenvalues in magnitude","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_largest_real","Compute eigenvalues with largest real parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_LARGEST_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_smallest_real","Compute eigenvalues with smallest real parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_SMALLEST_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_largest_imaginary","Compute eigenvalues with largest imaginary parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_LARGEST_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_smallest_imaginary","Compute eigenvalues with smallest imaginary parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_SMALLEST_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_target_magnitude","Compute eigenvalues closest to target","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_target_real","Compute eigenvalues with real parts closest to target","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroupEnd("-pep_target_imaginary","Compute eigenvalues with imaginary parts closest to target","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_IMAGINARY);CHKERRQ(ierr); }
 
     ierr = PetscOptionsScalar("-pep_target","Value of the target","PEPSetTarget",pep->target,&s,&flg);CHKERRQ(ierr);
     if (flg) {
-      ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
+      if (pep->which!=PEP_TARGET_REAL && pep->which!=PEP_TARGET_IMAGINARY) {
+        ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
+      }
       ierr = PEPSetTarget(pep,s);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsEnum("-pep_basis","Polynomial basis","PEPSetBasis",PEPBasisTypes,(PetscEnum)pep->basis,(PetscEnum*)&pep->basis,NULL);CHKERRQ(ierr);
-
     /* -----------------------------------------------------------------------*/
     /*
       Cancels all monitors hardwired into code before call to PEPSetFromOptions()
@@ -244,27 +245,8 @@ PetscErrorCode PEPSetFromOptions(PEP pep)
       ierr = PEPMonitorSet(pep,PEPMonitorLGAll,lg,(PetscErrorCode (*)(void**))PetscDrawLGDestroy);CHKERRQ(ierr);
       ierr = PEPSetTrackAll(pep,PETSC_TRUE);CHKERRQ(ierr);
     }
-  /* -----------------------------------------------------------------------*/
-
-    ierr = PetscOptionsBoolGroupBegin("-pep_largest_magnitude","compute largest eigenvalues in magnitude","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_LARGEST_MAGNITUDE);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_smallest_magnitude","compute smallest eigenvalues in magnitude","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_largest_real","compute largest real parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_LARGEST_REAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_smallest_real","compute smallest real parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_SMALLEST_REAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_largest_imaginary","compute largest imaginary parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_LARGEST_IMAGINARY);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_smallest_imaginary","compute smallest imaginary parts","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_SMALLEST_IMAGINARY);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_target_magnitude","compute nearest eigenvalues to target","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_MAGNITUDE);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroup("-pep_target_real","compute eigenvalues with real parts close to target","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_REAL);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroupEnd("-pep_target_imaginary","compute eigenvalues with imaginary parts close to target","PEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPSetWhichEigenpairs(pep,PEP_TARGET_IMAGINARY);CHKERRQ(ierr); }
 
+    /* -----------------------------------------------------------------------*/
     ierr = PetscOptionsName("-pep_view","Print detailed information on solver used","PEPView",NULL);CHKERRQ(ierr);
     ierr = PetscOptionsName("-pep_view_vectors","View computed eigenvectors","PEPVectorsView",NULL);CHKERRQ(ierr);
     ierr = PetscOptionsName("-pep_view_values","View computed eigenvalues","PEPValuesView",NULL);CHKERRQ(ierr);
@@ -286,15 +268,14 @@ PetscErrorCode PEPSetFromOptions(PEP pep)
   if (!pep->ds) { ierr = PEPGetDS(pep,&pep->ds);CHKERRQ(ierr); }
   ierr = DSSetFromOptions(pep->ds);CHKERRQ(ierr);
   if (!pep->st) { ierr = PEPGetST(pep,&pep->st);CHKERRQ(ierr); }
+  ierr = PEPSetDefaultST(pep);CHKERRQ(ierr);
   ierr = STSetFromOptions(pep->st);CHKERRQ(ierr);
   if (!pep->refineksp) { ierr = PEPRefineGetKSP(pep,&pep->refineksp);CHKERRQ(ierr); }
   ierr = KSPSetFromOptions(pep->refineksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetTolerances"
-/*@
+/*@C
    PEPGetTolerances - Gets the tolerance and maximum iteration count used
    by the PEP convergence tests.
 
@@ -323,8 +304,6 @@ PetscErrorCode PEPGetTolerances(PEP pep,PetscReal *tol,PetscInt *maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetTolerances"
 /*@
    PEPSetTolerances - Sets the tolerance and maximum iteration count used
    by the PEP convergence tests.
@@ -370,9 +349,7 @@ PetscErrorCode PEPSetTolerances(PEP pep,PetscReal tol,PetscInt maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetDimensions"
-/*@
+/*@C
    PEPGetDimensions - Gets the number of eigenvalues to compute
    and the dimension of the subspace.
 
@@ -403,8 +380,6 @@ PetscErrorCode PEPGetDimensions(PEP pep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetDimensions"
 /*@
    PEPSetDimensions - Sets the number of eigenvalues to compute
    and the dimension of the subspace.
@@ -464,8 +439,6 @@ PetscErrorCode PEPSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetWhichEigenpairs"
 /*@
    PEPSetWhichEigenpairs - Specifies which portion of the spectrum is
    to be sought.
@@ -545,8 +518,6 @@ PetscErrorCode PEPSetWhichEigenpairs(PEP pep,PEPWhich which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetWhichEigenpairs"
 /*@
     PEPGetWhichEigenpairs - Returns which portion of the spectrum is to be
     sought.
@@ -575,8 +546,6 @@ PetscErrorCode PEPGetWhichEigenpairs(PEP pep,PEPWhich *which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetEigenvalueComparison"
 /*@C
    PEPSetEigenvalueComparison - Specifies the eigenvalue comparison function
    when PEPSetWhichEigenpairs() is set to PEP_WHICH_USER.
@@ -618,8 +587,6 @@ PetscErrorCode PEPSetEigenvalueComparison(PEP pep,PetscErrorCode (*func)(PetscSc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetProblemType"
 /*@
    PEPSetProblemType - Specifies the type of the polynomial eigenvalue problem.
 
@@ -662,8 +629,6 @@ PetscErrorCode PEPSetProblemType(PEP pep,PEPProblemType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetProblemType"
 /*@
    PEPGetProblemType - Gets the problem type from the PEP object.
 
@@ -673,7 +638,7 @@ PetscErrorCode PEPSetProblemType(PEP pep,PEPProblemType type)
 .  pep - the polynomial eigensolver context
 
    Output Parameter:
-.  type - name of PEP problem type
+.  type - the problem type
 
    Level: intermediate
 
@@ -688,8 +653,6 @@ PetscErrorCode PEPGetProblemType(PEP pep,PEPProblemType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetBasis"
 /*@
    PEPSetBasis - Specifies the type of polynomial basis used to describe the
    polynomial eigenvalue problem.
@@ -705,7 +668,7 @@ PetscErrorCode PEPGetProblemType(PEP pep,PEPProblemType *type)
 
    Notes:
    By default, the coefficient matrices passed via PEPSetOperators() are
-   expressed in the monomial basis, i.e. 
+   expressed in the monomial basis, i.e.
    P(lambda) = A_0 + lambda*A_1 + lambda^2*A_2 + ... + lambda^d*A_d.
    Other polynomial bases may have better numerical behaviour, but the user
    must then pass the coefficient matrices accordingly.
@@ -723,8 +686,6 @@ PetscErrorCode PEPSetBasis(PEP pep,PEPBasis basis)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetBasis"
 /*@
    PEPGetBasis - Gets the type of polynomial basis from the PEP object.
 
@@ -749,8 +710,6 @@ PetscErrorCode PEPGetBasis(PEP pep,PEPBasis *basis)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetTrackAll"
 /*@
    PEPSetTrackAll - Specifies if the solver must compute the residual of all
    approximate eigenpairs or not.
@@ -782,8 +741,6 @@ PetscErrorCode PEPSetTrackAll(PEP pep,PetscBool trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetTrackAll"
 /*@
    PEPGetTrackAll - Returns the flag indicating whether all residual norms must
    be computed or not.
@@ -809,8 +766,6 @@ PetscErrorCode PEPGetTrackAll(PEP pep,PetscBool *trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetConvergenceTestFunction"
 /*@C
    PEPSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
@@ -850,18 +805,19 @@ PetscErrorCode PEPSetConvergenceTestFunction(PEP pep,PetscErrorCode (*func)(PEP,
   if (pep->convergeddestroy) {
     ierr = (*pep->convergeddestroy)(pep->convergedctx);CHKERRQ(ierr);
   }
-  pep->converged        = func;
+  pep->convergeduser    = func;
   pep->convergeddestroy = destroy;
   pep->convergedctx     = ctx;
   if (func == PEPConvergedRelative) pep->conv = PEP_CONV_REL;
   else if (func == PEPConvergedNorm) pep->conv = PEP_CONV_NORM;
   else if (func == PEPConvergedAbsolute) pep->conv = PEP_CONV_ABS;
-  else pep->conv = PEP_CONV_USER;
+  else {
+    pep->conv      = PEP_CONV_USER;
+    pep->converged = pep->convergeduser;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetConvergenceTest"
 /*@
    PEPSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
@@ -895,10 +851,13 @@ PetscErrorCode PEPSetConvergenceTest(PEP pep,PEPConv conv)
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidLogicalCollectiveEnum(pep,conv,2);
   switch (conv) {
-    case PEP_CONV_ABS:    pep->converged = PEPConvergedAbsolute; break;
-    case PEP_CONV_REL:    pep->converged = PEPConvergedRelative; break;
-    case PEP_CONV_NORM:   pep->converged = PEPConvergedNorm; break;
-    case PEP_CONV_USER: break;
+    case PEP_CONV_ABS:  pep->converged = PEPConvergedAbsolute; break;
+    case PEP_CONV_REL:  pep->converged = PEPConvergedRelative; break;
+    case PEP_CONV_NORM: pep->converged = PEPConvergedNorm; break;
+    case PEP_CONV_USER:
+      if (!pep->convergeduser) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ORDER,"Must call PEPSetConvergenceTestFunction() first");
+      pep->converged = pep->convergeduser;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'conv' value");
   }
@@ -906,8 +865,6 @@ PetscErrorCode PEPSetConvergenceTest(PEP pep,PEPConv conv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetConvergenceTest"
 /*@
    PEPGetConvergenceTest - Gets the method used to compute the error estimate
    used in the convergence test.
@@ -933,8 +890,6 @@ PetscErrorCode PEPGetConvergenceTest(PEP pep,PEPConv *conv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetStoppingTestFunction"
 /*@C
    PEPSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the eigensolver.
@@ -977,16 +932,17 @@ PetscErrorCode PEPSetStoppingTestFunction(PEP pep,PetscErrorCode (*func)(PEP,Pet
   if (pep->stoppingdestroy) {
     ierr = (*pep->stoppingdestroy)(pep->stoppingctx);CHKERRQ(ierr);
   }
-  pep->stopping        = func;
+  pep->stoppinguser    = func;
   pep->stoppingdestroy = destroy;
   pep->stoppingctx     = ctx;
   if (func == PEPStoppingBasic) pep->stop = PEP_STOP_BASIC;
-  else pep->stop = PEP_STOP_USER;
+  else {
+    pep->stop     = PEP_STOP_USER;
+    pep->stopping = pep->stoppinguser;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetStoppingTest"
 /*@
    PEPSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the eigensolver.
@@ -1017,7 +973,10 @@ PetscErrorCode PEPSetStoppingTest(PEP pep,PEPStop stop)
   PetscValidLogicalCollectiveEnum(pep,stop,2);
   switch (stop) {
     case PEP_STOP_BASIC: pep->stopping = PEPStoppingBasic; break;
-    case PEP_STOP_USER:  break;
+    case PEP_STOP_USER:
+      if (!pep->stoppinguser) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ORDER,"Must call PEPSetStoppingTestFunction() first");
+      pep->stopping = pep->stoppinguser;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'stop' value");
   }
@@ -1025,8 +984,6 @@ PetscErrorCode PEPSetStoppingTest(PEP pep,PEPStop stop)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetStoppingTest"
 /*@
    PEPGetStoppingTest - Gets the method used to decide the termination of the outer
    loop of the eigensolver.
@@ -1052,9 +1009,7 @@ PetscErrorCode PEPGetStoppingTest(PEP pep,PEPStop *stop)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetScale"
-/*@
+/*@C
    PEPSetScale - Specifies the scaling strategy to be used.
 
    Logically Collective on PEP
@@ -1143,9 +1098,7 @@ PetscErrorCode PEPSetScale(PEP pep,PEPScale scale,PetscReal alpha,Vec Dl,Vec Dr,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetScale"
-/*@
+/*@C
    PEPGetScale - Gets the scaling strategy used by the PEP object, and the
    associated parameters.
 
@@ -1185,8 +1138,6 @@ PetscErrorCode PEPGetScale(PEP pep,PEPScale *scale,PetscReal *alpha,Vec *Dl,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetExtract"
 /*@
    PEPSetExtract - Specifies the extraction strategy to be used.
 
@@ -1212,8 +1163,6 @@ PetscErrorCode PEPSetExtract(PEP pep,PEPExtract extract)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetExtract"
 /*@
    PEPGetExtract - Gets the extraction strategy used by the PEP object.
 
@@ -1237,8 +1186,6 @@ PetscErrorCode PEPGetExtract(PEP pep,PEPExtract *extract)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetRefine"
 /*@
    PEPSetRefine - Specifies the refinement type (and options) to be used
    after the solve.
@@ -1280,7 +1227,7 @@ PetscErrorCode PEPGetExtract(PEP pep,PEPExtract *extract)
    refinement iterations (just one by default).
 
    The scheme argument is used to change the way in which linear systems are
-   solved. Possible choices are: explicit, mixed block elimination (MBE), 
+   solved. Possible choices are: explicit, mixed block elimination (MBE),
    and Schur complement.
 
    Level: intermediate
@@ -1313,7 +1260,7 @@ PetscErrorCode PEPSetRefine(PEP pep,PEPRefine refine,PetscInt npart,PetscReal to
       pep->npart = npart;
     }
     if (tol == PETSC_DEFAULT || tol == PETSC_DECIDE) {
-      pep->rtol = PetscMax(pep->tol/1000,PETSC_MACHINE_EPSILON);
+      pep->rtol = PETSC_DEFAULT;
     } else {
       if (tol<=0.0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
       pep->rtol = tol;
@@ -1330,9 +1277,7 @@ PetscErrorCode PEPSetRefine(PEP pep,PEPRefine refine,PetscInt npart,PetscReal to
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetRefine"
-/*@
+/*@C
    PEPGetRefine - Gets the refinement strategy used by the PEP object, and the
    associated parameters.
 
@@ -1367,8 +1312,6 @@ PetscErrorCode PEPGetRefine(PEP pep,PEPRefine *refine,PetscInt *npart,PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetOptionsPrefix"
 /*@C
    PEPSetOptionsPrefix - Sets the prefix used for searching for all
    PEP options in the database.
@@ -1413,8 +1356,6 @@ PetscErrorCode PEPSetOptionsPrefix(PEP pep,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPAppendOptionsPrefix"
 /*@C
    PEPAppendOptionsPrefix - Appends to the prefix used for searching for all
    PEP options in the database.
@@ -1436,31 +1377,21 @@ PetscErrorCode PEPSetOptionsPrefix(PEP pep,const char *prefix)
 PetscErrorCode PEPAppendOptionsPrefix(PEP pep,const char *prefix)
 {
   PetscErrorCode ierr;
-  PetscBool      flg;
-  EPS            eps;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   if (!pep->st) { ierr = PEPGetST(pep,&pep->st);CHKERRQ(ierr); }
   ierr = STAppendOptionsPrefix(pep->st,prefix);CHKERRQ(ierr);
   if (!pep->V) { ierr = PEPGetBV(pep,&pep->V);CHKERRQ(ierr); }
-  ierr = BVSetOptionsPrefix(pep->V,prefix);CHKERRQ(ierr);
+  ierr = BVAppendOptionsPrefix(pep->V,prefix);CHKERRQ(ierr);
   if (!pep->ds) { ierr = PEPGetDS(pep,&pep->ds);CHKERRQ(ierr); }
-  ierr = DSSetOptionsPrefix(pep->ds,prefix);CHKERRQ(ierr);
+  ierr = DSAppendOptionsPrefix(pep->ds,prefix);CHKERRQ(ierr);
   if (!pep->rg) { ierr = PEPGetRG(pep,&pep->rg);CHKERRQ(ierr); }
-  ierr = RGSetOptionsPrefix(pep->rg,prefix);CHKERRQ(ierr);
+  ierr = RGAppendOptionsPrefix(pep->rg,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)pep,prefix);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)pep,PEPLINEAR,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PEPLinearGetEPS(pep,&eps);CHKERRQ(ierr);
-    ierr = EPSSetOptionsPrefix(eps,((PetscObject)pep)->prefix);CHKERRQ(ierr);
-    ierr = EPSAppendOptionsPrefix(eps,"pep_");CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetOptionsPrefix"
 /*@C
    PEPGetOptionsPrefix - Gets the prefix used for searching for all
    PEP options in the database.
