@@ -1,25 +1,15 @@
 /*
-   Newton refinement for PEP, simple version.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Newton refinement for PEP, simple version
+*/
 
 #include <slepc/private/pepimpl.h>
 #include <slepcblaslapack.h>
@@ -38,14 +28,12 @@ typedef struct {
   PetscScalar  M4,m3;
 } FSubctx;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFSMult"
 static PetscErrorCode MatFSMult(Mat M ,Vec x,Vec y)
 {
   PetscErrorCode ierr;
   FSubctx        *ctx;
   PetscScalar    t;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(M,&ctx);CHKERRQ(ierr);
   ierr = VecDot(x,ctx->M3,&t);CHKERRQ(ierr);
@@ -55,8 +43,6 @@ static PetscErrorCode MatFSMult(Mat M ,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSimpleNRefSetUp"
 static PetscErrorCode PEPSimpleNRefSetUp(PEP pep,PEPSimpNRefctx **ctx_)
 {
   PetscErrorCode ierr;
@@ -117,7 +103,7 @@ static PetscErrorCode PEPSimpleNRefSetUp(PEP pep,PEPSimpNRefctx **ctx_)
       ierr = VecCreateMPI(PetscObjectComm((PetscObject)ctx->A[0]),ne,PETSC_DECIDE,&ctx->nv);CHKERRQ(ierr);
       ierr = PetscMalloc1(m0-n0,&idx1);CHKERRQ(ierr);
       for (i=n0;i<m0;i++) idx1[i-n0] = i;
-      ierr = ISCreateGeneral(PetscObjectComm((PetscObject)pep),(m0-n0),idx1,PETSC_COPY_VALUES,&is1);CHKERRQ(ierr);
+      ierr = ISCreateGeneral(PetscObjectComm((PetscObject)ctx->A[0]),(m0-n0),idx1,PETSC_COPY_VALUES,&is1);CHKERRQ(ierr);
       ierr = VecScatterCreate(v,is1,ctx->nv,is1,&ctx->nst);CHKERRQ(ierr);
       if (pep->npart==1) {
         ierr = BVRestoreColumn(pep->V,0,&v);CHKERRQ(ierr);
@@ -125,15 +111,13 @@ static PetscErrorCode PEPSimpleNRefSetUp(PEP pep,PEPSimpNRefctx **ctx_)
       ierr = PetscFree(idx1);CHKERRQ(ierr);
       ierr = ISDestroy(&is1);CHKERRQ(ierr);
     }
-  } 
-  PetscFunctionReturn(0);  
+  }
+  PetscFunctionReturn(0);
 }
 
 /*
   Gather Eigenpair idx from subcommunicator with color sc
 */
-#undef __FUNCT__
-#define __FUNCT__ "PEPSimpleNRefGatherEigenpair"
 static PetscErrorCode PEPSimpleNRefGatherEigenpair(PEP pep,PEPSimpNRefctx *ctx,PetscInt sc,PetscInt idx,PetscInt *fail)
 {
   PetscErrorCode    ierr;
@@ -143,13 +127,13 @@ static PetscErrorCode PEPSimpleNRefGatherEigenpair(PEP pep,PEPSimpNRefctx *ctx,P
   const PetscScalar *array;
 
   PetscFunctionBegin;
+  ierr = MPI_Comm_size(comm,&nproc);CHKERRQ(ierr);
+  p = (nproc/pep->npart)*(sc+1)+PetscMin(nproc%pep->npart,sc+1)-1;
   if (pep->npart>1) {
-    ierr = MPI_Comm_size(comm,&nproc);CHKERRQ(ierr);
-    p = (nproc/pep->npart)*sc+PetscMin(sc,nproc%pep->npart);
     /* Communicate convergence successful */
     ierr = MPI_Bcast(fail,1,MPIU_INT,p,comm);CHKERRQ(ierr);
     if (!(*fail)) {
-      /* Process 0 of subcommunicator sc broadcasts the eigenvalue */ 
+      /* Process 0 of subcommunicator sc broadcasts the eigenvalue */
       ierr = MPI_Bcast(&pep->eigr[idx],1,MPIU_SCALAR,p,comm);CHKERRQ(ierr);
       /* Gather pep->V[idx] from the subcommuniator sc */
       ierr = BVGetColumn(pep->V,idx,&v);CHKERRQ(ierr);
@@ -167,22 +151,18 @@ static PetscErrorCode PEPSimpleNRefGatherEigenpair(PEP pep,PEPSimpNRefctx *ctx,P
     }
   } else {
     if (pep->scheme==PEP_REFINE_SCHEME_EXPLICIT && !(*fail)) {
-      ierr = MPI_Comm_size(comm,&nproc);CHKERRQ(ierr);
-      p = (nproc/pep->npart)*sc+PetscMin(sc,nproc%pep->npart);
       ierr = MPI_Bcast(&pep->eigr[idx],1,MPIU_SCALAR,p,comm);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSimpleNRefScatterEigenvector"
 static PetscErrorCode PEPSimpleNRefScatterEigenvector(PEP pep,PEPSimpNRefctx *ctx,PetscInt sc,PetscInt idx)
 {
   PetscErrorCode    ierr;
   Vec               v;
   const PetscScalar *array;
-  
+
   PetscFunctionBegin;
   if (pep->npart>1) {
     ierr = BVGetColumn(pep->V,idx,&v);CHKERRQ(ierr);
@@ -201,8 +181,6 @@ static PetscErrorCode PEPSimpleNRefScatterEigenvector(PEP pep,PEPSimpNRefctx *ct
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPEvaluateFunctionDerivatives"
 static PetscErrorCode PEPEvaluateFunctionDerivatives(PEP pep,PetscScalar alpha,PetscScalar *vals)
 {
   PetscInt    i,nmat=pep->nmat;
@@ -222,8 +200,6 @@ static PetscErrorCode PEPEvaluateFunctionDerivatives(PEP pep,PetscScalar alpha,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSimpleNRefSetUpSystem"
 static PetscErrorCode PEPSimpleNRefSetUpSystem(PEP pep,Mat *A,PEPSimpNRefctx *ctx,PetscInt idx,Mat *Mt,Mat *T,Mat *P,PetscBool ini,Vec t,Vec v)
 {
   PetscErrorCode    ierr;
@@ -251,7 +227,7 @@ static PetscErrorCode PEPSimpleNRefSetUpSystem(PEP pep,Mat *A,PEPSimpNRefctx *ct
       ierr = MatShellSetOperation(*T,MATOP_MULT,(void(*)(void))MatFSMult);CHKERRQ(ierr);
     } else {
       ierr = MatShellGetContext(*T,&fctx);CHKERRQ(ierr);
-    }    
+    }
     M=fctx->M1;
     break;
   case PEP_REFINE_SCHEME_MBE:
@@ -326,7 +302,7 @@ static PetscErrorCode PEPSimpleNRefSetUpSystem(PEP pep,Mat *A,PEPSimpNRefctx *ct
       *Mt = M;
       *P  = *T;
     }
-  
+
     /* Set values */
     ierr = VecGetArrayRead(w,&array);CHKERRQ(ierr);
     for (i=m0;i<m1;i++) {
@@ -362,9 +338,9 @@ static PetscErrorCode PEPSimpleNRefSetUpSystem(PEP pep,Mat *A,PEPSimpNRefctx *ct
     }
     ierr = VecConjugate(v);CHKERRQ(ierr);
     ierr = MatAssemblyBegin(*T,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = MatAssemblyEnd(*T,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr); 
+    ierr = MatAssemblyEnd(*T,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
     ierr = PetscFree(cols2);CHKERRQ(ierr);
-    break; 
+    break;
   case PEP_REFINE_SCHEME_SCHUR:
     fctx->M2 = ctx->w;
     fctx->M3 = v;
@@ -378,11 +354,13 @@ static PetscErrorCode PEPSimpleNRefSetUpSystem(PEP pep,Mat *A,PEPSimpNRefctx *ct
     } else {
       ierr = MatCopy(M,*P,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
     }
-    ierr = VecConjugate(v);CHKERRQ(ierr);
-    ierr = VecPointwiseMult(t,v,w);CHKERRQ(ierr);
-    ierr = VecConjugate(v);CHKERRQ(ierr);
-    ierr = VecScale(t,-fctx->m3/fctx->M4);CHKERRQ(ierr);
-    ierr = MatDiagonalSet(*P,t,ADD_VALUES);CHKERRQ(ierr);
+    if (fctx->M4!=0.0) {
+      ierr = VecConjugate(v);CHKERRQ(ierr);
+      ierr = VecPointwiseMult(t,v,w);CHKERRQ(ierr);
+      ierr = VecConjugate(v);CHKERRQ(ierr);
+      ierr = VecScale(t,-fctx->m3/fctx->M4);CHKERRQ(ierr);
+      ierr = MatDiagonalSet(*P,t,ADD_VALUES);CHKERRQ(ierr);
+    }
     break;
   case PEP_REFINE_SCHEME_MBE:
     *T = M;
@@ -393,8 +371,6 @@ static PetscErrorCode PEPSimpleNRefSetUpSystem(PEP pep,Mat *A,PEPSimpNRefctx *ct
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPNewtonRefinementSimple"
 PetscErrorCode PEPNewtonRefinementSimple(PEP pep,PetscInt *maxits,PetscReal tol,PetscInt k)
 {
   PetscErrorCode     ierr;
@@ -433,7 +409,7 @@ PetscErrorCode PEPNewtonRefinementSimple(PEP pep,PetscInt *maxits,PetscReal tol,
   for (i=0;i<pep->npart;i++) fail_sc[i] = 0;
   for (i=0;i<pep->npart;i++) its_sc[i] = 0;
   color = (pep->npart==1)?0:pep->refinesubc->color;
-   
+
   /* Loop performing iterative refinements */
   while (!solved) {
     for (i=0;i<pep->npart;i++) {
@@ -455,6 +431,7 @@ PetscErrorCode PEPNewtonRefinementSimple(PEP pep,PetscInt *maxits,PetscReal tol,
         if (error<=tol || its_sc[i]>=its || fail_sc[i]) {
           idx_sc[i] = idx++;
           its_sc[i] = 0;
+          fail_sc[i] = 0;
           if (idx_sc[i]<k) { ierr = PEPSimpleNRefScatterEigenvector(pep,ctx,i,idx_sc[i]);CHKERRQ(ierr); }
         } else {
           sc_pend = PETSC_FALSE;
@@ -471,7 +448,7 @@ PetscErrorCode PEPNewtonRefinementSimple(PEP pep,PetscInt *maxits,PetscReal tol,
 #endif
       if (pep->npart==1) {
         ierr = BVGetColumn(pep->V,idx_sc[color],&v);CHKERRQ(ierr);
-      } else v = ctx->v; 
+      } else v = ctx->v;
       ierr = PEPSimpleNRefSetUpSystem(pep,ctx->A,ctx,idx_sc[color],&Mt,&T,&P,ini,t[0],v);CHKERRQ(ierr);
       ierr = KSPSetOperators(pep->refineksp,T,P);CHKERRQ(ierr);
       if (ini) {
@@ -484,7 +461,7 @@ PetscErrorCode PEPNewtonRefinementSimple(PEP pep,PetscInt *maxits,PetscReal tol,
       }
 
       switch (pep->scheme) {
-      case PEP_REFINE_SCHEME_EXPLICIT: 
+      case PEP_REFINE_SCHEME_EXPLICIT:
         ierr = MatMult(Mt,v,r);CHKERRQ(ierr);
         ierr = VecGetArrayRead(r,&array);CHKERRQ(ierr);
         if (rank==size-1) {
@@ -549,22 +526,25 @@ PetscErrorCode PEPNewtonRefinementSimple(PEP pep,PetscInt *maxits,PetscReal tol,
         }
         break;
       case PEP_REFINE_SCHEME_SCHUR:
+        fail_sc[color] = 1;
         ierr = MatShellGetContext(T,&fctx);CHKERRQ(ierr);
-        ierr = MatMult(fctx->M1,v,r);CHKERRQ(ierr);
-        ierr = KSPSolve(pep->refineksp,r,dv);CHKERRQ(ierr);
-        ierr = KSPGetConvergedReason(pep->refineksp,&reason);CHKERRQ(ierr);
-        if (reason>0) {
-          ierr = VecDot(dv,v,&deig);CHKERRQ(ierr);
-          deig *= -fctx->m3/fctx->M4;
-          ierr = VecAXPY(v,-1.0,dv);CHKERRQ(ierr);
-          ierr = VecNorm(v,NORM_2,&norm);CHKERRQ(ierr);
-          ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
-          pep->eigr[idx_sc[color]] -= deig;
-          fail_sc[color] = 0;
-        } else fail_sc[color] = 1;
+        if (fctx->M4!=0.0) {
+          ierr = MatMult(fctx->M1,v,r);CHKERRQ(ierr);
+          ierr = KSPSolve(pep->refineksp,r,dv);CHKERRQ(ierr);
+          ierr = KSPGetConvergedReason(pep->refineksp,&reason);CHKERRQ(ierr);
+          if (reason>0) {
+            ierr = VecDot(dv,v,&deig);CHKERRQ(ierr);
+            deig *= -fctx->m3/fctx->M4;
+            ierr = VecAXPY(v,-1.0,dv);CHKERRQ(ierr);
+            ierr = VecNorm(v,NORM_2,&norm);CHKERRQ(ierr);
+            ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
+            pep->eigr[idx_sc[color]] -= deig;
+            fail_sc[color] = 0;
+          }
+        }
         break;
       }
-      if (pep->npart==1) { ierr = BVRestoreColumn(pep->V,idx_sc[color],&v);CHKERRQ(ierr); } 
+      if (pep->npart==1) { ierr = BVRestoreColumn(pep->V,idx_sc[color],&v);CHKERRQ(ierr); }
     }
   }
   ierr = VecDestroy(&t[0]);CHKERRQ(ierr);
