@@ -1,25 +1,15 @@
 /*
-      PEP routines related to the solution process.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   PEP routines related to the solution process
+*/
 
 #include <slepc/private/pepimpl.h>       /*I "slepcpep.h" I*/
 #include <petscdraw.h>
@@ -30,57 +20,39 @@ static const char citation[] =
   "   author = \"C. Campos and J. E. Roman\",\n"
   "   title = \"Parallel iterative refinement in polynomial eigenvalue problems\",\n"
   "   journal = \"Numer. Linear Algebra Appl.\",\n"
-  "   volume = \"to appear\",\n"
-  "   number = \"\",\n"
-  "   pages = \"\",\n"
+  "   volume = \"23\",\n"
+  "   number = \"4\",\n"
+  "   pages = \"730--745\",\n"
   "   year = \"2016,\"\n"
-  "   doi = \"http://dx.doi.org/10.1002/nla.2052\"\n"
+  "   doi = \"https://doi.org/10.1002/nla.2052\"\n"
   "}\n";
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPComputeVectors"
 PetscErrorCode PEPComputeVectors(PEP pep)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PEPCheckSolved(pep,1);
-  switch (pep->state) {
-  case PEP_STATE_SOLVED:
-    if (pep->ops->computevectors) {
-      ierr = (*pep->ops->computevectors)(pep);CHKERRQ(ierr);
-    }
-    break;
-  default:
-    break;
+  if (pep->state==PEP_STATE_SOLVED && pep->ops->computevectors) {
+    ierr = (*pep->ops->computevectors)(pep);CHKERRQ(ierr);
   }
   pep->state = PEP_STATE_EIGENVECTORS;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPExtractVectors"
 PetscErrorCode PEPExtractVectors(PEP pep)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PEPCheckSolved(pep,1);
-  switch (pep->state) {
-  case PEP_STATE_SOLVED:
+  if (pep->state==PEP_STATE_SOLVED && pep->ops->extractvectors) {
     ierr = BVSetActiveColumns(pep->V,0,pep->nconv);CHKERRQ(ierr);
-    if (pep->ops->extractvectors) {
-      ierr = (*pep->ops->extractvectors)(pep);CHKERRQ(ierr);
-    }
-    break;
-  default:
-    break;
+    ierr = (*pep->ops->extractvectors)(pep);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSolve"
 /*@
    PEPSolve - Solves the polynomial eigensystem.
 
@@ -131,7 +103,7 @@ PetscErrorCode PEPSolve(PEP pep)
   ierr = PEPViewFromOptions(pep,NULL,"-pep_view_pre");CHKERRQ(ierr);
 
   ierr = (*pep->ops->solve)(pep);CHKERRQ(ierr);
-  
+
   if (!pep->reason) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
 
   ierr = PetscObjectTypeCompare((PetscObject)pep,PEPLINEAR,&islinear);CHKERRQ(ierr);
@@ -191,8 +163,6 @@ PetscErrorCode PEPSolve(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetIterationNumber"
 /*@
    PEPGetIterationNumber - Gets the current iteration number. If the
    call to PEPSolve() is complete, then it returns the number of iterations
@@ -206,8 +176,6 @@ PetscErrorCode PEPSolve(PEP pep)
    Output Parameter:
 .  its - number of iterations
 
-   Level: intermediate
-
    Note:
    During the i-th iteration this call returns i-1. If PEPSolve() is
    complete, then parameter "its" contains either the iteration number at
@@ -215,6 +183,8 @@ PetscErrorCode PEPSolve(PEP pep)
    Call PEPGetConvergedReason() to determine if the solver converged or
    failed and why.
 
+   Level: intermediate
+
 .seealso: PEPGetConvergedReason(), PEPSetTolerances()
 @*/
 PetscErrorCode PEPGetIterationNumber(PEP pep,PetscInt *its)
@@ -226,8 +196,6 @@ PetscErrorCode PEPGetIterationNumber(PEP pep,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetConverged"
 /*@
    PEPGetConverged - Gets the number of converged eigenpairs.
 
@@ -256,8 +224,6 @@ PetscErrorCode PEPGetConverged(PEP pep,PetscInt *nconv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetConvergedReason"
 /*@
    PEPGetConvergedReason - Gets the reason why the PEPSolve() iteration was
    stopped.
@@ -295,9 +261,7 @@ PetscErrorCode PEPGetConvergedReason(PEP pep,PEPConvergedReason *reason)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetEigenpair"
-/*@
+/*@C
    PEPGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    PEPSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
@@ -378,8 +342,6 @@ PetscErrorCode PEPGetEigenpair(PEP pep,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPGetErrorEstimate"
 /*@
    PEPGetErrorEstimate - Returns the error estimate associated to the i-th
    computed eigenpair.
@@ -408,13 +370,11 @@ PetscErrorCode PEPGetErrorEstimate(PEP pep,PetscInt i,PetscReal *errest)
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidPointer(errest,3);
   PEPCheckSolved(pep,1);
-  if (i<0 || i>=pep->nconv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
-  if (errest) *errest = pep->errest[pep->perm[i]];
+  if (i<0 || i>=pep->nconv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  *errest = pep->errest[pep->perm[i]];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPComputeResidualNorm_Private"
 /*
    PEPComputeResidualNorm_Private - Computes the norm of the residual vector
    associated with an eigenpair.
@@ -443,7 +403,7 @@ PetscErrorCode PEPComputeResidualNorm_Private(PEP pep,PetscScalar kr,PetscScalar
   ierr = VecSet(u,0.0);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
   ui = z[2]; wi = z[3];
-  ivals = it; 
+  ivals = it;
 #endif
   if (nmat>20) {
     ierr = PetscMalloc1(nmat,&vals);CHKERRQ(ierr);
@@ -499,8 +459,6 @@ PetscErrorCode PEPComputeResidualNorm_Private(PEP pep,PetscScalar kr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPComputeError"
 /*@
    PEPComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
