@@ -1,31 +1,19 @@
 /*
-   The PEP routines related to various viewers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   The PEP routines related to various viewers
+*/
 
 #include <slepc/private/pepimpl.h>      /*I "slepcpep.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPView"
 /*@C
    PEPView - Prints the PEP data structure.
 
@@ -56,7 +44,7 @@
 PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  const char     *type;
+  const char     *type=NULL;
   char           str[50];
   PetscBool      isascii,islinear,istrivial;
   PetscInt       i;
@@ -86,7 +74,6 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
         case PEP_GENERAL:    type = "general polynomial eigenvalue problem"; break;
         case PEP_HERMITIAN:  type = HERM " polynomial eigenvalue problem"; break;
         case PEP_GYROSCOPIC: type = "gyroscopic polynomial eigenvalue problem"; break;
-        default: SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of pep->problem_type");
       }
     } else type = "not yet set";
     ierr = PetscViewerASCIIPrintf(viewer,"  problem type: %s\n",type);CHKERRQ(ierr);
@@ -140,7 +127,6 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
       case PEP_SMALLEST_IMAGINARY:
         ierr = PetscViewerASCIIPrintf(viewer,"smallest imaginary parts\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of pep->which");
     }
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",pep->nev);CHKERRQ(ierr);
@@ -208,12 +194,10 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
     } else {
       ierr = KSPView(pep->refineksp,viewer);CHKERRQ(ierr);
     }
-  } 
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPReasonView"
 /*@C
    PEPReasonView - Displays the reason a PEP solve converged or diverged.
 
@@ -249,11 +233,9 @@ PetscErrorCode PEPReasonView(PEP pep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPReasonViewFromOptions"
 /*@
    PEPReasonViewFromOptions - Processes command line options to determine if/how
-   the PEP converged reason is to be viewed. 
+   the PEP converged reason is to be viewed.
 
    Collective on PEP
 
@@ -284,14 +266,10 @@ PetscErrorCode PEPReasonViewFromOptions(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPErrorView_ASCII"
 static PetscErrorCode PEPErrorView_ASCII(PEP pep,PEPErrorType etype,PetscViewer viewer)
 {
-  PetscBool      errok;
-  PetscReal      error,re,im;
-  PetscScalar    kr,ki;
-  PetscInt       i,j;
+  PetscReal      error;
+  PetscInt       i,j,k;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -299,34 +277,19 @@ static PetscErrorCode PEPErrorView_ASCII(PEP pep,PEPErrorType etype,PetscViewer
     ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D eigenvalues converged\n\n",pep->nev);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-  errok = PETSC_TRUE;
   for (i=0;i<pep->nev;i++) {
     ierr = PEPComputeError(pep,i,etype,&error);CHKERRQ(ierr);
-    errok = (errok && error<5.0*pep->tol)? PETSC_TRUE: PETSC_FALSE;
-  }
-  if (!errok) {
-    ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",pep->nev);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
+    if (error>=5.0*pep->tol) {
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",pep->nev);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
   }
   ierr = PetscViewerASCIIPrintf(viewer," All requested eigenvalues computed up to the required tolerance:");CHKERRQ(ierr);
   for (i=0;i<=(pep->nev-1)/8;i++) {
     ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
     for (j=0;j<PetscMin(8,pep->nev-8*i);j++) {
-      ierr = PEPGetEigenpair(pep,8*i+j,&kr,&ki,NULL,NULL);CHKERRQ(ierr);
-#if defined(PETSC_USE_COMPLEX)
-      re = PetscRealPart(kr);
-      im = PetscImaginaryPart(kr);
-#else
-      re = kr;
-      im = ki;
-#endif
-      if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
-      if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
-      if (im!=0.0) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%.5f%+.5fi",(double)re,(double)im);CHKERRQ(ierr);
-      } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"%.5f",(double)re);CHKERRQ(ierr);
-      }
+      k = pep->perm[8*i+j];
+      ierr = SlepcPrintEigenvalueASCII(pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
       if (8*i+j+1<pep->nev) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
     }
   }
@@ -334,8 +297,6 @@ static PetscErrorCode PEPErrorView_ASCII(PEP pep,PEPErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPErrorView_DETAIL"
 static PetscErrorCode PEPErrorView_DETAIL(PEP pep,PEPErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -379,8 +340,6 @@ static PetscErrorCode PEPErrorView_DETAIL(PEP pep,PEPErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPErrorView_MATLAB"
 static PetscErrorCode PEPErrorView_MATLAB(PEP pep,PEPErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -393,14 +352,12 @@ static PetscErrorCode PEPErrorView_MATLAB(PEP pep,PEPErrorType etype,PetscViewer
   ierr = PetscViewerASCIIPrintf(viewer,"Error_%s = [\n",name);CHKERRQ(ierr);
   for (i=0;i<pep->nconv;i++) {
     ierr = PEPComputeError(pep,i,etype,&error);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",error);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",(double)error);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"];\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPErrorView"
 /*@C
    PEPErrorView - Displays the errors associated with the computed solution
    (as well as the eigenvalues).
@@ -460,11 +417,9 @@ PetscErrorCode PEPErrorView(PEP pep,PEPErrorType etype,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPErrorViewFromOptions"
 /*@
    PEPErrorViewFromOptions - Processes command line options to determine if/how
-   the errors of the computed solution are to be viewed. 
+   the errors of the computed solution are to be viewed.
 
    Collective on PEP
 
@@ -509,8 +464,6 @@ PetscErrorCode PEPErrorViewFromOptions(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPValuesView_DRAW"
 static PetscErrorCode PEPValuesView_DRAW(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -521,8 +474,8 @@ static PetscErrorCode PEPValuesView_DRAW(PEP pep,PetscViewer viewer)
 
   PetscFunctionBegin;
   if (!pep->nconv) PetscFunctionReturn(0);
-  ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+  ierr = PetscDrawSetTitle(draw,"Computed Eigenvalues");CHKERRQ(ierr);
   ierr = PetscDrawSPCreate(draw,1,&drawsp);CHKERRQ(ierr);
   for (i=0;i<pep->nconv;i++) {
     k = pep->perm[i];
@@ -538,15 +491,11 @@ static PetscErrorCode PEPValuesView_DRAW(PEP pep,PetscViewer viewer)
   ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscDrawSPSave(drawsp);CHKERRQ(ierr);
   ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPValuesView_ASCII"
 static PetscErrorCode PEPValuesView_ASCII(PEP pep,PetscViewer viewer)
 {
-  PetscReal      re,im;
   PetscInt       i,k;
   PetscErrorCode ierr;
 
@@ -554,27 +503,14 @@ static PetscErrorCode PEPValuesView_ASCII(PEP pep,PetscViewer viewer)
   ierr = PetscViewerASCIIPrintf(viewer,"Eigenvalues = \n");CHKERRQ(ierr);
   for (i=0;i<pep->nconv;i++) {
     k = pep->perm[i];
-#if defined(PETSC_USE_COMPLEX)
-    re = PetscRealPart(pep->eigr[k]);
-    im = PetscImaginaryPart(pep->eigr[k]);
-#else
-    re = pep->eigr[k];
-    im = pep->eigi[k];
-#endif
-    if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
-    if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
-    if (im!=0.0) {
-      ierr = PetscViewerASCIIPrintf(viewer,"   %.5f%+.5fi\n",(double)re,(double)im);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"   %.5f\n",(double)re);CHKERRQ(ierr);
-    }
+    ierr = PetscViewerASCIIPrintf(viewer,"   ");CHKERRQ(ierr);
+    ierr = SlepcPrintEigenvalueASCII(pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPValuesView_MATLAB"
 static PetscErrorCode PEPValuesView_MATLAB(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -604,8 +540,6 @@ static PetscErrorCode PEPValuesView_MATLAB(PEP pep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPValuesView"
 /*@C
    PEPValuesView - Displays the computed eigenvalues in a viewer.
 
@@ -656,11 +590,9 @@ PetscErrorCode PEPValuesView(PEP pep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPValuesViewFromOptions"
 /*@
    PEPValuesViewFromOptions - Processes command line options to determine if/how
-   the computed eigenvalues are to be viewed. 
+   the computed eigenvalues are to be viewed.
 
    Collective on PEP
 
@@ -691,8 +623,6 @@ PetscErrorCode PEPValuesViewFromOptions(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPVectorsView"
 /*@C
    PEPVectorsView - Outputs computed eigenvectors to a viewer.
 
@@ -744,11 +674,9 @@ PetscErrorCode PEPVectorsView(PEP pep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPVectorsViewFromOptions"
 /*@
    PEPVectorsViewFromOptions - Processes command line options to determine if/how
-   the computed eigenvectors are to be viewed. 
+   the computed eigenvectors are to be viewed.
 
    Collective on PEP
 
