@@ -1,30 +1,18 @@
 /*
-   This file implements a wrapper to the LAPACK SVD subroutines.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   This file implements a wrapper to the LAPACK SVD subroutines
+*/
 
 #include <slepc/private/svdimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetUp_LAPACK"
 PetscErrorCode SVDSetUp_LAPACK(SVD svd)
 {
   PetscErrorCode ierr;
@@ -43,19 +31,19 @@ PetscErrorCode SVDSetUp_LAPACK(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSolve_LAPACK"
 PetscErrorCode SVDSolve_LAPACK(SVD svd)
 {
   PetscErrorCode ierr;
-  PetscInt       M,N,n,i,j,k,ld;
-  Mat            mat;
+  PetscInt       M,N,n,i,j,k,ld,lowu,lowv,highu,highv;
+  Mat            Ar,mat;
   Vec            u,v;
   PetscScalar    *pU,*pVT,*pmat,*pu,*pv,*A,*w;
 
   PetscFunctionBegin;
   ierr = DSGetLeadingDimension(svd->ds,&ld);CHKERRQ(ierr);
-  ierr = MatConvert(svd->OP,MATSEQDENSE,MAT_INITIAL_MATRIX,&mat);CHKERRQ(ierr);
+  ierr = MatCreateRedundantMatrix(svd->OP,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
+  ierr = MatConvert(Ar,MATSEQDENSE,MAT_INITIAL_MATRIX,&mat);CHKERRQ(ierr);
+  ierr = MatDestroy(&Ar);CHKERRQ(ierr);
   ierr = MatGetSize(mat,&M,&N);CHKERRQ(ierr);
   ierr = DSSetDimensions(svd->ds,M,N,0,0);CHKERRQ(ierr);
   ierr = MatDenseGetArray(mat,&pmat);CHKERRQ(ierr);
@@ -71,6 +59,7 @@ PetscErrorCode SVDSolve_LAPACK(SVD svd)
   ierr = PetscMalloc1(n,&w);CHKERRQ(ierr);
   ierr = DSSolve(svd->ds,w,NULL);CHKERRQ(ierr);
   ierr = DSSort(svd->ds,w,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = DSSynchronize(svd->ds,w,NULL);CHKERRQ(ierr);
 
   /* copy singular vectors */
   ierr = DSGetArray(svd->ds,DS_MAT_U,&pU);CHKERRQ(ierr);
@@ -81,14 +70,16 @@ PetscErrorCode SVDSolve_LAPACK(SVD svd)
     svd->sigma[k] = PetscRealPart(w[i]);
     ierr = BVGetColumn(svd->U,k,&u);CHKERRQ(ierr);
     ierr = BVGetColumn(svd->V,k,&v);CHKERRQ(ierr);
+    ierr = VecGetOwnershipRange(u,&lowu,&highu);CHKERRQ(ierr);
+    ierr = VecGetOwnershipRange(v,&lowv,&highv);CHKERRQ(ierr);
     ierr = VecGetArray(u,&pu);CHKERRQ(ierr);
     ierr = VecGetArray(v,&pv);CHKERRQ(ierr);
     if (M>=N) {
-      for (j=0;j<M;j++) pu[j] = pU[i*ld+j];
-      for (j=0;j<N;j++) pv[j] = PetscConj(pVT[j*ld+i]);
+      for (j=lowu;j<highu;j++) pu[j-lowu] = pU[i*ld+j];
+      for (j=lowv;j<highv;j++) pv[j-lowv] = PetscConj(pVT[j*ld+i]);
     } else {
-      for (j=0;j<N;j++) pu[j] = PetscConj(pVT[j*ld+i]);
-      for (j=0;j<M;j++) pv[j] = pU[i*ld+j];
+      for (j=lowu;j<highu;j++) pu[j-lowu] = PetscConj(pVT[j*ld+i]);
+      for (j=lowv;j<highv;j++) pv[j-lowv] = pU[i*ld+j];
     }
     ierr = VecRestoreArray(u,&pu);CHKERRQ(ierr);
     ierr = VecRestoreArray(v,&pv);CHKERRQ(ierr);
@@ -106,8 +97,6 @@ PetscErrorCode SVDSolve_LAPACK(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDCreate_LAPACK"
 PETSC_EXTERN PetscErrorCode SVDCreate_LAPACK(SVD svd)
 {
   PetscFunctionBegin;
