@@ -1,25 +1,15 @@
 /*
-   The basic SVD routines, Create, Destroy, etc. are here.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic SVD routines
+*/
 
 #include <slepc/private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
@@ -28,8 +18,6 @@ PetscBool         SVDRegisterAllCalled = PETSC_FALSE;
 PetscClassId      SVD_CLASSID = 0;
 PetscLogEvent     SVD_SetUp = 0,SVD_Solve = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDCreate"
 /*@
    SVDCreate - Creates the default SVD context.
 
@@ -74,8 +62,10 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   svd->trackall         = PETSC_FALSE;
 
   svd->converged        = SVDConvergedRelative;
+  svd->convergeduser    = NULL;
   svd->convergeddestroy = NULL;
   svd->stopping         = SVDStoppingBasic;
+  svd->stoppinguser     = NULL;
   svd->stoppingdestroy  = NULL;
   svd->convergedctx     = NULL;
   svd->stoppingctx      = NULL;
@@ -104,11 +94,9 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDReset"
 /*@
-   SVDReset - Resets the SVD context to the initial state and removes any
-   allocated objects.
+   SVDReset - Resets the SVD context to the initial state (prior to setup)
+   and destroys any allocated Vecs and Mats.
 
    Collective on SVD
 
@@ -122,27 +110,20 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
 PetscErrorCode SVDReset(SVD svd)
 {
   PetscErrorCode ierr;
-  PetscInt       ncols;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (svd) PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (!svd) PetscFunctionReturn(0);
   if (svd->ops->reset) { ierr = (svd->ops->reset)(svd);CHKERRQ(ierr); }
-  if (svd->ds) { ierr = DSReset(svd->ds);CHKERRQ(ierr); }
   ierr = MatDestroy(&svd->OP);CHKERRQ(ierr);
   ierr = MatDestroy(&svd->A);CHKERRQ(ierr);
   ierr = MatDestroy(&svd->AT);CHKERRQ(ierr);
-  ierr = BVGetSizes(svd->V,NULL,NULL,&ncols);CHKERRQ(ierr);
-  if (ncols) {
-    ierr = PetscFree3(svd->sigma,svd->perm,svd->errest);CHKERRQ(ierr);
-  }
   ierr = BVDestroy(&svd->U);CHKERRQ(ierr);
   ierr = BVDestroy(&svd->V);CHKERRQ(ierr);
   svd->state = SVD_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDDestroy"
 /*@
    SVDDestroy - Destroys the SVD context.
 
@@ -165,6 +146,9 @@ PetscErrorCode SVDDestroy(SVD *svd)
   if (--((PetscObject)(*svd))->refct > 0) { *svd = 0; PetscFunctionReturn(0); }
   ierr = SVDReset(*svd);CHKERRQ(ierr);
   if ((*svd)->ops->destroy) { ierr = (*(*svd)->ops->destroy)(*svd);CHKERRQ(ierr); }
+  if ((*svd)->sigma) {
+    ierr = PetscFree3((*svd)->sigma,(*svd)->perm,(*svd)->errest);CHKERRQ(ierr);
+  }
   ierr = DSDestroy(&(*svd)->ds);CHKERRQ(ierr);
   ierr = PetscFree((*svd)->sc);CHKERRQ(ierr);
   /* just in case the initial vectors have not been used */
@@ -175,8 +159,6 @@ PetscErrorCode SVDDestroy(SVD *svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetType"
 /*@C
    SVDSetType - Selects the particular solver to be used in the SVD object.
 
@@ -230,8 +212,6 @@ PetscErrorCode SVDSetType(SVD svd,SVDType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetType"
 /*@C
    SVDGetType - Gets the SVD type as a string from the SVD object.
 
@@ -256,8 +236,6 @@ PetscErrorCode SVDGetType(SVD svd,SVDType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDRegister"
 /*@C
    SVDRegister - Adds a method to the singular value solver package.
 
@@ -293,8 +271,6 @@ PetscErrorCode SVDRegister(const char *name,PetscErrorCode (*function)(SVD))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetBV"
 /*@
    SVDSetBV - Associates basis vectors objects to the singular value solver.
 
@@ -338,8 +314,6 @@ PetscErrorCode SVDSetBV(SVD svd,BV V,BV U)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetBV"
 /*@
    SVDGetBV - Obtain the basis vectors objects associated to the singular
    value solver object.
@@ -380,8 +354,6 @@ PetscErrorCode SVDGetBV(SVD svd,BV *V,BV *U)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetDS"
 /*@
    SVDSetDS - Associates a direct solver object to the singular value solver.
 
@@ -414,8 +386,6 @@ PetscErrorCode SVDSetDS(SVD svd,DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetDS"
 /*@
    SVDGetDS - Obtain the direct solver object associated to the singular value
    solver object.
