@@ -1,31 +1,19 @@
 /*
-     SVD routines for setting solver options.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   SVD routines for setting solver options
+*/
 
 #include <slepc/private/svdimpl.h>      /*I "slepcsvd.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetImplicitTranspose"
 /*@
    SVDSetImplicitTranspose - Indicates how to handle the transpose of the matrix
    associated with the singular value problem.
@@ -65,8 +53,6 @@ PetscErrorCode SVDSetImplicitTranspose(SVD svd,PetscBool impl)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetImplicitTranspose"
 /*@
    SVDGetImplicitTranspose - Gets the mode used to handle the transpose
    of the matrix associated with the singular value problem.
@@ -92,8 +78,6 @@ PetscErrorCode SVDGetImplicitTranspose(SVD svd,PetscBool *impl)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetTolerances"
 /*@
    SVDSetTolerances - Sets the tolerance and maximum
    iteration count used by the default SVD convergence testers.
@@ -140,9 +124,7 @@ PetscErrorCode SVDSetTolerances(SVD svd,PetscReal tol,PetscInt maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetTolerances"
-/*@
+/*@C
    SVDGetTolerances - Gets the tolerance and maximum
    iteration count used by the default SVD convergence tests.
 
@@ -171,8 +153,6 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetDimensions"
 /*@
    SVDSetDimensions - Sets the number of singular values to compute
    and the dimension of the subspace.
@@ -232,9 +212,7 @@ PetscErrorCode SVDSetDimensions(SVD svd,PetscInt nsv,PetscInt ncv,PetscInt mpd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetDimensions"
-/*@
+/*@C
    SVDGetDimensions - Gets the number of singular values to compute
    and the dimension of the subspace.
 
@@ -265,8 +243,6 @@ PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetWhichSingularTriplets"
 /*@
     SVDSetWhichSingularTriplets - Specifies which singular triplets are
     to be sought.
@@ -312,8 +288,6 @@ PetscErrorCode SVDSetWhichSingularTriplets(SVD svd,SVDWhich which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetWhichSingularTriplets"
 /*@
     SVDGetWhichSingularTriplets - Returns which singular triplets are
     to be sought.
@@ -342,8 +316,6 @@ PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetConvergenceTestFunction"
 /*@C
    SVDSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
@@ -382,17 +354,18 @@ PetscErrorCode SVDSetConvergenceTestFunction(SVD svd,PetscErrorCode (*func)(SVD,
   if (svd->convergeddestroy) {
     ierr = (*svd->convergeddestroy)(svd->convergedctx);CHKERRQ(ierr);
   }
-  svd->converged        = func;
+  svd->convergeduser    = func;
   svd->convergeddestroy = destroy;
   svd->convergedctx     = ctx;
   if (func == SVDConvergedRelative) svd->conv = SVD_CONV_REL;
   else if (func == SVDConvergedAbsolute) svd->conv = SVD_CONV_ABS;
-  else svd->conv = SVD_CONV_USER;
+  else {
+    svd->conv      = SVD_CONV_USER;
+    svd->converged = svd->convergeduser;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetConvergenceTest"
 /*@
    SVDSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
@@ -426,7 +399,10 @@ PetscErrorCode SVDSetConvergenceTest(SVD svd,SVDConv conv)
   switch (conv) {
     case SVD_CONV_ABS:  svd->converged = SVDConvergedAbsolute; break;
     case SVD_CONV_REL:  svd->converged = SVDConvergedRelative; break;
-    case SVD_CONV_USER: break;
+    case SVD_CONV_USER:
+      if (!svd->convergeduser) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ORDER,"Must call SVDSetConvergenceTestFunction() first");
+      svd->converged = svd->convergeduser;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'conv' value");
   }
@@ -434,8 +410,6 @@ PetscErrorCode SVDSetConvergenceTest(SVD svd,SVDConv conv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetConvergenceTest"
 /*@
    SVDGetConvergenceTest - Gets the method used to compute the error estimate
    used in the convergence test.
@@ -461,8 +435,6 @@ PetscErrorCode SVDGetConvergenceTest(SVD svd,SVDConv *conv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetStoppingTestFunction"
 /*@C
    SVDSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the singular value solver.
@@ -505,16 +477,17 @@ PetscErrorCode SVDSetStoppingTestFunction(SVD svd,PetscErrorCode (*func)(SVD,Pet
   if (svd->stoppingdestroy) {
     ierr = (*svd->stoppingdestroy)(svd->stoppingctx);CHKERRQ(ierr);
   }
-  svd->stopping        = func;
+  svd->stoppinguser    = func;
   svd->stoppingdestroy = destroy;
   svd->stoppingctx     = ctx;
   if (func == SVDStoppingBasic) svd->stop = SVD_STOP_BASIC;
-  else svd->stop = SVD_STOP_USER;
+  else {
+    svd->stop     = SVD_STOP_USER;
+    svd->stopping = svd->stoppinguser;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetStoppingTest"
 /*@
    SVDSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the singular value solver.
@@ -545,7 +518,10 @@ PetscErrorCode SVDSetStoppingTest(SVD svd,SVDStop stop)
   PetscValidLogicalCollectiveEnum(svd,stop,2);
   switch (stop) {
     case SVD_STOP_BASIC: svd->stopping = SVDStoppingBasic; break;
-    case SVD_STOP_USER:  break;
+    case SVD_STOP_USER:
+      if (!svd->stoppinguser) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ORDER,"Must call SVDSetStoppingTestFunction() first");
+      svd->stopping = svd->stoppinguser;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'stop' value");
   }
@@ -553,8 +529,6 @@ PetscErrorCode SVDSetStoppingTest(SVD svd,SVDStop stop)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetStoppingTest"
 /*@
    SVDGetStoppingTest - Gets the method used to decide the termination of the outer
    loop of the singular value solver.
@@ -580,8 +554,6 @@ PetscErrorCode SVDGetStoppingTest(SVD svd,SVDStop *stop)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDMonitorSetFromOptions"
 /*@C
    SVDMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
@@ -621,8 +593,6 @@ PetscErrorCode SVDMonitorSetFromOptions(SVD svd,const char name[],const char hel
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDConvMonitorSetFromOptions"
 /*@C
    SVDConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
@@ -658,8 +628,6 @@ PetscErrorCode SVDConvMonitorSetFromOptions(SVD svd,const char name[],const char
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetFromOptions"
 /*@
    SVDSetFromOptions - Sets SVD options from the options database.
    This routine must be called before SVDSetUp() if the user is to be
@@ -690,32 +658,21 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   ierr = SVDRegisterAll();CHKERRQ(ierr);
   ierr = PetscObjectOptionsBegin((PetscObject)svd);CHKERRQ(ierr);
-    ierr = PetscOptionsFList("-svd_type","Singular Value Solver method","SVDSetType",SVDList,(char*)(((PetscObject)svd)->type_name?((PetscObject)svd)->type_name:SVDCROSS),type,256,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsFList("-svd_type","SVD solver method","SVDSetType",SVDList,(char*)(((PetscObject)svd)->type_name?((PetscObject)svd)->type_name:SVDCROSS),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = SVDSetType(svd,type);CHKERRQ(ierr);
     } else if (!((PetscObject)svd)->type_name) {
       ierr = SVDSetType(svd,SVDCROSS);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsName("-svd_view","Print detailed information on solver used","SVDView",NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsName("-svd_view_vectors","View computed singular vectors","SVDVectorsView",NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsName("-svd_view_values","View computed singular values","SVDValuesView",NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsName("-svd_converged_reason","Print reason for convergence, and number of iterations","SVDReasonView",NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsName("-svd_error_absolute","Print absolute errors of each singular triplet","SVDErrorView",NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsName("-svd_error_relative","Print relative errors of each singular triplet","SVDErrorView",NULL);CHKERRQ(ierr);
-
     ierr = PetscOptionsBool("-svd_implicittranspose","Handle matrix transpose implicitly","SVDSetImplicitTranspose",svd->impltrans,&val,&flg);CHKERRQ(ierr);
-    if (flg) {
-      ierr = SVDSetImplicitTranspose(svd,val);CHKERRQ(ierr);
-    }
+    if (flg) { ierr = SVDSetImplicitTranspose(svd,val);CHKERRQ(ierr); }
 
     i = svd->max_it? svd->max_it: PETSC_DEFAULT;
     ierr = PetscOptionsInt("-svd_max_it","Maximum number of iterations","SVDSetTolerances",svd->max_it,&i,&flg1);CHKERRQ(ierr);
     r = svd->tol;
     ierr = PetscOptionsReal("-svd_tol","Tolerance","SVDSetTolerances",svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:svd->tol,&r,&flg2);CHKERRQ(ierr);
-    if (flg1 || flg2) {
-      ierr = SVDSetTolerances(svd,r,i);CHKERRQ(ierr);
-    }
+    if (flg1 || flg2) { ierr = SVDSetTolerances(svd,r,i);CHKERRQ(ierr); }
 
     ierr = PetscOptionsBoolGroupBegin("-svd_conv_rel","Relative error convergence test","SVDSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) { ierr = SVDSetConvergenceTest(svd,SVD_CONV_REL);CHKERRQ(ierr); }
@@ -735,13 +692,11 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
     ierr = PetscOptionsInt("-svd_ncv","Number of basis vectors","SVDSetDimensions",svd->ncv,&j,&flg2);CHKERRQ(ierr);
     k = svd->mpd? svd->mpd: PETSC_DEFAULT;
     ierr = PetscOptionsInt("-svd_mpd","Maximum dimension of projected problem","SVDSetDimensions",svd->mpd,&k,&flg3);CHKERRQ(ierr);
-    if (flg1 || flg2 || flg3) {
-      ierr = SVDSetDimensions(svd,i,j,k);CHKERRQ(ierr);
-    }
+    if (flg1 || flg2 || flg3) { ierr = SVDSetDimensions(svd,i,j,k);CHKERRQ(ierr); }
 
-    ierr = PetscOptionsBoolGroupBegin("-svd_largest","compute largest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupBegin("-svd_largest","Compute largest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
     if (flg) { ierr = SVDSetWhichSingularTriplets(svd,SVD_LARGEST);CHKERRQ(ierr); }
-    ierr = PetscOptionsBoolGroupEnd("-svd_smallest","compute smallest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupEnd("-svd_smallest","Compute smallest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
     if (flg) { ierr = SVDSetWhichSingularTriplets(svd,SVD_SMALLEST);CHKERRQ(ierr); }
 
     /* -----------------------------------------------------------------------*/
@@ -773,6 +728,14 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
       ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
     }
 
+    /* -----------------------------------------------------------------------*/
+    ierr = PetscOptionsName("-svd_view","Print detailed information on solver used","SVDView",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsName("-svd_view_vectors","View computed singular vectors","SVDVectorsView",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsName("-svd_view_values","View computed singular values","SVDValuesView",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsName("-svd_converged_reason","Print reason for convergence, and number of iterations","SVDReasonView",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsName("-svd_error_absolute","Print absolute errors of each singular triplet","SVDErrorView",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsName("-svd_error_relative","Print relative errors of each singular triplet","SVDErrorView",NULL);CHKERRQ(ierr);
+
     if (svd->ops->setfromoptions) {
       ierr = (*svd->ops->setfromoptions)(PetscOptionsObject,svd);CHKERRQ(ierr);
     }
@@ -787,8 +750,6 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetTrackAll"
 /*@
    SVDSetTrackAll - Specifies if the solver must compute the residual norm of all
    approximate singular value or not.
@@ -821,8 +782,6 @@ PetscErrorCode SVDSetTrackAll(SVD svd,PetscBool trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetTrackAll"
 /*@
    SVDGetTrackAll - Returns the flag indicating whether all residual norms must
    be computed or not.
@@ -849,8 +808,6 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetOptionsPrefix"
 /*@C
    SVDSetOptionsPrefix - Sets the prefix used for searching for all
    SVD options in the database.
@@ -880,8 +837,6 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall)
 PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
 {
   PetscErrorCode ierr;
-  PetscBool      flg1,flg2;
-  EPS            eps;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
@@ -891,22 +846,9 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
   if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
   ierr = DSSetOptionsPrefix(svd->ds,prefix);CHKERRQ(ierr);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
-  if (flg1) {
-    ierr = SVDCrossGetEPS(svd,&eps);CHKERRQ(ierr);
-  } else if (flg2) {
-      ierr = SVDCyclicGetEPS(svd,&eps);CHKERRQ(ierr);
-  }
-  if (flg1 || flg2) {
-    ierr = EPSSetOptionsPrefix(eps,prefix);CHKERRQ(ierr);
-    ierr = EPSAppendOptionsPrefix(eps,"svd_");CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDAppendOptionsPrefix"
 /*@C
    SVDAppendOptionsPrefix - Appends to the prefix used for searching for all
    SVD options in the database.
@@ -928,33 +870,18 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
 PetscErrorCode SVDAppendOptionsPrefix(SVD svd,const char *prefix)
 {
   PetscErrorCode ierr;
-  PetscBool      flg1,flg2;
-  EPS            eps;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (!svd->V) { ierr = SVDGetBV(svd,&svd->V,&svd->U);CHKERRQ(ierr); }
-  ierr = BVSetOptionsPrefix(svd->V,prefix);CHKERRQ(ierr);
-  ierr = BVSetOptionsPrefix(svd->U,prefix);CHKERRQ(ierr);
+  ierr = BVAppendOptionsPrefix(svd->V,prefix);CHKERRQ(ierr);
+  ierr = BVAppendOptionsPrefix(svd->U,prefix);CHKERRQ(ierr);
   if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
-  ierr = DSSetOptionsPrefix(svd->ds,prefix);CHKERRQ(ierr);
+  ierr = DSAppendOptionsPrefix(svd->ds,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
-  if (flg1) {
-    ierr = SVDCrossGetEPS(svd,&eps);CHKERRQ(ierr);
-  } else if (flg2) {
-    ierr = SVDCyclicGetEPS(svd,&eps);CHKERRQ(ierr);
-  }
-  if (flg1 || flg2) {
-    ierr = EPSSetOptionsPrefix(eps,((PetscObject)svd)->prefix);CHKERRQ(ierr);
-    ierr = EPSAppendOptionsPrefix(eps,"svd_");CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetOptionsPrefix"
 /*@C
    SVDGetOptionsPrefix - Gets the prefix used for searching for all
    SVD options in the database.
