@@ -1,30 +1,18 @@
 /*
-     SVD routines for setting up the solver.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   SVD routines for setting up the solver
+*/
 
 #include <slepc/private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetOperator"
 /*@
    SVDSetOperator - Set the matrix associated with the singular value problem.
 
@@ -47,14 +35,13 @@ PetscErrorCode SVDSetOperator(SVD svd,Mat mat)
   PetscValidHeaderSpecific(mat,MAT_CLASSID,2);
   PetscCheckSameComm(svd,1,mat,2);
   if (svd->state) { ierr = SVDReset(svd);CHKERRQ(ierr); }
+  else { ierr = MatDestroy(&svd->OP);CHKERRQ(ierr); }
   ierr = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);
-  ierr = MatDestroy(&svd->OP);CHKERRQ(ierr);
   svd->OP = mat;
+  svd->state = SVD_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDGetOperator"
 /*@
    SVDGetOperator - Get the matrix associated with the singular value problem.
 
@@ -79,8 +66,6 @@ PetscErrorCode SVDGetOperator(SVD svd,Mat *A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetUp"
 /*@
    SVDSetUp - Sets up all the internal data structures necessary for the
    execution of the singular value solver.
@@ -120,7 +105,6 @@ PetscErrorCode SVDSetUp(SVD svd)
     ierr = SVDSetType(svd,SVDCROSS);CHKERRQ(ierr);
   }
   if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
-  ierr = DSReset(svd->ds);CHKERRQ(ierr);
 
   /* check matrix */
   if (!svd->OP) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONGSTATE,"SVDSetOperator must be called first");
@@ -211,21 +195,23 @@ PetscErrorCode SVDSetUp(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetInitialSpace"
-/*@
-   SVDSetInitialSpace - Specify a basis of vectors that constitute the initial
-   (right) space, that is, a rough approximation to the right singular subspace
-   from which the solver starts to iterate.
+/*@C
+   SVDSetInitialSpaces - Specify two basis of vectors that constitute the initial
+   right and/or left spaces, that is, a rough approximation to the right and/or
+   left singular subspaces from which the solver starts to iterate.
 
    Collective on SVD and Vec
 
    Input Parameter:
 +  svd   - the singular value solver context
-.  n     - number of vectors
--  is    - set of basis vectors of the initial space
+.  nr    - number of right vectors
+.  isr   - set of basis vectors of the right initial space
+.  nl    - number of left vectors
+-  isl   - set of basis vectors of the left initial space
 
    Notes:
+   It is not necessary to provide both sets of vectors.
+
    Some solvers start to iterate on a single vector (initial vector). In that case,
    the other vectors are ignored.
 
@@ -239,68 +225,23 @@ PetscErrorCode SVDSetUp(SVD svd)
    of the wanted singular space. Then, convergence may be faster.
 
    Level: intermediate
-
-.seealso: SVDSetInitialSpaceLeft()
 @*/
-PetscErrorCode SVDSetInitialSpace(SVD svd,PetscInt n,Vec *is)
+PetscErrorCode SVDSetInitialSpaces(SVD svd,PetscInt nr,Vec *isr,PetscInt nl,Vec *isl)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidLogicalCollectiveInt(svd,n,2);
-  if (n<0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
-  ierr = SlepcBasisReference_Private(n,is,&svd->nini,&svd->IS);CHKERRQ(ierr);
-  if (n>0) svd->state = SVD_STATE_INITIAL;
+  PetscValidLogicalCollectiveInt(svd,nr,2);
+  PetscValidLogicalCollectiveInt(svd,nl,4);
+  if (nr<0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Argument nr cannot be negative");
+  if (nl<0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Argument nl cannot be negative");
+  ierr = SlepcBasisReference_Private(nr,isr,&svd->nini,&svd->IS);CHKERRQ(ierr);
+  ierr = SlepcBasisReference_Private(nl,isl,&svd->ninil,&svd->ISL);CHKERRQ(ierr);
+  if (nr>0 || nl>0) svd->state = SVD_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetInitialSpaceLeft"
-/*@
-   SVDSetInitialSpaceLeft - Specify a basis of vectors that constitute the initial
-   left space, that is, a rough approximation to the left singular subspace
-   from which the solver starts to iterate.
-
-   Collective on SVD and Vec
-
-   Input Parameter:
-+  svd   - the singular value solver context
-.  n     - number of vectors
--  is    - set of basis vectors of the initial space
-
-   Notes:
-   Some solvers start to iterate on a single vector (initial vector). In that case,
-   the other vectors are ignored.
-
-   These vectors do not persist from one SVDSolve() call to the other, so the
-   initial space should be set every time.
-
-   The vectors do not need to be mutually orthonormal, since they are explicitly
-   orthonormalized internally.
-
-   Common usage of this function is when the user can provide a rough approximation
-   of the wanted singular space. Then, convergence may be faster.
-
-   Level: intermediate
-
-.seealso: SVDSetInitialSpace()
-@*/
-PetscErrorCode SVDSetInitialSpaceLeft(SVD svd,PetscInt n,Vec *is)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidLogicalCollectiveInt(svd,n,2);
-  if (n<0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
-  ierr = SlepcBasisReference_Private(n,is,&svd->ninil,&svd->ISL);CHKERRQ(ierr);
-  if (n>0) svd->state = SVD_STATE_INITIAL;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SVDSetDimensions_Default"
 /*
   SVDSetDimensions_Default - Set reasonable values for ncv, mpd if not set
   by the user. This is called at setup.
@@ -327,8 +268,6 @@ PetscErrorCode SVDSetDimensions_Default(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDAllocateSolution"
 /*@
    SVDAllocateSolution - Allocate memory storage for common variables such
    as the singular values and the basis vectors.
@@ -362,9 +301,7 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
 
   /* allocate sigma */
   if (requested != oldsize || !svd->sigma) {
-    if (oldsize) {
-      ierr = PetscFree3(svd->sigma,svd->perm,svd->errest);CHKERRQ(ierr);
-    }
+    ierr = PetscFree3(svd->sigma,svd->perm,svd->errest);CHKERRQ(ierr);
     ierr = PetscMalloc3(requested,&svd->sigma,requested,&svd->perm,requested,&svd->errest);CHKERRQ(ierr);
     ierr = PetscLogObjectMemory((PetscObject)svd,PetscMax(0,requested-oldsize)*(2*sizeof(PetscReal)+sizeof(PetscInt)));CHKERRQ(ierr);
   }
@@ -374,7 +311,7 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
     if (!((PetscObject)(svd->V))->type_name) {
       ierr = BVSetType(svd->V,BVSVEC);CHKERRQ(ierr);
     }
-    ierr = SVDMatCreateVecs(svd,&tr,NULL);CHKERRQ(ierr);
+    ierr = SVDMatCreateVecsEmpty(svd,&tr,NULL);CHKERRQ(ierr);
     ierr = BVSetSizesFromVec(svd->V,tr,requested);CHKERRQ(ierr);
     ierr = VecDestroy(&tr);CHKERRQ(ierr);
   } else {
@@ -387,7 +324,7 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
       if (!((PetscObject)(svd->U))->type_name) {
         ierr = BVSetType(svd->U,BVSVEC);CHKERRQ(ierr);
       }
-      ierr = SVDMatCreateVecs(svd,NULL,&tl);CHKERRQ(ierr);
+      ierr = SVDMatCreateVecsEmpty(svd,NULL,&tl);CHKERRQ(ierr);
       ierr = BVSetSizesFromVec(svd->U,tl,requested);CHKERRQ(ierr);
       ierr = VecDestroy(&tl);CHKERRQ(ierr);
     } else {
