@@ -1,31 +1,19 @@
 /*
-   The SVD routines related to various viewers.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   The SVD routines related to various viewers
+*/
 
 #include <slepc/private/svdimpl.h>      /*I "slepcsvd.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDView"
 /*@C
    SVDView - Prints the SVD data structure.
 
@@ -105,7 +93,7 @@ PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
       ierr = (*svd->ops->view)(svd,viewer);CHKERRQ(ierr);
     }
   }
-  ierr = PetscObjectTypeCompareAny((PetscObject)svd,&isshell,SVDCROSS,SVDCYCLIC,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)svd,&isshell,SVDCROSS,SVDCYCLIC,SVDPRIMME,"");CHKERRQ(ierr);
   if (!isshell) {
     ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
     if (!svd->V) { ierr = SVDGetBV(svd,&svd->V,NULL);CHKERRQ(ierr); }
@@ -117,8 +105,6 @@ PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDReasonView"
 /*@C
    SVDReasonView - Displays the reason an SVD solve converged or diverged.
 
@@ -154,11 +140,9 @@ PetscErrorCode SVDReasonView(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDReasonViewFromOptions"
 /*@
    SVDReasonViewFromOptions - Processes command line options to determine if/how
-   the SVD converged reason is to be viewed. 
+   the SVD converged reason is to be viewed.
 
    Collective on SVD
 
@@ -189,11 +173,8 @@ PetscErrorCode SVDReasonViewFromOptions(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDErrorView_ASCII"
 static PetscErrorCode SVDErrorView_ASCII(SVD svd,SVDErrorType etype,PetscViewer viewer)
 {
-  PetscBool      errok;
   PetscReal      error,sigma;
   PetscInt       i,j;
   PetscErrorCode ierr;
@@ -203,14 +184,12 @@ static PetscErrorCode SVDErrorView_ASCII(SVD svd,SVDErrorType etype,PetscViewer
     ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D singular values converged\n\n",svd->nsv);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-  errok = PETSC_TRUE;
   for (i=0;i<svd->nsv;i++) {
     ierr = SVDComputeError(svd,i,etype,&error);CHKERRQ(ierr);
-    errok = (errok && error<5.0*svd->tol)? PETSC_TRUE: PETSC_FALSE;
-  }
-  if (!errok) {
-    ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",svd->nsv);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
+    if (error>=5.0*svd->tol) {
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",svd->nsv);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
   }
   ierr = PetscViewerASCIIPrintf(viewer," All requested singular values computed up to the required tolerance:");CHKERRQ(ierr);
   for (i=0;i<=(svd->nsv-1)/8;i++) {
@@ -225,8 +204,6 @@ static PetscErrorCode SVDErrorView_ASCII(SVD svd,SVDErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDErrorView_DETAIL"
 static PetscErrorCode SVDErrorView_DETAIL(SVD svd,SVDErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -255,8 +232,6 @@ static PetscErrorCode SVDErrorView_DETAIL(SVD svd,SVDErrorType etype,PetscViewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDErrorView_MATLAB"
 static PetscErrorCode SVDErrorView_MATLAB(SVD svd,SVDErrorType etype,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -269,14 +244,12 @@ static PetscErrorCode SVDErrorView_MATLAB(SVD svd,SVDErrorType etype,PetscViewer
   ierr = PetscViewerASCIIPrintf(viewer,"Error_%s = [\n",name);CHKERRQ(ierr);
   for (i=0;i<svd->nconv;i++) {
     ierr = SVDComputeError(svd,i,etype,&error);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",error);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%18.16e\n",(double)error);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"];\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDErrorView"
 /*@C
    SVDErrorView - Displays the errors associated with the computed solution
    (as well as the singular values).
@@ -335,11 +308,9 @@ PetscErrorCode SVDErrorView(SVD svd,SVDErrorType etype,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDErrorViewFromOptions"
 /*@
    SVDErrorViewFromOptions - Processes command line options to determine if/how
-   the errors of the computed solution are to be viewed. 
+   the errors of the computed solution are to be viewed.
 
    Collective on SVD
 
@@ -377,8 +348,6 @@ PetscErrorCode SVDErrorViewFromOptions(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDValuesView_DRAW"
 static PetscErrorCode SVDValuesView_DRAW(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -389,8 +358,8 @@ static PetscErrorCode SVDValuesView_DRAW(SVD svd,PetscViewer viewer)
 
   PetscFunctionBegin;
   if (!svd->nconv) PetscFunctionReturn(0);
-  ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed singular values",PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+  ierr = PetscDrawSetTitle(draw,"Computed singular values");CHKERRQ(ierr);
   ierr = PetscDrawSPCreate(draw,1,&drawsp);CHKERRQ(ierr);
   for (i=0;i<svd->nconv;i++) {
     re = svd->sigma[svd->perm[i]];
@@ -399,12 +368,9 @@ static PetscErrorCode SVDValuesView_DRAW(SVD svd,PetscViewer viewer)
   ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscDrawSPSave(drawsp);CHKERRQ(ierr);
   ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDValuesView_ASCII"
 static PetscErrorCode SVDValuesView_ASCII(SVD svd,PetscViewer viewer)
 {
   PetscInt       i;
@@ -419,8 +385,6 @@ static PetscErrorCode SVDValuesView_ASCII(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDValuesView_MATLAB"
 static PetscErrorCode SVDValuesView_MATLAB(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -437,8 +401,6 @@ static PetscErrorCode SVDValuesView_MATLAB(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDValuesView"
 /*@C
    SVDValuesView - Displays the computed singular values in a viewer.
 
@@ -489,11 +451,9 @@ PetscErrorCode SVDValuesView(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDValuesViewFromOptions"
 /*@
    SVDValuesViewFromOptions - Processes command line options to determine if/how
-   the computed singular values are to be viewed. 
+   the computed singular values are to be viewed.
 
    Collective on SVD
 
@@ -524,8 +484,6 @@ PetscErrorCode SVDValuesViewFromOptions(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDVectorsView"
 /*@C
    SVDVectorsView - Outputs computed singular vectors to a viewer.
 
@@ -577,11 +535,9 @@ PetscErrorCode SVDVectorsView(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SVDVectorsViewFromOptions"
 /*@
    SVDVectorsViewFromOptions - Processes command line options to determine if/how
-   the computed singular vectors are to be viewed. 
+   the computed singular vectors are to be viewed.
 
    Collective on SVD
 
