@@ -1,25 +1,15 @@
 /*
-   BV implemented with a dense Mat
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   BV implemented with a dense Mat
+*/
 
 #include <slepc/private/bvimpl.h>
 
@@ -28,8 +18,6 @@ typedef struct {
   PetscBool mpi;
 } BV_MAT;
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMult_Mat"
 PetscErrorCode BVMult_Mat(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 {
   PetscErrorCode ierr;
@@ -53,25 +41,23 @@ PetscErrorCode BVMult_Mat(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultVec_Mat"
 PetscErrorCode BVMultVec_Mat(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScalar *q)
 {
   PetscErrorCode ierr;
   BV_MAT         *x = (BV_MAT*)X->data;
-  PetscScalar    *px,*py;
+  PetscScalar    *px,*py,*qq=q;
 
   PetscFunctionBegin;
   ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
-  ierr = BVMultVec_BLAS_Private(X,X->n,X->k-X->l,alpha,px+(X->nc+X->l)*X->n,q,beta,py);CHKERRQ(ierr);
+  if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
+  ierr = BVMultVec_BLAS_Private(X,X->n,X->k-X->l,alpha,px+(X->nc+X->l)*X->n,qq,beta,py);CHKERRQ(ierr);
+  if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
   ierr = VecRestoreArray(y,&py);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlace_Mat"
 PetscErrorCode BVMultInPlace_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
 {
   PetscErrorCode ierr;
@@ -89,8 +75,6 @@ PetscErrorCode BVMultInPlace_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlaceTranspose_Mat"
 PetscErrorCode BVMultInPlaceTranspose_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
 {
   PetscErrorCode ierr;
@@ -108,8 +92,6 @@ PetscErrorCode BVMultInPlaceTranspose_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDot_Mat"
 PetscErrorCode BVDot_Mat(BV X,BV Y,Mat M)
 {
   PetscErrorCode ierr;
@@ -129,13 +111,11 @@ PetscErrorCode BVDot_Mat(BV X,BV Y,Mat M)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_Mat"
-PetscErrorCode BVDotVec_Mat(BV X,Vec y,PetscScalar *m)
+PetscErrorCode BVDotVec_Mat(BV X,Vec y,PetscScalar *q)
 {
   PetscErrorCode    ierr;
   BV_MAT            *x = (BV_MAT*)X->data;
-  PetscScalar       *px;
+  PetscScalar       *px,*qq=q;
   const PetscScalar *py;
   Vec               z = y;
 
@@ -146,14 +126,14 @@ PetscErrorCode BVDotVec_Mat(BV X,Vec y,PetscScalar *m)
   }
   ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
   ierr = VecGetArrayRead(z,&py);CHKERRQ(ierr);
-  ierr = BVDotVec_BLAS_Private(X,X->n,X->k-X->l,px+(X->nc+X->l)*X->n,py,m,x->mpi);CHKERRQ(ierr);
+  if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
+  ierr = BVDotVec_BLAS_Private(X,X->n,X->k-X->l,px+(X->nc+X->l)*X->n,py,qq,x->mpi);CHKERRQ(ierr);
+  if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = VecRestoreArrayRead(z,&py);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_Local_Mat"
 PetscErrorCode BVDotVec_Local_Mat(BV X,Vec y,PetscScalar *m)
 {
   PetscErrorCode ierr;
@@ -174,8 +154,6 @@ PetscErrorCode BVDotVec_Local_Mat(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVScale_Mat"
 PetscErrorCode BVScale_Mat(BV bv,PetscInt j,PetscScalar alpha)
 {
   PetscErrorCode ierr;
@@ -193,8 +171,6 @@ PetscErrorCode BVScale_Mat(BV bv,PetscInt j,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Mat"
 PetscErrorCode BVNorm_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -212,8 +188,6 @@ PetscErrorCode BVNorm_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Local_Mat"
 PetscErrorCode BVNorm_Local_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -231,25 +205,6 @@ PetscErrorCode BVNorm_Local_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVOrthogonalize_Mat"
-PetscErrorCode BVOrthogonalize_Mat(BV V,Mat R)
-{
-  PetscErrorCode ierr;
-  BV_MAT         *ctx = (BV_MAT*)V->data;
-  PetscScalar    *pv,*r=NULL;
-
-  PetscFunctionBegin;
-  if (R) { ierr = MatDenseGetArray(R,&r);CHKERRQ(ierr); }
-  ierr = MatDenseGetArray(ctx->A,&pv);CHKERRQ(ierr);
-  ierr = BVOrthogonalize_LAPACK_Private(V,V->n,V->k,pv+V->nc*V->n,r,ctx->mpi);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(ctx->A,&pv);CHKERRQ(ierr);
-  if (R) { ierr = MatDenseRestoreArray(R,&r);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "BVMatMult_Mat"
 PetscErrorCode BVMatMult_Mat(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
@@ -297,8 +252,6 @@ PetscErrorCode BVMatMult_Mat(BV V,Mat A,BV W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCopy_Mat"
 PetscErrorCode BVCopy_Mat(BV V,BV W)
 {
   PetscErrorCode ierr;
@@ -316,8 +269,6 @@ PetscErrorCode BVCopy_Mat(BV V,BV W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVResize_Mat"
 PetscErrorCode BVResize_Mat(BV bv,PetscInt m,PetscBool copy)
 {
   PetscErrorCode ierr;
@@ -347,8 +298,6 @@ PetscErrorCode BVResize_Mat(BV bv,PetscInt m,PetscBool copy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetColumn_Mat"
 PetscErrorCode BVGetColumn_Mat(BV bv,PetscInt j,Vec *v)
 {
   PetscErrorCode ierr;
@@ -363,8 +312,6 @@ PetscErrorCode BVGetColumn_Mat(BV bv,PetscInt j,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreColumn_Mat"
 PetscErrorCode BVRestoreColumn_Mat(BV bv,PetscInt j,Vec *v)
 {
   PetscErrorCode ierr;
@@ -379,8 +326,6 @@ PetscErrorCode BVRestoreColumn_Mat(BV bv,PetscInt j,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetArray_Mat"
 PetscErrorCode BVGetArray_Mat(BV bv,PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -391,8 +336,6 @@ PetscErrorCode BVGetArray_Mat(BV bv,PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreArray_Mat"
 PetscErrorCode BVRestoreArray_Mat(BV bv,PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -403,8 +346,6 @@ PetscErrorCode BVRestoreArray_Mat(BV bv,PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetArrayRead_Mat"
 PetscErrorCode BVGetArrayRead_Mat(BV bv,const PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -415,8 +356,6 @@ PetscErrorCode BVGetArrayRead_Mat(BV bv,const PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreArrayRead_Mat"
 PetscErrorCode BVRestoreArrayRead_Mat(BV bv,const PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -427,8 +366,6 @@ PetscErrorCode BVRestoreArrayRead_Mat(BV bv,const PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVView_Mat"
 PetscErrorCode BVView_Mat(BV bv,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -454,8 +391,6 @@ PetscErrorCode BVView_Mat(BV bv,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDestroy_Mat"
 PetscErrorCode BVDestroy_Mat(BV bv)
 {
   PetscErrorCode ierr;
@@ -469,8 +404,6 @@ PetscErrorCode BVDestroy_Mat(BV bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCreate_Mat"
 PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
 {
   PetscErrorCode ierr;
@@ -486,7 +419,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
   ierr = PetscObjectTypeCompare((PetscObject)bv->t,VECMPI,&ctx->mpi);CHKERRQ(ierr);
   if (!ctx->mpi) {
     ierr = PetscObjectTypeCompare((PetscObject)bv->t,VECSEQ,&seq);CHKERRQ(ierr);
-    if (!seq) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot create a BVMAT from a non-standard template vector");
+    if (!seq) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Cannot create a BVMAT from a non-standard template vector");
   }
 
   ierr = VecGetLocalSize(bv->t,&nloc);CHKERRQ(ierr);
@@ -501,6 +434,11 @@ PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
     ierr = PetscObjectSetName((PetscObject)ctx->A,str);CHKERRQ(ierr);
   }
 
+  if (bv->Acreate) {
+    ierr = MatCopy(bv->Acreate,ctx->A,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+    ierr = MatDestroy(&bv->Acreate);CHKERRQ(ierr);
+  }
+
   if (ctx->mpi) {
     ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,NULL,&bv->cv[0]);CHKERRQ(ierr);
     ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,NULL,&bv->cv[1]);CHKERRQ(ierr);
@@ -519,7 +457,6 @@ PETSC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
   bv->ops->scale            = BVScale_Mat;
   bv->ops->norm             = BVNorm_Mat;
   bv->ops->norm_local       = BVNorm_Local_Mat;
-  /*bv->ops->orthogonalize    = BVOrthogonalize_Mat;*/
   bv->ops->matmult          = BVMatMult_Mat;
   bv->ops->copy             = BVCopy_Mat;
   bv->ops->resize           = BVResize_Mat;
