@@ -1,35 +1,19 @@
 /*
-   BV implemented as a single Vec
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   BV implemented as a single Vec
+*/
 
 #include <slepc/private/bvimpl.h>
+#include "./svecimpl.h"
 
-typedef struct {
-  Vec       v;
-  PetscBool mpi;
-} BV_SVEC;
-
-#undef __FUNCT__
-#define __FUNCT__ "BVMult_Svec"
 PetscErrorCode BVMult_Svec(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 {
   PetscErrorCode    ierr;
@@ -54,25 +38,23 @@ PetscErrorCode BVMult_Svec(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultVec_Svec"
 PetscErrorCode BVMultVec_Svec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScalar *q)
 {
   PetscErrorCode ierr;
   BV_SVEC        *x = (BV_SVEC*)X->data;
-  PetscScalar    *px,*py;
+  PetscScalar    *px,*py,*qq=q;
 
   PetscFunctionBegin;
   ierr = VecGetArray(x->v,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
-  ierr = BVMultVec_BLAS_Private(X,X->n,X->k-X->l,alpha,px+(X->nc+X->l)*X->n,q,beta,py);CHKERRQ(ierr);
+  if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
+  ierr = BVMultVec_BLAS_Private(X,X->n,X->k-X->l,alpha,px+(X->nc+X->l)*X->n,qq,beta,py);CHKERRQ(ierr);
+  if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = VecRestoreArray(x->v,&px);CHKERRQ(ierr);
   ierr = VecRestoreArray(y,&py);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlace_Svec"
 PetscErrorCode BVMultInPlace_Svec(BV V,Mat Q,PetscInt s,PetscInt e)
 {
   PetscErrorCode ierr;
@@ -90,8 +72,6 @@ PetscErrorCode BVMultInPlace_Svec(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlaceTranspose_Svec"
 PetscErrorCode BVMultInPlaceTranspose_Svec(BV V,Mat Q,PetscInt s,PetscInt e)
 {
   PetscErrorCode ierr;
@@ -109,8 +89,6 @@ PetscErrorCode BVMultInPlaceTranspose_Svec(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDot_Svec"
 PetscErrorCode BVDot_Svec(BV X,BV Y,Mat M)
 {
   PetscErrorCode    ierr;
@@ -131,13 +109,12 @@ PetscErrorCode BVDot_Svec(BV X,BV Y,Mat M)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_Svec"
-PetscErrorCode BVDotVec_Svec(BV X,Vec y,PetscScalar *m)
+PetscErrorCode BVDotVec_Svec(BV X,Vec y,PetscScalar *q)
 {
   PetscErrorCode    ierr;
   BV_SVEC           *x = (BV_SVEC*)X->data;
   const PetscScalar *px,*py;
+  PetscScalar       *qq=q;
   Vec               z = y;
 
   PetscFunctionBegin;
@@ -147,14 +124,14 @@ PetscErrorCode BVDotVec_Svec(BV X,Vec y,PetscScalar *m)
   }
   ierr = VecGetArrayRead(x->v,&px);CHKERRQ(ierr);
   ierr = VecGetArrayRead(z,&py);CHKERRQ(ierr);
-  ierr = BVDotVec_BLAS_Private(X,X->n,X->k-X->l,px+(X->nc+X->l)*X->n,py,m,x->mpi);CHKERRQ(ierr);
+  if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
+  ierr = BVDotVec_BLAS_Private(X,X->n,X->k-X->l,px+(X->nc+X->l)*X->n,py,qq,x->mpi);CHKERRQ(ierr);
+  if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = VecRestoreArrayRead(z,&py);CHKERRQ(ierr);
   ierr = VecRestoreArrayRead(x->v,&px);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_Local_Svec"
 PetscErrorCode BVDotVec_Local_Svec(BV X,Vec y,PetscScalar *m)
 {
   PetscErrorCode ierr;
@@ -175,8 +152,6 @@ PetscErrorCode BVDotVec_Local_Svec(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVScale_Svec"
 PetscErrorCode BVScale_Svec(BV bv,PetscInt j,PetscScalar alpha)
 {
   PetscErrorCode ierr;
@@ -194,8 +169,6 @@ PetscErrorCode BVScale_Svec(BV bv,PetscInt j,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Svec"
 PetscErrorCode BVNorm_Svec(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -213,8 +186,6 @@ PetscErrorCode BVNorm_Svec(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Local_Svec"
 PetscErrorCode BVNorm_Local_Svec(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -232,25 +203,6 @@ PetscErrorCode BVNorm_Local_Svec(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVOrthogonalize_Svec"
-PetscErrorCode BVOrthogonalize_Svec(BV V,Mat R)
-{
-  PetscErrorCode ierr;
-  BV_SVEC        *ctx = (BV_SVEC*)V->data;
-  PetscScalar    *pv,*r=NULL;
-
-  PetscFunctionBegin;
-  if (R) { ierr = MatDenseGetArray(R,&r);CHKERRQ(ierr); }
-  ierr = VecGetArray(ctx->v,&pv);CHKERRQ(ierr);
-  ierr = BVOrthogonalize_LAPACK_Private(V,V->n,V->k,pv+V->nc*V->n,r,ctx->mpi);CHKERRQ(ierr);
-  ierr = VecRestoreArray(ctx->v,&pv);CHKERRQ(ierr);
-  if (R) { ierr = MatDenseRestoreArray(R,&r);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "BVMatMult_Svec"
 PetscErrorCode BVMatMult_Svec(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
@@ -298,8 +250,6 @@ PetscErrorCode BVMatMult_Svec(BV V,Mat A,BV W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCopy_Svec"
 PetscErrorCode BVCopy_Svec(BV V,BV W)
 {
   PetscErrorCode ierr;
@@ -317,8 +267,6 @@ PetscErrorCode BVCopy_Svec(BV V,BV W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVResize_Svec"
 PetscErrorCode BVResize_Svec(BV bv,PetscInt m,PetscBool copy)
 {
   PetscErrorCode ierr;
@@ -351,8 +299,6 @@ PetscErrorCode BVResize_Svec(BV bv,PetscInt m,PetscBool copy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetColumn_Svec"
 PetscErrorCode BVGetColumn_Svec(BV bv,PetscInt j,Vec *v)
 {
   PetscErrorCode ierr;
@@ -367,8 +313,6 @@ PetscErrorCode BVGetColumn_Svec(BV bv,PetscInt j,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreColumn_Svec"
 PetscErrorCode BVRestoreColumn_Svec(BV bv,PetscInt j,Vec *v)
 {
   PetscErrorCode ierr;
@@ -382,8 +326,6 @@ PetscErrorCode BVRestoreColumn_Svec(BV bv,PetscInt j,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetArray_Svec"
 PetscErrorCode BVGetArray_Svec(BV bv,PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -394,8 +336,6 @@ PetscErrorCode BVGetArray_Svec(BV bv,PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreArray_Svec"
 PetscErrorCode BVRestoreArray_Svec(BV bv,PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -406,8 +346,6 @@ PetscErrorCode BVRestoreArray_Svec(BV bv,PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetArrayRead_Svec"
 PetscErrorCode BVGetArrayRead_Svec(BV bv,const PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -418,8 +356,6 @@ PetscErrorCode BVGetArrayRead_Svec(BV bv,const PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreArrayRead_Svec"
 PetscErrorCode BVRestoreArrayRead_Svec(BV bv,const PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -430,8 +366,6 @@ PetscErrorCode BVRestoreArrayRead_Svec(BV bv,const PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVView_Svec"
 PetscErrorCode BVView_Svec(BV bv,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -457,8 +391,6 @@ PetscErrorCode BVView_Svec(BV bv,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDestroy_Svec"
 PetscErrorCode BVDestroy_Svec(BV bv)
 {
   PetscErrorCode ierr;
@@ -469,35 +401,38 @@ PetscErrorCode BVDestroy_Svec(BV bv)
   ierr = VecDestroy(&bv->cv[0]);CHKERRQ(ierr);
   ierr = VecDestroy(&bv->cv[1]);CHKERRQ(ierr);
   ierr = PetscFree(bv->data);CHKERRQ(ierr);
+  bv->cuda = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCreate_Svec"
 PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
 {
   PetscErrorCode ierr;
   BV_SVEC        *ctx;
-  PetscInt       nloc,bs;
+  PetscInt       nloc,N,bs,tlocal;
   PetscBool      seq;
+  PetscScalar    *aa,*vv;
   char           str[50];
 
   PetscFunctionBegin;
   ierr = PetscNewLog(bv,&ctx);CHKERRQ(ierr);
   bv->data = (void*)ctx;
 
-  ierr = PetscObjectTypeCompare((PetscObject)bv->t,VECMPI,&ctx->mpi);CHKERRQ(ierr);
-  if (!ctx->mpi) {
-    ierr = PetscObjectTypeCompare((PetscObject)bv->t,VECSEQ,&seq);CHKERRQ(ierr);
-    if (!seq) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot create a BVSVEC from a non-standard template vector");
-  }
+  ierr = PetscObjectTypeCompareAny((PetscObject)bv->t,&bv->cuda,VECSEQCUDA,VECMPICUDA,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)bv->t,&ctx->mpi,VECMPI,VECMPICUDA,"");CHKERRQ(ierr);
+
+  ierr = PetscObjectTypeCompare((PetscObject)bv->t,VECSEQ,&seq);CHKERRQ(ierr);
+  if (!seq && !ctx->mpi && !bv->cuda) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"BVSVEC does not support the type of the provided template vector");
 
   ierr = VecGetLocalSize(bv->t,&nloc);CHKERRQ(ierr);
+  ierr = VecGetSize(bv->t,&N);CHKERRQ(ierr);
   ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
+  tlocal = bv->m*nloc;
+  if (tlocal<0) SETERRQ2(PetscObjectComm((PetscObject)bv),1,"The product %D times %D overflows the size of PetscInt; consider reducing the number of columns, or use BVVECS instead",bv->m,nloc);
 
   ierr = VecCreate(PetscObjectComm((PetscObject)bv->t),&ctx->v);CHKERRQ(ierr);
   ierr = VecSetType(ctx->v,((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
-  ierr = VecSetSizes(ctx->v,bv->m*nloc,PETSC_DECIDE);CHKERRQ(ierr);
+  ierr = VecSetSizes(ctx->v,tlocal,bv->m*N);CHKERRQ(ierr);
   ierr = VecSetBlockSize(ctx->v,bs);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)ctx->v);CHKERRQ(ierr);
   if (((PetscObject)bv)->name) {
@@ -505,30 +440,52 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
     ierr = PetscObjectSetName((PetscObject)ctx->v,str);CHKERRQ(ierr);
   }
 
-  if (ctx->mpi) {
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,NULL,&bv->cv[0]);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,NULL,&bv->cv[1]);CHKERRQ(ierr);
-  } else {
-    ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,NULL,&bv->cv[0]);CHKERRQ(ierr);
-    ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,NULL,&bv->cv[1]);CHKERRQ(ierr);
+  if (bv->Acreate) {
+    ierr = MatDenseGetArray(bv->Acreate,&aa);CHKERRQ(ierr);
+    ierr = VecGetArray(ctx->v,&vv);CHKERRQ(ierr);
+    ierr = PetscMemcpy(vv,aa,tlocal*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = VecRestoreArray(ctx->v,&vv);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArray(bv->Acreate,&aa);CHKERRQ(ierr);
+    ierr = MatDestroy(&bv->Acreate);CHKERRQ(ierr);
   }
 
-  bv->ops->mult             = BVMult_Svec;
-  bv->ops->multvec          = BVMultVec_Svec;
-  bv->ops->multinplace      = BVMultInPlace_Svec;
-  bv->ops->multinplacetrans = BVMultInPlaceTranspose_Svec;
-  bv->ops->dot              = BVDot_Svec;
-  bv->ops->dotvec           = BVDotVec_Svec;
-  bv->ops->dotvec_local     = BVDotVec_Local_Svec;
-  bv->ops->scale            = BVScale_Svec;
+  ierr = VecDuplicateEmpty(bv->t,&bv->cv[0]);CHKERRQ(ierr);
+  ierr = VecDuplicateEmpty(bv->t,&bv->cv[1]);CHKERRQ(ierr);
+  ierr = VecSetType(bv->cv[0],((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
+  ierr = VecSetType(bv->cv[1],((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
+  if (bv->cuda) {
+#if defined(PETSC_HAVE_VECCUDA)
+    bv->ops->mult             = BVMult_Svec_CUDA;
+    bv->ops->multvec          = BVMultVec_Svec_CUDA;
+    bv->ops->multinplace      = BVMultInPlace_Svec_CUDA;
+    bv->ops->multinplacetrans = BVMultInPlaceTranspose_Svec_CUDA;
+    bv->ops->dot              = BVDot_Svec_CUDA;
+    bv->ops->dotvec           = BVDotVec_Svec_CUDA;
+    bv->ops->dotvec_local     = BVDotVec_Local_Svec_CUDA;
+    bv->ops->scale            = BVScale_Svec_CUDA;
+    bv->ops->matmult          = BVMatMult_Svec_CUDA;
+    bv->ops->copy             = BVCopy_Svec_CUDA;
+    bv->ops->resize           = BVResize_Svec_CUDA;
+    bv->ops->getcolumn        = BVGetColumn_Svec_CUDA;
+    bv->ops->restorecolumn    = BVRestoreColumn_Svec_CUDA;
+#endif
+  } else {
+    bv->ops->mult             = BVMult_Svec;
+    bv->ops->multvec          = BVMultVec_Svec;
+    bv->ops->multinplace      = BVMultInPlace_Svec;
+    bv->ops->multinplacetrans = BVMultInPlaceTranspose_Svec;
+    bv->ops->dot              = BVDot_Svec;
+    bv->ops->dotvec           = BVDotVec_Svec;
+    bv->ops->dotvec_local     = BVDotVec_Local_Svec;
+    bv->ops->scale            = BVScale_Svec;
+    bv->ops->matmult          = BVMatMult_Svec;
+    bv->ops->copy             = BVCopy_Svec;
+    bv->ops->resize           = BVResize_Svec;
+    bv->ops->getcolumn        = BVGetColumn_Svec;
+    bv->ops->restorecolumn    = BVRestoreColumn_Svec;
+  }
   bv->ops->norm             = BVNorm_Svec;
   bv->ops->norm_local       = BVNorm_Local_Svec;
-  /*bv->ops->orthogonalize    = BVOrthogonalize_Svec;*/
-  bv->ops->matmult          = BVMatMult_Svec;
-  bv->ops->copy             = BVCopy_Svec;
-  bv->ops->resize           = BVResize_Svec;
-  bv->ops->getcolumn        = BVGetColumn_Svec;
-  bv->ops->restorecolumn    = BVRestoreColumn_Svec;
   bv->ops->getarray         = BVGetArray_Svec;
   bv->ops->restorearray     = BVRestoreArray_Svec;
   bv->ops->getarrayread     = BVGetArrayRead_Svec;
