@@ -1,25 +1,15 @@
 /*
-   BV implemented as an array of independent Vecs
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   BV implemented as an array of independent Vecs
+*/
 
 #include <slepc/private/bvimpl.h>
 
@@ -30,8 +20,6 @@ typedef struct {
        1: version that allocates (e-s) work vectors in every call (default in GPU) */
 } BV_VECS;
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMult_Vecs"
 PetscErrorCode BVMult_Vecs(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 {
   PetscErrorCode ierr;
@@ -64,13 +52,11 @@ PetscErrorCode BVMult_Vecs(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultVec_Vecs"
 PetscErrorCode BVMultVec_Vecs(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScalar *q)
 {
   PetscErrorCode ierr;
   BV_VECS        *x = (BV_VECS*)X->data;
-  PetscScalar    *s=NULL;
+  PetscScalar    *s=NULL,*qq=q;
   PetscInt       i;
 
   PetscFunctionBegin;
@@ -78,16 +64,16 @@ PetscErrorCode BVMultVec_Vecs(BV X,PetscScalar alpha,PetscScalar beta,Vec y,Pets
     ierr = BVAllocateWork_Private(X,X->k-X->l);CHKERRQ(ierr);
     s = X->work;
   }
+  if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = VecScale(y,beta);CHKERRQ(ierr);
   if (alpha!=1.0) {
-    for (i=0;i<X->k-X->l;i++) s[i] = alpha*q[i];
-  } else s = q;
+    for (i=0;i<X->k-X->l;i++) s[i] = alpha*qq[i];
+  } else s = qq;
   ierr = VecMAXPY(y,X->k-X->l,s,x->V+X->nc+X->l);CHKERRQ(ierr);
+  if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlace_Vecs_ME"
 /*
    BVMultInPlace_Vecs_ME - V(:,s:e-1) = V*Q(:,s:e-1) for regular vectors.
 
@@ -122,8 +108,6 @@ PetscErrorCode BVMultInPlace_Vecs_ME(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlace_Vecs_Alloc"
 /*
    BVMultInPlace_Vecs_Alloc - V(:,s:e-1) = V*Q(:,s:e-1) for regular vectors.
 
@@ -152,8 +136,6 @@ PetscErrorCode BVMultInPlace_Vecs_Alloc(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlaceTranspose_Vecs"
 /*
    BVMultInPlaceTranspose_Vecs - V(:,s:e-1) = V*Q'(:,s:e-1) for regular vectors.
 */
@@ -182,8 +164,6 @@ PetscErrorCode BVMultInPlaceTranspose_Vecs(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDot_Vecs"
 PetscErrorCode BVDot_Vecs(BV X,BV Y,Mat M)
 {
   PetscErrorCode ierr;
@@ -201,25 +181,24 @@ PetscErrorCode BVDot_Vecs(BV X,BV Y,Mat M)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_Vecs"
-PetscErrorCode BVDotVec_Vecs(BV X,Vec y,PetscScalar *m)
+PetscErrorCode BVDotVec_Vecs(BV X,Vec y,PetscScalar *q)
 {
   PetscErrorCode ierr;
   BV_VECS        *x = (BV_VECS*)X->data;
   Vec            z = y;
+  PetscScalar    *qq=q;
 
   PetscFunctionBegin;
   if (X->matrix) {
     ierr = BV_IPMatMult(X,y);CHKERRQ(ierr);
     z = X->Bx;
   }
-  ierr = VecMDot(z,X->k-X->l,x->V+X->nc+X->l,m);CHKERRQ(ierr);
+  if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
+  ierr = VecMDot(z,X->k-X->l,x->V+X->nc+X->l,qq);CHKERRQ(ierr);
+  if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_Begin_Vecs"
 PetscErrorCode BVDotVec_Begin_Vecs(BV X,Vec y,PetscScalar *m)
 {
   PetscErrorCode ierr;
@@ -235,8 +214,6 @@ PetscErrorCode BVDotVec_Begin_Vecs(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec_End_Vecs"
 PetscErrorCode BVDotVec_End_Vecs(BV X,Vec y,PetscScalar *m)
 {
   PetscErrorCode ierr;
@@ -247,8 +224,6 @@ PetscErrorCode BVDotVec_End_Vecs(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVScale_Vecs"
 PetscErrorCode BVScale_Vecs(BV bv,PetscInt j,PetscScalar alpha)
 {
   PetscErrorCode ierr;
@@ -266,8 +241,6 @@ PetscErrorCode BVScale_Vecs(BV bv,PetscInt j,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Vecs"
 PetscErrorCode BVNorm_Vecs(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -277,64 +250,46 @@ PetscErrorCode BVNorm_Vecs(BV bv,PetscInt j,NormType type,PetscReal *val)
 
   PetscFunctionBegin;
   if (j<0) {
-    switch (type) {
-    case NORM_FROBENIUS:
+    if (type==NORM_FROBENIUS) {
       *val = 0.0;
       for (i=bv->l;i<bv->k;i++) {
         ierr = VecNorm(ctx->V[bv->nc+i],NORM_2,&nrm);CHKERRQ(ierr);
         *val += nrm*nrm;
       }
       *val = PetscSqrtReal(*val);
-      break;
-    default:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Requested norm not implemented in BVVECS");
-    }
+    } else SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not implemented in BVVECS");
   } else {
     ierr = VecNorm(ctx->V[bv->nc+j],type,val);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Begin_Vecs"
 PetscErrorCode BVNorm_Begin_Vecs(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
   BV_VECS        *ctx = (BV_VECS*)bv->data;
 
   PetscFunctionBegin;
-  if (j<0) {
-    switch (type) {
-    default:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Requested norm not implemented in BVVECS");
-    }
-  } else {
+  if (j<0) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not implemented in BVVECS");
+  else {
     ierr = VecNormBegin(ctx->V[bv->nc+j],type,val);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_End_Vecs"
 PetscErrorCode BVNorm_End_Vecs(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
   BV_VECS        *ctx = (BV_VECS*)bv->data;
 
   PetscFunctionBegin;
-  if (j<0) {
-    switch (type) {
-    default:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Requested norm not implemented in BVVECS");
-    }
-  } else {
+  if (j<0) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not implemented in BVVECS");
+  else {
     ierr = VecNormEnd(ctx->V[bv->nc+j],type,val);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatMult_Vecs"
 PetscErrorCode BVMatMult_Vecs(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
@@ -349,8 +304,6 @@ PetscErrorCode BVMatMult_Vecs(BV V,Mat A,BV W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCopy_Vecs"
 PetscErrorCode BVCopy_Vecs(BV V,BV W)
 {
   PetscErrorCode ierr;
@@ -364,8 +317,6 @@ PetscErrorCode BVCopy_Vecs(BV V,BV W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVResize_Vecs"
 PetscErrorCode BVResize_Vecs(BV bv,PetscInt m,PetscBool copy)
 {
   PetscErrorCode ierr;
@@ -393,8 +344,6 @@ PetscErrorCode BVResize_Vecs(BV bv,PetscInt m,PetscBool copy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetColumn_Vecs"
 PetscErrorCode BVGetColumn_Vecs(BV bv,PetscInt j,Vec *v)
 {
   BV_VECS  *ctx = (BV_VECS*)bv->data;
@@ -406,8 +355,6 @@ PetscErrorCode BVGetColumn_Vecs(BV bv,PetscInt j,Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetArray_Vecs"
 PetscErrorCode BVGetArray_Vecs(BV bv,PetscScalar **a)
 {
   PetscErrorCode    ierr;
@@ -425,8 +372,6 @@ PetscErrorCode BVGetArray_Vecs(BV bv,PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreArray_Vecs"
 PetscErrorCode BVRestoreArray_Vecs(BV bv,PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -444,8 +389,6 @@ PetscErrorCode BVRestoreArray_Vecs(BV bv,PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetArrayRead_Vecs"
 PetscErrorCode BVGetArrayRead_Vecs(BV bv,const PetscScalar **a)
 {
   PetscErrorCode    ierr;
@@ -463,8 +406,6 @@ PetscErrorCode BVGetArrayRead_Vecs(BV bv,const PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRestoreArrayRead_Vecs"
 PetscErrorCode BVRestoreArrayRead_Vecs(BV bv,const PetscScalar **a)
 {
   PetscErrorCode ierr;
@@ -474,8 +415,6 @@ PetscErrorCode BVRestoreArrayRead_Vecs(BV bv,const PetscScalar **a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVVecsSetVmip"
 /*
    Sets the value of vmip flag and resets ops->multinplace accordingly
  */
@@ -491,8 +430,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVVecsSetVmip(BV bv,PetscInt vmip)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVSetFromOptions_Vecs"
 PetscErrorCode BVSetFromOptions_Vecs(PetscOptionItems *PetscOptionsObject,BV bv)
 {
   PetscErrorCode ierr;
@@ -500,15 +437,15 @@ PetscErrorCode BVSetFromOptions_Vecs(PetscOptionItems *PetscOptionsObject,BV bv)
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"BV Vecs Options");CHKERRQ(ierr);
+
     ierr = PetscOptionsInt("-bv_vecs_vmip","Version of BVMultInPlace operation","",ctx->vmip,&ctx->vmip,NULL);CHKERRQ(ierr);
     if (ctx->vmip<0 || ctx->vmip>1) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_OUTOFRANGE,"Wrong version of BVMultInPlace");
     ierr = BVVecsSetVmip(bv,ctx->vmip);CHKERRQ(ierr);
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVView_Vecs"
 PetscErrorCode BVView_Vecs(BV bv,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -538,8 +475,6 @@ PetscErrorCode BVView_Vecs(BV bv,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDestroy_Vecs"
 PetscErrorCode BVDestroy_Vecs(BV bv)
 {
   PetscErrorCode ierr;
@@ -551,8 +486,6 @@ PetscErrorCode BVDestroy_Vecs(BV bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDuplicate_Vecs"
 PetscErrorCode BVDuplicate_Vecs(BV V,BV *W)
 {
   PetscErrorCode ierr;
@@ -563,14 +496,12 @@ PetscErrorCode BVDuplicate_Vecs(BV V,BV *W)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCreate_Vecs"
 PETSC_EXTERN PetscErrorCode BVCreate_Vecs(BV bv)
 {
   PetscErrorCode ierr;
   BV_VECS        *ctx;
   PetscInt       j;
-  PetscBool      iscusp;
+  PetscBool      isgpu;
   char           str[50];
 
   PetscFunctionBegin;
@@ -586,9 +517,16 @@ PETSC_EXTERN PetscErrorCode BVCreate_Vecs(BV bv)
     }
   }
 
+  if (bv->Acreate) {
+    for (j=0;j<bv->m;j++) {
+      ierr = MatGetColumnVector(bv->Acreate,ctx->V[j],j);CHKERRQ(ierr);
+    }
+    ierr = MatDestroy(&bv->Acreate);CHKERRQ(ierr);
+  }
+
   /* Default version of BVMultInPlace */
-  ierr = PetscObjectTypeCompareAny((PetscObject)bv->t,&iscusp,VECSEQCUSP,VECMPICUSP,"");CHKERRQ(ierr);
-  ctx->vmip = iscusp? 1: 0;
+  ierr = PetscObjectTypeCompareAny((PetscObject)bv->t,&isgpu,VECSEQCUDA,VECMPICUDA,VECSEQCUSP,VECMPICUSP,"");CHKERRQ(ierr);
+  ctx->vmip = isgpu? 1: 0;
 
   /* Deferred call to setfromoptions */
   if (bv->defersfo) {
