@@ -1,30 +1,18 @@
 /*
-   BV operations involving global communication.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   BV operations involving global communication
+*/
 
 #include <slepc/private/bvimpl.h>      /*I "slepcbv.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDot_Private"
 /*
   BVDot for the particular case of non-standard inner product with
   matrix B, which is assumed to be symmetric (or complex Hermitian)
@@ -36,6 +24,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVDot_Private(BV X,BV Y,Mat M)
   PetscInt       i,j,jend,m;
   PetscScalar    *marray;
   PetscBool      symm=PETSC_FALSE;
+  Mat            B;
   Vec            z;
 
   PetscFunctionBegin;
@@ -43,24 +32,25 @@ PETSC_STATIC_INLINE PetscErrorCode BVDot_Private(BV X,BV Y,Mat M)
   ierr = MatDenseGetArray(M,&marray);CHKERRQ(ierr);
   ierr = PetscObjectGetId((PetscObject)X,&idx);CHKERRQ(ierr);
   ierr = PetscObjectGetId((PetscObject)Y,&idy);CHKERRQ(ierr);
+  B = Y->matrix;
+  Y->matrix = NULL;
   if (idx==idy) symm=PETSC_TRUE;  /* M=X'BX is symmetric */
   jend = X->k;
   for (j=X->l;j<jend;j++) {
     if (symm) Y->k = j+1;
-    ierr = BVGetColumn(X,j,&z);CHKERRQ(ierr);
+    ierr = BVGetColumn(X->cached,j,&z);CHKERRQ(ierr);
     ierr = (*Y->ops->dotvec)(Y,z,marray+j*m+Y->l);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(X,j,&z);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(X->cached,j,&z);CHKERRQ(ierr);
     if (symm) {
       for (i=X->l;i<j;i++)
         marray[j+i*m] = PetscConj(marray[i+j*m]);
     }
   }
   ierr = MatDenseRestoreArray(M,&marray);CHKERRQ(ierr);
+  Y->matrix = B;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDot"
 /*@
    BVDot - Computes the 'block-dot' product of two basis vectors objects.
 
@@ -98,7 +88,6 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
   PetscErrorCode ierr;
   PetscBool      match;
   PetscInt       m,n;
-  Mat            B;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(X,BV_CLASSID,1);
@@ -116,22 +105,19 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
   ierr = MatGetSize(M,&m,&n);CHKERRQ(ierr);
   if (m<Y->k) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Mat argument has %D rows, should have at least %D",m,Y->k);
   if (n<X->k) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Mat argument has %D columns, should have at least %D",n,X->k);
-  if (X->n!=Y->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, Y %D",X->n,Y->n);
-  if (X->matrix!=Y->matrix) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"X and Y must have the same inner product matrix");
+  if (X->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, Y %D",X->n,Y->n);
+  if (X->matrix!=Y->matrix) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"X and Y must have the same inner product matrix");
   if (X->l==X->k || Y->l==Y->k) PetscFunctionReturn(0);
 
   ierr = PetscLogEventBegin(BV_Dot,X,Y,0,0);CHKERRQ(ierr);
   if (X->matrix) { /* non-standard inner product */
+    /* compute BX first */
+    ierr = BV_IPMatMultBV(X);CHKERRQ(ierr);
     if (X->vmm==BV_MATMULT_VECS) {
       /* perform computation column by column */
       ierr = BVDot_Private(X,Y,M);CHKERRQ(ierr);
     } else {
-      /* compute BX first */
-      ierr = BV_IPMatMultBV(X);CHKERRQ(ierr);
-      B = X->matrix;
-      X->matrix = NULL;
       ierr = (*X->ops->dot)(X->cached,Y,M);CHKERRQ(ierr);
-      X->matrix = B;
     }
   } else {
     ierr = (*X->ops->dot)(X,Y,M);CHKERRQ(ierr);
@@ -140,8 +126,6 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVec"
 /*@
    BVDotVec - Computes multiple dot products of a vector against all the
    column vectors of a BV.
@@ -171,7 +155,7 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
 
 .seealso: BVDot(), BVDotColumn(), BVSetActiveColumns(), BVSetMatrix()
 @*/
-PetscErrorCode BVDotVec(BV X,Vec y,PetscScalar *m)
+PetscErrorCode BVDotVec(BV X,Vec y,PetscScalar m[])
 {
   PetscErrorCode ierr;
   PetscInt       n;
@@ -185,7 +169,7 @@ PetscErrorCode BVDotVec(BV X,Vec y,PetscScalar *m)
   PetscCheckSameTypeAndComm(X,1,y,2);
 
   ierr = VecGetLocalSize(y,&n);CHKERRQ(ierr);
-  if (X->n!=n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, y %D",X->n,n);
+  if (X->n!=n) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, y %D",X->n,n);
 
   ierr = PetscLogEventBegin(BV_DotVec,X,y,0,0);CHKERRQ(ierr);
   ierr = (*X->ops->dotvec)(X,y,m);CHKERRQ(ierr);
@@ -193,8 +177,6 @@ PetscErrorCode BVDotVec(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVecBegin"
 /*@
    BVDotVecBegin - Starts a split phase dot product computation.
 
@@ -226,7 +208,7 @@ PetscErrorCode BVDotVecBegin(BV X,Vec y,PetscScalar *m)
   PetscCheckSameTypeAndComm(X,1,y,2);
 
   ierr = VecGetLocalSize(y,&n);CHKERRQ(ierr);
-  if (X->n!=n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, y %D",X->n,n);
+  if (X->n!=n) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, y %D",X->n,n);
 
   if (X->ops->dotvec_begin) {
     ierr = (*X->ops->dotvec_begin)(X,y,m);CHKERRQ(ierr);
@@ -234,7 +216,7 @@ PetscErrorCode BVDotVecBegin(BV X,Vec y,PetscScalar *m)
     nv = X->k-X->l;
     ierr = PetscObjectGetComm((PetscObject)X,&comm);CHKERRQ(ierr);
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
-    if (sr->state != STATE_BEGIN) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
+    if (sr->state != STATE_BEGIN) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
     for (i=0;i<nv;i++) {
       if (sr->numopsbegin+i >= sr->maxops) {
         ierr = PetscSplitReductionExtend(sr);CHKERRQ(ierr);
@@ -250,8 +232,6 @@ PetscErrorCode BVDotVecBegin(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotVecEnd"
 /*@
    BVDotVecEnd - Ends a split phase dot product computation.
 
@@ -287,9 +267,9 @@ PetscErrorCode BVDotVecEnd(BV X,Vec y,PetscScalar *m)
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
     ierr = PetscSplitReductionEnd(sr);CHKERRQ(ierr);
 
-    if (sr->numopsend >= sr->numopsbegin) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times than VecxxxBegin()");
-    if ((void*)X != sr->invecs[sr->numopsend]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called BVxxxEnd() in a different order or with a different BV than BVxxxBegin()");
-    if (sr->reducetype[sr->numopsend] != REDUCE_SUM) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
+    if (sr->numopsend >= sr->numopsbegin) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times than VecxxxBegin()");
+    if ((void*)X != sr->invecs[sr->numopsend]) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called BVxxxEnd() in a different order or with a different BV than BVxxxBegin()");
+    if (sr->reducetype[sr->numopsend] != REDUCE_SUM) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
     for (i=0;i<nv;i++) m[i] = sr->gvalues[sr->numopsend++];
 
     /* Finished getting all the results so reset to no outstanding requests */
@@ -302,8 +282,6 @@ PetscErrorCode BVDotVecEnd(BV X,Vec y,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotColumn"
 /*@
    BVDotColumn - Computes multiple dot products of a column against all the
    previous columns of a BV.
@@ -315,21 +293,25 @@ PetscErrorCode BVDotVecEnd(BV X,Vec y,PetscScalar *m)
 -  j - the column index
 
    Output Parameter:
-.  m - an array where the result must be placed
+.  q - an array where the result must be placed
 
    Notes:
    This operation is equivalent to BVDotVec() but it uses column j of X
    rather than taking a Vec as an argument. The number of active columns of
    X is set to j before the computation, and restored afterwards.
    If X has leading columns specified, then these columns do not participate
-   in the computation. Therefore, the length of array m must be equal to j
+   in the computation. Therefore, the length of array q must be equal to j
    minus the number of leading columns.
 
+   Developer Notes:
+   If q is NULL, then the result is written in position nc+l of the internal
+   buffer vector, see BVGetBufferVec().
+
    Level: advanced
 
 .seealso: BVDot(), BVDotVec(), BVSetActiveColumns(), BVSetMatrix()
 @*/
-PetscErrorCode BVDotColumn(BV X,PetscInt j,PetscScalar *m)
+PetscErrorCode BVDotColumn(BV X,PetscInt j,PetscScalar *q)
 {
   PetscErrorCode ierr;
   PetscInt       ksave;
@@ -347,16 +329,15 @@ PetscErrorCode BVDotColumn(BV X,PetscInt j,PetscScalar *m)
   ierr = PetscLogEventBegin(BV_DotVec,X,0,0,0);CHKERRQ(ierr);
   ksave = X->k;
   X->k = j;
+  if (!q && !X->buffer) { ierr = BVGetBufferVec(X,&X->buffer);CHKERRQ(ierr); }
   ierr = BVGetColumn(X,j,&y);CHKERRQ(ierr);
-  ierr = (*X->ops->dotvec)(X,y,m);CHKERRQ(ierr);
+  ierr = (*X->ops->dotvec)(X,y,q);CHKERRQ(ierr);
   ierr = BVRestoreColumn(X,j,&y);CHKERRQ(ierr);
   X->k = ksave;
   ierr = PetscLogEventEnd(BV_DotVec,X,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotColumnBegin"
 /*@
    BVDotColumnBegin - Starts a split phase dot product computation.
 
@@ -398,7 +379,7 @@ PetscErrorCode BVDotColumnBegin(BV X,PetscInt j,PetscScalar *m)
     nv = X->k-X->l;
     ierr = PetscObjectGetComm((PetscObject)X,&comm);CHKERRQ(ierr);
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
-    if (sr->state != STATE_BEGIN) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
+    if (sr->state != STATE_BEGIN) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
     for (i=0;i<nv;i++) {
       if (sr->numopsbegin+i >= sr->maxops) {
         ierr = PetscSplitReductionExtend(sr);CHKERRQ(ierr);
@@ -416,8 +397,6 @@ PetscErrorCode BVDotColumnBegin(BV X,PetscInt j,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDotColumnEnd"
 /*@
    BVDotColumnEnd - Ends a split phase dot product computation.
 
@@ -462,9 +441,9 @@ PetscErrorCode BVDotColumnEnd(BV X,PetscInt j,PetscScalar *m)
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
     ierr = PetscSplitReductionEnd(sr);CHKERRQ(ierr);
 
-    if (sr->numopsend >= sr->numopsbegin) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times than VecxxxBegin()");
-    if ((void*)X != sr->invecs[sr->numopsend]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called BVxxxEnd() in a different order or with a different BV than BVxxxBegin()");
-    if (sr->reducetype[sr->numopsend] != REDUCE_SUM) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
+    if (sr->numopsend >= sr->numopsbegin) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times than VecxxxBegin()");
+    if ((void*)X != sr->invecs[sr->numopsend]) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called BVxxxEnd() in a different order or with a different BV than BVxxxBegin()");
+    if (sr->reducetype[sr->numopsend] != REDUCE_SUM) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
     for (i=0;i<nv;i++) m[i] = sr->gvalues[sr->numopsend++];
 
     /* Finished getting all the results so reset to no outstanding requests */
@@ -478,8 +457,6 @@ PetscErrorCode BVDotColumnEnd(BV X,PetscInt j,PetscScalar *m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Private"
 PETSC_STATIC_INLINE PetscErrorCode BVNorm_Private(BV bv,Vec z,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -492,8 +469,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVNorm_Private(BV bv,Vec z,NormType type,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_Begin_Private"
 PETSC_STATIC_INLINE PetscErrorCode BVNorm_Begin_Private(BV bv,Vec z,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -505,8 +480,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVNorm_Begin_Private(BV bv,Vec z,NormType typ
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm_End_Private"
 PETSC_STATIC_INLINE PetscErrorCode BVNorm_End_Private(BV bv,Vec z,NormType type,PetscReal *val)
 {
   PetscErrorCode ierr;
@@ -518,8 +491,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVNorm_End_Private(BV bv,Vec z,NormType type,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNorm"
 /*@
    BVNorm - Computes the matrix norm of the BV.
 
@@ -563,8 +534,6 @@ PetscErrorCode BVNorm(BV bv,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNormVec"
 /*@
    BVNormVec - Computes the norm of a given vector.
 
@@ -603,12 +572,12 @@ PetscErrorCode BVNormVec(BV bv,Vec v,NormType type,PetscReal *val)
   PetscValidType(v,2);
   PetscCheckSameComm(bv,1,v,2);
 
-  if (type==NORM_1_AND_2) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
+  if (type==NORM_1_AND_2 && !bv->matrix) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
 
   ierr = PetscLogEventBegin(BV_NormVec,bv,0,0,0);CHKERRQ(ierr);
   if (bv->matrix) { /* non-standard inner product */
     ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
-    if (bv->n!=n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension bv %D, v %D",bv->n,n);
+    if (bv->n!=n) SETERRQ2(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension bv %D, v %D",bv->n,n);
     ierr = BVNorm_Private(bv,v,type,val);CHKERRQ(ierr);
   } else {
     ierr = VecNorm(v,type,val);CHKERRQ(ierr);
@@ -617,8 +586,6 @@ PetscErrorCode BVNormVec(BV bv,Vec v,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNormVecBegin"
 /*@
    BVNormVecBegin - Starts a split phase norm computation.
 
@@ -650,12 +617,12 @@ PetscErrorCode BVNormVecBegin(BV bv,Vec v,NormType type,PetscReal *val)
   PetscValidType(v,2);
   PetscCheckSameTypeAndComm(bv,1,v,2);
 
-  if (type==NORM_1_AND_2) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
+  if (type==NORM_1_AND_2 && !bv->matrix) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
 
   ierr = PetscLogEventBegin(BV_NormVec,bv,0,0,0);CHKERRQ(ierr);
   if (bv->matrix) { /* non-standard inner product */
     ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
-    if (bv->n!=n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension bv %D, v %D",bv->n,n);
+    if (bv->n!=n) SETERRQ2(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension bv %D, v %D",bv->n,n);
     ierr = BVNorm_Begin_Private(bv,v,type,val);CHKERRQ(ierr);
   } else {
     ierr = VecNormBegin(v,type,val);CHKERRQ(ierr);
@@ -664,8 +631,6 @@ PetscErrorCode BVNormVecBegin(BV bv,Vec v,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNormVecEnd"
 /*@
    BVNormVecEnd - Ends a split phase norm computation.
 
@@ -703,8 +668,6 @@ PetscErrorCode BVNormVecEnd(BV bv,Vec v,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNormColumn"
 /*@
    BVNormColumn - Computes the vector norm of a selected column.
 
@@ -721,7 +684,7 @@ PetscErrorCode BVNormVecEnd(BV bv,Vec v,NormType type,PetscReal *val)
    Notes:
    The norm of V[j] is computed (NORM_1, NORM_2, or NORM_INFINITY).
    If a non-standard inner product has been specified with BVSetMatrix(),
-   then the returned value is sqrt(V[j]'*B*V[j]), 
+   then the returned value is sqrt(V[j]'*B*V[j]),
    where B is the inner product matrix (argument 'type' is ignored).
 
    Level: intermediate
@@ -742,7 +705,7 @@ PetscErrorCode BVNormColumn(BV bv,PetscInt j,NormType type,PetscReal *val)
   BVCheckSizes(bv,1);
 
   if (j<0 || j>=bv->m) SETERRQ2(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_OUTOFRANGE,"Argument j has wrong value %D, the number of columns is %D",j,bv->m);
-  if (type==NORM_1_AND_2) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
+  if (type==NORM_1_AND_2 && !bv->matrix) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
 
   ierr = PetscLogEventBegin(BV_NormVec,bv,0,0,0);CHKERRQ(ierr);
   if (bv->matrix) { /* non-standard inner product */
@@ -756,8 +719,6 @@ PetscErrorCode BVNormColumn(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNormColumnBegin"
 /*@
    BVNormColumnBegin - Starts a split phase norm computation.
 
@@ -791,7 +752,7 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
   BVCheckSizes(bv,1);
 
   if (j<0 || j>=bv->m) SETERRQ2(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_OUTOFRANGE,"Argument j has wrong value %D, the number of columns is %D",j,bv->m);
-  if (type==NORM_1_AND_2) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
+  if (type==NORM_1_AND_2 && !bv->matrix) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_SUP,"Requested norm not available");
 
   ierr = PetscLogEventBegin(BV_NormVec,bv,0,0,0);CHKERRQ(ierr);
   ierr = BVGetColumn(bv,j,&z);CHKERRQ(ierr);
@@ -802,7 +763,7 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
   } else {
     ierr = PetscObjectGetComm((PetscObject)z,&comm);CHKERRQ(ierr);
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
-    if (sr->state != STATE_BEGIN) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
+    if (sr->state != STATE_BEGIN) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
     if (sr->numopsbegin >= sr->maxops) {
       ierr = PetscSplitReductionExtend(sr);CHKERRQ(ierr);
     }
@@ -818,8 +779,6 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVNormColumnEnd"
 /*@
    BVNormColumnEnd - Ends a split phase norm computation.
 
@@ -863,9 +822,9 @@ PetscErrorCode BVNormColumnEnd(BV bv,PetscInt j,NormType type,PetscReal *val)
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
     ierr = PetscSplitReductionEnd(sr);CHKERRQ(ierr);
 
-    if (sr->numopsend >= sr->numopsbegin) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times then VecxxxBegin()");
-    if ((void*)bv != sr->invecs[sr->numopsend]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() in a different order or with a different vector than VecxxxBegin()");
-    if (sr->reducetype[sr->numopsend] != REDUCE_MAX && type == NORM_MAX) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Called BVNormEnd(,NORM_MAX,) on a reduction started with VecDotBegin() or NORM_1 or NORM_2");
+    if (sr->numopsend >= sr->numopsbegin) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times then VecxxxBegin()");
+    if ((void*)bv != sr->invecs[sr->numopsend]) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() in a different order or with a different vector than VecxxxBegin()");
+    if (sr->reducetype[sr->numopsend] != REDUCE_MAX && type == NORM_MAX) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called BVNormEnd(,NORM_MAX,) on a reduction started with VecDotBegin() or NORM_1 or NORM_2");
     *val = PetscRealPart(sr->gvalues[sr->numopsend++]);
     if (type == NORM_2) *val = PetscSqrtReal(*val);
     if (sr->numopsend == sr->numopsbegin) {
@@ -878,8 +837,6 @@ PetscErrorCode BVNormColumnEnd(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatProject_Vec"
 /*
   Compute Y^H*A*X: right part column by column (with MatMult) and bottom
   part row by row (with MatMultHermitianTranspose); result placed in marray[*,ldm]
@@ -923,8 +880,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Vec(BV X,Mat A,BV Y,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatProject_MatMult"
 /*
   Compute Y^H*A*X= [   --   | Y0'*W1 ]
                    [ Y1'*W0 | Y1'*W1 ]
@@ -980,8 +935,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult(BV X,Mat A,BV Y,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatProject_MatMult_2"
 /*
   Compute Y^H*A*X= [   --   | Y0'*W1 ]
                    [ Y1'*W0 | Y1'*W1 ]
@@ -1046,8 +999,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult_2(BV X,Mat A,BV Y,PetscS
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatProject_Dot"
 /*
   Compute Y^H*X = [   --   | Y0'*X1 ]     (X contains A*X):
                   [ Y1'*X0 | Y1'*X1 ]
@@ -1097,8 +1048,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Dot(BV X,BV Y,PetscScalar *marra
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatProject"
 /*@
    BVMatProject - Computes the projection of a matrix onto a subspace.
 
@@ -1172,7 +1121,7 @@ PetscErrorCode BVMatProject(BV X,Mat A,BV Y,Mat M)
   ierr = MatGetSize(M,&m,&n);CHKERRQ(ierr);
   if (m<Y->k) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Matrix M has %D rows, should have at least %D",m,Y->k);
   if (n<X->k) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Matrix M has %D columns, should have at least %D",n,X->k);
-  if (X->n!=Y->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, Y %D",X->n,Y->n);
+  if (X->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, Y %D",X->n,Y->n);
 
   ierr = PetscLogEventBegin(BV_MatProject,X,A,Y,0);CHKERRQ(ierr);
   /* temporarily set standard inner product */
@@ -1182,16 +1131,14 @@ PetscErrorCode BVMatProject(BV X,Mat A,BV Y,Mat M)
 
   ierr = PetscObjectGetId((PetscObject)X,&idx);CHKERRQ(ierr);
   ierr = PetscObjectGetId((PetscObject)Y,&idy);CHKERRQ(ierr);
-  if (!A && idx==idy) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Cannot set X=Y if A=NULL");
-
-  ierr = MatDenseGetArray(M,&marray);CHKERRQ(ierr);
-
   if (A && idx==idy) { /* check symmetry of M=X'AX */
     ierr = MatIsHermitianKnown(A,&set,&flg);CHKERRQ(ierr);
     symm = set? flg: PETSC_FALSE;
   }
 
-  if (A) { 
+  ierr = MatDenseGetArray(M,&marray);CHKERRQ(ierr);
+
+  if (A) {
     if (X->vmm==BV_MATMULT_VECS) {
       /* perform computation column by column */
       ierr = BVMatProject_Vec(X,A,Y,marray,m,symm);CHKERRQ(ierr);
