@@ -1,30 +1,19 @@
 /*
-   BV operations, except those involving global communication.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   BV operations, except those involving global communication
+*/
 
 #include <slepc/private/bvimpl.h>      /*I "slepcbv.h" I*/
+#include <slepcds.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMult"
 /*@
    BVMult - Computes Y = beta*Y + alpha*X*Q.
 
@@ -84,7 +73,7 @@ PetscErrorCode BVMult(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
     if (m<X->k) SETERRQ2(PetscObjectComm((PetscObject)Y),PETSC_ERR_ARG_SIZ,"Mat argument has %D rows, should have at least %D",m,X->k);
     if (n<Y->k) SETERRQ2(PetscObjectComm((PetscObject)Y),PETSC_ERR_ARG_SIZ,"Mat argument has %D columns, should have at least %D",n,Y->k);
   }
-  if (X->n!=Y->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, Y %D",X->n,Y->n);
+  if (X->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)Y),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension X %D, Y %D",X->n,Y->n);
 
   ierr = PetscLogEventBegin(BV_Mult,X,Y,0,0);CHKERRQ(ierr);
   ierr = (*Y->ops->mult)(Y,alpha,beta,X,Q);CHKERRQ(ierr);
@@ -93,8 +82,6 @@ PetscErrorCode BVMult(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultVec"
 /*@
    BVMultVec - Computes y = beta*y + alpha*X*q.
 
@@ -125,7 +112,7 @@ PetscErrorCode BVMult(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 
 .seealso: BVMult(), BVMultColumn(), BVMultInPlace(), BVSetActiveColumns()
 @*/
-PetscErrorCode BVMultVec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScalar *q)
+PetscErrorCode BVMultVec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScalar q[])
 {
   PetscErrorCode ierr;
   PetscInt       n,N;
@@ -151,8 +138,6 @@ PetscErrorCode BVMultVec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultColumn"
 /*@
    BVMultColumn - Computes y = beta*y + alpha*X*q, where y is the j-th column
    of X.
@@ -173,6 +158,10 @@ PetscErrorCode BVMultVec(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScal
    in the computation. Therefore, the length of array q must be equal to j
    minus the number of leading columns.
 
+   Developer Notes:
+   If q is NULL, then the coefficients are taken from position nc+l of the
+   internal buffer vector, see BVGetBufferVec().
+
    Level: advanced
 
 .seealso: BVMult(), BVMultVec(), BVMultInPlace(), BVSetActiveColumns()
@@ -188,7 +177,6 @@ PetscErrorCode BVMultColumn(BV X,PetscScalar alpha,PetscScalar beta,PetscInt j,P
   PetscValidLogicalCollectiveScalar(X,alpha,2);
   PetscValidLogicalCollectiveScalar(X,beta,3);
   PetscValidLogicalCollectiveInt(X,j,4);
-  PetscValidPointer(q,5);
   PetscValidType(X,1);
   BVCheckSizes(X,1);
 
@@ -198,16 +186,16 @@ PetscErrorCode BVMultColumn(BV X,PetscScalar alpha,PetscScalar beta,PetscInt j,P
   ierr = PetscLogEventBegin(BV_MultVec,X,0,0,0);CHKERRQ(ierr);
   ksave = X->k;
   X->k = j;
+  if (!q && !X->buffer) { ierr = BVGetBufferVec(X,&X->buffer);CHKERRQ(ierr); }
   ierr = BVGetColumn(X,j,&y);CHKERRQ(ierr);
   ierr = (*X->ops->multvec)(X,alpha,beta,y,q);CHKERRQ(ierr);
   ierr = BVRestoreColumn(X,j,&y);CHKERRQ(ierr);
   X->k = ksave;
   ierr = PetscLogEventEnd(BV_MultVec,X,0,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlace"
 /*@
    BVMultInPlace - Update a set of vectors as V(:,s:e-1) = V*Q(:,s:e-1).
 
@@ -265,8 +253,6 @@ PetscErrorCode BVMultInPlace(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMultInPlaceTranspose"
 /*@
    BVMultInPlaceTranspose - Update a set of vectors as V(:,s:e-1) = V*Q'(:,s:e-1).
 
@@ -319,8 +305,6 @@ PetscErrorCode BVMultInPlaceTranspose(BV V,Mat Q,PetscInt s,PetscInt e)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVScale"
 /*@
    BVScale - Multiply the BV entries by a scalar value.
 
@@ -357,8 +341,6 @@ PetscErrorCode BVScale(BV bv,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVScaleColumn"
 /*@
    BVScaleColumn - Scale one column of a BV.
 
@@ -396,8 +378,30 @@ PetscErrorCode BVScaleColumn(BV bv,PetscInt j,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVSetRandom"
+PETSC_STATIC_INLINE PetscErrorCode BVSetRandomColumn_Private(BV bv,PetscInt k)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,low,high;
+  PetscScalar    *px,t;
+  Vec            x;
+
+  PetscFunctionBegin;
+  ierr = BVGetColumn(bv,k,&x);CHKERRQ(ierr);
+  if (bv->rrandom) {  /* generate the same vector irrespective of number of processes */
+    ierr = VecGetOwnershipRange(x,&low,&high);CHKERRQ(ierr);
+    ierr = VecGetArray(x,&px);CHKERRQ(ierr);
+    for (i=0;i<bv->N;i++) {
+      ierr = PetscRandomGetValue(bv->rand,&t);CHKERRQ(ierr);
+      if (i>=low && i<high) px[i-low] = t;
+    }
+    ierr = VecRestoreArray(x,&px);CHKERRQ(ierr);
+  } else {
+    ierr = VecSetRandom(x,bv->rand);CHKERRQ(ierr);
+  }
+  ierr = BVRestoreColumn(bv,k,&x);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    BVSetRandom - Set the columns of a BV to random numbers.
 
@@ -411,14 +415,12 @@ PetscErrorCode BVScaleColumn(BV bv,PetscInt j,PetscScalar alpha)
 
    Level: advanced
 
-.seealso: BVSetRandomContext(), BVSetRandomColumn(), BVSetActiveColumns()
+.seealso: BVSetRandomContext(), BVSetRandomColumn(), BVSetRandomCond(), BVSetActiveColumns()
 @*/
 PetscErrorCode BVSetRandom(BV bv)
 {
   PetscErrorCode ierr;
-  PetscInt       i,low,high,k;
-  PetscScalar    *px,t;
-  Vec            x;
+  PetscInt       k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
@@ -428,27 +430,13 @@ PetscErrorCode BVSetRandom(BV bv)
   ierr = BVGetRandomContext(bv,&bv->rand);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
   for (k=bv->l;k<bv->k;k++) {
-    ierr = BVGetColumn(bv,k,&x);CHKERRQ(ierr);
-    if (bv->rrandom) {  /* generate the same vector irrespective of number of processes */
-      ierr = VecGetOwnershipRange(x,&low,&high);CHKERRQ(ierr);
-      ierr = VecGetArray(x,&px);CHKERRQ(ierr);
-      for (i=0;i<bv->N;i++) {
-        ierr = PetscRandomGetValue(bv->rand,&t);CHKERRQ(ierr);
-        if (i>=low && i<high) px[i-low] = t;
-      }
-      ierr = VecRestoreArray(x,&px);CHKERRQ(ierr);
-    } else {
-      ierr = VecSetRandom(x,bv->rand);CHKERRQ(ierr);
-    }
-    ierr = BVRestoreColumn(bv,k,&x);CHKERRQ(ierr);
+    ierr = BVSetRandomColumn_Private(bv,k);CHKERRQ(ierr);
   }
   ierr = PetscLogEventEnd(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)bv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVSetRandomColumn"
 /*@
    BVSetRandomColumn - Set one column of a BV to random numbers.
 
@@ -465,9 +453,6 @@ PetscErrorCode BVSetRandom(BV bv)
 PetscErrorCode BVSetRandomColumn(BV bv,PetscInt j)
 {
   PetscErrorCode ierr;
-  PetscInt       i,low,high;
-  PetscScalar    *px,t;
-  Vec            x;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
@@ -478,26 +463,86 @@ PetscErrorCode BVSetRandomColumn(BV bv,PetscInt j)
 
   ierr = BVGetRandomContext(bv,&bv->rand);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
-  ierr = BVGetColumn(bv,j,&x);CHKERRQ(ierr);
-  if (bv->rrandom) {  /* generate the same vector irrespective of number of processes */
-    ierr = VecGetOwnershipRange(x,&low,&high);CHKERRQ(ierr);
-    ierr = VecGetArray(x,&px);CHKERRQ(ierr);
-    for (i=0;i<bv->N;i++) {
-      ierr = PetscRandomGetValue(bv->rand,&t);CHKERRQ(ierr);
-      if (i>=low && i<high) px[i-low] = t;
-    }
-    ierr = VecRestoreArray(x,&px);CHKERRQ(ierr);
-  } else {
-    ierr = VecSetRandom(x,bv->rand);CHKERRQ(ierr);
+  ierr = BVSetRandomColumn_Private(bv,j);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)bv);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVSetRandomCond - Set the columns of a BV to random numbers, in a way that
+   the generated matrix has a given condition number.
+
+   Logically Collective on BV
+
+   Input Parameters:
++  bv    - basis vectors
+-  condn - condition number
+
+   Note:
+   All active columns (except the leading ones) are modified.
+
+   Level: advanced
+
+.seealso: BVSetRandomContext(), BVSetRandomColumn(), BVSetActiveColumns()
+@*/
+PetscErrorCode BVSetRandomCond(BV bv,PetscReal condn)
+{
+  PetscErrorCode ierr;
+  PetscInt       k,i;
+  PetscScalar    *eig,*d;
+  DS             ds;
+  Mat            A,X,Y,Xt,M;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  PetscValidType(bv,1);
+  BVCheckSizes(bv,1);
+
+  ierr = BVGetRandomContext(bv,&bv->rand);CHKERRQ(ierr);
+  ierr = PetscLogEventBegin(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
+  /* B = rand(n,k) */
+  for (k=bv->l;k<bv->k;k++) {
+    ierr = BVSetRandomColumn_Private(bv,k);CHKERRQ(ierr);
   }
-  ierr = BVRestoreColumn(bv,j,&x);CHKERRQ(ierr);
+  ierr = DSCreate(PetscObjectComm((PetscObject)bv),&ds);CHKERRQ(ierr);
+  ierr = DSSetType(ds,DSHEP);CHKERRQ(ierr);
+  ierr = DSAllocate(ds,bv->m);CHKERRQ(ierr);
+  ierr = DSSetDimensions(ds,bv->k,0,bv->l,bv->k);CHKERRQ(ierr);
+  /* [V,S] = eig(B'*B) */
+  ierr = DSGetMat(ds,DS_MAT_A,&A);CHKERRQ(ierr);
+  ierr = BVDot(bv,bv,A);CHKERRQ(ierr);
+  ierr = DSRestoreMat(ds,DS_MAT_A,&A);CHKERRQ(ierr);
+  ierr = PetscMalloc1(bv->m,&eig);CHKERRQ(ierr);
+  ierr = DSSolve(ds,eig,NULL);CHKERRQ(ierr);
+  ierr = DSSynchronize(ds,eig,NULL);CHKERRQ(ierr);
+  ierr = DSVectors(ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
+  /* M = diag(linspace(1/condn,1,n)./sqrt(diag(S)))' */
+  ierr = MatCreateSeqDense(PETSC_COMM_SELF,bv->k,bv->k,NULL,&M);CHKERRQ(ierr);
+  ierr = MatZeroEntries(M);CHKERRQ(ierr);
+  ierr = MatDenseGetArray(M,&d);CHKERRQ(ierr);
+  for (i=0;i<bv->k;i++) d[i+i*bv->m] = (1.0/condn+(1.0-1.0/condn)/(bv->k-1)*i)/PetscSqrtScalar(eig[i]);
+  ierr = MatDenseRestoreArray(M,&d);CHKERRQ(ierr);
+  /* M = X*M*X' */
+  ierr = MatCreateSeqDense(PETSC_COMM_SELF,bv->k,bv->k,NULL,&Y);CHKERRQ(ierr);
+  ierr = MatCreateSeqDense(PETSC_COMM_SELF,bv->k,bv->k,NULL,&Xt);CHKERRQ(ierr);
+  ierr = DSGetMat(ds,DS_MAT_X,&X);CHKERRQ(ierr);
+  ierr = MatMatMult(X,M,MAT_REUSE_MATRIX,PETSC_DEFAULT,&Y);CHKERRQ(ierr);
+  ierr = MatTranspose(X,MAT_REUSE_MATRIX,&Xt);CHKERRQ(ierr);
+  ierr = MatMatMult(Y,Xt,MAT_REUSE_MATRIX,PETSC_DEFAULT,&M);CHKERRQ(ierr);
+  ierr = MatDestroy(&X);CHKERRQ(ierr);
+  /* B = B*M */
+  ierr = BVMultInPlace(bv,M,bv->l,bv->k);CHKERRQ(ierr);
+  ierr = MatDestroy(&Y);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xt);CHKERRQ(ierr);
+  ierr = MatDestroy(&M);CHKERRQ(ierr);
+  ierr = PetscFree(eig);CHKERRQ(ierr);
+  ierr = DSDestroy(&ds);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(BV_SetRandom,bv,0,0,0);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)bv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatMult"
 /*@
    BVMatMult - Computes the matrix-vector product for each column, Y=A*V.
 
@@ -537,17 +582,16 @@ PetscErrorCode BVMatMult(BV V,Mat A,BV Y)
   BVCheckSizes(Y,3);
   PetscCheckSameComm(V,1,A,2);
   PetscCheckSameTypeAndComm(V,1,Y,3);
-  if (V->n!=Y->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
-  if (V->k-V->l>Y->m-Y->l) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
+  if (V->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
+  if (V->k-V->l>Y->m-Y->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
 
   ierr = PetscLogEventBegin(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
   ierr = (*V->ops->matmult)(V,A,Y);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)Y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatMultHermitianTranspose"
 /*@
    BVMatMultHermitianTranspose - Computes the matrix-vector product with the
    conjugate transpose of a matrix for each column, Y=A^H*V.
@@ -590,8 +634,8 @@ PetscErrorCode BVMatMultHermitianTranspose(BV V,Mat A,BV Y)
   BVCheckSizes(Y,3);
   PetscCheckSameComm(V,1,A,2);
   PetscCheckSameTypeAndComm(V,1,Y,3);
-  if (V->n!=Y->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
-  if (V->k-V->l>Y->m-Y->l) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
+  if (V->n!=Y->n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_INCOMP,"Mismatching local dimension V %D, Y %D",V->n,Y->n);
+  if (V->k-V->l>Y->m-Y->l) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Y has %D non-leading columns, not enough to store %D columns",Y->m-Y->l,V->k-V->l);
 
   ierr = PetscLogEventBegin(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
   for (j=0;j<V->k-V->l;j++) {
@@ -602,11 +646,10 @@ PetscErrorCode BVMatMultHermitianTranspose(BV V,Mat A,BV Y)
     ierr = BVRestoreColumn(Y,Y->l+j,&f);CHKERRQ(ierr);
   }
   ierr = PetscLogEventEnd(BV_MatMult,V,A,Y,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)Y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVMatMultColumn"
 /*@
    BVMatMultColumn - Computes the matrix-vector product for a specified
    column, storing the result in the next column: v_{j+1}=A*v_j.
@@ -646,6 +689,7 @@ PetscErrorCode BVMatMultColumn(BV V,Mat A,PetscInt j)
   ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
   ierr = BVRestoreColumn(V,j+1,&vj1);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(BV_MatMultVec,V,A,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)V);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
