@@ -1,38 +1,29 @@
 /*
-   Basic DS routines
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic DS routines
+*/
 
 #include <slepc/private/dsimpl.h>      /*I "slepcds.h" I*/
 
 PetscFunctionList DSList = 0;
 PetscBool         DSRegisterAllCalled = PETSC_FALSE;
 PetscClassId      DS_CLASSID = 0;
-PetscLogEvent     DS_Solve = 0,DS_Vectors = 0,DS_Other = 0;
+PetscLogEvent     DS_Solve = 0,DS_Vectors = 0,DS_Synchronize = 0,DS_Other = 0;
 static PetscBool  DSPackageInitialized = PETSC_FALSE;
-const char        *DSMatName[DS_NUM_MAT] = {"A","B","C","T","D","Q","Z","X","Y","U","VT","W","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9"};
-DSMatType         DSMatExtra[DS_NUM_EXTRA] = {DS_MAT_E0,DS_MAT_E1,DS_MAT_E2,DS_MAT_E3,DS_MAT_E4,DS_MAT_E5,DS_MAT_E6,DS_MAT_E7,DS_MAT_E8,DS_MAT_E9};
 
-#undef __FUNCT__
-#define __FUNCT__ "DSFinalizePackage"
+const char *DSStateTypes[] = {"RAW","INTERMEDIATE","CONDENSED","TRUNCATED","DSStateType","DS_STATE_",0};
+const char *DSParallelTypes[] = {"REDUNDANT","SYNCHRONIZED","DSParallelType","DS_PARALLEL_",0};
+const char *DSMatName[DS_NUM_MAT] = {"A","B","C","T","D","Q","Z","X","Y","U","VT","W","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9"};
+DSMatType  DSMatExtra[DS_NUM_EXTRA] = {DS_MAT_E0,DS_MAT_E1,DS_MAT_E2,DS_MAT_E3,DS_MAT_E4,DS_MAT_E5,DS_MAT_E6,DS_MAT_E7,DS_MAT_E8,DS_MAT_E9};
+
 /*@C
    DSFinalizePackage - This function destroys everything in the SLEPc interface
    to the DS package. It is called from SlepcFinalize().
@@ -52,8 +43,6 @@ PetscErrorCode DSFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSInitializePackage"
 /*@C
   DSInitializePackage - This function initializes everything in the DS package.
   It is called from PetscDLLibraryRegister() when using dynamic libraries, and
@@ -80,6 +69,7 @@ PetscErrorCode DSInitializePackage()
   /* Register Events */
   ierr = PetscLogEventRegister("DSSolve",DS_CLASSID,&DS_Solve);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DSVectors",DS_CLASSID,&DS_Vectors);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("DSSynchronize",DS_CLASSID,&DS_Synchronize);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DSOther",DS_CLASSID,&DS_Other);CHKERRQ(ierr);
   /* Process info exclusions */
   ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
@@ -101,8 +91,6 @@ PetscErrorCode DSInitializePackage()
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCreate"
 /*@
    DSCreate - Creates a DS context.
 
@@ -147,6 +135,7 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
   ds->t             = 0;
   ds->bs            = 1;
   ds->sc            = NULL;
+  ds->pmode         = DS_PARALLEL_REDUNDANT;
 
   for (i=0;i<DS_NUM_MAT;i++) {
     ds->mat[i]      = NULL;
@@ -166,8 +155,6 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetOptionsPrefix"
 /*@C
    DSSetOptionsPrefix - Sets the prefix used for searching for all
    DS options in the database.
@@ -197,8 +184,6 @@ PetscErrorCode DSSetOptionsPrefix(DS ds,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSAppendOptionsPrefix"
 /*@C
    DSAppendOptionsPrefix - Appends to the prefix used for searching for all
    DS options in the database.
@@ -227,8 +212,6 @@ PetscErrorCode DSAppendOptionsPrefix(DS ds,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetOptionsPrefix"
 /*@C
    DSGetOptionsPrefix - Gets the prefix used for searching for all
    DS options in the database.
@@ -260,8 +243,6 @@ PetscErrorCode DSGetOptionsPrefix(DS ds,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetType"
 /*@C
    DSSetType - Selects the type for the DS object.
 
@@ -297,8 +278,6 @@ PetscErrorCode DSSetType(DS ds,DSType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetType"
 /*@C
    DSGetType - Gets the DS type name (as a string) from the DS context.
 
@@ -323,8 +302,6 @@ PetscErrorCode DSGetType(DS ds,DSType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetMethod"
 /*@
    DSSetMethod - Selects the method to be used to solve the problem.
 
@@ -334,6 +311,9 @@ PetscErrorCode DSGetType(DS ds,DSType *type)
 +  ds   - the direct solver context
 -  meth - an index indentifying the method
 
+   Options Database Key:
+.  -ds_method <meth> - Sets the method
+
    Level: intermediate
 
 .seealso: DSGetMethod()
@@ -349,8 +329,6 @@ PetscErrorCode DSSetMethod(DS ds,PetscInt meth)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetMethod"
 /*@
    DSGetMethod - Gets the method currently used in the DS.
 
@@ -375,8 +353,66 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetCompact"
+/*@
+   DSSetParallel - Selects the mode of operation in parallel runs.
+
+   Logically Collective on DS
+
+   Input Parameter:
++  ds    - the direct solver context
+-  pmode - the parallel mode
+
+   Options Database Key:
+.  -ds_parallel <mode> - Sets the parallel mode, either 'redundant' or 'synchronized'
+
+   Notes:
+   In the 'redundant' parallel mode, all processes will make the computation
+   redundantly, starting from the same data, and producing the same result.
+   This result may be slightly different in the different processes if using a
+   multithreaded BLAS library, which may cause issues in ill-conditioned problems.
+
+   In the 'synchronized' parallel mode, only the first MPI process performs the
+   computation and then the computed quantities are broadcast to the other
+   processes in the communicator. This communication is not done automatically,
+   an explicit call to DSSynchronize() is required.
+
+   Level: advanced
+
+.seealso: DSSynchronize(), DSGetParallel()
+@*/
+PetscErrorCode DSSetParallel(DS ds,DSParallelType pmode)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(ds,pmode,2);
+  ds->pmode = pmode;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   DSGetParallel - Gets the mode of operation in parallel runs.
+
+   Not Collective
+
+   Input Parameter:
+.  ds - the direct solver context
+
+   Output Parameter:
+.  pmode - the parallel mode
+
+   Level: advanced
+
+.seealso: DSSetParallel()
+@*/
+PetscErrorCode DSGetParallel(DS ds,DSParallelType *pmode)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidPointer(pmode,2);
+  *pmode = ds->pmode;
+  PetscFunctionReturn(0);
+}
+
 /*@
    DSSetCompact - Switch to compact storage of matrices.
 
@@ -407,8 +443,6 @@ PetscErrorCode DSSetCompact(DS ds,PetscBool comp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetCompact"
 /*@
    DSGetCompact - Gets the compact storage flag.
 
@@ -433,8 +467,6 @@ PetscErrorCode DSGetCompact(DS ds,PetscBool *comp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetExtraRow"
 /*@
    DSSetExtraRow - Sets a flag to indicate that the matrix has one extra
    row.
@@ -467,8 +499,6 @@ PetscErrorCode DSSetExtraRow(DS ds,PetscBool ext)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetExtraRow"
 /*@
    DSGetExtraRow - Gets the extra row flag.
 
@@ -493,8 +523,6 @@ PetscErrorCode DSGetExtraRow(DS ds,PetscBool *ext)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetRefined"
 /*@
    DSSetRefined - Sets a flag to indicate that refined vectors must be
    computed.
@@ -528,8 +556,6 @@ PetscErrorCode DSSetRefined(DS ds,PetscBool ref)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetRefined"
 /*@
    DSGetRefined - Gets the refined vectors flag.
 
@@ -554,8 +580,6 @@ PetscErrorCode DSGetRefined(DS ds,PetscBool *ref)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetBlockSize"
 /*@
    DSSetBlockSize - Sets the block size.
 
@@ -565,6 +589,9 @@ PetscErrorCode DSGetRefined(DS ds,PetscBool *ref)
 +  ds - the direct solver context
 -  bs - the block size
 
+   Options Database Key:
+.  -ds_block_size <bs> - Sets the block size
+
    Level: intermediate
 
 .seealso: DSGetBlockSize()
@@ -579,8 +606,6 @@ PetscErrorCode DSSetBlockSize(DS ds,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetBlockSize"
 /*@
    DSGetBlockSize - Gets the block size.
 
@@ -605,8 +630,6 @@ PetscErrorCode DSGetBlockSize(DS ds,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetSlepcSC"
 /*@C
    DSSetSlepcSC - Sets the sorting criterion context.
 
@@ -634,8 +657,6 @@ PetscErrorCode DSSetSlepcSC(DS ds,SlepcSC sc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetSlepcSC"
 /*@C
    DSGetSlepcSC - Gets the sorting criterion context.
 
@@ -665,8 +686,6 @@ PetscErrorCode DSGetSlepcSC(DS ds,SlepcSC *sc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetFromOptions"
 /*@
    DSSetFromOptions - Sets DS options from the options database.
 
@@ -685,6 +704,7 @@ PetscErrorCode DSSetFromOptions(DS ds)
   PetscErrorCode ierr;
   PetscInt       bs,meth;
   PetscBool      flag;
+  DSParallelType pmode;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
@@ -694,17 +714,21 @@ PetscErrorCode DSSetFromOptions(DS ds)
     ierr = DSSetType(ds,DSNHEP);CHKERRQ(ierr);
   }
   ierr = PetscObjectOptionsBegin((PetscObject)ds);CHKERRQ(ierr);
+
     ierr = PetscOptionsInt("-ds_block_size","Block size for the dense system solver","DSSetBlockSize",ds->bs,&bs,&flag);CHKERRQ(ierr);
     if (flag) { ierr = DSSetBlockSize(ds,bs);CHKERRQ(ierr); }
+
     ierr = PetscOptionsInt("-ds_method","Method to be used for the dense system","DSSetMethod",ds->method,&meth,&flag);CHKERRQ(ierr);
     if (flag) { ierr = DSSetMethod(ds,meth);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsEnum("-ds_parallel","Operation mode in parallel runs","DSSetParallel",DSParallelTypes,(PetscEnum)ds->pmode,(PetscEnum*)&pmode,&flag);CHKERRQ(ierr);
+    if (flag) { ierr = DSSetParallel(ds,pmode);CHKERRQ(ierr); }
+
     ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)ds);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSView"
 /*@C
    DSView - Prints the DS data structure.
 
@@ -732,9 +756,9 @@ PetscErrorCode DSSetFromOptions(DS ds)
 PetscErrorCode DSView(DS ds,PetscViewer viewer)
 {
   PetscBool         isascii,issvd;
-  const char        *state;
   PetscViewerFormat format;
   PetscErrorCode    ierr;
+  PetscMPIInt       size;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
@@ -745,15 +769,12 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
   if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ds,viewer);CHKERRQ(ierr);
+    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRQ(ierr);
+    if (size>1) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]);CHKERRQ(ierr);
+    }
     if (format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
-      switch (ds->state) {
-        case DS_STATE_RAW:          state = "raw"; break;
-        case DS_STATE_INTERMEDIATE: state = "intermediate"; break;
-        case DS_STATE_CONDENSED:    state = "condensed"; break;
-        case DS_STATE_TRUNCATED:    state = "truncated"; break;
-        default: SETERRQ(PetscObjectComm((PetscObject)ds),1,"Wrong value of ds->state");
-      }
-      ierr = PetscViewerASCIIPrintf(viewer,"  current state: %s\n",state);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  current state: %s\n",DSStateTypes[ds->state]);CHKERRQ(ierr);
       ierr = PetscObjectTypeCompare((PetscObject)ds,DSSVD,&issvd);CHKERRQ(ierr);
       if (issvd) {
         ierr = PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%D, n=%D, m=%D, l=%D, k=%D",ds->ld,ds->n,ds->m,ds->l,ds->k);CHKERRQ(ierr);
@@ -776,8 +797,6 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSAllocate"
 /*@
    DSAllocate - Allocates memory for internal storage or matrices in DS.
 
@@ -788,9 +807,13 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
 -  ld - leading dimension (maximum allowed dimension for the matrices, including
         the extra row if present)
 
+   Note:
+   If the leading dimension is different from a previously set value, then
+   all matrices are destroyed with DSReset().
+
    Level: intermediate
 
-.seealso: DSGetLeadingDimension(), DSSetDimensions(), DSSetExtraRow()
+.seealso: DSGetLeadingDimension(), DSSetDimensions(), DSSetExtraRow(), DSReset()
 @*/
 PetscErrorCode DSAllocate(DS ds,PetscInt ld)
 {
@@ -799,14 +822,16 @@ PetscErrorCode DSAllocate(DS ds,PetscInt ld)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidLogicalCollectiveInt(ds,ld,2);
+  PetscValidType(ds,1);
   if (ld<1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Leading dimension should be at least one");
-  ds->ld = ld;
-  ierr = (*ds->ops->allocate)(ds,ld);CHKERRQ(ierr);
+  if (ld!=ds->ld) {
+    ierr = DSReset(ds);CHKERRQ(ierr);
+    ds->ld = ld;
+    ierr = (*ds->ops->allocate)(ds,ld);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSReset"
 /*@
    DSReset - Resets the DS context to the initial state.
 
@@ -815,9 +840,13 @@ PetscErrorCode DSAllocate(DS ds,PetscInt ld)
    Input Parameter:
 .  ds - the direct solver context
 
+   Note:
+   All data structures with size depending on the leading dimension
+   of DSAllocate() are released.
+
    Level: advanced
 
-.seealso: DSDestroy()
+.seealso: DSDestroy(), DSAllocate()
 @*/
 PetscErrorCode DSReset(DS ds)
 {
@@ -825,11 +854,9 @@ PetscErrorCode DSReset(DS ds)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  if (ds) PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  if (!ds) PetscFunctionReturn(0);
   ds->state    = DS_STATE_RAW;
-  ds->compact  = PETSC_FALSE;
-  ds->refined  = PETSC_FALSE;
-  ds->extrarow = PETSC_FALSE;
   ds->ld       = 0;
   ds->l        = 0;
   ds->n        = 0;
@@ -841,17 +868,9 @@ PetscErrorCode DSReset(DS ds)
     ierr = MatDestroy(&ds->omat[i]);CHKERRQ(ierr);
   }
   ierr = PetscFree(ds->perm);CHKERRQ(ierr);
-  ierr = PetscFree(ds->work);CHKERRQ(ierr);
-  ierr = PetscFree(ds->rwork);CHKERRQ(ierr);
-  ierr = PetscFree(ds->iwork);CHKERRQ(ierr);
-  ds->lwork         = 0;
-  ds->lrwork        = 0;
-  ds->liwork        = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSDestroy"
 /*@
    DSDestroy - Destroys DS context that was created with DSCreate().
 
@@ -874,13 +893,14 @@ PetscErrorCode DSDestroy(DS *ds)
   if (--((PetscObject)(*ds))->refct > 0) { *ds = 0; PetscFunctionReturn(0); }
   ierr = DSReset(*ds);CHKERRQ(ierr);
   if ((*ds)->ops->destroy) { ierr = (*(*ds)->ops->destroy)(*ds);CHKERRQ(ierr); }
+  ierr = PetscFree((*ds)->work);CHKERRQ(ierr);
+  ierr = PetscFree((*ds)->rwork);CHKERRQ(ierr);
+  ierr = PetscFree((*ds)->iwork);CHKERRQ(ierr);
   ierr = PetscFree((*ds)->sc);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(ds);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSRegister"
 /*@C
    DSRegister - Adds a direct solver to the DS package.
 
@@ -916,8 +936,6 @@ PETSC_EXTERN PetscErrorCode DSCreate_SVD(DS);
 PETSC_EXTERN PetscErrorCode DSCreate_PEP(DS);
 PETSC_EXTERN PetscErrorCode DSCreate_NEP(DS);
 
-#undef __FUNCT__
-#define __FUNCT__ "DSRegisterAll"
 /*@C
    DSRegisterAll - Registers all of the direct solvers in the DS package.
 
