@@ -1,30 +1,18 @@
 /*
-   DS operations: DSSolve(), DSVectors(), etc
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   DS operations: DSSolve(), DSVectors(), etc
+*/
 
 #include <slepc/private/dsimpl.h>      /*I "slepcds.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetLeadingDimension"
 /*@
    DSGetLeadingDimension - Returns the leading dimension of the allocated
    matrices.
@@ -50,8 +38,6 @@ PetscErrorCode DSGetLeadingDimension(DS ds,PetscInt *ld)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetState"
 /*@
    DSSetState - Change the state of the DS object.
 
@@ -95,8 +81,6 @@ PetscErrorCode DSSetState(DS ds,DSStateType state)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetState"
 /*@
    DSGetState - Returns the current state.
 
@@ -121,8 +105,6 @@ PetscErrorCode DSGetState(DS ds,DSStateType *state)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetDimensions"
 /*@
    DSSetDimensions - Resize the matrices in the DS object.
 
@@ -184,8 +166,6 @@ PetscErrorCode DSSetDimensions(DS ds,PetscInt n,PetscInt m,PetscInt l,PetscInt k
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetDimensions"
 /*@
    DSGetDimensions - Returns the current dimensions.
 
@@ -201,12 +181,12 @@ PetscErrorCode DSSetDimensions(DS ds,PetscInt n,PetscInt m,PetscInt l,PetscInt k
 .  k  - intermediate dimension (e.g., position of arrow)
 -  t  - truncated length
 
-   Level: intermediate
-
    Note:
    The t parameter makes sense only if DSTruncate() has been called.
    Otherwise its value equals n.
 
+   Level: intermediate
+
 .seealso: DSSetDimensions(), DSTruncate()
 @*/
 PetscErrorCode DSGetDimensions(DS ds,PetscInt *n,PetscInt *m,PetscInt *l,PetscInt *k,PetscInt *t)
@@ -222,8 +202,6 @@ PetscErrorCode DSGetDimensions(DS ds,PetscInt *n,PetscInt *m,PetscInt *l,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTruncate"
 /*@
    DSTruncate - Truncates the system represented in the DS object.
 
@@ -247,11 +225,12 @@ PetscErrorCode DSTruncate(DS ds,PetscInt n)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   DSCheckSolved(ds,1);
   PetscValidLogicalCollectiveInt(ds,n,2);
   if (!ds->ops->truncate) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  if (n<ds->l || n>ds->n) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of n. Must be between l and n");
+  if (n<ds->l || n>ds->n) SETERRQ3(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of n (%D). Must be between l (%D) and n (%D)",n,ds->l,ds->n);
   ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
   ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
   ierr = (*ds->ops->truncate)(ds,n);CHKERRQ(ierr);
@@ -262,11 +241,85 @@ PetscErrorCode DSTruncate(DS ds,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetMat"
+/*@
+   DSMatGetSize - Returns the numbers of rows and columns of one of the DS matrices.
+
+   Not Collective
+
+   Input Parameters:
++  ds - the direct solver context
+-  t  - the requested matrix
+
+   Output Parameters:
++  n  - the number of rows
+-  m  - the number of columns
+
+   Note:
+   This is equivalent to MatGetSize() on a matrix obtained with DSGetMat().
+
+   Level: developer
+
+.seealso: DSSetDimensions(), DSGetMat()
+@*/
+PetscErrorCode DSMatGetSize(DS ds,DSMatType t,PetscInt *m,PetscInt *n)
+{
+  PetscErrorCode ierr;
+  PetscInt       rows,cols;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
+  DSCheckValidMat(ds,t,2);
+  if (ds->ops->matgetsize) {
+    ierr = (*ds->ops->matgetsize)(ds,t,&rows,&cols);CHKERRQ(ierr);
+  } else {
+    if (ds->state==DS_STATE_TRUNCATED && t>=DS_MAT_Q) rows = ds->t;
+    else rows = (t==DS_MAT_A && ds->extrarow)? ds->n+1: ds->n;
+    cols = ds->n;
+  }
+  if (m) *m = rows;
+  if (n) *n = cols;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   DSMatIsHermitian - Checks if one of the DS matrices is known to be Hermitian.
+
+   Not Collective
+
+   Input Parameters:
++  ds - the direct solver context
+-  t  - the requested matrix
+
+   Output Parameter:
+.  flg - the Hermitian flag
+
+   Note:
+   Does not check the matrix values directly. The flag is set according to the
+   problem structure. For instance, in DSHEP matrix A is Hermitian.
+
+   Level: developer
+
+.seealso: DSGetMat()
+@*/
+PetscErrorCode DSMatIsHermitian(DS ds,DSMatType t,PetscBool *flg)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
+  DSCheckValidMat(ds,t,2);
+  PetscValidPointer(flg,3);
+  if (ds->ops->hermitian) {
+    ierr = (*ds->ops->hermitian)(ds,t,flg);CHKERRQ(ierr);
+  } else *flg = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 /*@
    DSGetMat - Returns a sequential dense Mat object containing the requested
-   matrix. 
+   matrix.
 
    Not Collective
 
@@ -295,19 +348,17 @@ PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
 {
   PetscErrorCode ierr;
   PetscInt       j,rows,cols,arows,acols;
-  PetscBool      create=PETSC_FALSE;
+  PetscBool      create=PETSC_FALSE,flg;
   PetscScalar    *pA,*M;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
-  PetscValidLogicalCollectiveEnum(ds,m,2);
+  DSCheckValidMat(ds,m,2);
   PetscValidPointer(A,3);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
-  if (!ds->mat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Requested matrix was not created in this DS");
+  if (m==DS_MAT_T || m==DS_MAT_D) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Not implemented for T or D matrices");
 
-  rows = PetscMax(ds->n,ds->t);
-  cols = ds->m? ds->m: ds->n;
+  ierr = DSMatGetSize(ds,m,&rows,&cols);CHKERRQ(ierr);
   if (!ds->omat[m]) create=PETSC_TRUE;
   else {
     ierr = MatGetSize(ds->omat[m],&arows,&acols);CHKERRQ(ierr);
@@ -319,6 +370,12 @@ PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
   if (create) {
     ierr = MatCreateSeqDense(PETSC_COMM_SELF,rows,cols,NULL,&ds->omat[m]);CHKERRQ(ierr);
   }
+
+  /* set Hermitian flag */
+  ierr = DSMatIsHermitian(ds,m,&flg);CHKERRQ(ierr);
+  ierr = MatSetOption(ds->omat[m],MAT_HERMITIAN,flg);CHKERRQ(ierr);
+
+  /* copy entries */
   ierr = PetscObjectReference((PetscObject)ds->omat[m]);CHKERRQ(ierr);
   *A = ds->omat[m];
   M  = ds->mat[m];
@@ -330,8 +387,6 @@ PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSRestoreMat"
 /*@
    DSRestoreMat - Restores the matrix after DSGetMat() was called.
 
@@ -363,9 +418,8 @@ PetscErrorCode DSRestoreMat(DS ds,DSMatType m,Mat *A)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
-  PetscValidLogicalCollectiveEnum(ds,m,2);
+  DSCheckValidMat(ds,m,2);
   PetscValidPointer(A,3);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
   if (!ds->omat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"DSRestoreMat must match a previous call to DSGetMat");
   if (ds->omat[m]!=*A) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Mat argument is not the same as the one obtained with DSGetMat");
 
@@ -380,8 +434,6 @@ PetscErrorCode DSRestoreMat(DS ds,DSMatType m,Mat *A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetArray"
 /*@C
    DSGetArray - Returns a pointer to one of the internal arrays used to
    represent matrices. You MUST call DSRestoreArray() when you no longer
@@ -405,16 +457,13 @@ PetscErrorCode DSGetArray(DS ds,DSMatType m,PetscScalar *a[])
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
+  DSCheckValidMat(ds,m,2);
   PetscValidPointer(a,2);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
-  if (!ds->mat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Requested matrix was not created in this DS");
   *a = ds->mat[m];
   CHKMEMQ;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSRestoreArray"
 /*@C
    DSRestoreArray - Restores the matrix after DSGetArray() was called.
 
@@ -436,16 +485,14 @@ PetscErrorCode DSRestoreArray(DS ds,DSMatType m,PetscScalar *a[])
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
+  DSCheckValidMat(ds,m,2);
   PetscValidPointer(a,2);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
   CHKMEMQ;
   *a = 0;
   ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGetArrayReal"
 /*@C
    DSGetArrayReal - Returns a pointer to one of the internal arrays used to
    represent real matrices. You MUST call DSRestoreArrayReal() when you no longer
@@ -469,16 +516,13 @@ PetscErrorCode DSGetArrayReal(DS ds,DSMatType m,PetscReal *a[])
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
+  DSCheckValidMatReal(ds,m,2);
   PetscValidPointer(a,2);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
-  if (!ds->rmat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Requested matrix was not created in this DS");
   *a = ds->rmat[m];
   CHKMEMQ;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSRestoreArrayReal"
 /*@C
    DSRestoreArrayReal - Restores the matrix after DSGetArrayReal() was called.
 
@@ -500,16 +544,14 @@ PetscErrorCode DSRestoreArrayReal(DS ds,DSMatType m,PetscReal *a[])
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
+  DSCheckValidMatReal(ds,m,2);
   PetscValidPointer(a,2);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
   CHKMEMQ;
   *a = 0;
   ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSolve"
 /*@
    DSSolve - Solves the problem.
 
@@ -528,12 +570,13 @@ PetscErrorCode DSRestoreArrayReal(DS ds,DSMatType m,PetscReal *a[])
 
 .seealso: DSSort(), DSStateType
 @*/
-PetscErrorCode DSSolve(DS ds,PetscScalar *eigr,PetscScalar *eigi)
+PetscErrorCode DSSolve(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   PetscValidPointer(eigr,2);
   if (ds->state>=DS_STATE_CONDENSED) PetscFunctionReturn(0);
@@ -548,8 +591,6 @@ PetscErrorCode DSSolve(DS ds,PetscScalar *eigr,PetscScalar *eigi)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSort"
 /*@C
    DSSort - Sorts the result of DSSolve() according to a given sorting
    criterion.
@@ -590,6 +631,7 @@ PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckSolved(ds,1);
   PetscValidPointer(eigr,2);
   if (rr) PetscValidPointer(rr,4);
@@ -608,8 +650,57 @@ PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors"
+/*@
+   DSSynchronize - Make sure that all processes have the same data, performing
+   communication if necessary.
+
+   Collective on DS
+
+   Input Parameter:
++  ds   - the direct solver context
+
+   Input/Output Parameters:
++  eigr - (optional) array with the computed eigenvalues (real part)
+-  eigi - (optional) array with the computed eigenvalues (imaginary part)
+
+   Notes:
+   When the DS has been created with a communicator with more than one process,
+   the internal data, especially the computed matrices, may diverge in the
+   different processes. This happens when using multithreaded BLAS and may
+   cause numerical issues in some ill-conditioned problems. This function
+   performs the necessary communication among the processes so that the
+   internal data is exactly equal in all of them.
+
+   Depending on the parallel mode as set with DSSetParallel(), this function
+   will either do nothing or synchronize the matrices computed by DSSolve()
+   and DSSort(). The arguments eigr and eigi are typically those used in the
+   calls to DSSolve() and DSSort().
+
+   Level: developer
+
+.seealso: DSSetParallel(), DSSolve(), DSSort()
+@*/
+PetscErrorCode DSSynchronize(DS ds,PetscScalar eigr[],PetscScalar eigi[])
+{
+  PetscErrorCode ierr;
+  PetscMPIInt    size;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
+  DSCheckAlloc(ds,1);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRQ(ierr);
+  if (size>1 && ds->pmode==DS_PARALLEL_SYNCHRONIZED) {
+    ierr = PetscLogEventBegin(DS_Synchronize,ds,0,0,0);CHKERRQ(ierr);
+    if (ds->ops->synchronize) {
+      ierr = (*ds->ops->synchronize)(ds,eigr,eigi);CHKERRQ(ierr);
+    }
+    ierr = PetscLogEventEnd(DS_Synchronize,ds,0,0,0);CHKERRQ(ierr);
+    ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    DSVectors - Compute vectors associated to the dense system such
    as eigenvectors.
@@ -653,8 +744,10 @@ PetscErrorCode DSVectors(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   PetscValidLogicalCollectiveEnum(ds,mat,2);
+  if (mat>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
   if (!ds->ops->vectors) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
   if (rnorm && !j) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Must give a value of j");
   if (!ds->mat[mat]) { ierr = DSAllocateMat_Private(ds,mat);CHKERRQ(ierr); }
@@ -667,50 +760,6 @@ PetscErrorCode DSVectors(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSNormalize"
-/*@
-   DSNormalize - Normalize a column or all the columns of a matrix. Considers
-   the case when the columns represent the real and the imaginary part of a vector.
-
-   Logically Collective on DS
-
-   Input Parameter:
-+  ds  - the direct solver context
-.  mat - the matrix to be modified
--  col - the column to normalize or -1 to normalize all of them
-
-   Notes:
-   The columns are normalized with respect to the 2-norm.
-
-   If col and col+1 (or col-1 and col) represent the real and the imaginary
-   part of a vector, both columns are scaled.
-
-   Level: advanced
-
-.seealso: DSMatType
-@*/
-PetscErrorCode DSNormalize(DS ds,DSMatType mat,PetscInt col)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  DSCheckSolved(ds,1);
-  PetscValidLogicalCollectiveEnum(ds,mat,2);
-  PetscValidLogicalCollectiveInt(ds,col,3);
-  if (!ds->ops->normalize) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  if (col<-1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"col should be at least minus one");
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->normalize)(ds,mat,col);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "DSUpdateExtraRow"
 /*@
    DSUpdateExtraRow - Performs all necessary operations so that the extra
    row gets up-to-date after a call to DSSolve().
@@ -730,6 +779,7 @@ PetscErrorCode DSUpdateExtraRow(DS ds)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   if (!ds->ops->update) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
   if (!ds->extrarow) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Should have called DSSetExtraRow");
@@ -741,8 +791,6 @@ PetscErrorCode DSUpdateExtraRow(DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCond"
 /*@
    DSCond - Compute the inf-norm condition number of the first matrix
    as cond(A) = norm(A)*norm(inv(A)).
@@ -763,6 +811,7 @@ PetscErrorCode DSCond(DS ds,PetscReal *cond)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   PetscValidPointer(cond,2);
   if (!ds->ops->cond) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
@@ -774,8 +823,6 @@ PetscErrorCode DSCond(DS ds,PetscReal *cond)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTranslateHarmonic"
 /*@C
    DSTranslateHarmonic - Computes a translation of the dense system.
 
@@ -812,6 +859,7 @@ PetscErrorCode DSTranslateHarmonic(DS ds,PetscScalar tau,PetscReal beta,PetscBoo
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   if (!ds->ops->transharm) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
   ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
@@ -824,8 +872,6 @@ PetscErrorCode DSTranslateHarmonic(DS ds,PetscScalar tau,PetscReal beta,PetscBoo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTranslateRKS"
 /*@
    DSTranslateRKS - Computes a modification of the dense system corresponding
    to an update of the shift in a rational Krylov method.
@@ -856,6 +902,7 @@ PetscErrorCode DSTranslateRKS(DS ds,PetscScalar alpha)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   if (!ds->ops->transrks) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
   ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
@@ -869,11 +916,9 @@ PetscErrorCode DSTranslateRKS(DS ds,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCopyMat"
 /*@
    DSCopyMat - Copies the contents of a sequential dense Mat object to
-   the indicated DS matrix, or vice versa. 
+   the indicated DS matrix, or vice versa.
 
    Not Collective
 
@@ -907,21 +952,20 @@ PetscErrorCode DSCopyMat(DS ds,DSMatType m,PetscInt mr,PetscInt mc,Mat A,PetscIn
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   PetscValidLogicalCollectiveEnum(ds,m,2);
-  PetscValidHeaderSpecific(A,MAT_CLASSID,3);
-  PetscValidLogicalCollectiveBool(ds,out,4);
-  PetscValidLogicalCollectiveInt(ds,mr,5);
-  PetscValidLogicalCollectiveInt(ds,mc,6);
-  PetscValidLogicalCollectiveInt(ds,Ar,7);
-  PetscValidLogicalCollectiveInt(ds,Ac,8);
-  PetscValidLogicalCollectiveInt(ds,rows,9);
-  PetscValidLogicalCollectiveInt(ds,rows,10);
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
-  if (!ds->mat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Requested matrix was not created in this DS");
+  DSCheckValidMat(ds,m,2);
+  PetscValidLogicalCollectiveInt(ds,mr,3);
+  PetscValidLogicalCollectiveInt(ds,mc,4);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,5);
+  PetscValidLogicalCollectiveInt(ds,Ar,6);
+  PetscValidLogicalCollectiveInt(ds,Ac,7);
+  PetscValidLogicalCollectiveInt(ds,rows,8);
+  PetscValidLogicalCollectiveInt(ds,cols,9);
+  PetscValidLogicalCollectiveBool(ds,out,10);
   if (!rows || !cols) PetscFunctionReturn(0);
 
-  mrows = PetscMax(ds->n,ds->t);
-  mcols = ds->m? ds->m: ds->n;
+  ierr = DSMatGetSize(ds,m,&mrows,&mcols);CHKERRQ(ierr);
   ierr = MatGetSize(A,&arows,&acols);CHKERRQ(ierr);
+  if (m==DS_MAT_T || m==DS_MAT_D) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Not implemented for T or D matrices");
   if (mr<0 || mr>=mrows) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial row in m");
   if (mc<0 || mc>=mcols) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial column in m");
   if (Ar<0 || Ar>=arows) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial row in A");
