@@ -1,35 +1,23 @@
 /*
-   Private DS routines
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Private DS routines
+*/
 
 #include <slepc/private/dsimpl.h>      /*I "slepcds.h" I*/
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "DSAllocateMat_Private"
-PetscErrorCode DSAllocateMat_Private(DS ds,DSMatType m)
+PetscErrorCode DSAllocateMatrix_Private(DS ds,DSMatType m,PetscBool isreal)
 {
   size_t         sz;
-  PetscInt       n,d;
+  PetscInt       n,d,nelem;
   PetscBool      ispep;
   PetscErrorCode ierr;
 
@@ -42,51 +30,40 @@ PetscErrorCode DSAllocateMat_Private(DS ds,DSMatType m)
   else n = ds->ld;
   switch (m) {
     case DS_MAT_T:
-      sz = 3*ds->ld*sizeof(PetscScalar);
+      nelem = 3*ds->ld;
       break;
     case DS_MAT_D:
-      sz = ds->ld*sizeof(PetscScalar);
+      nelem = ds->ld;
       break;
     case DS_MAT_X:
-      sz = ds->ld*n*sizeof(PetscScalar);
+      nelem = ds->ld*n;
       break;
     case DS_MAT_Y:
-      sz = ds->ld*n*sizeof(PetscScalar);
+      nelem = ds->ld*n;
       break;
     default:
-      sz = n*n*sizeof(PetscScalar);
+      nelem = n*n;
   }
-  if (ds->mat[m]) {
-    ierr = PetscFree(ds->mat[m]);CHKERRQ(ierr);
+  if (isreal) {
+    sz = nelem*sizeof(PetscReal);
+    if (ds->rmat[m]) {
+      ierr = PetscFree(ds->rmat[m]);CHKERRQ(ierr);
+    } else {
+      ierr = PetscLogObjectMemory((PetscObject)ds,sz);CHKERRQ(ierr);
+    }
+    ierr = PetscCalloc1(nelem,&ds->rmat[m]);CHKERRQ(ierr);
   } else {
-    ierr = PetscLogObjectMemory((PetscObject)ds,sz);CHKERRQ(ierr);
+    sz = nelem*sizeof(PetscScalar);
+    if (ds->mat[m]) {
+      ierr = PetscFree(ds->mat[m]);CHKERRQ(ierr);
+    } else {
+      ierr = PetscLogObjectMemory((PetscObject)ds,sz);CHKERRQ(ierr);
+    }
+    ierr = PetscCalloc1(nelem,&ds->mat[m]);CHKERRQ(ierr);
   }
-  ierr = PetscMalloc(sz,&ds->mat[m]);CHKERRQ(ierr);
-  ierr = PetscMemzero(ds->mat[m],sz);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSAllocateMatReal_Private"
-PetscErrorCode DSAllocateMatReal_Private(DS ds,DSMatType m)
-{
-  size_t         sz;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (m==DS_MAT_T) sz = 3*ds->ld*sizeof(PetscReal);
-  else if (m==DS_MAT_D) sz = ds->ld*sizeof(PetscReal);
-  else sz = ds->ld*ds->ld*sizeof(PetscReal);
-  if (!ds->rmat[m]) {
-    ierr = PetscLogObjectMemory((PetscObject)ds,sz);CHKERRQ(ierr);
-    ierr = PetscMalloc(sz,&ds->rmat[m]);CHKERRQ(ierr);
-  }
-  ierr = PetscMemzero(ds->rmat[m],sz);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "DSAllocateWork_Private"
 PetscErrorCode DSAllocateWork_Private(DS ds,PetscInt s,PetscInt r,PetscInt i)
 {
   PetscErrorCode ierr;
@@ -113,8 +90,6 @@ PetscErrorCode DSAllocateWork_Private(DS ds,PetscInt s,PetscInt r,PetscInt i)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSViewMat"
 /*@C
    DSViewMat - Prints one of the internal DS matrices.
 
@@ -136,28 +111,24 @@ PetscErrorCode DSAllocateWork_Private(DS ds,PetscInt s,PetscInt r,PetscInt i)
 PetscErrorCode DSViewMat(DS ds,PetscViewer viewer,DSMatType m)
 {
   PetscErrorCode    ierr;
-  PetscInt          i,j,rows,cols,d;
+  PetscInt          i,j,rows,cols;
   PetscScalar       *v;
   PetscViewerFormat format;
-  PetscBool         ispep;
 #if defined(PETSC_USE_COMPLEX)
   PetscBool         allreal = PETSC_TRUE;
 #endif
 
   PetscFunctionBegin;
-  if (m>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
-  if (!ds->mat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Requested matrix was not created in this DS");
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(ds,m,3);
+  DSCheckValidMat(ds,m,3);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ds));
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
+  PetscCheckSameComm(ds,1,viewer,2);
   ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
   if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) PetscFunctionReturn(0);
   ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
-  if (ds->state==DS_STATE_TRUNCATED && m>=DS_MAT_Q) rows = ds->t;
-  else rows = (m==DS_MAT_A && ds->extrarow)? ds->n+1: ds->n;
-  cols = (ds->m!=0)? ds->m: ds->n;
-  ierr = PetscObjectTypeCompare((PetscObject)ds,DSPEP,&ispep);CHKERRQ(ierr);
-  if (ispep) {
-    ierr = DSPEPGetDegree(ds,&d);CHKERRQ(ierr);
-  }
-  if (ispep && (m==DS_MAT_X || m==DS_MAT_Y)) cols = d*ds->n;
+  ierr = DSMatGetSize(ds,m,&rows,&cols);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
   /* determine if matrix has all real values */
   v = ds->mat[m];
@@ -177,12 +148,12 @@ PetscErrorCode DSViewMat(DS ds,PetscViewer viewer,DSMatType m)
     for (j=0;j<cols;j++) {
 #if defined(PETSC_USE_COMPLEX)
       if (allreal) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%18.16e ",PetscRealPart(*v));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"%18.16e ",(double)PetscRealPart(*v));CHKERRQ(ierr);
       } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"%18.16e%+18.16ei ",PetscRealPart(*v),PetscImaginaryPart(*v));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"%18.16e%+18.16ei ",(double)PetscRealPart(*v),(double)PetscImaginaryPart(*v));CHKERRQ(ierr);
       }
 #else
-      ierr = PetscViewerASCIIPrintf(viewer,"%18.16e ",*v);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"%18.16e ",(double)*v);CHKERRQ(ierr);
 #endif
       v += ds->ld;
     }
@@ -197,8 +168,6 @@ PetscErrorCode DSViewMat(DS ds,PetscViewer viewer,DSMatType m)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSortEigenvalues_Private"
 PetscErrorCode DSSortEigenvalues_Private(DS ds,PetscScalar *wr,PetscScalar *wi,PetscInt *perm,PetscBool isghiep)
 {
   PetscErrorCode ierr;
@@ -252,8 +221,6 @@ PetscErrorCode DSSortEigenvalues_Private(DS ds,PetscScalar *wr,PetscScalar *wi,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSortEigenvaluesReal_Private"
 PetscErrorCode DSSortEigenvaluesReal_Private(DS ds,PetscReal *eig,PetscInt *perm)
 {
   PetscErrorCode ierr;
@@ -278,8 +245,6 @@ PetscErrorCode DSSortEigenvaluesReal_Private(DS ds,PetscReal *eig,PetscInt *perm
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCopyMatrix_Private"
 /*
   DSCopyMatrix_Private - Copies the trailing block of a matrix (from
   rows/columns l to n).
@@ -302,8 +267,6 @@ PetscErrorCode DSCopyMatrix_Private(DS ds,DSMatType dst,DSMatType src)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSPermuteColumns_Private"
 PetscErrorCode DSPermuteColumns_Private(DS ds,PetscInt l,PetscInt n,DSMatType mat,PetscInt *perm)
 {
   PetscInt    i,j,k,p,ld;
@@ -327,8 +290,6 @@ PetscErrorCode DSPermuteColumns_Private(DS ds,PetscInt l,PetscInt n,DSMatType ma
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSPermuteRows_Private"
 PetscErrorCode DSPermuteRows_Private(DS ds,PetscInt l,PetscInt n,DSMatType mat,PetscInt *perm)
 {
   PetscInt    i,j,m=ds->m,k,p,ld;
@@ -353,8 +314,6 @@ PetscErrorCode DSPermuteRows_Private(DS ds,PetscInt l,PetscInt n,DSMatType mat,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSPermuteBoth_Private"
 PetscErrorCode DSPermuteBoth_Private(DS ds,PetscInt l,PetscInt n,DSMatType mat1,DSMatType mat2,PetscInt *perm)
 {
   PetscInt    i,j,m=ds->m,k,p,ld;
@@ -384,8 +343,6 @@ PetscErrorCode DSPermuteBoth_Private(DS ds,PetscInt l,PetscInt n,DSMatType mat1,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSetIdentity"
 /*@
    DSSetIdentity - Copy the identity (a diagonal matrix with ones) on the
    active part of a matrix.
@@ -393,8 +350,8 @@ PetscErrorCode DSPermuteBoth_Private(DS ds,PetscInt l,PetscInt n,DSMatType mat1,
    Logically Collective on DS
 
    Input Parameters:
-+  ds     - the direct solver context
--  mat    - the matrix to modify
++  ds  - the direct solver context
+-  mat - the matrix to modify
 
    Level: intermediate
 @*/
@@ -405,21 +362,21 @@ PetscErrorCode DSSetIdentity(DS ds,DSMatType mat)
   PetscInt       i,ld,n,l;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(ds,mat,2);
+  DSCheckValidMat(ds,mat,2);
+
   ierr = DSGetDimensions(ds,&n,NULL,&l,NULL,NULL);CHKERRQ(ierr);
   ierr = DSGetLeadingDimension(ds,&ld);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
   ierr = DSGetArray(ds,mat,&x);CHKERRQ(ierr);
   ierr = PetscMemzero(&x[ld*l],ld*(n-l)*sizeof(PetscScalar));CHKERRQ(ierr);
-  for (i=l;i<n;i++) {
-    x[ld*i+i] = 1.0;
-  }
+  for (i=l;i<n;i++) x[i+i*ld] = 1.0;
   ierr = DSRestoreArray(ds,mat,&x);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSOrthogonalize"
 /*@C
    DSOrthogonalize - Orthogonalize the columns of a matrix.
 
@@ -452,6 +409,7 @@ PetscErrorCode DSOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscInt *lindc
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   PetscValidLogicalCollectiveEnum(ds,mat,2);
+  DSCheckValidMat(ds,mat,2);
   PetscValidLogicalCollectiveInt(ds,cols,3);
 
   ierr = DSGetDimensions(ds,&n,NULL,&l,NULL,NULL);CHKERRQ(ierr);
@@ -469,15 +427,15 @@ PetscErrorCode DSOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscInt *lindc
   ierr = PetscBLASIntCast(cols,&cA);CHKERRQ(ierr);
   lw = -1;
   PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&rA,&cA,A,&ld_,&dummy,&saux,&lw,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQRF %d",info);
+  SlepcCheckLapackInfo("geqrf",info);
   lw = (PetscBLASInt)PetscRealPart(saux);
   ierr = DSAllocateWork_Private(ds,lw+ltau,0,0);CHKERRQ(ierr);
   tau = ds->work;
   w = &tau[ltau];
   PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&rA,&cA,&A[ld*l+l],&ld_,tau,w,&lw,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQRF %d",info);
+  SlepcCheckLapackInfo("geqrf",info);
   PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&rA,&ltau,&ltau,&A[ld*l+l],&ld_,tau,w,&lw,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGQR %d",info);
+  SlepcCheckLapackInfo("ungqr",info);
   if (lindcols) *lindcols = ltau;
 
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
@@ -488,8 +446,6 @@ PetscErrorCode DSOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscInt *lindc
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SlepcMatDenseMult"
 /*
   Compute C <- a*A*B + b*C, where
     ldC, the leading dimension of C,
@@ -518,7 +474,7 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
   if (Bt) tmp = rB, rB = cB, cB = tmp, qB = T;
 
   /* Check size */
-  if (cA != rB) SETERRQ(PETSC_COMM_SELF,1, "Matrix dimensions do not match");
+  if (cA != rB) SETERRQ(PETSC_COMM_SELF,1,"Matrix dimensions do not match");
 
   /* Do stub */
   if ((rA == 1) && (cA == 1) && (cB == 1)) {
@@ -532,12 +488,10 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
     PetscStackCallBLAS("BLASgemm",BLASgemm_(qA,qB,&m,&n,&k,&a,(PetscScalar*)A,&ldA,(PetscScalar*)B,&ldB,&b,C,&ldC));
   }
 
-  ierr = PetscLogFlops(m*n*2*k);CHKERRQ(ierr);
+  ierr = PetscLogFlops(2.0*m*n*k);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSPseudoOrthogonalize"
 /*@C
    DSPseudoOrthogonalize - Orthogonalize the columns of a matrix with Modified
    Gram-Schmidt in an indefinite inner product space defined by a signature.
@@ -552,7 +506,10 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
 
    Output Parameters:
 +  lindcols - (optional) linearly independent columns of the matrix
--  ns   - (optional) the new norm of the vectors
+-  ns   - (optional) the new signature of the vectors
+
+   Note:
+   After the call the matrix satisfies A'*s*A = ns.
 
    Level: developer
 
@@ -570,6 +527,7 @@ PetscErrorCode DSPseudoOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscReal
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   PetscValidLogicalCollectiveEnum(ds,mat,2);
+  DSCheckValidMat(ds,mat,2);
   PetscValidLogicalCollectiveInt(ds,cols,3);
   PetscValidRealPointer(s,4);
   ierr = DSGetDimensions(ds,&n,NULL,&l,NULL,NULL);CHKERRQ(ierr);
