@@ -1,27 +1,17 @@
+/*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
 /*
    Phi functions
       phi_0(x) = exp(x)
       phi_1(x) = (exp(x)-1)/x
       phi_k(x) = (phi_{k-1}(x)-1/(k-1)!)/x
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/fnimpl.h>      /*I "slepcfn.h" I*/
@@ -45,8 +35,6 @@ static void PhiFunction(PetscScalar x,PetscScalar *y,PetscInt k)
   }
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunction_Phi"
 PetscErrorCode FNEvaluateFunction_Phi(FN fn,PetscScalar x,PetscScalar *y)
 {
   FN_PHI *ctx = (FN_PHI*)fn->data;
@@ -72,8 +60,6 @@ static void PhiDerivative(PetscScalar x,PetscScalar *y,PetscInt k)
   }
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateDerivative_Phi"
 PetscErrorCode FNEvaluateDerivative_Phi(FN fn,PetscScalar x,PetscScalar *y)
 {
   FN_PHI *ctx = (FN_PHI*)fn->data;
@@ -83,19 +69,17 @@ PetscErrorCode FNEvaluateDerivative_Phi(FN fn,PetscScalar x,PetscScalar *y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNPhiSetIndex_Phi"
 static PetscErrorCode FNPhiSetIndex_Phi(FN fn,PetscInt k)
 {
   FN_PHI *ctx = (FN_PHI*)fn->data;
 
   PetscFunctionBegin;
+  if (k<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Index cannot be negative");
+  if (k>10) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Only implemented for k<=10");
   ctx->k = k;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNPhiSetIndex"
 /*@
    FNPhiSetIndex - Sets the index of the phi-function.
 
@@ -124,14 +108,10 @@ PetscErrorCode FNPhiSetIndex(FN fn,PetscInt k)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,k,2);
-  if (k<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Index cannot be negative");
-  if (k>10) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Only implemented for k<=10");
   ierr = PetscTryMethod(fn,"FNPhiSetIndex_C",(FN,PetscInt),(fn,k));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNPhiGetIndex_Phi"
 static PetscErrorCode FNPhiGetIndex_Phi(FN fn,PetscInt *k)
 {
   FN_PHI *ctx = (FN_PHI*)fn->data;
@@ -141,8 +121,6 @@ static PetscErrorCode FNPhiGetIndex_Phi(FN fn,PetscInt *k)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNPhiGetIndex"
 /*@
    FNPhiGetIndex - Gets the index of the phi-function.
 
@@ -169,8 +147,6 @@ PetscErrorCode FNPhiGetIndex(FN fn,PetscInt *k)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNView_Phi"
 PetscErrorCode FNView_Phi(FN fn,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -205,8 +181,6 @@ PetscErrorCode FNView_Phi(FN fn,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNSetFromOptions_Phi"
 PetscErrorCode FNSetFromOptions_Phi(PetscOptionItems *PetscOptionsObject,FN fn)
 {
   PetscErrorCode ierr;
@@ -216,16 +190,14 @@ PetscErrorCode FNSetFromOptions_Phi(PetscOptionItems *PetscOptionsObject,FN fn)
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"FN Phi Options");CHKERRQ(ierr);
+
     ierr = PetscOptionsInt("-fn_phi_index","Index of the phi-function","FNPhiSetIndex",ctx->k,&k,&flag);CHKERRQ(ierr);
-    if (flag) {
-      ierr = FNPhiSetIndex(fn,k);CHKERRQ(ierr);
-    }
+    if (flag) { ierr = FNPhiSetIndex(fn,k);CHKERRQ(ierr); }
+
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNDuplicate_Phi"
 PetscErrorCode FNDuplicate_Phi(FN fn,MPI_Comm comm,FN *newfn)
 {
   FN_PHI *ctx = (FN_PHI*)fn->data,*ctx2 = (FN_PHI*)(*newfn)->data;
@@ -235,8 +207,6 @@ PetscErrorCode FNDuplicate_Phi(FN fn,MPI_Comm comm,FN *newfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNDestroy_Phi"
 PetscErrorCode FNDestroy_Phi(FN fn)
 {
   PetscErrorCode ierr;
@@ -248,8 +218,6 @@ PetscErrorCode FNDestroy_Phi(FN fn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNCreate_Phi"
 PETSC_EXTERN PetscErrorCode FNCreate_Phi(FN fn)
 {
   PetscErrorCode ierr;
