@@ -1,25 +1,15 @@
 /*
-   Rational function  r(x) = p(x)/q(x), where p(x) and q(x) are polynomials
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Rational function  r(x) = p(x)/q(x), where p(x) and q(x) are polynomials
+*/
 
 #include <slepc/private/fnimpl.h>      /*I "slepcfn.h" I*/
 #include <slepcblaslapack.h>
@@ -31,8 +21,6 @@ typedef struct {
   PetscInt    nq;         /* length of array qcoeff, q(x) has degree nq-1 */
 } FN_RATIONAL;
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunction_Rational"
 PetscErrorCode FNEvaluateFunction_Rational(FN fn,PetscScalar x,PetscScalar *y)
 {
   FN_RATIONAL *ctx = (FN_RATIONAL*)fn->data;
@@ -57,9 +45,7 @@ PetscErrorCode FNEvaluateFunction_Rational(FN fn,PetscScalar x,PetscScalar *y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMat_Private"
-static PetscErrorCode FNEvaluateFunctionMat_Private(FN fn,PetscScalar *Aa,PetscScalar *Ba,PetscInt m,PetscBool firstonly)
+static PetscErrorCode FNEvaluateFunctionMat_Rational_Private(FN fn,PetscScalar *Aa,PetscScalar *Ba,PetscInt m,PetscBool firstonly)
 {
 #if defined(PETSC_MISSING_LAPACK_GESV)
   PetscFunctionBegin;
@@ -91,6 +77,7 @@ static PetscErrorCode FNEvaluateFunctionMat_Private(FN fn,PetscScalar *Aa,PetscS
       ierr = PetscMemcpy(P,W,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
       for (i=0;i<m;i++) P[i+i*ld] += ctx->pcoeff[j];
     }
+    ierr = PetscLogFlops(2.0*n*n*n*(ctx->np-1));CHKERRQ(ierr);
   }
   if (ctx->nq) {
     ierr = PetscMemzero(Q,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
@@ -101,7 +88,8 @@ static PetscErrorCode FNEvaluateFunctionMat_Private(FN fn,PetscScalar *Aa,PetscS
       for (i=0;i<m;i++) Q[i+i*ld] += ctx->qcoeff[j];
     }
     PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&n,&k,Q,&ld,ipiv,P,&ld,&info));
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESV %d",info);
+    SlepcCheckLapackInfo("gesv",info);
+    ierr = PetscLogFlops(2.0*n*n*n*(ctx->nq-1)+2.0*n*n*n/3.0+2.0*n*n*k);CHKERRQ(ierr);
   }
   if (Aa==Ba) {
     ierr = PetscMemcpy(Aa,P,m*k*sizeof(PetscScalar));CHKERRQ(ierr);
@@ -113,8 +101,6 @@ static PetscErrorCode FNEvaluateFunctionMat_Private(FN fn,PetscScalar *Aa,PetscS
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMat_Rational"
 PetscErrorCode FNEvaluateFunctionMat_Rational(FN fn,Mat A,Mat B)
 {
   PetscErrorCode ierr;
@@ -125,14 +111,12 @@ PetscErrorCode FNEvaluateFunctionMat_Rational(FN fn,Mat A,Mat B)
   ierr = MatDenseGetArray(A,&Aa);CHKERRQ(ierr);
   ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
   ierr = MatGetSize(A,&m,NULL);CHKERRQ(ierr);
-  ierr = FNEvaluateFunctionMat_Private(fn,Aa,Ba,m,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = FNEvaluateFunctionMat_Rational_Private(fn,Aa,Ba,m,PETSC_FALSE);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(A,&Aa);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMatVec_Rational"
 PetscErrorCode FNEvaluateFunctionMatVec_Rational(FN fn,Mat A,Vec v)
 {
   PetscErrorCode ierr;
@@ -145,7 +129,7 @@ PetscErrorCode FNEvaluateFunctionMatVec_Rational(FN fn,Mat A,Vec v)
   ierr = MatDenseGetArray(A,&Aa);CHKERRQ(ierr);
   ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
   ierr = MatGetSize(A,&m,NULL);CHKERRQ(ierr);
-  ierr = FNEvaluateFunctionMat_Private(fn,Aa,Ba,m,PETSC_TRUE);CHKERRQ(ierr);
+  ierr = FNEvaluateFunctionMat_Rational_Private(fn,Aa,Ba,m,PETSC_TRUE);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(A,&Aa);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
   ierr = MatGetColumnVector(B,v,0);CHKERRQ(ierr);
@@ -153,8 +137,6 @@ PetscErrorCode FNEvaluateFunctionMatVec_Rational(FN fn,Mat A,Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateDerivative_Rational"
 PetscErrorCode FNEvaluateDerivative_Rational(FN fn,PetscScalar x,PetscScalar *yp)
 {
   FN_RATIONAL *ctx = (FN_RATIONAL*)fn->data;
@@ -187,8 +169,6 @@ PetscErrorCode FNEvaluateDerivative_Rational(FN fn,PetscScalar x,PetscScalar *yp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNView_Rational"
 PetscErrorCode FNView_Rational(FN fn,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -256,8 +236,6 @@ PetscErrorCode FNView_Rational(FN fn,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalSetNumerator_Rational"
 static PetscErrorCode FNRationalSetNumerator_Rational(FN fn,PetscInt np,PetscScalar *pcoeff)
 {
   PetscErrorCode ierr;
@@ -265,6 +243,7 @@ static PetscErrorCode FNRationalSetNumerator_Rational(FN fn,PetscInt np,PetscSca
   PetscInt       i;
 
   PetscFunctionBegin;
+  if (np<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument np cannot be negative");
   ctx->np = np;
   ierr = PetscFree(ctx->pcoeff);CHKERRQ(ierr);
   if (np) {
@@ -275,9 +254,7 @@ static PetscErrorCode FNRationalSetNumerator_Rational(FN fn,PetscInt np,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalSetNumerator"
-/*@
+/*@C
    FNRationalSetNumerator - Sets the parameters defining the numerator of the
    rational function.
 
@@ -308,14 +285,11 @@ PetscErrorCode FNRationalSetNumerator(FN fn,PetscInt np,PetscScalar *pcoeff)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,np,2);
-  if (np<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument np cannot be negative");
   if (np) PetscValidPointer(pcoeff,3);
   ierr = PetscTryMethod(fn,"FNRationalSetNumerator_C",(FN,PetscInt,PetscScalar*),(fn,np,pcoeff));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalGetNumerator_Rational"
 static PetscErrorCode FNRationalGetNumerator_Rational(FN fn,PetscInt *np,PetscScalar *pcoeff[])
 {
   PetscErrorCode ierr;
@@ -334,9 +308,7 @@ static PetscErrorCode FNRationalGetNumerator_Rational(FN fn,PetscInt *np,PetscSc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalGetNumerator"
-/*@
+/*@C
    FNRationalGetNumerator - Gets the parameters that define the numerator of the
    rational function.
 
@@ -368,8 +340,6 @@ PetscErrorCode FNRationalGetNumerator(FN fn,PetscInt *np,PetscScalar *pcoeff[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalSetDenominator_Rational"
 static PetscErrorCode FNRationalSetDenominator_Rational(FN fn,PetscInt nq,PetscScalar *qcoeff)
 {
   PetscErrorCode ierr;
@@ -377,6 +347,7 @@ static PetscErrorCode FNRationalSetDenominator_Rational(FN fn,PetscInt nq,PetscS
   PetscInt       i;
 
   PetscFunctionBegin;
+  if (nq<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument nq cannot be negative");
   ctx->nq = nq;
   ierr = PetscFree(ctx->qcoeff);CHKERRQ(ierr);
   if (nq) {
@@ -387,9 +358,7 @@ static PetscErrorCode FNRationalSetDenominator_Rational(FN fn,PetscInt nq,PetscS
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalSetDenominator"
-/*@
+/*@C
    FNRationalSetDenominator - Sets the parameters defining the denominator of the
    rational function.
 
@@ -420,14 +389,11 @@ PetscErrorCode FNRationalSetDenominator(FN fn,PetscInt nq,PetscScalar *qcoeff)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,nq,2);
-  if (nq<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument nq cannot be negative");
   if (nq) PetscValidPointer(qcoeff,3);
   ierr = PetscTryMethod(fn,"FNRationalSetDenominator_C",(FN,PetscInt,PetscScalar*),(fn,nq,qcoeff));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalGetDenominator_Rational"
 static PetscErrorCode FNRationalGetDenominator_Rational(FN fn,PetscInt *nq,PetscScalar *qcoeff[])
 {
   PetscErrorCode ierr;
@@ -446,9 +412,7 @@ static PetscErrorCode FNRationalGetDenominator_Rational(FN fn,PetscInt *nq,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRationalGetDenominator"
-/*@
+/*@C
    FNRationalGetDenominator - Gets the parameters that define the denominator of the
    rational function.
 
@@ -480,8 +444,6 @@ PetscErrorCode FNRationalGetDenominator(FN fn,PetscInt *nq,PetscScalar *qcoeff[]
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNSetFromOptions_Rational"
 PetscErrorCode FNSetFromOptions_Rational(PetscOptionItems *PetscOptionsObject,FN fn)
 {
   PetscErrorCode ierr;
@@ -493,26 +455,20 @@ PetscErrorCode FNSetFromOptions_Rational(PetscOptionItems *PetscOptionsObject,FN
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"FN Rational Options");CHKERRQ(ierr);
 
-  k = PARMAX;
-  for (i=0;i<k;i++) array[i] = 0;
-  ierr = PetscOptionsScalarArray("-fn_rational_numerator","Numerator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetNumerator",array,&k,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = FNRationalSetNumerator(fn,k,array);CHKERRQ(ierr);
-  }
+    k = PARMAX;
+    for (i=0;i<k;i++) array[i] = 0;
+    ierr = PetscOptionsScalarArray("-fn_rational_numerator","Numerator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetNumerator",array,&k,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = FNRationalSetNumerator(fn,k,array);CHKERRQ(ierr); }
 
-  k = PARMAX;
-  for (i=0;i<k;i++) array[i] = 0;
-  ierr = PetscOptionsScalarArray("-fn_rational_denominator","Denominator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetDenominator",array,&k,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = FNRationalSetDenominator(fn,k,array);CHKERRQ(ierr);
-  }
+    k = PARMAX;
+    for (i=0;i<k;i++) array[i] = 0;
+    ierr = PetscOptionsScalarArray("-fn_rational_denominator","Denominator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetDenominator",array,&k,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = FNRationalSetDenominator(fn,k,array);CHKERRQ(ierr); }
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNDuplicate_Rational"
 PetscErrorCode FNDuplicate_Rational(FN fn,MPI_Comm comm,FN *newfn)
 {
   PetscErrorCode ierr;
@@ -535,8 +491,6 @@ PetscErrorCode FNDuplicate_Rational(FN fn,MPI_Comm comm,FN *newfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNDestroy_Rational"
 PetscErrorCode FNDestroy_Rational(FN fn)
 {
   PetscErrorCode ierr;
@@ -553,8 +507,6 @@ PetscErrorCode FNDestroy_Rational(FN fn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNCreate_Rational"
 PETSC_EXTERN PetscErrorCode FNCreate_Rational(FN fn)
 {
   PetscErrorCode ierr;
@@ -564,14 +516,14 @@ PETSC_EXTERN PetscErrorCode FNCreate_Rational(FN fn)
   ierr = PetscNewLog(fn,&ctx);CHKERRQ(ierr);
   fn->data = (void*)ctx;
 
-  fn->ops->evaluatefunction       = FNEvaluateFunction_Rational;
-  fn->ops->evaluatederivative     = FNEvaluateDerivative_Rational;
-  fn->ops->evaluatefunctionmat    = FNEvaluateFunctionMat_Rational;
-  fn->ops->evaluatefunctionmatvec = FNEvaluateFunctionMatVec_Rational;
-  fn->ops->setfromoptions         = FNSetFromOptions_Rational;
-  fn->ops->view                   = FNView_Rational;
-  fn->ops->duplicate              = FNDuplicate_Rational;
-  fn->ops->destroy                = FNDestroy_Rational;
+  fn->ops->evaluatefunction          = FNEvaluateFunction_Rational;
+  fn->ops->evaluatederivative        = FNEvaluateDerivative_Rational;
+  fn->ops->evaluatefunctionmat[0]    = FNEvaluateFunctionMat_Rational;
+  fn->ops->evaluatefunctionmatvec[0] = FNEvaluateFunctionMatVec_Rational;
+  fn->ops->setfromoptions            = FNSetFromOptions_Rational;
+  fn->ops->view                      = FNView_Rational;
+  fn->ops->duplicate                 = FNDuplicate_Rational;
+  fn->ops->destroy                   = FNDestroy_Rational;
   ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetNumerator_C",FNRationalSetNumerator_Rational);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetNumerator_C",FNRationalGetNumerator_Rational);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetDenominator_C",FNRationalSetDenominator_Rational);CHKERRQ(ierr);
