@@ -1,25 +1,15 @@
 /*
-     Basic routines
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic FN routines
+*/
 
 #include <slepc/private/fnimpl.h>      /*I "slepcfn.h" I*/
 #include <slepcblaslapack.h>
@@ -30,8 +20,8 @@ PetscClassId      FN_CLASSID = 0;
 PetscLogEvent     FN_Evaluate = 0;
 static PetscBool  FNPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "FNFinalizePackage"
+const char *FNParallelTypes[] = {"REDUNDANT","SYNCHRONIZED","FNParallelType","FN_PARALLEL_",0};
+
 /*@C
    FNFinalizePackage - This function destroys everything in the Slepc interface
    to the FN package. It is called from SlepcFinalize().
@@ -51,8 +41,6 @@ PetscErrorCode FNFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNInitializePackage"
 /*@C
   FNInitializePackage - This function initializes everything in the FN package.
   It is called from PetscDLLibraryRegister() when using dynamic libraries, and
@@ -98,8 +86,6 @@ PetscErrorCode FNInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNCreate"
 /*@
    FNCreate - Creates an FN context.
 
@@ -128,6 +114,7 @@ PetscErrorCode FNCreate(MPI_Comm comm,FN *newfn)
 
   fn->alpha    = 1.0;
   fn->beta     = 1.0;
+  fn->method   = 0;
 
   fn->nw       = 0;
   fn->cw       = 0;
@@ -137,8 +124,6 @@ PetscErrorCode FNCreate(MPI_Comm comm,FN *newfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNSetOptionsPrefix"
 /*@C
    FNSetOptionsPrefix - Sets the prefix used for searching for all
    FN options in the database.
@@ -168,8 +153,6 @@ PetscErrorCode FNSetOptionsPrefix(FN fn,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNAppendOptionsPrefix"
 /*@C
    FNAppendOptionsPrefix - Appends to the prefix used for searching for all
    FN options in the database.
@@ -198,8 +181,6 @@ PetscErrorCode FNAppendOptionsPrefix(FN fn,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNGetOptionsPrefix"
 /*@C
    FNGetOptionsPrefix - Gets the prefix used for searching for all
    FN options in the database.
@@ -231,8 +212,6 @@ PetscErrorCode FNGetOptionsPrefix(FN fn,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNSetType"
 /*@C
    FNSetType - Selects the type for the FN object.
 
@@ -273,8 +252,6 @@ PetscErrorCode FNSetType(FN fn,FNType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNGetType"
 /*@C
    FNGetType - Gets the FN type name (as a string) from the FN context.
 
@@ -299,8 +276,6 @@ PetscErrorCode FNGetType(FN fn,FNType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNSetScale"
 /*@
    FNSetScale - Sets the scaling parameters that define the matematical function.
 
@@ -336,8 +311,6 @@ PetscErrorCode FNSetScale(FN fn,PetscScalar alpha,PetscScalar beta)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNGetScale"
 /*@
    FNGetScale - Gets the scaling parameters that define the matematical function.
 
@@ -363,12 +336,132 @@ PetscErrorCode FNGetScale(FN fn,PetscScalar *alpha,PetscScalar *beta)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunction"
+/*@
+   FNSetMethod - Selects the method to be used to evaluate functions of matrices.
+
+   Logically Collective on FN
+
+   Input Parameter:
++  fn   - the math function context
+-  meth - an index indentifying the method
+
+   Options Database Key:
+.  -fn_method <meth> - Sets the method
+
+   Notes:
+   In some FN types there are more than one algorithms available for computing
+   matrix functions. In that case, this function allows choosing the wanted method.
+
+   If meth is currently set to 0 (the default) and the input argument A of
+   FNEvaluateFunctionMat() is a symmetric/Hermitian matrix, then the computation
+   is done via the eigendecomposition of A, rather than with the general algorithm.
+
+   Level: intermediate
+
+.seealso: FNGetMethod(), FNEvaluateFunctionMat()
+@*/
+PetscErrorCode FNSetMethod(FN fn,PetscInt meth)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(fn,FN_CLASSID,1);
+  PetscValidLogicalCollectiveInt(fn,meth,2);
+  if (meth<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"The method must be a non-negative integer");
+  if (meth>FN_MAX_SOLVE) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Too large value for the method");
+  fn->method = meth;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   FNGetMethod - Gets the method currently used in the FN.
+
+   Not Collective
+
+   Input Parameter:
+.  fn - the math function context
+
+   Output Parameter:
+.  meth - identifier of the method
+
+   Level: intermediate
+
+.seealso: FNSetMethod()
+@*/
+PetscErrorCode FNGetMethod(FN fn,PetscInt *meth)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(fn,FN_CLASSID,1);
+  PetscValidPointer(meth,2);
+  *meth = fn->method;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   FNSetParallel - Selects the mode of operation in parallel runs.
+
+   Logically Collective on FN
+
+   Input Parameter:
++  fn    - the math function context
+-  pmode - the parallel mode
+
+   Options Database Key:
+.  -fn_parallel <mode> - Sets the parallel mode, either 'redundant' or 'synchronized'
+
+   Notes:
+   This is relevant only when the function is evaluated on a matrix, with
+   either FNEvaluateFunctionMat() or FNEvaluateFunctionMatVec().
+
+   In the 'redundant' parallel mode, all processes will make the computation
+   redundantly, starting from the same data, and producing the same result.
+   This result may be slightly different in the different processes if using a
+   multithreaded BLAS library, which may cause issues in ill-conditioned problems.
+
+   In the 'synchronized' parallel mode, only the first MPI process performs the
+   computation and then the computed matrix is broadcast to the other
+   processes in the communicator. This communication is done automatically at
+   the end of FNEvaluateFunctionMat() or FNEvaluateFunctionMatVec().
+
+   Level: advanced
+
+.seealso: FNEvaluateFunctionMat() or FNEvaluateFunctionMatVec(), FNGetParallel()
+@*/
+PetscErrorCode FNSetParallel(FN fn,FNParallelType pmode)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(fn,FN_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(fn,pmode,2);
+  fn->pmode = pmode;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   FNGetParallel - Gets the mode of operation in parallel runs.
+
+   Not Collective
+
+   Input Parameter:
+.  fn - the math function context
+
+   Output Parameter:
+.  pmode - the parallel mode
+
+   Level: advanced
+
+.seealso: FNSetParallel()
+@*/
+PetscErrorCode FNGetParallel(FN fn,FNParallelType *pmode)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(fn,FN_CLASSID,1);
+  PetscValidPointer(pmode,2);
+  *pmode = fn->pmode;
+  PetscFunctionReturn(0);
+}
+
 /*@
    FNEvaluateFunction - Computes the value of the function f(x) for a given x.
 
-   Logically Collective on FN
+   Not collective
 
    Input Parameters:
 +  fn - the math function context
@@ -392,7 +485,6 @@ PetscErrorCode FNEvaluateFunction(FN fn,PetscScalar x,PetscScalar *y)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
-  PetscValidLogicalCollectiveScalar(fn,x,2);
   PetscValidType(fn,1);
   PetscValidPointer(y,3);
   ierr = PetscLogEventBegin(FN_Evaluate,fn,0,0,0);CHKERRQ(ierr);
@@ -403,12 +495,10 @@ PetscErrorCode FNEvaluateFunction(FN fn,PetscScalar x,PetscScalar *y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateDerivative"
 /*@
    FNEvaluateDerivative - Computes the value of the derivative f'(x) for a given x.
 
-   Logically Collective on FN
+   Not Collective
 
    Input Parameters:
 +  fn - the math function context
@@ -432,7 +522,6 @@ PetscErrorCode FNEvaluateDerivative(FN fn,PetscScalar x,PetscScalar *y)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
-  PetscValidLogicalCollectiveScalar(fn,x,2);
   PetscValidType(fn,1);
   PetscValidPointer(y,3);
   ierr = PetscLogEventBegin(FN_Evaluate,fn,0,0,0);CHKERRQ(ierr);
@@ -443,8 +532,6 @@ PetscErrorCode FNEvaluateDerivative(FN fn,PetscScalar x,PetscScalar *y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMat_Sym_Private"
 static PetscErrorCode FNEvaluateFunctionMat_Sym_Private(FN fn,PetscScalar *As,PetscScalar *Bs,PetscInt m,PetscBool firstonly)
 {
 #if defined(PETSC_MISSING_LAPACK_SYEV) || defined(SLEPC_MISSING_LAPACK_LACPY)
@@ -484,13 +571,13 @@ static PetscErrorCode FNEvaluateFunctionMat_Sym_Private(FN fn,PetscScalar *As,Pe
 #else
   PetscStackCallBLAS("LAPACKsyev",LAPACKsyev_("V","L",&n,Q,&ld,eig,work,&lwork,&info));
 #endif
-  if (info) SETERRQ1(PetscObjectComm((PetscObject)fn),PETSC_ERR_LIB,"Error in Lapack xSYEV %i",info);
+  SlepcCheckLapackInfo("syev",info);
 
   /* W = f(Lambda)*Q' */
   for (i=0;i<n;i++) {
-    x = eig[i];
+    x = fn->alpha*eig[i];
     ierr = (*fn->ops->evaluatefunction)(fn,x,&y);CHKERRQ(ierr);  /* y = f(x) */
-    for (j=0;j<k;j++) W[i+j*ld] = Q[j+i*ld]*y;
+    for (j=0;j<k;j++) W[i+j*ld] = PetscConj(Q[j+i*ld])*fn->beta*y;
   }
   /* Bs = Q*W */
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&k,&n,&one,Q,&ld,W,&ld,&zero,Bs,&ld));
@@ -499,12 +586,11 @@ static PetscErrorCode FNEvaluateFunctionMat_Sym_Private(FN fn,PetscScalar *As,Pe
 #else
   ierr = PetscFree4(eig,Q,W,work);CHKERRQ(ierr);
 #endif
+  ierr = PetscLogFlops(9.0*n*n*n+2.0*n*n*n);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMat_Sym_Default"
 /*
    FNEvaluateFunctionMat_Sym_Default - given a symmetric matrix A,
    compute the matrix function as f(A)=Q*f(D)*Q' where the spectral
@@ -526,8 +612,57 @@ static PetscErrorCode FNEvaluateFunctionMat_Sym_Default(FN fn,Mat A,Mat B)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMat"
+PetscErrorCode FNEvaluateFunctionMat_Private(FN fn,Mat A,Mat B,PetscBool sync)
+{
+  PetscErrorCode ierr;
+  PetscBool      set,flg,symm=PETSC_FALSE;
+  PetscInt       m,n;
+  PetscMPIInt    size,rank;
+  PetscScalar    *pF;
+  Mat            M,F;
+
+  PetscFunctionBegin;
+  /* destination matrix */
+  F = B?B:A;
+
+  /* check symmetry of A */
+  ierr = MatIsHermitianKnown(A,&set,&flg);CHKERRQ(ierr);
+  symm = set? flg: PETSC_FALSE;
+
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)fn),&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)fn),&rank);CHKERRQ(ierr);
+  if (size==1 || fn->pmode==FN_PARALLEL_REDUNDANT || (fn->pmode==FN_PARALLEL_SYNCHRONIZED && !rank)) {
+    
+    ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+    if (symm && !fn->method) {  /* prefer diagonalization */
+      ierr = PetscInfo(fn,"Computing matrix function via diagonalization\n");CHKERRQ(ierr);
+      ierr = FNEvaluateFunctionMat_Sym_Default(fn,A,F);CHKERRQ(ierr);
+    } else {
+      /* scale argument */
+      if (fn->alpha!=(PetscScalar)1.0) {
+        ierr = FN_AllocateWorkMat(fn,A,&M);CHKERRQ(ierr);
+        ierr = MatScale(M,fn->alpha);CHKERRQ(ierr);
+      } else M = A;
+      if (fn->ops->evaluatefunctionmat[fn->method]) {
+        ierr = (*fn->ops->evaluatefunctionmat[fn->method])(fn,M,F);CHKERRQ(ierr);
+      } else SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"The specified method number does not exist for this FN");
+      if (fn->alpha!=(PetscScalar)1.0) {
+        ierr = FN_FreeWorkMat(fn,&M);CHKERRQ(ierr);
+      }
+      /* scale result */
+      ierr = MatScale(F,fn->beta);CHKERRQ(ierr);
+    }
+    ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  }
+  if (size>1 && fn->pmode==FN_PARALLEL_SYNCHRONIZED && sync) {  /* synchronize */
+    ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
+    ierr = MatDenseGetArray(F,&pF);CHKERRQ(ierr);
+    ierr = MPI_Bcast(pF,n*n,MPIU_SCALAR,0,PetscObjectComm((PetscObject)fn));CHKERRQ(ierr);
+    ierr = MatDenseRestoreArray(F,&pF);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
    FNEvaluateFunctionMat - Computes the value of the function f(A) for a given
    matrix A, where the result is also a matrix.
@@ -549,22 +684,21 @@ static PetscErrorCode FNEvaluateFunctionMat_Sym_Default(FN fn,Mat A,Mat B)
    function will perform an in-place computation, overwriting A with f(A).
 
    If A is known to be real symmetric or complex Hermitian then it is
-   recommended to set the appropriate flag with MatSetOption(), so that
-   a different algorithm that exploits symmetry is used.
+   recommended to set the appropriate flag with MatSetOption(), because
+   symmetry can sometimes be exploited by the algorithm.
 
    Scaling factors are taken into account, so the actual function evaluation
    will return beta*f(alpha*A).
 
    Level: advanced
 
-.seealso: FNEvaluateFunction(), FNEvaluateFunctionMatVec()
+.seealso: FNEvaluateFunction(), FNEvaluateFunctionMatVec(), FNSetMethod()
 @*/
 PetscErrorCode FNEvaluateFunctionMat(FN fn,Mat A,Mat B)
 {
   PetscErrorCode ierr;
-  PetscBool      match,set,flg,symm=PETSC_FALSE,inplace=PETSC_FALSE;
+  PetscBool      match,inplace=PETSC_FALSE;
   PetscInt       m,n,n1;
-  Mat            M,F;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
@@ -588,47 +722,13 @@ PetscErrorCode FNEvaluateFunctionMat(FN fn,Mat A,Mat B)
     if (n1!=n) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_SIZ,"Matrices A and B must have the same dimension");
   }
 
-  /* check symmetry of A */
-  ierr = MatIsHermitianKnown(A,&set,&flg);CHKERRQ(ierr);
-  symm = set? flg: PETSC_FALSE;
-
-  /* scale argument */
-  if (fn->alpha!=(PetscScalar)1.0) {
-    ierr = FN_AllocateWorkMat(fn,A,&M);CHKERRQ(ierr);
-    ierr = MatScale(M,fn->alpha);CHKERRQ(ierr);
-  } else M = A;
-
-  /* destination matrix */
-  F = inplace? A: B;
-
   /* evaluate matrix function */
   ierr = PetscLogEventBegin(FN_Evaluate,fn,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  if (symm) {
-    if (fn->ops->evaluatefunctionmatsym) {
-      ierr = (*fn->ops->evaluatefunctionmatsym)(fn,M,F);CHKERRQ(ierr);
-    } else {
-      ierr = FNEvaluateFunctionMat_Sym_Default(fn,M,F);CHKERRQ(ierr);
-    }
-  } else {
-    if (fn->ops->evaluatefunctionmat) {
-      ierr = (*fn->ops->evaluatefunctionmat)(fn,M,F);CHKERRQ(ierr);
-    } else SETERRQ1(PetscObjectComm((PetscObject)fn),PETSC_ERR_SUP,"Matrix function not implemented in FN type %s",((PetscObject)fn)->type_name);
-  }
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  ierr = FNEvaluateFunctionMat_Private(fn,A,B,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(FN_Evaluate,fn,0,0,0);CHKERRQ(ierr);
-
-  if (fn->alpha!=(PetscScalar)1.0) {
-    ierr = FN_FreeWorkMat(fn,&M);CHKERRQ(ierr);
-  }
-
-  /* scale result */
-  ierr = MatScale(F,fn->beta);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMatVec_Default"
 /*
    FNEvaluateFunctionMatVec_Default - computes the full matrix f(A)
    and then copies the first column.
@@ -640,16 +740,14 @@ static PetscErrorCode FNEvaluateFunctionMatVec_Default(FN fn,Mat A,Vec v)
 
   PetscFunctionBegin;
   ierr = FN_AllocateWorkMat(fn,A,&F);CHKERRQ(ierr);
-  if (fn->ops->evaluatefunctionmat) {
-    ierr = (*fn->ops->evaluatefunctionmat)(fn,A,F);CHKERRQ(ierr);
-  } else SETERRQ1(PetscObjectComm((PetscObject)fn),PETSC_ERR_SUP,"Matrix function not implemented in FN type %s",((PetscObject)fn)->type_name);
+  if (fn->ops->evaluatefunctionmat[fn->method]) {
+    ierr = (*fn->ops->evaluatefunctionmat[fn->method])(fn,A,F);CHKERRQ(ierr);
+  } else SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"The specified method number does not exist for this FN");
   ierr = MatGetColumnVector(F,v,0);CHKERRQ(ierr);
   ierr = FN_FreeWorkMat(fn,&F);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMatVec_Sym_Default"
 /*
    FNEvaluateFunctionMatVec_Sym_Default - given a symmetric matrix A,
    compute the matrix function as f(A)=Q*f(D)*Q' where the spectral
@@ -671,8 +769,58 @@ static PetscErrorCode FNEvaluateFunctionMatVec_Sym_Default(FN fn,Mat A,Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNEvaluateFunctionMatVec"
+PetscErrorCode FNEvaluateFunctionMatVec_Private(FN fn,Mat A,Vec v,PetscBool sync)
+{
+  PetscErrorCode ierr;
+  PetscBool      set,flg,symm=PETSC_FALSE;
+  PetscInt       m,n;
+  Mat            M;
+  PetscMPIInt    size,rank;
+  PetscScalar    *pv;
+
+  PetscFunctionBegin;
+  /* check symmetry of A */
+  ierr = MatIsHermitianKnown(A,&set,&flg);CHKERRQ(ierr);
+  symm = set? flg: PETSC_FALSE;
+
+  /* evaluate matrix function */
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)fn),&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)fn),&rank);CHKERRQ(ierr);
+  if (size==1 || fn->pmode==FN_PARALLEL_REDUNDANT || (fn->pmode==FN_PARALLEL_SYNCHRONIZED && !rank)) {
+    ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+    if (symm && !fn->method) {  /* prefer diagonalization */
+      ierr = PetscInfo(fn,"Computing matrix function via diagonalization\n");CHKERRQ(ierr);
+      ierr = FNEvaluateFunctionMatVec_Sym_Default(fn,A,v);CHKERRQ(ierr);
+    } else {
+      /* scale argument */
+      if (fn->alpha!=(PetscScalar)1.0) {
+        ierr = FN_AllocateWorkMat(fn,A,&M);CHKERRQ(ierr);
+        ierr = MatScale(M,fn->alpha);CHKERRQ(ierr);
+      } else M = A;
+      if (fn->ops->evaluatefunctionmatvec[fn->method]) {
+        ierr = (*fn->ops->evaluatefunctionmatvec[fn->method])(fn,M,v);CHKERRQ(ierr);
+      } else {
+        ierr = FNEvaluateFunctionMatVec_Default(fn,M,v);CHKERRQ(ierr);
+      }
+      if (fn->alpha!=(PetscScalar)1.0) {
+        ierr = FN_FreeWorkMat(fn,&M);CHKERRQ(ierr);
+      }
+      /* scale result */
+      ierr = VecScale(v,fn->beta);CHKERRQ(ierr);
+    }
+    ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  }
+
+  /* synchronize */
+  if (size>1 && fn->pmode==FN_PARALLEL_SYNCHRONIZED && sync) { 
+    ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
+    ierr = VecGetArray(v,&pv);CHKERRQ(ierr);
+    ierr = MPI_Bcast(pv,n,MPIU_SCALAR,0,PetscObjectComm((PetscObject)fn));CHKERRQ(ierr);
+    ierr = VecRestoreArray(v,&pv);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
    FNEvaluateFunctionMatVec - Computes the first column of the matrix f(A)
    for a given matrix A.
@@ -692,14 +840,13 @@ static PetscErrorCode FNEvaluateFunctionMatVec_Sym_Default(FN fn,Mat A,Vec v)
 
    Level: advanced
 
-.seealso: FNEvaluateFunction(), FNEvaluateFunctionMat()
+.seealso: FNEvaluateFunction(), FNEvaluateFunctionMat(), FNSetMethod()
 @*/
 PetscErrorCode FNEvaluateFunctionMatVec(FN fn,Mat A,Vec v)
 {
   PetscErrorCode ierr;
-  PetscBool      match,set,flg,symm=PETSC_FALSE;
+  PetscBool      match;
   PetscInt       m,n;
-  Mat            M;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
@@ -714,47 +861,12 @@ PetscErrorCode FNEvaluateFunctionMatVec(FN fn,Mat A,Vec v)
   if (m!=n) SETERRQ2(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_SIZ,"Mat A is not square (has %D rows, %D cols)",m,n);
   ierr = VecGetSize(v,&m);CHKERRQ(ierr);
   if (m!=n) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_SIZ,"Matrix A and vector v must have the same size");
-
-  /* check symmetry of A */
-  ierr = MatIsHermitianKnown(A,&set,&flg);CHKERRQ(ierr);
-  symm = set? flg: PETSC_FALSE;
-
-  /* scale argument */
-  if (fn->alpha!=(PetscScalar)1.0) {
-    ierr = FN_AllocateWorkMat(fn,A,&M);CHKERRQ(ierr);
-    ierr = MatScale(M,fn->alpha);CHKERRQ(ierr);
-  } else M = A;
-
-  /* evaluate matrix function */
   ierr = PetscLogEventBegin(FN_Evaluate,fn,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  if (symm) {
-    if (fn->ops->evaluatefunctionmatvecsym) {
-      ierr = (*fn->ops->evaluatefunctionmatvecsym)(fn,M,v);CHKERRQ(ierr);
-    } else {
-      ierr = FNEvaluateFunctionMatVec_Sym_Default(fn,M,v);CHKERRQ(ierr);
-    }
-  } else {
-    if (fn->ops->evaluatefunctionmatvec) {
-      ierr = (*fn->ops->evaluatefunctionmatvec)(fn,M,v);CHKERRQ(ierr);
-    } else {
-      ierr = FNEvaluateFunctionMatVec_Default(fn,M,v);CHKERRQ(ierr);
-    }
-  }
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  ierr = FNEvaluateFunctionMatVec_Private(fn,A,v,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(FN_Evaluate,fn,0,0,0);CHKERRQ(ierr);
-
-  if (fn->alpha!=(PetscScalar)1.0) {
-    ierr = FN_FreeWorkMat(fn,&M);CHKERRQ(ierr);
-  }
-
-  /* scale result */
-  ierr = VecScale(v,fn->beta);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNSetFromOptions"
 /*@
    FNSetFromOptions - Sets FN options from the options database.
 
@@ -773,8 +885,9 @@ PetscErrorCode FNSetFromOptions(FN fn)
   PetscErrorCode ierr;
   char           type[256];
   PetscScalar    array[2];
-  PetscInt       k;
+  PetscInt       k,meth;
   PetscBool      flg;
+  FNParallelType pmode;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
@@ -783,11 +896,7 @@ PetscErrorCode FNSetFromOptions(FN fn)
     ierr = PetscOptionsFList("-fn_type","Math function type","FNSetType",FNList,(char*)(((PetscObject)fn)->type_name?((PetscObject)fn)->type_name:FNRATIONAL),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = FNSetType(fn,type);CHKERRQ(ierr);
-    }
-    /*
-      Set the type if it was never set.
-    */
-    if (!((PetscObject)fn)->type_name) {
+    } else if (!((PetscObject)fn)->type_name) {
       ierr = FNSetType(fn,FNRATIONAL);CHKERRQ(ierr);
     }
 
@@ -799,6 +908,12 @@ PetscErrorCode FNSetFromOptions(FN fn)
       ierr = FNSetScale(fn,array[0],array[1]);CHKERRQ(ierr);
     }
 
+    ierr = PetscOptionsInt("-fn_method","Method to be used for computing matrix functions","FNSetMethod",fn->method,&meth,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = FNSetMethod(fn,meth);CHKERRQ(ierr); }
+
+    ierr = PetscOptionsEnum("-fn_parallel","Operation mode in parallel runs","FNSetParallel",FNParallelTypes,(PetscEnum)fn->pmode,(PetscEnum*)&pmode,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = FNSetParallel(fn,pmode);CHKERRQ(ierr); }
+
     if (fn->ops->setfromoptions) {
       ierr = (*fn->ops->setfromoptions)(PetscOptionsObject,fn);CHKERRQ(ierr);
     }
@@ -807,8 +922,6 @@ PetscErrorCode FNSetFromOptions(FN fn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNView"
 /*@C
    FNView - Prints the FN data structure.
 
@@ -835,6 +948,7 @@ PetscErrorCode FNView(FN fn,PetscViewer viewer)
 {
   PetscBool      isascii;
   PetscErrorCode ierr;
+  PetscMPIInt    size;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
@@ -844,6 +958,10 @@ PetscErrorCode FNView(FN fn,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)fn,viewer);CHKERRQ(ierr);
+    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)fn),&size);CHKERRQ(ierr);
+    if (size>1) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",FNParallelTypes[fn->pmode]);CHKERRQ(ierr);
+    }
     if (fn->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = (*fn->ops->view)(fn,viewer);CHKERRQ(ierr);
@@ -853,8 +971,6 @@ PetscErrorCode FNView(FN fn,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNDuplicate"
 /*@
    FNDuplicate - Duplicates a math function, copying all parameters, possibly with a
    different communicator.
@@ -863,11 +979,15 @@ PetscErrorCode FNView(FN fn,PetscViewer viewer)
 
    Input Parameters:
 +  fn   - the math function context
--  comm - MPI communicator (may be NULL)
+-  comm - MPI communicator
 
    Output Parameter:
 .  newfn - location to put the new FN context
 
+   Note:
+   In order to use the same MPI communicator as in the original object,
+   use PetscObjectComm((PetscObject)fn).
+
    Level: developer
 
 .seealso: FNCreate()
@@ -877,25 +997,28 @@ PetscErrorCode FNDuplicate(FN fn,MPI_Comm comm,FN *newfn)
   PetscErrorCode ierr;
   FNType         type;
   PetscScalar    alpha,beta;
+  PetscInt       meth;
+  FNParallelType ptype;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidType(fn,1);
   PetscValidPointer(newfn,3);
-  if (!comm) comm = PetscObjectComm((PetscObject)fn);
   ierr = FNCreate(comm,newfn);CHKERRQ(ierr);
   ierr = FNGetType(fn,&type);CHKERRQ(ierr);
   ierr = FNSetType(*newfn,type);CHKERRQ(ierr);
   ierr = FNGetScale(fn,&alpha,&beta);CHKERRQ(ierr);
   ierr = FNSetScale(*newfn,alpha,beta);CHKERRQ(ierr);
+  ierr = FNGetMethod(fn,&meth);CHKERRQ(ierr);
+  ierr = FNSetMethod(*newfn,meth);CHKERRQ(ierr);
+  ierr = FNGetParallel(fn,&ptype);CHKERRQ(ierr);
+  ierr = FNSetParallel(*newfn,ptype);CHKERRQ(ierr);
   if (fn->ops->duplicate) {
     ierr = (*fn->ops->duplicate)(fn,comm,newfn);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNDestroy"
 /*@
    FNDestroy - Destroys FN context that was created with FNCreate().
 
@@ -925,8 +1048,6 @@ PetscErrorCode FNDestroy(FN *fn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FNRegister"
 /*@C
    FNRegister - Adds a mathematical function to the FN package.
 
