@@ -1,25 +1,15 @@
 /*
-   Basic routines
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic RG routines
+*/
 
 #include <slepc/private/rgimpl.h>      /*I "slepcrg.h" I*/
 
@@ -28,8 +18,6 @@ PetscBool         RGRegisterAllCalled = PETSC_FALSE;
 PetscClassId      RG_CLASSID = 0;
 static PetscBool  RGPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "RGFinalizePackage"
 /*@C
    RGFinalizePackage - This function destroys everything in the Slepc interface
    to the RG package. It is called from SlepcFinalize().
@@ -49,8 +37,6 @@ PetscErrorCode RGFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGInitializePackage"
 /*@C
   RGInitializePackage - This function initializes everything in the RG package.
   It is called from PetscDLLibraryRegister() when using dynamic libraries, and
@@ -94,8 +80,6 @@ PetscErrorCode RGInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGCreate"
 /*@
    RGCreate - Creates an RG context.
 
@@ -130,8 +114,6 @@ PetscErrorCode RGCreate(MPI_Comm comm,RG *newrg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGSetOptionsPrefix"
 /*@C
    RGSetOptionsPrefix - Sets the prefix used for searching for all
    RG options in the database.
@@ -161,8 +143,6 @@ PetscErrorCode RGSetOptionsPrefix(RG rg,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGAppendOptionsPrefix"
 /*@C
    RGAppendOptionsPrefix - Appends to the prefix used for searching for all
    RG options in the database.
@@ -191,8 +171,6 @@ PetscErrorCode RGAppendOptionsPrefix(RG rg,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGGetOptionsPrefix"
 /*@C
    RGGetOptionsPrefix - Gets the prefix used for searching for all
    RG options in the database.
@@ -224,8 +202,6 @@ PetscErrorCode RGGetOptionsPrefix(RG rg,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGSetType"
 /*@C
    RGSetType - Selects the type for the RG object.
 
@@ -262,8 +238,6 @@ PetscErrorCode RGSetType(RG rg,RGType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGGetType"
 /*@C
    RGGetType - Gets the RG type name (as a string) from the RG context.
 
@@ -288,8 +262,6 @@ PetscErrorCode RGGetType(RG rg,RGType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGSetFromOptions"
 /*@
    RGSetFromOptions - Sets RG options from the options database.
 
@@ -317,19 +289,14 @@ PetscErrorCode RGSetFromOptions(RG rg)
     ierr = PetscOptionsFList("-rg_type","Region type","RGSetType",RGList,(char*)(((PetscObject)rg)->type_name?((PetscObject)rg)->type_name:RGINTERVAL),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = RGSetType(rg,type);CHKERRQ(ierr);
-    }
-    /*
-      Set the type if it was never set.
-    */
-    if (!((PetscObject)rg)->type_name) {
+    } else if (!((PetscObject)rg)->type_name) {
       ierr = RGSetType(rg,RGINTERVAL);CHKERRQ(ierr);
     }
 
     ierr = PetscOptionsBool("-rg_complement","Whether region is complemented or not","RGSetComplement",rg->complement,&rg->complement,NULL);CHKERRQ(ierr);
+
     ierr = PetscOptionsReal("-rg_scale","Scaling factor","RGSetScale",1.0,&sfactor,&flg);CHKERRQ(ierr);
-    if (flg) {
-      ierr = RGSetScale(rg,sfactor);CHKERRQ(ierr);
-    }
+    if (flg) { ierr = RGSetScale(rg,sfactor);CHKERRQ(ierr); }
 
     if (rg->ops->setfromoptions) {
       ierr = (*rg->ops->setfromoptions)(PetscOptionsObject,rg);CHKERRQ(ierr);
@@ -339,8 +306,6 @@ PetscErrorCode RGSetFromOptions(RG rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGView"
 /*@C
    RGView - Prints the RG data structure.
 
@@ -391,8 +356,6 @@ PetscErrorCode RGView(RG rg,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGIsTrivial"
 /*@
    RGIsTrivial - Whether it is the trivial region (whole complex plane).
 
@@ -422,8 +385,6 @@ PetscErrorCode RGIsTrivial(RG rg,PetscBool *trivial)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGCheckInside"
 /*@
    RGCheckInside - Determines if a set of given points are inside the region or not.
 
@@ -482,8 +443,6 @@ PetscErrorCode RGCheckInside(RG rg,PetscInt n,PetscScalar *ar,PetscScalar *ai,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGComputeContour"
 /*@
    RGComputeContour - Computes the coordinates of several points lying in the
    contour of the region.
@@ -494,13 +453,13 @@ PetscErrorCode RGCheckInside(RG rg,PetscInt n,PetscScalar *ar,PetscScalar *ai,Pe
 +  rg - the region context
 -  n  - number of points to compute
 
-   Output Parameter:
+   Output Parameters:
 +  cr - location to store real parts
 -  ci - location to store imaginary parts
 
    Level: intermediate
 @*/
-PetscErrorCode RGComputeContour(RG rg,PetscInt n,PetscScalar *cr,PetscScalar *ci)
+PetscErrorCode RGComputeContour(RG rg,PetscInt n,PetscScalar cr[],PetscScalar ci[])
 {
   PetscInt       i;
   PetscErrorCode ierr;
@@ -512,6 +471,7 @@ PetscErrorCode RGComputeContour(RG rg,PetscInt n,PetscScalar *cr,PetscScalar *ci
 #if !defined(PETSC_USE_COMPLEX)
   PetscValidPointer(ci,4);
 #endif
+  if (rg->complement) SETERRQ(PetscObjectComm((PetscObject)rg),PETSC_ERR_SUP,"Cannot compute contour of region with complement flag set");
   ierr = (*rg->ops->computecontour)(rg,n,cr,ci);CHKERRQ(ierr);
   for (i=0;i<n;i++) {
     cr[i] *= rg->sfactor;
@@ -520,8 +480,51 @@ PetscErrorCode RGComputeContour(RG rg,PetscInt n,PetscScalar *cr,PetscScalar *ci
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGSetComplement"
+/*@
+   RGComputeBoundingBox - Determines the endpoints of a rectangle in the complex plane that
+   contains the region.
+
+   Not Collective
+
+   Input Parameters:
+.  rg - the region context
+
+   Output Parameters:
++  a,b - endpoints of the bounding box in the real axis
+-  c,d - endpoints of the bounding box in the imaginary axis
+
+   Notes:
+   The bounding box is defined as [a,b]x[c,d]. In regions that are not bounded (e.g. an
+   open interval) or with the complement flag set, it makes no sense to compute a bounding
+   box, so the return values are infinite.
+
+   Level: intermediate
+
+.seealso: RGSetComplement()
+@*/
+PetscErrorCode RGComputeBoundingBox(RG rg,PetscReal *a,PetscReal *b,PetscReal *c,PetscReal *d)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(rg,RG_CLASSID,1);
+  PetscValidType(rg,1);
+
+  if (rg->complement) {  /* cannot compute bounding box */
+    if (a) *a = -PETSC_MAX_REAL;
+    if (b) *b =  PETSC_MAX_REAL;
+    if (c) *c = -PETSC_MAX_REAL;
+    if (d) *d =  PETSC_MAX_REAL;
+  } else {
+    ierr = (*rg->ops->computebbox)(rg,a,b,c,d);CHKERRQ(ierr);
+    if (a && *a!=-PETSC_MAX_REAL) *a *= rg->sfactor;
+    if (b && *b!= PETSC_MAX_REAL) *b *= rg->sfactor;
+    if (c && *c!=-PETSC_MAX_REAL) *c *= rg->sfactor;
+    if (d && *d!= PETSC_MAX_REAL) *d *= rg->sfactor;
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
    RGSetComplement - Sets a flag to indicate that the region is the complement
    of the specified one.
@@ -548,8 +551,6 @@ PetscErrorCode RGSetComplement(RG rg,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGGetComplement"
 /*@
    RGGetComplement - Gets a flag that that indicates whether the region
    is complemented or not.
@@ -575,8 +576,6 @@ PetscErrorCode RGGetComplement(RG rg,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGSetScale"
 /*@
    RGSetScale - Sets the scaling factor to be used when checking that a
    point is inside the region and when computing the contour.
@@ -607,8 +606,6 @@ PetscErrorCode RGSetScale(RG rg,PetscReal sfactor)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGGetScale"
 /*@
    RGGetScale - Gets the scaling factor.
 
@@ -633,8 +630,6 @@ PetscErrorCode RGGetScale(RG rg,PetscReal *sfactor)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGPushScale"
 /*@
    RGPushScale - Sets an additional scaling factor, that will multiply the
    user-defined scaling factor.
@@ -667,8 +662,6 @@ PetscErrorCode RGPushScale(RG rg,PetscReal sfactor)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGPopScale"
 /*@
    RGPopScale - Pops the scaling factor set with RGPushScale().
 
@@ -691,8 +684,6 @@ PetscErrorCode RGPopScale(RG rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGDestroy"
 /*@
    RGDestroy - Destroys RG context that was created with RGCreate().
 
@@ -718,8 +709,6 @@ PetscErrorCode RGDestroy(RG *rg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RGRegister"
 /*@C
    RGRegister - Adds a region to the RG package.
 
