@@ -1,31 +1,19 @@
 /*
-    Shift spectral transformation, applies (A + sigma I) as operator, or
-    inv(B)(A + sigma B) for generalized problems
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Shift spectral transformation, applies (A + sigma I) as operator, or
+   inv(B)(A + sigma B) for generalized problems
+*/
 
 #include <slepc/private/stimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "STApply_Shift"
 PetscErrorCode STApply_Shift(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -33,8 +21,8 @@ PetscErrorCode STApply_Shift(ST st,Vec x,Vec y)
   PetscFunctionBegin;
   if (st->nmat>1) {
     /* generalized eigenproblem: y = B^-1 (A - sB) x */
-    ierr = MatMult(st->T[0],x,st->w);CHKERRQ(ierr);
-    ierr = STMatSolve(st,st->w,y);CHKERRQ(ierr);
+    ierr = MatMult(st->T[0],x,st->work[0]);CHKERRQ(ierr);
+    ierr = STMatSolve(st,st->work[0],y);CHKERRQ(ierr);
   } else {
     /* standard eigenproblem: y = (A - sI) x */
     ierr = MatMult(st->T[0],x,y);CHKERRQ(ierr);
@@ -42,8 +30,6 @@ PetscErrorCode STApply_Shift(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STApplyTranspose_Shift"
 PetscErrorCode STApplyTranspose_Shift(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -51,8 +37,8 @@ PetscErrorCode STApplyTranspose_Shift(ST st,Vec x,Vec y)
   PetscFunctionBegin;
   if (st->nmat>1) {
     /* generalized eigenproblem: y = (A - sB)^T B^-T  x */
-    ierr = STMatSolveTranspose(st,x,st->w);CHKERRQ(ierr);
-    ierr = MatMultTranspose(st->T[0],st->w,y);CHKERRQ(ierr);
+    ierr = STMatSolveTranspose(st,x,st->work[0]);CHKERRQ(ierr);
+    ierr = MatMultTranspose(st->T[0],st->work[0],y);CHKERRQ(ierr);
   } else {
     /* standard eigenproblem: y = (A^T - sI) x */
     ierr = MatMultTranspose(st->T[0],x,y);CHKERRQ(ierr);
@@ -60,8 +46,6 @@ PetscErrorCode STApplyTranspose_Shift(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STBackTransform_Shift"
 PetscErrorCode STBackTransform_Shift(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
   PetscInt j;
@@ -73,8 +57,6 @@ PetscErrorCode STBackTransform_Shift(ST st,PetscInt n,PetscScalar *eigr,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STPostSolve_Shift"
 PetscErrorCode STPostSolve_Shift(ST st)
 {
   PetscErrorCode ierr;
@@ -92,8 +74,6 @@ PetscErrorCode STPostSolve_Shift(ST st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetUp_Shift"
 PetscErrorCode STSetUp_Shift(ST st)
 {
   PetscErrorCode ierr;
@@ -102,7 +82,7 @@ PetscErrorCode STSetUp_Shift(ST st)
 
   PetscFunctionBegin;
   if (st->nmat>1) {
-    ierr = ST_AllocateWorkVec(st);CHKERRQ(ierr);
+    ierr = STSetWorkVecs(st,1);CHKERRQ(ierr);
   }
   if (nmat<3 || st->transform) {
     if (nmat>2) {
@@ -136,14 +116,11 @@ PetscErrorCode STSetUp_Shift(ST st)
     if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
     ierr = STCheckFactorPackage(st);CHKERRQ(ierr);
     ierr = KSPSetOperators(st->ksp,st->P,st->P);CHKERRQ(ierr);
-    ierr = KSPSetErrorIfNotConverged(st->ksp,PETSC_TRUE);CHKERRQ(ierr);
     ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetShift_Shift"
 PetscErrorCode STSetShift_Shift(ST st,PetscScalar newshift)
 {
   PetscErrorCode ierr;
@@ -168,36 +145,6 @@ PetscErrorCode STSetShift_Shift(ST st,PetscScalar newshift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetFromOptions_Shift"
-PetscErrorCode STSetFromOptions_Shift(PetscOptionItems *PetscOptionsObject,ST st)
-{
-  PetscErrorCode ierr;
-  PC             pc;
-  PCType         pctype;
-  KSPType        ksptype;
-
-  PetscFunctionBegin;
-  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
-  ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
-  ierr = KSPGetType(st->ksp,&ksptype);CHKERRQ(ierr);
-  ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
-  if (!pctype && !ksptype) {
-    if (st->shift_matrix == ST_MATMODE_SHELL) {
-      /* in shell mode use GMRES with Jacobi as the default */
-      ierr = KSPSetType(st->ksp,KSPGMRES);CHKERRQ(ierr);
-      ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
-    } else {
-      /* use direct solver as default */
-      ierr = KSPSetType(st->ksp,KSPPREONLY);CHKERRQ(ierr);
-      ierr = PCSetType(pc,PCLU);CHKERRQ(ierr);
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "STCreate_Shift"
 PETSC_EXTERN PetscErrorCode STCreate_Shift(ST st)
 {
   PetscFunctionBegin;
@@ -206,8 +153,8 @@ PETSC_EXTERN PetscErrorCode STCreate_Shift(ST st)
   st->ops->applytrans      = STApplyTranspose_Shift;
   st->ops->postsolve       = STPostSolve_Shift;
   st->ops->backtransform   = STBackTransform_Shift;
-  st->ops->setfromoptions  = STSetFromOptions_Shift;
   st->ops->setup           = STSetUp_Shift;
   st->ops->setshift        = STSetShift_Shift;
+  st->ops->setdefaultksp   = STSetDefaultKSP_Default;
   PetscFunctionReturn(0);
 }
