@@ -1,25 +1,15 @@
 /*
-    The ST (spectral transformation) interface routines, callable by users.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   The ST interface routines, callable by users
+*/
 
 #include <slepc/private/stimpl.h>            /*I "slepcst.h" I*/
 
@@ -27,8 +17,8 @@ PetscClassId     ST_CLASSID = 0;
 PetscLogEvent    ST_SetUp = 0,ST_Apply = 0,ST_ApplyTranspose = 0,ST_MatSetUp = 0,ST_MatMult = 0,ST_MatMultTranspose = 0,ST_MatSolve = 0,ST_MatSolveTranspose = 0;
 static PetscBool STPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "STFinalizePackage"
+const char *STMatModes[] = {"COPY","INPLACE","SHELL","STMatMode","ST_MATMODE_",0};
+
 /*@C
    STFinalizePackage - This function destroys everything in the Slepc interface
    to the ST package. It is called from SlepcFinalize().
@@ -48,8 +38,6 @@ PetscErrorCode STFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STInitializePackage"
 /*@C
    STInitializePackage - This function initializes everything in the ST package.
    It is called from PetscDLLibraryRegister() when using dynamic libraries, and
@@ -102,10 +90,9 @@ PetscErrorCode STInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STReset"
 /*@
-   STReset - Resets the ST context and removes any allocated objects.
+   STReset - Resets the ST context to the initial state (prior to setup)
+   and destroys any allocated Vecs and Mats.
 
    Collective on ST
 
@@ -121,18 +108,22 @@ PetscErrorCode STReset(ST st)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  if (st) PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  if (!st) PetscFunctionReturn(0);
   if (st->ops->reset) { ierr = (*st->ops->reset)(st);CHKERRQ(ierr); }
   if (st->ksp) { ierr = KSPReset(st->ksp);CHKERRQ(ierr); }
   ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->T);CHKERRQ(ierr);
-  ierr = VecDestroy(&st->w);CHKERRQ(ierr);
+  ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->A);CHKERRQ(ierr);
+  ierr = PetscFree(st->Astate);CHKERRQ(ierr);
+  ierr = MatDestroy(&st->P);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(st->nwork,&st->work);CHKERRQ(ierr);
+  st->nwork = 0;
   ierr = VecDestroy(&st->wb);CHKERRQ(ierr);
+  ierr = VecDestroy(&st->D);CHKERRQ(ierr);
   st->state = ST_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STDestroy"
 /*@
    STDestroy - Destroys ST context that was created with STCreate().
 
@@ -154,18 +145,12 @@ PetscErrorCode STDestroy(ST *st)
   PetscValidHeaderSpecific(*st,ST_CLASSID,1);
   if (--((PetscObject)(*st))->refct > 0) { *st = 0; PetscFunctionReturn(0); }
   ierr = STReset(*st);CHKERRQ(ierr);
-  ierr = MatDestroyMatrices(PetscMax(2,(*st)->nmat),&(*st)->A);CHKERRQ(ierr);
-  ierr = PetscFree((*st)->Astate);CHKERRQ(ierr);
   if ((*st)->ops->destroy) { ierr = (*(*st)->ops->destroy)(*st);CHKERRQ(ierr); }
-  ierr = MatDestroy(&(*st)->P);CHKERRQ(ierr);
-  ierr = VecDestroy(&(*st)->D);CHKERRQ(ierr);
   ierr = KSPDestroy(&(*st)->ksp);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(st);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STCreate"
 /*@
    STCreate - Creates a spectral transformation context.
 
@@ -205,7 +190,8 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   st->transform    = PETSC_FALSE;
 
   st->ksp          = NULL;
-  st->w            = NULL;
+  st->nwork        = 0;
+  st->work         = NULL;
   st->D            = NULL;
   st->wb           = NULL;
   st->data         = NULL;
@@ -215,10 +201,8 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetOperators"
 /*@
-   STSetOperators - Sets the matrices associated with the eigenvalue problem.
+   STSetMatrices - Sets the matrices associated with the eigenvalue problem.
 
    Collective on ST and Mat
 
@@ -233,9 +217,9 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
 
    Level: intermediate
 
-.seealso: STGetOperators(), STGetNumMatrices(), STSetUp(), STReset()
+.seealso: STGetMatrix(), STGetNumMatrices(), STSetUp(), STReset()
  @*/
-PetscErrorCode STSetOperators(ST st,PetscInt n,Mat A[])
+PetscErrorCode STSetMatrices(ST st,PetscInt n,Mat A[])
 {
   PetscInt       i;
   PetscErrorCode ierr;
@@ -244,7 +228,7 @@ PetscErrorCode STSetOperators(ST st,PetscInt n,Mat A[])
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidLogicalCollectiveInt(st,n,2);
-  if (n <= 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more matrices, you have %D",n);
+  if (n <= 0) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more matrices, you have %D",n);
   PetscValidPointer(A,3);
   PetscCheckSameComm(st,1,*A,3);
   if (st->state) {
@@ -279,10 +263,8 @@ PetscErrorCode STSetOperators(ST st,PetscInt n,Mat A[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STGetOperators"
 /*@
-   STGetOperators - Gets the matrices associated with the original eigensystem.
+   STGetMatrix - Gets the matrices associated with the original eigensystem.
 
    Not collective, though parallel Mats are returned if the ST is parallel
 
@@ -295,25 +277,23 @@ PetscErrorCode STSetOperators(ST st,PetscInt n,Mat A[])
 
    Level: intermediate
 
-.seealso: STSetOperators(), STGetNumMatrices()
+.seealso: STSetMatrices(), STGetNumMatrices()
 @*/
-PetscErrorCode STGetOperators(ST st,PetscInt k,Mat *A)
+PetscErrorCode STGetMatrix(ST st,PetscInt k,Mat *A)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidLogicalCollectiveInt(st,k,2);
   PetscValidPointer(A,3);
   STCheckMatrices(st,1);
-  if (k<0 || k>=st->nmat) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",st->nmat-1);
-  if (((PetscObject)st->A[k])->state!=st->Astate[k]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot retrieve original matrices (have been modified)");
+  if (k<0 || k>=st->nmat) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",st->nmat-1);
+  if (((PetscObject)st->A[k])->state!=st->Astate[k]) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_SUP,"Cannot retrieve original matrices (have been modified)");
   *A = st->A[k];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STGetTOperators"
 /*@
-   STGetTOperators - Gets the matrices associated with the transformed eigensystem.
+   STGetMatrixTransformed - Gets the matrices associated with the transformed eigensystem.
 
    Not collective, though parallel Mats are returned if the ST is parallel
 
@@ -326,23 +306,21 @@ PetscErrorCode STGetOperators(ST st,PetscInt k,Mat *A)
 
    Level: developer
 
-.seealso: STGetOperators(), STGetNumMatrices()
+.seealso: STGetMatrix(), STGetNumMatrices()
 @*/
-PetscErrorCode STGetTOperators(ST st,PetscInt k,Mat *T)
+PetscErrorCode STGetMatrixTransformed(ST st,PetscInt k,Mat *T)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidLogicalCollectiveInt(st,k,2);
   PetscValidPointer(T,3);
   STCheckMatrices(st,1);
-  if (k<0 || k>=st->nmat) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",st->nmat-1);
-  if (!st->T) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_POINTER,"There are no transformed matrices");
+  if (k<0 || k>=st->nmat) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",st->nmat-1);
+  if (!st->T) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_POINTER,"There are no transformed matrices");
   *T = st->T[k];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STGetNumMatrices"
 /*@
    STGetNumMatrices - Returns the number of matrices stored in the ST.
 
@@ -352,11 +330,11 @@ PetscErrorCode STGetTOperators(ST st,PetscInt k,Mat *T)
 .  st - the spectral transformation context
 
    Output Parameters:
-.  n - the number of matrices passed in STSetOperators()
+.  n - the number of matrices passed in STSetMatrices()
 
    Level: intermediate
 
-.seealso: STSetOperators()
+.seealso: STSetMatrices()
 @*/
 PetscErrorCode STGetNumMatrices(ST st,PetscInt *n)
 {
@@ -367,8 +345,34 @@ PetscErrorCode STGetNumMatrices(ST st,PetscInt *n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetShift"
+/*@
+   STResetMatrixState - Resets the stored state of the matrices in the ST.
+
+   Logically Collective on ST
+
+   Input Parameter:
+.  st - the spectral transformation context
+
+   Note:
+   This is useful in solvers where the user matrices are modified during
+   the computation, as in nonlinear inverse iteration. The effect is that
+   STGetMatrix() will retrieve the modified matrices as if they were
+   the matrices originally provided by the user.
+
+   Level: developer
+
+.seealso: STGetMatrix(), EPSPowerSetNonlinear()
+@*/
+PetscErrorCode STResetMatrixState(ST st)
+{
+  PetscInt i;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  for (i=0;i<st->nmat;i++) st->Astate[i] = ((PetscObject)st->A[i])->state;
+  PetscFunctionReturn(0);
+}
+
 /*@
    STSetShift - Sets the shift associated with the spectral transformation.
 
@@ -395,6 +399,7 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidType(st,1);
   PetscValidLogicalCollectiveScalar(st,shift,2);
   if (st->state==ST_STATE_SETUP && st->sigma != shift) {
     if (st->ops->setshift) {
@@ -406,8 +411,6 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STGetShift"
 /*@
    STGetShift - Gets the shift associated with the spectral transformation.
 
@@ -432,8 +435,6 @@ PetscErrorCode STGetShift(ST st,PetscScalar* shift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetDefaultShift"
 /*@
    STSetDefaultShift - Sets the value of the shift that should be employed if
    the user did not specify one.
@@ -457,8 +458,6 @@ PetscErrorCode STSetDefaultShift(ST st,PetscScalar defaultshift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STScaleShift"
 /*@
    STScaleShift - Multiply the shift with a given factor.
 
@@ -485,8 +484,6 @@ PetscErrorCode STScaleShift(ST st,PetscScalar factor)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetBalanceMatrix"
 /*@
    STSetBalanceMatrix - Sets the diagonal matrix to be used for balancing.
 
@@ -521,8 +518,6 @@ PetscErrorCode STSetBalanceMatrix(ST st,Vec D)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STGetBalanceMatrix"
 /*@
    STGetBalanceMatrix - Gets the balance matrix used by the spectral transformation.
 
@@ -550,8 +545,6 @@ PetscErrorCode STGetBalanceMatrix(ST st,Vec *D)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STMatCreateVecs"
 /*@C
    STMatCreateVecs - Get vector(s) compatible with the ST matrices.
 
@@ -576,8 +569,33 @@ PetscErrorCode STMatCreateVecs(ST st,Vec *right,Vec *left)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STMatGetSize"
+/*@C
+   STMatCreateVecsEmpty - Get vector(s) compatible with the ST matrices, i.e. with the same
+   parallel layout, but without internal array.
+
+   Collective on ST
+
+   Input Parameter:
+.  st - the spectral transformation context
+
+   Output Parameters:
++  right - (optional) vector that the matrix can be multiplied against
+-  left  - (optional) vector that the matrix vector product can be stored in
+
+   Level: developer
+
+.seealso: MatCreateVecsEmpty()
+@*/
+PetscErrorCode STMatCreateVecsEmpty(ST st,Vec *right,Vec *left)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  STCheckMatrices(st,1);
+  ierr = MatCreateVecsEmpty(st->A[0],right,left);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    STMatGetSize - Returns the number of rows and columns of the ST matrices.
 
@@ -602,8 +620,6 @@ PetscErrorCode STMatGetSize(ST st,PetscInt *m,PetscInt *n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STMatGetLocalSize"
 /*@
    STMatGetLocalSize - Returns the number of local rows and columns of the ST matrices.
 
@@ -626,10 +642,8 @@ PetscErrorCode STMatGetLocalSize(ST st,PetscInt *m,PetscInt *n)
   STCheckMatrices(st,1);
   ierr = MatGetLocalSize(st->A[0],m,n);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-} 
+}
 
-#undef __FUNCT__
-#define __FUNCT__ "STSetOptionsPrefix"
 /*@C
    STSetOptionsPrefix - Sets the prefix used for searching for all
    ST options in the database.
@@ -662,8 +676,6 @@ PetscErrorCode STSetOptionsPrefix(ST st,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STAppendOptionsPrefix"
 /*@C
    STAppendOptionsPrefix - Appends to the prefix used for searching for all
    ST options in the database.
@@ -696,8 +708,6 @@ PetscErrorCode STAppendOptionsPrefix(ST st,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STGetOptionsPrefix"
 /*@C
    STGetOptionsPrefix - Gets the prefix used for searching for all
    ST options in the database.
@@ -729,8 +739,6 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STView"
 /*@C
    STView - Prints the ST data structure.
 
@@ -759,7 +767,7 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   STType         cstr;
-  const char*    pat;
+  const char*    pat=NULL;
   char           str[50];
   PetscBool      isascii,isstring,flg;
 
@@ -796,7 +804,6 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
         case SAME_NONZERO_PATTERN:      pat = "same nonzero pattern";break;
         case DIFFERENT_NONZERO_PATTERN: pat = "different nonzero pattern";break;
         case SUBSET_NONZERO_PATTERN:    pat = "subset nonzero pattern";break;
-        default: SETERRQ(PetscObjectComm((PetscObject)st),1,"Wrong structure flag");
       }
       ierr = PetscViewerASCIIPrintf(viewer,"  all matrices have %s\n",pat);CHKERRQ(ierr);
     }
@@ -818,8 +825,6 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "STRegister"
 /*@C
    STRegister - Adds a method to the spectral transformation package.
 
