@@ -1,21 +1,10 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
@@ -26,10 +15,12 @@ static MPI_Datatype MPIU_NORM2=0, MPIU_NORM1_AND_2=0;
 static MPI_Op MPIU_NORM2_SUM=0;
 static PetscBool VecCompInitialized = PETSC_FALSE;
 
-/* Private inline functions */
-PETSC_STATIC_INLINE void SumNorm2(PetscReal *,PetscReal *,PetscReal *,PetscReal *);
+/* Private functions */
+PETSC_STATIC_INLINE void SumNorm2(PetscReal*,PetscReal*,PetscReal*,PetscReal*);
 PETSC_STATIC_INLINE PetscReal GetNorm2(PetscReal,PetscReal);
-PETSC_STATIC_INLINE void AddNorm2(PetscReal *,PetscReal *,PetscReal);
+PETSC_STATIC_INLINE void AddNorm2(PetscReal*,PetscReal*,PetscReal);
+static PetscErrorCode VecCompSetSubVecs_Comp(Vec,PetscInt,Vec*);
+static PetscErrorCode VecCompGetSubVecs_Comp(Vec,PetscInt*,const Vec**);
 
 #include "veccomp0.h"
 
@@ -70,8 +61,6 @@ PETSC_STATIC_INLINE void AddNorm2(PetscReal *ssq,PetscReal *scale,PetscReal x)
   }
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SlepcSumNorm2_Local"
 static void SlepcSumNorm2_Local(void *in,void *out,PetscMPIInt *cnt,MPI_Datatype *datatype)
 {
   PetscInt i,count = *cnt;
@@ -95,8 +84,6 @@ static void SlepcSumNorm2_Local(void *in,void *out,PetscMPIInt *cnt,MPI_Datatype
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecNormCompEnd"
 static PetscErrorCode VecNormCompEnd(void)
 {
   PetscErrorCode ierr;
@@ -109,8 +96,6 @@ static PetscErrorCode VecNormCompEnd(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecNormCompInit"
 static PetscErrorCode VecNormCompInit()
 {
   PetscErrorCode ierr;
@@ -125,8 +110,6 @@ static PetscErrorCode VecNormCompInit()
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroy_Comp"
 PetscErrorCode VecDestroy_Comp(Vec v)
 {
   Vec_Comp       *vs = (Vec_Comp*)v->data;
@@ -145,6 +128,8 @@ PetscErrorCode VecDestroy_Comp(Vec v)
   }
   ierr = PetscFree(vs->x);CHKERRQ(ierr);
   ierr = PetscFree(vs);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)v,"VecCompSetSubVecs_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)v,"VecCompGetSubVecs_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -184,10 +169,10 @@ static struct _VecOps DvOps = {VecDuplicate_Comp, /* 1 */
             0/*VecPlaceArray_Seq*/,
             0/*VecReplaceArray_Seq*/,
             VecDot_Comp_Seq,
-            0,
+            VecTDot_Comp_Seq,
             VecNorm_Comp_Seq,
             VecMDot_Comp_Seq,
-            0, /* 40 */
+            VecMTDot_Comp_Seq, /* 40 */
             0,
             VecReciprocal_Comp,
             VecConjugate_Comp,
@@ -203,15 +188,13 @@ static struct _VecOps DvOps = {VecDuplicate_Comp, /* 1 */
             VecAbs_Comp,
             VecExp_Comp,
             VecLog_Comp,
-            0/*VecShift_Comp*/,
+            VecShift_Comp,
             0,
             0,
             0,
             VecDotNorm2_Comp_MPI
           };
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicateVecs_Comp"
 PetscErrorCode VecDuplicateVecs_Comp(Vec w,PetscInt m,Vec *V[])
 {
   PetscErrorCode ierr;
@@ -220,14 +203,12 @@ PetscErrorCode VecDuplicateVecs_Comp(Vec w,PetscInt m,Vec *V[])
   PetscFunctionBegin;
   PetscValidHeaderSpecific(w,VEC_CLASSID,1);
   PetscValidPointer(V,3);
-  if (m<=0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"m must be > 0: m = %D",m);
+  if (m<=0) SETERRQ1(PetscObjectComm((PetscObject)w),PETSC_ERR_ARG_OUTOFRANGE,"m must be > 0: m = %D",m);
   ierr = PetscMalloc1(m,V);CHKERRQ(ierr);
   for (i=0;i<m;i++) { ierr = VecDuplicate(w,*V+i);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroyVecs_Comp"
 PetscErrorCode VecDestroyVecs_Comp(PetscInt m,Vec v[])
 {
   PetscErrorCode ierr;
@@ -236,13 +217,11 @@ PetscErrorCode VecDestroyVecs_Comp(PetscInt m,Vec v[])
   PetscFunctionBegin;
   PetscValidPointer(v,1);
   if (m<=0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"m must be > 0: m = %D",m);
-  for (i=0;i<m;i++) if (v[i]) { ierr = VecDestroy(&v[i]);CHKERRQ(ierr); }
+  for (i=0;i<m;i++) { ierr = VecDestroy(&v[i]);CHKERRQ(ierr); }
   ierr = PetscFree(v);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCreate_Comp_Private"
 static PetscErrorCode VecCreate_Comp_Private(Vec v,Vec *x,PetscInt nx,PetscBool x_to_me,Vec_Comp_N *n)
 {
   Vec_Comp       *s;
@@ -264,41 +243,43 @@ static PetscErrorCode VecCreate_Comp_Private(Vec v,Vec *x,PetscInt nx,PetscBool
   v->petscnative = PETSC_FALSE;
 
   /* Allocate the array of Vec, if it is needed to be done */
-  if (x_to_me != PETSC_TRUE) {
-    ierr = PetscMalloc(sizeof(Vec)*nx,&s->x);CHKERRQ(ierr);
-    ierr = PetscMemcpy(s->x,x,sizeof(Vec)*nx);CHKERRQ(ierr);
+  if (!x_to_me) {
+    if (nx) { ierr = PetscMalloc1(nx,&s->x);CHKERRQ(ierr); }
+    if (x) { ierr = PetscMemcpy(s->x,x,sizeof(Vec)*nx);CHKERRQ(ierr); }
   } else s->x = x;
 
   s->nx = nx;
 
-  /* Allocate the shared structure, if it is not given */
-  if (!n) {
-    for (i=0;i<nx;i++) {
-      ierr = VecGetSize(x[i],&k);CHKERRQ(ierr);
-      N+= k;
-      ierr = VecGetLocalSize(x[i],&k);CHKERRQ(ierr);
-      lN+= k;
+  if (nx) {
+    /* Allocate the shared structure, if it is not given */
+    if (!n) {
+      for (i=0;i<nx;i++) {
+        ierr = VecGetSize(x[i],&k);CHKERRQ(ierr);
+        N+= k;
+        ierr = VecGetLocalSize(x[i],&k);CHKERRQ(ierr);
+        lN+= k;
+      }
+      ierr = PetscNewLog(v,&n);CHKERRQ(ierr);
+      s->n = n;
+      n->n = nx;
+      n->N = N;
+      n->lN = lN;
+      n->friends = 1;
+    } else { /* If not, check in the vector in the shared structure */
+      s->n = n;
+      s->n->friends++;
     }
-    ierr = PetscNewLog(v,&n);CHKERRQ(ierr);
-    s->n = n;
-    n->n = nx;
-    n->N = N;
-    n->lN = lN;
-    n->friends = 1;
-  } else { /* If not, check in the vector in the shared structure */
-    s->n = n;
-    s->n->friends++;
+
+    /* Set the virtual sizes as the real sizes of the vector */
+    ierr = VecSetSizes(v,s->n->lN,s->n->N);CHKERRQ(ierr);
   }
 
-  /* Set the virtual sizes as the real sizes of the vector */
-  ierr = VecSetSizes(v,s->n->lN,s->n->N);CHKERRQ(ierr);
-
   ierr = PetscObjectChangeTypeName((PetscObject)v,VECCOMP);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)v,"VecCompSetSubVecs_C",VecCompSetSubVecs_Comp);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)v,"VecCompGetSubVecs_C",VecCompGetSubVecs_Comp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCreate_Comp"
 PETSC_EXTERN PetscErrorCode VecCreate_Comp(Vec V)
 {
   PetscErrorCode ierr;
@@ -308,8 +289,6 @@ PETSC_EXTERN PetscErrorCode VecCreate_Comp(Vec V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCreateComp"
 /*@C
    VecCreateComp - Creates a new vector containing several subvectors,
    each stored separately.
@@ -353,8 +332,6 @@ PetscErrorCode VecCreateComp(MPI_Comm comm,PetscInt *Nx,PetscInt n,VecType t,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCreateCompWithVecs"
 /*@C
    VecCreateCompWithVecs - Creates a new vector containing several subvectors,
    each stored separately, from an array of Vecs.
@@ -387,8 +364,6 @@ PetscErrorCode VecCreateCompWithVecs(Vec *x,PetscInt n,Vec Vparent,Vec *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicate_Comp"
 PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
 {
   PetscErrorCode ierr;
@@ -397,7 +372,7 @@ PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
   Vec_Comp       *s = (Vec_Comp*)win->data;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(win);
+  SlepcValidVecComp(win,1);
   ierr = VecCreate(PetscObjectComm((PetscObject)win),V);CHKERRQ(ierr);
   ierr = PetscMalloc1(s->nx,&x);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)*V,s->nx*sizeof(Vec));CHKERRQ(ierr);
@@ -410,8 +385,16 @@ PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCompGetSubVecs"
+static PetscErrorCode VecCompGetSubVecs_Comp(Vec win,PetscInt *n,const Vec **x)
+{
+  Vec_Comp *s = (Vec_Comp*)win->data;
+
+  PetscFunctionBegin;
+  if (x) *x = s->x;
+  if (n) *n = s->n->n;
+  PetscFunctionReturn(0);
+}
+
 /*@C
    VecCompGetSubVecs - Returns the entire array of vectors defining a
    compound vector.
@@ -431,20 +414,55 @@ PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
 @*/
 PetscErrorCode VecCompGetSubVecs(Vec win,PetscInt *n,const Vec **x)
 {
-  Vec_Comp *s = (Vec_Comp*)win->data;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(win);
-  if (x) *x = s->x;
-  if (n) *n = s->nx;
+  PetscValidHeaderSpecific(win,VEC_CLASSID,1);
+  ierr = PetscUseMethod(win,"VecCompGetSubVecs_C",(Vec,PetscInt*,const Vec**),(win,n,x));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode VecCompSetSubVecs_Comp(Vec win,PetscInt n,Vec *x)
+{
+  Vec_Comp       *s = (Vec_Comp*)win->data;
+  PetscInt       i,N,nlocal;
+  Vec_Comp_N     *nn;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (!s) SETERRQ(PetscObjectComm((PetscObject)win),PETSC_ERR_ORDER,"Must call VecSetSizes first");
+  if (!s->nx) {
+    /* vector has been created via VecCreate+VecSetType+VecSetSizes, so allocate data structures */
+    ierr = PetscMalloc1(n,&s->x);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)win,n*sizeof(Vec));CHKERRQ(ierr);
+    ierr = VecGetSize(win,&N);CHKERRQ(ierr);
+    if (N%n) SETERRQ2(PetscObjectComm((PetscObject)win),1,"Global dimension %D is not divisible by %D",N,n);
+    ierr = VecGetLocalSize(win,&nlocal);CHKERRQ(ierr);
+    if (nlocal%n) SETERRQ2(PetscObjectComm((PetscObject)win),1,"Local dimension %D is not divisible by %D",nlocal,n);
+    s->nx = n;
+    for (i=0;i<n;i++) {
+      ierr = VecCreate(PetscObjectComm((PetscObject)win),&s->x[i]);CHKERRQ(ierr);
+      ierr = VecSetSizes(s->x[i],nlocal/n,N/n);CHKERRQ(ierr);
+      ierr = VecSetFromOptions(s->x[i]);CHKERRQ(ierr);
+    }
+    if (!s->n) {
+      ierr = PetscNewLog(win,&nn);CHKERRQ(ierr);
+      s->n = nn;
+      nn->N = N;
+      nn->lN = nlocal;
+      nn->friends = 1;
+    }
+  } else if (n > s->nx) SETERRQ1(PetscObjectComm((PetscObject)win),PETSC_ERR_SUP,"Number of child vectors cannot be larger than %D",s->nx);
+  if (x) {
+    ierr = PetscMemcpy(s->x,x,sizeof(Vec)*n);CHKERRQ(ierr);
+  }
+  s->n->n = n;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCompSetSubVecs"
 /*@C
    VecCompSetSubVecs - Resets the number of subvectors defining a compound vector,
-   of replaces the subvectors.
+   or replaces the subvectors.
 
    Collective on Vec
 
@@ -453,30 +471,25 @@ PetscErrorCode VecCompGetSubVecs(Vec win,PetscInt *n,const Vec **x)
 .  n - number of child vectors
 -  x - array of child vectors
 
+   Note:
+   It is not possible to increase the number of subvectors with respect to the
+   number set at its creation.
+
    Level: developer
 
-.seealso: VecCreateComp()
+.seealso: VecCreateComp(), VecCompGetSubVecs()
 @*/
 PetscErrorCode VecCompSetSubVecs(Vec win,PetscInt n,Vec *x)
 {
-  Vec_Comp       *s = (Vec_Comp*)win->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (x) {
-    if (n > s->nx) {
-      ierr = PetscFree(s->x);CHKERRQ(ierr);
-      ierr = PetscMalloc(sizeof(Vec)*n,&s->x);CHKERRQ(ierr);
-    }
-    ierr = PetscMemcpy(s->x,x,sizeof(Vec)*n);CHKERRQ(ierr);
-    s->nx = n;
-  }
-  s->n->n = n;
+  PetscValidHeaderSpecific(win,VEC_CLASSID,1);
+  PetscValidLogicalCollectiveInt(win,n,2);
+  ierr = PetscTryMethod(win,"VecCompSetSubVecs_C",(Vec,PetscInt,Vec*),(win,n,x));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAXPY_Comp"
 PetscErrorCode VecAXPY_Comp(Vec v,PetscScalar alpha,Vec w)
 {
   PetscErrorCode ierr;
@@ -484,16 +497,14 @@ PetscErrorCode VecAXPY_Comp(Vec v,PetscScalar alpha,Vec w)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  SlepcValidVecComp(w);
+  SlepcValidVecComp(v,1);
+  SlepcValidVecComp(w,3);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAXPY(vs->x[i],alpha,ws->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAYPX_Comp"
 PetscErrorCode VecAYPX_Comp(Vec v,PetscScalar alpha,Vec w)
 {
   PetscErrorCode ierr;
@@ -501,16 +512,14 @@ PetscErrorCode VecAYPX_Comp(Vec v,PetscScalar alpha,Vec w)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  SlepcValidVecComp(w);
+  SlepcValidVecComp(v,1);
+  SlepcValidVecComp(w,3);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAYPX(vs->x[i],alpha,ws->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAXPBY_Comp"
 PetscErrorCode VecAXPBY_Comp(Vec v,PetscScalar alpha,PetscScalar beta,Vec w)
 {
   PetscErrorCode ierr;
@@ -518,16 +527,14 @@ PetscErrorCode VecAXPBY_Comp(Vec v,PetscScalar alpha,PetscScalar beta,Vec w)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  SlepcValidVecComp(w);
+  SlepcValidVecComp(v,1);
+  SlepcValidVecComp(w,4);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAXPBY(vs->x[i],alpha,beta,ws->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecMAXPY_Comp"
 PetscErrorCode VecMAXPY_Comp(Vec v,PetscInt n,const PetscScalar *alpha,Vec *w)
 {
   PetscErrorCode ierr;
@@ -536,10 +543,10 @@ PetscErrorCode VecMAXPY_Comp(Vec v,PetscInt n,const PetscScalar *alpha,Vec *w)
   PetscInt       i,j;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  for (i=0;i<n;i++) SlepcValidVecComp(w[i]);
+  SlepcValidVecComp(v,1);
+  for (i=0;i<n;i++) SlepcValidVecComp(w[i],4);
 
-  ierr = PetscMalloc(sizeof(Vec)*n,&wx);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n,&wx);CHKERRQ(ierr);
 
   for (j=0;j<vs->n->n;j++) {
     for (i=0;i<n;i++) wx[i] = ((Vec_Comp*)w[i]->data)->x[j];
@@ -550,8 +557,6 @@ PetscErrorCode VecMAXPY_Comp(Vec v,PetscInt n,const PetscScalar *alpha,Vec *w)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecWAXPY_Comp"
 PetscErrorCode VecWAXPY_Comp(Vec v,PetscScalar alpha,Vec w,Vec z)
 {
   PetscErrorCode ierr;
@@ -559,17 +564,15 @@ PetscErrorCode VecWAXPY_Comp(Vec v,PetscScalar alpha,Vec w,Vec z)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  SlepcValidVecComp(w);
-  SlepcValidVecComp(z);
+  SlepcValidVecComp(v,1);
+  SlepcValidVecComp(w,3);
+  SlepcValidVecComp(z,4);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecWAXPY(vs->x[i],alpha,ws->x[i],zs->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAXPBYPCZ_Comp"
 PetscErrorCode VecAXPBYPCZ_Comp(Vec v,PetscScalar alpha,PetscScalar beta,PetscScalar gamma,Vec w,Vec z)
 {
   PetscErrorCode  ierr;
@@ -577,43 +580,41 @@ PetscErrorCode VecAXPBYPCZ_Comp(Vec v,PetscScalar alpha,PetscScalar beta,PetscSc
   PetscInt        i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  SlepcValidVecComp(w);
-  SlepcValidVecComp(z);
+  SlepcValidVecComp(v,1);
+  SlepcValidVecComp(w,5);
+  SlepcValidVecComp(z,6);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAXPBYPCZ(vs->x[i],alpha,beta,gamma,ws->x[i],zs->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetSize_Comp"
 PetscErrorCode VecGetSize_Comp(Vec v,PetscInt *size)
 {
   Vec_Comp *vs = (Vec_Comp*)v->data;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
   PetscValidIntPointer(size,2);
-  *size = vs->n->N;
+  if (vs->n) {
+    SlepcValidVecComp(v,1);
+    *size = vs->n->N;
+  } else *size = v->map->N;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetLocalSize_Comp"
 PetscErrorCode VecGetLocalSize_Comp(Vec v,PetscInt *size)
 {
   Vec_Comp *vs = (Vec_Comp*)v->data;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
   PetscValidIntPointer(size,2);
-  *size = vs->n->lN;
+  if (vs->n) {
+    SlepcValidVecComp(v,1);
+    *size = vs->n->lN;
+  } else *size = v->map->n;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecMax_Comp"
 PetscErrorCode VecMax_Comp(Vec v,PetscInt *idx,PetscReal *z)
 {
   PetscErrorCode ierr;
@@ -623,7 +624,7 @@ PetscErrorCode VecMax_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
+  SlepcValidVecComp(v,1);
   if (!idx && !z) PetscFunctionReturn(0);
 
   if (vs->n->n > 0) {
@@ -645,8 +646,6 @@ PetscErrorCode VecMax_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecMin_Comp"
 PetscErrorCode VecMin_Comp(Vec v,PetscInt *idx,PetscReal *z)
 {
   PetscErrorCode ierr;
@@ -656,7 +655,7 @@ PetscErrorCode VecMin_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
+  SlepcValidVecComp(v,1);
   if (!idx && !z) PetscFunctionReturn(0);
 
   if (vs->n->n > 0) {
@@ -678,8 +677,6 @@ PetscErrorCode VecMin_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecMaxPointwiseDivide_Comp"
 PetscErrorCode VecMaxPointwiseDivide_Comp(Vec v,Vec w,PetscReal *m)
 {
   PetscErrorCode ierr;
@@ -688,8 +685,8 @@ PetscErrorCode VecMaxPointwiseDivide_Comp(Vec v,Vec w,PetscReal *m)
   PetscInt       i;
 
   PetscFunctionBegin;
-  SlepcValidVecComp(v);
-  SlepcValidVecComp(w);
+  SlepcValidVecComp(v,1);
+  SlepcValidVecComp(w,2);
   if (!m || vs->n->n == 0) PetscFunctionReturn(0);
   ierr = VecMaxPointwiseDivide(vs->x[0],ws->x[0],m);CHKERRQ(ierr);
   for (i=1;i<vs->n->n;i++) {
@@ -712,38 +709,20 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  SlepcValidVecComp(v); \
+  SlepcValidVecComp(v,1); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i]);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecConjugate_Comp"
 __FUNC_TEMPLATE1__(Conjugate)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecReciprocal_Comp"
 __FUNC_TEMPLATE1__(Reciprocal)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecSqrtAbs_Comp"
 __FUNC_TEMPLATE1__(SqrtAbs)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecAbs_Comp"
 __FUNC_TEMPLATE1__(Abs)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecExp_Comp"
 __FUNC_TEMPLATE1__(Exp)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecLog_Comp"
 __FUNC_TEMPLATE1__(Log)
 
-
 #define __FUNC_TEMPLATE2__(NAME,T0) \
 PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,T0 __a) \
 { \
@@ -752,34 +731,19 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,T0 __a) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  SlepcValidVecComp(v); \
+  SlepcValidVecComp(v,1); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i],__a);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSet_Comp"
 __FUNC_TEMPLATE2__(Set,PetscScalar)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Comp"
 __FUNC_TEMPLATE2__(View,PetscViewer)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecScale_Comp"
 __FUNC_TEMPLATE2__(Scale,PetscScalar)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecSetRandom_Comp"
 __FUNC_TEMPLATE2__(SetRandom,PetscRandom)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecShift_Comp"
 __FUNC_TEMPLATE2__(Shift,PetscScalar)
 
-
 #define __FUNC_TEMPLATE3__(NAME) \
 PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,Vec w) \
 { \
@@ -789,23 +753,17 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,Vec w) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  SlepcValidVecComp(v); \
-  SlepcValidVecComp(w); \
+  SlepcValidVecComp(v,1); \
+  SlepcValidVecComp(w,2); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i],ws->x[i]);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCopy_Comp"
 __FUNC_TEMPLATE3__(Copy)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecSwap_Comp"
 __FUNC_TEMPLATE3__(Swap)
 
-
 #define __FUNC_TEMPLATE4__(NAME) \
 PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,Vec w,Vec z) \
 { \
@@ -816,31 +774,17 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,Vec w,Vec z) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  SlepcValidVecComp(v); \
-  SlepcValidVecComp(w); \
-  SlepcValidVecComp(z); \
+  SlepcValidVecComp(v,1); \
+  SlepcValidVecComp(w,2); \
+  SlepcValidVecComp(z,3); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i],ws->x[i],zs->x[i]);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMax_Comp"
 __FUNC_TEMPLATE4__(PointwiseMax)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMaxAbs_Comp"
 __FUNC_TEMPLATE4__(PointwiseMaxAbs)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMin_Comp"
 __FUNC_TEMPLATE4__(PointwiseMin)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMult_Comp"
 __FUNC_TEMPLATE4__(PointwiseMult)
-
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseDivide_Comp"
 __FUNC_TEMPLATE4__(PointwiseDivide)
